/*
 * @(#)AwtTools.java  1.0.0 / 2008-05-05
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.awt;

import java.awt.Component;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.DisplayMode;
import java.awt.GraphicsEnvironment;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.Toolkit;
import java.awt.Window;

/**
 * AwtTools Class.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public final class AwtTools {

  // FINISH: add resolutions change listener.

/*^**************************************************************************^*/
/*- Private Static Fields.                                                   -*/
/*^**************************************************************************^*/

  private static final GraphicsEnvironment GE;
  private static Rectangle maximumWindowBounds;
  private static int screenWidth;
  private static int screenHeight;
  private static int screenWidthGap;
  private static int screenHeightGap;
  private static int resolution;
  private static boolean lowResolution;
  private static boolean trueColorModel;

/*^**************************************************************************^*/
/*- Static Initialization.                                                   -*/
/*^**************************************************************************^*/

  static {
    GE = GraphicsEnvironment.getLocalGraphicsEnvironment();
    initialize();
  }

/*^**************************************************************************^*/
/*- Private Constructor.                                                     -*/
/*^**************************************************************************^*/

  /**
   * Class has no instances.
   */
  private AwtTools() {
    // empty constructor body
  }

/*^**************************************************************************^*/
/*- Public Static Methods.                                                   -*/
/*^**************************************************************************^*/

  /**
   * Tools initialization.
   */
  public static void initialize() {
    maximumWindowBounds = GE.getMaximumWindowBounds();
    DisplayMode dm = GE.getDefaultScreenDevice().getDisplayMode();
    screenWidth = dm.getWidth();
    screenHeight = dm.getHeight();
    screenWidthGap = screenWidth - maximumWindowBounds.width;
    screenHeightGap = screenHeight - maximumWindowBounds.height;
    Toolkit dtk = Toolkit.getDefaultToolkit();
    resolution = dtk.getScreenResolution();
    lowResolution = (resolution < 120);
    trueColorModel = (dtk.getColorModel().getPixelSize() >= 24);
  }

  /**
   * Returns screen width.
   *
   * @return screen width
   */
  public static int getScreenWidth() {
    return screenWidth;
  }

  /**
   * Returns screen height.
   *
   * @return screen height
   */
  public static int getScreenHeight() {
    return screenHeight;
  }

  /**
   * Returns maximum window width.
   *
   * @return maximum window width
   */
  public static int getMaximumWindowWidth() {
    return maximumWindowBounds.width;
  }

  /**
   * Returns maximum window height.
   *
   * @return maximum window height
   */
  public static int getMaximumWindowHeight() {
    return maximumWindowBounds.height;
  }

  /**
   * Returns screen width gap.
   *
   * @return screen width gap
   */
  public static int getScreenWidthGap() {
    return screenWidthGap;
  }

  /**
   * Returns screen height gap.
   *
   * @return screen height gap
   */
  public static int getScreenHeightGap() {
    return screenHeightGap;
  }

  /**
   * Returns screen rectangle.
   *
   * @return screen rectangle
   */
  public static Rectangle getScreenRectangle() {
    return new Rectangle(0, 0, screenWidth, screenHeight);
  }

  /**
   * Returns screen size.
   *
   * @return screen size
   */
  public static Dimension getScreenSize() {
    return Toolkit.getDefaultToolkit().getScreenSize();
  }

  /**
   * Returns maximum window bounds.
   *
   * @return maximum window bounds
   */
  public static Rectangle getMaximumWindowBounds() {
    return GE.getMaximumWindowBounds();
  }

  /**
   * Returns screen resolution.
   *
   * @return screen resolution
   */
  public static int getScreenResolution() {
    return resolution;
  }

  /**
   * Returns <code>true</code> if resolution is less then 120 dots-per-inch.
   *
   * @return <code>true</code> if resolution is less then 120 dots-per-inch
   */
  public static boolean isLowResolution() {
    return lowResolution;
  }

  /**
   * Color depth check.
   *
   * @return <tt>true</tt> if the display uses 24- or 32-bit color depth
   *         (= true color)
   */
  public static boolean isTrueColorModel() {
    return trueColorModel;
  }

  /**
   * Returns component orientation.
   *
   * @param c the component
   *
   * @return component orientation
   */
  public static boolean isLeftToRight(Component c) {
    return c.getComponentOrientation().isLeftToRight();
  }

  /**
   * Adjusts window position to fit in the system desktop.
   *
   * @param w the window
   */
  public static void fitPosition(Window w) {
    fitPosition(w.getBounds(), maximumWindowBounds);
  }

  /**
   * Adjusts window position to fit in the specified rectangle.
   *
   * @param w the window
   * @param r the rectangle
   */
  public static void fitPosition(Window w, Rectangle r) {
    Point p = calcFitPoint(w.getBounds(), r);
    if (!p.equals(w.getLocation())) {
      w.setLocation(p);
    }
  }

  /**
   * Adjusts rectangle's position to fit in the other specified rectangle.
   *
   * @param w the window rectangle
   * @param r the rectangle
   */
  public static void fitPosition(Rectangle w, Rectangle r) {
    Point p = calcFitPoint(w, r);
    w.setLocation(p.x, p.y);
  }

  /**
   * Centers window relative to the parent (owner) window.
   *
   * @param w the window to be centered
   */
  public static void parentCenterWindow(Window w) {
    Window owner = w.getOwner();
    if ((owner != null) && owner.isVisible()) {
      centerWindow(w, owner);
    }
  }

  /**
   * Centers window relative to the system screen.
   *
   * @param w the window to be centered
   */
  public static void screenCenterWindow(Window w) {
    int xp = (screenWidth - w.getWidth()) / 2;
    int yp = (screenHeight - w.getHeight()) / 2;
    if (xp < 0) {
      xp = 0;
    }
    if (yp < 0) {
      yp = 0;
    }
    w.setLocation(xp, yp);
  }

  /**
   * Centers window relative to the system desktop.
   *
   * @param w the window to be centered
   */
  public static void desktopCenterWindow(Window w) {
    Rectangle mwb = maximumWindowBounds;
    int xp = (mwb.width - w.getWidth()) / 2 + mwb.x;
    int yp = (mwb.height - w.getHeight()) / 2 + mwb.y;
    if (xp < mwb.x) {
      xp = mwb.x;
    }
    if (yp < mwb.y) {
      yp = mwb.y;
    }
    w.setLocation(xp, yp);
  }

  /**
   * Centers window relative to the other window.
   *
   * @param w the window to be centered
   * @param parent the parent of the window
   */
  public static void centerWindow(Window w, Window parent) {
    int xp = (parent.getWidth() - w.getWidth()) / 2 + parent.getX();
    int yp = (parent.getHeight() - w.getHeight()) / 2 + parent.getY();
    Rectangle r = maximumWindowBounds;
    if (xp < r.x) {
      xp = r.x;
    } else if ((xp + w.getWidth()) > (r.x + r.width)) {
      xp = r.x + (r.width - w.getWidth());
    }
    if (yp < r.y) {
      yp = r.y;
    } else if ((yp + w.getHeight()) > (r.y + r.height)) {
      yp = r.y + (r.height - w.getHeight());
    }
    w.setLocation(xp, yp);
  }

  /**
   * Centers window relative to the rectangle.
   *
   * @param w the window to be centered
   * @param r the rectangle
   */
  public static void centerWindow(Window w, Rectangle r) {
    int xp = (r.width - w.getWidth()) / 2 + r.x;
    int yp = (r.height - w.getHeight()) / 2 + r.y;
    Rectangle mwb = maximumWindowBounds;
    if (xp < mwb.x) {
      xp = mwb.x;
    } else if ((xp + w.getWidth()) > (mwb.x + mwb.width)) {
      xp = mwb.x + (mwb.width - w.getWidth());
    }
    if (yp < mwb.y) {
      yp = mwb.y;
    } else if ((yp + w.getHeight()) > (mwb.y + mwb.height)) {
      yp = mwb.y + (mwb.height - w.getHeight());
    }
    w.setLocation(xp, yp);
  }

  /**
   * Grid bag helper.
   *
   * @param parent the parent
   * @param child the child
   * @param gbc the grid bag constraints
   * @param gridx the grid x coordinate
   * @param gridy the grid y coordinate
   * @param gridwidth the grid width
   * @param gridheight the grid height
   * @param top the top
   * @param left the left
   * @param bottom the bottom
   * @param right the right
   * @param anchor the anchor
   * @param fill the fill
   * @param weightx the weight x
   * @param weighty the weight y
   * @param ipadx the internal padding x
   * @param ipady the internal padding y
   */
  public static void addGBC(Container parent, Component child,
                            GridBagConstraints gbc, int gridx, int gridy,
                            int gridwidth, int gridheight, int top, int left,
                            int bottom, int right, int anchor, int fill,
                            double weightx, double weighty, int ipadx,
                            int ipady) {
    if (!(parent.getLayout() instanceof GridBagLayout)) {
      throw new IllegalArgumentException("Invalid parent's layout");
    }
    gbc.gridx = gridx;
    gbc.gridy = gridy;
    gbc.gridwidth = gridwidth;
    gbc.gridheight = gridheight;
    gbc.insets.top = top;
    gbc.insets.left = left;
    gbc.insets.bottom = bottom;
    gbc.insets.right = right;
    gbc.anchor = anchor;
    gbc.fill = fill;
    gbc.weightx = weightx;
    gbc.weighty = weighty;
    gbc.ipadx = ipadx;
    gbc.ipady = ipady;
    parent.add(child, gbc);
  }

  /**
   * Grid bag helper.
   *
   * @param parent the parent
   * @param child the child
   * @param gbc the grid bag constraints
   * @param gridx the grid x coordinate
   * @param gridy the grid y coordinate
   * @param gridwidth the grid width
   * @param gridheight the grid height
   * @param top the top
   * @param left the left
   * @param bottom the bottom
   * @param right the right
   * @param anchor the anchor
   * @param fill the fill
   * @param weightx the weight x
   * @param weighty the weight y
   */
  public static void addGBC(Container parent, Component child,
                            GridBagConstraints gbc, int gridx, int gridy,
                            int gridwidth, int gridheight, int top, int left,
                            int bottom, int right, int anchor, int fill,
                            double weightx, double weighty) {
    addGBC(parent, child, gbc, gridx, gridy, gridwidth, gridheight, top, left,
           bottom, right, anchor, fill, weightx, weighty, 0, 0);
  }

/*^**************************************************************************^*/
/*- Private Static Method.                                                   -*/
/*^**************************************************************************^*/

  private static Point calcFitPoint(Rectangle c, Rectangle r) {
    int x = c.x;
    int y = c.y;
    if ((c.width > r.width) || (c.x < r.x)) {
      x = r.x;
    } else if ((c.width + c.x) > (r.width + r.x)) {
      x = r.x + (r.width - c.width);
    }
    if ((c.height > r.height) || (c.y < r.y)) {
      y = r.y;
    } else if (c.height + c.y > r.height) {
      y = r.y + (r.height - c.height);
    }
    return new Point(x, y);
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-05 22:17:06 Slobodan
   Initial version

*****************************************************************************^*/
