/*
 * @(#)NativeManager.java  1.0.0 / 2008-05-13
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.natives;

import java.lang.reflect.Method;

import net.wrka.LibraryInfo;

/**
 * Manager of native libraries.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public final class NativeManager {

/*^**************************************************************************^*/
/*- Public Static Fields.                                                    -*/
/*^**************************************************************************^*/

  /**
   * Unknown OS (the value is {@value}).
   */
  public static final int OS_UNKNOWN = 0;

  /**
   * Linux OS (the value is {@value}).
   */
  public static final int OS_LINUX = 1;

  /**
   * Windows OS (the value is {@value}).
   */
  public static final int OS_WINDOWS = 2;

  /**
   * MAC OS (the value is {@value}).
   */
  public static final int OS_MAC = 3;

  /**
   * OS2 OS (the value is {@value}).
   */
  public static final int OS_OS2 = 4;

  /**
   * Beos OS (the value is {@value}).
   */
  public static final int OS_BEOS = 5;

  /**
   * Solaris OS (the value is {@value}).
   */
  public static final int OS_SOLARIS = 6;

/*^**************************************************************************^*/
/*- Private Static Field.                                                    -*/
/*^**************************************************************************^*/

  private static NativeSupport nativeSupport;

/*^**************************************************************************^*/
/*- Static Initialization.                                                   -*/
/*^**************************************************************************^*/

  static {
    initialize();
  }

/*^**************************************************************************^*/
/*- Private Constructor.                                                     -*/
/*^**************************************************************************^*/

  /**
   * Class has no instances.
   */
  private NativeManager() {
    // empty constructor body
  }

/*^**************************************************************************^*/
/*- Public Static Methods.                                                   -*/
/*^**************************************************************************^*/

  /**
   * Returns <code>true</code> if there is native support.
   *
   * @return <code>true</code> if there is native support
   */
  public static boolean hasNativeSupport() {
    return (nativeSupport != null);
  }

  /**
   * Registers native support.
   *
   * @param nativeSupport native support
   */
  public static void registerNativeSupport(NativeSupport nativeSupport) {
    if ((nativeSupport != null) && nativeSupport.isLoaded()) {
      NativeManager.nativeSupport = nativeSupport;
    } else {
      NativeManager.nativeSupport = null;
    }
  }

  /**
   * Returns current native support.
   *
   * @return current native support
   */
  public static NativeSupport getNativeSupport() {
    return nativeSupport;
  }

/*^**************************************************************************^*/
/*- Private Static Methods.                                                  -*/
/*^**************************************************************************^*/

  private static void initialize() {
    if (LibraryInfo.OS_LINUX) {
      loadSupport("net.wrka.natives.linux.LinuxNative"); //$NON-NLS-1$
    } else if (LibraryInfo.OS_WINDOWS) {
      loadSupport("net.wrka.natives.win32.WindowsNative"); //$NON-NLS-1$
    } else if (LibraryInfo.OS_MAC) {
      loadSupport("net.wrka.natives.mac.MacNative"); //$NON-NLS-1$
    } else if (LibraryInfo.OS_OS2) {
      loadSupport("net.wrka.natives.os2.OS2Native"); //$NON-NLS-1$
    } else if (LibraryInfo.OS_BEOS) {
      loadSupport("net.wrka.natives.beos.BOESNative"); //$NON-NLS-1$
    } else if (LibraryInfo.OS_SOLARIS) {
      loadSupport("net.wrka.natives.solaris.SolarisNative"); //$NON-NLS-1$
    } else {
      nativeSupport = null;
    }
  }

  private static void loadSupport(String className) {
    if (className != null) {
      try {
        Class nsClass = Class.forName(className);
        Method nsGetInstance = nsClass.getMethod("getInstance", //$NON-NLS-1$
                                                 null);
        nativeSupport = (NativeSupport)nsGetInstance.invoke(null, null);
        if (!nativeSupport.isLoaded()) {
          nativeSupport = null;
        }
      } catch (Throwable t) { // NOPMD
        nativeSupport = null;
      }
    }
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-13 13:39:54 Slobodan
   Initial version

*****************************************************************************^*/
