/*
 * @(#)ActionGroup.java  1.0.0 / 2008-05-15
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.swing.actions;

import java.awt.event.ActionEvent;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.ArrayList;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.Icon;

/**
 * Action Group.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public class ActionGroup extends AbstractAction {

/*^**************************************************************************^*/
/*- Public Static Field.                                                     -*/
/*^**************************************************************************^*/

  /**
   * Property string <tt>CurrentActionChanged</tt>.
   */
  public static final String CURRENT_ACTION_CHANGED =
                                          "CurrentActionChanged"; //$NON-NLS-1$

  /**
   * Property string <tt>ActionChanged</tt>.
   */
  public static final String ACTION_CHANGED = "ActionChanged"; //$NON-NLS-1$

  /**
   * Property string <tt>ActionAdded</tt>.
   */
  public static final String ACTION_ADDED = "ActionAdded"; //$NON-NLS-1$

  /**
   * Property string <tt>ActionRemoved</tt>.
   */
  public static final String ACTION_REMOVED = "ActionRemoved"; //$NON-NLS-1$

/*^**************************************************************************^*/
/*- Private Static Final Field.                                              -*/
/*^**************************************************************************^*/

  private static final long serialVersionUID = -6445686539292855694L;

/*^**************************************************************************^*/
/*- Private Fields.                                                          -*/
/*^**************************************************************************^*/

  private final ArrayList actions;
  private Action currentAction;
  private final transient ActionPropertyListener actionListener;

/*^**************************************************************************^*/
/*- Public Constructor.                                                      -*/
/*^**************************************************************************^*/

  /**
   * Constructs new ActionGroup object.
   */
  public ActionGroup() {
    super();
    actions = new ArrayList();
    actionListener = new ActionPropertyListener();
  }

  /**
   * Constructs new ActionGroup object.
   *
   * @param name the action name
   */
  public ActionGroup(String name) {
    this();
    if (name != null) {
      super.putValue(NAME, name);
    }
  }

  /**
   * Constructs new ActionGroup object.
   *
   * @param name the action name
   * @param shortdesc the short description
   */
  public ActionGroup(String name, String shortdesc) {
    this(name);
    if (shortdesc != null) {
      super.putValue(SHORT_DESCRIPTION, shortdesc);
    }
  }

  /**
   * Constructs new ActionGroup object.
   *
   * @param name the action name
   * @param icon the action icon
   */
  public ActionGroup(String name, Icon icon) {
    this(name);
    if (name != null) {
      super.putValue(SMALL_ICON, icon);
    }
  }

/*^**************************************************************************^*/
/*- Public Methods.                                                          -*/
/*^**************************************************************************^*/

  /**
   * Returns the number of the actions in the group.
   *
   * @return the number of the actions in the group
   */
  public int getActionCount() {
    return actions.size();
  }

  /**
   * Adds an action to the group. (Note: Method does not check for duplicates.)
   *
   * @param a the action to be added
   *
   * @throws IllegalArgumentException if action is <tt>null</tt>
   */
  public void addAction(Action a) {
    if (a == null) {
      throw new IllegalArgumentException("Null action"); //$NON-NLS-1$
    }
    actions.add(a);
    firePropertyChange(ACTION_ADDED, null, a);
  }

  /**
   * Sets action at specified position.
   *
   * @param index the action index
   * @param a the new action
   *
   * @throws IllegalArgumentException if action is <tt>null</tt>
   * @throws IndexOutOfBoundsException if index is out of range
   */
  public void setAction(int index, Action a) {
    if (a == null) {
      throw new IllegalArgumentException("Null action"); //$NON-NLS-1$
    }
    Action oldAction = (Action)actions.set(index, a);
    firePropertyChange(ACTION_CHANGED, oldAction, a);
  }

  /**
   * Removes specified action from the group.
   *
   * @param a the action to be removed
   *
   * @return <tt>true</tt> if the group contained the specified action
   *
   * @throws IllegalArgumentException if action is <tt>null</tt>
   */
  public boolean removeAction(Action a) {
    if (currentAction == a) { // NOPMD
      currentAction = null;
    }
    boolean res = actions.remove(a);
    firePropertyChange(ACTION_REMOVED, a, null);
    return res;
  }

  /**
   * Removes and returns action at specified index.
   *
   * @param index the action index
   *
   * @return removed action
   *
   * @throws IndexOutOfBoundsException if index is out of range
   */
  public Action removeAction(int index) {
    Action action = (Action)actions.remove(index);
    if (currentAction == action) { // NOPMD
      currentAction = null;
    }
    firePropertyChange(ACTION_REMOVED, action, null);
    return action;
  }

  /**
   * Returns current (active) action.
   *
   * @return current action
   */
  public Action getCurrentAction() {
    return currentAction;
  }

  /**
   * Selects current (active) action.
   *
   * @param index the new active action index
   *
   * @throws IndexOutOfBoundsException if index is out of range
   */
  public void selectCurrentAction(int index) {
    Action action = (Action)actions.get(index);
    if (action != currentAction) { // NOPMD
      Action oldCurrentAction = currentAction;
      currentAction = action;
      if (oldCurrentAction != null) {
        oldCurrentAction.removePropertyChangeListener(actionListener);
      }
      action.addPropertyChangeListener(actionListener);
      super.setEnabled(action.isEnabled());
      firePropertyChange(CURRENT_ACTION_CHANGED, oldCurrentAction, action);
    }
  }

  /**
   * Selects current (active) action.
   *
   * @param a the new active action
   *
   * @throws IllegalArgumentException if there is no such action in the group
   */
  public void selectCurrentAction(Action a) {
    int index = actions.indexOf(a);
    if (index < 0) {
      throw new IllegalArgumentException("No such action"); //$NON-NLS-1$
    }
    if (a != currentAction) { // NOPMD
      Action oldCurrentAction = currentAction;
      currentAction = a;
      if (oldCurrentAction != null) {
        oldCurrentAction.removePropertyChangeListener(actionListener);
      }
      a.addPropertyChangeListener(actionListener);
      setEnabled(a.isEnabled());
      firePropertyChange(CURRENT_ACTION_CHANGED, oldCurrentAction, a);
    }
  }

/*^**************************************************************************^*/
/*- ActionListener Implementation.                                           -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  public void actionPerformed(ActionEvent e) {
    if (currentAction != null) {
      currentAction.actionPerformed(e);
    }
  }

/*^**************************************************************************^*/
/*- Overridden Methods.                                                      -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  public boolean isEnabled() {
    if (currentAction != null) {
      return currentAction.isEnabled();
    }
    return super.isEnabled();
  }

  /**
   * {@inheritDoc}
   */
  public void setEnabled(boolean newValue) {
    if (currentAction == null) {
      super.setEnabled(newValue);
    } else {
      currentAction.setEnabled(newValue);
    }
  }

  /**
   * {@inheritDoc}
   */
  public Object getValue(String key) {
    if (currentAction != null) {
      return currentAction.getValue(key);
    }
    return super.getValue(key);
  }

  /**
   * {@inheritDoc}
   */
  public void putValue(String key, Object newValue) {
    if (currentAction == null) {
      super.putValue(key, newValue);
    } else {
      currentAction.putValue(key, newValue);
    }
  }

  /**
   * {@inheritDoc}
   */
  public Object[] getKeys() {
    if (currentAction instanceof AbstractAction) {
      return ((AbstractAction)currentAction).getKeys(); // NOPMD
    }
    return super.getKeys();
  }

/*^**************************************************************************^*/
/*- Package-Private Method.                                                  -*/
/*^**************************************************************************^*/

  final void firePropertyChange(PropertyChangeEvent e) { // optimization
    firePropertyChange(e.getPropertyName(), e.getOldValue(), e.getNewValue());
  }

/*^**************************************************************************^*/
/*- Inner Class.                                                             -*/
/*^**************************************************************************^*/

  /**
   * Action Property Listener.
   */
  class ActionPropertyListener implements PropertyChangeListener {
    /**
     * {@inheritDoc}
     */
    public void propertyChange(PropertyChangeEvent e) {
      firePropertyChange(e);
    }
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-15 12:34:58 Slobodan
   Initial version

*****************************************************************************^*/
