/*
 * @(#)CommonAction.java  1.0.0 / 2008-05-15
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.swing.actions;

import java.util.ResourceBundle;

import javax.swing.AbstractAction;
import javax.swing.Icon;
import javax.swing.KeyStroke;

/**
 * Abstract Common Action.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public abstract class CommonAction extends AbstractAction {

/*^**************************************************************************^*/
/*- Public Constructors.                                                     -*/
/*^**************************************************************************^*/

  /**
   * Constructs new CommonAction object.
   */
  public CommonAction() {
    super();
    setDefaults(); // NOPMD
  }

  /**
   * Constructs new CommonAction object.
   *
   * @param name the action name
   */
  public CommonAction(String name) {
    super();
    setDefaults(); // NOPMD
    if (name != null) {
      putValue(NAME, name);
    }
  }

  /**
   * Constructs new CommonAction object.
   *
   * @param name the action name
   * @param description the short description
   */
  public CommonAction(String name, String description) {
    this(name);
    if (description != null) {
      putValue(SHORT_DESCRIPTION, description);
    }
  }

  /**
   * Creates new CommonAction object.
   *
   * @param name the action name
   * @param icon the icon
   */
  public CommonAction(String name, Icon icon) {
    super();
    setDefaults(); // NOPMD
    if (name != null) {
      putValue(NAME, name);
    }
    if (icon != null) {
      putValue(SMALL_ICON, icon);
    }
  }

  /**
   * Creates new CommonAction object.
   *
   * @param name the action name
   * @param icon the icon
   * @param mnemonic the mnemonic
   * @param shortdesc the short description
   */
  public CommonAction(String name, Icon icon, int mnemonic,
                          String shortdesc) {
    this(name, icon);
    putValue(MNEMONIC_KEY, new Integer(mnemonic));
    if (shortdesc != null) {
      putValue(SHORT_DESCRIPTION, shortdesc);
    }
  }

  /**
   * Creates new CommonAction object.
   *
   * @param name the action name
   * @param icon the icon
   * @param mnemonic the mnemonic
   * @param acc the accelerator
   * @param shortdesc the short description
   */
  public CommonAction(String name, Icon icon, int mnemonic, KeyStroke acc,
                          String shortdesc) {
    this(name, icon);
    putValue(MNEMONIC_KEY, new Integer(mnemonic));
    if (acc != null) {
      putValue(ACCELERATOR_KEY, acc);
    }
    if (shortdesc != null) {
      putValue(SHORT_DESCRIPTION, shortdesc);
    }
  }

/*^**************************************************************************^*/
/*- Public Methods.                                                          -*/
/*^**************************************************************************^*/

  /**
   * Returns action's name.
   *
   * @return action's name
   */
  public String getName() {
    Object name = getValue(NAME);
    if (name instanceof String) {
      return (String)name;
    }
    return null;
  }

  /**
   * Sets action's name.
   *
   * @param name the name
   */
  public void setName(String name) {
    putValue(NAME, name);
  }

  /**
   * Returns action's mnemonic key.
   *
   * @return action's mnemonic key
   */
  public int getMnemonicKey() {
    Object mnemonic = getValue(MNEMONIC_KEY);
    if (mnemonic instanceof Integer) {
      return ((Integer)mnemonic).intValue();
    }
    return -1;
  }

  /**
   * Sets action's mnemonic key.
   *
   * @param mnemonic the mnemonic key
   */
  public void setMnemonicKey(int mnemonic) {
    putValue(MNEMONIC_KEY, new Integer(mnemonic));
  }

  /**
   * Returns action's accelerator key.
   *
   * @return action's accelerator key
   */
  public KeyStroke getAcceleratorKey() {
    Object accelerator = getValue(ACCELERATOR_KEY);
    if (accelerator instanceof KeyStroke) {
      return (KeyStroke)accelerator;
    }
    return null;
  }

  /**
   * Sets action's accelerator key.
   *
   * @param accelerator the accelerator key
   */
  public void setAcceleratorKey(KeyStroke accelerator) {
    putValue(ACCELERATOR_KEY, accelerator);
  }

  /**
   * Returns action's small icon.
   *
   * @return action's small icon
   */
  public Icon getSmallIcon() {
    Object icon = getValue(SMALL_ICON);
    if (icon instanceof Icon) {
      return (Icon)icon;
    }
    return null;
  }

  /**
   * Sets action's small icon.
   *
   * @param icon the small icon
   */
  public void setSmallIcon(Icon icon) {
    putValue(SMALL_ICON, icon);
  }

  /**
   * Returns action's small icon.
   *
   * @return action's small icon
   */
  public String getShortDescription() {
    Object shortdesc = getValue(SHORT_DESCRIPTION);
    if (shortdesc instanceof String) {
      return (String)shortdesc;
    }
    return null;
  }

  /**
   * Sets action's small icon.
   *
   * @param shortdesc the short description
   */
  public void setShortDescription(String shortdesc) {
    putValue(SHORT_DESCRIPTION, shortdesc);
  }

  /**
   * Setups this action.
   *
   * @param name action name
   * @param mnemonic action mnemonic
   * @param accelerator action accelerator
   * @param icon action icon
   * @param shortdesc short description
   */
  public void setupAction(String name, int mnemonic, KeyStroke accelerator,
                          Icon icon, String shortdesc) {
    if (name != null) {
      setName(name);
    }
    setMnemonicKey(mnemonic);
    if (accelerator != null) {
      setAcceleratorKey(accelerator);
    }
    if (icon != null) {
      setSmallIcon(icon);
    }
    if (shortdesc != null) {
      setShortDescription(shortdesc);
    }
  }

  /**
   * Setups this action.
   *
   * @param name action name
   * @param mnemonic action mnemonic
   */
  public void setupAction(String name, int mnemonic) {
    setupAction(name, mnemonic, null, null, null);
  }

/*^**************************************************************************^*/
/*- Protected Methods.                                                       -*/
/*^**************************************************************************^*/

  /**
   * Sets action default properties.
   */
  protected void setDefaults() { // NOPMD
    // empty method
  }

  /**
   * Sets name property.
   *
   * @param resourceBundle the resource bundle
   * @param resourceName the resource name
   */
  protected void setName(ResourceBundle resourceBundle, String resourceName) {
    putValue(NAME, resourceBundle.getObject(resourceName));
  }

  /**
   * Sets short description property.
   *
   * @param resourceBundle the resource bundle
   * @param resourceName the resource name
   */
  protected void setShortDescription(ResourceBundle resourceBundle,
                                     String resourceName) {
    putValue(SHORT_DESCRIPTION, resourceBundle.getObject(resourceName));
  }

  /**
   * Sets long description property.
   *
   * @param resourceBundle the resource bundle
   * @param resourceName the resource name
   */
  protected void setLongDescription(ResourceBundle resourceBundle,
                                    String resourceName) {
    putValue(LONG_DESCRIPTION, resourceBundle.getObject(resourceName));
  }

  /**
   * Sets small icon property.
   *
   * @param resourceBundle the resource bundle
   * @param resourceName the resource name
   */
  protected void setSmallIcon(ResourceBundle resourceBundle,
                              String resourceName) {
    putValue(SMALL_ICON, resourceBundle.getObject(resourceName));
  }

  /**
   * Sets action command property.
   *
   * @param resourceBundle the resource bundle
   * @param resourceName the resource name
   */
  protected void setActionCommandKey(ResourceBundle resourceBundle,
                                     String resourceName) {
    putValue(ACTION_COMMAND_KEY, resourceBundle.getObject(resourceName));
  }

  /**
   * Sets accelerator key property.
   *
   * @param resourceBundle the resource bundle
   * @param resourceName the resource name
   */
  protected void setAcceleratorKey(ResourceBundle resourceBundle,
                                   String resourceName) {
    putValue(ACCELERATOR_KEY, resourceBundle.getObject(resourceName));
  }

  /**
   * Sets mnemonic key property.
   *
   * @param resourceBundle the resource bundle
   * @param resourceName the resource name
   */
  protected void setMnemonicKey(ResourceBundle resourceBundle,
                                String resourceName) {
    putValue(MNEMONIC_KEY, resourceBundle.getObject(resourceName));
  }

}

/*^*****************************************************************************
  File History:

 - 2008-03-07 15:17:24 Slobodan
   Initial version

*****************************************************************************^*/
