/*
 * @(#)SelectableAction.java  1.0.0 / 2008-05-15
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.swing.actions;

import java.awt.event.ActionEvent;

import javax.swing.Icon;
import javax.swing.KeyStroke;

/**
 * Selectable Action.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public abstract class SelectableAction extends CommonAction {

/*^**************************************************************************^*/
/*- Public Static Field.                                                     -*/
/*^**************************************************************************^*/

  /**
   * The key that indicates whether or not the action is selected.
   */
  public static final String SELECTED = "selected"; //$NON-NLS-1$

/*^**************************************************************************^*/
/*- Public Constructors.                                                     -*/
/*^**************************************************************************^*/

  /**
   * Constructs new SelectableAction object.
   */
  public SelectableAction() {
    super();
    putValue(SELECTED, Boolean.FALSE);
  }

  /**
   * Constructs new SelectableAction object.
   *
   * @param name the action name
   */
  public SelectableAction(String name) {
    super(name);
    putValue(SELECTED, Boolean.FALSE);
  }

  /**
   * Constructs new SelectableAction object.
   *
   * @param name the action name
   * @param description the action description
   */
  public SelectableAction(String name, String description) {
    super(name, description);
    putValue(SELECTED, Boolean.FALSE);
  }

  /**
   * Constructs new SelectableAction object.
   *
   * @param name the action name
   * @param icon the action icon
   */
  public SelectableAction(String name, Icon icon) {
    super(name, icon);
    putValue(SELECTED, Boolean.FALSE);
  }

  /**
   * Constructs new SelectableAction object.
   *
   * @param name the action name
   * @param icon the action icon
   * @param mnemonic the action mnemonic
   * @param description the action description
   */
  public SelectableAction(String name, Icon icon, int mnemonic,
                          String description) {
    super(name, icon, mnemonic, description);
    putValue(SELECTED, Boolean.FALSE);
  }

  /**
   * Constructs new SelectableAction object.
   *
   * @param name the action name
   * @param icon the action icon
   * @param mnemonic the action mnemonic
   * @param acc the action accelerator
   * @param description the action description
   */
  public SelectableAction(String name, Icon icon, int mnemonic, KeyStroke acc,
                          String description) {
    super(name, icon, mnemonic, acc, description);
    putValue(SELECTED, Boolean.FALSE);
  }

/*^**************************************************************************^*/
/*- Public Methods.                                                          -*/
/*^**************************************************************************^*/

  /**
   * Sets selected state of the action.
   *
   * @param b the new selected state
   */
  public void setSelected(boolean b) {
    putValue(SELECTED, Boolean.valueOf(b));
  }

  /**
   * Returns selected state of the action.
   * (WARNING: method throws <tt>exception</tt> if there is no <tt>boolean</tt>
   *  value for the key <tt>selected</tt>)
   *
   * @return selected state of the action
   */
  public boolean isSelected() {
    return ((Boolean)getValue(SELECTED)).booleanValue();
  }

  /**
   * Invoked when an action occurs.
   *
   * @param e the action event
   */
  public void actionPerformed(ActionEvent e) {
    if (enabled) {
      boolean select = !isSelected();
      setSelected(select);
      selectActionPerformed(e, select);
    }
  }

  /**
   * Changes select state and fires action.
   *
   * @param e the ActionEvent
   * @param select the select state
   */
  public void actionSelectPerformed(ActionEvent e, boolean select) {
    if (select != isSelected()) {
      actionPerformed(e);
    }
  }

/*^**************************************************************************^*/
/*- Protected Methods.                                                       -*/
/*^**************************************************************************^*/

  /**
   * Invoked when an select action occurs.
   *
   * @param e the action event
   * @param select the select state
   */
  protected abstract void selectActionPerformed(ActionEvent e, boolean select);

}

/*^*****************************************************************************
  File History:

 - 2008-05-15 18:28:56 Slobodan
   Initial version

*****************************************************************************^*/
