/*
 * @(#)AppContainer.java  1.0.0 / 2008-05-17
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.swing.appcontainer;

import java.io.File;
import java.io.PrintStream;
import java.lang.reflect.Method;
import java.util.ResourceBundle;

import net.wrka.LibraryConfig;
import net.wrka.swing.resources.BuiltInResources;
import net.wrka.util.GPTools;

/**
 * Simple application container.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public final class AppContainer {

/*^**************************************************************************^*/
/*- Public Static Final Fields.                                              -*/
/*^**************************************************************************^*/

  /**
   * Resources string key.
   */
  public static final String RESOURCES = "AppContainerResources"; //$NON-NLS-1$

  /**
   * Error dialog title string key.
   */
  public static final String RK_ERROR_DIALOG_TITLE =
                                             "ErrorDialog.title"; //$NON-NLS-1$

  /**
   * Error dialog exception message string key.
   */
  public static final String RK_ERROR_DIALOG_MSG_EXCEPTION =
                                  "ErrorDialog.exceptionMessage"; //$NON-NLS-1$

  /**
   * Error dialog terminate message string key.
   */
  public static final String RK_ERROR_DIALOG_MSG_TERMINATE =
                                  "ErrorDialog.terminateMessage"; //$NON-NLS-1$

  /**
   * Error dialog terminated message string key.
   */
  public static final String RK_ERROR_DIALOG_MSG_TERMINATED =
                                 "ErrorDialog.terminatedMessage"; //$NON-NLS-1$
  /**
   * Error dialog options string key.
   */
  public static final String RK_ERROR_DIALOG_OPTIONS =
                                    "ErrorDialog.optionsMessage"; //$NON-NLS-1$

  /**
   * Error dialog terminate string key.
   */
  public static final String RK_ERROR_DIALOG_TERMINATE =
                                         "ErrorDialog.terminate"; //$NON-NLS-1$

  /**
   * Error dialog ignore string key.
   */
  public static final String RK_ERROR_DIALOG_IGNORE =
                                            "ErrorDialog.ignore"; //$NON-NLS-1$

  /**
   * Error dialog details string key.
   */
  public static final String RK_ERROR_DIALOG_DETAILS =
                                           "ErrorDialog.details"; //$NON-NLS-1$

  /**
   * Stack trace title string key.
   */
  public static final String RK_STACK_TRACE_TITLE =
                                              "StackTrace.title"; //$NON-NLS-1$

  /**
   * <tt>True</tt> when the class is initialized.
   */
  public static final boolean INITIALIZED;

/*^**************************************************************************^*/
/*- Private Static Fields.                                                   -*/
/*^**************************************************************************^*/

  private static Class appClass;
  private static String appClassName;
  private static String appName;
  private static File appClassPath;
  private static File appClassFile;
  static PrintStream errorPrintStream;
  private static ErrorOutputStream errorOutputStream;
  static ExceptionWatchThreadGroup applicationThreadGroup;
  private static boolean guiApplication;

/*^**************************************************************************^*/
/*- Static Initialization.                                                   -*/
/*^**************************************************************************^*/

  static {
    appClass = null;
    INITIALIZED = ApplicationUtils.initClasses();
  }

/*^**************************************************************************^*/
/*- Private Constructor.                                                     -*/
/*^**************************************************************************^*/

 /**
  * Class has no instances.
  */
  private AppContainer() {
    // empty constructor body
  }

/*^**************************************************************************^*/
/*- Public Static Methods.                                                   -*/
/*^**************************************************************************^*/

  /**
   * Returns application class name.
   *
   * @return application class name
   */
  public static String getApplicationClassName() {
    return appClassName;
  }

  /**
   * Returns application name.
   *
   * @return application name
   */
  public static String getApplicationName() {
    return appName;
  }

  /**
   * Returns application class path.
   *
   * @return application class path
   */
  public static File getApplicationClassPath() {
    return appClassPath;
  }

  /**
   * Returns application class file.
   *
   * @return application class file
   */
  public static File getApplicationClassFile() {
    return appClassFile;
  }

  /**
   * ShutsDown current application.
   */
  public static void shutDownApplicaiton() {
    GPTools.shutdownVM();
  }

  /**
   * ShutsDown current application.
   *
   * @param errCode error code
   */
  public static void shutDownApplicaiton(int errCode) {
    GPTools.shutdownVM(errCode);
  }

  /**
   * Returns <code>true</code> if application running.
   *
   * @return <code>true</code> if application running
   */
  public static boolean isRunning() {
    return (appClass != null);
  }

  /**
   * Returns <tt>true</tt> if application has GUI.
   *
   * @return <tt>true</tt> if application has GUI
   */
  public static boolean isGUIApplication() {
    return guiApplication;
  }

  /**
   * Runs the specified application.
   *
   * @param application the application
   */
  public static void run(Application application) {
    if (application == null) {
      throw new IllegalArgumentException("Null Application"); //$NON-NLS-1$
    }
    synchronized (AppContainer.class) {
      if (appClass == null) {
        appClass = application.getClass();
        appClassName = appClass.getName();
        appName = application.getName();
        if (appName == null) {
          appName = GPTools.extractClassName(appClass);
        }
        ApplicationUtils.checkJavaVMVersion();
        guiApplication = (application instanceof GUIApplication);
        AppRunner appRunner;
        if (LibraryConfig.DEBUG) {
          appRunner = new AppRunner(null, application);
          appRunner.run(); // FB: OK - debug
        } else {
          initialize();
          appRunner = new AppRunner(applicationThreadGroup, application);
          appRunner.start();
        }
        return;
      }
    }
    throw new IllegalStateException(
                               "Application is already started"); //$NON-NLS-1$
  }

  /**
   * Runs the specified application class.
   *
   * @param mainClass application class
   * @param mainArgs main arguments
   * @param applicationName application name
   * @param gui gui application
   */
  public static void run(Class mainClass, String[] mainArgs,
                         String applicationName, boolean gui) {
    if (mainClass == null) {
      throw new IllegalArgumentException("Null main class"); //$NON-NLS-1$
    }
    synchronized (AppContainer.class) {
      if (appClass == null) {
        appClass = mainClass;
        appClassName = appClass.getName();
        appName = applicationName;
        try {
          Class[] mainClassArgs = new Class[] {
            String[].class
          };
          Method appMain = appClass.getMethod("main", //$NON-NLS-1$
                                              mainClassArgs);
          ApplicationUtils.checkJavaVMVersion();
          if (appName == null) {
            appName = GPTools.extractClassName(appClass);
          }
          guiApplication = gui;
          AppClassRunner appClassRunner;
          if (LibraryConfig.DEBUG) {
            ApplicationUtils.initLabrary();
            appClassRunner = new AppClassRunner(null, appMain, mainArgs);
            appClassRunner.run(); // FB: OK - debug
          } else {
            initialize();
            ApplicationUtils.initLabrary();
            appClassRunner = new AppClassRunner(applicationThreadGroup, appMain,
                                                mainArgs);
            appClassRunner.start();
          }
        } catch (NoSuchMethodException nsme) {
          System.out.print("AppContainer: Main method is" + //$NON-NLS-1$ NOPMD
                           " not found in class \""); //$NON-NLS-1$
          System.out.print(appClassName); // NOPMD
          System.out.println("\""); //$NON-NLS-1$ NOPMD
          shutDownApplicaiton(-1);
        }
        return;
      }
    }
    throw new IllegalStateException(
                             "Application is already started"); //$NON-NLS-1$
  }

  /**
   * Runs the specified application class.
   *
   * @param mainClass application class
   * @param mainArgs main arguments
   * @param applicationName application name
   */
  public static void run(Class mainClass, String[] mainArgs,
                         String applicationName) {
    run(mainClass, mainArgs, applicationName, false);
  }

  /**
   * Runs the specified application class.
   *
   * @param mainClass application class
   * @param mainArgs main arguments
   */
  public static void run(Class mainClass, String[] mainArgs) {
    run(mainClass, mainArgs, null, true);
  }

  /**
   * Runs the specified application class.
   *
   * @param className the application class name
   * @param mainArgs the main arguments
   * @param applicationName the application name
   * @param gui the gui application
   */
  public static void run(String className, String[] mainArgs,
                         String applicationName, boolean gui) {
    if (className == null) {
      throw new IllegalArgumentException("Null className"); //$NON-NLS-1$
    }
    synchronized (AppContainer.class) {
      if (appClass == null) {
        try {
          appClass = Class.forName(className);
          className = null;
          appClassName = appClass.getName();
          appName = applicationName;
          Class[] mainClassArgs = new Class[] {
            String[].class
          };
          Method appMain = appClass.getMethod("main", //$NON-NLS-1$
                                              mainClassArgs);
          ApplicationUtils.checkJavaVMVersion();
          if (appName == null) {
            appName = GPTools.extractClassName(appClass);
          }
          guiApplication = gui;
          AppClassRunner appClassRunner;
          if (LibraryConfig.DEBUG) {
            ApplicationUtils.initLabrary();
            appClassRunner = new AppClassRunner(null, appMain, mainArgs);
            appClassRunner.run(); // FB: OK - debug
          } else {
            initialize();
            ApplicationUtils.initLabrary();
            appClassRunner = new AppClassRunner(applicationThreadGroup, appMain,
                                                mainArgs);
            appClassRunner.start();
          }
        } catch (ClassNotFoundException clnf) {
          System.out.print("Class \""); //$NON-NLS-1$ // NOPMD
          System.out.print(appClassName); // NOPMD
          System.out.println("\" is not found."); //$NON-NLS-1$ NOPMD
          shutDownApplicaiton(-1);
        } catch (NoSuchMethodException nsme) {
          System.out.print("AppContainer: Main method is" + //$NON-NLS-1$ NOPMD
                           " not found in class \""); //$NON-NLS-1$
          System.out.print(appClassName); // NOPMD
          System.out.println("\""); //$NON-NLS-1$ NOPMD
          shutDownApplicaiton(-1);
        }
        return;
      }
    }
    throw new IllegalStateException(
                               "Application is already started"); //$NON-NLS-1$
  }

  /**
   * Runs the specified application class.
   *
   * @param mainClass application class
   * @param mainArgs main arguments
   * @param applicationName application name
   */
  public static void run(String mainClass, String[] mainArgs,
                         String applicationName) {
    run(mainClass, mainArgs, applicationName, true);
  }

  /**
   * Runs the specified application class.
   *
   * @param mainClass application class name
   * @param mainArgs main arguments
   */
  public static void run(String mainClass, String[] mainArgs) {
    run(mainClass, mainArgs, null, true);
  }

/*^**************************************************************************^*/
/*- Package-Private Static Methods.                                          -*/
/*^**************************************************************************^*/

  /**
   * Returns ErrorOutputStream.
   *
   * @return ErrorOutputStream.
   */
  static ErrorOutputStream getErrorOutputStream() {
    return errorOutputStream;
  }

  /**
   * Fires error.
   */
  static void fireError() {
    ErrorHandler errorHandler = new ErrorHandler();
    errorHandler.start();
  }

  /**
   * Displays error.
   */
  static void displayError() {
    if (guiApplication) {
      ErrorDialog.display();
    } else {
      ResourceBundle rb = BuiltInResources.INSTANCE.
                                                  getResourceBundle(RESOURCES);
      System.out.println(rb.getString(RK_ERROR_DIALOG_MSG_EXCEPTION)); // NOPMD
      System.out.println(rb.getString(RK_ERROR_DIALOG_MSG_TERMINATED)); // NOPMD
      shutDownApplicaiton(-1);
    }
  }

/*^**************************************************************************^*/
/*- Private Static Method.                                                   -*/
/*^**************************************************************************^*/

  private static void initialize() { // FB: if DEBUG never called
    errorOutputStream = new ErrorOutputStream();
    errorPrintStream = new PrintStream(errorOutputStream);
    System.err.close();
    System.setErr(errorPrintStream);
    applicationThreadGroup =
          new ExceptionWatchThreadGroup("ApplicationThreadGroup", //$NON-NLS-1$
                                        true);
    String sc = GPTools.getCodeSource(appClass);
    if (sc != null) {
      appClassPath = new File(sc);
      if (appClassPath.exists()) {
        if (appClassPath.isFile()) {
          appClassFile = appClassPath;
          appClassPath = appClassPath.getParentFile();
        } else {
          String appFileName = appClass.getName();
          appFileName = appFileName.replace('.', File.separatorChar);
          appFileName = appFileName.concat(".class"); //$NON-NLS-1$
          appClassFile = new File(appClassPath, appFileName);
          if (!appClassFile.isFile() || !appClassFile.exists()) {
            appClassFile = null;
          }
        }
      } else {
        appClassPath = null;
      }
    }
  }

/*^**************************************************************************^*/
/*- Inner Classes.                                                           -*/
/*^**************************************************************************^*/

  /**
   * Application Runner.
   */
  static final class AppRunner extends Thread {
    private final Application application;
    /**
     * Constructs new AppRunner object.
     *
     * @param tg the thread group
     * @param application the application
     */
    public AppRunner(ThreadGroup tg, Application application) {
      super(tg, "ApplicationThread"); //$NON-NLS-1$
      this.application = application;
    }
    /**
     * {@inheritDoc}
     */
    public void run() {
      try {
        application.initialize();
        application.exec();
      } catch (Throwable t) { // NOPMD
        if (errorPrintStream == null) {
          t.printStackTrace(System.out);
          shutDownApplicaiton(-1);
        } else {
          t.printStackTrace(errorPrintStream);
        }
      }
    }
  }

  /**
   * Application Class Runner.
   */
  static final class AppClassRunner extends Thread {
    private final Method appMain;
    private final String[] mainArgs;
    /**
     * Constructs new AppClassRunner object.
     *
     * @param tg the thread group
     * @param appMain the main method
     * @param mainArgs the main method arguments
     */
    public AppClassRunner(ThreadGroup tg, Method appMain,
                          String[] mainArgs) { // NOPMD
      super(tg, "ApplicationThread"); //$NON-NLS-1$
      this.appMain = appMain;
      if (mainArgs == null) {
        this.mainArgs = null;
      } else {
        this.mainArgs = (String[])mainArgs.clone();
      }
    }
    /**
     * {@inheritDoc}
     */
    public void run() {
      try {
        appMain.setAccessible(true);
      } catch (Throwable t) { // NOPMD
        // empty catch block
      }
      try {
        appMain.invoke(null, new Object[] {
          mainArgs
        });
      } catch (Throwable t) { // NOPMD
        t.printStackTrace(errorPrintStream);
      }
    }
  }

  /**
   * Error Handler.
   */
  static final class ErrorHandler extends Thread {
    /**
     * Constructs new ErrorHandler object.
     *
     */
    public ErrorHandler() {
      super(applicationThreadGroup.getParent(), "ErrorHandler"); //$NON-NLS-1$
    }
    /**
     * {@inheritDoc}
     */
    public void run() {
      displayError();
    }
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-17 10:47:47 Slobodan
   Initial version

*****************************************************************************^*/
