/*
 * @(#)Application.java  1.0.0 / 2008-05-17
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.swing.appcontainer;

import java.util.Locale;
import java.util.prefs.Preferences;

import net.wrka.swing.preferences.ApplicationPreferences;
import net.wrka.swing.preferences.GeneralPreferences;
import net.wrka.swing.preferences.PersistentPreferences;
import net.wrka.util.GPTools;

/**
 * Simple Application.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public abstract class Application {

/*^**************************************************************************^*/
/*- Public Static Fields.                                                    -*/
/*^**************************************************************************^*/

  /**
   * Property <tt>name</tt>.
   */
  public static final String PROPERTY_NAME = "application.name"; //$NON-NLS-1$

  /**
   * Property <tt>description</tt>.
   */
  public static final String PROPERTY_DESCRIPTION =
                                       "application.description"; //$NON-NLS-1$

  /**
   * Property <tt>version</tt>.
   */
  public static final String PROPERTY_VERSION =
                                           "application.version"; //$NON-NLS-1$

  /**
   * Property <tt>vendor</tt>.
   */
  public static final String PROPERTY_VENDOR =
                                            "application.vendor"; //$NON-NLS-1$

/*^**************************************************************************^*/
/*- Protected Fields.                                                        -*/
/*^**************************************************************************^*/

  /**
   * <tt>True</tt> when the application is warmed up.
   */
  protected boolean warmed;

  /**
   * Application name.
   */
  protected final String name;

  /**
   * Application description.
   */
  protected final String description;

  /**
   * Application version.
   */
  protected final String version;

  /**
   * Application vendor.
   */
  protected final String vendor;

  /**
   * Application arguments.
   */
  protected final String[] args;

  /**
   * Application preferences.
   */
  protected Preferences preferences;

/*^**************************************************************************^*/
/*- Protected Constructors.                                                  -*/
/*^**************************************************************************^*/

  /**
   * Constructs new Application object.
   *
   * @param args command line arguments
   */
  protected Application(String[] args) {
    this(null, null, null, null, args);
  }

  /**
   * Constructs new Application object.
   *
   * @param name the application name
   * @param args the command line arguments
   */
  protected Application(String name, String[] args) {
    this(name, null, null, null, args);
  }

  /**
   * Constructs new Application object.
   *
   * @param name the application name
   * @param description the application description
   * @param version the application version
   * @param vendor the application vendor
   * @param args the command line arguments
   */
  protected Application(String name, String description, String version,
                        String vendor, String[] args) {
    this.args = (String[])args.clone();
    ApplicationPreferences.putArgs(args);
    if (name == null) {
      this.name = GPTools.extractClassName(getClass());
    } else {
      this.name = name;
    }
    this.description = description;
    this.version = version;
    this.vendor = vendor;
    System.setProperty(PROPERTY_NAME, name);
    if (description != null) {
      System.setProperty(PROPERTY_DESCRIPTION, description);
    }
    if (version != null) {
      System.setProperty(PROPERTY_VERSION, version);
    }
    if (vendor != null) {
      System.setProperty(PROPERTY_VENDOR, vendor);
    }
  }

/*^**************************************************************************^*/
/*- Public Methods.                                                          -*/
/*^**************************************************************************^*/

  /**
   * Returns application name.
   *
   * @return application name
   */
  public String getName() {
    return name;
  }

  /**
   * Prints information about application.
   */
  public void printInfo() {
    System.out.print(System.getProperty(PROPERTY_NAME)); // NOPMD
    System.out.print(" - "); //$NON-NLS-1$ NOPMD
    System.out.println(System.getProperty(PROPERTY_DESCRIPTION)); // NOPMD
    System.out.print("Version: "); //$NON-NLS-1$ NOPMD
    System.out.println(System.getProperty(PROPERTY_VERSION)); // NOPMD
    System.out.print("Vendor:  "); //$NON-NLS-1$ NOPMD
    System.out.println(System.getProperty(PROPERTY_VENDOR)); // NOPMD
  }

  /**
   * Returns <tt>true</tt> if this application uses native preferences.
   *
   * @return <tt>true</tt> if this application uses native preferences
   */
  public boolean usesNativePreferences() { // NOPMD
    return false;
  }

  /**
   * Executes the application.
   */
  public abstract void exec();

  /**
   * Terminates this application.
   */
  public void shutdown() {
    shutdown(0);
  }

  /**
   * Terminates this application.
   *
   * @param status the exit status.
   */
  public void shutdown(int status) {
    GPTools.shutdownVM(status);
  }

/*^**************************************************************************^*/
/*- Package-Private Methods.                                                 -*/
/*^**************************************************************************^*/

  /**
   * Initialize the application.
   */
  void initialize() {
    initPreferences();
    String locale = preferences.get(GeneralPreferences.LOCALE, null);
    if (locale != null) {
      Locale.setDefault(GPTools.parseLocale(locale));
    }
    init();
    warmed = warmUp();
  }

  /**
   * Preferences initialization.
   */
  void initPreferences() {
    PersistentPreferences.initialize(usesNativePreferences(), name,
                                     name.toLowerCase(Locale.getDefault()));
    preferences = PersistentPreferences.getUserPreferences();
    addPreferenceGroups();
  }

/*^**************************************************************************^*/
/*- Protected Methods.                                                       -*/
/*^**************************************************************************^*/

  /**
   * Custom initialization.
   */
  protected void init() { // NOPMD
    // empty method body
  }

  /**
   * Warms up application classes.
   *
   * @return <tt>false</tt> if failed
   */
  protected boolean warmUp() { // NOPMD
    return true;
  }

  /**
   * Adds preference groups.
   */
  protected void addPreferenceGroups() { // NOPMD
    // empty method body
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-17 09:50:51 Slobodan
   Initial version

*****************************************************************************^*/
