/*
 * @(#)ApplicationUtils.java  1.0.0 / 2008-05-18
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.swing.appcontainer;

import java.awt.Insets;

import javax.swing.JLabel;
import javax.swing.JOptionPane;

import net.wrka.LibraryConfig;
import net.wrka.LibraryInfo;
import net.wrka.awt.AwtTools;
import net.wrka.swing.components.JWxGriddedPanel;
import net.wrka.swing.components.JWxMultilineLabel;
import net.wrka.swing.graphics.FontUtils;
import net.wrka.swing.patch.PatchManager;
import net.wrka.swing.plaf.LookAndFeelManager;
import net.wrka.swing.plaf.smooth.SmoothLookAndFeel;
import net.wrka.swing.tooltips.ToolTipManager;
import net.wrka.util.CommonConstants;
import net.wrka.util.GPTools;

/**
 * Application Utilities.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
final class ApplicationUtils {

/*^**************************************************************************^*/
/*- Private Static Fields.                                                   -*/
/*^**************************************************************************^*/

  private static boolean classesInited;
  private static boolean libsInited;

/*^**************************************************************************^*/
/*- Private Constructor.                                                     -*/
/*^**************************************************************************^*/

  /**
   * Class has no instances.
   */
  private ApplicationUtils() {
    // empty constructor body
  }

/*^**************************************************************************^*/
/*- Public Static Methods.                                                   -*/
/*^**************************************************************************^*/

  /**
   * Initialize library classes.
   *
   * @return <tt>true</tt> if succeed.
   */
  public static boolean initClasses() {
    if (!classesInited) {
      AwtTools.initialize();
      classesInited = LibraryInfo.INITIALIZED;
    }
    return classesInited;
  }

  /**
   * Library initialization.
   */
  public static void initLabrary() {
    if (!libsInited) {
      //System.setProperty("sun.java2d.ddscale", CommonConstants.STRING_TRUE);
      classesInited |= FontUtils.INITIALIZED;
      LookAndFeelManager.initialize();
      LookAndFeelManager.installLookAndFeel(SmoothLookAndFeel.NAME,
                                            SmoothLookAndFeel.class.getName());
      if (PatchManager.APPLAY_PATCH_14) {
        ToolTipManager.setEnabled(true);
      }
      if (LibraryConfig.VERBOSE) {
        System.out.println(// NOPMD
                        "ApplicationUtils: Library initialized"); //$NON-NLS-1$
      }
      libsInited = true;
    }
  }

  /**
   * Returns <tt>true</tt> if library is initialized.
   *
   * @return <tt>true</tt> if library is initialized
   */
  public static boolean isInitialized() {
    return libsInited;
  }

  /**
   * Checks JVM Version.
   */
  public static void checkJavaVMVersion() {
    if (!GPTools.isValidVersion(LibraryInfo.JAVA_VERSION,
                                LibraryInfo.LIBRARY_PLATFORM)) {
      showInvalidVMError();
      GPTools.shutdownVM(-1);
    }
  }

/*^**************************************************************************^*/
/*- Private Static Method.                                                   -*/
/*^**************************************************************************^*/

  private static void showInvalidVMError() {
    String javaVMName = GPTools.getSystemProperty(
                                          "java.vm.name"); // $NON-NLS-1$ NOPMD
    String javaVMVersion = GPTools.getSystemProperty(
                                       "java.vm.version"); // $NON-NLS-1$ NOPMD
    String javaVMVendor = GPTools.getSystemProperty(
                                        "java.vm.vendor"); // $NON-NLS-1$ NOPMD
    System.out.println(); // NOPMD
    System.out.println("Unsuported Java Virtual Machine"); // $NON-NLS-1$ NOPMD
    System.out.println(); // NOPMD
    System.out.print("JavaVM Name:    "); // $NON-NLS-1$ NOPMD
    System.out.println(javaVMName); // NOPMD
    System.out.print("JavaVM Version: "); // $NON-NLS-1$ NOPMD
    System.out.println(javaVMVersion); // NOPMD
    System.out.print("JavaVM Vendor:  "); // $NON-NLS-1$ NOPMD
    System.out.println(javaVMVendor); // NOPMD
    System.out.println(); // NOPMD
    System.out.print("Required JavaVM version is "); // $NON-NLS-1$ NOPMD
    System.out.print(LibraryInfo.LIBRARY_PLATFORM); // NOPMD
    System.out.println(" or higher.\n"); // $NON-NLS-1$ NOPMD
    try {
      StringBuffer tsb = new StringBuffer();
      if (AppContainer.getApplicationName() == null) {
        tsb.append("ApplicationUtils"); // $NON-NLS-1$ NOPMD
      } else {
        tsb.append(AppContainer.getApplicationName());
      }
      tsb.append(" - Error"); // $NON-NLS-1$ NOPMD
      JWxGriddedPanel jwxgpJavaVMInfo = new JWxGriddedPanel(
          new Insets(2, 3, 6, 4));
      jwxgpJavaVMInfo.addComponent(
          new JLabel("Unsuported Java Virtual Machine"), 0, // $NON-NLS-1$ NOPMD
          0, 2, 1, JWxGriddedPanel.GBC_CENTER, JWxGriddedPanel.GBC_NONE, null,
          0.0, 0.0);
      jwxgpJavaVMInfo.addComponent(
          new JWxMultilineLabel(
                      "VM Name:\nVM Version:\nVM Vendor:"), // $NON-NLS-1$ NOPMD
          0, 1, 1, 1, JWxGriddedPanel.GBC_CENTER, JWxGriddedPanel.GBC_WEST,
          null, 0.0, 0.0);
      StringBuffer isb = new StringBuffer(80);
      isb.append(javaVMName);
      isb.append(CommonConstants.STRING_DEFAULT_LINE_SEPARATOR);
      isb.append(javaVMVersion);
      isb.append(CommonConstants.STRING_DEFAULT_LINE_SEPARATOR);
      isb.append(javaVMVendor);
      jwxgpJavaVMInfo.addComponent(new JWxMultilineLabel(isb.toString()),
                                   1, 1, 1, 1, JWxGriddedPanel.GBC_CENTER,
                                   JWxGriddedPanel.GBC_WEST, null, 0.0, 0.0);
      isb = new StringBuffer(60);
      isb.append("Required VM version is "); // $NON-NLS-1$ NOPMD
      isb.append(LibraryInfo.LIBRARY_PLATFORM);
      isb.append(" or higher."); // $NON-NLS-1$ NOPMD
      jwxgpJavaVMInfo.addComponent(new JWxMultilineLabel(isb.toString()),
                                   0, 2, 2, 1, JWxGriddedPanel.GBC_CENTER,
                                   JWxGriddedPanel.GBC_WEST, null, 0.0, 0.0);
      JOptionPane.showMessageDialog(null, jwxgpJavaVMInfo, tsb.toString(),
                                    JOptionPane.ERROR_MESSAGE);
    } catch (Throwable t) { // NOPMD
      // empty catch block
    }
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-18 17:04:06 Slobodan
   Initial version

*****************************************************************************^*/
