/*
 * @(#)GUIApplication.java  1.0.0 / 2008-05-17
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.swing.appcontainer;

import java.awt.Window;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.util.ResourceBundle;

import javax.swing.ImageIcon;
import javax.swing.plaf.basic.BasicHTML;

import net.wrka.swing.components.text.JWxEditorPane;
import net.wrka.swing.plaf.LookAndFeelManager;
import net.wrka.swing.preferences.GeneralPreferences;
import net.wrka.swing.preferences.PlafPreferences;
import net.wrka.swing.resources.GUIResourceManager;
import net.wrka.swing.resources.BuiltInResources;
import net.wrka.swing.splash.Splash;

/**
 * GUI Application.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public abstract class GUIApplication extends Application {

/*^**************************************************************************^*/
/*- Protected Fields.                                                        -*/
/*^**************************************************************************^*/

  /**
   * GUI resource manager.
   */
  protected GUIResourceManager guiResourceManager;

  /**
   * Application's main window.
   */
  protected Window mainWindow;

/*^**************************************************************************^*/
/*- Protected Constructors.                                                  -*/
/*^**************************************************************************^*/

  /**
   * Constructs new GUIApplication object.
   *
   * @param args command line arguments
   */
  protected GUIApplication(String[] args) {
    super(args);
  }

  /**
   * Constructs new GUIApplication object.
   *
   * @param guiResourceManager the resource manager
   * @param args the command line arguments
   */
  protected GUIApplication(GUIResourceManager guiResourceManager,
                           String[] args) {
    this(null, null, null, null, guiResourceManager, args);
  }

  /**
   * Constructs new GUIApplication object.
   *
   * @param name the application name
   * @param guiResourceManager the resource manager
   * @param args the command line arguments
   */
  protected GUIApplication(String name, GUIResourceManager guiResourceManager,
                           String[] args) {
    this(name, null, null, null, guiResourceManager, args);
  }

  /**
   * Constructs new GUIApplication object.
   *
   * @param name the application name
   * @param description the application description
   * @param version the application version
   * @param vendor the application vendor
   * @param guiResourceManager the resource manager
   * @param args the command line arguments
   */
  protected GUIApplication(String name, String description, String version,
                           String vendor, GUIResourceManager guiResourceManager,
                           String[] args) {
    super(name, description, version, vendor, args);
    this.guiResourceManager = guiResourceManager;
  }

/*^**************************************************************************^*/
/*- Public Methods.                                                          -*/
/*^**************************************************************************^*/

  /**
   * Returns application main window.
   *
   * @return application main window
   */
  public Window getMainWindow() {
    return mainWindow;
  }

/*^**************************************************************************^*/
/*- Overridden Methods.                                                      -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  public void exec() {
    if (mainWindow != null) {
      mainWindow.setVisible(true);
      mainWindow.toFront();
    }
  }

  /**
   * {@inheritDoc}
   */
  protected boolean warmUp() {
    return (BasicHTML.createHTMLView(new JWxEditorPane(),
                                    "<html>html</html>") != null); //$NON-NLS-1$
  }

  /**
   * {@inheritDoc}
   */
  void initialize() {
    initPreferences();
    LookAndFeelManager.initialize();
    ResourceBundle splRes = null;
    int spuc = getSplashUpdatesCount();
    boolean showsplash = ((spuc >= 0) && preferences.getBoolean(
                                       GeneralPreferences.SHOW_SPLASH, true));
    if (guiResourceManager == null) {
      ApplicationUtils.initLabrary();
    } else {
      if (showsplash) {
        if (guiResourceManager.containsResourceBundle(Splash.RESOURCES)) {
          splRes = guiResourceManager.getResourceBundle(Splash.RESOURCES);
        } else {
          splRes = BuiltInResources.INSTANCE.getResourceBundle(
                                                             Splash.RESOURCES);
        }
        Splash.show(getSplashImage(), spuc,
                    splRes.getString(Splash.RK_MESSAGE_START));
        Splash.update(splRes.getString(Splash.RK_MESSAGE_INITIALIZING));
      }
      ApplicationUtils.initLabrary();
      init();
      initHelp();
    }
    mainWindow = createMainWindow();
    if (mainWindow != null) {
      mainWindow.addWindowListener(new MainWindowListener());
      if (PlafPreferences.initialized()) {
        PlafPreferences.getInstance().addLnFChangeComponent(mainWindow);
      }
    }
    warmed = warmUp();
    if (showsplash && (splRes != null)) {
      Splash.close(splRes.getString(Splash.RK_MESSAGE_FINISH));
    }
  }

/*^**************************************************************************^*/
/*- Protected Methods.                                                       -*/
/*^**************************************************************************^*/

  /**
   * Create and returns application main window.
   *
   * @return application main window
   */
  protected abstract Window createMainWindow();

  /**
   * Returns splash image.
   *
   * @return splash image
   */
  protected ImageIcon getSplashImage() {
    return guiResourceManager.getImageIcon("Splash.png"); //$NON-NLS-1$
  }

  /**
   * Returns the number of splash updates.
   *
   * @return the number of splash updates
   */
  protected int getSplashUpdatesCount() {
    return -1;
  }

  /**
   * Help support initialization.
   */
  protected void initHelp() { // NOPMD
    // empty method body
  }

/*^**************************************************************************^*/
/*- Inner Class.                                                             -*/
/*^**************************************************************************^*/

  /**
   * Main Window Listener.
   */
  protected class MainWindowListener extends WindowAdapter {
    /**
     * {@inheritDoc}
     */
    public void windowClosed(WindowEvent e) {
      shutdown();
    }
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-17 10:11:08 Slobodan
   Initial version

*****************************************************************************^*/
