/*
 * @(#)GlassPaneDialogBorder.java  1.0.0 / 2008-05-16
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.swing.borders;

import java.awt.AlphaComposite;
import java.awt.Color;
import java.awt.Component;
import java.awt.Composite;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Insets;
import java.awt.Paint;

import javax.swing.LookAndFeel;
import javax.swing.border.AbstractBorder;

import net.wrka.swing.graphics.FastGradientPaint;
import net.wrka.swing.plaf.LookAndFeelChangeListener;
import net.wrka.swing.plaf.LookAndFeelManager;

/**
 * Glass Pane Dialog Border.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public class GlassPaneDialogBorder extends AbstractBorder
                                   implements LookAndFeelChangeListener {

/*^**************************************************************************^*/
/*- Private Static Final Fields.                                             -*/
/*^**************************************************************************^*/

  private static final long serialVersionUID = 2055091685828493162L;
  private static final int SHADOW_DEPTH = 7;
  private static final int SHADOW_DEPTH_P1 = SHADOW_DEPTH + 1;
  private static final Insets INSETS;
  private static final AlphaComposite SHADOW_ALPHA_COMPOSITE;

/*^**************************************************************************^*/
/*- Static Initialization.                                                   -*/
/*^**************************************************************************^*/

  static {
    INSETS = new Insets(0, 0, SHADOW_DEPTH, SHADOW_DEPTH);
    SHADOW_ALPHA_COMPOSITE = AlphaComposite.getInstance(AlphaComposite.SRC_OVER,
                                                        0.6f);
  }

/*^**************************************************************************^*/
/*- Private Fields.                                                          -*/
/*^**************************************************************************^*/

  private transient Paint verticalGradientPaint;
  private transient Paint horizontalGradientPaint;

/*^**************************************************************************^*/
/*- Public Constructor.                                                      -*/
/*^**************************************************************************^*/

  /**
   * Constructs new GlassPaneDialogBorder object.
   */
  public GlassPaneDialogBorder() {
    super();
    initColors();
    LookAndFeelManager.addLnFListener(this); // NOTE: this is weak listener list
  }

/*^**************************************************************************^*/
/*- LookAndFeelChangeListener Implementation.                                -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  public void lookAndFeelChanged(LookAndFeel oldLnF, LookAndFeel newLnF) {
    initColors();
  }

/*^**************************************************************************^*/
/*- Overridden Methods.                                                      -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  public Insets getBorderInsets(Component c) {
    return (Insets)INSETS.clone();
  }

  /**
   * {@inheritDoc}
   */
  public Insets getBorderInsets(Component c, Insets insets) {
    insets.left = INSETS.left;
    insets.top = INSETS.top;
    insets.right = INSETS.right;
    insets.bottom = INSETS.bottom;
    return insets;
  }

  /**
   * {@inheritDoc}
   */
  public void paintBorder(Component c, Graphics g, int x, int y, int w, int h) {
    if (g instanceof Graphics2D) {
      Graphics2D g2 = (Graphics2D)g;
      g2.translate(x, y);
      Composite oldc = g2.getComposite();
      g2.setComposite(SHADOW_ALPHA_COMPOSITE);
      g2.setPaint(verticalGradientPaint);
      g2.fillRect(3, h - SHADOW_DEPTH, w - SHADOW_DEPTH, SHADOW_DEPTH_P1);
      g2.setPaint(horizontalGradientPaint);
      g2.fillRect(w - SHADOW_DEPTH, 3, SHADOW_DEPTH_P1, h - 6);
      g2.translate(-x, -y);
      g2.setComposite(oldc);
    }
  }

/*^**************************************************************************^*/
/*- Protected Method.                                                        -*/
/*^**************************************************************************^*/

  /**
   * Initializes colors.
   */
  protected void initColors() {
    Color background = LookAndFeelManager.getControlColor();
    if (background == null) {
      background = Color.gray;
    }
    Color darkShadow = LookAndFeelManager.getControlDarkShadow();
    if (darkShadow == null) {
      background = Color.darkGray;
    }
    horizontalGradientPaint = new FastGradientPaint(darkShadow, background,
                                                    false);
    verticalGradientPaint = new FastGradientPaint(darkShadow, background, true);
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-16 21:42:18 Slobodan
   Initial version

*****************************************************************************^*/
