/*
 * @(#)ShadowBorder.java  1.0.0 / 2008-05-22
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.swing.borders;

import java.awt.Color;
import java.awt.Component;
import java.awt.Graphics;
import java.awt.Insets;
import java.lang.ref.SoftReference;

import javax.swing.LookAndFeel;
import javax.swing.border.AbstractBorder;
import javax.swing.plaf.UIResource;

import net.wrka.swing.plaf.LookAndFeelChangeListener;
import net.wrka.swing.plaf.LookAndFeelManager;

/**
 * Border with shadow.
 * Based on JGoodies' InternalFrameBorder (www.jgoodies.com)
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public class ShadowBorder extends AbstractBorder
                            implements LookAndFeelChangeListener {

/*^**************************************************************************^*/
/*- Private Static Final Fields.                                             -*/
/*^**************************************************************************^*/

  private static final int ALPHA1 = 150;
  private static final int ALPHA2 = 50;

/*^**************************************************************************^*/
/*- Private Static Fields.                                                   -*/
/*^**************************************************************************^*/

  private static SoftReference softRefInsets;
  private static ShadowBorder instance;
  private static WxShadowBorderUIResource uiInstance;

/*^**************************************************************************^*/
/*- Static Initialization.                                                   -*/
/*^**************************************************************************^*/

  static {
    softRefInsets = new SoftReference(new Insets(1, 1, 3, 3));
  }

/*^**************************************************************************^*/
/*- Public Static Methods.                                                   -*/
/*^**************************************************************************^*/

  /**
   * Returns shared instance.
   *
   * @return shared instance
   */
  public static ShadowBorder getInstance() {
    if (instance == null) { // NOPMD - not singleton
      instance = new ShadowBorder();
    }
    return instance;
  }

  /**
   * Returns shared UI instance.
   *
   * @return shared UI instance
   */
  public static ShadowBorder getUIInstance() {
    if (uiInstance == null) { // NOPMD - not singleton
      uiInstance = new WxShadowBorderUIResource();
    }
    return uiInstance;
  }

/*^**************************************************************************^*/
/*- Private Fields.                                                          -*/
/*^**************************************************************************^*/

  private final Insets insets;
  private Color background;
  private Color darkShadow;
  private Color lightShadow;
  private Color lighterShadow;

/*^**************************************************************************^*/
/*- Public Constructor.                                                      -*/
/*^**************************************************************************^*/

  /**
   * Constructs new ShadowBorder object.
   */
  public ShadowBorder() {
    super();
    insets = getCachedInsets();
    initColors();
    LookAndFeelManager.addLnFListener(this);
  }

/*^**************************************************************************^*/
/*- LookAndFeelChangeListener Implementation.                                -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  public void lookAndFeelChanged(LookAndFeel oldLnF, LookAndFeel newLnF) {
    initColors();
  }

/*^**************************************************************************^*/
/*- Overridden Methods.                                                      -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  public void paintBorder(Component c, Graphics g, int x, int y, int w, int h) {
    g.translate(x, y);
    // Dark border
    g.setColor(darkShadow);
    g.drawRect(0, 0, w - 3, h - 3);
    // Paint background before painting the shadow
    if (c.isOpaque()) { // c should not be opaque
      Component parent = c.getParent();
      if (parent == null) {
        g.setColor(background);
      } else {
        g.setColor(parent.getBackground());
      }
      g.fillRect(w - 2, 0, 2, h);
      g.fillRect(0, h - 2, w, 2);
    }
    // Shadow line1
    g.setColor(lightShadow);
    g.drawLine(w - 2, 1, w - 2, h - 2);
    g.drawLine(1, h - 2, w - 3, h - 2);
    // Shadow line2
    g.setColor(lighterShadow);
    g.drawLine(w - 1, 2, w - 1, h - 2);
    g.drawLine(2, h - 1, w - 2, h - 1);
    g.translate(-x, -y);
  }

  /**
   * {@inheritDoc}
   */
  public Insets getBorderInsets(Component c) {
    return (Insets)insets.clone();
  }

  /**
   * {@inheritDoc}
   */
  public Insets getBorderInsets(Component c, Insets insets) {
    Insets ti = this.insets;
    insets.left = ti.left;
    insets.top = ti.top;
    insets.right = ti.right;
    insets.bottom = ti.bottom;
    return insets;
  }

/*^**************************************************************************^*/
/*- Protected Method.                                                        -*/
/*^**************************************************************************^*/

  /**
   * Initializes colors.
   */
  protected void initColors() {
    background = LookAndFeelManager.getControlColor();
    if (background == null) {
      background = Color.gray;
    }
    darkShadow = LookAndFeelManager.getControlDarkShadow();
    if (darkShadow == null) {
      darkShadow = Color.BLACK;
    }
    lightShadow = new Color(darkShadow.getRed(), darkShadow.getGreen(),
                            darkShadow.getBlue(), ALPHA1);
    lighterShadow = new Color(darkShadow.getRed(), darkShadow.getGreen(),
                              darkShadow.getBlue(), ALPHA2);
  }

/*^**************************************************************************^*/
/*- Private Static Method.                                                   -*/
/*^**************************************************************************^*/

  private static Insets getCachedInsets() {
    Insets insets = (Insets)softRefInsets.get();
    if (insets == null) {
      insets = new Insets(1, 1, 3, 3);
      softRefInsets = new SoftReference(insets);
    }
    return insets;
  }

/*^**************************************************************************^*/
/*- Inner Class.                                                             -*/
/*^**************************************************************************^*/

  /**
   * ShadowBorder UIResource.
   */
  public static class WxShadowBorderUIResource extends ShadowBorder
                                               implements UIResource {
    // empty class body
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-22 20:29:17 Slobodan
   Initial version

*****************************************************************************^*/
