/*
 * @(#)ThinBevelBorder.java  1.0.0 / 2008-05-15
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.swing.borders;

import java.awt.Color;
import java.awt.Component;
import java.awt.Graphics;
import java.awt.Insets;

import javax.swing.LookAndFeel;
import javax.swing.border.AbstractBorder;
import javax.swing.border.Border;

import net.wrka.swing.graphics.GraphicUtils;
import net.wrka.swing.plaf.LookAndFeelChangeListener;
import net.wrka.swing.plaf.LookAndFeelManager;

/**
 * Thin Bevel Border.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public class ThinBevelBorder extends AbstractBorder
                             implements LookAndFeelChangeListener {

/*^**************************************************************************^*/
/*- Public Static Fields.                                                    -*/
/*^**************************************************************************^*/

  /**
   * Raised border type.
   */
  public static final int TYPE_RAISED = 0;

  /**
   * Lowered border type.
   */
  public static final int TYPE_LOWERED = 1;

/*^**************************************************************************^*/
/*- Private Static Fields.                                                   -*/
/*^**************************************************************************^*/

  private static final long serialVersionUID = -4536653645385803975L;
  private static final Insets INSETS = new Insets(1, 1, 1, 1);
  private static Border sheredRaisedInstance;
  private static Border sharedLoweredInstance;

/*^**************************************************************************^*/
/*- Protected Fields.                                                        -*/
/*^**************************************************************************^*/

  /**
   * Bevel type.
   */
  protected final int type;

  /**
   * Highlight factor.
   */
  protected float highlightFactor;

  /**
   * Shadow factor.
   */
  protected float shadowFactor;

  /**
   * Background color.
   */
  protected Color background;

  /**
   * Highlight color.
   */
  protected Color highlight;

  /**
   * Shadow color.
   */
  protected Color shadow;

/*^**************************************************************************^*/
/*- Public Constructors.                                                     -*/
/*^**************************************************************************^*/

  /**
   * Constructs new ThinBevelBorder object.
   *
   * @param type the border type
   */
  public ThinBevelBorder(int type) {
    this(type, 0.9f, 0.3f);
  }

  /**
   * Constructs new ThinBevelBorder object.
   *
   * @param type the border type
   * @param highlightFactor the highlight factor
   * @param shadowFactor the shadow factor
   */
  public ThinBevelBorder(int type, float highlightFactor, float shadowFactor) {
    super();
    this.type = type;
    this.highlightFactor = highlightFactor;
    this.shadowFactor = shadowFactor;
    initColors();
    LookAndFeelManager.addLnFListener(this); // NOTE: this is weak listener list
  }

/*^**************************************************************************^*/
/*- Public Static Methods.                                                   -*/
/*^**************************************************************************^*/

  /**
   * Returns shared lowered instance.
   *
   * @return shared lowered instance
   */
  public static Border getSharedLoweredInstance() {
    if (sharedLoweredInstance == null) { // NOPMD - not thread safe
      sharedLoweredInstance = new ThinBevelBorder(TYPE_LOWERED);
    }
    return sharedLoweredInstance;
  }

  /**
   * Returns raised lowered instance.
   *
   * @return raised lowered instance
   */
  public static Border getSheredRaisedInstance() {
    if (sheredRaisedInstance == null) { // NOPMD - not thread safe
      sheredRaisedInstance = new ThinBevelBorder(TYPE_RAISED);
    }
    return sheredRaisedInstance;
  }

/*^**************************************************************************^*/
/*- LookAndFeelChangeListener Implementation.                                -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  public void lookAndFeelChanged(LookAndFeel oldLnF, LookAndFeel newLnF) {
    initColors();
  }

/*^**************************************************************************^*/
/*- Overridden Methods.                                                      -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  public boolean isBorderOpaque() {
    return true;
  }

  /**
   * {@inheritDoc}
   */
  public Insets getBorderInsets(Component c) {
    return (Insets)INSETS.clone();
  }

  /**
   * {@inheritDoc}
   */
  public Insets getBorderInsets(Component c, Insets insets) {
    insets.left = INSETS.left;
    insets.top = INSETS.top;
    insets.right = INSETS.right;
    insets.bottom = INSETS.bottom;
    return insets;
  }

  /**
   * {@inheritDoc}
   */
  public void paintBorder(Component c, Graphics g, int x, int y, int width,
                          int height) {
    if (type == TYPE_RAISED) {
      paintRaisedBevel(c, g, x, y, width, height);
    } else if (type == TYPE_LOWERED) {
      paintLoweredBevel(c, g, x, y, width, height);
    }
  }

/*^**************************************************************************^*/
/*- Protected Methods.                                                       -*/
/*^**************************************************************************^*/

  /**
   * Initializes colors.
   */
  protected void initColors() {
    highlight = null;
    shadow = null;
    background = LookAndFeelManager.getControlColor();
    if (background == null) {
      background = Color.gray;
    }
  }

  /**
   * Returns shadow color.
   *
   * @param c the component
   *
   * @return shadow color
   */
  protected Color getShadowColor(Component c) {
    Color bg = c.getBackground();
    if (background.equals(bg)) {
      if (shadow == null) {
        shadow = GraphicUtils.darkerColor(bg, shadowFactor);
      }
      return shadow;
    }
    return GraphicUtils.darkerColor(bg, shadowFactor);
  }

  /**
   * Returns highlight color.
   *
   * @param c the component
   *
   * @return highlight color
   */
  protected Color getHighlightColor(Component c) {
    Color bg = c.getBackground();
    if (background.equals(bg)) {
      if (highlight == null) {
        highlight = GraphicUtils.brighterColor(background, highlightFactor);
      }
      return highlight;
    }
    return GraphicUtils.brighterColor(bg, highlightFactor);
  }

  /**
   * Paints raised bevel.
   *
   * @param c the component
   * @param g the graphics
   * @param x the x coordinate
   * @param y the y coordinate
   * @param w the width
   * @param h the height
   */
  protected void paintRaisedBevel(Component c, Graphics g, int x, int y,
                                  int w, int h)  {
    GraphicUtils.draw2ColorsRect(g, x, y, w, h, getHighlightColor(c),
                                 getShadowColor(c));
  }

  /**
   * Paints lowered bevel.
   *
   * @param c the component
   * @param g the graphics
   * @param x the x coordinate
   * @param y the y coordinate
   * @param w the width
   * @param h the height
   */
  protected void paintLoweredBevel(Component c, Graphics g, int x, int y, int w,
                                   int h)  {
    GraphicUtils.draw2ColorsRect(g, x, y, w, h, getShadowColor(c),
                                 getHighlightColor(c));
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-15 22:00:47 Slobodan
   Initial version

*****************************************************************************^*/
