/*
 * @(#)TitleBorder.java  1.0.0 / 2008-05-22
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.swing.borders;

import java.awt.Color;
import java.awt.Component;
import java.awt.Graphics;
import java.awt.Insets;

import javax.swing.LookAndFeel;
import javax.swing.border.AbstractBorder;

import net.wrka.swing.plaf.LookAndFeelChangeListener;
import net.wrka.swing.plaf.LookAndFeelManager;

/**
 * Title Pane Border.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public class TitleBorder extends AbstractBorder
                         implements LookAndFeelChangeListener {

/*^**************************************************************************^*/
/*- Public Static Fields.                                                    -*/
/*^**************************************************************************^*/

  /**
   * Raised type.
   */
  public static final int RAISED = 0;

  /**
   * Flat type.
   */
  public static final int FLAT = 2;

/*^**************************************************************************^*/
/*- Private Static Fields.                                                   -*/
/*^**************************************************************************^*/

  private static final Insets INSETS_RAISED = new Insets(1, 1, 1, 0);
  private static final Insets INSETS_FLAT = new Insets(0, 0, 1, 0);

/*^**************************************************************************^*/
/*- Private Fields.                                                          -*/
/*^**************************************************************************^*/

  private int type;
  private Color defhighlight;
  private Color highlight;
  private Color darkShadow;

/*^**************************************************************************^*/
/*- Public Constructors.                                                     -*/
/*^**************************************************************************^*/

  /**
   * Constructs new TitleBorder object.
   */
  public TitleBorder() {
    this(FLAT);
  }

  /**
   * Constructs new TitleBorder object.
   *
   * @param type the border type
   */
  public TitleBorder(int type) {
    super();
    this.type = type;
    initColors();
    LookAndFeelManager.addLnFListener(this);
  }

/*^**************************************************************************^*/
/*- Public Methods.                                                          -*/
/*^**************************************************************************^*/

  /**
   * Returns highlight color.
   *
   * @return highlight color
   */
  public Color getHighlightColor() {
    return highlight;
  }

  /**
   * Sets highlight color.
   *
   * @param color the highlight color
   */
  public void setHighlightColor(Color color) {
    highlight = color;
  }

/*^**************************************************************************^*/
/*- LookAndFeelChangeListener Implementation.                                -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  public void lookAndFeelChanged(LookAndFeel oldLnF, LookAndFeel newLnF) {
    initColors();
  }

/*^**************************************************************************^*/
/*- Overridden Methods.                                                      -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  public Insets getBorderInsets(Component c) {
    if (type == RAISED) {
      return (Insets)INSETS_RAISED.clone();
    }
    return (Insets)INSETS_FLAT.clone();
  }

  /**
   * {@inheritDoc}
   */
  public boolean isBorderOpaque() {
    return true;
  }

  /**
   * {@inheritDoc}
   */
  public void paintBorder(Component c, Graphics g, int x, int y, int width,
                          int height) {
    if (type == RAISED) {
      paintRaisedBevel(c, g, x, y, width, height);
    }
    g.setColor(darkShadow);
    g.drawLine(0, height - 1, width, height - 1);
  }

/*^**************************************************************************^*/
/*- Protected Methods.                                                       -*/
/*^**************************************************************************^*/

  /**
   * Initializes color.
   */
  protected void initColors() {
    darkShadow = LookAndFeelManager.getControlDarkShadow();
    if (darkShadow == null) {
      darkShadow = Color.BLACK;
    }
    defhighlight = LookAndFeelManager.getControlColor();
    if (defhighlight == null) {
      highlight = Color.gray;
    } else {
      defhighlight = defhighlight.brighter();
    }
  }

  /**
   * Paints raised bevel.
   *
   * @param c the component
   * @param g the graphics
   * @param x the x coordinate
   * @param y the y coordinate
   * @param w the width
   * @param h the height
   */
  protected void paintRaisedBevel(Component c, Graphics g, int x, int y, int w,
                                  int h) {
    Color oldColor = g.getColor();
    g.translate(x, y);
    if (highlight == null) {
      g.setColor(defhighlight);
    } else {
      g.setColor(highlight);
    }
    g.drawLine(0, 0, 0, h - 1);
    g.drawLine(1, 0, w - 1, 0);
    g.translate(-x, -y);
    g.setColor(oldColor);
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-22 20:35:57 Slobodan
   Initial version

*****************************************************************************^*/
