/*
 * @(#)JWxFlatSplitPane.java  1.0.0 / 2008-05-21
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.swing.components;

import java.awt.Component;
import java.awt.Insets;
import java.awt.event.ComponentEvent;
import java.awt.event.ComponentListener;

import javax.swing.JButton;
import javax.swing.JSplitPane;
import javax.swing.UIManager;
import javax.swing.border.EmptyBorder;
import javax.swing.plaf.SplitPaneUI;

import net.wrka.swing.plaf.LookAndFeelManager;
import net.wrka.swing.plaf.basic.BasicWxFlatSplitPaneUI;

/**
 * Flat Split Pane.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public class JWxFlatSplitPane extends JSplitPane {

/*^**************************************************************************^*/
/*- Protected Static Fields.                                                 -*/
/*^**************************************************************************^*/

  private static final EmptyBorder EMPTY_BORDER = new EmptyBorder(0, 0, 0, 0);
  private static final String UI_CLASS_ID = "WxFlatSplitPaneUI"; //$NON-NLS-1$

/*^**************************************************************************^*/
/*- Private Fields.                                                          -*/
/*^**************************************************************************^*/

  private boolean blocksds;
  private int divsize;
  private final ComponentListener cl;

/*^**************************************************************************^*/
/*- Public Constructors.                                                     -*/
/*^**************************************************************************^*/

  /**
   * Constructs new JWxFlatSplitPane object.
   */
  public JWxFlatSplitPane() {
    this(JSplitPane.HORIZONTAL_SPLIT, false,
        new JButton(UIManager.getString(
                                 "SplitPane.leftButtonText")), //$NON-NLS-1$
        new JButton(UIManager.getString(
                                  "SplitPane.rightButtonText"))); //$NON-NLS-1$
  }

  /**
   * Constructs new JWxFlatSplitPane object.
   *
   * @param orientation the orientation
   */
  public JWxFlatSplitPane(int orientation) {
    this(orientation, false);
  }

  /**
   * Constructs new JWxFlatSplitPane object.
   *
   * @param orientation the orientation
   * @param continuousLayout <tt>true</tt> if the layout is continuous
   */
  public JWxFlatSplitPane(int orientation, boolean continuousLayout) {
    this(orientation, continuousLayout, null, null);
  }

  /**
   * Constructs new JWxFlatSplitPane object.
   *
   * @param orientation the orientation
   * @param leftComponent the left component
   * @param rightComponent the right component
   */
  public JWxFlatSplitPane(int orientation, Component leftComponent,
                          Component rightComponent) {
    this(orientation, false, leftComponent, rightComponent);
  }

  /**
   * Constructs new JWxFlatSplitPane object.
   *
   * @param orientation the orientation
   * @param continuousLayout <tt>true</tt> if the layout is continuous
   * @param leftComponent the left component
   * @param rightComponent the right component
   */
  public JWxFlatSplitPane(int orientation, boolean continuousLayout,
                          Component leftComponent, Component rightComponent) {
    super(orientation, continuousLayout, leftComponent, rightComponent);
    setDividerSize(6);
    setBorder(EMPTY_BORDER);
    cl = new SplitComponentAdapter();
    Component c = getLeftComponent();
    if (c != null) {
      c.removeComponentListener(cl);
      c.addComponentListener(cl);
    }
    c = getRightComponent();
    if (c != null) {
      c.removeComponentListener(cl);
      c.addComponentListener(cl);
    }
  }

/*^**************************************************************************^*/
/*- Overridden Methods.                                                      -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  public void setLeftComponent(Component comp) {
    Component leftComp = getLeftComponent();
    if ((leftComp != null) && (cl != null)) {
      leftComp.removeComponentListener(cl);
    }
    super.setLeftComponent(comp);
    if ((comp != null) && (cl != null)) {
      comp.addComponentListener(cl);
    }
  }

  /**
   * {@inheritDoc}
   */
  public void setRightComponent(Component comp) {
    Component rightComp = getRightComponent();
    if ((rightComp != null) && (cl != null)) {
      rightComp.removeComponentListener(cl);
    }
    super.setRightComponent(comp);
    if ((comp != null) && (cl != null)) {
      comp.addComponentListener(cl);
    }
  }

  /**
   * {@inheritDoc}
   */
  public void setDividerSize(int newDivSize) {
    if (!blocksds) {
      divsize = newDivSize;
      super.setDividerSize(newDivSize);
    }
  }

  /**
   * {@inheritDoc}
   */
  public String getUIClassID() {
    return UI_CLASS_ID;
  }

  /**
   * {@inheritDoc}
   */
  public void updateUI() {
    setUI((SplitPaneUI)LookAndFeelManager.getComponentUI(this));
  }

  /**
   * {@inheritDoc}
   */
  public void setUI(SplitPaneUI ui) {
    blocksds = true;
    if (ui == null) {
      super.setUI(new BasicWxFlatSplitPaneUI());
    } else {
      super.setUI(ui);
    }
    blocksds = false;
  }

/*^**************************************************************************^*/
/*- Protected Methods.                                                       -*/
/*^**************************************************************************^*/

  /**
   * Called when component is shown.
   *
   * @param e the component event
   */
  protected void onComponentShow(ComponentEvent e) {
    if ((rightComponent != null) && (leftComponent != null) &&
        rightComponent.isVisible() && leftComponent.isVisible()) {
      if (lastDividerLocation <= 0) {
        if (orientation == JSplitPane.HORIZONTAL_SPLIT) {
          lastDividerLocation = getWidth() -
                                rightComponent.getPreferredSize().width;
        } else {
          lastDividerLocation = getHeight() -
                                rightComponent.getPreferredSize().height;
        }
      }
      super.setDividerSize(divsize);
      setDividerLocation(lastDividerLocation);
    }
  }

  /**
   * Called when component is hidden.
   *
   * @param e the component event
   */
  protected void onComponentHide(ComponentEvent e) {
    Insets insets = getInsets();
    if (orientation == JSplitPane.HORIZONTAL_SPLIT) {
      Component rc = getRightComponent();
      if (rc == e.getComponent()) {
        lastDividerLocation = getWidth() - rc.getWidth() - insets.left -
                              insets.right - getDividerSize() + 1;
      } else {
        lastDividerLocation = getWidth() - getLeftComponent().getWidth() -
                              insets.left - 1;
      }
    } else {
      Component bc = getBottomComponent();
      if (bc == e.getComponent()) {
        lastDividerLocation = getHeight() - bc.getHeight() - insets.top -
                              insets.bottom - getDividerSize() + 1;
      } else {
        lastDividerLocation = getHeight() - getTopComponent().getHeight() -
                              insets.top + 1;
      }
    }
    super.setDividerSize(0);
  }

/*^**************************************************************************^*/
/*- Inner Class.                                                             -*/
/*^**************************************************************************^*/

  /**
   * Split component adapter.
   */
  final class SplitComponentAdapter implements ComponentListener {
    /**
     * {@inheritDoc}
     */
    public void componentHidden(ComponentEvent e) {
      onComponentHide(e);
    }
    /**
     * {@inheritDoc}
     */
    public void componentShown(ComponentEvent e) {
      onComponentShow(e);
    }
    /**
     * {@inheritDoc}
     */
    public void componentResized(ComponentEvent e) {
      // empty method body
    }
    /**
     * {@inheritDoc}
     */
    public void componentMoved(ComponentEvent e) {
      // empty method body
    }
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-21 22:08:26 Slobodan
   Initial version

*****************************************************************************^*/
