/*
 * @(#)JWxGlassPane.java  1.0.0 / 2008-05-21
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.swing.components;

import java.awt.AlphaComposite;
import java.awt.Color;
import java.awt.Component;
import java.awt.Composite;
import java.awt.Container;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.GridBagLayout;
import java.awt.event.KeyAdapter;
import java.awt.event.MouseAdapter;

import javax.swing.JComponent;
import javax.swing.JPanel;

import net.wrka.util.SimpleArrayList;

/**
 * Blocking GlassPane.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public class JWxGlassPane extends JPanel {

/*^**************************************************************************^*/
/*- Private Static Fields.                                                   -*/
/*^**************************************************************************^*/

  private static final EmptyMouseAdapter EMPTY_MOUSE_ADAPTER;
  private static final EmptyKeyAdapter EMPTY_KEY_ADAPTER;

/*^**************************************************************************^*/
/*- Static Initialization.                                                   -*/
/*^**************************************************************************^*/

  static {
    EMPTY_MOUSE_ADAPTER = new EmptyMouseAdapter();
    EMPTY_KEY_ADAPTER = new EmptyKeyAdapter();
  }

/*^**************************************************************************^*/
/*- Private Fields.                                                          -*/
/*^**************************************************************************^*/

  private boolean addingEnabled;
  private boolean canceled;
  private float alpha;
  private boolean paint;
  private Color transluctColor;
  private JComponent dialogComponent;
  private Composite alphaComposite;
  private SimpleArrayList fileredComponents;

/*^**************************************************************************^*/
/*- Public Constructors.                                                     -*/
/*^**************************************************************************^*/

  /**
   * Constructs new JWxGlassPane object.
   *
   * @param dcomp the dialog component
   */
  public JWxGlassPane(JComponent dcomp) {
    this(dcomp, 0.3f, null, true);
  }

  /**
   * Constructs new JWxGlassPane object.
   *
   * @param alpha the alpha intensity
   */
  public JWxGlassPane(float alpha) {
    this(null, alpha, null, true);
  }

  /**
   * Constructs new JWxGlassPane object.
   */
  public JWxGlassPane() {
    this(null, 0.3f, null, true);
  }

  /**
   * Constructs new JWxGlassPane object.
   *
   * @param transComponents the transluct components
   */
  public JWxGlassPane(JComponent[] transComponents) {
    this(null, 0.0f, null, false);
    registerTransparentComponents(transComponents);
  }

  /**
   * Constructs new JWxGlassPane object.
   *
   * @param dcomp the dialog component
   * @param alpha the alpha intensity
   * @param transluctColor transluct color
   * @param waitCursor if <tt>true</tt> wait cursor will be set
   */
  public JWxGlassPane(JComponent dcomp, float alpha, Color transluctColor,
                      boolean waitCursor) {
    super(new GridBagLayout());
    setOpaque(false);
    setRequestFocusEnabled(true);
    setMinimumSize(new Dimension(0, 0));
    setMaximumSize(new Dimension(Short.MAX_VALUE, Short.MAX_VALUE));
    setTransluctColor(transluctColor);
    setAlpha(alpha);
    addMouseListener(EMPTY_MOUSE_ADAPTER);
    addKeyListener(EMPTY_KEY_ADAPTER);
    if (dcomp != null) {
      setDialogComponent(dcomp);
    }
    if (waitCursor) {
      setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
    }
    super.setVisible(false);
  }

/*^**************************************************************************^*/
/*- Public Static Methods.                                                   -*/
/*^**************************************************************************^*/

  /**
   * Returns glass pane with default block dialog.
   *
   * @param blockMessage the block message
   * @param cancelMessage the cancel message
   * @param cancelButtonText the cancel button text
   *
   * @return glass pane
   */
  public static JWxGlassPane getDefaultInstance(String blockMessage,
                                                String cancelMessage,
                                                String cancelButtonText) {
    JWxGlassPane newInstance = new JWxGlassPane();
    GlassPaneDialog glassDialog = new GlassPaneDialog(newInstance, blockMessage,
                                                      cancelMessage,
                                                      cancelButtonText);
    newInstance.setDialogComponent(glassDialog);
    return newInstance;
  }

  /**
   * Returns transparent glass pane.
   *
   * @return transparent glass pane
   */
  public static JWxGlassPane getTransparentInstance() {
    return new JWxGlassPane(null, 0.0f, null, false);
  }

/*^**************************************************************************^*/
/*- Public Methods.                                                          -*/
/*^**************************************************************************^*/

  /**
   * Returns glass pane alpha intensity.
   *
   * @return glass pane alpha intensity
   */
  public float getAlpha() {
    return alpha;
  }

  /**
   * Sets glass pane alpha intensity.
   *
   * @param alpha the new alpha intensity
   */
  public void setAlpha(float alpha) {
    if ((alpha > 1) || (alpha < 0)) {
      alpha = 0;
    } else {
      this.alpha = alpha;
      alphaComposite = AlphaComposite.getInstance(AlphaComposite.SRC_OVER,
                                                  alpha);
    }
    paint = (alpha > 0);
  }

  /**
   * Returns transluct color.
   *
   * @return transluct color
   */
  public Color getTransluctColor() {
    return transluctColor;
  }

  /**
   * Sets transluct color.
   *
   * @param transluctColor the new transluct color
   */
  public void setTransluctColor(Color transluctColor) {
    if (transluctColor == null) {
      this.transluctColor = Color.darkGray;
    } else {
      this.transluctColor = transluctColor;
    }
  }

  /**
   * Sets wait cursor.
   *
   * @param waitCursor if <tt>true</tt> wait cursor is set
   */
  public void setWaitCursor(boolean waitCursor) {
    if (waitCursor) {
      setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
    } else {
      setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
    }
  }

  /**
   * Returns dialog component.
   *
   * @return dialog component
   */
  public JComponent getDialogComponent() {
    return dialogComponent;
  }

  /**
   * Sets dialog component.
   *
   * @param dcomp the dialog component
   */
  public void setDialogComponent(JComponent dcomp) {
    removeAll();
    if (dcomp != null) {
      addingEnabled = true;
      add(dcomp);
      addingEnabled = false;
      dialogComponent = dcomp;
      dialogComponent.setCursor(Cursor.getDefaultCursor());
    }
  }

  /**
   * Registers transparent region.
   *
   * @param c the transparent component
   */
  public void registerTransparentComponent(JComponent c) {
    if (fileredComponents == null) {
      fileredComponents = new SimpleArrayList(10);
    }
    if (!fileredComponents.contains(c)) {
      fileredComponents.add(c);
    }
  }

  /**
   * Registers transparent regions.
   *
   * @param comps the transparent components
   */
  public void registerTransparentComponents(JComponent[] comps) {
    if (comps != null) {
      if (fileredComponents == null) {
        fileredComponents = new SimpleArrayList(comps.length);
      }
      for (int i = 0; i < comps.length; i++) {
        if (!fileredComponents.contains(comps[i])) {
          fileredComponents.add(comps[i]);
        }
      }
    }
  }

  /**
   * Unregisters transparent region.
   *
   * @param c the transparent component
   */
  public void unregisterTransparentComponent(JComponent c) {
    if (fileredComponents != null) {
      fileredComponents.remove(c);
      if (fileredComponents.isEmpty()) {
        fileredComponents = null;
      }
    }
  }

  /**
   * Unregisters all transparent regions.
   */
  public void unregisterAllTransparentComponent() {
    if (fileredComponents != null) {
      fileredComponents.clear();
      fileredComponents = null;
    }
  }

  /**
   * Activates this glass pane.
   */
  public void block() {
    canceled = false;
    if (!isVisible()) {
      setVisible(true);
      requestFocus();
    }
  }

  /**
   * Deactivates this glass pane.
   */
  public void release() {
    if (isVisible()) {
      setVisible(false);
      transferFocus();
    }
  }

  /**
   * Returns <tt>true</tt> if cancel state is set.
   *
   * @return <tt>true</tt> if cancel state is set
   */
  public boolean isCanceled() {
    return canceled;
  }

  /**
   * Sets glass pane cancel state.
   */
  public void cancel() {
    canceled = true;
  }

/*^**************************************************************************^*/
/*- Overridden Methods.                                                      -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  public void setVisible(boolean visible) {
    super.setVisible(visible);
    if (visible) {
      block();
    }
  }

  /**
   * {@inheritDoc}
   */
  public boolean contains(int x, int y) {
    if ((fileredComponents != null) && isVisible()) {
      for (int i = fileredComponents.size() - 1; i >= 0; i--) {
        JComponent comp = (JComponent)fileredComponents.get(i);
        int cx = x - comp.getX();
        int cy = y - comp.getY();
        Container parent = comp.getParent();
        while ((parent != null) && (parent != getParent())) {
          int px = parent.getX();
          int py = parent.getY();
          cx = cx - px;
          cy = cy - py;
          parent = parent.getParent();
        }
        if (comp.contains(cx, cy)) {
          return false;
        }
      }
    }
    return true;
  }

  /**
   * {@inheritDoc}
   */
  protected void addImpl(Component comp, Object constraints, int index) {
    if (addingEnabled) {
      super.addImpl(comp, constraints, index);
    }
  }

  /**
   * {@inheritDoc}
   */
  protected void paintComponent(Graphics g) {
    if (paint) {
      Graphics2D g2 = (Graphics2D)g;
      Composite oldc = g2.getComposite();
      g2.setComposite(alphaComposite);
      g2.setColor(transluctColor);
      g2.fillRect(0, 0, getWidth() - 1, getHeight() - 1);
      g2.setComposite(oldc);
    }
  }

/*^**************************************************************************^*/
/*- Inner Classes.                                                           -*/
/*^**************************************************************************^*/

  /**
   * Empty Mouse Adapter.
   */
  protected static final class EmptyMouseAdapter extends MouseAdapter {
    // empty class body
  }

  /**
   * Empty Key Adapter.
   */
  protected static final class EmptyKeyAdapter extends KeyAdapter {
    // empty class body
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-21 20:53:03 Slobodan
   Initial version

*****************************************************************************^*/
