/*
 * @(#)JWxGradientPanel.java  1.0.0 / 2008-05-22
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.swing.components;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.Insets;
import java.awt.Rectangle;
import java.awt.geom.Rectangle2D;

import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.plaf.PanelUI;

import net.wrka.swing.graphics.FontUtils;
import net.wrka.swing.graphics.GraphicUtils;
import net.wrka.swing.plaf.LookAndFeelManager;

/**
 * Gradient panel with caption.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public class JWxGradientPanel extends JPanel {

/*^**************************************************************************^*/
/*- Public Static Fields.                                                    -*/
/*^**************************************************************************^*/

  /**
   * Property key <i>secondary background</i>.
   */
  public static final String SECONDARY_BACKGROUND =
                                           "secondarybackground"; //$NON-NLS-1$

  /**
   * Property key <i>caption</i>.
   */
  public static final String CAPTION = "caption"; //$NON-NLS-1$

  /**
   * Gradient type <i>no gradient</i>.
   */
  public static final int NO_GRADIENT = 0;

  /**
   * Gradient type <i>horizontal</i>.
   */
  public static final int HORIZONTAL_GRADIENT = 1;

  /**
   * Gradient type <i>vertical</i>.
   */
  public static final int VERTICAL_GRADIENT = 2;

  /**
   * Gradient type <i>diagonal</i>.
   */
  public static final int DIAGONAL_GRADIENT = 3;

  /**
   * Caption position <i>center</i>.
   */
  public static final int CENTER_CAPTION = 0;

  /**
   * Caption position <i>center</i>.
   */
  public static final int NORTH_CAPTION = 1;

  /**
   * Caption position <i>north east</i>.
   */
  public static final int NORTH_EAST_CAPTION = 2;

  /**
   * Caption position <i>east</i>.
   */
  public static final int EAST_CAPTION = 3;

  /**
   * Caption position <i>south east</i>.
   */
  public static final int SOUTH_EAST_CAPTION = 4;

  /**
   * Caption position <i>south</i>.
   */
  public static final int SOUTH_CAPTION = 5;

  /**
   * Caption position <i>south west</i>.
   */
  public static final int SOUTH_WEST_CAPTION = 6;

  /**
   * Caption position <i>west</i>.
   */
  public static final int WEST_CAPTION = 7;

  /**
   * Caption position <i>north west</i>.
   */
  public static final int NORTH_WEST_CAPTION = 8;

/*^**************************************************************************^*/
/*- Private Static Fields.                                                   -*/
/*^**************************************************************************^*/

  private static final String UI_CLASS_ID = "WxGradientPanelUI"; //$NON-NLS-1$
  private static final Rectangle VIEW_RECT = new Rectangle();

/*^**************************************************************************^*/
/*- Private Fields.                                                          -*/
/*^**************************************************************************^*/

  private Color secondaryBackground;
  private float intensity;
  private float org;
  private int orientation;
  private int captionPosition;
  private int horizontalGap;
  private int verticalGap;
  private String caption;

/*^**************************************************************************^*/
/*- Public Constructors.                                                     -*/
/*^**************************************************************************^*/

  /**
   * Constructs new JWxGradientPanel object.
   *
   * @param background the background color
   * @param secondaryBackground the secondary background color
   * @param orientation the gradient orientation
   * @param intensity the gradient intensity
   * @param caption the panel caption
   * @param captionPosition the caption position
   */
  public JWxGradientPanel(Color background, Color secondaryBackground,
                          int orientation, float intensity, String caption,
                          int captionPosition) {
    this(background, secondaryBackground,  orientation, intensity, caption,
         captionPosition, null);
  }

  /**
   * Constructs new JWxGradientPanel object.
   */
  public JWxGradientPanel() {
    super();
  }

  /**
   * Constructs new JWxGradientPanel object.
   *
   * @param secondaryBackground the secondary background color
   */
  public JWxGradientPanel(Color secondaryBackground) {
    this(secondaryBackground, HORIZONTAL_GRADIENT);
  }

  /**
   * Constructs new JWxGradientPanel object.
   *
   * @param background the background color
   * @param orientation the gradient orientation
   */
  public JWxGradientPanel(Color background, int orientation) {
    super();
    setSecondaryBackground(background);
    setGradientOrientation(orientation);
  }

  /**
   * Constructs new JWxGradientPanel object.
   *
   * @param background the background color
   * @param secondaryBackground the secondary background color
   */
  public JWxGradientPanel(Color background, Color secondaryBackground) {
    this(background, secondaryBackground, 1.0f);
  }

  /**
   * Constructs new JWxGradientPanel object.
   *
   * @param background the background color
   * @param secondaryBackground the secondary background color
   * @param intensity the gradient intensity
   */
  public JWxGradientPanel(Color background, Color secondaryBackground,
                          float intensity) {
    this(background, secondaryBackground, HORIZONTAL_GRADIENT, intensity);
  }

  /**
   * Constructs new JWxGradientPanel object.
   *
   * @param background the background color
   * @param secondaryBackground the secondary background color
   * @param orientation the gradient orientation
   */
  public JWxGradientPanel(Color background, Color secondaryBackground,
                          int orientation) {
    this(background, secondaryBackground, orientation, 1.0f);
  }

  /**
   * Constructs new JWxGradientPanel object.
   *
   * @param background the background color
   * @param secondaryBackground the secondary background color
   * @param orientation the gradient orientation
   * @param intensity the gradient intensity
   */
  public JWxGradientPanel(Color background, Color secondaryBackground,
                          int orientation, float intensity) {
    this(background, secondaryBackground,  orientation, intensity, null);
  }

  /**
   * Constructs new JWxGradientPanel object.
   *
   * @param background the background color
   * @param secondaryBackground the secondary background color
   * @param caption the panel caption
   */
  public JWxGradientPanel(Color background, Color secondaryBackground,
                          String caption) {
    this(background, secondaryBackground, HORIZONTAL_GRADIENT, 1.0f, caption,
         NORTH_WEST_CAPTION);
  }

  /**
   * Constructs new JWxGradientPanel object.
   *
   * @param background the background color
   * @param secondaryBackground the secondary background color
   * @param intensity the gradient intensity
   * @param caption the panel caption
   */
  public JWxGradientPanel(Color background, Color secondaryBackground,
                          float intensity, String caption) {
    this(background, secondaryBackground, HORIZONTAL_GRADIENT, intensity,
         caption, NORTH_WEST_CAPTION);
  }

  /**
   * Constructs new JWxGradientPanel object.
   *
   * @param background the background color
   * @param secondaryBackground the secondary background color
   * @param caption the panel caption
   * @param captionPosition the caption position
   * @param font the caption font
   */
  public JWxGradientPanel(Color background, Color secondaryBackground,
                          String caption, int captionPosition, Font font) {
    this(background, secondaryBackground, HORIZONTAL_GRADIENT, 1.0f, caption,
         captionPosition, font);
  }

  /**
   * Constructs new JWxGradientPanel object.
   *
   * @param background the background color
   * @param secondaryBackground the secondary background color
   * @param intensity the gradient intensity
   * @param caption the panel caption
   * @param captionPosition the caption position
   * @param font the caption font
   */
  public JWxGradientPanel(Color background, Color secondaryBackground,
                          float intensity, String caption, int captionPosition,
                          Font font) {
    this(background, secondaryBackground, HORIZONTAL_GRADIENT, intensity,
         caption, captionPosition, font);
  }

  /**
   * Constructs new JWxGradientPanel object.
   *
   * @param background the background color
   * @param secondaryBackground the secondary background color
   * @param caption the panel caption
   * @param captionPosition the caption position
   */
  public JWxGradientPanel(Color background, Color secondaryBackground,
                          String caption, int captionPosition) {
    this(background, secondaryBackground, HORIZONTAL_GRADIENT, caption,
         captionPosition);
  }

  /**
   * Constructs new JWxGradientPanel object.
   *
   * @param background the background color
   * @param secondaryBackground the secondary background color
   * @param orientation the gradient orientation
   * @param caption the panel caption
   */
  public JWxGradientPanel(Color background, Color secondaryBackground,
                          int orientation, String caption) {
    this(background, secondaryBackground, orientation, 1.0f, caption,
         JLabel.WEST);
  }

  /**
   * Constructs new JWxGradientPanel object.
   *
   * @param background the background color
   * @param secondaryBackground the secondary background color
   * @param orientation the gradient orientation
   * @param intensity the gradient intensity
   * @param caption the panel caption
   */
  public JWxGradientPanel(Color background, Color secondaryBackground,
                          int orientation, float intensity, String caption) {
    this(background, secondaryBackground,  orientation, intensity, caption,
         JLabel.WEST);
  }

  /**
   * Constructs new JWxGradientPanel object.
   *
   * @param background the background color
   * @param secondaryBackground the secondary background color
   * @param orientation the gradient orientation
   * @param caption the panel caption
   * @param captionPosition the caption position
   */
  public JWxGradientPanel(Color background, Color secondaryBackground,
                          int orientation, String caption,
                          int captionPosition) {
    this(background, secondaryBackground,  orientation, 1.0f, caption,
         captionPosition, null);
  }

  /**
   * Constructs new JWxGradientPanel object.
   *
   * @param background the background color
   * @param secondaryBackground the secondary background color
   * @param orientation the gradient orientation
   * @param intensity the gradient intensity
   * @param caption the panel caption
   * @param captionPosition the caption position
   * @param font the caption font
   */
  public JWxGradientPanel(Color background, Color secondaryBackground,
                          int orientation, float intensity, String caption,
                          int captionPosition, Font font) {
    super();
    setBackground(background);
    setSecondaryBackground(secondaryBackground);
    this.orientation = orientation;
    this.caption = caption;
    this.intensity = intensity;
    this.captionPosition = captionPosition;
    if (font != null) {
      setFont(font);
    }
  }

/*^**************************************************************************^*/
/*- Public Methods.                                                          -*/
/*^**************************************************************************^*/

  /**
   * Returns secondary background.
   *
   * @return secondary background
   */
  public Color getSecondaryBackground() {
    return secondaryBackground;
  }

  /**
   * Sets secondary background.
   *
   * @param sbg the secondary background
   */
  public void setSecondaryBackground(Color sbg) {
    Color oldBg = secondaryBackground;
    secondaryBackground = sbg;
    if (((oldBg != null) && !oldBg.equals(sbg)) ||
        ((sbg != null) && !sbg.equals(oldBg))) {
      repaint();
      firePropertyChange(SECONDARY_BACKGROUND, oldBg, sbg);
    }
  }

  /**
   * Returns caption.
   *
   * @return caption
   */
  public String getCaption() {
    return caption;
  }

  /**
   * Sets caption.
   *
   * @param caption the caption
   */
  public void setCaption(String caption) {
    String oldCaption = this.caption;
    this.caption = caption;
    if (((oldCaption != null) && !oldCaption.equals(caption)) ||
        ((caption != null) && !caption.equals(oldCaption))) {
      repaint();
      firePropertyChange(CAPTION, oldCaption, caption);
    }
  }

  /**
   * Returns caption position.
   *
   * @return caption position
   */
  public int getCaptionPosition() {
    return captionPosition;
  }

  /**
   * Sets caption position.
   *
   * @param cp the caption position
   */
  public void setCaptionPosition(int cp) {
    if (cp != captionPosition) {
      captionPosition = cp;
      repaint();
    }
  }

  /**
   * Returns horizontal text gap.
   *
   * @return horizontal text gap
   */
  public int getHorizontalGap() {
    return horizontalGap;
  }

  /**
   * Sets horizontal text gap.
   *
   * @param gap the horizontal text gap
   */
  public void setHorizontalGap(int gap) {
    horizontalGap = gap;
  }

  /**
   * Returns vertical text gap.
   *
   * @return vertical text gap
   */
  public int getVerticalGap() {
    return verticalGap;
  }

  /**
   * Sets vertical text gap.
   *
   * @param gap the vertical text gap
   */
  public void setVerticalGap(int gap) {
    verticalGap = gap;
  }

  /**
   * Returns gradient orientation.
   *
   * @return gradient orientation
   */
  public int getGradientOrientation() {
    return orientation;
  }

  /**
   * Sets gradient orientation.
   *
   * @param orientation the gradient orientation
   */
  public void setGradientOrientation(int orientation) {
    this.orientation = orientation;
  }

  /**
   * Returns gradient intensity.
   *
   * @return gradient intensity
   */
  public float getGradientIntensity() {
    return intensity;
  }

  /**
   * Sets gradient intensity.
   *
   * @param intensity the gradient intensity
   */
  public void setGradientIntensity(float intensity) {
    this.intensity = intensity;
  }

  /**
   * Returns gradient origin.
   *
   * @return gradient origin
   */
  public float getGradientOrg() {
    return org;
  }

  /**
   * Sets gradient origin.
   *
   * @param org the gradient origin.
   */
  public void setGradientOrg(float org) {
    this.org = org;
  }

/*^**************************************************************************^*/
/*- Overridden Methods.                                                      -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  public Dimension getPreferredSize() {
    Dimension sd = super.getPreferredSize();
    if ((caption == null) || (caption.length() == 0)) {
      return sd;
    }
    Font font = getFont();
    if (font == null) {
      font = FontUtils.DEFAULT_FONT;
    }
    FontMetrics fm = getFontMetrics(font);
    int width = fm.stringWidth(caption) + horizontalGap;
    if (width < sd.width) {
      width = sd.width;
    }
    int height = fm.getHeight() + verticalGap;
    if (height < sd.height) {
      height = sd.height;
    }
    return new Dimension(width, height);
  }

  /**
   * {@inheritDoc}
   */
  protected void paintComponent(Graphics g) {
    if (ui == null) {
      if (isOpaque()) {
        VIEW_RECT.width = getWidth();
        VIEW_RECT.height = getHeight();
        switch (orientation) {
          case HORIZONTAL_GRADIENT:
            GraphicUtils.drawHorizontalGradient(g, getSecondaryBackground(),
                                                getBackground(), VIEW_RECT,
                                                intensity, org);
            break;
          case VERTICAL_GRADIENT:
            GraphicUtils.drawVerticalGradient(g, getSecondaryBackground(),
                                              getBackground(), VIEW_RECT,
                                              intensity, org);
            break;
          case DIAGONAL_GRADIENT:
            GraphicUtils.drawDiagonalGradient(g, getSecondaryBackground(),
                                              getBackground(), VIEW_RECT,
                                              intensity, org);
            break;
          default:
            g.setColor(getBackground());
            g.fillRect(VIEW_RECT.x, VIEW_RECT.y, VIEW_RECT.width,
                       VIEW_RECT.height);
        }
      }
      if ((caption != null) && (caption.length() > 0)) {
        g.setFont(getFont());
        g.setColor(getForeground());
        Rectangle2D textRect = g.getFontMetrics().getStringBounds(caption, g);
        g.drawString(caption, calcHTextPosiotion(textRect),
                     calcVTextPosiotion(g));
      }
    } else {
      super.paintComponent(g);
    }
  }

  /**
   * {@inheritDoc}
   */
  public String getUIClassID() {
    return UI_CLASS_ID;
  }

  /**
   * {@inheritDoc}
   */
  public void updateUI() {
    setUI((PanelUI)LookAndFeelManager.getComponentUI(this));
  }

/*^**************************************************************************^*/
/*- Private Methods.                                                         -*/
/*^**************************************************************************^*/

  private int calcHTextPosiotion(Rectangle2D textRect) {
    Insets i = getInsets();
    int hp;
    switch (captionPosition) {
      case EAST_CAPTION:
      case NORTH_EAST_CAPTION:
      case SOUTH_EAST_CAPTION:
        hp = getWidth() - (int)textRect.getWidth() - i.right - horizontalGap;
        break;
      case NORTH_CAPTION:
      case CENTER_CAPTION:
      case SOUTH_CAPTION:
        hp = ((getWidth() - (int)textRect.getWidth()) >> 1) - i.left - i.right;
        break;
      default:
        hp = i.left + horizontalGap;
        break;
    }
    return hp;
  }

  private int calcVTextPosiotion(Graphics g) {
    Insets i = getInsets();
    int vp;
    switch (captionPosition) {
      case SOUTH_CAPTION:
      case SOUTH_EAST_CAPTION:
      case SOUTH_WEST_CAPTION:
        vp = getHeight() - g.getFontMetrics().getDescent() - verticalGap;
        break;
      case WEST_CAPTION:
      case CENTER_CAPTION:
      case EAST_CAPTION:
        vp = ((getHeight() + g.getFontMetrics().getAscent()) >> 1) - i.top -
             i.bottom;
        break;
      default:
        vp = i.top + g.getFontMetrics().getAscent() + verticalGap;
        break;
    }
    return vp;
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-22 08:43:15 Slobodan
   Initial version

*****************************************************************************^*/
