/*
 * @(#)JWxGriddedPanel.java  1.0.0 / 2008-05-22
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.swing.components;

import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.LayoutManager;

import javax.swing.JPanel;

/**
 * JPanel with grid bag layout.
 *
 * Based on JGriddedPanel (author: James Tan jamestan@earthling.net)
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public class JWxGriddedPanel extends JPanel {

/*^**************************************************************************^*/
/*- Public Static Final Fields.                                              -*/
/*^**************************************************************************^*/

  /**
   * GridBagConstraints constant <i>RELATIVE</i>.
   */
  public static final int GBC_RELATIVE = GridBagConstraints.RELATIVE;

  /**
   * GridBagConstraints constant <i>REMAINDER</i>.
   */
  public static final int GBC_REMAINDER = GridBagConstraints.REMAINDER;

  /**
   * GridBagConstraints constant <i>HORIZONTAL</i>.
   */
  public static final int GBC_HORIZONTAL = GridBagConstraints.HORIZONTAL;

  /**
   * GridBagConstraints constant <i>VERTICAL</i>.
   */
  public static final int GBC_VERTICAL = GridBagConstraints.VERTICAL;

  /**
   * GridBagConstraints constant <i>NONE</i>.
   */
  public static final int GBC_NONE = GridBagConstraints.NONE;

  /**
   * GridBagConstraints constant <i>WEST</i>.
   */
  public static final int GBC_WEST = GridBagConstraints.WEST;

  /**
   * GridBagConstraints constant <i>EAST</i>.
   */
  public static final int GBC_EAST = GridBagConstraints.EAST;

  /**
   * GridBagConstraints constant <i>SOUTH</i>.
   */
  public static final int GBC_SOUTH = GridBagConstraints.SOUTH;

  /**
   * GridBagConstraints constant <i>SOUTHWEST</i>.
   */
  public static final int GBC_SOUTHWEST = GridBagConstraints.SOUTHWEST;

  /**
   * GridBagConstraints constant <i>SOUTHEAST</i>.
   */
  public static final int GBC_SOUTHEAST = GridBagConstraints.SOUTHEAST;

  /**
   * GridBagConstraints constant <i>NORTH</i>.
   */
  public static final int GBC_NORTH = GridBagConstraints.NORTH;

  /**
   * GridBagConstraints constant <i>NORTHWEST</i>.
   */
  public static final int GBC_NORTHWEST = GridBagConstraints.NORTHWEST;

  /**
   * GridBagConstraints constant <i>NORTHEAST</i>.
   */
  public static final int GBC_NORTHEAST = GridBagConstraints.NORTHEAST;

  /**
   * GridBagConstraints constant <i>CENTER</i>.
   */
  public static final int GBC_CENTER = GridBagConstraints.CENTER;

  /**
   * GridBagConstraints constant <i>BOTH</i>.
   */
  public static final int GBC_BOTH = GridBagConstraints.BOTH;

/*^**************************************************************************^*/
/*- Private Static Field.                                                    -*/
/*^**************************************************************************^*/

  private static final Insets DEF_INSETS = new Insets(2, 2, 2, 2);

/*^**************************************************************************^*/
/*- Private Fields.                                                          -*/
/*^**************************************************************************^*/

  private GridBagConstraints constraints;
  private Insets insets;
  private boolean invalidLayout;

/*^**************************************************************************^*/
/*- Public Constructors.                                                     -*/
/*^**************************************************************************^*/

  /**
   * Constructs new JWxGriddedPanel object.
   */
  public JWxGriddedPanel() {
    this((Insets)null);
  }

  /**
   * Creates a grid bag layout panel using the specified insets constraints.
   *
   * @param insets specified insets
   */
  public JWxGriddedPanel(Insets insets) {
    super(new GridBagLayout());
    constraints = new GridBagConstraints();
    constraints.anchor = GridBagConstraints.WEST;
    setConstraintsInsets(insets);
  }

  /**
   * Constructs new JWxGriddedPanel object.
   *
   * @param insets the insets
   * @param size the size
   * @param c the color
   */
  public JWxGriddedPanel(Insets insets, Dimension size, Color c) {
    this(insets);
    setSize(size);
    setPreferredSize(size);
    setMinimumSize(size);
    setBackground(c);
  }

/*^**************************************************************************^*/
/*- Protected Constructor.                                                   -*/
/*^**************************************************************************^*/

  /**
   * Constructs new JWxGriddedPanel object.
   *
   * @param layout the layout manager
   */
  protected JWxGriddedPanel(LayoutManager layout) {
    super(layout);
  }

/*^**************************************************************************^*/
/*- Public Methods.                                                          -*/
/*^**************************************************************************^*/

  /**
   * Sets constraint insets.
   *
   * @param insets the insets
   */
  public void setConstraintsInsets(Insets insets) {
    if (insets == null) {
      this.insets = DEF_INSETS;
    } else {
      this.insets = insets;
    }
    constraints.insets = this.insets;
  }

  /**
   * Returns constraint insets.
   *
   * @return constraint insets
   */
  public Insets getConstraintsInsets() {
    return (Insets)insets.clone();
  }

  /**
   * Sets the layout manager for this panel.
   *
   * @param mgr the specified layout manager
   */
  public final void setLayout(LayoutManager mgr) {
    super.setLayout(mgr);
    invalidLayout = (!(mgr instanceof GridBagLayout));
  }

  /**
   * Adds the component to the specified row and column.
   *
   * @param component the component to be added
   * @param col the specified column
   * @param row the specified row
   */
  public final void addComponent(Component component, int col, int row) {
    addComponent(component, col, row, 1, 1, GBC_WEST, GBC_NONE);
  }

  /**
   * Adds the component to the specified row and column.
   *
   * @param component the component to be added
   * @param col the specified column
   * @param row the specified row
   * @param insets the specified inserts
   */
  public final void addComponent(Component component, int col, int row,
                                 Insets insets) {
    addComponent(component, col, row, 1, 1, GBC_WEST, GBC_NONE, insets);
  }

  /**
   * Adds the component to the specified row and column that spans across a
   * specified number of columns and rows.
   *
   * @param component the component to be added
   * @param col the specified column
   * @param row the specified row
   * @param width the specified width
   * @param height the specified height
   */
  public final void addComponent(Component component, int col, int row,
                                 int width, int height) {
    addComponent(component, col, row, width, height, GBC_WEST, GBC_NONE);
  }

  /**
   * Adds the component to the specified row and column that spans across a
   * specified number of columns and rows.
   *
   * @param component the component to be added
   * @param col the specified column
   * @param row the specified row
   * @param width the specified width
   * @param height the specified height
   * @param insets the specified inserts
   */
  public final void addComponent(Component component, int col, int row,
                                 int width, int height, Insets insets) {
    addComponent(component, col, row, width, height, GBC_WEST, GBC_NONE,
                 insets);
  }

  /**
   * Adds the component to the specified row and column that spans across a
   * specified number of columns and rows with the specified filling direction
   * and an anchoring position.
   *
   * @param component the component to be added
   * @param col the column
   * @param row the row
   * @param width the width
   * @param height the height
   * @param anchor the anchor
   * @param fill the fill
   */
  public final void addComponent(Component component, int col, int row,
                                 int width, int height, int anchor, int fill) {
    addComponent(component, col, row, width, height, anchor, fill, null);
  }

  /**
   * Adds the component to the specified row and column that spans across a
   * specified number of columns and rows with the specified filling direction
   * and an anchoring position.
   *
   * @param component the component to be added
   * @param col the column
   * @param row the row
   * @param width the width
   * @param height the height
   * @param anchor the anchor
   * @param fill the fill
   * @param insets the insets
   */
  public final void addComponent(Component component, int col, int row,
                                 int width, int height, int anchor, int fill,
                                 Insets insets) {
    addComponent(component, col, row, width, height, anchor, fill, insets, 0.0,
                 0.0);
  }

  /**
   * Adds the component to the specified row and column that spans across a
   * specified number of columns and rows with the specified filling direction
   * and an anchoring position.
   *
   * @param component the component to be added
   * @param col the column
   * @param row the row
   * @param width the width
   * @param height the height
   * @param anchor the anchor
   * @param fill the fill
   * @param iTop the insets top
   * @param iLeft the insets left
   * @param iBottom the insets bottom
   * @param iRight the insets right
   */
  public final void addComponent(Component component, int col, int row,
                                 int width, int height, int anchor, int fill,
                                 int iTop, int iLeft, int iBottom, int iRight) {
    addComponent(component, col, row, width, height, anchor, fill, iTop, iLeft,
                 iBottom, iRight, 0.0, 0.0);
  }

  /**
   * Adds the component to the specified row and column that spans across a
   * specified number of columns and rows with the specified filling direction
   * and an anchoring position.
   *
   * @param component the component to be added
   * @param col the column
   * @param row the row
   * @param width the width
   * @param height the height
   * @param anchor the anchor
   * @param fill the fill
   * @param insets the insets
   * @param weightx the x weight
   * @param weighty the y weight
   */
  public final void addComponent(Component component, int col, int row,
                                 int width, int height, int anchor, int fill,
                                 Insets insets, double weightx,
                                 double weighty) {
    addComponent(component, col, row, width, height, anchor, fill, insets,
                 weightx, weighty, 0, 0);
  }

  /**
   * Adds the component to the specified row and column that spans across a
   * specified number of columns and rows with the specified filling direction
   * and an anchoring position.
   *
   * @param component the component to be added
   * @param col the column
   * @param row the row
   * @param width the width
   * @param height the height
   * @param anchor the anchor
   * @param fill the fill
   * @param iTop the insets top
   * @param iLeft the insets left
   * @param iBottom the insets bottom
   * @param iRight the insets right
   * @param weightx the x weight
   * @param weighty the y weight
   */
  public final void addComponent(Component component, int col, int row,
                                 int width, int height, int anchor, int fill,
                                 int iTop, int iLeft, int iBottom, int iRight,
                                 double weightx, double weighty) {
    addComponent(component, col, row, width, height, anchor, fill, iTop, iLeft,
                 iBottom, iRight, weightx, weighty, 0, 0);
  }

  /**
   * Adds the component to the specified row and column that spans across a
   * specified number of columns and rows with the specified filling direction
   * and an anchoring position.
   *
   * @param component the component to be added
   * @param col the column
   * @param row the row
   * @param width the width
   * @param height the height
   * @param anchor the anchor
   * @param fill the fill
   * @param insets the insets
   * @param weightx the x weight
   * @param weighty the y weight
   * @param ipadx the y pad
   * @param ipady the y pad
   */
  public final void addComponent(Component component, int col, int row,
                                 int width, int height, int anchor, int fill,
                                 Insets insets, double weightx, double weighty,
                                 int ipadx, int ipady) {
    if (invalidLayout) {
      throw new IllegalStateException("Invalid layout"); //$NON-NLS-1$
    }
    if (insets == null) {
      constraints.insets = this.insets;
    } else {
      constraints.insets = insets;
    }
    constraints.gridx = col;
    constraints.gridy = row;
    constraints.gridwidth = width;
    constraints.gridheight = height;
    constraints.anchor = anchor;
    constraints.ipadx = ipadx;
    constraints.ipady = ipady;
    constraints.weightx = weightx;
    constraints.weighty = weighty;
    constraints.fill = fill;
    add(component, constraints);
  }

  /**
   * Adds the component to the specified row and column that spans across a
   * specified number of columns and rows with the specified filling direction
   * and an anchoring position.
   *
   * @param component the component to be added
   * @param col the column
   * @param row the row
   * @param width the width
   * @param height the height
   * @param anchor the anchor
   * @param fill the fill
   * @param iTop the insets top
   * @param iLeft the insets left
   * @param iBottom the insets bottom
   * @param iRight the insets right
   * @param weightx the x weight
   * @param weighty the y weight
   * @param ipadx the y pad
   * @param ipady the y pad
   */
  public final void addComponent(Component component, int col, int row,
                                 int width, int height, int anchor, int fill,
                                 int iTop, int iLeft, int iBottom, int iRight,
                                 double weightx, double weighty,
                                 int ipadx, int ipady) {
    if (invalidLayout) {
      throw new IllegalStateException("Invalid layout"); //$NON-NLS-1$
    }
    constraints.insets.top = iTop;
    constraints.insets.left = iLeft;
    constraints.insets.bottom = iBottom;
    constraints.insets.right = iRight;
    constraints.gridx = col;
    constraints.gridy = row;
    constraints.gridwidth = width;
    constraints.gridheight = height;
    constraints.anchor = anchor;
    constraints.ipadx = ipadx;
    constraints.ipady = ipady;
    constraints.weightx = weightx;
    constraints.weighty = weighty;
    constraints.fill = fill;
    add(component, constraints);
  }

  /**
   * Adds the component to the specified row and column that anchors at the
   * specified position.
   *
   * @param component the component to be added
   * @param col the column
   * @param row the row
   * @param anchor the anchor
   */
  public final void addAnchoredComponent(Component component, int col, int row,
                                         int anchor) {
    addComponent(component, col, row, 1, 1, anchor, GBC_NONE, null, 1.0, 1.0);
  }

  /**
   * Adds the component to the specified row and column that anchors at the
   * specified position.
   *
   * @param component the component to be added
   * @param col the column
   * @param row the row
   * @param anchor the anchor
   * @param insets the insets
   */
  public final void addAnchoredComponent(Component component, int col,
                                         int row, int anchor, Insets insets) {
    addComponent(component, col, row, 1, 1, anchor, GBC_NONE, insets, 1.0, 1.0);
  }

  /**
   * Adds the component to the specified row and column that spans across a
   * specified number of columns and rows that anchors at the specified
   * position.
   *
   * @param component the component to be added
   * @param col the column
   * @param row the row
   * @param width the width
   * @param height the height
   * @param anchor the anchor
   */
  public final void addAnchoredComponent(Component component, int col, int row,
                                         int width, int height, int anchor) {
    addComponent(component, col, row, width, height, anchor, GBC_NONE, null,
                 1.0, 1.0);
  }

  /**
   * Adds the component to the specified row and column that spans across a
   * specified number of columns and rows that anchors at the specified
   * position.
   *
   * @param component the component to be added
   * @param col the column
   * @param row the row
   * @param width the width
   * @param height the height
   * @param anchor the anchor
   * @param insets the insets
   */
  public final void addAnchoredComponent(Component component, int col, int row,
                                         int width, int height, int anchor,
                                         Insets insets) {
    addComponent(component, col, row, width, height, anchor, GBC_NONE, insets,
                 1.0, 1.0);
  }

  /**
   * Adds the component to the specified row and column with the specified
   * filling direction.
   *
   * @param component the component to be added
   * @param col the column
   * @param row the row
   */
  public final void addFilledComponent(Component component, int col, int row) {
    addComponent(component, col, row, 1, 1, GBC_WEST, GBC_HORIZONTAL);
  }

  /**
   * Adds the component to the specified row and column with the specified
   * filling direction.
   *
   * @param component the component to be added
   * @param col the column
   * @param row the row
   * @param insets the insets
   */
  public final void addFilledComponent(Component component, int col, int row,
                                       Insets insets) {
    addComponent(component, col, row, 1, 1, GBC_WEST, GBC_BOTH, insets);
  }

  /**
   * Adds the component to the specified row and column with the specified
   * filling direction.
   *
   * @param component the component to be added
   * @param col the column
   * @param row the row
   * @param width the width
   * @param height the height
   */
  public final void addFilledComponent(Component component, int col, int row,
                                       int width, int height) {
    addComponent(component, col, row, width, height, GBC_WEST, GBC_BOTH);
  }

  /**
   * Adds the component to the specified row and column with the specified
   * filling direction.
   *
   * @param component the component to be added
   * @param col the column
   * @param row the row
   * @param fill the fill
   */
  public final void addFilledComponent(Component component, int col, int row,
                                       int fill) {
    addComponent(component, col, row, 1, 1, GBC_WEST, fill);
  }

  /**
   * Adds the component to the specified row and column with the specified
   * filling direction.
   *
   * @param component the component to be added
   * @param col the column
   * @param row the row
   * @param fill the fill
   * @param insets the insets
   */
  public final void addFilledComponent(Component component, int col, int row,
                                       int fill, Insets insets) {
    addComponent(component, col, row, 1, 1, GBC_WEST, fill, insets);
  }

  /**
   * Adds the component to the specified row and column that spans across
   * a specified number of columns and rows with the specified filling
   * direction.
   *
   * @param component the component to be added
   * @param col the column
   * @param row the row
   * @param width the width
   * @param height the height
   * @param fill the fill
   */
  public final void addFilledComponent(Component component, int col, int row,
                                       int width, int height, int fill) {
    addComponent(component, col, row, width, height, GBC_WEST, fill);
  }

  /**
   * Adds the component to the specified row and column that spans across
   * a specified number of columns and rows with the specified filling
   * direction.
   *
   * @param component the component to be added
   * @param col the column
   * @param row the row
   */
  public final void addFilledResizeableComponent(Component component,
                                                 int col, int row) {
    addComponent(component, col, row, 1, 1, GBC_CENTER, GBC_BOTH, null, 1.0,
                 1.0);
  }

  /**
   * Adds the component to the specified row and column that spans across
   * a specified number of columns and rows with the specified filling
   * direction.
   *
   * @param component the component to be added
   * @param col the column
   * @param row the row
   * @param insets the insets
   */
  public final void addFilledResizeableComponent(Component component,
                                                 int col, int row,
                                                 Insets insets) {
    addComponent(component, col, row, 1, 1, GBC_CENTER, GBC_BOTH, insets, 1.0,
                 1.0);
  }

  /**
   * Adds the component to the specified row and column that spans across
   * a specified number of columns and rows with the specified filling
   * direction.
   *
   * @param component the component to be added
   * @param col the column
   * @param row the row
   * @param width the width
   * @param height the height
   */
  public final void addFilledResizeableComponent(Component component,
                                                 int col, int row, int width,
                                                 int height) {
    addComponent(component, col, row, width, height, GBC_CENTER, GBC_BOTH, null,
                 1.0, 1.0);
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-22 09:01:35 Slobodan
   Initial version

*****************************************************************************^*/
