/*
 * @(#)JWxHtmlBrowser.java  1.0.0 / 2008-05-20
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.swing.components;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.event.ActionEvent;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.LinkedList;
import java.util.ResourceBundle;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.JComponent;
import javax.swing.JScrollPane;
import javax.swing.SwingUtilities;
import javax.swing.event.HyperlinkEvent;
import javax.swing.event.HyperlinkListener;
import javax.swing.text.Document;
import javax.swing.text.html.HTMLDocument;
import javax.swing.text.html.HTMLFrameHyperlinkEvent;

import net.wrka.natives.ExternalLauncher;
import net.wrka.swing.components.text.JWxEditorPane;
import net.wrka.swing.resources.BuiltInResources;

/**
 * Simple HTML Browser.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public class JWxHtmlBrowser extends JComponent {

/*^**************************************************************************^*/
/*- Public Static Fields.                                                    -*/
/*^**************************************************************************^*/

  /**
   * Browser resources.
   */
  public static final String RESOURCES =
                                       "JWxHtmlBrowserResources"; //$NON-NLS-1$

  /**
   * Resource key <i>message failed</i>.
   */
  public static final String RK_MESSAGE_FAILED =
                                  "JWxHtmlBrowser.messageFailed"; //$NON-NLS-1$

  /**
   * Resource key <i>message failed file start</i>.
   */
  public static final String RK_MESSAGE_FAILED_FILE_START =
                         "JWxHtmlBrowser.messageFailedFileStart"; //$NON-NLS-1$

  /**
   * Resource key <i>message failed file end</i>.
   */
  public static final String RK_MESSAGE_FAILED_FILE_END =
                           "JWxHtmlBrowser.messageFailedFileEnd"; //$NON-NLS-1$

  /**
   * Resource key <i>back label</i>.
   */
  public static final String RK_LABEL_BACK =
                                           "JWxHtmlBrowser.back"; //$NON-NLS-1$

  /**
   * Resource key <i>go back label</i>.
   */
  public static final String RK_LABEL_GO_BACK =
                                         "JWxHtmlBrowser.goBack"; //$NON-NLS-1$

  /**
   * Resource key <i>forward label</i>.
   */
  public static final String RK_LABEL_FORWARD =
                                        "JWxHtmlBrowser.forward"; //$NON-NLS-1$

  /**
   * Resource key <i>go forward label</i>.
   */
  public static final String RK_LABEL_GO_FORWARD =
                                      "JWxHtmlBrowser.goForward"; //$NON-NLS-1$

  /**
   * Resource key <i>home label</i>.
   */
  public static final String RK_LABEL_HOME =
                                           "JWxHtmlBrowser.home"; //$NON-NLS-1$

  /**
   * Resource key <i>go home label</i>.
   */
  public static final String RK_LABEL_GO_HOME =
                                         "JWxHtmlBrowser.goHome"; //$NON-NLS-1$

  /**
   * Resource key <i>reload label</i>.
   */
  public static final String RK_LABEL_RELOAD =
                                         "JWxHtmlBrowser.reload"; //$NON-NLS-1$

  /**
   * Resource key <i>do reload label</i>.
   */
  public static final String RK_LABEL_DO_REALOD =
                                       "JWxHtmlBrowser.doReload"; //$NON-NLS-1$

  /**
   * Empty HTML page.
   */
  public static final String EMPTY_PAGE = "<HTML></HTML>"; //$NON-NLS-1$

/*^**************************************************************************^*/
/*- Package-Private Fields.                                                  -*/
/*^**************************************************************************^*/

  LinkedList backHistory;
  LinkedList forwardHistory;
  HyperlinkEvent heCurrent;
  HyperlinkEvent heHome;
  JWxEditorPane htmlBrowser;
  String messageFailed;
  String messageFailedFileStart;
  String messageFailedFileEnd;
  String labelBack;
  String labelGoBack;
  String labelForward;
  String labelGoForward;
  String labelHome;
  String labelGoHome;
  String labelReload;
  String labelDoReload;
  Action actionBack;
  Action actionForward;
  Action actionReload;
  Action actionHome;
  HyperlinkListener hyperlinkListener;
  boolean blocListener;
  boolean blockNavigation;
  ExternalWaiter externalWaiter;

/*^**************************************************************************^*/
/*- Protected Constructor.                                                   -*/
/*^**************************************************************************^*/

  /**
   * Constructs new HTMLBrowser object.
   */
  protected JWxHtmlBrowser() {
    super();
    htmlBrowser = new JWxEditorPane(false);
    htmlBrowser.setLineWrap(true);
    backHistory = new LinkedList();
    forwardHistory = new LinkedList();
    hyperlinkListener = createHyperLinkListener();
    htmlBrowser.addHyperlinkListener(hyperlinkListener);
    setPreferredSize(new Dimension(250, 250));
    setLayout(new BorderLayout());
    add(new JScrollPane(htmlBrowser), BorderLayout.CENTER);
  }

/*^**************************************************************************^*/
/*- Public Constructors.                                                     -*/
/*^**************************************************************************^*/

  /**
   * Constructs new HTMLBrowser object.
   *
   * @param home the home page url
   */
  public JWxHtmlBrowser(URL home) {
    this();
    loadDefaults();
    createActions();
    heHome = new HyperlinkEvent(this, HyperlinkEvent.EventType.ACTIVATED, home);
    if (home != null) {
      navigate(heHome, false);
    }
  }

  /**
   * Constructs new JWxHtmlBrowser object.
   *
   * @param home the home url
   * @param messageFailed the failed message
   * @param messageFailedFileStart the failed message start
   * @param labelBack the back label
   * @param labelGoBack the go back label
   * @param labelForward the forward label
   * @param labelGoForward the go forward label
   * @param labelHome the home label
   * @param labelGoHome the go home label
   * @param labelReload the reloead label
   * @param labelDoReload the do reload label
   */
  public JWxHtmlBrowser(URL home, String messageFailed,
                        String messageFailedFileStart, String labelBack,
                        String labelGoBack, String labelForward,
                        String labelGoForward, String labelHome,
                        String labelGoHome, String labelReload,
                        String labelDoReload) {
    this();
    this.messageFailed = messageFailed;
    this.messageFailedFileStart = messageFailedFileStart;
    this.labelBack = labelBack;
    this.labelGoBack = labelGoBack;
    this.labelForward = labelForward;
    this.labelGoForward = labelGoForward;
    this.labelHome = labelHome;
    this.labelGoHome = labelGoHome;
    this.labelReload = labelReload;
    this.labelDoReload = labelDoReload;
    createActions();
    heHome = new HyperlinkEvent(this, HyperlinkEvent.EventType.ACTIVATED, home);
    if (home != null) {
      navigate(heHome, false);
    }
  }

/*^**************************************************************************^*/
/*- Public Methods.                                                          -*/
/*^**************************************************************************^*/

  /**
   * Renders new html page.
   *
   * @param url the page location
   *
   * @return <code>false</code> if navigation failed
   */
  public boolean navigate(URL url) {
    return navigate(new HyperlinkEvent(this,
                                       HyperlinkEvent.EventType.ACTIVATED, url),
                    true);
  }

  /**
   * Renders new html page.
   *
   * @param url the page location
   *
   * @return <code>false</code> if navigation failed
   */
  public boolean navigate(String url) {
    if (url != null) {
      URL turl = null;
      try {
        turl = new URL(url);
      } catch (MalformedURLException me) {
        navigateError((URL)null);
        return false;
      }
      return navigate(new HyperlinkEvent(this,
                                         HyperlinkEvent.EventType.ACTIVATED,
                                         turl), true);
    }
    navigateError((URL)null);
    return false;
  }

  /**
   * Displays navigation error.
   *
   * @param url the invalid url
   */
  public void navigateError(URL url) {
    htmlBrowser.setContentType("text/html"); //$NON-NLS-1$
    StringBuffer sb = new StringBuffer(messageFailed);
    if (url != null) {
      sb.append(messageFailedFileStart);
      sb.append(url.toExternalForm());
    }
    sb.append(messageFailedFileEnd);
    htmlBrowser.setText(sb.toString());
  }

  /**
   * Displays navigation error.
   *
   * @param url the invalid url
   */
  public void navigateError(String url) {
    URL turl = null;
    try {
      turl = new URL(url);
    } catch (MalformedURLException me) { // NOPMD
      // empty catch block
    }
    navigateError(turl);
  }

  /**
   * Displays navigation error.
   */
  public void navigateError() {
    navigateError((URL)null);
  }
  /**
   * Goes back.
   */
  public void goBack() {
    if (!blockNavigation && !backHistory.isEmpty()) {
      forwardHistory.addFirst(heCurrent);
      navigate((HyperlinkEvent)backHistory.removeLast(), false);
      if (backHistory.isEmpty()) {
        actionBack.setEnabled(false);
      }
      if (!actionForward.isEnabled()) {
        actionForward.setEnabled(true);
      }
    }
  }

  /**
   * Goes forward.
   */
  public void goForward() {
    if (!blockNavigation && !forwardHistory.isEmpty()) {
      backHistory.addLast(heCurrent);
      navigate((HyperlinkEvent)forwardHistory.removeFirst(), false);
      if (!actionBack.isEnabled()) {
        actionBack.setEnabled(true);
      }
      if (forwardHistory.isEmpty()) {
        actionForward.setEnabled(false);
      }
    }
  }

  /**
   * Goes home.
   */
  public void goHome() {
    if (!blockNavigation) {
      if (heHome.getURL() == null) {
        htmlBrowser.setContentType("text/html"); //$NON-NLS-1$
        htmlBrowser.setText(EMPTY_PAGE);
      } else {
        navigate(heHome, true);
      }
    }
  }

  /**
   * Reloads current page.
   */
  public void reload() {
    if (!blockNavigation && (heCurrent != null) &&
        (heCurrent.getURL() != null)) {
      htmlBrowser.getDocument().putProperty(Document.StreamDescriptionProperty,
                                            null);
      HyperlinkEvent hec = heCurrent;
      heCurrent = null;
      navigate(hec, false);
    }
  }

  /**
   * Sets home url.
   *
   * @param home the new home url
   */
  public void setHome(URL home) {
    if (!blockNavigation) {
      heHome = new HyperlinkEvent(this, HyperlinkEvent.EventType.ACTIVATED,
                                  home);
      backHistory.clear();
      actionBack.setEnabled(false);
      forwardHistory.clear();
      actionForward.setEnabled(false);
      navigate(heHome, false);
    }
  }

  /**
   * Returns home url.
   *
   * @return home url
   */
  public URL getHome() {
    return heHome.getURL();
  }

  /**
   * Returns back action.
   *
   * @return back action
   */
  public Action getBackAction() {
    return actionBack;
  }

  /**
   * Returns forward action.
   *
   * @return forward action
   */
  public Action getForwardAction() {
    return actionForward;
  }

  /**
   * Returns reload action.
   *
   * @return reload action
   */
  public Action getReloadAction() {
    return actionReload;
  }

  /**
   * Returns home action.
   *
   * @return home action
   */
  public Action getHomeAction() {
    return actionHome;
  }

  /**
   * Registers hypertext link listener.
   *
   * @param hl the hypertext link listener
   */
  public void addHyperlinkListener(HyperlinkListener hl) {
    htmlBrowser.removeHyperlinkListener(hyperlinkListener);
    htmlBrowser.addHyperlinkListener(hl);
    htmlBrowser.addHyperlinkListener(hyperlinkListener);
  }

  /**
   * Unregisters hypertext link listener.
   *
   * @param hl the hypertext link listener
   */
  public void removeHyperlinkListener(HyperlinkListener hl) {
    htmlBrowser.removeHyperlinkListener(hl);
  }

/*^**************************************************************************^*/
/*- Overridden Methods.                                                      -*/
/*^**************************************************************************^*/

  /**
   * Returns background color.
   *
   * @return background color
   */
  public Color getBackground() {
    return htmlBrowser.getBackground();
  }

  /**
   * Sets background color.
   *
   * @param bg new background color
   */
  public void setBackground(Color bg) {
    super.setBackground(bg);
    htmlBrowser.setBackground(bg);
  }

  /**
   * Returns foreground color.
   *
   * @return foreground color
   */
  public Color getForeground() {
    return htmlBrowser.getForeground();
  }

  /**
   * Sets foreground color.
   *
   * @param fg new foreground color
   */
  public void setForeground(Color fg) {
    super.setForeground(fg);
    htmlBrowser.setForeground(fg);
  }

  /**
   * Returns font.
   *
   * @return font
   */
  public Font getFont() {
    return htmlBrowser.getFont();
  }

  /**
   * Sets font.
   *
   * @param font new font
   */
  public void setFont(Font font) {
    super.setFont(font);
    htmlBrowser.setFont(font);
  }

  /**
   * Prints html content.
   *
   * @param g graphics
   */
  public void print(Graphics g) {
    htmlBrowser.print(g);
  }

/*^**************************************************************************^*/
/*- Protected Methods.                                                       -*/
/*^**************************************************************************^*/

  /**
   * Loads defaults.
   */
  protected void loadDefaults() {
    ResourceBundle rb = BuiltInResources.INSTANCE.getResourceBundle(RESOURCES);
    messageFailed = rb.getString(RK_MESSAGE_FAILED);
    messageFailedFileStart = rb.getString(RK_MESSAGE_FAILED_FILE_START);
    messageFailedFileEnd = rb.getString(RK_MESSAGE_FAILED_FILE_END);
    labelBack = rb.getString(RK_LABEL_BACK);
    labelGoBack = rb.getString(RK_LABEL_GO_BACK);
    labelForward = rb.getString(RK_LABEL_FORWARD);
    labelGoForward = rb.getString(RK_LABEL_GO_FORWARD);
    labelHome = rb.getString(RK_LABEL_HOME);
    labelGoHome = rb.getString(RK_LABEL_GO_HOME);
    labelReload = rb.getString(RK_LABEL_RELOAD);
    labelDoReload = rb.getString(RK_LABEL_DO_REALOD);
  }

  /**
   * Creates actions.
   */
  protected void createActions() {
    actionBack = new BackAction();
    actionForward = new ForwardAction();
    actionHome = new HomeAction();
    actionReload = new ReloadAction();
  }

  /**
   * Creates hypertext link listener.
   *
   * @return hypertext link listener
   */
  protected HyperlinkListener createHyperLinkListener() {
    return new NavigateHyperlinkListener();
  }

  /**
   * Navigates hypertext link.
   *
   * @param he the hyper link event
   * @param log <tt>true</tt> if log history
   *
   * @return <tt>true</tt> if succeed
   */
  protected boolean navigate(HyperlinkEvent he, boolean log) {
    if (blockNavigation || (he == null)) {
      return false;
    }
    boolean navsuccs = false;
    if ((heCurrent != null) && (heCurrent.getURL() != null) &&
        (he.getURL() != null) && (heCurrent.getURL().equals(he.getURL()))) {
      return true;
    }
    URL url = he.getURL();
    String protocol = url.getProtocol();
    if ("file".equalsIgnoreCase(protocol)) { //$NON-NLS-1$
      navsuccs = navigateFile(he, heCurrent, log);
    } else if ("jar".equalsIgnoreCase(protocol)) { //$NON-NLS-1$
      navsuccs = navigateFile(he, heCurrent, log);
    } else if ("http".equalsIgnoreCase(protocol)) { //$NON-NLS-1$
      navsuccs = navigateHTTP(he, heCurrent, log);
    } else if ("mailto".equalsIgnoreCase(protocol)) { //$NON-NLS-1$
      navsuccs = navigateMailTo(he, heCurrent, log);
    } else if ("ftp".equalsIgnoreCase(protocol)) { //$NON-NLS-1$
      navsuccs = navigateFTP(he, heCurrent, log);
    } else {
      ExternalLauncher.openURL(he.getURL());
      return true;
    }
    return navsuccs;
  }

  /* Protocol specific navigation */

  /**
   * Navigates file.
   *
   * @param heNew the new hyper link event
   * @param heCurrent the current hyper link event
   * @param log <tt>true</tt> if log history
   *
   * @return <tt>true</tt> if succeed
   */
  protected boolean navigateFile(HyperlinkEvent heNew, HyperlinkEvent heCurrent,
                                 boolean log) {
    boolean navscs = false;
    blocListener = true;
    try {
      if (heNew instanceof HTMLFrameHyperlinkEvent) {
        ((HTMLDocument)htmlBrowser.getDocument()).
            processHTMLFrameHyperlinkEvent((HTMLFrameHyperlinkEvent)heNew);
      } else {
        URL url = heNew.getURL();
        if (url == null) {
          displayNavigationFailed(heNew);
        } else {
          htmlBrowser.setPage(heNew.getURL());
          navscs = true;
        }
      }
    } catch (IOException ioe) {
      displayNavigationFailed(heNew);
    }
    if (navscs) {
      this.heCurrent = heNew;
      if (log) {
        backHistory.addLast(heCurrent);
        if (!forwardHistory.isEmpty()) {
          forwardHistory.clear();
        }
        if (!actionBack.isEnabled()) {
          actionBack.setEnabled(true);
        }
        if (actionForward.isEnabled()) {
          actionForward.setEnabled(false);
        }
      }
    }
    htmlBrowser.fireHyperlinkUpdate(heNew);
    blocListener = false;
    return navscs;
  }

  /**
   * Navigates JAR.
   *
   * @param heNew the new hyper link event
   * @param heCurrent the current hyper link event
   * @param log <tt>true</tt> if log history
   *
   * @return <tt>true</tt> if succeed
   */
  protected boolean navigateJAR(HyperlinkEvent heNew, HyperlinkEvent heCurrent,
                                boolean log) {
    return navigateFile(heNew, heCurrent, log);
  }

  /**
   * Navigates HTTP.
   *
   * @param heNew the new hyper link event
   * @param heCurrent the current hyper link event
   * @param log <tt>true</tt> if log history
   *
   * @return <tt>true</tt> if succeed
   */
  protected boolean navigateHTTP(HyperlinkEvent heNew, HyperlinkEvent heCurrent,
                                 boolean log) {
    return navigateExternal(heNew);
  }

  /**
   * Navigates MailTo.
   *
   * @param heNew the new hyper link event
   * @param heCurrent the current hyper link event
   * @param log <tt>true</tt> if log history
   *
   * @return <tt>true</tt> if succeed
   */
  protected boolean navigateMailTo(HyperlinkEvent heNew,
                                   HyperlinkEvent heCurrent, boolean log) {
    return navigateExternal(heNew);
  }

  /**
   * Navigates FTP.
   *
   * @param heNew the new hyper link event
   * @param heCurrent the current hyper link event
   * @param log <tt>true</tt> if log history
   *
   * @return <tt>true</tt> if succeed
   */
  protected boolean navigateFTP(HyperlinkEvent heNew, HyperlinkEvent heCurrent,
                                boolean log) {
    return navigateExternal(heNew);
  }

  /* External program navigation */

  /**
   * Navigates external program.
   *
   * @param he the hyper link event
   *
   * @return <tt>true</tt> if succeed
   */
  protected boolean navigateExternal(HyperlinkEvent he) {
    block();
    boolean extnav = ExternalLauncher.openURL(he.getURL());
    if (!extnav) {
      release();
      if (ExternalLauncher.AVAILABLE) {
        displayNavigationFailed(he);
      }
    }
    return extnav;
  }

  /* Input blocking */

  /**
   * Blocks input.
   */
  protected void block() {
    if (!blockNavigation) {
      htmlBrowser.setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
      blockNavigation = true;
      if (externalWaiter == null) {
        externalWaiter = new ExternalWaiter();
      }
      SwingUtilities.invokeLater(externalWaiter);
    }
  }

  /**
   * Releases input.
   */
  protected void release() {
    if (blockNavigation) {
      blockNavigation = false;
      if (htmlBrowser.getCursor().getType() == Cursor.WAIT_CURSOR) {
        htmlBrowser.setCursor(Cursor.getPredefinedCursor(
                                                       Cursor.DEFAULT_CURSOR));
      }
    }
  }

  /**
   * Displays navigation failure.
   *
   * @param he the hyper link event
   */
  protected void displayNavigationFailed(HyperlinkEvent he) {
    if (he == null) {
      navigateError((URL)null);
    } else {
      navigateError(he.getURL());
    }
  }

/*^**************************************************************************^*/
/*- Inner Classes.                                                           -*/
/*^**************************************************************************^*/

  /**
   * Navigate Hyperlink Listener.
   */
  protected class NavigateHyperlinkListener implements HyperlinkListener {
    /**
     * {@inheritDoc}
     */
    public void hyperlinkUpdate(HyperlinkEvent e) {
      if (!blocListener &&
          (e.getEventType() == HyperlinkEvent.EventType.ACTIVATED)) {
        navigate(e, true);
      }
    }
  }

  /**
   * Back Action.
   */
  protected class BackAction extends AbstractAction {
    /**
     * Constructs new BackAction object.
     */
    public BackAction() {
      super();
      putValue(Action.NAME, labelBack);
      setEnabled(false);
      putValue(Action.SHORT_DESCRIPTION, labelGoBack);
    }
    /**
     * {@inheritDoc}
     */
    public void actionPerformed(ActionEvent e) {
      goBack();
    }
  }

  /**
   * Forward Action.
   */
  protected class ForwardAction extends AbstractAction {
    /**
     * Constructs new ForwardAction object.
     */
    public ForwardAction() {
      super();
      putValue(Action.NAME, labelForward);
      setEnabled(false);
      putValue(Action.SHORT_DESCRIPTION, labelGoForward);
    }
    /**
     * {@inheritDoc}
     */
    public void actionPerformed(ActionEvent e) {
      goForward();
    }
  }

  /**
   * Home Action.
   */
  protected class HomeAction extends AbstractAction {
    /**
     * Constructs new HomeAction object.
     */
    public HomeAction() {
      super();
      putValue(Action.NAME, labelHome);
      putValue(Action.SHORT_DESCRIPTION, labelGoHome);
    }
    /**
     * {@inheritDoc}
     */
    public void actionPerformed(ActionEvent e) {
      goHome();
    }
  }

  /**
   * Reload Action.
   */
  protected class ReloadAction extends AbstractAction {
    /**
     * Constructs new ReloadAction object.
     */
    public ReloadAction() {
      super();
      putValue(Action.NAME, labelReload);
      putValue(Action.SHORT_DESCRIPTION, labelDoReload);
    }
    /**
     * {@inheritDoc}
     */
    public void actionPerformed(ActionEvent e) {
      reload();
    }
  }

  /**
   * External Program Waiter.
   */
  protected class ExternalWaiter implements Runnable {
    /**
     * {@inheritDoc}
     */
    public void run() {
      try {
        Thread.sleep(1000);
      } catch (InterruptedException ie) { // NOPMD
        // empty catch block.
      }
      release();
    }
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-20 12:28:04 Slobodan
   Initial version

*****************************************************************************^*/
