/*
 * @(#)JWxImageIconComponent.java  1.0.0 / 2008-05-22
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.swing.components;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Insets;

import javax.swing.ImageIcon;
import javax.swing.JComponent;
import javax.swing.SwingUtilities;

/**
 * Component for ImageIcon.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public class JWxImageIconComponent extends JComponent {

/*^**************************************************************************^*/
/*- Private Fields.                                                          -*/
/*^**************************************************************************^*/

  private ImageIcon image;
  private boolean resizeable;
  private Dimension prefSize;

/*^**************************************************************************^*/
/*- Public Constructors.                                                     -*/
/*^**************************************************************************^*/

  /**
   * Constructs new JWxImageIconComponent object.
   */
  public JWxImageIconComponent() {
    this(null);
  }

  /**
   * Constructs new JWxImageIconComponent object.
   *
   * @param i the image icon
   */
  public JWxImageIconComponent(ImageIcon i) {
    super();
    setImageIcon(i);
  }

/*^**************************************************************************^*/
/*- Public Methods.                                                          -*/
/*^**************************************************************************^*/

  /**
   * Returns image icon.
   *
   * @return image icon
   */
  public ImageIcon getImageIcon() {
    return image;
  }

  /**
   * Sets image icon.
   *
   * @param image the image icon
   */
  public void setImageIcon(ImageIcon image) {
    this.image = image;
    if (image == null) {
      prefSize = new Dimension();
    } else {
      prefSize = new Dimension(image.getIconWidth(), image.getIconHeight());
    }
    setSize(prefSize);
    if (isVisible() && isShowing()) {
      Component rp = SwingUtilities.getRootPane(this);
      if (rp != null) {
        rp.validate();
      }
    }
  }

  /**
   * Returns <code>true</code> if image is resizeable.
   *
   * @return <code>true</code> if image is resizeable
   */
  public boolean isResizeable() {
    return resizeable;
  }

  /**
   * Sets image resizeable state.
   *
   * @param resizeable if <code>true</code> image is resizeable.
   */
  public void setResizeable(boolean resizeable) {
    this.resizeable = resizeable;
  }

/*^**************************************************************************^*/
/*- Overridden Methods.                                                      -*/
/*^**************************************************************************^*/

  /**
   * Paint the component.
   *
   * @param g graphics
   */
  public void paintComponent(Graphics g) {
    if (image == null) {
      super.paintComponent(g);
    } else {
      if (resizeable) {
        g.drawImage(image.getImage(), 0, 0, getWidth(), getHeight(), this);
      } else {
        Insets insets = getInsets();
        int x = 0;
        int y = 0;
        if (insets != null) {
          x = insets.left;
          y = insets.top;
        }
        g.drawImage(image.getImage(), x, y, image.getIconWidth(),
                    image.getIconHeight(), this);
      }
    }
  }

  /**
   * Returns preferred size.
   *
   * @return preferred size
   */
  public Dimension getPreferredSize() {
    return (Dimension)prefSize.clone();
  }

  /**
   * Returns minimum size.
   *
   * @return minimum size
   */
  public Dimension getMinimumSize() {
    return getPreferredSize();
  }

  /**
   * Returns maximum size.
   *
   * @return maximum size
   */
  public Dimension getMaximumSize() {
    return getPreferredSize();
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-22 14:03:49 Slobodan
   Initial version

*****************************************************************************^*/
