/*
 * @(#)JWxMultilineLabel.java  1.0.0 / 2008-05-21
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.swing.components;

import java.awt.Dimension;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.Insets;

import javax.swing.Icon;
import javax.swing.JLabel;
import javax.swing.plaf.LabelUI;

import net.wrka.util.CommonConstants;

/**
 * Multiline Label.
 * <p>
 *  <tt>NOTE: The label icon is not painted!</tt>
 * </p>
 * *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public class JWxMultilineLabel extends JLabel {

/*^**************************************************************************^*/
/*- Public Static Fields.                                                    -*/
/*^**************************************************************************^*/

  /**
   * Center alignment.
   */
  public static final int CENTER = 0;

  /**
   * Top alignment.
   */
  public static final int TOP = 1;

  /**
   * Left alignment.
   */
  public static final int LEFT = 2;

  /**
   * Bottom alignment.
   */
  public static final int BOTTOM = 3;

  /**
   * Right alignment.
   */
  public static final int RIGHT = 4;

/*^**************************************************************************^*/
/*- Private Static Final Field.                                              -*/
/*^**************************************************************************^*/

  private static final String[] EMPTY_LINES = new String[] {
    CommonConstants.STRING_EMPTY
  };

/*^**************************************************************************^*/
/*- Protected Fields.                                                        -*/
/*^**************************************************************************^*/

  private String[] lines;
  private String lineSeparator;

/*^**************************************************************************^*/
/*- Public Constructors.                                                     -*/
/*^**************************************************************************^*/

  /**
   * Constructs new JWxMultilineLabel object.
   */
  public JWxMultilineLabel() {
    this(CommonConstants.STRING_EMPTY, null, LEADING);
  }

  /**
   * Constructs new JWxMultilineLabel object.
   *
   * @param text the text
   */
  public JWxMultilineLabel(String text) {
    this(text, null, LEADING);
  }

  /**
   * Constructs new JWxMultilineLabel object.
   *
   * @param image the image
   */
  public JWxMultilineLabel(Icon image) {
    this(null, image, CENTER);
  }

  /**
   * Constructs new JWxMultilineLabel object.
   *
   * @param image the image
   * @param horizontalAlignment the horizontal alignment
   */
  public JWxMultilineLabel(Icon image, int horizontalAlignment) {
    this(null, image, horizontalAlignment);
  }

  /**
   * Constructs new JWxMultilineLabel object.
   *
   * @param text the text
   * @param horizontalAlignment the horizontal alignment
   */
  public JWxMultilineLabel(String text, int horizontalAlignment) {
    this(text, null, horizontalAlignment);
  }

  /**
   * Constructs new JWxMultilineLabel object.
   *
   * @param text the text
   * @param icon the icon
   * @param horizontalAlignment the horizontal alignment
   */
  public JWxMultilineLabel(String text, Icon icon, int horizontalAlignment) {
    this(text, "\n", icon, horizontalAlignment); //$NON-NLS-1$
  }

  /**
   * Constructs new JWxMultilineLabel object.
   *
   * @param text the label text
   * @param lineSeparator the line separator
   * @param icon the label icon
   * @param horizontalAlignment the horizontal alignment
   */
  public JWxMultilineLabel(String text, String lineSeparator, Icon icon,
                           int horizontalAlignment) {
    super(text, icon, horizontalAlignment);
    this.lineSeparator = lineSeparator;
    if ((text == null) || (lineSeparator == null)) {
      lines = EMPTY_LINES;
    } else {
      lines = text.split(lineSeparator);
    }
    setPreferredSize(calculateTextDimension());
  }

/*^**************************************************************************^*/
/*- Public Methods.                                                          -*/
/*^**************************************************************************^*/

  /**
   * Returns lines.
   *
   * @return lines
   */
  public String[] getLines() {
    return (String[])lines.clone();
  }

  /**
   * Returns line separator.
   *
   * @return line separator
   */
  public String getLineSeparator() {
    return lineSeparator;
  }

  /**
   * Sets line separator.
   *
   * @param lineSeparator String
   */
  public void setLineSeparator(String lineSeparator) {
    this.lineSeparator = lineSeparator;
  }

/*^**************************************************************************^*/
/*- Protected Methods.                                                       -*/
/*^**************************************************************************^*/

  /**
   * Calculates text dimension.
   *
   * @return text dimension
   */
  protected Dimension calculateTextDimension() {
    Dimension d = new Dimension(0, 0);
    Font f = getFont();
    final String[] lines = this.lines;
    if ((f != null) && (lines.length > 0)) {
      FontMetrics fm = getFontMetrics(getFont());
      d.height = fm.getHeight() * lines.length;
      for (int i = lines.length - 1; i >= 0; i--) {
        int sw = fm.stringWidth(lines[i]);
        if (sw > d.width) {
          d.width = sw;
        }
      }
    }
    return d;
  }

/*^**************************************************************************^*/
/*- Overridden Methods.                                                      -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  public void setUI(LabelUI ui) {
    super.setUI(ui);
    setPreferredSize(calculateTextDimension());
  }

  /**
   * {@inheritDoc}
   */
  public void setText(String text) {
    if ((text == null) || (lineSeparator == null)) {
      lines = EMPTY_LINES;
    } else {
      lines = text.split(lineSeparator);
    }
    super.setText(text);
  }

  /**
   * {@inheritDoc}
   */
  public Dimension getMinimumSize() {
    return getPreferredSize();
  }

  /**
   * {@inheritDoc}
   */
  public Dimension getMaximumSize() {
    return getPreferredSize();
  }

  /**
   * {@inheritDoc}
   */
  protected void paintComponent(Graphics g) {
    Insets ins = getInsets();
    FontMetrics fm = g.getFontMetrics();
    int w = getWidth();
    int h = getHeight();
    int ac = fm.getAscent();
    int fh = fm.getHeight();
    int x;
    int y = ins.top + ac;
    int sw;
    g.setColor(getForeground());
    g.setFont(getFont());
    for (int i = 0; i < lines.length; i++) {
      switch (getHorizontalAlignment()) {
        case CENTER:
          sw = fm.stringWidth(lines[i]);
          x = ins.left + ((w - ins.right - ins.left - sw) >> 1) + 1;
          break;
        case RIGHT:
          sw = fm.stringWidth(lines[i]);
          x = w - ins.right - sw + 1;
          break;
        default:
          x = ins.left + 1;
      }
      switch (getVerticalAlignment()) {
        case CENTER:
          y = ((h - ins.top - ins.bottom - fh * lines.length) >> 1) +
              ins.top + ac + i * fh;
          break;
        case BOTTOM:
          y = h - ins.bottom - fh * lines.length + ac + i * fh;
          break;
        default:
          break;
      }
      g.drawString(lines[i], x, y);
    }
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-21 23:51:59 Slobodan
   Initial version

*****************************************************************************^*/
