/*
 * @(#)JWxPlafMenu.java  1.0.0 / 2008-05-22
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.swing.components;

import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Iterator;

import javax.swing.ButtonGroup;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JRadioButtonMenuItem;
import javax.swing.LookAndFeel;
import javax.swing.SwingUtilities;

import net.wrka.swing.plaf.LookAndFeelChangeListener;
import net.wrka.swing.plaf.LookAndFeelManager;
import net.wrka.swing.plaf.UIUpdater;

/**
 * Look and Feel Chooser Menu.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public final class JWxPlafMenu extends JMenu
                               implements LookAndFeelChangeListener {

/*^**************************************************************************^*/
/*- Private Fields.                                                          -*/
/*^**************************************************************************^*/

  private ActionListener defMenuListener;
  private ActionListener menuListener;
  private ButtonGroup buttonGroup;

/*^**************************************************************************^*/
/*- Package-Private Fields.                                                  -*/
/*^**************************************************************************^*/

  UIUpdater uiUpdater;
  boolean updateEnabled;

/*^**************************************************************************^*/
/*- Public Constructors.                                                     -*/
/*^**************************************************************************^*/

  /**
   * Constructs new JWxPlafMenu object.
   *
   * @param menuName the menu name
   */
  public JWxPlafMenu(String menuName) {
    this(menuName, null, null);
  }

  /**
   * Constructs new JWxPlafMenu object.
   *
   * @param menuName the menu name
   * @param al the menu listener
   */
  public JWxPlafMenu(String menuName, ActionListener al) {
    this(menuName, null, al);
  }

  /**
   * Constructs new JWxPlafMenu object.
   *
   * @param menuName the menu name
   * @param c the target component
   */
  public JWxPlafMenu(String menuName, Component c) {
    this(menuName, c, null);
  }

  /**
   * Constructs new JWxPlafMenu object.
   *
   * @param menuName the menu name
   * @param c the target component
   * @param al the menu listener
   */
  public JWxPlafMenu(String menuName, Component c, ActionListener al) {
    super(menuName);
    setComponent(c);
    menuListener = al;
    defMenuListener =  new MenuActionListener();
    buttonGroup = new ButtonGroup();
    initMenu();
    updateEnabled = true;
    LookAndFeelManager.addLnFListener(this);
  }

/*^**************************************************************************^*/
/*- Public methods.                                                          -*/
/*^**************************************************************************^*/

  /**
   * Sets target component.
   *
   * @param c target component
   */
  public void setComponent(Component c) {
    if (c == null) {
      uiUpdater = null;
    } else {
      uiUpdater = new UIUpdater(c);
    }
  }

  /**
   * Returns target component.
   *
   * @return target component
   */
  public Component getComponent() {
    if (uiUpdater == null) {
      return null;
    }
    return uiUpdater.getComponent();
  }

/*^**************************************************************************^*/
/*- Protected Method.                                                        -*/
/*^**************************************************************************^*/

  /**
   * Initializes the menu.
   *
   */
  protected void initMenu() {
    Iterator items = LookAndFeelManager.lnfNames();
    String currentLnFName = LookAndFeelManager.getCurrentLookAndFeelName();
    JRadioButtonMenuItem jrbmi;
    while (items.hasNext()) {
      jrbmi = new JRadioButtonMenuItem((String)items.next());
      if (menuListener != null) {
        jrbmi.addActionListener(menuListener);
      }
      jrbmi.addActionListener(defMenuListener);
      jrbmi.setSelected(currentLnFName.equals(jrbmi.getText()));
      buttonGroup.add(jrbmi);
      add(jrbmi);
    }
  }

/*^**************************************************************************^*/
/*- LookAndFeelChangeListener Implementation.                                -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  public void lookAndFeelChanged(LookAndFeel oldLnF, LookAndFeel newLnF) {
    if (updateEnabled) {
      String currentLnFName = newLnF.getName();
      for (int i = 0; i < getItemCount(); i++) {
        JMenuItem jmi = getItem(i);
        if (currentLnFName.equals(jmi.getText())) {
          jmi.setSelected(true);
          return;
        }
      }
      JRadioButtonMenuItem jrbmilf = new JRadioButtonMenuItem(currentLnFName);
      jrbmilf.addActionListener(defMenuListener);
      jrbmilf.setSelected(currentLnFName.equals(jrbmilf.getText()));
      buttonGroup.add(jrbmilf);
      add(jrbmilf);
    }
  }

/*^**************************************************************************^*/
/*- Inner Class.                                                             -*/
/*^**************************************************************************^*/

  /**
   * Menu Action Listener.
   */
  final class MenuActionListener implements ActionListener {
    /**
     * {@inheritDoc}
     */
    public void actionPerformed(ActionEvent e) {
      updateEnabled = false;
      LookAndFeelManager.setInstalledLookAndFeel(e.getActionCommand());
      updateEnabled = true;
      if (uiUpdater != null) {
        SwingUtilities.invokeLater(uiUpdater);
      }
    }
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-22 13:05:14 Slobodan
   Initial version

*****************************************************************************^*/
