/*
 * @(#)JWxPlafThemeMenu.java  1.0.0 / 2008-05-22
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.swing.components;

import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Enumeration;

import javax.swing.ButtonGroup;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JRadioButtonMenuItem;
import javax.swing.LookAndFeel;
import javax.swing.SwingUtilities;
import javax.swing.plaf.metal.MetalLookAndFeel;
import javax.swing.plaf.metal.MetalTheme;

import net.wrka.swing.plaf.LookAndFeelChangeListener;
import net.wrka.swing.plaf.LookAndFeelManager;
import net.wrka.swing.plaf.UIUpdater;
import net.wrka.swing.plaf.metal.MetalThemeManager;
import net.wrka.swing.plaf.smooth.SmoothLookAndFeel;
import net.wrka.swing.plaf.smooth.SmoothTheme;
import net.wrka.swing.plaf.smooth.SmoothThemeManager;
import net.wrka.util.CommonConstants;

/**
 * Look and Feel Chooser Theme Menu.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public final class JWxPlafThemeMenu extends JMenu
                                   implements ActionListener,
                                              LookAndFeelChangeListener {

/*^**************************************************************************^*/
/*- Protected Fields.                                                        -*/
/*^**************************************************************************^*/

  private ButtonGroup metalGroup;
  private ButtonGroup smootOfficeGroup;
  private boolean updateEnabled;
  private int lnfIndex;
  private UIUpdater uiUpdater;

/*^**************************************************************************^*/
/*- Public Constructors.                                                     -*/
/*^**************************************************************************^*/

  /**
   * Constructs new JWxPlafThemeMenu object.
   *
   * @param menuName menu name.
   */
  public JWxPlafThemeMenu(String menuName) {
    this(menuName, null, null);
  }

  /**
   * Constructs new JWxPlafThemeMenu object.
   *
   * @param menuName menu name.
   * @param a action listener
   */
  public JWxPlafThemeMenu(String menuName, ActionListener a) {
    this(menuName, null, a);
  }

  /**
   * Constructs new JWxPlafThemeMenu object.
   *
   * @param menuName menu name.
   * @param c target component
   */
  public JWxPlafThemeMenu(String menuName, Component c) {
    this(menuName, c, null);
  }

  /**
   * Constructs new JWxPlafThemeMenu object.
   *
   * @param menuName menu name.
   * @param c target component
   * @param a action listener
   */
  public JWxPlafThemeMenu(String menuName, Component c, ActionListener a) {
    super(menuName);
    setComponent(c);
    metalGroup = new ButtonGroup();
    JRadioButtonMenuItem jrbmit;
    for (int i = 0; i < MetalThemeManager.getThemeCount(); i++) {
      MetalTheme metalTheme = MetalThemeManager.getTheme(i);
      jrbmit = new JRadioButtonMenuItem(metalTheme.getName());
      if (a != null) {
        jrbmit.addActionListener(a);
      }
      jrbmit.addActionListener(this);
      metalGroup.add(jrbmit);
    }
    smootOfficeGroup = new ButtonGroup();
    for (int i = 0; i < SmoothThemeManager.getThemeCount(); i++) {
      SmoothTheme smoothTheme = SmoothThemeManager.getTheme(i);
      jrbmit = new JRadioButtonMenuItem(smoothTheme.getName());
      if (a != null) {
        jrbmit.addActionListener(a);
      }
      jrbmit.addActionListener(this);
      smootOfficeGroup.add(jrbmit);
    }
    updateMenu();
    updateEnabled = true;
    LookAndFeelManager.addLnFListener(this);
  }

/*^**************************************************************************^*/
/*- Public Methods.                                                          -*/
/*^**************************************************************************^*/

  /**
   * Sets target component.
   *
   * @param c target component
   */
  public void setComponent(Component c) {
    if (c == null) {
      uiUpdater = null;
    } else {
      uiUpdater = new UIUpdater(c);
    }
  }

  /**
   * Returns target component.
   *
   * @return target component
   */
  public Component getComponent() {
    if (uiUpdater == null) {
      return null;
    }
    return uiUpdater.getComponent();
  }

/*^**************************************************************************^*/
/*- ActionListener Implementation.                                           -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  public void actionPerformed(ActionEvent e) {
    updateEnabled = false;
    LookAndFeel lnf;
    String themeName;
    switch (lnfIndex) {
      case 0:
        lnf = LookAndFeelManager.getCurrentLookAndFeel();
        if (!(lnf instanceof MetalLookAndFeel)) {
          lnf = new MetalLookAndFeel();
        }
        themeName = ((JMenuItem)e.getSource()).getText();
        MetalTheme metalTheme = MetalThemeManager.getTheme(themeName);
        MetalLookAndFeel.setCurrentTheme(metalTheme);
        LookAndFeelManager.setNewThemeLookAndFeel(lnf);
        break;
      case 1:
        lnf = LookAndFeelManager.getCurrentLookAndFeel();
        if (!(lnf instanceof MetalLookAndFeel)) {
          lnf = new SmoothLookAndFeel();
        }
        themeName = ((JMenuItem)e.getSource()).getText();
        SmoothTheme officeTheme = SmoothThemeManager.getTheme(themeName);
        SmoothLookAndFeel.setCurrentTheme(officeTheme);
        LookAndFeelManager.setNewThemeLookAndFeel(lnf);
        break;
      default:
        break;
    }
    updateEnabled = true;
    if (uiUpdater != null) {
      SwingUtilities.invokeLater(uiUpdater);
    }
  }

/*^**************************************************************************^*/
/*- LookAndFeelChangeListener Implementation.                                -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  public void lookAndFeelChanged(LookAndFeel oldLnF, LookAndFeel newLnF) {
    if (updateEnabled) {
      updateMenu();
    }
  }

/*^**************************************************************************^*/
/*- Private Methods.                                                         -*/
/*^**************************************************************************^*/

  /**
   * Update menu.
   */
  protected void updateMenu() {
    LookAndFeel currentLnF = LookAndFeelManager.getCurrentLookAndFeel();
    String lnfName = currentLnF.getName();
    if ("Metal".equals(lnfName)) { //$NON-NLS-1$
      removeAll();
      MetalTheme lastMetalTheme = MetalThemeManager.getCurrentMetalTheme();
      String lastMetalThemeName;
      if (lastMetalTheme == null) {
        lastMetalThemeName = CommonConstants.STRING_EMPTY;
      } else {
        lastMetalThemeName = lastMetalTheme.getName();
      }
      Enumeration items = metalGroup.getElements();
      while (items.hasMoreElements()) {
        JMenuItem item = (JMenuItem)items.nextElement();
        item.setSelected(item.getText().equals(lastMetalThemeName));
        add(item);
      }
      setEnabled(true);
      lnfIndex = 0;
    } else if (SmoothLookAndFeel.NAME.equals(lnfName)) {
      removeAll();
      SmoothTheme lastSmootTheme = SmoothLookAndFeel.getCurrentSmoothTheme();
      String lastSmootThemeName;
      if (lastSmootTheme == null) {
        lastSmootThemeName = CommonConstants.STRING_EMPTY;
      } else {
        lastSmootThemeName = lastSmootTheme.getName();
      }
      Enumeration items = smootOfficeGroup.getElements();
      while (items.hasMoreElements()) {
        JMenuItem item = (JMenuItem)items.nextElement();
        item.setSelected(item.getText().equals(lastSmootThemeName));
        add(item);
      }
      setEnabled(true);
      lnfIndex = 1;
    } else {
      removeAll();
      setEnabled(false);
      lnfIndex = 2;
    }
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-22 13:09:57 Slobodan
   Initial version

*****************************************************************************^*/
