/*
 * @(#)JWxRootPane.java  1.0.0 / 2008-05-21
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.swing.components;

import java.awt.Component;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.Insets;
import java.awt.LayoutManager;
import java.awt.Rectangle;

import javax.swing.JLayeredPane;
import javax.swing.JRootPane;

import net.wrka.util.GPTools;

/**
 * RootPane with Status Bar.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public class JWxRootPane extends JRootPane {

/*^**************************************************************************^*/
/*- Private Field.                                                           -*/
/*^**************************************************************************^*/

  private JWxStatusBar statusBar;

/*^**************************************************************************^*/
/*- Public Constructors.                                                     -*/
/*^**************************************************************************^*/

  /**
   * Constructs new JWxRootPane object.
   */
  public JWxRootPane() {
    super();
  }

  /**
   * Constructs new JWxRootPane object.
   *
   * @param opaque if <code>true</code> frame is opaque
   */
  public JWxRootPane(boolean opaque) {
    super();
    setOpaque(opaque);
  }

/*^**************************************************************************^*/
/*- Public Methods.                                                          -*/
/*^**************************************************************************^*/

  /**
   * Returns the status bar from the layered pane.
   *
   * @return the JWxStatusBar used in the pane
   */
  public JWxStatusBar getJWxStatusBar() {
    return statusBar;
  }

  /**
   * Adds or changes the status bar used in the layered pane.
   *
   * @param statusBar the new status bar
   */
  public void setJWxStatusBar(JWxStatusBar statusBar) {
    if ((this.statusBar != null) &&
        (this.statusBar.getParent() == layeredPane)) {
      layeredPane.remove(this.statusBar);
    }
    this.statusBar = statusBar;
    if (statusBar != null) {
      layeredPane.add(statusBar, JLayeredPane.FRAME_CONTENT_LAYER);
    }
  }

/*^**************************************************************************^*/
/*- Overridden Methods.                                                      -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  protected LayoutManager createRootLayout() {
    return new JWxRootLayout();
  }

  /**
   * {@inheritDoc}
   */
  protected Component createGlassPane() {
    JWxGlassPane gp = new JWxGlassPane(0);
    gp.setName(getName() + ".glassPane"); //$NON-NLS-1$
    return gp;
  }

/*^**************************************************************************^*/
/*- Protected Inner Class.                                                   -*/
/*^**************************************************************************^*/

  /**
   * A custom layout manager that is responsible for the layout of layeredPane,
   * glassPane, menuBar and statusBar.
   */
  protected class JWxRootLayout extends RootLayout {
    /**
     * {@inheritDoc}
     */
    public Dimension preferredLayoutSize(Container parent) {
      Dimension rd;
      Dimension mbd;
      Dimension sbd;
      Insets i = getInsets();
      Component cp = getContentPane();
      if (cp == null) {
        rd = parent.getSize();
      } else {
        rd = cp.getPreferredSize();
      }
      Component mb = getJMenuBar();
      if ((mb != null) && mb.isVisible()) {
        mbd = mb.getPreferredSize();
      } else {
        mbd = new Dimension(0, 0);
      }
      Component sb = getJWxStatusBar();
      if ((sb != null) && sb.isVisible()) {
        sbd = sb.getPreferredSize();
      } else {
        sbd = new Dimension(0, 0);
      }
      return new Dimension(GPTools.max(rd.width, mbd.width, sbd.width) +
                           i.left + i.right,
                           rd.height + mbd.height + sbd.height + i.top +
                           i.bottom);
    }
    /**
     * {@inheritDoc}
     */
    public Dimension minimumLayoutSize(Container parent) {
      Dimension rd;
      Dimension mbd;
      Dimension sbd;
      Insets i = getInsets();
      Component cp = getContentPane();
      if (cp == null) {
        rd = parent.getSize();
      } else {
        rd = cp.getMinimumSize();
      }
      Component mb = getJMenuBar();
      if ((mb != null) && mb.isVisible()) {
        mbd = mb.getMinimumSize();
      } else {
        mbd = new Dimension(0, 0);
      }
      Component sb = getJWxStatusBar();
      if ((sb != null) && sb.isVisible()) {
        sbd = sb.getMinimumSize();
      } else {
        sbd = new Dimension(0, 0);
      }
      return new Dimension(GPTools.max(rd.width, mbd.width, sbd.width) +
                           i.left + i.right,
                           rd.height + mbd.height + sbd.height + i.top +
                           i.bottom);
    }
    /**
     * {@inheritDoc}
     */
    public Dimension maximumLayoutSize(Container target) {
      Dimension rd;
      Dimension mbd;
      Dimension sbd;
      Insets i = getInsets();
      Component mb = getJMenuBar();
      if ((mb != null) && mb.isVisible()) {
        mbd = mb.getMaximumSize();
      } else {
        mbd = new Dimension(0, 0);
      }
      Component cp = getContentPane();
      if (cp == null) {
        // This is silly, but should stop an overflow error !?
        rd = new Dimension(Integer.MAX_VALUE,
                           Integer.MAX_VALUE - i.top - i.bottom - mbd.height -
                           1);
      } else {
        rd = cp.getMaximumSize();
      }
      Component sb = getJWxStatusBar();
      if ((sb != null) && sb.isVisible()) {
        sbd = sb.getMaximumSize();
      } else {
        sbd = new Dimension(0, 0);
      }
      return new Dimension(GPTools.min(rd.width, mbd.width, sbd.width) +
                           i.left + i.right,
                           rd.height + mbd.height + sbd.height + i.top +
                           i.bottom);
    }
    /**
     * {@inheritDoc}
     */
    public void layoutContainer(Container parent) {
      Rectangle b = parent.getBounds();
      Insets i = getInsets();
      int contentY = 0;
      int w = b.width - i.right - i.left;
      int h = b.height - i.top - i.bottom;
      Component lp = getLayeredPane();
      if (lp != null) {
        lp.setBounds(i.left, i.top, w, h);
      }
      Component gp = getGlassPane();
      if (gp != null) {
        gp.setBounds(i.left, i.top, w, h);
      }
      Component mb = getJMenuBar();
      if ((mb != null) && mb.isVisible()) {
        Dimension mbd = mb.getPreferredSize();
        mb.setBounds(0, 0, w, mbd.height);
        contentY += mbd.height;
      }
      Component sb = getJWxStatusBar();
      if ((sb != null) && sb.isVisible()) {
        Dimension sbd = sb.getPreferredSize();
        sb.setBounds(0, b.height - i.bottom - sbd.height, w, sbd.height);
        h -= sbd.height;
      }
      Component cp = getContentPane();
      if (cp != null) {
        cp.setBounds(0, contentY, w, h - contentY);
      }
    }
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-21 23:20:37 Slobodan
   Initial version

*****************************************************************************^*/
