/*
 * @(#)JWxSeparator.java  1.0.0 / 2008-05-21
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.swing.components;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Insets;

import javax.swing.JComponent;

import net.wrka.swing.graphics.GraphicUtils;

/**
 * Component separator with customizable line.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public class JWxSeparator extends JComponent {

/*^**************************************************************************^*/
/*- Public Static Fields.                                                    -*/
/*^**************************************************************************^*/

  /**
   * Kind <i>none</i>.
   */
  public static final int NONE = 0;

  /**
   * Kind <i>top</i>.
   */
  public static final int TOP = 1;

  /**
   * Kind <i>vertical center</i>.
   */
  public static final int VERTICAL_CENTER = 2;

  /**
   * Kind <i>bottom</i>.
   */
  public static final int BOTTOM = 3;

  /**
   * Kind <i>left</i>.
   */
  public static final int LEFT = 4;

  /**
   * Kind <i>horizontal center</i>.
   */
  public static final int HORIZONTAL_CENTER = 5;

  /**
   * Kind <i>right</i>.
   */
  public static final int RIGHT = 6;

  /**
   * Kind <i>horizontal simple</i>.
   */
  public static final int HORIZONTAL_SIMPLE = 7;

  /**
   * Kind <i>vertical simple</i>.
   */
  public static final int VERTICAL_SIMPLE = 8;

  /**
   * Kind <i>upper</i>.
   */
  public static final boolean UPPER = true;

  /**
   * Kind <i>lower</i>.
   */
  public static final boolean LOWER = false;

/*^**************************************************************************^*/
/*- Protected Fields.                                                        -*/
/*^**************************************************************************^*/

  /**
   * The separator kind.
   */
  protected int kind;

  /**
   * The upper flag.
   */
  protected boolean upper;

  /**
   * The margins.
   */
  protected Insets margins;

/*^**************************************************************************^*/
/*- Public Constructors.                                                     -*/
/*^**************************************************************************^*/

  /**
   * Constructs new JWxSeparator object.
   *
   */
  public JWxSeparator() {
    this(new Dimension(1, 4));
  }

  /**
   * Constructs new JWxSeparator object.
   *
   * @param d the dimension
   */
  public JWxSeparator(Dimension d) {
    this(d, new Insets(1, 1, 1, 1));
  }

  /**
   * Constructs new JWxSeparator object.
   *
   * @param kind the kind
   */
  public JWxSeparator(int kind) {
    this(kind, true);
  }

  /**
   * Constructs new JWxSeparator object.
   *
   * @param kind the kind
   * @param upper the upper flag
   */
  public JWxSeparator(int kind, boolean upper) {
    this(new Dimension(1, 4), new Insets(1, 1, 1, 1), kind, upper);
  }

  /**
   * Constructs new JWxSeparator object.
   *
   * @param d the dimension
   * @param margins the margins
   */
  public JWxSeparator(Dimension d, Insets margins) {
    this(d, margins, TOP);
  }

  /**
   * Constructs new JWxSeparator object.
   *
   * @param d the dimension
   * @param margins the margins
   * @param kind the kind
   */
  public JWxSeparator(Dimension d, Insets margins, int kind) {
    this(d, margins, kind, LOWER);
  }

  /**
   * Constructs new JWxSeparator object.
   *
   * @param d the dimension
   * @param margins the margins
   * @param kind the kind
   * @param upper the upper flag
   */
  public JWxSeparator(Dimension d, Insets margins, int kind, boolean upper) {
    this(d, margins, kind, upper, false);
  }

  /**
   * Constructs new JWxSeparator object.
   *
   * @param d the component dimensions
   * @param margins the component margins
   * @param kind component kind (TOP, VERTICAL_CENTER, BOTTOM,  LEFT,
   *                             HORIZONTAL_CENTER, RIGHT)
   * @param upper UPPER/LOWER line
   * @param fixedDimension if true dimensions are fixed
   */
  public JWxSeparator(Dimension d, Insets margins, int kind, boolean upper,
                      boolean fixedDimension) {
    super();
    this.upper = upper;
    this.kind = kind;
    setDoubleBuffered(true);
    if (d != null) {
      setPreferredSize(d);
      setMinimumSize(d);
    }
    if (fixedDimension) {
      setMaximumSize(d);
    }
    if (margins == null) {
      this.margins = new Insets(0, 0, 0, 0);
    } else {
      this.margins = margins;
    }
  }

/*^**************************************************************************^*/
/*- Public Static Methods.                                                   -*/
/*^**************************************************************************^*/

  /**
   * Creates and returns flat separator.
   *
   * @param d the dimension
   * @return flat separator
   */
  public static JWxSeparator createFlat(Dimension d) {
    return new JWxSeparator(d, null, NONE, false);
  }

  /**
   * Creates and returns simple vertical separator.
   *
   * @param d the dimension
   * @return simple vertical separator
   */
  public static JWxSeparator createSimpleVertical(Dimension d) {
    return new JWxSeparator(d, new Insets(1, 1, 1, 1), VERTICAL_SIMPLE, false);
  }

  /**
   * Creates and returns simple horizontal separator.
   *
   * @param d the dimension
   * @return simple horizontal separator
   */
  public static JWxSeparator createSimpleHorizontal(Dimension d) {
    return new JWxSeparator(d, new Insets(1, 1, 1, 1), HORIZONTAL_SIMPLE,
                            false);
  }

/*^**************************************************************************^*/
/*- Overridden Method.                                                       -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  public void paintComponent(Graphics g) {
    super.paintComponent(g);
    if (kind != NONE) {
      Color c1;
      Color c2;
      Color bg = getBackground();
      if (upper) {
        c1 = GraphicUtils.brighterColor(bg, 0.6f);
        c2 = GraphicUtils.darkerColor(bg, 0.3f);
      } else {
        c1 = GraphicUtils.darkerColor(bg, 0.3f);
        c2 = GraphicUtils.brighterColor(bg, 0.6f);
      }
      int width = getWidth();
      int height = getHeight();
      int wmmr = width - margins.right;
      int wmmrm1 = wmmr - 1;
      int wh2 = width >> 1;
      int wh2p1 = wh2 + 1;
      int mtp1 = margins.top + 1;
      int hh2 = height >> 1;
      int hh2p1 = hh2 + 1;
      int hmmb = height - margins.bottom;
      int hmmbp1 = hmmb + 1;
      int mlp1 = margins.left + 1;
      switch (kind) { // NOPMD
        case TOP:
          g.setColor(c1);
          g.drawLine(margins.left, margins.top, wmmr, margins.top);
          g.setColor(c2);
          g.drawLine(margins.left, mtp1, wmmr, mtp1);
          break;
        case VERTICAL_CENTER:
          g.setColor(c1);
          g.drawLine(margins.left, hh2, wmmr, hh2);
          g.setColor(c2);
          g.drawLine(margins.left, hh2p1, wmmr, hh2p1);
          break;
        case HORIZONTAL_SIMPLE:
          g.setColor(c1);
          g.drawLine(margins.left, hh2, wmmr, hh2);
          break;
        case BOTTOM:
          g.setColor(c1);
          g.drawLine(margins.left, hmmb, wmmr, hmmb);
          g.setColor(c2);
          g.drawLine(margins.left, hmmbp1, wmmr, hmmbp1);
          break;
        case LEFT:
          g.setColor(c1);
          g.drawLine(margins.left, margins.top, margins.left, hmmb);
          g.setColor(c2);
          g.drawLine(mlp1, margins.top, mlp1, hmmb);
          break;
        case HORIZONTAL_CENTER:
          g.setColor(c1);
          g.drawLine(wh2, margins.top, wh2, hmmb);
          g.setColor(c2);
          g.drawLine(wh2p1, margins.top, wh2p1, hmmb);
          break;
        case VERTICAL_SIMPLE:
          g.setColor(c1);
          g.drawLine(wh2, margins.top, wh2, hmmb);
          break;
        case RIGHT:
          g.setColor(c1);
          g.drawLine(wmmrm1, margins.top, wmmrm1, hmmb);
          g.setColor(c2);
          g.drawLine(wmmr, margins.top, wmmr, hmmb);
          break;
      }
    }
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-21 19:34:48 Slobodan
   Initial version

*****************************************************************************^*/
