/*
 * @(#)JWxStatusBar.java  1.0.0 / 2008-05-20
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.swing.components;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.LayoutManager;

import javax.swing.BoxLayout;
import javax.swing.JComponent;

import net.wrka.swing.components.plaf.WxStatusBarUI;
import net.wrka.swing.components.statusbar.StatusBarPanel;
import net.wrka.swing.components.statusbar.StatusBarSeparator;
import net.wrka.swing.plaf.LookAndFeelManager;
import net.wrka.swing.plaf.basic.BasicWxStatusBarUI;

/**
 * Status Bar.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public class JWxStatusBar extends JComponent {

/*^**************************************************************************^*/
/*- Private Static Field.                                                    -*/
/*^**************************************************************************^*/

  private static final String UI_CLASS_ID = "WxStatusBarUI"; //$NON-NLS-1$

/*^**************************************************************************^*/
/*- Private Fields.                                                          -*/
/*^**************************************************************************^*/

  private boolean addingEnabled;
  private int height;

/*^**************************************************************************^*/
/*- Public Constructors.                                                     -*/
/*^**************************************************************************^*/

  /**
   * Constructs new JWxStatusBar object.
   */
  public JWxStatusBar() {
    super();
    super.setLayout(new BoxLayout(this, BoxLayout.X_AXIS));
    updateUI();
  }

  /**
   * Constructs new JWxStatusBar object.
   *
   * @param height the status bar height
   */
  public JWxStatusBar(int height) {
    this();
    setHeight(height);
  }

  /**
   * Constructs new JWxStatusBar object.
   *
   * @param panels the status bar panels
   * @param height the status bar height
   */
  public JWxStatusBar(JComponent[] panels, int height) {
    this(height);
    for (int i = 0; i < panels.length; i++) {
      addStatusPanel(panels[i]);
    }
  }

  /**
   * Constructs new JWxStatusBar object.
   *
   * @param panel the status bar panel
   * @param height the status bar height
   */
  public JWxStatusBar(JComponent panel, int height) {
    this(height);
    addStatusPanel(panel);
  }

/*^**************************************************************************^*/
/*- Public Methods.                                                          -*/
/*^**************************************************************************^*/

  /**
   * Sets panel's height.
   *
   * @param height the new height
   */
  public void setHeight(int height) {
    this.height = height;
    Dimension ps = super.getPreferredSize();
    ps.height = height;
    setPreferredSize(ps);
  }

  /**
   * Sets width of the specified status panel.
   *
   * @param index the panel index
   * @param width the new width
   */
  public void setStatusPanelDimension(int index, int width) {
    JComponent c;
    if (index == -1) {
      c = (JComponent)getComponent(getComponentCount() - 1);
    } else {
      c = (JComponent)getComponent(index << 1);
    }
    if (c != null) {
      Dimension tdim = new Dimension(width, height + 1);
      c.setPreferredSize(tdim);
      c.setMinimumSize(tdim);
      c.setMaximumSize(tdim);
    }
  }

  /**
   * Adds empty status panel.
   */
  public void addEmptyStatusPanel() {
    StatusBarPanel sp = new StatusBarPanel();
    addingEnabled = true;
    add(sp);
    addingEnabled = false;
  }

  /**
   * Adds status panel.
   *
   * @param c the status panel
   *
   * @throws IllegalArgumentException if c is <tt>null</tt>
   */
  public void addStatusPanel(JComponent c) {
    if (c == null) {
      throw new IllegalArgumentException("Null status component"); //$NON-NLS-1$
    }
    StatusBarPanel sp = new StatusBarPanel(c);
    addingEnabled = true;
    add(sp);
    addingEnabled = false;
  }

  /**
   * Adds status panel.
   *
   * @param c the status panel
   * @param width the status panel width
   */
  public void addStatusPanel(JComponent c, int width) {
    addStatusPanel(c);
    setStatusPanelDimension(-1, width);
  }

  /**
   * Removes status panel.
   *
   * @param index the status panel index
   */
  public void removeStatusPanel(int index) {
    index = (index << 1);
    Component c = getComponent(index);
    if (c == null) {
      throw new IllegalArgumentException(
                                    "Invalid status panel index"); //$NON-NLS-1$
    }
    remove(c);
    c = getComponent(index - 1);
    if ((c != null) && (c instanceof StatusBarSeparator)) {
      remove(c);
    }
  }

  /**
   * Removes last status panel.
   */
  public void removeStatusPanel() {
    removeStatusPanel(getComponentCount() - 1);
  }

  /**
   * Returns look and feel delegate.
   *
   * @return look and feel delegate
   */
  public WxStatusBarUI getUI() {
    return (WxStatusBarUI)ui;
  }

  /**
   * Sets the look and feel delegate for this component.
   *
   * @param ui the new UI delegate
   */
  public void setUI(WxStatusBarUI ui) {
    if (this.ui != null) {
      this.ui.uninstallUI(this);
    }
    WxStatusBarUI oldUI = (WxStatusBarUI)this.ui;
    if (ui == null) {
      if (this.ui == null) {
        this.ui = BasicWxStatusBarUI.createUI(this);
      }
      ((WxStatusBarUI)this.ui).installUI(this);
    } else {
      this.ui = ui;
      ui.installUI(this);
      firePropertyChange("UI", oldUI, this.ui); //$NON-NLS-1$
    }
    revalidate();
    repaint();
  }


/*^**************************************************************************^*/
/*- Overridden Methods.                                                      -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  public void setLayout(LayoutManager layoutManager) {
    // empty method body
  }

  /**
   * {@inheritDoc}
   */
  public Dimension getMinimumSize() {
    Dimension d = super.getMinimumSize();
    d.height = height;
    return d;
  }

  /**
   * {@inheritDoc}
   */
  public Dimension getMaximumSize() {
    Dimension d = super.getMaximumSize();
    d.height = height;
    return d;
  }

  /**
   * {@inheritDoc}
   */
  public Dimension getPreferredSize() {
    Dimension d = super.getPreferredSize();
    d.height = height;
    return d;
  }

  /**
   * {@inheritDoc}
   */
  public String getUIClassID() {
    return UI_CLASS_ID;
  }

  /**
   * {@inheritDoc}
   */
  public void updateUI() {
    setUI((WxStatusBarUI)LookAndFeelManager.getComponentUI(this));
  }

  /**
   * {@inheritDoc}
   */
  protected void addImpl(Component comp, Object constraints, int index) {
    if (!addingEnabled) {
      throw new IllegalStateException(
                          "To add component into JWxStatusBar " + //$NON-NLS-1$
                          "use addStatusPanel(JComponent c)"); //$NON-NLS-1$
    }
    if (getComponentCount() > 0) {
      super.addImpl(new StatusBarSeparator(), null, -1);
    }
    super.addImpl(comp, constraints, index);
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-20 21:35:32 Slobodan
   Initial version

*****************************************************************************^*/
