/*
 * @(#)JWxTabbedPane.java  1.0.0 / 2008-05-22
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.swing.components;

import java.awt.AWTEvent;
import java.awt.Color;
import java.awt.Component;
import java.awt.Graphics;
import java.awt.Point;
import java.awt.event.MouseEvent;
import java.util.ResourceBundle;

import javax.swing.Icon;
import javax.swing.JTabbedPane;

import net.wrka.swing.resources.BuiltInResources;
import net.wrka.util.ref.WeakArrayList;

/**
 * Closeable Tabbed Pane.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public class JWxTabbedPane extends JTabbedPane {

/*^**************************************************************************^*/
/*- Public Static Final Field.                                               -*/
/*^**************************************************************************^*/

  /**
   * Resource string.
   */
  public static final String RESOURCES = "JWxTabbedPaneResources"; //$NON-NLS-1$

  /**
   * Resource key <i>close icon tooltip</i>.
   */
  public static final String RK_X_ICON_TOOLTIP =
                                    "JWxTabbedPane.xicon.tooltip"; //$NON-NLS-1$

/*^**************************************************************************^*/
/*- Private Fields.                                                          -*/
/*^**************************************************************************^*/

  private boolean closeableTabs;
  private int lastRolloverTab;
  private String lastTabTooltip;
  private String xiconTooltip;
  private final WeakArrayList listeners;

/*^**************************************************************************^*/
/*- Public Constructors.                                                     -*/
/*^**************************************************************************^*/

  /**
   * Constructs new JWxTabbedPane object.
   */
  public JWxTabbedPane() {
    this(TOP, WRAP_TAB_LAYOUT);
  }

  /**
   * Constructs new JWxTabbedPane object.
   *
   * @param closeableTabs if <tt>true</tt> tabs are closeable
   */
  public JWxTabbedPane(boolean closeableTabs) {
    this(TOP, WRAP_TAB_LAYOUT, closeableTabs);
  }

  /**
   * Constructs new JWxTabbedPane object.
   *
   * @param tabPlacement the tab placement
   * @param closeableTabs if <tt>true</tt> tabs are closeable
   */
  public JWxTabbedPane(int tabPlacement, boolean closeableTabs) {
    this(tabPlacement, WRAP_TAB_LAYOUT, closeableTabs);
  }

  /**
   * Constructs new JWxTabbedPane object.
   *
   * @param tabPlacement the tab placement
   * @param tabLayoutPolicy the tab layout policy
   */
  public JWxTabbedPane(int tabPlacement, int tabLayoutPolicy) {
    this(tabPlacement, tabLayoutPolicy, true);
  }

  /**
   * Constructs new JWxTabbedPane object.
   *
   * @param tabPlacement the tab placement
   * @param tabLayoutPolicy the tab layout policy
   * @param closeableTabs if <tt>true</tt> tabs are closeable
   */
  public JWxTabbedPane(int tabPlacement, int tabLayoutPolicy,
                       boolean closeableTabs) {
    super(tabPlacement, tabLayoutPolicy);
    lastRolloverTab = -1;
    this.closeableTabs = closeableTabs;
    enableEvents(AWTEvent.MOUSE_EVENT_MASK);
    enableEvents(AWTEvent.MOUSE_MOTION_EVENT_MASK);
    ResourceBundle rb = BuiltInResources.INSTANCE.getResourceBundle(RESOURCES);
    xiconTooltip = rb.getString(RK_X_ICON_TOOLTIP);
    listeners = new WeakArrayList(4);
  }

/*^**************************************************************************^*/
/*- Public Methods.                                                          -*/
/*^**************************************************************************^*/

  /**
   * Returns <tt>true</tt> if tabs are closeable.
   *
   * @return <tt>true</tt> if tabs are closeable
   */
  public boolean isTabCloseable() {
    return closeableTabs;
  }

  /**
   * Sets tab closeable state.
   *
   * @param tabCloseable it <tt>true</tt> tabs are closeable
   */
  public void setTabCloseable(boolean tabCloseable) {
    if (closeableTabs != tabCloseable) {
      if (tabCloseable) {
        for (int i = getTabCount() - 1; i >= 0; i--) {
          Icon icon = getIconAt(i);
          super.setIconAt(i, new TabCloseIcon(icon));
        }
      } else {
        for (int i = getTabCount() - 1; i >= 0; i--) {
          Icon icon = getIconAt(i);
          if (icon instanceof TabCloseIcon) {
            TabCloseIcon tci = (TabCloseIcon)icon;
            super.setIconAt(i, tci.icon);
          }
        }
      }
      closeableTabs = tabCloseable;
    }
  }

  /**
   * Adds <tt>TabCloseListener</tt> to the listener list.
   *
   * @param tcl the TabCloseListener
   */
  public void addTabCloseListener(TabCloseListener tcl) {
    listeners.add(tcl);
  }

  /**
   * Removes <tt>TabCloseListener</tt> to the listener list.
   *
   * @param tcl the TabCloseListener
   */
  public void removeTabCloseListener(TabCloseListener tcl) {
    listeners.remove(tcl);
  }

  /**
   * Closes specified tab.
   *
   * @param index the tab index
   *
   * @throws IndexOutOfBoundsException if index is out of range
   *         ((index < 0) || (index >= tab count))
   */
  public void closeTabAt(int index) {
    if (canCloseTab(index)) {
      removeTabAt(index);
      fireTabClosed(index);
    }
  }

  /**
   * Returns <tt>true</tt> if tab at specified index can be closed.
   *
   * @param index the tab index
   *
   * @return <tt>true</tt> if tab at specified index can be closed
   */
  public boolean canCloseTab(int index) {
    boolean hasnull = false;
    boolean canClose = true;
    final WeakArrayList listeners = this.listeners;
    for (int i = listeners.size() - 1; i >= 0; i--) {
      TabCloseListener tcl = (TabCloseListener)listeners.get(i);
      if (tcl == null) {
        hasnull = true;
      } else if (!tcl.canClose(this, index)) {
        canClose = false;
        break;
      }
    }
    if (hasnull) {
      listeners.expungeStaleReferences();
    }
    return canClose;
  }

/*^**************************************************************************^*/
/*- Protected Methods.                                                       -*/
/*^**************************************************************************^*/

  /**
   * Notifies tab close listener that the specified tab is closed.
   *
   * @param tabIndex the tab index.
   */
  protected void fireTabClosed(int tabIndex) {
    boolean hasnull = false;
    final WeakArrayList listeners = this.listeners;
    for (int i = listeners.size() - 1; i >= 0; i--) {
      TabCloseListener tcl = (TabCloseListener)listeners.get(i);
      if (tcl == null) {
        hasnull = true;
      } else {
        tcl.tabClosed(this, tabIndex);
      }
    }
    if (hasnull) {
      listeners.expungeStaleReferences();
    }
  }

/*^**************************************************************************^*/
/*- Overridden Methods.                                                      -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  public void insertTab(String title, Icon icon, Component component,
                        String tip, int index) {
    if (closeableTabs) {
      icon = new TabCloseIcon(icon);
    }
    super.insertTab(title, icon, component, tip, index);
  }

  /**
   * {@inheritDoc}
   */
  public void setIconAt(int index, Icon icon) {
    if (closeableTabs) {
      icon = new TabCloseIcon(icon);
    }
    super.setIconAt(index, icon);
  }

  /**
   * {@inheritDoc}
   */
  public void removeTabAt(int index) {
    super.removeTabAt(index);
    lastRolloverTab = -1;
    lastTabTooltip = null;
  }

  /**
   * {@inheritDoc}
   */
  protected void processMouseEvent(MouseEvent e) {
    super.processMouseEvent(e);
    if (closeableTabs) {
      switch (e.getID()) { // NOPMD
        case MouseEvent.MOUSE_CLICKED:
          Point p = e.getPoint();
          int px = p.x;
          int py = p.y;
          int tabIndex = getUI().tabForCoordinate(this, px, py);
          if (tabIndex >= 0) {
            TabCloseIcon closeIcon = (TabCloseIcon)getIconAt(tabIndex);
            if (closeIcon.contains(px, py)) {
              closeTabAt(tabIndex);
            }
          }
          break;
        case MouseEvent.MOUSE_EXITED:
          if (lastRolloverTab >= 0) {
            TabCloseIcon closeIcon =
                (TabCloseIcon)getIconAt(lastRolloverTab);
            if (closeIcon.rollover) {
              lastRolloverTab = -1;
              lastTabTooltip = null;
              closeIcon.rollover = false;
              repaint(closeIcon.x, closeIcon.y, 8, 8);
            }
          }
          break;
      }
    }
  }

  /**
   * {@inheritDoc}
   */
  protected void processMouseMotionEvent(MouseEvent e) {
    super.processMouseMotionEvent(e);
    if (closeableTabs) {
      Point p = e.getPoint();
      int px = p.x;
      int py = p.y;
      int tabIndex = getUI().tabForCoordinate(this, px, py);
      if (tabIndex >= 0) {
        TabCloseIcon closeIcon = (TabCloseIcon)getIconAt(tabIndex);
        if (closeIcon.contains(px, py)) {
          if (!closeIcon.rollover) {
            lastRolloverTab = tabIndex;
            lastTabTooltip = getToolTipTextAt(tabIndex);
            setToolTipTextAt(tabIndex, xiconTooltip);
            closeIcon.rollover = true;
            repaint(closeIcon.x, closeIcon.y, 8, 8);
          }
        } else {
          if (closeIcon.rollover) {
            lastRolloverTab = -1;
            setToolTipTextAt(tabIndex, lastTabTooltip);
            closeIcon.rollover = false;
            repaint(closeIcon.x, closeIcon.y, 8, 8);
          }
        }
      }
    }
  }

/*^**************************************************************************^*/
/*- Inner Class.                                                             -*/
/*^**************************************************************************^*/

  /**
   * Tab Close Icon.
   */
  static final class TabCloseIcon implements Icon {

    // Package-Private Static Final Fields--------------------------------------

    static final Color X_ICON_INNER_COLOR = new Color(92, 92, 92);
    static final Color X_ICON_OUTER_COLOR = Color.BLACK;
    static final Color X_ICON_ROLLOVER_COLOR = Color.LIGHT_GRAY;

    // Package-Private Fields---------------------------------------------------

    Icon icon;
    int x;
    int y;
    int w;
    int h;
    int dh;
    boolean rollover;

    // Public Constructor ------------------------------------------------------

    /**
     * Constructs new TabCloseIcon object.
     *
     * @param icon the icon
     */
    public TabCloseIcon(Icon icon) {
      w = 8;
      h = 8;
      if (icon != null) {
        this.icon = icon;
        int ih = icon.getIconHeight();
        if (ih > h) {
          int d = ih - h;
          dh = (d >> 1)  + (d % 2);
          h = ih;
        }
        w = w + 4 + icon.getIconWidth();
      }
    }

    // Public Method -----------------------------------------------------------

    /**
     * Returns <tt>true</tt> if the icon contains the specified point.
     *
     * @param px point x coordinate
     * @param py point y coordinate
     *
     * @return <tt>true</tt> if the icon contains the specified point
     */
    public boolean contains(int px, int py) {
      return ((px >= x) && (py >= y) && (px < x + 8) && (py < y + 8));
    }

    // Overridden Methods ------------------------------------------------------

    /**
     * {@inheritDoc}
     */
    public void paintIcon(Component c, Graphics g, int x, int y) {
      this.x = x;
      if (icon != null) {
        icon.paintIcon(c, g, x + 12, y);
        y = y + dh;
      }
      this.y = y;
      g.translate(x, y);
      g.setColor(X_ICON_OUTER_COLOR);
      g.fillRect(0, 0, 1, 1);
      g.drawLine(1, 0, 3, 2);
      g.drawLine(4, 2, 6, 0);
      g.fillRect(7, 0, 1, 1);
      g.drawLine(7, 1, 5, 3);
      g.drawLine(5, 4, 7, 6);
      g.fillRect(7, 7, 1, 1);
      g.drawLine(6, 7, 4, 5);
      g.drawLine(3, 5, 1, 7);
      g.fillRect(0, 7, 1, 1);
      g.drawLine(0, 6, 2, 4);
      g.drawLine(2, 3, 0, 1);
      if (rollover) {
        g.setColor(X_ICON_ROLLOVER_COLOR);
      } else {
        g.setColor(X_ICON_INNER_COLOR);
      }
      g.drawLine(1, 1, 6, 6);
      g.drawLine(1, 1, 6, 6);
      g.drawLine(6, 1, 1, 6);
      g.translate(-x, -y);
    }

    /**
     * {@inheritDoc}
     */
    public int getIconWidth() {
      return w;
    }

    /**
     * {@inheritDoc}
     */
    public int getIconHeight() {
      return h;
    }

  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-22 09:23:00 Slobodan
   Initial version

*****************************************************************************^*/
