/*
 * @(#)JWxTitlePane.java  1.0.0 / 2008-05-22
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.swing.components;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.LayoutManager;
import java.awt.Paint;

import javax.swing.JComponent;

import net.wrka.swing.components.plaf.WxTitlePaneUI;
import net.wrka.swing.graphics.FastGradientPaint;
import net.wrka.swing.plaf.LookAndFeelManager;
import net.wrka.swing.plaf.basic.BasicWxTitlePaneUI;

/**
 * Title Pane.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public class JWxTitlePane extends JComponent {

/*^**************************************************************************^*/
/*- Public Static Fields.                                                    -*/
/*^**************************************************************************^*/

  /**
   * Simple pane type.
   */
  public static final int TYPE_SIMPLE = 0;

  /**
   * Simple shadow pane type.
   */
  public static final int TYPE_SIMPLE_SHADOW = 1;

  /**
   * Raised pane type.
   */
  public static final int TYPE_RAISED = 2;

  /**
   * Raised shadow pane type.
   */
  public static final int TYPE_RAISED_SHADOW = 3;

/*^**************************************************************************^*/
/*- Private Static Field.                                                    -*/
/*^**************************************************************************^*/

  private static final String UI_CLASS_ID = "WxTitlePaneUI";

/*^**************************************************************************^*/
/*- Private Fields.                                                          -*/
/*^**************************************************************************^*/

  private JComponent titlePanel;
  private JComponent contentPanel;
  private boolean contentCheck;
  private final int type;

/*^**************************************************************************^*/
/*- Public Constructors.                                                     -*/
/*^**************************************************************************^*/

  /**
   * Constructs new JWxTitlePane object.
   *
   * @param type the pane type
   */
  public JWxTitlePane(int type) {
    super();
    this.type = type;
    super.setLayout(new BorderLayout());
    titlePanel = createTitlePanel();
    add(titlePanel, BorderLayout.NORTH);
    contentPanel = createContentPanel();
    add(contentPanel, BorderLayout.CENTER);
    setContentPaneCheckingEnabled(true);
    updateUI();
  }

  /**
   * Constructs new simple JWxTitlePane object.
   */
  public JWxTitlePane() {
    this(TYPE_SIMPLE);
  }

/*^**************************************************************************^*/
/*- Public Methods.                                                          -*/
/*^**************************************************************************^*/

  /**
   * Returns pane type.
   *
   * @return pane type
   */
  public int getType() {
    return type;
  }

  /**
   * Returns content pane.
   *
   * @return content pane
   */
  public JComponent getContentPanel() {
    return contentPanel;
  }

  /**
   * Returns title pane.
   *
   * @return title pane
   */
  public JComponent getTitlePanel() {
    return titlePanel;
  }

  /**
   * Returns look and feel delegate.
   *
   * @return look and feel delegate
   */
  public WxTitlePaneUI getUI() {
    return (WxTitlePaneUI)ui;
  }

  /**
   * Sets the look and feel delegate for this component.
   *
   * @param ui the new UI delegate
   */
  public void setUI(WxTitlePaneUI ui) {
    if (this.ui != null) {
      this.ui.uninstallUI(this);
    }
    WxTitlePaneUI oldUI = (WxTitlePaneUI)this.ui;
    if (ui == null) {
      if (this.ui == null) {
        setDefaultUI();
      }
      ((WxTitlePaneUI)this.ui).installUI(this);
    } else {
      this.ui = ui;
      ui.installUI(this);
      firePropertyChange("UI", oldUI, ui); //$NON-NLS-1$
    }
    revalidate();
    repaint();
  }

/*^**************************************************************************^*/
/*- Overridden Methods.                                                      -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  protected void addImpl(Component comp, Object constraints, int index) {
    if (isContentPaneCheckingEnabled()) {
      throw createContentPaneException("add"); //$NON-NLS-1$
    }
    super.addImpl(comp, constraints, index);
  }

  /**
   * {@inheritDoc}
   */
  public void setLayout(LayoutManager manager) {
    if (isContentPaneCheckingEnabled()) {
      throw createContentPaneException("setLayout"); //$NON-NLS-1$
    }
    super.setLayout(manager);
  }

  /**
   * {@inheritDoc}
   */
  public void setPreferredSize(Dimension preferredSize) {
    if (isContentPaneCheckingEnabled()) {
      throw createContentPaneException("setPreferredSize"); //$NON-NLS-1$
    }
    super.setPreferredSize(preferredSize);
  }

  /**
   * {@inheritDoc}
   */
  public String getUIClassID() {
    return UI_CLASS_ID;
  }

  /**
   * {@inheritDoc}
   */
  public void updateUI() {
    setUI((WxTitlePaneUI)LookAndFeelManager.getComponentUI(this));
  }

/*^**************************************************************************^*/
/*- Protected Methods.                                                       -*/
/*^**************************************************************************^*/

  /**
   * Creates title panel.
   *
   * @return title panel
   */
  protected JComponent createTitlePanel() {
    TitlePane tp = new TitlePane();
    String name = getName();
    if (name != null) {
      tp.setName(name.concat(".titlePane")); //$NON-NLS-1$
    }
    return tp;
  }

  /**
   * Creates content panel.
   *
   * @return content panel
   */
  protected JComponent createContentPanel() {
    JWxGradientPanel c = new JWxGradientPanel();
    String name = getName();
    if (name != null) {
      c.setName(name.concat(".contentPane")); //$NON-NLS-1$
    }
    return c;
  }

  /**
   * Returns <tt>true</tt> if content pane checking is enabled.
   *
   * @return <tt>true</tt> if content pane checking is enabled
   */
  protected boolean isContentPaneCheckingEnabled() {
    return contentCheck;
  }

  /**
   * Sets content pane checking.
   *
   * @param enabled <tt>true</tt> if content pane checking is enabled
   */
  protected void setContentPaneCheckingEnabled(boolean enabled) {
    contentCheck = enabled;
  }

  /**
   * Sets default ui.
   */
  protected void setDefaultUI() {
    ui = BasicWxTitlePaneUI.createUI(this);
  }

  /**
   * Creates operation error.
   *
   * @param operation the error operation
   *
   * @return operation error
   */
  protected RuntimeException createContentPaneException(String operation) {
    return new IllegalArgumentException(getClass().getName() +
                                   "Do not use ." + operation + //$NON-NLS-1$
                                   "() use .getContentPanel()." + //$NON-NLS-1$
                                   operation + "() instead"); //$NON-NLS-1$
  }

/*^**************************************************************************^*/
/*- Protected Inner Class.                                                   -*/
/*^**************************************************************************^*/

  /**
   * Title Pane.
   */
  public static class TitlePane extends JWxToolBar {

    // Private Field. ----------------------------------------------------------

    private Color secondaryBackground;

    // Public Constructor. -----------------------------------------------------

    /**
     * Constructs new TitlePane object.
     */
    public TitlePane() {
      super(null, HORIZONTAL, false);
      setFlat(true);
    }

    // Public Methods. ---------------------------------------------------------

    /**
     * Returns secondary background.
     *
     * @return secondary background
     */
    public Color getSecondaryBackground() {
      if (secondaryBackground == null) {
        return getBackground();
      }
      return secondaryBackground;
    }

    /**
     * Sets secondary background.
     *
     * @param secondaryBackground the secondary background
     */
    public void setSecondaryBackground(Color secondaryBackground) {
      Color oldsb = this.secondaryBackground;
      if (oldsb != secondaryBackground) { // NOPMD
        this.secondaryBackground = secondaryBackground;
        repaint();
        firePropertyChange("secondaryBackground", oldsb, //$NON-NLS-1$
                           secondaryBackground);
      }
    }

    // Overridden Method. ------------------------------------------------------

    /**
     * {@inheritDoc}
     */
    protected void paintComponent(Graphics g) {
      if (isOpaque()) {
        Graphics2D g2D = (Graphics2D)g;
        Paint gradient = new FastGradientPaint(getSecondaryBackground(),
                                               getBackground(), false);
        g2D.setPaint(gradient);
        g2D.fillRect(getX(), getY(), getWidth(), getHeight());
      }
    }

  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-22 10:37:41 Slobodan
   Initial version

*****************************************************************************^*/
