/*
 * @(#)JWxToolBar.java  1.0.0 / 2008-05-22
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.swing.components;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.Insets;

import javax.swing.AbstractButton;
import javax.swing.Action;
import javax.swing.JButton;
import javax.swing.JSeparator;
import javax.swing.JToggleButton;
import javax.swing.JToolBar;
import javax.swing.border.Border;
import javax.swing.plaf.DimensionUIResource;
import javax.swing.plaf.SeparatorUI;
import javax.swing.plaf.UIResource;
import javax.swing.plaf.basic.BasicToolBarSeparatorUI;

import net.wrka.swing.patch.BorderNonUIResource;
import net.wrka.swing.patch.PatchManager;
import net.wrka.swing.plaf.metal.MetalThemeManager;
import net.wrka.swing.plaf.smooth.SmoothToolBarSeparatorUI;

/**
 * Extended JToolBar.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public class JWxToolBar extends JToolBar {

/*^**************************************************************************^*/
/*- Public Static Final Field.                                               -*/
/*^**************************************************************************^*/

  /**
   * <i>Is flat</i> client property.
   */
  public static final String IS_FLAT = "JWxToolBar.isFlat"; //$NON-NLS-1$

/*^**************************************************************************^*/
/*- Private Static Final Field.                                              -*/
/*^**************************************************************************^*/

  private static final Insets TB_MARGIN = new Insets(1, 1, 1, 1);

/*^**************************************************************************^*/
/*- Private Field.                                                           -*/
/*^**************************************************************************^*/

  private Insets toolButtonMargin;

/*^**************************************************************************^*/
/*- Public Constructors.                                                     -*/
/*^**************************************************************************^*/

  /**
   * Constructs new JWxToolBar object.
   */
  public JWxToolBar() {
    this(null, HORIZONTAL);
  }

  /**
   * Constructs new JWxToolBar object.
   *
   * @param orientation the orientation
   */
  public JWxToolBar(int orientation) {
    this(null, orientation);
  }

  /**
   * Constructs new JWxToolBar object.
   *
   * @param name the name
   */
  public JWxToolBar(String name) {
    this(name, HORIZONTAL);
  }

  /**
   * Constructs new JWxToolBar object.
   *
   * @param name the name
   * @param orientation the orientation
   */
  public JWxToolBar(String name, int orientation) {
    super(name, orientation);
    toolButtonMargin = TB_MARGIN;
    setRollover(true);
  }

  /**
   * Constructs new JWxToolBar object.
   *
   * @param name the name
   * @param orientation the orientation
   * @param floatable the floatable flag
   */
  public JWxToolBar(String name, int orientation, boolean floatable) {
    this(name, orientation);
    setFloatable(floatable);
  }

/*^**************************************************************************^*/
/*- Public Methods.                                                          -*/
/*^**************************************************************************^*/

  /**
   * Returns <tt>true</tt> if tool bar is flat.
   *
   * @return <tt>true</tt> if tool bar is flat
   */
  public boolean isFlat() {
    Object flat = getClientProperty(IS_FLAT);
    if (flat instanceof Boolean) {
      return ((Boolean)flat).booleanValue();
    }
    return false;
  }

  /**
   * Sets toolbar's flat state.
   *
   * @param flat the new flat state
   */
  public void setFlat(boolean flat) {
    if (flat) {
      putClientProperty(IS_FLAT, Boolean.TRUE);
    } else {
      putClientProperty(IS_FLAT, Boolean.FALSE);
    }
  }

  /**
   * Returns tool button margin.
   *
   * @return tool button margin
   */
  public Insets getToolButtonMargin() {
    return (Insets)toolButtonMargin.clone();
  }

  /**
   * Sets tool button margin.
   *
   * @param toolButtonMargin the new margin
   */
  public void setToolButtonMargins(Insets toolButtonMargin) {
    if (toolButtonMargin == null) {
      throw new IllegalArgumentException("Null margin"); //$NON-NLS-1$
    }
    this.toolButtonMargin = toolButtonMargin;
  }

  /**
   * Adds an AbstractButton to this tool bar.
   *
   * @param button the button
   *
   * @return the button argument
   */
  public Component add(AbstractButton button) {
    super.add(button);
    button.setFocusable(false);
    Insets m = button.getMargin();
    if ((m == null) || (m instanceof UIResource)) {
      button.setMargin(toolButtonMargin);
    }
    return button;
  }

  /**
   * Creates and adds new tool button to this tool bar.
   *
   * @param action the action
   *
   * @return the tool button
   */
  public AbstractButton addToolButton(Action action) {
    if (action == null) {
      throw new IllegalArgumentException("Null action"); //$NON-NLS-1$
    }
    AbstractButton button = createToolButton(action);
    add(button);
    return button;
  }

/*^**************************************************************************^*/
/*- Protected Methods.                                                       -*/
/*^**************************************************************************^*/

  /**
   * Creates tool button.
   *
   * @param action the action
   *
   * @return tool button
   */
  protected AbstractButton createToolButton(Action action) {
    AbstractButton button;
    if (action.getValue("selected") instanceof Boolean) { //$NON-NLS-1$
      button = new JWxToggleButton(action);
    } else {
      button = new JWxButton(action);
    }
    button.setText(null);
    return button;
  }

/*^**************************************************************************^*/
/*- Overridden Methods.                                                      -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  public void addSeparator(Dimension size) {
    JWxToolBarSeparator s = new JWxToolBarSeparator(size);
    if (getOrientation() == VERTICAL) {
      s.setOrientation(JSeparator.HORIZONTAL);
    } else {
      s.setOrientation(JSeparator.VERTICAL);
    }
    add(s);
  }

  /**
   * {@inheritDoc}
   */
  public void updateUI() {
    super.updateUI();
    if (PatchManager.APPLAY_PATCH_14) {
      final Component[] comps = getComponents();
      for (int i = comps.length - 1; i >= 0; i--) {
        Component c = comps[i];
        if ((c instanceof JButton) || (c instanceof JToggleButton)) {
          AbstractButton button = (AbstractButton)c;
          Border border = button.getBorder();
          if (border instanceof UIResource) {
            button.setBorder(new BorderNonUIResource(border));
          }
        }
      }
    }
  }

/*^**************************************************************************^*/
/*- Protected Inner Classes.                                                 -*/
/*^**************************************************************************^*/

  /**
   * UpdateUI Bug workaround.
   */
  protected static class JWxToolBarSeparator extends Separator {

    // Private Field -----------------------------------------------------------

    private Dimension separatorSize;

    // Public Constructors -----------------------------------------------------

    /**
     * Constructs new JWxToolBarSeparator object.
     *
     */
    public JWxToolBarSeparator() {
      super(null);
    }

    /**
     * Constructs new JWxToolBarSeparator object.
     *
     * @param size the size
     */
    public JWxToolBarSeparator(Dimension size) {
      super(size);
      if (size == null) {
        super.updateUI();
      } else {
        separatorSize = size;
      }
    }

    // Overridden Methods ------------------------------------------------------

    /**
     * {@inheritDoc}
     */
    public void setSeparatorSize(Dimension size) {
      if (size == null) {
        super.updateUI();
      } else {
        if (size instanceof UIResource) {
          separatorSize = size;
        } else {
          separatorSize = new DimensionUIResource(size.width, size.height);
        }
      }
      invalidate();
    }

    /**
     * {@inheritDoc}
     */
    public Dimension getSeparatorSize() {
      return separatorSize;
    }

    /**
     * {@inheritDoc}
     */
    public Dimension getMinimumSize() {
      if (separatorSize != null) {
        return separatorSize.getSize();
      }
      return super.getMinimumSize();
    }

    /**
     * {@inheritDoc}
     */
    public Dimension getMaximumSize() {
      if (separatorSize != null) {
        return separatorSize.getSize();
      }
      return super.getMaximumSize();
    }

    /**
     * {@inheritDoc}
     */
    public Dimension getPreferredSize() {
      if (separatorSize != null) {
        return separatorSize.getSize();
      }
      return super.getPreferredSize();
    }

    /**
     * {@inheritDoc}
     */
    public void updateUI() {
      if (separatorSize instanceof UIResource) {
        separatorSize = null;
      }
      super.updateUI();
    }

    /**
     * {@inheritDoc}
     */
    public void setUI(SeparatorUI ui) {
      if (ui == null) {
        super.setUI(null);
      } else {
        boolean useSmooth =
                         (ui.getClass().equals(BasicToolBarSeparatorUI.class) &&
                          !MetalThemeManager.isCurrentOcean());
        if (useSmooth) {
          ui = new SmoothToolBarSeparatorUI();
        }
        super.setUI(ui);
        if (useSmooth) {
          separatorSize = null;
        }
      }
    }
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-22 10:38:18 Slobodan
   Initial version

*****************************************************************************^*/
