/*
 * @(#)TitlePaneFactory.java  1.0.0 / 2008-05-22
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.swing.components;

import java.awt.Color;
import java.awt.Font;
import java.awt.Insets;
import java.awt.event.ActionEvent;

import javax.swing.Box;
import javax.swing.JComponent;
import javax.swing.JLabel;

import net.wrka.swing.resources.BuiltInResources;

/**
 * Helper class for creating JWxTitlePanes.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public final class TitlePaneFactory {

/*^**************************************************************************^*/
/*- Private Constructor.                                                     -*/
/*^**************************************************************************^*/

 /**
  * Class has no instances.
  */
  private TitlePaneFactory() {
    // empty constructor body
  }

/*^**************************************************************************^*/
/*- Public Static Methods.                                                   -*/
/*^**************************************************************************^*/

  /**
   * Creates and return title pane.
   *
   * @param type the title pane type
   *
   * @return title pane
   */
  public static JWxTitlePane createPane(int type) {
    return new JWxTitlePane(type);
  }

  /**
   * Creates and return title pane.
   *
   * @param type the title pane type
   * @param title the title pane title
   * @param font the title pane font
   * @param color the title pane color
   *
   * @return title pane
   */
  public static JWxTitlePane createPane(int type, String title, Font font,
                                        Color color) {
    JWxTitlePane jwxtp = createPane(type);
    JLabel jlTitle = new JLabel(title);
    if (font != null) {
      jlTitle.setFont(font);
    }
    if (color != null) {
      jlTitle.setForeground(color);
    }
    JComponent titlePanel = jwxtp.getTitlePanel();
    titlePanel.add(Box.createHorizontalStrut(4));
    titlePanel.add(jlTitle);
    return jwxtp;
  }

  /**
   * Creates and return title pane.
   *
   * @param type the title pane type
   * @param title the title pane title
   *
   * @return title pane
   */
  public static JWxTitlePane createPane(int type, String title) {
    return createPane(type, title, null, null);
  }

  /**
   * Creates and return closeable title pane.
   *
   * @param type the title pane type
   * @param title the title pane title
   * @param closeToolTip the close button tool tip
   * @param font the title pane font
   * @param color the title pane color
   *
   * @return title pane
   */
  public static JWxTitlePane createClosePane(int type, String title,
                                             String closeToolTip,
                                             Font font, Color color) {
    JWxTitlePane jwxtp = createPane(type, title, font, color);
    JComponent titlePanel = jwxtp.getTitlePanel();
    titlePanel.add(Box.createHorizontalGlue());
    titlePanel.add(new CloseButton(jwxtp, closeToolTip));
    return jwxtp;
  }

  /**
   * Creates and return closeable title pane.
   *
   * @param type the title pane type
   * @param title the title pane title
   * @param closeToolTip the close button tool tip
   *
   * @return title pane
   */
  public static JWxTitlePane createClosePane(int type, String title,
                                             String closeToolTip) {
    return createClosePane(type, title, closeToolTip, null, null);
  }

/*^**************************************************************************^*/
/*- Private Inner Class.                                                     -*/
/*^**************************************************************************^*/

  /**
   * Close Button.
   */
  public static final class CloseButton extends JWxButton {

    // Private Static Final Field. ---------------------------------------------
    private static final Insets NULL_INSETS = new Insets(0, 0, 0, 0);

    // Private Final Field. ----------------------------------------------------
    private final JWxTitlePane jwxtp;

    // Public Constructor. -----------------------------------------------------

    /**
     * Constructs new CloseButton object.
     *
     * @param jwxtp the title pan
     * @param toolTip the tool tip
     */
    public CloseButton(JWxTitlePane jwxtp, String toolTip) {
      super(BuiltInResources.INSTANCE.getImageIcon("XClose.png"), //$NON-NLS-1$
            toolTip);
      this.jwxtp = jwxtp;
    }

    // Overridden Methods. -----------------------------------------------------

    /**
     * {@inheritDoc}
     */
    protected void onAction(ActionEvent e) {
      jwxtp.setVisible(false);
    }

    /**
     * {@inheritDoc}
     */
    public boolean isOpaque() {
      return false;
    }

    /**
     * {@inheritDoc}
     */
    public boolean isFocusable() {
      return false;
    }

    /**
     * {@inheritDoc}
     */
    public Insets getMargin() {
      return (Insets)NULL_INSETS.clone();
    }

  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-22 14:58:04 Slobodan
   Initial version

*****************************************************************************^*/
