/*
 * @(#)AboutBox.java  1.0.0 / 2008-05-19
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.swing.components.about;

import java.awt.Dialog;
import java.awt.Frame;
import java.awt.Window;
import java.lang.ref.WeakReference;

import javax.swing.ImageIcon;
import javax.swing.LookAndFeel;
import javax.swing.SwingUtilities;

import net.wrka.swing.plaf.LookAndFeelChangeListener;
import net.wrka.swing.plaf.LookAndFeelManager;
import net.wrka.util.SimpleArrayList;

/**
 * About Box.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public final class AboutBox implements LookAndFeelChangeListener {

/*^**************************************************************************^*/
/*- Public Static Final Fields.                                              -*/
/*^**************************************************************************^*/

  /**
   * Resources.
   */
  public static final String RESOURCES = "AboutResources"; //$NON-NLS-1$

  /**
   * Resource key <i>about dialog title</i>.
   */
  public static final String RK_DIALOG_TITLE = "About.title"; //$NON-NLS-1$

  /**
   * Resource key <i>close button</i>.
   */
  public static final String RK_DIALOG_CLOSE_BUTTON =
                                             "About.closeButton"; //$NON-NLS-1$

  /**
   * Resource key <i>system group name</i>.
   */
  public static final String RK_SYSTEM_NAME = "About.System.groupName";

  /**
   * Resource key <i>system properties</i>.
   */
  public static final String RK_SYSTEM_PROPERTIES =
                                       "About.System.properties"; //$NON-NLS-1$

  /**
   * Resource key <i>system property key</i>.
   */
  public static final String RK_SYSTEM_PROPERTY_NAME =
                                     "About.System.propertyName"; //$NON-NLS-1$

  /**
   * Resource key <i>system property value</i>.
   */
  public static final String RK_SYSTEM_PROPERTY_VALUE =
                                    "About.System.propertyValue"; //$NON-NLS-1$

  /**
   * Resource key <i>java heap</i>.
   */
  public static final String RK_SYSTEM_JVM_HEAP =
                                         "About.System.javaHeap"; //$NON-NLS-1$

  /**
   * Resource key <i>JVM allocated</i>.
   */
  public static final String RK_SYSTEM_JVM_ALLOCATED =
                                    "About.System.heapAllocated"; //$NON-NLS-1$

  /**
   * Resource key <i>JVM used</i>.
   */
  public static final String RK_SYSTEM_JVM_USED =
                                         "About.System.heapUsed"; //$NON-NLS-1$

/*^**************************************************************************^*/
/*- Private Fields.                                                          -*/
/*^**************************************************************************^*/

  private final Window owner;
  private boolean lnfChanged;
  private WeakReference dialogCache;

/*^**************************************************************************^*/
/*- Package-Private Fields.                                                  -*/
/*^**************************************************************************^*/

  AboutDialog dialog;
  ImageIcon logoIcon;
  SimpleArrayList aboutGroups;

/*^**************************************************************************^*/
/*- Public Constructor.                                                      -*/
/*^**************************************************************************^*/

  /**
   * Constructs new JWxAboutBox object.
   *
   * @param owner the dialog window owner
   * @param logoIcon the logo icon
   */
  public AboutBox(Window owner, ImageIcon logoIcon) {
    if (owner == null) {
      throw new IllegalArgumentException("Null Window Owner"); //$NON-NLS-1$
    }
    this.owner = owner;
    if (logoIcon == null) {
      throw new IllegalArgumentException("Null Logo Icon"); //$NON-NLS-1$
    }
    this.logoIcon = logoIcon;
    aboutGroups = new SimpleArrayList(5);
    LookAndFeelManager.addLnFListener(this);
  }

/*^**************************************************************************^*/
/*- Public Methods.                                                          -*/
/*^**************************************************************************^*/

  /**
   * Adds about group.
   *
   * @param aboutGroup the about group
   */
  public void addAboutGroup(AboutGroup aboutGroup) {
    aboutGroups.add(aboutGroup);
  }

  /**
   * Removes about group.
   *
   * @param aboutGroup the about group
   */
  public void removeAboutGroup(AboutGroup aboutGroup) {
    aboutGroups.remove(aboutGroup);
  }

  /**
   * Displays about box dialog.
   */
  public void show() {
    if (dialog == null) {
      if (dialogCache != null) {
        dialog = (AboutDialog)dialogCache.get();
      }
      if (dialog == null) {
        if (owner instanceof Dialog) {
          dialog = new AboutDialog((Dialog)owner, this); // FB: OK
        } else if (owner instanceof Frame) {
          dialog = new AboutDialog((Frame)owner, this);  // FB: OK
        } else {
          return;
        }
        dialogCache = new WeakReference(dialog);
      } else {
        if (lnfChanged) {
          SwingUtilities.updateComponentTreeUI(dialog);
          lnfChanged = false;
        }
        dialog.updatePanels();
      }
    }
    if (dialog.isVisible()) {
      dialog.toFront();
    } else {
      dialog.setVisible(true);
    }
  }

  /**
   * Closes about box dialog.
   */
  public void close() {
    if (dialog != null) {
      dialog.dispose();
    }
  }

/*^**************************************************************************^*/
/*- LookAndFeelChangeListener Implementation.                                -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  public void lookAndFeelChanged(LookAndFeel oldLnF, LookAndFeel newLnF) {
    if (!lnfChanged) {
      lnfChanged = true;
    }
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-19 21:30:34 Slobodan
   Initial version

*****************************************************************************^*/
