/*
 * @(#)PlafComboBoxModel.java  1.0.0 / 2008-05-22
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.swing.components.models;

import java.lang.ref.SoftReference;
import java.util.Iterator;

import javax.swing.ComboBoxModel;
import javax.swing.LookAndFeel;
import javax.swing.event.ListDataEvent;
import javax.swing.event.ListDataListener;

import net.wrka.swing.plaf.LookAndFeelChangeListener;
import net.wrka.swing.plaf.LookAndFeelManager;
import net.wrka.util.SimpleStringList;
import net.wrka.util.ref.WeakArrayList;

/**
 * Look and Feels ComboBox Model.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public final class PlafComboBoxModel implements ComboBoxModel,
                                                LookAndFeelChangeListener {

/*^**************************************************************************^*/
/*- Private Static Field.                                                    -*/
/*^**************************************************************************^*/

  private static SoftReference cachedInstance;

/*^**************************************************************************^*/
/*- Static Initialization.                                                   -*/
/*^**************************************************************************^*/

  static {
    cachedInstance = new SoftReference(new PlafComboBoxModel());
  }

/*^**************************************************************************^*/
/*- Private Fields.                                                          -*/
/*^**************************************************************************^*/

  private final SimpleStringList lnfNames;
  private final WeakArrayList listeners;
  private Object selectedObject;
  private boolean updateEnabled;

/*^**************************************************************************^*/
/*- Private Constructor.                                                     -*/
/*^**************************************************************************^*/

  /**
   * Constructs new PlafComboBoxModel object.
   */
  private PlafComboBoxModel() {
    updateEnabled = true;
    lnfNames = new SimpleStringList(5);
    listeners = new WeakArrayList(12);
    updateModel();
    LookAndFeelManager.addLnFListener(this);
  }

/*^**************************************************************************^*/
/*- Public Static Method.                                                    -*/
/*^**************************************************************************^*/

  /**
   * Returns cached instance of this class.
   *
   * @return cached instance of this class
   */
  public static PlafComboBoxModel getInstance() {
    synchronized (PlafComboBoxModel.class) {
      PlafComboBoxModel instance = (PlafComboBoxModel)cachedInstance.get();
      if (instance == null) {
        instance = new PlafComboBoxModel();
        cachedInstance = new SoftReference(instance);
      }
      return instance;
    }
  }

/*^**************************************************************************^*/
/*- ComboBoxModel Interface Implementation.                                  -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  public void setSelectedItem(Object anItem) {
    synchronized (listeners) {
      if (((selectedObject != null) && (!selectedObject.equals(anItem))) ||
          (selectedObject == null) && (anItem != null)) {
        selectedObject = anItem;
        updateEnabled = false;
        LookAndFeelManager.setInstalledLookAndFeel(anItem.toString());
        updateEnabled = true;
        fireContentsChanged();
      }
    }
  }

  /**
   * {@inheritDoc}
   */
  public Object getSelectedItem() {
    synchronized (listeners)  {
      return selectedObject;
    }
  }

  /**
   * {@inheritDoc}
   */
  public int getSize() {
    synchronized (listeners)  {
      return lnfNames.size();
    }
  }

  /**
   * {@inheritDoc}
   */
  public Object getElementAt(int index) {
    synchronized (listeners)  {
      return lnfNames.get(index);
    }
  }

  /**
   * {@inheritDoc}
   */
  public void addListDataListener(ListDataListener l) {
    synchronized (listeners)  {
      listeners.add(l);
    }
  }

  /**
   * {@inheritDoc}
   */
  public void removeListDataListener(ListDataListener l) {
    synchronized (listeners)  {
      listeners.remove(l);
    }
  }

/*^**************************************************************************^*/
/*- LookAndFeelChangeListener Interface Implementation.                      -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  public void lookAndFeelChanged(LookAndFeel oldLnF, LookAndFeel newLnF) {
    synchronized (listeners)  {
      if (updateEnabled) {
        int ind = lnfNames.indexOf(newLnF.getName());
        if (ind == -1) {
          lnfNames.add(newLnF.getName());
          selectedObject = newLnF.getName();
        } else {
          selectedObject = lnfNames.get(ind);
        }
        fireContentsChanged();
      }
    }
  }

/*^**************************************************************************^*/
/*- Private Methods.                                                         -*/
/*^**************************************************************************^*/

  private void updateModel() {
    lnfNames.clear();
    Iterator items = LookAndFeelManager.lnfNames();
    while (items.hasNext()) {
      lnfNames.add((String)items.next());
    }
    lnfNames.sort();
    selectedObject = LookAndFeelManager.getCurrentLookAndFeel().getName();
    fireContentsChanged();
  }

  private void fireContentsChanged() {
    ListDataEvent e = new ListDataEvent(this, ListDataEvent.CONTENTS_CHANGED,
                                        -1, -1);
    boolean hasnull = false;
    final WeakArrayList ls = listeners;
    for (int i = ls.size() - 1; i >= 0; i--) {
      ListDataListener ldl = (ListDataListener)ls.get(i);
      if (ldl == null) {
        hasnull = true;
      } else {
        ldl.contentsChanged(e);
      }
    }
    if (hasnull) {
      ls.expungeStaleReferences();
    }
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-22 13:26:56 Slobodan
   Initial version

*****************************************************************************^*/
