/*
 * @(#)PlafThemeComboBoxModel.java  1.0.0 / 2008-05-22
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.swing.components.models;

import java.lang.ref.SoftReference;

import javax.swing.ComboBoxModel;
import javax.swing.LookAndFeel;
import javax.swing.event.ListDataEvent;
import javax.swing.event.ListDataListener;
import javax.swing.plaf.metal.MetalLookAndFeel;
import javax.swing.plaf.metal.MetalTheme;

import net.wrka.swing.plaf.LookAndFeelChangeListener;
import net.wrka.swing.plaf.LookAndFeelManager;
import net.wrka.swing.plaf.metal.MetalThemeManager;
import net.wrka.swing.plaf.smooth.SmoothLookAndFeel;
import net.wrka.swing.plaf.smooth.SmoothTheme;
import net.wrka.swing.plaf.smooth.SmoothThemeManager;
import net.wrka.util.CommonConstants;
import net.wrka.util.ref.WeakArrayList;

/**
 * Look and Feel Themes ComboBox Model.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public final class PlafThemeComboBoxModel implements ComboBoxModel,
                                                     LookAndFeelChangeListener {

/*^**************************************************************************^*/
/*- Public Static Final Fields.                                              -*/
/*^**************************************************************************^*/

  /**
   * NO theme name.
   */
  public static final String NO_THEME;

/*^**************************************************************************^*/
/*- Private Static Final Fields.                                             -*/
/*^**************************************************************************^*/

  private static SoftReference cachedInstance;

/*^**************************************************************************^*/
/*- Static Initialization.                                                   -*/
/*^**************************************************************************^*/

  static {
    NO_THEME = "None"; //$NON-NLS-1$
    cachedInstance = new SoftReference(new PlafThemeComboBoxModel());
  }

/*^**************************************************************************^*/
/*- Private Fields.                                                          -*/
/*^**************************************************************************^*/

  private final ThemeWrapper[] metalThemes;
  private final ThemeWrapper[] smoothThemes;
  private Object selectedObject;
  private Object metalSelectedObject;
  private Object smoothSelectedObject;
  private int lnfIndex;
  private boolean updateEnabled;
  private final WeakArrayList listeners;

/*^**************************************************************************^*/
/*- Private Constructor.                                                     -*/
/*^**************************************************************************^*/

  /**
   * Constructs new PlafThemeComboBoxModel object.
   */
  private PlafThemeComboBoxModel() {
    int metalThemeCount = MetalThemeManager.getThemeCount();
    metalThemes = new ThemeWrapper[metalThemeCount];
    for (int i = 0; i < metalThemeCount; i++) {
      metalThemes[i] = new ThemeWrapper(MetalThemeManager.getTheme(i));
    }
    int smoothThemeCount = SmoothThemeManager.getThemeCount();
    smoothThemes = new ThemeWrapper[smoothThemeCount];
    for (int i = 0; i < smoothThemeCount; i++) {
      smoothThemes[i] = new ThemeWrapper(SmoothThemeManager.getTheme(i));
    }
    updateEnabled = true;
    metalSelectedObject = metalThemes[0];
    smoothSelectedObject = smoothThemes[0];
    listeners = new WeakArrayList(12);
    updateModel();
    LookAndFeelManager.addLnFListener(this);
  }

/*^**************************************************************************^*/
/*- Public Static Methods.                                                   -*/
/*^**************************************************************************^*/

  /**
   * Returns cached instance of the class.
   *
   * @return cached instance of the class
   */
  public static PlafThemeComboBoxModel getInstance() {
    synchronized (PlafThemeComboBoxModel.class) {
      PlafThemeComboBoxModel instance =
                                  (PlafThemeComboBoxModel)cachedInstance.get();
      if (instance == null) {
        instance = new PlafThemeComboBoxModel();
        cachedInstance = new SoftReference(instance);
      }
      return instance;
    }
  }

/*^**************************************************************************^*/
/*- ComboBoxModel Implementation.                                            -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  public void setSelectedItem(Object anItem) {
    synchronized (listeners) {
      if (((selectedObject != null) && (!selectedObject.equals(anItem))) ||
          (selectedObject == null) && (anItem != null)) {
        selectedObject = anItem;
        updateEnabled = false;
        LookAndFeel lnf;
        ThemeWrapper themeWrapper;
        switch (lnfIndex) {
          case 0:
            metalSelectedObject = selectedObject;
            lnf = LookAndFeelManager.getCurrentLookAndFeel();
            if (!(lnf instanceof MetalLookAndFeel)) {
              lnf = new MetalLookAndFeel();
            }
            themeWrapper = (ThemeWrapper)selectedObject;
            MetalTheme metalTheme = themeWrapper.getMetalTheme();
            MetalLookAndFeel.setCurrentTheme(metalTheme);
            LookAndFeelManager.setNewThemeLookAndFeel(lnf);
            break;
          case 1:
            smoothSelectedObject = selectedObject;
            lnf = LookAndFeelManager.getCurrentLookAndFeel();
            if (!(lnf instanceof MetalLookAndFeel)) {
              lnf = new SmoothLookAndFeel();
            }
            themeWrapper = (ThemeWrapper)selectedObject;
            SmoothTheme smoothTheme = themeWrapper.getSmoothTheme();
            SmoothLookAndFeel.setCurrentTheme(smoothTheme);
            LookAndFeelManager.setNewThemeLookAndFeel(lnf);
            break;
          default:
            break;
        }
        updateEnabled = true;
      }
      fireContentsChanged();
    }
  }

  /**
   * {@inheritDoc}
   */
  public Object getSelectedItem() {
    synchronized (listeners) {
      return selectedObject;
    }
  }

  /**
   * {@inheritDoc}
   */
  public int getSize() {
    synchronized (listeners) {
      switch (lnfIndex) {
        case 0:
          return metalThemes.length;
        case 1:
          return smoothThemes.length;
        default:
          return 1;
      }
    }
  }

  /**
   * {@inheritDoc}
   */
  public Object getElementAt(int index) {
    synchronized (listeners) {
      switch (lnfIndex) {
        case 0:
          return metalThemes[index];
        case 1:
          return smoothThemes[index];
        default:
          return NO_THEME;
      }
    }
  }

  /**
   * {@inheritDoc}
   */
  public void addListDataListener(ListDataListener l) {
    synchronized (listeners) {
      listeners.add(l);
    }
  }

  /**
   * {@inheritDoc}
   */
  public void removeListDataListener(ListDataListener l) {
    synchronized (listeners) {
      listeners.remove(l);
    }
  }

/*^**************************************************************************^*/
/*- LookAndFeelChangeListener Implementation.                                -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  public void lookAndFeelChanged(LookAndFeel oldLnF, LookAndFeel newLnF) {
    synchronized (listeners) {
      if (updateEnabled) {
        updateModel();
      }
    }
  }

/*^**************************************************************************^*/
/*- Private Methods.                                                         -*/
/*^**************************************************************************^*/

  private void fireContentsChanged() {
    ListDataEvent e = new ListDataEvent(this, ListDataEvent.CONTENTS_CHANGED,
                                        -1, -1);
    boolean hasnull = false;
    WeakArrayList ls = listeners;
    for (int i = ls.size() - 1; i >= 0; i--) {
      ListDataListener ldl = (ListDataListener)ls.get(i);
      if (ldl == null) {
        hasnull = true;
      } else {
        ldl.contentsChanged(e);
      }
    }
    if (hasnull) {
      ls.expungeStaleReferences();
    }
  }

  private void updateModel() {
    LookAndFeel currentLnF = LookAndFeelManager.getCurrentLookAndFeel();
    String lnfName = currentLnF.getName();
    if ("Metal".equals(lnfName)) { //$NON-NLS-1$
      lnfIndex = 0;
      selectedObject = metalSelectedObject;
    } else if (SmoothLookAndFeel.NAME.equals(lnfName)) {
      lnfIndex = 1;
      selectedObject = smoothSelectedObject;
    } else {
      lnfIndex = 2;
      selectedObject = NO_THEME;
    }
    fireContentsChanged();
  }

/*^**************************************************************************^*/
/*- Inner Helper Class.                                                      -*/
/*^**************************************************************************^*/

  /**
   * Helper Class.
   */
  static final class ThemeWrapper {
    private final MetalTheme theme;
    /**
     * Constructs new ThemeWrapper object.
     *
     * @param theme the theme
     */
    public ThemeWrapper(MetalTheme theme) {
      this.theme = theme;
    }
    /**
     * Returns metal theme.
     *
     * @return metal theme
     */
    public MetalTheme getMetalTheme() {
      return theme;
    }
    /**
     * Returns Smooth Theme.
     *
     * @return Smooth Theme
     */
    public SmoothTheme getSmoothTheme() {
      return (SmoothTheme)theme;
    }
    /**
     * {@inheritDoc}
     */
    public String toString() {
      if (theme == null) {
        return CommonConstants.STRING_EMPTY;
      }
      return theme.getName();
    }
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-22 13:38:48 Slobodan
   Initial version

*****************************************************************************^*/
