/*
 * @(#)JWxEditorPane.java  1.0.0 / 2008-05-23
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.swing.components.text;

import java.awt.AWTEvent;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.awt.event.MouseEvent;
import java.io.IOException;
import java.net.URL;

import javax.swing.Action;
import javax.swing.Icon;
import javax.swing.JEditorPane;
import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;
import javax.swing.SwingUtilities;
import javax.swing.event.CaretEvent;
import javax.swing.event.CaretListener;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import javax.swing.event.UndoableEditEvent;
import javax.swing.event.UndoableEditListener;
import javax.swing.text.AbstractDocument;
import javax.swing.text.BadLocationException;
import javax.swing.text.Caret;
import javax.swing.text.Document;
import javax.swing.text.Element;
import javax.swing.undo.CannotRedoException;
import javax.swing.undo.CannotUndoException;
import javax.swing.undo.UndoManager;

import net.wrka.awt.Clipboard;
import net.wrka.swing.actions.ClearAction;
import net.wrka.swing.actions.CopyAction;
import net.wrka.swing.actions.CutAction;
import net.wrka.swing.actions.PasteAction;
import net.wrka.swing.actions.RedoAction;
import net.wrka.swing.actions.SelectAllAction;
import net.wrka.swing.actions.UndoAction;

/**
 * Extended JEditorPane.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public class JWxEditorPane extends JEditorPane {

/*^**************************************************************************^*/
/*- Public Static Final Field.                                               -*/
/*^**************************************************************************^*/

  /**
   * Property <i>line wrap changed</i>.
   */
  public static final String LINE_WRAP_CHANGED =
                                               "LineWrapChanged"; //$NON-NLS-1$

  /**
   * Property <i>document modified</i>.
   */
  public static final String DOCUMENT_MODIFIED =
                                              "DocumentModified"; //$NON-NLS-1$

  /**
   * Property <i>popup changed</i>.
   */
  public static final String POPUP_CHANGED = "JPopupChanged"; //$NON-NLS-1$

  /**
   * Property <i>undo manager changed</i>.
   */
  public static final String UNDO_MANAGER_CHANGED =
                                            "UndoManagerChanged"; //$NON-NLS-1$

  /**
   * Property <i>undo action changed</i>.
   */
  public static final String UNDO_ACTION_CHANGED =
                                             "UndoActionChanged"; //$NON-NLS-1$

  /**
   * Property <i>redo action changed</i>.
   */
  public static final String REDO_ACTION_CHANGED =
                                             "RedoActionChanged"; //$NON-NLS-1$

  /**
   * Property <i>cut action changed</i>.
   */
  public static final String CUT_ACTION_CHANGED =
                                              "CutActionChanged"; //$NON-NLS-1$

  /**
   * Property <i>copy action changed</i>.
   */
  public static final String COPY_ACTION_CHANGED =
                                             "CopyActionChanged"; //$NON-NLS-1$

  /**
   * Property <i>paste action changed</i>.
   */
  public static final String PASTE_ACTION_CHANGED =
                                            "PasteActionChanged"; //$NON-NLS-1$

  /**
   * Property <i>clear action changed</i>.
   */
  public static final String CLEAR_ACTION_CHANGED =
                                            "ClearActionChanged"; //$NON-NLS-1$

  /**
   * Property <i>select all action changed</i>.
   */
  public static final String SELECTALL_ACTION_CHANGED =
                                        "SelectAllActionChanged"; //$NON-NLS-1$

/*^**************************************************************************^*/
/*- Private Fields.                                                          -*/
/*^**************************************************************************^*/

  /**
   * Wrap lines flag.
   */
  protected boolean wrapLines;

  /**
   * Undo action.
   */
  protected Action actionUndo;

  /**
   * Redo action.
   */
  protected Action actionRedo;

  /**
   * Cut action.
   */
  protected Action actionCut;

  /**
   * Copy action.
   */
  protected Action actionCopy;

  /**
   * Paste action.
   */
  protected Action actionPaste;

  /**
   * Clear action.
   */
  protected Action actionClear;

  /**
   * Select All action.
   */
  protected Action actionSelectAll;

  /**
   * Modified flag.
   */
  protected boolean modified;

  /**
   * Caret listener.
   */
  protected CaretListener caretListener;

  /**
   * Focus listener.
   */
  protected FocusListener focusListener;

  /**
   * Modified listener.
   */
  protected ModifiedListener modifiedListener;

  /**
   * Undo listener.
   */
  protected UndoableEditListener undoListener;
  /**
   * Undo manager.
   */
  protected UndoManager undoManager;

  /**
   * Popup.
   */
  protected JPopupMenu popup;

/*^**************************************************************************^*/
/*- Public Constructors.                                                     -*/
/*^**************************************************************************^*/

  /**
   * Constructs new JWxEditorPane object.
   */
  public JWxEditorPane() {
    super();
    initialize();
  }

  /**
   * Constructs new JWxEditorPane object.
   *
   * @param initialPage the initial page
   *
   * @throws IOException  the initial page is invalid
   */
  public JWxEditorPane(URL initialPage) throws IOException {
    super(initialPage);
    initialize();
  }

  /**
   * Constructs new JWxEditorPane object.
   *
   * @param url the initial page
   *
   * @throws IOException  the initial page is invalid
   */
  public JWxEditorPane(String url) throws IOException {
    super(url);
    initialize();
  }

  /**
   * Constructs new JWxEditorPane object.
   *
   * @param type the editor type
   * @param text the text
   */
  public JWxEditorPane(String type, String text) {
    super(type, text);
    initialize();
  }

  /**
   * Constructs new JWxEditorPane object.
   *
   * @param editable indicates whether or not this JWxEditorPane is editable
   */
  public JWxEditorPane(boolean editable) {
    super();
    super.setEditable(editable);
    initialize();
  }

  /**
   * Constructs new JWxEditorPane object.
   *
   * @param url the URL
   * @param editable indicates whether or not this JWxEditorPane is editable
   *
   * @exception IOException if the URL is <code>null</code> or
   *                        cannot be accessed
   */
  public JWxEditorPane(String url, boolean editable) throws IOException {
    super(url);
    super.setEditable(editable);
    initialize();
  }

  /**
   * Constructs new JWxEditorPane object.
   *
   * @param type MIME type of the given text
   * @param text the text to initialize with
   * @param editable indicates whether or not this JWxEditorPane is editable
   */
  public JWxEditorPane(String type, String text, boolean editable) {
    super(type, text);
    super.setEditable(editable);
    initialize();
  }

/*^**************************************************************************^*/
/*- Public Methods.                                                          -*/
/*^**************************************************************************^*/

  /**
   * Returns <tt>true</tt> if text can undo.
   *
   * @return <tt>true</tt> if text can undo
   */
  public boolean canUndo() {
    return undoManager.canUndo();
  }

  /**
   * Returns <tt>true</tt> if text can redo.
   *
   * @return <tt>true</tt> if text can redo
   */
  public boolean canRedo() {
    return undoManager.canRedo();
  }

  /**
   * Returns <tt>true</tt> if text can be cut.
   *
   * @return <tt>true</tt> if text can be cut
   */
  public boolean canCut() {
    return (isEditable() && isEnabled() && hasSelection());
  }

  /**
   * Returns <tt>true</tt> if text can be copied.
   *
   * @return <tt>true</tt> if text can be copied
   */
  public boolean canCopy() {
    return (isEnabled() && hasSelection());
  }

  /**
   * Returns <tt>true</tt> if system clipboard text can be pasted.
   *
   * @return <tt>true</tt> if system clipboard text can be pasted
   */
  public boolean canPaste() {
    return (isEditable() && isEnabled() && Clipboard.containsString());
  }

  /**
   * Returns <tt>true</tt> if text can be cleared.
   *
   * @return <tt>true</tt> if text can be cleared
   */
  public boolean canClear() {
    return (isEditable() && isEnabled() && hasSelection());
  }

  /**
   * Returns <tt>true</tt> if all taxt can be selected.
   *
   * @return <tt>true</tt> if all taxt can be selected
   */
  public boolean canSelectAll() {
    Document doc = getDocument();
    return ((doc != null) && (doc.getLength() > 0));
  }

  /**
   * Returns <tt>true</tt> if text selection exists.
   *
   * @return <tt>true</tt> if text selection exists
   */
  public boolean hasSelection() {
    Caret caret = getCaret();
    return ((caret != null) && (caret.getDot() != caret.getMark()));
  }

  /**
   * Returns the number of lines.
   *
   * @return the number of lines
   */
  public int getLineCount() {
    Document doc = getDocument();
    Element map = doc.getDefaultRootElement();
    return map.getElementCount();
  }

  /**
   * Translates an offset into the components text to a line number.
   *
   * @param offset the offset >= 0
   *
   * @return the line number >= 0
   *
   * @exception BadLocationException thrown if the offset is less than zero or
   *                                 greater than the document length.
   */
  public int getLineOfOffset(int offset) throws BadLocationException {
    Document doc = getDocument();
    if (offset < 0) {
      throw new BadLocationException(
                           "Can't translate offset to line", -1); //$NON-NLS-1$
    } else if (offset > doc.getLength()) {
      throw new BadLocationException(
                            "Can't translate offset to line", //$NON-NLS-1$
                            doc.getLength() + 1);
    } else {
      Element map = doc.getDefaultRootElement();
      return map.getElementIndex(offset);
    }
  }

  /**
   * Determines the offset of the start of the given line.
   *
   * @param line  the line number to translate >= 0
   *
   * @return the offset >= 0
   *
   * @exception BadLocationException thrown if the line is less than zero or
   *            greater or equal to the number of lines contained in the
   *            document (as reported by getLineCount)
   */
  public int getLineStartOffset(int line) throws BadLocationException {
    Document doc = getDocument();
    int lineCount = getLineCount();
    if (line < 0) {
      throw new BadLocationException("Negative line", -1); //$NON-NLS-1$
    } else if (line >= lineCount) {
      throw new BadLocationException("No such line", //$NON-NLS-1$
                                     doc.getLength() + 1);
    } else {
      Element map = doc.getDefaultRootElement();
      Element lineElem = map.getElement(line);
      return lineElem.getStartOffset();
    }
  }

  /**
   * Determines the offset of the end of the given line.
   *
   * @param line  the line >= 0
   *
   * @return the offset >= 0
   *
   * @exception BadLocationException Thrown if the line is less than zero or
   *            greater or equal to the number of lines contained in the
   *            document (as reported by getLineCount).
   */
  public int getLineEndOffset(int line) throws BadLocationException {
    Document doc = getDocument();
    int lineCount = getLineCount();
    if (line < 0) {
      throw new BadLocationException("Negative line", -1); //$NON-NLS-1$
    } else if (line >= lineCount) {
      throw new BadLocationException("No such line", //$NON-NLS-1$
                                     doc.getLength() + 1);
    } else {
      Element map = doc.getDefaultRootElement();
      Element lineElem = map.getElement(line);
      int endOffset = lineElem.getEndOffset();
      // hide the implicit break at the end of the document
      if (line == (lineCount - 1)) {
        return endOffset - 1;
      }
      return endOffset;
    }
  }

  /**
   * Sets caret at specified position.
   *
   * @param line line number (starts from 0)
   * @param col int column number (starts from 0)
   *
   * @throws BadLocationException if position is invalid;
   */
  public void setCaretAt(int line, int col) throws BadLocationException {
    Caret caret = getCaret();
    if (caret != null) {
      caret.setDot(getLineStartOffset(line) + col);
    }
  }

  /**
   * Returns <tt>true</tt> if wrap lines.
   *
   * @return <tt>true</tt> if wrap lines
   */
  public boolean getLineWrap() { // NOPMD
    return wrapLines;
  }

  /**
   * Sets wrap line state.
   *
   * @param wrap if <tt>true</tt> wraps lines
   */
  public void setLineWrap(boolean wrap) {
    if (wrapLines != wrap) {
      boolean oldWrapLines = wrapLines;
      wrapLines = wrap;
      firePropertyChange(LINE_WRAP_CHANGED, oldWrapLines, wrap);
    }
  }

  /**
   * Inserts the specified text at the specified position.
   *
   * @param str the text to insert
   * @param pos the position at which to insert >= 0
   *
   * @exception IllegalArgumentException if pos is an invalid position in
   *                                     the model
   */
  public void insert(String str, int pos) {
    Document doc = getDocument();
    if (doc != null) {
      try {
        doc.insertString(pos, str, null);
      } catch (BadLocationException e) {
        throw new IllegalArgumentException(e.getMessage()); // NOPMD
      }
    }
  }

  /**
   * Appends the given text to the end of the document.
   *
   * @param str the text to append
   */
  public void append(String str) {
    Document doc = getDocument();
    if (doc != null) {
      try {
        doc.insertString(doc.getLength(), str, null);
      } catch (BadLocationException e) { // NOPMD
        // empty catch block
      }
    }
  }

  /**
   * Replaces text from the indicated start to end position with
   * the new text specified.
   *
   * @param str the text to use as the replacement
   * @param start the start position >= 0
   * @param end the end position >= start
   *
   * @exception IllegalArgumentException if part of the range is an invalid
   *                                     position in the model
   */
  public void replaceRange(String str, int start, int end) {
    if (end < start) {
      throw new IllegalArgumentException("end before start"); //$NON-NLS-1$
    }
    Document doc = getDocument();
    if (doc != null) {
      try {
        if (doc instanceof AbstractDocument) {
          ((AbstractDocument)doc).replace(start, end - start, str, null);
        } else {
          doc.remove(start, end - start);
          doc.insertString(start, str, null);
        }
      } catch (BadLocationException e) {
        throw new IllegalArgumentException(e.getMessage()); // NOPMD
      }
    }
  }

  /**
   * Returns <tt>true</tt> if document is modified.
   *
   * @return <tt>true</tt> if document is modified
   */
  public boolean isDocumentModified() {
    return modified;
  }

  /**
   * Sets document modified status.
   *
   * @param b boolean
   */
  public void setDocumentModified(boolean b) {
    if (modified != b) {
      boolean oldModified = modified;
      modified = b;
      firePropertyChange(DOCUMENT_MODIFIED, oldModified, b);
    }
  }

  /**
   * Undoes last action.
   */
  public void undo() {
    try {
      undoManager.undo();
      updateUndoActions();
      modified = undoManager.canUndo();
    } catch (CannotUndoException cude) { // NOPMD
      // empty catch block
    }
  }

  /**
   * Redoes last action.
   */
  public void redo() {
    try {
      undoManager.redo();
      updateUndoActions();
    } catch (CannotRedoException ctde) { // NOPMD
      // empty catch block
    }
  }

  /**
   * Clears current selection.
   */
  public void clearSelection() {
    if (isEditable() && isEnabled()) {
      Document doc = getDocument();
      Caret caret = getCaret();
      try {
        int p0 = caret.getDot();
        int p1 = caret.getMark();
        if (p0 != p1) {
          if (p1 < p0) {
            doc.remove(p1, p0 - p1);
          } else {
            doc.remove(p0, p1 - p0);
          }
          updateEditActions();
        }
      } catch (BadLocationException e) { // NOPMD
        // empty catch block
      }
    }
  }

  /**
   * Clears document.
   */
  public void clearDocument() {
    try {
      Document doc = getDocument();
      doc.remove(0, doc.getLength());
      modified = false;
      undoManager.discardAllEdits();
      updateActions();
    } catch (BadLocationException ble) { // NOPMD
      // empty catch block
    }
  }

  /**
   * Returns component's popup.
   *
   * @return JPopupMenu
   */
  public JPopupMenu getJPopupMenu() {
    return popup;
  }

  /**
   * Sets component's popup.
   *
   * @param popup JPopupMenu
   */
  public void setJPopupMenu(JPopupMenu popup) {
    JPopupMenu oldPopup = this.popup;
    if (oldPopup != popup) { // NOPMD
      this.popup = popup;
      firePropertyChange(POPUP_CHANGED, oldPopup, popup);
    }
  }

  /**
   * Returns undo manager.
   *
   * @return undo manager
   */
  public UndoManager getUndoManager() {
    return undoManager;
  }

  /**
   * Sets undo manager.
   *
   * @param undoManager the undo manager
   */
  public void setUndoManager(UndoManager undoManager) {
    if (undoManager == null) {
      throw new IllegalArgumentException("Null UndoManager"); //$NON-NLS-1$
    }
    UndoManager oldUndoManager = this.undoManager;
    if (oldUndoManager != undoManager) { // NOPMD
      this.undoManager = undoManager;
      firePropertyChange(UNDO_MANAGER_CHANGED, oldUndoManager, undoManager);
    }
  }

  /**
   * Resets UndoManager.
   */
  public void resetUndoManager() {
    undoManager.discardAllEdits();
    updateActions();
  }

  /**
   * Returns <tt>undo</tt> action.
   *
   * @return <tt>undo</tt> action
   */
  public Action getUndoAction() {
    return actionUndo;
  }

  /**
   * Sets <tt>undo</tt> action.
   *
   * @param action <tt>undo</tt> action
   */
  public void setUndoAction(Action action) {
    if (actionUndo != action) { // NOPMD
      Action oldActionUndo = actionUndo;
      actionUndo = action;
      firePropertyChange(UNDO_ACTION_CHANGED, oldActionUndo, action);
    }
  }

  /**
   * Returns <tt>redo</tt> action.
   *
   * @return <tt>redo</tt> action
   */
  public Action getRedoAction() {
    return actionRedo;
  }

  /**
   * Sets <tt>redo</tt> action.
   *
   * @param action <tt>redo</tt> action
   */
  public void setRedoAction(Action action) {
    if (actionRedo != action) { // NOPMD
      Action oldActionRedo = actionRedo;
      actionRedo = action;
      firePropertyChange(REDO_ACTION_CHANGED, oldActionRedo, action);
    }
  }

  /**
   * Returns <tt>cut</tt> action.
   *
   * @return <tt>cut</tt> action
   */
  public Action getCutAction() {
    return actionCut;
  }

  /**
   * Sets <tt>cut</tt> action.
   *
   * @param action <tt>cut</tt> action
   */
  public void setCutAction(Action action) {
    if (actionCut != action) { // NOPMD
      Action oldActionCut = actionCut;
      actionCut = action;
      firePropertyChange(CUT_ACTION_CHANGED, oldActionCut, action);
    }
  }

  /**
   * Returns <tt>copy</tt> action.
   *
   * @return <tt>copy</tt> action
   */
  public Action getCopyAction() {
    return actionCopy;
  }

  /**
   * Sets <tt>copy</tt> action.
   *
   * @param action <tt>copy</tt> action
   */
  public void setCopyAction(Action action) {
    if (actionCopy != action) { // NOPMD
      Action oldActionCopy = actionCopy;
      actionCopy = action;
      firePropertyChange(COPY_ACTION_CHANGED, oldActionCopy, action);
    }
  }

  /**
   * Returns <tt>paste</tt> action.
   *
   * @return <tt>paste</tt> action
   */
  public Action getPasteAction() {
    return actionPaste;
  }

  /**
   * Sets <tt>paste</tt> action.
   *
   * @param action <tt>paste</tt> action
   */
  public void setPasteAction(Action action) {
    if (actionPaste != action) { // NOPMD
      Action oldActionPaste = actionPaste;
      actionPaste = action;
      firePropertyChange(PASTE_ACTION_CHANGED, oldActionPaste, action);
    }
  }

  /**
   * Returns <tt>clear</tt> action.
   *
   * @return <tt>clear</tt> action
   */
  public Action getClearAction() {
    return actionClear;
  }

  /**
   * Sets <tt>clear</tt> action.
   *
   * @param action <tt>clear</tt> action
   */
  public void setClearAction(Action action) {
    if (actionClear != action) { // NOPMD
      Action oldActionClear = actionClear;
      actionClear = action;
      firePropertyChange(CLEAR_ACTION_CHANGED, oldActionClear, action);
    }
  }

  /**
   * Returns <tt>select all</tt> action.
   *
   * @return <tt>select all</tt> action
   */
  public Action getSelectAllAction() {
    return actionSelectAll;
  }

  /**
   * Sets <tt>select all</tt> action.
   *
   * @param action <tt>select all</tt> action
   */
  public void setSelectAllAction(Action action) {
    if (actionSelectAll != action) { // NOPMD
      Action oldActionSelectAll = actionSelectAll;
      actionSelectAll = action;
      firePropertyChange(SELECTALL_ACTION_CHANGED, oldActionSelectAll, action);
    }
  }

  /**
   * Updates undo actions.
   */
  public void updateUndoActions() {
    if ((actionUndo != null) && (undoManager != null)) {
      actionUndo.setEnabled(undoManager.canUndo());
    }
    if ((actionRedo != null) && (undoManager != null)) {
      actionRedo.setEnabled(undoManager.canRedo());
    }
  }

  /**
   * Updates edit actions.
   */
  public void updateEditActions() { // NOPMD
    boolean enabled = isEnabled();
    boolean editable = isEditable();
    boolean hasSelection = hasSelection();
    boolean canCut = (enabled && editable && hasSelection);
    boolean canCopy = (enabled && hasSelection);
    boolean canPaste = (enabled && editable && Clipboard.containsString());
    boolean canClear = canCut;
    boolean canSelectAll = (enabled && canSelectAll());
    if ((actionCut != null) && (actionCut.isEnabled() != canCut)) {
      actionCut.setEnabled(canCut);
    }
    if ((actionCopy != null) && (actionCopy.isEnabled() != canCopy)) {
      actionCopy.setEnabled(canCopy);
    }
    if ((actionPaste != null) && (actionPaste.isEnabled() != canPaste)) {
      actionPaste.setEnabled(canPaste);
    }
    if ((actionClear != null) && (actionClear.isEnabled() != canClear)) {
      actionClear.setEnabled(canClear);
    }
    if ((actionSelectAll != null) &&
        (actionSelectAll.isEnabled() != canSelectAll)) {
      actionSelectAll.setEnabled(canSelectAll);
    }
  }

  /**
   * Updates actions.
   */
  public void updateActions() {
    updateUndoActions();
    updateEditActions();
  }

/*^**************************************************************************^*/
/*- Overridden Methods.                                                      -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  public void cut() {
    super.cut();
    updateEditActions();
  }

  /**
   * {@inheritDoc}
   */
  public void copy() {
    super.copy();
    updateEditActions();
  }

  /**
   * {@inheritDoc}
   */
  public void paste() {
    super.paste();
    updateEditActions();
  }

  /**
   * {@inheritDoc}
   */
  public void selectAll() {
    super.selectAll();
    updateEditActions();
  }

  /**
   * {@inheritDoc}
   */
  public void setDocument(Document doc) {
    Document olddoc = getDocument();
    if (olddoc != null) {
      if (undoListener != null) {
        olddoc.removeUndoableEditListener(undoListener);
      }
      if (modifiedListener != null) {
        olddoc.removeDocumentListener(modifiedListener);
      }
    }
    super.setDocument(doc);
    if (doc != null) {
      if (undoListener != null) {
        doc.addUndoableEditListener(undoListener);
        undoManager.discardAllEdits();
        updateUndoActions();
      }
      if (modifiedListener != null) {
        doc.addDocumentListener(modifiedListener);
      }
    }
    modified = false;
  }

  /**
   * {@inheritDoc}
   */
  public void setBounds(int x, int y, int width, int height) {
    if (!wrapLines) {
      Dimension psize = getParent().getSize();
      if (width < psize.width) {
        width = psize.width;
      }
    }
    super.setBounds(x, y, width, height);
  }

  /**
   * {@inheritDoc}
   */
  public boolean getScrollableTracksViewportWidth() { // NOPMD
    if (wrapLines) {
      return super.getScrollableTracksViewportWidth();
    }
    // Disable text wrap problem solution
    return (getSize().width < getParent().getSize().width);
  }

  /**
   * {@inheritDoc}
   */
  public void setEditable(boolean b) {
    super.setEditable(b);
    if (popup == null) {
      popup = craeteJPopupMenu();
    } else {
      if (b) {
        if (actionUndo != null) {
          actionUndo.setEnabled(true);
        }
        if (actionRedo != null) {
          actionRedo.setEnabled(true);
        }
        if (actionCut != null) {
          actionCut.setEnabled(true);
          popup.add(new JMenuItem(actionCut), 0);
        }
        if (actionPaste != null) {
          actionPaste.setEnabled(true);
          popup.add(new JMenuItem(actionPaste), 2);
        }
        if (actionClear != null) {
          actionClear.setEnabled(true);
          popup.add(new JMenuItem(actionClear), 3);
        }
      } else {
        if (actionUndo != null) {
          actionUndo.setEnabled(false);
        }
        if (actionRedo != null) {
          actionRedo.setEnabled(false);
        }
        if (actionClear != null) {
          actionClear.setEnabled(false);
          popup.remove(3);
        }
        if (actionPaste != null) {
          actionPaste.setEnabled(false);
          popup.remove(2);
        }
        if (actionCut != null) {
          actionCut.setEnabled(false);
          popup.remove(0);
        }
      }
    }
  }

  /**
   * {@inheritDoc}
   */
  public void updateUI() {
    super.updateUI();
    if (popup != null) {
      SwingUtilities.updateComponentTreeUI(popup);
    }
  }

  /**
   * {@inheritDoc}
   */
  protected void processMouseEvent(MouseEvent e) {
    if ((popup != null) && e.isPopupTrigger()) {
      Caret caret = getCaret();
      int p0 = caret.getDot();
      int p1 = caret.getMark();
      if (p1 < p0) {
        p0 = p1;
        p1 = caret.getDot();
      }
      int cp = viewToModel(e.getPoint());
      if (!((p0 < p1) && (p0 <= cp) && (cp <= p1))) {
        setCaretPosition(cp);
      }
      if (!hasFocus()) {
        requestFocus(isEditable());
      }
      updateEditActions();
      popup.show(this, e.getX(), e.getY());
    }
    super.processMouseEvent(e);
  }

/*^**************************************************************************^*/
/*- Protected Methods.                                                       -*/
/*^**************************************************************************^*/

  /**
   * Initializes the component.
   */
  protected void initialize() {
    enableEvents(AWTEvent.MOUSE_EVENT_MASK);
    createUndoManager();
    createActions();
    createListners();
    Document doc = getDocument();
    if ((doc != null) && (undoListener != null)) {
      doc.addUndoableEditListener(undoListener);
      undoManager.discardAllEdits();
      updateUndoActions();
    }
    popup = craeteJPopupMenu();
  }

  /**
   * Creates undo manager.
   */
  protected void createUndoManager() {
    undoManager = new UndoManager();
  }

  /**
   * Creates edit actions.
   */
  protected void createEditActions() {
    if (isEditable()) {
      actionCut = new ActionCut();
      actionPaste = new ActionPaste();
      actionClear = new ActionClear();
    }
    actionCopy = new ActionCopy();
    actionSelectAll = new ActionSelectAll();
  }

  /**
   * Creates undo actions.
   */
  protected void createUndoActions() {
    if (isEditable()) {
      actionUndo = new ActionUndo();
      actionRedo = new ActionRedo();
    }
  }

  /**
   * Creates actions.
   */
  protected void createActions() {
    createEditActions();
    createUndoActions();
  }

  /**
   * Create popup menu.
   *
   * @return popup menu
   */
  protected JPopupMenu craeteJPopupMenu() {
    JPopupMenu popup = new JPopupMenu();
    if (actionCut != null) {
      popup.add(new JMenuItem(actionCut));
    }
    if (actionCopy != null) {
      popup.add(new JMenuItem(actionCopy));
    }
    if (actionPaste != null) {
      popup.add(new JMenuItem(actionPaste));
    }
    if (actionClear != null) {
      popup.add(new JMenuItem(actionClear));
    }
    if (actionSelectAll != null) {
      popup.addSeparator();
      popup.add(new JMenuItem(actionSelectAll));
    }
    return popup;
  }

  /**
   * Creates edit listeners.
   */
  protected void createEditListners() {
    focusListener = new UpdateFocusListener();
    addFocusListener(focusListener);
    caretListener = new UpdateCaretListener();
    addCaretListener(caretListener);
  }

  /**
   * Creates undo listeners.
   */
  protected void createUndoListner() {
    undoListener = new UndoListener();
  }

  /**
   * Creates listeners.
   */
  protected void createListners() {
    createEditListners();
    createUndoListner();
  }

  /**
   * Adds undoable edit event.
   *
   * @param e the event
   */
  protected void addUndoableEditEvent(UndoableEditEvent e) {
    setDocumentModified(true);
    undoManager.addEdit(e.getEdit());
    updateUndoActions();
  }

  /**
   * Updates caret action states.
   *
   * @param hasSelection <tt>true</tt> if has selection
   */
  protected void updateCaretActionStates(boolean hasSelection) {
    boolean enabled = isEnabled();
    boolean editable = isEditable();
    boolean canCutClear = (enabled && editable && hasSelection);
    if (actionCut != null) {
      actionCut.setEnabled(canCutClear);
    }
    if (actionCopy != null) {
      actionCopy.setEnabled(enabled && hasSelection);
    }
    if (actionClear != null) {
      actionClear.setEnabled(canCutClear);
    }
    if (actionSelectAll != null) {
      actionSelectAll.setEnabled(enabled && canSelectAll());
    }
  }

/*^**************************************************************************^*/
/*- Inner Classes.                                                           -*/
/*^**************************************************************************^*/

  /**
   * Action Undo.
   */
  protected class ActionUndo extends UndoAction {
    /**
     * Constructs new ActionUndo object.
     */
    public ActionUndo() {
      super();
    }
    /**
     * Constructs new ActionUndo object.
     *
     * @param name the name
     */
    public ActionUndo(String name) {
      super(name);
    }
    /**
     * Constructs new ActionUndo object.
     *
     * @param name the name
     * @param icon the icon
     */
    public ActionUndo(String name, Icon icon) {
      super(name, icon);
    }
    /**
     * {@inheritDoc}
     */
    public void actionPerformed(ActionEvent e) {
      undo();
    }
  }

  /**
   * Action Redo.
   */
  protected class ActionRedo extends RedoAction {
    /**
     * Constructs new ActionRedo object.
     */
    public ActionRedo() {
      super();
    }
    /**
     * Constructs new ActionRedo object.
     *
     * @param name the name
     */
    public ActionRedo(String name) {
      super(name);
    }
    /**
     * Constructs new ActionRedo object.
     *
     * @param name the name
     * @param icon the icon
     */
    public ActionRedo(String name, Icon icon) {
      super(name, icon);
    }
    /**
     * {@inheritDoc}
     */
    public void actionPerformed(ActionEvent e) {
      redo();
    }
  }

  /**
   * Action Cut.
   */
  protected class ActionCut extends CutAction {
    /**
     * Constructs new ActionCut object.
     */
    public ActionCut() {
      super();
    }
    /**
     * Constructs new ActionCut object.
     *
     * @param name the name
     */
    public ActionCut(String name) {
      super(name);
    }
    /**
     * Constructs new ActionCut object.
     *
     * @param name the name
     * @param icon the icon
     */
    public ActionCut(String name, Icon icon) {
      super(name, icon);
    }
    /**
     * {@inheritDoc}
     */
    public void actionPerformed(ActionEvent e) {
      cut();
    }
  }

  /**
   * Action Copy.
   */
  protected class ActionCopy extends CopyAction {
    /**
     * Constructs new ActionCopy object.
     */
    public ActionCopy() {
      super();
    }
    /**
     * Constructs new ActionCopy object.
     *
     * @param name the name
     */
    public ActionCopy(String name) {
      super(name);
    }
    /**
     * Constructs new ActionCopy object.
     *
     * @param name the name
     * @param icon the icon
     */
    public ActionCopy(String name, Icon icon) {
      super(name, icon);
    }
    /**
     * {@inheritDoc}
     */
    public void actionPerformed(ActionEvent e) {
      copy();
    }
  }

  /**
   * Action Paste.
   */
  protected class ActionPaste extends PasteAction {
    /**
     * Constructs new ActionPaste object.
     */
    public ActionPaste() {
      super();
    }
    /**
     * Constructs new ActionPaste object.
     *
     * @param name the name
     */
    public ActionPaste(String name) {
      super(name);
    }
    /**
     * Constructs new ActionPaste object.
     *
     * @param name the name
     * @param icon the icon
     */
    public ActionPaste(String name, Icon icon) {
      super(name, icon);
    }
    /**
     * {@inheritDoc}
     */
    public void actionPerformed(ActionEvent e) {
      paste();
    }
  }

  /**
   * Action Clear.
   */
  protected class ActionClear extends ClearAction {
    /**
     * Constructs new ActionClear object.
     */
    public ActionClear() {
      super();
    }
    /**
     * Constructs new ActionClear object.
     *
     * @param name the name
     */
    public ActionClear(String name) {
      super(name);
    }
    /**
     * Constructs new ActionClear object.
     *
     * @param name the name
     * @param icon the icon
     */
    public ActionClear(String name, Icon icon) {
      super(name, icon);
    }
    /**
     * {@inheritDoc}
     */
    public void actionPerformed(ActionEvent e) {
      clearSelection();
    }
  }

  /**
   * Action SelectAll.
   */
  protected class ActionSelectAll extends SelectAllAction {
    /**
     * Constructs new ActionSelectAll object.
     */
    public ActionSelectAll() {
      super();
    }
    /**
     * Constructs new ActionSelectAll object.
     *
     * @param name the name
     */
    public ActionSelectAll(String name) {
      super(name);
    }
    /**
     * Constructs new ActionSelectAll object.
     *
     * @param name the name
     * @param icon the icon
     */
    public ActionSelectAll(String name, Icon icon) {
      super(name, icon);
    }
    /**
     * {@inheritDoc}
     */
    public void actionPerformed(ActionEvent e) {
      selectAll();
    }
  }

  /**
   * Update Caret Listener.
   */
  protected class UpdateCaretListener implements CaretListener {
    private boolean hassel;
    private boolean hadsel;
    /**
     * {@inheritDoc}
     */
    public void caretUpdate(CaretEvent e) {
      hassel = (e.getDot() != e.getMark());
      if (hadsel || hassel) {
        updateCaretActionStates(hassel);
      }
      hadsel = hassel;
    }
  }

  /**
   * Update Focus Listener.
   */
  protected class UpdateFocusListener implements FocusListener {
    /**
     * {@inheritDoc}
     */
    public void focusGained(FocusEvent e) {
      updateEditActions();
    }
    /**
     * {@inheritDoc}
     */
    public void focusLost(FocusEvent e) {
      // empty method block
    }
  }

  /**
   * Undo Listener.
   */
  protected class UndoListener implements UndoableEditListener {
    /**
     * {@inheritDoc}
     */
    public void undoableEditHappened(UndoableEditEvent e) {
      addUndoableEditEvent(e);
    }
  }

  /**
   * Modified Listener.
   */
  protected class ModifiedListener implements DocumentListener {
    /**
     * {@inheritDoc}
     */
    public void changedUpdate(DocumentEvent e) {
      setDocumentModified(true);
    }
    /**
     * {@inheritDoc}
     */
    public void insertUpdate(DocumentEvent e) {
      setDocumentModified(true);
    }
    /**
     * {@inheritDoc}
     */
    public void removeUpdate(DocumentEvent e) {
      setDocumentModified(true);
    }
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-23 11:58:33 Slobodan
   Initial version

*****************************************************************************^*/
