/*
 * @(#)JWxTextArea.java  1.0.0 / 2008-05-23
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.swing.components.text;

import java.awt.AWTEvent;
import java.awt.event.ActionEvent;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.awt.event.MouseEvent;

import javax.swing.Action;
import javax.swing.Icon;
import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;
import javax.swing.JTextArea;
import javax.swing.SwingUtilities;
import javax.swing.event.CaretEvent;
import javax.swing.event.CaretListener;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import javax.swing.event.UndoableEditEvent;
import javax.swing.event.UndoableEditListener;
import javax.swing.text.BadLocationException;
import javax.swing.text.Caret;
import javax.swing.text.Document;
import javax.swing.undo.CannotRedoException;
import javax.swing.undo.CannotUndoException;
import javax.swing.undo.UndoManager;

import net.wrka.awt.Clipboard;
import net.wrka.swing.actions.ClearAction;
import net.wrka.swing.actions.CopyAction;
import net.wrka.swing.actions.CutAction;
import net.wrka.swing.actions.PasteAction;
import net.wrka.swing.actions.RedoAction;
import net.wrka.swing.actions.SelectAllAction;
import net.wrka.swing.actions.UndoAction;

/**
 * Extended JTextArea.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public class JWxTextArea extends JTextArea {

/*^**************************************************************************^*/
/*- Public Static Field.                                                     -*/
/*^**************************************************************************^*/

  /**
   * Property key <i>document modified</i>.
   */
  public static final String DOCUMENT_MODIFIED =
                                              "DocumentModified"; //$NON-NLS-1$

  /**
   * Property key <i>popup changed</i>.
   */
  public static final String POPUP_CHANGED = "JPopupChanged"; //$NON-NLS-1$

  /**
   * Property key <i>undo manager changed</i>.
   */
  public static final String UNDO_MANAGER_CHANGED =
                                            "UndoManagerChanged"; //$NON-NLS-1$

  /**
   * Property key <i>undo action changed</i>.
   */
  public static final String UNDO_ACTION_CHANGED =
                                             "UndoActionChanged"; //$NON-NLS-1$

  /**
   * Property key <i>redo action changed</i>.
   */
  public static final String REDO_ACTION_CHANGED =
                                             "RedoActionChanged"; //$NON-NLS-1$

  /**
   * Property key <i>cut action changed</i>.
   */
  public static final String CUT_ACTION_CHANGED =
                                              "CutActionChanged"; //$NON-NLS-1$

  /**
   * Property key <i>copy action changed</i>.
   */
  public static final String COPY_ACTION_CHANGED =
                                             "CopyActionChanged"; //$NON-NLS-1$

  /**
   * Property key <i>paste action changed</i>.
   */
  public static final String PASTE_ACTION_CHANGED =
                                            "PasteActionChanged"; //$NON-NLS-1$

  /**
   * Property key <i>clear action changed</i>.
   */
  public static final String CLEAR_ACTION_CHANGED =
                                            "ClearActionChanged"; //$NON-NLS-1$

  /**
   * Property key <i>select all action changed</i>.
   */
  public static final String SELECTALL_ACTION_CHANGED =
                                        "SelectAllActionChanged"; //$NON-NLS-1$

/*^**************************************************************************^*/
/*- Private Fields.                                                          -*/
/*^**************************************************************************^*/

  private Action actionUndo;
  private Action actionRedo;
  private Action actionCut;
  private Action actionCopy;
  private Action actionPaste;
  private Action actionClear;
  private Action actionSelectAll;
  private boolean modified;
  private ModifiedListener modifiedListener;
  private UndoableEditListener undoListener;
  private UndoManager undoManager;
  private JPopupMenu popup;

/*^**************************************************************************^*/
/*- Public Constructors.                                                     -*/
/*^**************************************************************************^*/

 /**
  * Constructs new JWxTextArea object.
  */
  public JWxTextArea() {
    this(null, null, 0, 0);
  }

  /**
   * Constructs new JWxTextArea object.
   *
   * @param editable indicates whether or not this JWxTextArea is editable
   */
  public JWxTextArea(boolean editable) {
    this(null, null, 0, 0, editable);
  }

  /**
   * Constructs new JWxTextArea object.
   *
   * @param text the text
   */
  public JWxTextArea(String text) {
    this(null, text, 0, 0);
  }

  /**
   * Constructs new JWxTextArea object.
   *
   * @param text the text to be displayed
   * @param editable indicates whether or not this JWxTextArea is editable
   */
  public JWxTextArea(String text, boolean editable) {
    this(null, text, 0, 0, editable);
  }

  /**
   * Constructs new JWxTextArea object.
   *
   * @param rows the row number
   * @param columns the column number
   */
  public JWxTextArea(int rows, int columns) {
    this(null, null, rows, columns);
  }

  /**
   * Constructs new JWxTextArea object.
   *
   * @param text the text
   * @param rows the row number
   * @param columns the column number
   */
  public JWxTextArea(String text, int rows, int columns) {
    this(null, text, rows, columns);
  }

  /**
   * Constructs new JWxTextArea object.
   *
   * @param doc the document
   */
  public JWxTextArea(Document doc) {
    this(doc, null, 0, 0);
  }

  /**
   * Constructs new JWxTextArea object.
   *
   * @param doc the Document
   * @param editable indicates whether or not this JWxTextArea is editable
   */
  public JWxTextArea(Document doc, boolean editable) {
    this(doc, null, 0, 0, editable);
  }

  /**
  /**
   * Constructs new JWxTextArea object.
   *
   * @param doc the Document
   * @param text the text to be displayed
   * @param rows the number of rows
   * @param columns the number of columns
   */
  public JWxTextArea(Document doc, String text, int rows, int columns) {
    this(doc, text, rows, columns, true);
  }

  /**
   * Constructs new JWxTextArea object.
   *
   * @param doc the Document
   * @param text the text to be displayed
   * @param rows the number of rows
   * @param columns the number of columns
   * @param editable indicates whether or not this JWxTextArea is editable
   */
  public JWxTextArea(Document doc, String text, int rows, int columns,
                     boolean editable) {
    super(doc, text, rows, columns);
    super.setEditable(editable);
    initalize();
  }

/*^**************************************************************************^*/
/*- Public Methods.                                                          -*/
/*^**************************************************************************^*/

  /**
   * Returns <tt>true</tt> if text can undo.
   *
   * @return <tt>true</tt> if text can undo
   */
  public boolean canUndo() {
    return undoManager.canUndo();
  }

  /**
   * Returns <tt>true</tt> if text can redo.
   *
   * @return <tt>true</tt> if text can redo
   */
  public boolean canRedo() {
    return undoManager.canRedo();
  }

  /**
   * Returns <tt>true</tt> if text can be cut.
   *
   * @return <tt>true</tt> if text can be cut
   */
  public boolean canCut() {
    return (isEditable() && isEnabled() && hasSelection());
  }

  /**
   * Returns <tt>true</tt> if text can be copied.
   *
   * @return <tt>true</tt> if text can be copied
   */
  public boolean canCopy() {
    return (isEnabled() && hasSelection());
  }

  /**
   * Returns <tt>true</tt> if system clipboard text can be pasted.
   *
   * @return <tt>true</tt> if system clipboard text can be pasted
   */
  public boolean canPaste() {
    return (isEditable() && isEnabled() && Clipboard.containsString());
  }

  /**
   * Returns <tt>true</tt> if text can be cleared.
   *
   * @return <tt>true</tt> if text can be cleared
   */
  public boolean canClear() {
    return (isEditable() && isEnabled() && hasSelection());
  }

  /**
   * Returns <tt>true</tt> if all taxt can be selected.
   *
   * @return <tt>true</tt> if all taxt can be selected
   */
  public boolean canSelectAll() {
    Document doc = getDocument();
    return ((doc != null) && (doc.getLength() > 0));
  }

  /**
   * Returns <tt>true</tt> if text selection exists.
   *
   * @return <tt>true</tt> if text selection exists
   */
  public boolean hasSelection() {
    Caret caret = getCaret();
    return ((caret != null) && (caret.getDot() != caret.getMark()));
  }

  /**
   * Sets caret at specified position.
   *
   * @param line line number (starts from 0)
   * @param col int column number (starts from 0)
   *
   * @throws BadLocationException if position is invalid;
   */
  public void setCaretAt(int line, int col) throws BadLocationException {
    Caret caret = getCaret();
    if (caret != null) {
      caret.setDot(getLineStartOffset(line) + col);
    }
  }

  /**
   * Undoes last action.
   */
  public void undo() {
    try {
      undoManager.undo();
      updateUndoActions();
      setDocumentModified(undoManager.canUndo());
    } catch (CannotUndoException cue) { // NOPMD
      // empty catch block
    }
  }

  /**
   * Redoes last action.
   */
  public void redo() {
    try {
      undoManager.redo();
      updateUndoActions();
    } catch (CannotRedoException cre) { // NOPMD
      // empty catch block
    }
  }

  /**
   * Clears current selection.
   */
  public void clearSelection() {
    if (isEnabled() && isEditable()) {
      Document doc = getDocument();
      if (doc != null) {
        Caret caret = getCaret();
        try {
          int p0 = caret.getDot();
          int p1 = caret.getMark();
          if (p0 != p1) {
            if (p1 < p0) {
              doc.remove(p1, p0 - p1);
            } else {
              doc.remove(p0, p1 - p0);
            }
            updateEditActions();
          }
        } catch (BadLocationException e) { // NOPMD
          // empty catch block
        }
      }
    }
  }

  /**
   * Clears document.
   */
  public void clearDocument() {
    try {
      Document doc = getDocument();
      if (doc != null) {
        doc.remove(0, doc.getLength());
        setDocumentModified(false);
        undoManager.discardAllEdits();
        updateActions();
      }
    } catch (BadLocationException ble) { // NOPMD
      // empty catch block
    }
  }

  /**
   * Returns undo manager.
   *
   * @return undo manager
   */
  public UndoManager getUndoManager() {
    return undoManager;
  }

  /**
   * Sets undo manager.
   *
   * @param undoManager the undo manager
   */
  public void setUndoManager(UndoManager undoManager) {
    if (undoManager == null) {
      throw new IllegalArgumentException("Null UndoManager"); //$NON-NLS-1$
    }
    UndoManager oldUndoManager = this.undoManager;
    if (oldUndoManager != undoManager) { // NOPMD
      this.undoManager = undoManager;
      firePropertyChange(UNDO_MANAGER_CHANGED, oldUndoManager, undoManager);
    }
  }

  /**
   * Resets UndoManager.
   */
  public void resetUndoManager() {
    undoManager.discardAllEdits();
    updateActions();
  }

  /**
   * Returns <tt>undo</tt> action.
   *
   * @return <tt>undo</tt> action
   */
  public Action getUndoAction() {
    return actionUndo;
  }

  /**
   * Sets <tt>undo</tt> action.
   *
   * @param action <tt>undo</tt> action
   */
  public void setUndoAction(Action action) {
    if (actionUndo != action) { // NOPMD
      Action oldActionUndo = actionUndo;
      actionUndo = action;
      firePropertyChange(UNDO_ACTION_CHANGED, oldActionUndo, action);
    }
  }

  /**
   * Returns <tt>redo</tt> action.
   *
   * @return <tt>redo</tt> action
   */
  public Action getRedoAction() {
    return actionRedo;
  }

  /**
   * Sets <tt>redo</tt> action.
   *
   * @param action <tt>redo</tt> action
   */
  public void setRedoAction(Action action) {
    if (actionRedo != action) { // NOPMD
      Action oldActionRedo = actionRedo;
      actionRedo = action;
      firePropertyChange(REDO_ACTION_CHANGED, oldActionRedo, action);
    }
  }

  /**
   * Returns <tt>cut</tt> action.
   *
   * @return <tt>cut</tt> action
   */
  public Action getCutAction() {
    return actionCut;
  }

  /**
   * Sets <tt>cut</tt> action.
   *
   * @param action <tt>cut</tt> action
   */
  public void setCutAction(Action action) {
    if (actionCut != action) { // NOPMD
      Action oldActionCut = actionCut;
      actionCut = action;
      firePropertyChange(CUT_ACTION_CHANGED, oldActionCut, action);
    }
  }

  /**
   * Returns <tt>copy</tt> action.
   *
   * @return <tt>copy</tt> action
   */
  public Action getCopyAction() {
    return actionCopy;
  }

  /**
   * Sets <tt>copy</tt> action.
   *
   * @param action <tt>copy</tt> action
   */
  public void setCopyAction(Action action) {
    if (actionCopy != action) { // NOPMD
      Action oldActionCopy = actionCopy;
      actionCopy = action;
      firePropertyChange(COPY_ACTION_CHANGED, oldActionCopy, action);
    }
  }

  /**
   * Returns <tt>paste</tt> action.
   *
   * @return <tt>paste</tt> action
   */
  public Action getPasteAction() {
    return actionPaste;
  }

  /**
   * Sets <tt>paste</tt> action.
   *
   * @param action <tt>paste</tt> action
   */
  public void setPasteAction(Action action) {
    if (actionPaste != action) { // NOPMD
      Action oldActionPaste = actionPaste;
      actionPaste = action;
      firePropertyChange(PASTE_ACTION_CHANGED, oldActionPaste, action);
    }
  }

  /**
   * Returns <tt>clear</tt> action.
   *
   * @return <tt>clear</tt> action
   */
  public Action getClearAction() {
    return actionClear;
  }

  /**
   * Sets <tt>clear</tt> action.
   *
   * @param action <tt>clear</tt> action
   */
  public void setClearAction(Action action) {
    if (actionClear != action) { // NOPMD
      Action oldActionClear = actionClear;
      actionClear = action;
      firePropertyChange(CLEAR_ACTION_CHANGED, oldActionClear, action);
    }
  }

  /**
   * Returns <tt>select all</tt> action.
   *
   * @return <tt>select all</tt> action
   */
  public Action getSelectAllAction() {
    return actionSelectAll;
  }

  /**
   * Sets <tt>select all</tt> action.
   *
   * @param action <tt>select all</tt> action
   */
  public void setSelectAllAction(Action action) {
    if (actionSelectAll != action) { // NOPMD
      Action oldActionSelectAll = actionSelectAll;
      actionSelectAll = action;
      firePropertyChange(SELECTALL_ACTION_CHANGED, oldActionSelectAll, action);
    }
  }

  /**
   * Returns <tt>true</tt> if document is modified.
   *
   * @return <tt>true</tt> if document is modified
   */
  public boolean isDocumentModified() {
    return modified;
  }

  /**
   * Sets document modified status.
   *
   * @param b boolean
   */
  public void setDocumentModified(boolean b) {
    if (modified != b) {
      boolean oldModified = modified;
      modified = b;
      firePropertyChange(DOCUMENT_MODIFIED, oldModified, b);
    }
  }

  /**
   * Returns component's popup.
   *
   * @return JPopupMenu
   */
  public JPopupMenu getJPopupMenu() {
    return popup;
  }

  /**
   * Sets component's popup.
   *
   * @param popup JPopupMenu
   */
  public void setJPopupMenu(JPopupMenu popup) {
    JPopupMenu oldPopup = this.popup;
    if (oldPopup != popup) { // NOPMD
      this.popup = popup;
      firePropertyChange(POPUP_CHANGED, oldPopup, popup);
    }
  }

  /**
   * Updates undo actions.
   */
  public void updateUndoActions() {
    if (actionUndo != null) {
      actionUndo.setEnabled(undoManager.canUndo());
    }
    if (actionRedo != null) {
      actionRedo.setEnabled(undoManager.canRedo());
    }
  }

  /**
   * Updates edit actions.
   */
  public void updateEditActions() { // NOPMD
    boolean enabled = isEnabled();
    boolean editable = isEditable();
    boolean hasSelection = hasSelection();
    boolean canCut = (enabled && editable && hasSelection);
    boolean canCopy = (enabled && hasSelection);
    boolean canPaste = (enabled && editable && Clipboard.containsString());
    boolean canClear = canCut;
    boolean canSelectAll = (enabled && canSelectAll());
    if ((actionCut != null) && (actionCut.isEnabled() != canCut)) {
      actionCut.setEnabled(canCut);
    }
    if ((actionCopy != null) && (actionCopy.isEnabled() != canCopy)) {
      actionCopy.setEnabled(canCopy);
    }
    if ((actionPaste != null) && (actionPaste.isEnabled() != canPaste)) {
      actionPaste.setEnabled(canPaste);
    }
    if ((actionClear != null) && (actionClear.isEnabled() != canClear)) {
      actionClear.setEnabled(canClear);
    }
    if ((actionSelectAll != null) &&
        (actionSelectAll.isEnabled() != canSelectAll)) {
      actionSelectAll.setEnabled(canSelectAll);
    }
  }

  /**
   * Updates actions.
   */
  public void updateActions() {
    updateUndoActions();
    updateEditActions();
  }

/*^**************************************************************************^*/
/*- Overridden Methods.                                                      -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  public void cut() {
    super.cut();
    updateEditActions();
  }

  /**
   * {@inheritDoc}
   */
  public void copy() {
    super.copy();
    updateEditActions();
  }

  /**
   * {@inheritDoc}
   */
  public void paste() {
    super.paste();
    updateEditActions();
  }

  /**
   * {@inheritDoc}
   */
  public void selectAll() {
    super.selectAll();
    updateEditActions();
  }

  /**
   * {@inheritDoc}
   */
  public void setDocument(Document doc) {
    Document olddoc = getDocument();
    if (olddoc != null) {
      if (undoListener != null) {
        olddoc.removeUndoableEditListener(undoListener);
      }
      if (modifiedListener != null) {
        olddoc.removeDocumentListener(modifiedListener);
      }
    }
    super.setDocument(doc);
    if (doc != null) {
      if (undoListener != null) {
        doc.addUndoableEditListener(undoListener);
        undoManager.discardAllEdits();
        updateUndoActions();
      }
      if (modifiedListener != null) {
        doc.addDocumentListener(modifiedListener);
      }
    }
    modified = false;
  }

  /**
   * {@inheritDoc}
   */
  public void setEditable(boolean b) {
    super.setEditable(b);
    if (popup == null) {
      popup = craeteJPopupMenu();
    } else {
      if (b) {
        if (actionUndo != null) {
          actionUndo.setEnabled(true);
        }
        if (actionRedo != null) {
          actionRedo.setEnabled(true);
        }
        if (actionCut != null) {
          actionCut.setEnabled(true);
          popup.add(new JMenuItem(actionCut), 0);
        }
        if (actionPaste != null) {
          actionPaste.setEnabled(true);
          popup.add(new JMenuItem(actionPaste), 2);
        }
        if (actionClear != null) {
          actionClear.setEnabled(true);
          popup.add(new JMenuItem(actionClear), 3);
        }
      } else {
        if (actionUndo != null) {
          actionUndo.setEnabled(false);
        }
        if (actionRedo != null) {
          actionRedo.setEnabled(false);
        }
        if (actionClear != null) {
          actionClear.setEnabled(false);
          popup.remove(3);
        }
        if (actionPaste != null) {
          actionPaste.setEnabled(false);
          popup.remove(2);
        }
        if (actionCut != null) {
          actionCut.setEnabled(false);
          popup.remove(0);
        }
      }
    }
  }

  /**
   * {@inheritDoc}
   */
  public void updateUI() {
    super.updateUI();
    if (popup != null) {
      SwingUtilities.updateComponentTreeUI(popup);
    }
  }

  /**
   * {@inheritDoc}
   */
  protected void processMouseEvent(MouseEvent e) {
    if ((popup != null) && e.isPopupTrigger()) {
      Caret caret = getCaret();
      int p0 = caret.getDot();
      int p1 = caret.getMark();
      if (p1 < p0) {
        p0 = p1;
        p1 = caret.getDot();
      }
      int cp = viewToModel(e.getPoint());
      if (!((p0 < p1) && (p0 <= cp) && (cp <= p1))) {
        setCaretPosition(cp);
      }
      if (!hasFocus()) {
        requestFocus(isEditable());
      }
      updateEditActions();
      popup.show(this, e.getX(), e.getY());
    }
    super.processMouseEvent(e);
  }

/*^**************************************************************************^*/
/*- Protected Methods.                                                       -*/
/*^**************************************************************************^*/

  /**
   * Initializes the component.
   */
  protected void initalize() {
    enableEvents(AWTEvent.MOUSE_EVENT_MASK);
    createUndoManager();
    createActions();
    createListners();
    Document doc = getDocument();
    if ((doc != null) && (undoListener != null)) {
      doc.addUndoableEditListener(undoListener);
      undoManager.discardAllEdits();
      updateUndoActions();
    }
    popup = craeteJPopupMenu();
  }

  /**
   * Creates undo manager.
   */
  protected void createUndoManager() {
    undoManager = new UndoManager();
  }

  /**
   * Creates edit actions.
   */
  protected void createEditActions() {
    if (isEditable()) {
      actionCut = new ActionCut();
      actionPaste = new ActionPaste();
      actionClear = new ActionClear();
    }
    actionCopy = new ActionCopy();
    actionSelectAll = new ActionSelectAll();
  }

  /**
   * Creates undo actions.
   */
  protected void createUndoActions() {
    if (isEditable()) {
      actionUndo = new ActionUndo();
      actionRedo = new ActionRedo();
    }
  }

  /**
   * Create actions.
   */
  protected void createActions() {
    createEditActions();
    createUndoActions();
  }

  /**
   * Creates popup menu.
   *
   * @return popup menu
   */
  protected JPopupMenu craeteJPopupMenu() {
    JPopupMenu popup = new JPopupMenu();
    if (actionCut != null) {
      popup.add(new JMenuItem(actionCut));
    }
    if (actionCopy != null) {
      popup.add(new JMenuItem(actionCopy));
    }
    if (actionPaste != null) {
      popup.add(new JMenuItem(actionPaste));
    }
    if (actionClear != null) {
      popup.add(new JMenuItem(actionClear));
    }
    if (actionSelectAll != null) {
      popup.addSeparator();
      popup.add(new JMenuItem(actionSelectAll));
    }
    return popup;
  }

  /**
   * Creates edit listeners.
   */
  protected void createEditListners() {
    addFocusListener(new UpdateFocusListener());
    addCaretListener(new UpdateCaretListener());
  }

  /**
   * Creates undo listener.
   */
  protected void createUndoListner() {
    undoListener = new UndoListener();
  }

  /**
   * Creates modified listener.
   */
  protected void createModifiedListner() {
    modifiedListener = new ModifiedListener();
  }

  /**
   * Creates listeners.
   */
  protected void createListners() {
    createEditListners();
    createUndoListner();
    createModifiedListner();
  }

  /**
   * Adds undoable edit event.
   *
   * @param e the event
   */
  protected void addUndoableEditEvent(UndoableEditEvent e) {
    setDocumentModified(true);
    undoManager.addEdit(e.getEdit());
    updateUndoActions();
  }

  /**
   * Update caret actions.
   *
   * @param hasSelection <tt>true</tt> if text area has selection
   */
  protected void updateCaretActionStates(boolean hasSelection) {
    boolean enabled = isEnabled();
    boolean editable = isEditable();
    boolean canCutClear = (enabled && editable && hasSelection);
    if (actionCut != null) {
      actionCut.setEnabled(canCutClear);
    }
    if (actionCopy != null) {
      actionCopy.setEnabled(enabled && hasSelection);
    }
    if (actionClear != null) {
      actionClear.setEnabled(canCutClear);
    }
    if (actionSelectAll != null) {
      actionSelectAll.setEnabled(enabled && canSelectAll());
    }
  }

/*^**************************************************************************^*/
/*- Inner Classes.                                                           -*/
/*^**************************************************************************^*/

  /**
   * Action Undo.
   */
  protected class ActionUndo extends UndoAction {
    /**
     * Constructs new ActionUndo object.
     */
    public ActionUndo() {
      super();
    }
    /**
     * Constructs new ActionUndo object.
     *
     * @param name the action name
     */
    public ActionUndo(String name) {
      super(name);
    }
    /**
     * Constructs new ActionUndo object.
     *
     * @param name the name
     * @param icon the icon
     */
    public ActionUndo(String name, Icon icon) {
      super(name, icon);
    }
    /**
     * {@inheritDoc}
     */
    public void actionPerformed(ActionEvent e) {
      undo();
    }
  }

  /**
   * Action Redo.
   */
  protected class ActionRedo extends RedoAction {
    /**
     * Constructs new ActionRedo object.
     */
    public ActionRedo() {
      super();
    }
    /**
     * Constructs new ActionRedo object.
     *
     * @param name the name
     */
    public ActionRedo(String name) {
      super(name);
    }
    /**
     * Constructs new ActionRedo object.
     *
     * @param name the name
     * @param icon the icon
     */
    public ActionRedo(String name, Icon icon) {
      super(name, icon);
    }
    /**
     * {@inheritDoc}
     */
    public void actionPerformed(ActionEvent e) {
      redo();
    }
  }

  /**
   * Action Cut.
   */
  protected class ActionCut extends CutAction {
    /**
     * Constructs new ActionCut object.
     */
    public ActionCut() {
      super();
    }
    /**
     * Constructs new ActionCut object.
     *
     * @param name the name
     */
    public ActionCut(String name) {
      super(name);
    }
    /**
     * Constructs new ActionCut object.
     *
     * @param name the name
     * @param icon the icon
     */
    public ActionCut(String name, Icon icon) {
      super(name, icon);
    }
    /**
     * {@inheritDoc}
     */
    public void actionPerformed(ActionEvent e) {
      cut();
    }
  }

  /**
   * Action Copy.
   */
  protected class ActionCopy extends CopyAction {
    /**
     * Constructs new ActionCopy object.
     */
    public ActionCopy() {
      super();
    }
    /**
     * Constructs new ActionCopy object.
     *
     * @param name the name
     */
    public ActionCopy(String name) {
      super(name);
    }
    /**
     * Constructs new ActionCopy object.
     *
     * @param name the name
     * @param icon the icon
     */
    public ActionCopy(String name, Icon icon) {
      super(name, icon);
    }
    /**
     * {@inheritDoc}
     */
    public void actionPerformed(ActionEvent e) {
      copy();
    }
  }

  /**
   * Action Paste.
   */
  protected class ActionPaste extends PasteAction {
    /**
     * Constructs new ActionPaste object.
     */
    public ActionPaste() {
      super();
    }
    /**
     * Constructs new ActionPaste object.
     *
     * @param name the name
     */
    public ActionPaste(String name) {
      super(name);
    }
    /**
     * Constructs new ActionPaste object.
     *
     * @param name the name
     * @param icon the icon
     */
    public ActionPaste(String name, Icon icon) {
      super(name, icon);
    }
    /**
     * {@inheritDoc}
     */
    public void actionPerformed(ActionEvent e) {
      paste();
    }
  }

  /**
   * Action Clear.
   */
  protected class ActionClear extends ClearAction {
    /**
     * Constructs new ActionClear object.
     */
    public ActionClear() {
      super();
    }
    /**
     * Constructs new ActionClear object.
     *
     * @param name the name
     */
    public ActionClear(String name) {
      super(name);
    }
    /**
     * Constructs new ActionClear object.
     *
     * @param name the name
     * @param icon the icon
     */
    public ActionClear(String name, Icon icon) {
      super(name, icon);
    }
    /**
     * {@inheritDoc}
     */
    public void actionPerformed(ActionEvent e) {
      clearSelection();
    }
  }

  /**
   * Action SelectAll.
   */
  protected class ActionSelectAll extends SelectAllAction {
    /**
     * Constructs new ActionSelectAll object.
     *
     */
    public ActionSelectAll() {
      super();
    }
    /**
     * Constructs new ActionSelectAll object.
     *
     * @param name the name
     */
    public ActionSelectAll(String name) {
      super(name);
    }
    /**
     * Constructs new ActionSelectAll object.
     *
     * @param name the name
     * @param icon the icon
     */
    public ActionSelectAll(String name, Icon icon) {
      super(name, icon);
    }
    /**
     * {@inheritDoc}
     */
    public void actionPerformed(ActionEvent e) {
      selectAll();
    }
  }

  /**
   * Update Caret Listener.
   */
  protected class UpdateCaretListener implements CaretListener {
    private boolean hassel;
    private boolean hadsel;
    /**
     * {@inheritDoc}
     */
    public void caretUpdate(CaretEvent e) {
      hassel = (e.getDot() != e.getMark());
      if (hadsel || hassel) {
        updateCaretActionStates(hassel);
      }
      hadsel = hassel;
    }
  }

  /**
   * Update Focus Listener.
   */
  protected class UpdateFocusListener implements FocusListener {
    /**
     * {@inheritDoc}
     */
    public void focusGained(FocusEvent e) {
      updateActions();
    }
    /**
     * {@inheritDoc}
     */
    public void focusLost(FocusEvent e) {
      // empty method body
    }
  }

  /**
   * Undo Listener.
   */
  protected class UndoListener implements UndoableEditListener {
    /**
     * {@inheritDoc}
     */
    public void undoableEditHappened(UndoableEditEvent e) {
      addUndoableEditEvent(e);
    }
  }

  /**
   * Modified Listener.
   */
  protected class ModifiedListener implements DocumentListener {
    /**
     * {@inheritDoc}
     */
    public void changedUpdate(DocumentEvent e) {
      setDocumentModified(true);
    }
    /**
     * {@inheritDoc}
     */
    public void insertUpdate(DocumentEvent e) {
      setDocumentModified(true);
    }
    /**
     * {@inheritDoc}
     */
    public void removeUpdate(DocumentEvent e) {
      setDocumentModified(true);
    }
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-23 11:10:16 Slobodan
   Initial version

*****************************************************************************^*/
