/*
 * @(#)LineEnumerator.java  1.0.0 / 2008-05-23
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.swing.components.text;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.Insets;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import javax.swing.JComponent;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import javax.swing.text.Document;
import javax.swing.text.Element;
import javax.swing.text.JTextComponent;

import net.wrka.swing.graphics.GraphicUtils;

/**
 * Line Enumerator.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0
 */
public class LineEnumerator extends JComponent
                            implements DocumentListener,
                                       PropertyChangeListener {

/*^**************************************************************************^*/
/*- Public Static Fields.                                                    -*/
/*^**************************************************************************^*/

  /**
   * Default foreground color.
   */
  public static final Color FOREGROUND_COLOR = new Color(163, 163, 204);

  /**
   * Default background color.
   */
  public static final Color BACKGROUND_COLOR =
                                  GraphicUtils.darkerColor(Color.WHITE, 0.05f);

  /**
   * Default line color.
   */
  public static final Color LINE_COLOR =
                                  GraphicUtils.darkerColor(Color.WHITE, 0.12f);

/*^**************************************************************************^*/
/*- Protected Fields.                                                        -*/
/*^**************************************************************************^*/

  private final JTextComponent jtc;
  private FontMetrics fm;
  private FontMetrics jtcfm;
  private int topInsets;
  private final int startLineNumber;
  private int lineCount;

/*^**************************************************************************^*/
/*- Public Constructor.                                                      -*/
/*^**************************************************************************^*/

  /**
   * Constructs new LineEnumerator object.
   *
   * @param jtc the text component
   * @param startLine the start line
   */
  public LineEnumerator(JTextComponent jtc, int startLine) {
    super();
    if (jtc == null) {
      throw new IllegalArgumentException("Null Text Component"); //$NON-NLS-1$
    }
    if (startLine < 0) {
      throw new IllegalArgumentException("Invalid start line"); //$NON-NLS-1$
    }
    setDoubleBuffered(true);
    this.jtc = jtc;
    lineCount = -1;
    startLineNumber = startLine;
    Document doc = jtc.getDocument();
    doc.addDocumentListener(this);
    jtc.addPropertyChangeListener("border", this); //$NON-NLS-1$
    jtc.addPropertyChangeListener("font", this); //$NON-NLS-1$
    jtc.addPropertyChangeListener("document", this); //$NON-NLS-1$
    updateFont();
    updateInstes();
  }

/*^**************************************************************************^*/
/*- Overridden Methods.                                                      -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  public Dimension getPreferredSize() {
    Dimension d = jtc.getPreferredSize();
    int prefWidth = getLineCount();
    fm = getFontMetrics(getFont());
    prefWidth = fm.stringWidth(String.valueOf(prefWidth)) + 4;
    if (prefWidth < 28) {
      prefWidth = 28;
    }
    d.width = prefWidth;
    return d;
  }

  /**
   * {@inheritDoc}
   */
  protected void paintComponent(Graphics g) {
    int w = getWidth();
    int wm1 = w - 1;
    int wm3 = w - 3;
    int h = getHeight();
    g.setColor(BACKGROUND_COLOR);
    g.fillRect(0, 0, wm1, h);
    g.setColor(LINE_COLOR);
    g.drawLine(wm1, 0, wm1, h);
    g.setColor(FOREGROUND_COLOR);
    int dh = jtcfm.getHeight();
    int currentLine = startLineNumber;
    int lc = getLineCount();
    if (currentLine == 0) {
      lc = lc - 1;
    }
    for (int i = jtcfm.getAscent() + topInsets; (currentLine <= lc) && (i < h);
         i += dh) {
      String line = String.valueOf(currentLine++);
      g.drawString(line, wm3 - fm.stringWidth(line), i);
    }
  }

/*^**************************************************************************^*/
/*- DocumentListener Implementation.                                         -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  public void insertUpdate(DocumentEvent e) {
    changedUpdate(e);
  }

  /**
   * {@inheritDoc}
   */
  public void removeUpdate(DocumentEvent e) {
    changedUpdate(e);
  }

  /**
   * {@inheritDoc}
   */
  public void changedUpdate(DocumentEvent e) {
    int lc = getLineCount();
    if (lc != lineCount) {
      repaint();
      lineCount = lc;
    }
  }

/*^**************************************************************************^*/
/*- PropertyChangeListener Implementation.                                   -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  public void propertyChange(PropertyChangeEvent evt) {
    String propname = evt.getPropertyName();
    if ("document".equals(evt.getPropertyName())) { //$NON-NLS-1$
      Document doc = (Document)evt.getOldValue();
      if (doc != null) {
        doc.removeDocumentListener(this);
      }
      doc = (Document)evt.getNewValue();
      if (doc != null) {
        doc.addDocumentListener(this);
      }
      repaint();
    } else if ("font".equals(propname)) { //$NON-NLS-1$
      updateFont();
      repaint();
    } else if ("border".equals(propname)) { //$NON-NLS-1$
      updateInstes();
      repaint();
    }
  }

/*^**************************************************************************^*/
/*- Private Methods.                                                         -*/
/*^**************************************************************************^*/

  private int getLineCount() {
    int lineCount = 0;
    Document doc = jtc.getDocument();
    if (doc != null) {
      Element dre = doc.getDefaultRootElement();
      if (dre != null) {
        lineCount = dre.getElementCount();
      }
    }
    return lineCount;
  }

  private void updateFont() {
    Font font = getFont();
    Font jtcFont = jtc.getFont();
    if ((font == null) || (!font.equals(jtcFont))) {
      jtcfm = jtc.getFontMetrics(jtcFont);
      jtcFont = jtcFont.deriveFont(Font.PLAIN, jtcFont.getSize() - 1);
      setFont(jtcFont);
      fm = getFontMetrics(jtcFont);
    }
  }

  private void updateInstes() {
    Insets ins = getInsets();
    Insets jtcins = jtc.getInsets();
    topInsets = ins.top + jtcins.top;
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-23 08:49:48 Slobodan
   Initial version

*****************************************************************************^*/
