/*
 * @(#)SearchSegment.java  1.0.0 / 2008-05-23
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.swing.components.text;

import javax.swing.text.Segment;

/**
 * Search Segment.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0
 */
public class SearchSegment extends Segment {

/*^**************************************************************************^*/
/*- Private Fields.                                                          -*/
/*^**************************************************************************^*/

  private final boolean up;
  private final boolean ignoreCase;
  private final char[] textChar;

/*^**************************************************************************^*/
/*- Public Constructor.                                                      -*/
/*^**************************************************************************^*/

  /**
   * Constructs new SearchSegment object.
   *
   * @param text the text to search for
   * @param up search direction
   * @param ignoreCase ignore case
   *
   * @throws IllegalArgumentException if text is <tt>null</tt>
   */
  public SearchSegment(String text, boolean up, boolean ignoreCase) {
    super();
    if (text == null) {
      throw new IllegalArgumentException("Null search text"); //$NON-NLS-1$
    }
    setPartialReturn(true);
    this.up = up;
    this.ignoreCase = ignoreCase;
    this.textChar = new char[text.length()];
    text.getChars(0, textChar.length, textChar, 0);
  }

/*^**************************************************************************^*/
/*- Public Methods.                                                          -*/
/*^**************************************************************************^*/

  /**
   * Finds specified word.
   *
   * @return <tt>index</tt> of specified word or
   *         <tt>-1</tt> if word does not exist
   */
  public int find() {
    int index;
    if (up) {
      if (ignoreCase) {
        index = lastIndexOfIgnoreCase(array, offset, count, textChar, 0,
                                      textChar.length, count);
      } else {
        index = lastIndexOf(array, offset, count, textChar, 0, textChar.length,
                            count);
      }
    } else {
      if (ignoreCase) {
        index = indexOfIgnoreCase(array, offset, count, textChar, 0,
                                  textChar.length, 0);
      } else {
        index = indexOf(array, offset, count, textChar, 0, textChar.length, 0);
      }
    }
    if (index < 0) {
      return index;
    }
    return offset + index;
  }

  /**
   * Finds whole words only.
   *
   * @return <tt>index</tt> of specified word or
   *         <tt>-1</tt> if word does not exist
   */
  public int findWhole() {
    int si = -1;
    while ((si = find()) != -1) {
      int ei = si + textChar.length;
      if (ei <= array.length) {
        if (Character.isLetterOrDigit(array[ei])) {
          if (up) {
            count = si;
          } else {
            offset = ei;
          }
        } else {
          return si;
        }
      } else {
        return -1;
      }
    }
    return si;
  }

/*^**************************************************************************^*/
/*- Protected Static Methods.                                                -*/
/*^**************************************************************************^*/

  /* Copied from java.lang.String */

  private static int indexOf(char[] source, int sourceOffset, // NOPMD
                             int sourceCount, char[] target, int targetOffset,
                             int targetCount, int fromIndex) {
    if (fromIndex >= sourceCount) {
      if (targetCount == 0) {
        return sourceCount;
      }
      return -1;
    }
    if (fromIndex < 0) {
      fromIndex = 0;
    }
    if (targetCount == 0) {
      return fromIndex;
    }
    char first = target[targetOffset];
    int i = sourceOffset + fromIndex;
    int max = sourceOffset + (sourceCount - targetCount);
  startSearchForFirstChar:
    while (true) {
      /* Look for first character. */
      while ((i <= max) && (source[i] != first)) {
        i++;
      }
      if (i > max) {
        return -1;
      }
      /* Found first character, now look at the rest of v2 */
      int j = i + 1;
      int end = j + targetCount - 1;
      int k = targetOffset + 1;
      while (j < end) {
        if (source[j++] != target[k++]) {
          i++;
          /* Look for str's first char again. */
          continue startSearchForFirstChar;
        }
      }
      return i - sourceOffset; /* Found whole string. */
    }
  }

  private static int lastIndexOf(char[] source, int sourceOffset,
                                 int sourceCount,
                                 char[] target, int targetOffset,
                                 int targetCount, int fromIndex) {
    int rightIndex = sourceCount - targetCount;
    if (fromIndex < 0) {
      return -1;
    }
    if (fromIndex > rightIndex) {
      fromIndex = rightIndex;
    }
    /* Empty string always matches. */
    if (targetCount == 0) {
      return fromIndex;
    }
    int strLastIndex = targetOffset + targetCount - 1;
    char strLastChar = target[strLastIndex];
    int min = sourceOffset + targetCount - 1;
    int i = min + fromIndex;
  startSearchForLastChar:
    while (true) {
      while ((i >= min) && (source[i] != strLastChar)) {
        i--;
      }
      if (i < min) {
        return -1;
      }
      int j = i - 1;
      int start = j - (targetCount - 1);
      int k = strLastIndex - 1;
      while (j > start) {
        if (source[j--] != target[k--]) {
          i--;
          continue startSearchForLastChar;
        }
      }
      return start - sourceOffset + 1;
    }
  }

  private static int indexOfIgnoreCase(char[] source, int sourceOffset, // NOPMD
                                       int sourceCount,
                                       char[] target, int targetOffset,
                                       int targetCount, int fromIndex) {
    if (fromIndex >= sourceCount) {
      if (targetCount == 0) {
        return targetCount;
      }
      return -1;
    }
    if (fromIndex < 0) {
      fromIndex = 0;
    }
    if (targetCount == 0) {
      return fromIndex;
    }
    char first = target[targetOffset];
    int i = sourceOffset + fromIndex;
    int max = sourceOffset + (sourceCount - targetCount);
  startSearchForFirstChar:
    while (true) {
      /* Look for first character. */
      while ((i <= max) && (Character.toLowerCase(source[i]) !=
                            Character.toLowerCase(first))) {
        i++;
      }
      if (i > max) {
        return -1;
      }
      /* Found first character, now look at the rest of v2 */
      int j = i + 1;
      int end = j + targetCount - 1;
      int k = targetOffset + 1;
      while (j < end) {
        if (Character.toLowerCase(source[j++]) !=
            Character.toLowerCase(target[k++])) {
          i++;
          /* Look for str's first char again. */
          continue startSearchForFirstChar;
        }
      }
      return i - sourceOffset; /* Found whole string. */
    }
  }

  private static int lastIndexOfIgnoreCase(char[] source, int sourceOffset,
                                           int sourceCount,
                                           char[] target, int targetOffset,
                                           int targetCount, int fromIndex) {
    int rightIndex = sourceCount - targetCount;
    if (fromIndex < 0) {
      return -1;
    }
    if (fromIndex > rightIndex) {
      fromIndex = rightIndex;
    }
    /* Empty string always matches. */
    if (targetCount == 0) {
      return fromIndex;
    }
    int strLastIndex = targetOffset + targetCount - 1;
    char strLastChar = target[strLastIndex];
    int min = sourceOffset + targetCount - 1;
    int i = min + fromIndex;
  startSearchForLastChar:
    while (true) {
      while ((i >= min) && (Character.toLowerCase(source[i]) !=
                            Character.toLowerCase(strLastChar))) {
        i--;
      }
      if (i < min) {
        return -1;
      }
      int j = i - 1;
      int start = j - (targetCount - 1);
      int k = strLastIndex - 1;
      while (j > start) {
        if (Character.toLowerCase(source[j--]) !=
            Character.toLowerCase(target[k--])) {
          i--;
          continue startSearchForLastChar;
        }
      }
      return start - sourceOffset + 1;
    }
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-23 08:35:06 Slobodan
   Initial version

*****************************************************************************^*/
