/*
 * @(#)AbstractCodeDocument.java  1.0.0 / 2008-05-23
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.swing.components.text.code;

import javax.swing.event.UndoableEditEvent;
import javax.swing.text.AttributeSet;
import javax.swing.text.BadLocationException;
import javax.swing.text.DefaultStyledDocument;
import javax.swing.text.Style;
import javax.swing.text.StyleConstants;

/**
 * Abstract Code Document.
 * (Based on article
 * http://www.ftponline.com/javapro/2003_07/magazine/columns/visualcomponents/
 * Created by Claude Duguay)
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public abstract class AbstractCodeDocument extends DefaultStyledDocument {

/*^**************************************************************************^*/
/*- Protected Fields.                                                        -*/
/*^**************************************************************************^*/

  /**
   * Highlight syntax flag.
   */
  protected boolean syntaxHighlight;

  /**
   * Highlighting flag.
   */
  protected boolean highlighting;

/*^**************************************************************************^*/
/*- Public Constructors.                                                     -*/
/*^**************************************************************************^*/

  /**
   * Constructs new AbstractCodeDocument object.
   *
   * @param syntaxHighlight syntax highlight
   */
  public AbstractCodeDocument(boolean syntaxHighlight) {
    this(syntaxHighlight, "DialogInput", 12); //$NON-NLS-1$
  }

  /**
   * Constructs new AbstractCodeDocument object.
   *
   * @param syntaxHighlight syntax highlight
   * @param fontFamily font size
   * @param fontSize font family
   */
  public AbstractCodeDocument(boolean syntaxHighlight, String fontFamily,
                              int fontSize) {
    super();
    this.syntaxHighlight = syntaxHighlight;
    Style defaultStyle = getStyle("default"); //$NON-NLS-1$
    StyleConstants.setFontFamily(defaultStyle, fontFamily);
    StyleConstants.setFontSize(defaultStyle, fontSize);
  }

/*^**************************************************************************^*/
/*- Public Methods.                                                          -*/
/*^**************************************************************************^*/

  /**
   * Returns <tt>true</tt> if highlighting is enabled.
   *
   * @return <tt>true</tt> if highlighting is enabled
   */
  public boolean getSyntaxHighlight() { // NOPMD
    return syntaxHighlight;
  }

  /**
   * Enables/disables syntax highlighting.
   *
   * @param enable if <tt>true</tt> enable
   */
  public void setSyntaxHighlight(boolean enable) {
    syntaxHighlight = enable;
  }

  /**
   * Highlight this document.
   */
  public void highlight() {
    highlightAll();
  }

/*^**************************************************************************^*/
/*- Overridden Methods.                                                      -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  public void insertString(int offset, String text,
                           AttributeSet style) throws BadLocationException {
    super.insertString(offset, text, style);
    highlightInserted(offset, text, style);
  }

  /**
   * {@inheritDoc}
   */
  public void remove(int offset, int length) throws BadLocationException {
    super.remove(offset, length);
    highlightRemoved(offset, length);
  }

  /**
   * {@inheritDoc}
   */
  protected void fireUndoableEditUpdate(UndoableEditEvent e) {
    if (!highlighting) {
      super.fireUndoableEditUpdate(e);
    }
  }

/*^**************************************************************************^*/
/*- Protected Methods.                                                       -*/
/*^**************************************************************************^*/

  /**
   * Highlight inserted text.
   *
   * @param offset the offset
   * @param text the text
   * @param style the style
   */
  protected void highlightInserted(int offset, String text,
                                   AttributeSet style) {
    if (syntaxHighlight) {
      highlightAll();
    }
  }

  /**
   * Highlight removed segment.
   *
   * @param offset the offset
   * @param length the length
   */
  protected void highlightRemoved(int offset, int length) {
    if (syntaxHighlight) {
      highlightAll();
    }
  }

  /**
   * Highlights all text.
   */
  protected void highlightAll() { // NOPMD
    // empty method body
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-23 23:57:33 Slobodan
   Initial version

*****************************************************************************^*/
