/*
 * @(#)KeywordList.java  1.0.0 / 2008-05-23
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.swing.components.text.code;

import java.awt.Color;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.HashSet;
import java.util.Set;

/**
 * Keyword List.
 * (Based on article
 * http://www.ftponline.com/javapro/2003_07/magazine/columns/visualcomponents/
 * Created by Claude Duguay)
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public final class KeywordList {

/*^**************************************************************************^*/
/*- Private Fields.                                                          -*/
/*^**************************************************************************^*/

  private String name;
  private Color foreground;
  private Color background;
  private boolean bold;
  private boolean italic;
  private Set wordList;

/*^**************************************************************************^*/
/*- Public Constructors.                                                     -*/
/*^**************************************************************************^*/

  /**
   * Constructs new KeywordList object.
   *
   * @param name list name
   */
  public KeywordList(String name) {
    this(name, new HashSet(), Color.WHITE, Color.BLACK, false, false);
  }

  /**
   * Constructs new KeywordList object.
   *
   * @param is keyword input stream
   *
   * @throws IOException if failed
   */
  public KeywordList(InputStream is) throws IOException {
    this(null, new HashSet(), null, null, false, false);
    load(is);
  }

  /**
   * Constructs new KeywordList object.
   *
   * @param name list name
   * @param background list's background color
   * @param foreground list's foreground color
   */
  public KeywordList(String name, Color foreground, Color background) {
    this(name, new HashSet(), background, foreground, false, false);
  }

  /**
   * Constructs new KeywordList object.
   *
   * @param name list name
   * @param background list's background color
   * @param foreground list's foreground color
   * @param bold list's bold state
   * @param italic list's italic state
   */
  public KeywordList(String name, Color background, Color foreground,
                     boolean bold, boolean italic) {
    this(name, new HashSet(), background, foreground, bold, italic);
  }

  /**
   * Constructs new KeywordList object.
   *
   * @param name list name
   * @param wordList list's words
   * @param background list's background color
   * @param foreground list's foreground color
   * @param bold list's bold state
   * @param italic list's italic state
   */
  public KeywordList(String name, Set wordList, Color background,
                     Color foreground, boolean bold, boolean italic) {
    this.name = name;
    this.wordList = wordList;
    this.foreground = foreground;
    this.background = background;
    this.bold = bold;
    this.italic = italic;
  }

/*^**************************************************************************^*/
/*- Public Methods.                                                          -*/
/*^**************************************************************************^*/

  /**
   * Returns list name.
   *
   * @return list name
   */
  public String getName() {
    return name;
  }

  /**
   * Returns list's foreground color.
   *
   * @return list's foreground color
   */
  public Color getForeground() {
    return foreground;
  }

  /**
   * Returns list's background color.
   *
   * @return list's background color
   */
  public Color getBackground() {
    return background;
  }

  /**
   * Returns list's bold state.
   *
   * @return list's bold state
   */
  public boolean isBold() {
    return bold;
  }

  /**
   * Returns list's italic state.
   *
   * @return list's italic state
   */
  public boolean isItalic() {
    return italic;
  }

  /**
   * Tests it the specified word is the member of this list.
   *
   * @param word the word
   *
   * @return <tt>true</tt> if the word is in this list
   */
  public boolean isMember(String word) {
    return wordList.contains(word);
  }

  /**
   * Loads list from the specified input stream.
   *
   * @param is the InputStream
   *
   * @throws IOException if failed
   */
  public void load(InputStream is) throws IOException {
    BufferedReader reader = null;
    try {
      wordList.clear();
      String line = null;
      reader = new BufferedReader(new InputStreamReader(is));
      name = reader.readLine();
      line = reader.readLine();
      if ((line != null) && (line.length() > 0) &&
          (!line.equals("null"))) { //$NON-NLS-1$
        foreground = Color.decode(line);
      }
      line = reader.readLine();
      if ((line != null) && (line.length() > 0) &&
          (!line.equals("null"))) { //$NON-NLS-1$
        background = Color.decode(line);
      }
      line = reader.readLine();
      bold = Boolean.valueOf(line).booleanValue();
      line = reader.readLine();
      italic = Boolean.valueOf(line).booleanValue();
      while ((line = reader.readLine()) != null) {
        if (line.length() != 0) {
          wordList.add(line);
        }
      }
    } catch (Throwable t) { // NOPMD
      throw new IOException(t.getMessage()); // NOPMD
    } finally {
      if (reader != null) {
        try {
          reader.close();
        } catch (IOException e) { // NOPMD
          // empty catch block
        }
      }
    }
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-23 23:52:41 Slobodan
   Initial version

*****************************************************************************^*/
