/*
 * @(#)KeywordManager.java  1.0.0 / 2008-05-23
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.swing.components.text.code;

import java.awt.Color;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;

import javax.swing.text.Style;
import javax.swing.text.StyleConstants;
import javax.swing.text.StyledDocument;

/**
 * Keyword manager.
 * (Based on article
 * http://www.ftponline.com/javapro/2003_07/magazine/columns/visualcomponents/
 * Created by Claude Duguay)
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public final class KeywordManager {

/*^**************************************************************************^*/
/*- Private Field.                                                           -*/
/*^**************************************************************************^*/

  private final ArrayList keywords;

/*^**************************************************************************^*/
/*- Public Constructors.                                                     -*/
/*^**************************************************************************^*/

  /**
   * Constructs new KeywordManager object.
   */
  public KeywordManager() {
    keywords = new ArrayList();
  }

  /**
   * Constructs new KeywordManager object.
   *
   * @param listCount list count
   */
  public KeywordManager(int listCount) {
    keywords = new ArrayList(listCount);
  }

/*^**************************************************************************^*/
/*- Public Methods.                                                          -*/
/*^**************************************************************************^*/

  /**
   * Adds specified keyword list.
   *
   * @param kl the KeywordList
   */
  public void addKeywordList(KeywordList kl) {
    if (kl == null) {
      throw new IllegalArgumentException("Null KeywordList"); //$NON-NLS-1$
    }
    String kln = kl.getName();
    if (kln == null) {
      throw new IllegalArgumentException("Null KeywordList Name"); //$NON-NLS-1$
    }
    ArrayList kw = keywords;
    for (int i = kw.size() - 1; i >= 0; i--) {
      KeywordList keys = (KeywordList)kw.get(i);
      if (kln.equals(keys.getName())) {
        return;
      }
    }
    kw.add(kl);
  }

  /**
   * Adds keyword list from stream.
   *
   * @param is keyword stream
   *
   * @throws IOException if failed
   */
  public void addKeywordList(InputStream is) throws IOException {
    addKeywordList(new KeywordList(is));
  }

  /**
   * Removes specified list.
   *
   * @param kl the KeywordList
   */
  public void removeKeywordList(KeywordList kl) {
    keywords.remove(kl);
  }

  /**
   * Clears this list.
   */
  public void clear() {
    keywords.clear();
  }

  /**
   * Loads keyword lists from the specified file list.
   *
   * @param files keyword files
   */
  public void loadKeywordFiles(File[] files) {
    for (int i = files.length - 1; i >= 0; i--) {
      String filename = files[i].getName();
      if (filename.endsWith(".keywords")) { //$NON-NLS-1$
        try {
          keywords.add(new KeywordList(new FileInputStream(files[i])));
        } catch (IOException ioe) { // NOPMD
          // empty catch block
        }
      }
    }
  }

  /**
   * Sets styles for the specified document.
   *
   * @param doc the StyledDocument
   */
  public void setStyles(StyledDocument doc) {
    ArrayList kw = keywords;
    for (int i = kw.size() - 1; i >= 0; i--) {
      KeywordList keys = (KeywordList)kw.get(i);
      Style style = doc.addStyle(keys.getName(), null);
      Color c = keys.getBackground();
      if (c != null) {
        StyleConstants.setBackground(style, c);
      }
      c = keys.getForeground();
      if (c != null) {
        StyleConstants.setForeground(style, c);
      }
      StyleConstants.setBold(style, keys.isBold());
      StyleConstants.setItalic(style, keys.isItalic());
    }
  }

  /**
   * Returns style name for the specified word.
   *
   * @param word the word
   *
   * @return style name for the specified word
   */
  public String getStyleName(String word) {
    ArrayList kw = keywords;
    for (int i = kw.size() - 1; i >= 0; i--) {
      KeywordList keys = (KeywordList)kw.get(i);
      if (keys.isMember(word)) {
        return keys.getName();
      }
    }
    return null;
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-23 23:50:40 Slobodan
   Initial version

*****************************************************************************^*/
