/*
 * @(#)RegExTypes.java  1.0.0 / 2008-05-23
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.swing.components.text.code;

import java.awt.Color;
import java.util.ArrayList;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.swing.text.Style;
import javax.swing.text.StyleConstants;
import javax.swing.text.StyledDocument;

/**
 * RegExTypes.
 * (Based on article
 * http://www.ftponline.com/javapro/2003_07/magazine/columns/visualcomponents/
 * Created by Claude Duguay)
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public class RegExTypes {

/*^**************************************************************************^*/
/*- Protected Fields.                                                        -*/
/*^**************************************************************************^*/

  /**
   * The type list.
   */
  protected ArrayList typeList;

  /**
   * The pattern.
   */
  protected Pattern pattern;

  /**
   * The expression.
   */
  protected String expression;

/*^**************************************************************************^*/
/*- Public Constructor.                                                      -*/
/*^**************************************************************************^*/

  /**
   * Constructs new RegExTypes object.
   */
  public RegExTypes() {
    typeList = new ArrayList();
  }

/*^**************************************************************************^*/
/*- Public Methods.                                                          -*/
/*^**************************************************************************^*/

  /**
   * Adds token type.
   *
   * @param name the name
   * @param expr the expression
   * @param foreground the foreground
   */
  public void addTokenType(String name, String expr, Color foreground) {
    addTokenType(name, expr, foreground, null, false, false);
  }

  /**
   * Adds token type.
   *
   * @param name the name
   * @param expr the expression
   * @param foreground the foreground
   * @param background the background
   */
  public void addTokenType(String name, String expr, Color foreground,
                           Color background) {
    addTokenType(name, expr, foreground, background, false, false);
  }

  /**
   * Adds token type.
   *
   * @param name the name
   * @param expr the expression
   * @param foreground the foreground
   * @param background the background
   * @param bold the bold
   * @param italic the italic
   */
  public void addTokenType(String name, String expr, Color foreground,
                           Color background, boolean bold, boolean italic) {
    typeList.add(new Type(name, expr, foreground, background, bold, italic));
    pattern = null;
    expression = null;
  }

  /**
   * Returns type count.
   *
   * @return type count
   */
  public int getTypeCount() {
    return typeList.size();
  }

  /**
   * Returns type name.
   *
   * @param index type index
   *
   * @return type name
   */
  public String getTypeName(int index) {
    return getType(index).name;
  }

  /**
   * Returns type expression.
   *
   * @param index type index
   *
   * @return type expression
   */
  public String getTypeExpression(int index) {
    return getType(index).expr;
  }

  /**
   * Returns type foreground color.
   *
   * @param index type index
   *
   * @return type foreground color
   */
  public Color getTypeForeground(int index) {
    return getType(index).foreground;
  }

  /**
   * Returns type background color.
   *
   * @param index type index
   *
   * @return type background color
   */
  public Color getTypeBackground(int index) {
    return getType(index).background;
  }

  /**
   * Returns bold state.
   *
   * @param index type index
   *
   * @return bold state
   */
  public boolean isTypeBold(int index) {
    return getType(index).bold;
  }

  /**
   * Returns italic state.
   *
   * @param index type index
   *
   * @return italic state
   */
  public boolean isTypeItalic(int index) {
    return getType(index).italic;
  }

  /**
   * Sets styles for the specified document.
   *
   * @param doc the StyledDocument
   */
  public void setStyles(StyledDocument doc) {
    ArrayList tl = typeList;
    for (int i = tl.size() - 1; i >= 0; i--) {
      Type t = (Type)tl.get(i);
      Style style = doc.addStyle(t.name, null);
      if (t.foreground != null) {
        StyleConstants.setForeground(style, t.foreground);
      }
      if (t.background != null) {
        StyleConstants.setBackground(style, t.background);
      }
      StyleConstants.setBold(style, t.bold);
      StyleConstants.setItalic(style, t.italic);
    }
    KeywordManager km = getKeywordManager();
    if (km != null) {
      km.setStyles(doc);
    }
  }

  /**
   * Returns expression.
   *
   * @return expression
   */
  public String getExpression() {
    if (expression == null) {
      StringBuffer buffer = new StringBuffer();
      ArrayList tl = typeList;
      int tls = tl.size();
      for (int i = 0; i < tls; i++) {
        Type t = (Type)tl.get(i);
        if (i > 0) {
          buffer.append('|');
        }
        buffer.append('(');
        buffer.append(t.expr);
        buffer.append(')');
      }
      expression = buffer.toString();
    }
    return expression;
  }

  /**
   * Returns pattern.
   *
   * @return Pattern
   */
  public Pattern getPattern() {
    if (pattern == null) {
      pattern = Pattern.compile(getExpression(), Pattern.DOTALL);
    }
    return pattern;
  }

  /**
   * Returns Matcher.
   *
   * @param text the text
   *
   * @return Matcher
   */
  public Matcher getMatcher(CharSequence text) {
    return getPattern().matcher(text);
  }

  /**
   * Returns style name for the specified word.
   *
   * @param word the word
   *
   * @return style name for the specified word
   */
  public String getStyleName(String word) {
    KeywordManager km = getKeywordManager();
    if (km != null) {
      return km.getStyleName(word);
    }
    return null;
  }

/*^**************************************************************************^*/
/*- Protected Methods.                                                       -*/
/*^**************************************************************************^*/

  /**
   * Returns type.
   *
   * @param index the type index
   *
   * @return type
   */
  protected Type getType(int index) {
    return (Type)typeList.get(index);
  }

  /**
   * Returns keyword manager.
   *
   * @return keyword manager
   */
  protected KeywordManager getKeywordManager() {
    return null;
  }

/*^**************************************************************************^*/
/*- Inner Class.                                                             -*/
/*^**************************************************************************^*/

  /**
   * Type class.
   */
  protected static final class Type {

    // Package-Private Fields. -------------------------------------------------

    String name;
    String expr;
    Color background;
    Color foreground;
    boolean bold;
    boolean italic;

    // Public Constructors. ----------------------------------------------------

    /**
     * Constructs new Type object.
     *
     * @param name the name
     * @param expr the regular expression
     * @param foreground the foreground color
     */
    public Type(String name, String expr, Color foreground) {
      this(name, expr, null, foreground, false, false);
    }

    /**
     * Constructs new Type object.
     *
     * @param name the name
     * @param expr the regular expression
     * @param foreground the foreground
     * @param background the background
     * @param bold the bold
     * @param italic the italic
     */
    public Type(String name, String expr, Color foreground, Color background,
                boolean bold, boolean italic) {
      this.name = name;
      this.expr = expr;
      this.background = background;
      this.foreground = foreground;
      this.bold = bold;
      this.italic = italic;
    }

  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-23 23:37:39 Slobodan
   Initial version

*****************************************************************************^*/
