/*
 * @(#)FastGradientPaintContext.java  1.0.0 / 2008-05-17
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 *
 * The original code is developed by
 * Sebastian Ferreyra (sebastianf@citycolor.net).
 * It is published under the terms of the GNU Lesser General Public License.
 */
package net.wrka.swing.graphics;

import java.awt.PaintContext;
import java.awt.Rectangle;
import java.awt.image.ColorModel;
import java.awt.image.Raster;
import java.awt.image.WritableRaster;
import java.lang.ref.WeakReference;
import java.util.HashMap;
import java.util.WeakHashMap;

/**
 * Fast Gradient PaintContext.
 *
 * @author Sebastian Ferreyra (sebastianf@citycolor.net)
 * @version original
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
final class FastGradientPaintContext implements PaintContext {

/*^**************************************************************************^*/
/*- Private Static Fields.                                                   -*/
/*^**************************************************************************^*/

  private static final ColorModel RGB_DEFAULT;
  private static final WeakHashMap GRADIENT_CACHE;
  private static final GradientInfo[] RECENT_INFOS; // NOPMD
  private static int riIndex; // NOPMD

/*^**************************************************************************^*/
/*- Static Initialization.                                                   -*/
/*^**************************************************************************^*/

  static {
    GRADIENT_CACHE = new WeakHashMap();
    RECENT_INFOS = new GradientInfo[16];
    RGB_DEFAULT = ColorModel.getRGBdefault();
  }

/*^**************************************************************************^*/
/*- Private Fields.                                                          -*/
/*^**************************************************************************^*/

  private final GradientInfo info;
  private int parallelDevicePos;
  private Gradient gradient;

/*^**************************************************************************^*/
/*- Public Constructor.                                                      -*/
/*^**************************************************************************^*/

  /**
   * Constructs new FastGradientPaintContext object.
   *
   * @param cm the code model
   * @param r the rectangle
   * @param sc
   * @param ec
   * @param ver
   */
  public FastGradientPaintContext(ColorModel cm, Rectangle r, int sc, int ec,
                                  boolean ver) {
    info = new GradientInfo();
    if ((((sc & ec) >> 24) & 0xFF) == 0xFF) {
      info.model = cm;
    } else {
      info.model = RGB_DEFAULT;
    }
    info.startColor = sc;
    info.endColor = ec;
    info.isVertical = ver;
    if (ver) {
      parallelDevicePos = r.y;
      info.parallelLength = r.height;
    } else {
      parallelDevicePos = r.x;
      info.parallelLength = r.width;
    }
    RECENT_INFOS[riIndex] = info;
    riIndex = (riIndex + 1) % 16;
    Object o = GRADIENT_CACHE.get(info);
    if (o != null) {
      o = ((WeakReference)o).get();
    }
    if (o == null) {
      gradient = new Gradient(info);
      GRADIENT_CACHE.put(info, new WeakReference(gradient));
    } else {
      gradient = (Gradient)o;
    }
  }

/*^**************************************************************************^*/
/*- Overridden Methods.                                                      -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  public void dispose() {
    //gradient.dispose();
  }

  /**
   * {@inheritDoc}
   */
  public ColorModel getColorModel() {
    return info.model;
  }

  /**
   * {@inheritDoc}
   */
  public Raster getRaster(int x, int y, int w, int h) {
    synchronized (info) {
      if (info.isVertical) {
        return gradient.getRaster(y - parallelDevicePos, w, h);
      }
      return gradient.getRaster(x - parallelDevicePos, h, w);
    }
  }

/*^**************************************************************************^*/
/*- Inner Classes.                                                           -*/
/*^**************************************************************************^*/

  /**
   * GradientInfo.
   */
  static final class GradientInfo {
    ColorModel model;
    int parallelLength;
    int startColor;
    int endColor;
    boolean isVertical;
    /**
     * {@inheritDoc}
     */
    public boolean equals(Object o) {
      if (o instanceof GradientInfo) {
        GradientInfo gi = (GradientInfo)o;
        return (gi.model.equals(model) &&
                (gi.parallelLength == parallelLength) &&
                (gi.startColor == startColor) && (gi.endColor == endColor) &&
                (gi.isVertical == isVertical));
      }
      return false;
    }
    /**
     * {@inheritDoc}
     */
    public int hashCode() {
      return parallelLength;
    }
    /**
     * {@inheritDoc}
     */
    public String toString() {
      StringBuffer sb = new StringBuffer(43);
      sb.append("Direction:"); //$NON-NLS-1$
      if (isVertical) {
        sb.append("ver"); //$NON-NLS-1$
      } else {
        sb.append("hor"); //$NON-NLS-1$
      }
      sb.append(", Length: "); //$NON-NLS-1$
      sb.append(Integer.toString(parallelLength));
      sb.append(", Color1: "); //$NON-NLS-1$
      sb.append(Integer.toString(startColor, 16));
      sb.append(", Color2: "); //$NON-NLS-1$
      sb.append(Integer.toString(endColor, 16));
      return sb.toString();
    }
  }

  /**
   * Gradient Class.
   */
  static final class Gradient {
    GradientInfo info;
    int perpendicularLength = 0;
    WritableRaster raster;
    HashMap childRasterCache;
    /**
     * Constructs new Gradient object.
     *
     * @param i the gradient info
     */
    public Gradient(GradientInfo i) {
      info = i;
    }
    /**
     * REturns raster.
     *
     * @param parallelPos
     * @param perpendicularLength
     * @param parallelLength
     * @return raster
     */
    public Raster getRaster(int parallelPos, int perpendicularLength,
                            int parallelLength) {
      if ((raster == null) ||
          (this.perpendicularLength < perpendicularLength)) {
        createRaster(perpendicularLength);
      }
      Integer key = new Integer(parallelPos);
      Object o = childRasterCache.get(key);
      if (o != null) {
        return (Raster)o;
      }
      Raster r;
      if (info.isVertical) {
        r = raster.createChild(0, parallelPos, this.perpendicularLength,
                               info.parallelLength - parallelPos, 0, 0, null);
      } else {
        r = raster.createChild(parallelPos, 0,
                               info.parallelLength - parallelPos,
                               this.perpendicularLength, 0, 0, null);
      }
      childRasterCache.put(key, r);
      return r;
    }
    /**
     * Disposed the gradient.
     */
    public void dispose() {
      raster = null;
    }
    private void createRaster(int perpendicularLength) {
      int gradientWidth;
      int gradientHeight;
      if (info.isVertical) {
        gradientHeight = info.parallelLength;
        this.perpendicularLength = perpendicularLength;
        gradientWidth = perpendicularLength;
      } else {
        gradientWidth = info.parallelLength;
        this.perpendicularLength = perpendicularLength;
        gradientHeight = perpendicularLength;
      }
      int sa = (info.startColor >> 24) & 0xFF;
      int sr = (info.startColor >> 16) & 0xFF;
      int sg = (info.startColor >> 8) & 0xFF;
      int sb = info.startColor & 0xFF;
      int da = ((info.endColor >> 24) & 0xFF) - sa;
      int dr = ((info.endColor >> 16) & 0xFF) - sr;
      int dg = ((info.endColor >> 8) & 0xFF) - sg;
      int db = (info.endColor & 0xFF) - sb;
      raster = info.model.createCompatibleWritableRaster(gradientWidth,
                                                         gradientHeight);
      Object c = null;
      int pl = info.parallelLength;
      for (int i = pl - 1; i >= 0; i--) {
        c = info.model.getDataElements((sa + (i * da) / pl) << 24 |
                                       (sr + (i * dr) / pl) << 16 |
                                       (sg + (i * dg) / pl) << 8 |
                                       (sb + (i * db) / pl), c);
        if (info.isVertical) {
          for (int j = perpendicularLength - 1; j >= 0; j--) {
            raster.setDataElements(j, i, c);
          }
        } else {
          for (int j = perpendicularLength - 1; j >= 0; j--) {
            raster.setDataElements(i, j, c);
          }
        }
      }
      childRasterCache = new HashMap();
    }
  }

}

/*^*****************************************************************************
  File History:

 - Initial version Sebastian Ferreyra

 - 2008-05-17 22:54:11 Slobodan
   Initial version

*****************************************************************************^*/
