/*
 * @(#)FontUtils.java  1.0.0 / 2008-05-17
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.swing.graphics;

import java.awt.Font;
import java.awt.FontFormatException;
import java.awt.GraphicsEnvironment;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;

import net.wrka.LibraryConfig;
import net.wrka.swing.resources.GUIResourceManager;
import net.wrka.util.SimpleStringMap;

/**
 * Font Utilities.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public final class FontUtils {

/*^**************************************************************************^*/
/*- Public Static Fields.                                                    -*/
/*^**************************************************************************^*/

  /**
   * True when the class is initialized.
   */
  public static final boolean INITIALIZED;

  /**
   * Default Font.
   */
  public static final Font DEFAULT_FONT;

  /**
   * Default Fixed Font.
   */
  public static final Font DEFAULT_FIXED_FONT;

/*^**************************************************************************^*/
/*- Private Static Fields.                                                   -*/
/*^**************************************************************************^*/

  private static final GraphicsEnvironment GE;

/*^**************************************************************************^*/
/*- Static Initialization.                                                   -*/
/*^**************************************************************************^*/

  static {
    GE = GraphicsEnvironment.getLocalGraphicsEnvironment();
    String defFontName = "Tahoma"; //$NON-NLS-1$
    String defFixedFontName = "Courier New"; //$NON-NLS-1$
    String[] fonts = getAvailableFontFamilyNames();
    boolean defaultFound = false;
    boolean defaultFixedFound = false;
    for (int i = fonts.length - 1; i >= 0; i--) {
      String fi = fonts[i];
      if (!defaultFound && defFontName.equals(fi)) {
        defaultFound = true;
      }
      if (!defaultFixedFound && defFixedFontName.equals(fi)) {
        defaultFixedFound = true;
      }
      if (defaultFound && defaultFixedFound) {
        break;
      }
    }
    if (defaultFound) {
      DEFAULT_FONT = new Font(defFontName, Font.PLAIN, 11);
    } else {
      DEFAULT_FONT = new Font("Dialog", Font.PLAIN, 11); //$NON-NLS-1$
    }
    if (defaultFixedFound) {
      DEFAULT_FIXED_FONT = new Font(defFixedFontName, Font.PLAIN, 12);
    } else {
      DEFAULT_FIXED_FONT = new Font("DialogInput", //$NON-NLS-1$
                                    Font.PLAIN, 11);
    }
    if (LibraryConfig.VERBOSE) {
      // CHANGE: use logger
      System.out.print("Default font: "); //$NON-NLS-1$ NOPMD
      System.out.println(DEFAULT_FONT); // NOPMD
      System.out.print("Default fixed font: "); //$NON-NLS-1$ NOPMD
      System.out.println(DEFAULT_FIXED_FONT); // NOPMD
    }
    INITIALIZED = true;
  }

/*^**************************************************************************^*/
/*- Private Constructor.                                                     -*/
/*^**************************************************************************^*/

  /**
   * Class has no instances.
   */
  private FontUtils() {
    // empty constructor body
  }

/*^**************************************************************************^*/
/*- Public Static Methods.                                                   -*/
/*^**************************************************************************^*/

  /**
   * Checks and installs fonts.
   *
   * @param fontNames required font names
   * @param guiResourceManager resource manager
   */
  public static void checkAndInstallFonts(String[] fontNames,
                                       GUIResourceManager guiResourceManager) {
    SimpleStringMap availableFonts = new SimpleStringMap(
                                                getAvailableFontFamilyNames());
    for (int i = fontNames.length - 1; i >= 0; i--) {
      String fi = fontNames[i];
      if (!availableFonts.containsKey(fi)) {
        try {
          installTTFont(guiResourceManager.getFontInputStream(
                                             fi.concat(".ttf"))); //$NON-NLS-1$
        } catch (Throwable t) { // NOPMD
          // CHANGE: use logger
          System.out.print("Could not install font: "); //$NON-NLS-1$ NOPMD
          System.out.println(fi); // NOPMD
        }
      }
    }
  }

  /**
   * Returns all available fonts.
   *
   * @return font array that contains all available fonts
   */
  public static Font[] getAllFonts() {
    return GE.getAllFonts();
  }

  /**
   * Returns all available font family names.
   *
   * @return string array that contains all available font family names
   */
  public static String[] getAvailableFontFamilyNames() {
    return GE.getAvailableFontFamilyNames();
  }

  /**
   * Checks if the specified font is installed.
   *
   * @param name font name
   *
   * @return <code>true</code> if specified font is installed
   */
  public static boolean fontInstalled(String name) {
    String[] fonts = getAvailableFontFamilyNames();
    for (int i = fonts.length - 1; i >= 0; i--) {
      if (name.equalsIgnoreCase(fonts[i])) {
        return true;
      }
    }
    return false;
  }

  /**
   * Installs true type font from the specified font input stream.
   *
   * @param fontInputStream the font input stream
   *
   * @return <tt>true</tt> if succeed
   *
   * @throws IllegalArgumentException if fontInputStream is null
   * @throws IOException if an I/O error occurs
   * @throws FontFormatException if font format is invalid failed
   */
  public static boolean installTTFont(InputStream fontInputStream) throws
                                             IOException, FontFormatException {
    if (fontInputStream == null) {
      throw new IllegalArgumentException("Null font stream"); //$NON-NLS-1$
    }
    Font f = null;
    try {
      f = Font.createFont(Font.TRUETYPE_FONT, fontInputStream);
    } finally {
      fontInputStream.close();
    }
    return (f != null);
  }

  /**
   * Installs true type font from the specified font file.
   *
   * @param fontFile the font file
   *
   * @return <tt>true</tt> if succeed
   *
   * @throws IllegalArgumentException if font is null or not exists
   * @throws IOException if an I/O error occurs
   * @throws FontFormatException if font format is invalid failed
   */
  public static boolean installTTFont(File fontFile) throws IOException,
                                                       FontFormatException {
    if (fontFile == null) {
      throw new IllegalArgumentException("Null font file"); //$NON-NLS-1$
    }
    if (!fontFile.exists()) {
      throw new IllegalArgumentException("Font file not exists"); //$NON-NLS-1$
    }
    FileInputStream fis = null;
    Font f = null;
    try {
      fis = new FileInputStream(fontFile);
      f = Font.createFont(Font.TRUETYPE_FONT, fis);
    } finally {
      if (fis != null) {
        fis.close();
      }
    }
    return (f != null);
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-17 22:42:00 Slobodan
   Initial version

*****************************************************************************^*/
