/*
 * @(#)GraphicUtils.java  1.0.0 / 2008-05-17
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.swing.graphics;

import java.awt.AlphaComposite;
import java.awt.Color;
import java.awt.Composite;
import java.awt.FontMetrics;
import java.awt.GradientPaint;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Paint;
import java.awt.Rectangle;
import java.awt.Shape;
import java.awt.geom.Point2D;

import net.wrka.awt.AwtTools;
import net.wrka.swing.plaf.AlphaColorUIResource;

/**
 * Graphic Utilities.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public final class GraphicUtils {

/*^**************************************************************************^*/
/*- Private Constructor.                                                     -*/
/*^**************************************************************************^*/

  /**
   * Class has no instances.
   */
  private GraphicUtils() {
    // empty constructor body
  }

/*^**************************************************************************^*/
/*- Public Static Methods.                                                   -*/
/*^**************************************************************************^*/

  /**
   * Creates brighter color.
   *
   * @param c the base color
   * @param intensity the intensity factor
   * @param alpha the transparency
   *
   * @return brighter color c
   */
  public static Color brighterColor(Color c, float intensity, int alpha) {
    if (c == null) {
      return null;
    }
    if (intensity < 0) {
      intensity = 0;
    } else if (intensity > 1.0f) {
      intensity = 1.0f;
    }
    int rgb = c.getRGB();
    float[] hsb = Color.RGBtoHSB(((rgb >> 16) & 0xFF), ((rgb >> 8) & 0xFF),
                                 (rgb & 0xFF), null);
    rgb = Color.HSBtoRGB(hsb[0], (hsb[1] - hsb[1] * intensity),
                         (hsb[2] + intensity - intensity * hsb[2]));
    return new Color((((alpha & 0xFF) << 24) | 0x00FFFFFF) & rgb, true);
  }

  /**
   * Creates brighter color.
   *
   * @param c the base color
   * @param intensity the intensity factor
   * @param transparency the transparency
   *
   * @return brighter color c
   */
  public static Color brighterColor(Color c, float intensity,
                                    float transparency) {
    if (transparency < 0f) {
      transparency = 0f;
    } else if (transparency > 1f) {
      transparency = 1f;
    }
    return brighterColor(c, intensity, (int)(255 * transparency));
  }

  /**
   * Creates brighter color.
   *
   * @param c the base color
   * @param intensity the intensity factor
   *
   * @return brighter color c
   */
  public static Color brighterColor(Color c, float intensity) {
    return brighterColor(c, intensity, c.getAlpha());
  }

  /**
   * Creates brighter ColorUIResource.
   *
   * @param c the base color
   * @param intensity the intensity factor
   * @param alpha the transparency
   *
   * @return brighter color c
   */
  public static AlphaColorUIResource brighterColorUIResource(Color c,
                                                             float intensity,
                                                             int alpha) {
    if (c == null) {
      return null;
    }
    if (intensity < 0) {
      intensity = 0;
    } else if (intensity > 1.0f) {
      intensity = 1.0f;
    }
    int rgb = c.getRGB();
    float[] hsb = Color.RGBtoHSB(((rgb >> 16) & 0xFF), ((rgb >> 8) & 0xFF),
                                 (rgb & 0xFF), null);
    rgb = Color.HSBtoRGB(hsb[0], (hsb[1] - hsb[1] * intensity),
                         (hsb[2] + intensity - intensity * hsb[2]));
    return new AlphaColorUIResource((((alpha & 0xFF) << 24) | 0x00FFFFFF) & rgb,
                                    true);
  }

  /**
   * Creates brighter ColorUIResource.
   *
   * @param c the base color
   * @param intensity the intensity factor
   * @param transparency the transparency
   *
   * @return brighter color c
   */
  public static AlphaColorUIResource brighterColorUIResource(
                                Color c, float intensity, float transparency) {
    if (transparency < 0f) {
      transparency = 0f;
    } else if (transparency > 1f) {
      transparency = 1f;
    }
    return brighterColorUIResource(c, intensity, (int)(255 * transparency));
  }

  /**
   * Creates brighter ColorUIResource.
   *
   * @param c the base color
   * @param intensity the intensity factor
   *
   * @return brighter color c
   */
  public static AlphaColorUIResource brighterColorUIResource(Color c,
                                                             float intensity) {
    return brighterColorUIResource(c, intensity, c.getAlpha());
  }

  /**
   * Creates darker color.
   *
   * @param c the base color
   * @param intensity the intensity factor
   * @param alpha the transparency
   *
   * @return darker color c
   */
  public static Color darkerColor(Color c, float intensity, int alpha) {
    if (c == null) {
      return null;
    }
    if (intensity < 0) {
      intensity = 0;
    } else if (intensity > 1.0f) {
      intensity = 1.0f;
    }
    int rgb = c.getRGB();
    float[] hsb = Color.RGBtoHSB(((rgb >> 16) & 0xFF), ((rgb >> 8) & 0xFF),
                                 (rgb & 0xFF), null);
    rgb = Color.HSBtoRGB(hsb[0], (hsb[1] - hsb[1] * intensity),
                         (hsb[2] - intensity * hsb[2]));
    return new Color((((alpha & 0xFF) << 24) | 0x00FFFFFF) & rgb, true);
  }

  /**
   * Creates darker color.
   *
   * @param c the base color
   * @param intensity the intensity factor
   * @param transparency the transparency
   *
   * @return darker color c
   */
  public static Color darkerColor(Color c, float intensity,
                                  float transparency) {
    if (transparency < 0f) {
      transparency = 0f;
    } else if (transparency > 1f) {
      transparency = 1f;
    }
    return darkerColor(c, intensity, (int)(255 * transparency));
  }

  /**
   * Creates darker color.
   *
   * @param c the base color
   * @param intensity the intensity factor
   *
   * @return darker color c
   */
  public static Color darkerColor(Color c, float intensity) {
    return darkerColor(c, intensity, c.getAlpha());
  }

  /**
   * Creates darker ColorUIResource.
   *
   * @param c the base color
   * @param intensity the intensity factor
   * @param alpha the transparency
   *
   * @return darker color c
   */
  public static AlphaColorUIResource darkerColorUIResource(Color c,
                                                           float intensity,
                                                           int alpha) {
    if (c == null) {
      return null;
    }
    if (intensity < 0) {
      intensity = 0;
    } else if (intensity > 1.0f) {
      intensity = 1.0f;
    }
    int rgb = c.getRGB();
    float[] hsb = Color.RGBtoHSB(((rgb >> 16) & 0xFF), ((rgb >> 8) & 0xFF),
                                 (rgb & 0xFF), null);
    rgb = Color.HSBtoRGB(hsb[0], (hsb[1] - hsb[1] * intensity),
                         (hsb[2] - intensity * hsb[2]));
    return new AlphaColorUIResource((((alpha & 0xFF) << 24) | 0x00FFFFFF) & rgb,
                                 true);
  }

  /**
   * Creates darker ColorUIResource.
   *
   * @param c the base color
   * @param intensity the intensity factor
   * @param transparency the transparency
   *
   * @return darker color
   */
  public static AlphaColorUIResource darkerColorUIResource(Color c,
                                                           float intensity,
                                                           float transparency) {
    if (transparency < 0f) {
      transparency = 0f;
    } else if (transparency > 1f) {
      transparency = 1f;
    }
    return darkerColorUIResource(c, intensity, (int)(255 * transparency));
  }

  /**
   * Creates darker ColorUIResource.
   *
   * @param c the base color
   * @param intensity the intensity factor
   *
   * @return darker color c
   */
  public static AlphaColorUIResource darkerColorUIResource(Color c,
                                                           float intensity) {
    return darkerColorUIResource(c, intensity, c.getAlpha());
  }

  /**
   * Returns translucent color.
   *
   * @param color the color
   * @param alpha the alpha intensity
   *
   * @return translucent color
   */
  public static Color getTranslucentColor(Color color, int alpha) {
    if (color == null) {
      return null;
    } else if ((alpha < 0) || (alpha >= 255)) {
      return color;
    } else {
      return new Color(color.getRed(), color.getGreen(), color.getBlue(),
                       alpha);
    }
  }

  /**
   * Returns inverted color.
   *
   * @param color the original color
   *
   * @return inverted color;
   */
  public static Color invertColor(Color color) {
    int rgb = color.getRGB();
    int alpha = rgb & 0xFF000000;
    rgb = ~rgb;
    int rgba = rgb | alpha;
    return new Color(rgba);
  }

  /**
   * Fills shape with linear gradient (color1, p1) -> (color2, p2).
   *
   * @param g the graphics
   * @param shape the shape
   * @param color1 the first color
   * @param color2 the second color
   * @param p1 the first point
   * @param p2 the second point
   * @param alpha the transparency
   */
  public static void fillAlphaGradient(Graphics g, Shape shape,
                                       Point2D p1, Color color1,
                                       Point2D p2, Color color2, float alpha) {
    if (g instanceof Graphics2D) {
      Graphics2D g2 = (Graphics2D)g;
      if ((color1 == null) || (color2 == null) ||
          !AwtTools.isTrueColorModel()) {
        if ((color1 == null) && (color2 == null)) {
          return;
        }
        Color oldcolor = g.getColor();
        if (color1 == null) {
          g2.setColor(color2);
        } else if (color2 != null) {
          g2.setColor(color1);
        }
        g2.fill(shape);
        g2.setColor(oldcolor);
      } else {
        Paint oldpaint = g2.getPaint();
        Composite oldComp = g2.getComposite();
        AlphaComposite ac = AlphaComposite.getInstance(AlphaComposite.SRC_OVER,
                                                       alpha);
        g2.setComposite(ac);
        g2.setPaint(new GradientPaint(p1, color1, p2, color2));
        g2.fill(shape);
        g2.setComposite(oldComp);
        g2.setPaint(oldpaint);
      }
    }
  }

  /**
   * Fills shape with linear gradient (color1, p1) -> (color2, p2).
   *
   * @param g the graphics
   * @param shape the shape
   * @param color1 the first color
   * @param color2 the second color
   * @param p1 the first point
   * @param p2 the second point
   */
  public static void fillGradient(Graphics g, Shape shape,
                                  Point2D p1, Color color1,
                                  Point2D p2, Color color2) {
    fillAlphaGradient(g, shape, p1, color1, p2, color2, 1f);
  }

  /**
   * Draws horizontal gradient.
   *
   * @param g the graphics
   * @param color1 the first color
   * @param color2 the second color
   * @param rect the rectangle
   */
  public static void drawHorizontalGradient(Graphics g, Color color1,
                                            Color color2, Rectangle rect) {
    drawHorizontalGradient(g, color1, color2, rect, 1f);
  }

  /**
   * Draws horizontal gradient.
   *
   * @param g the graphics
   * @param color1 the first color
   * @param color2 the second color
   * @param rect the rectangle
   * @param intensity the gradient intensity
   * @param org the gradient origin
   */
  public static void drawHorizontalGradient(Graphics g, Color color1,
                                            Color color2, Rectangle rect,
                                            float intensity, float org) {
    Point2D.Float p1 = new Point2D.Float(rect.width * org, rect.y);
    Point2D.Float p2 = new Point2D.Float(rect.width * intensity, rect.x);
    fillGradient(g, rect, p1, color1, p2, color2);
  }

  /**
   * Draws horizontal gradient.
   *
   * @param g the graphics
   * @param color1 the first color
   * @param color2 the second color
   * @param rect the rectangle
   * @param intensity the gradient intensity
   */
  public static void drawHorizontalGradient(Graphics g, Color color1,
                                            Color color2, Rectangle rect,
                                            float intensity) {
    Point2D.Float p1 = new Point2D.Float(rect.x, rect.y);
    Point2D.Float p2 = new Point2D.Float(rect.width * intensity, rect.x);
    fillGradient(g, rect, p1, color1, p2, color2);
  }

  /**
   * Draws vertical gradient.
   *
   * @param g the graphics
   * @param color1 the first color
   * @param color2 the second color
   * @param rect the rectangle
   * @param intensity the gradient intensity
   */
  public static void drawVerticalGradient(Graphics g, Color color1,
                                          Color color2, Rectangle rect,
                                          float intensity) {
    Point2D.Float p1 = new Point2D.Float(rect.x, rect.y);
    Point2D.Float p2 = new Point2D.Float(rect.x, rect.height * intensity);
    fillGradient(g, rect, p1, color1, p2, color2);
  }

  /**
   * Draws vertical gradient.
   *
   * @param g the graphics
   * @param color1 the first color
   * @param color2 the second color
   * @param rect the rectangle
   */
  public static void drawVerticalGradient(Graphics g, Color color1,
                                          Color color2, Rectangle rect) {
    drawVerticalGradient(g, color1, color2, rect, 1f);
  }

  /**
   * Draws vertical gradient.
   *
   * @param g the graphics
   * @param color1 the first color
   * @param color2 the second color
   * @param rect the rectangle
   * @param intensity the gradient intensity
   * @param org the gradient origin
   */
  public static void drawVerticalGradient(Graphics g, Color color1,
                                          Color color2, Rectangle rect,
                                          float intensity, float org) {
    Point2D.Float p1 = new Point2D.Float(rect.x, rect.height * org);
    Point2D.Float p2 = new Point2D.Float(rect.x, rect.height * intensity);
    fillGradient(g, rect, p1, color1, p2, color2);
  }

  /**
   * Draws three color vertical gradient.
   *
   * @param g the graphics
   * @param color1 the first color
   * @param color2 the second color
   * @param color3 the third color
   * @param rect the rectangle
   * @param divider gradient divider
   */
  public static void drawVerticalGradient(Graphics g, Color color1,
                                          Color color2, Color color3,
                                          Rectangle rect, float divider) {
    Point2D.Float p1 = new Point2D.Float(rect.x, rect.y);
    Point2D.Float p2 = new Point2D.Float(rect.x, rect.height * divider);
    Point2D.Float p4 = new Point2D.Float(rect.x, rect.height);
    Rectangle r = new Rectangle(rect);
    r.height = (int)((rect.height) * divider);
    fillGradient(g, r, p1, color1, p2, color2);
    r = new Rectangle(rect);
    r.height = (int)((rect.height) * (1 - divider));
    r.y = rect.y + (int)((rect.height) * divider);
    fillGradient(g, r, p2, color2, p4, color3);
  }

  /**
   * Draws diagonal gradient.
   *
   * @param g the graphics
   * @param color1 the first color
   * @param color2 the second color
   * @param rect the rectangle
   */
  public static void drawDiagonalGradient(Graphics g, Color color1,
                                          Color color2, Rectangle rect) {
    Point2D.Float p1 = new Point2D.Float(rect.x, rect.y);
    Point2D.Float p2 = new Point2D.Float(rect.width, rect.height);
    fillGradient(g, rect, p1, color1, p2, color2);
  }

  /**
   * Draws diagonal gradient.
   *
   * @param g the graphics
   * @param color1 the first color
   * @param color2 the second color
   * @param rect the rectangle
   * @param intensity the gradient intensity
   */
  public static void drawDiagonalGradient(Graphics g, Color color1,
                                          Color color2, Rectangle rect,
                                          float intensity) {
    Point2D.Float p1 = new Point2D.Float(rect.x, rect.y);
    Point2D.Float p2 = new Point2D.Float(rect.width * intensity,
                                         rect.height * intensity);
    fillGradient(g, rect, p1, color1, p2, color2);
  }

  /**
   * Draws diagonal gradient.
   *
   * @param g the graphics
   * @param color1 the first color
   * @param color2 the second color
   * @param rect the rectangle
   * @param intensity the gradient intensity
   * @param org the gradient origin
   */
  public static void drawDiagonalGradient(Graphics g, Color color1,
                                          Color color2, Rectangle rect,
                                          float intensity, float org) {
    Point2D.Float p1 = new Point2D.Float(rect.width * org, rect.height * org);
    Point2D.Float p2 = new Point2D.Float(rect.width * intensity,
                                         rect.height * intensity);
    fillGradient(g, rect, p1, color1, p2, color2);
  }

  /**
   * Fast gradient paint.
   *
   * @param g the graphics
   * @param color1 the first color
   * @param color2 the second color
   * @param rect the rectangle
   * @param isVertical <tt>true</tt> if gradient is vertical
   */
  public static void drawFastGradient(Graphics g, Color color1, Color color2,
                                      Rectangle rect, boolean isVertical) {
    Graphics2D g2D = (Graphics2D)g;
    FastGradientPaint gradient = new FastGradientPaint(color1, color2,
                                                       isVertical);
    g2D.setPaint(gradient);
    g2D.fill(rect);
  }

  /**
   * Fast gradient paint.
   *
   * @param g the graphics
   * @param color1 the first color
   * @param color2 the second color
   * @param x the x coordinate
   * @param y the y coordinate
   * @param w the width
   * @param h the height
   * @param isVertical <tt>true</tt> if gradient is vertical
   */
  public static void drawFastGradient(Graphics g, Color color1, Color color2,
                                      int x, int y, int w, int h,
                                      boolean isVertical) {
    Graphics2D g2D = (Graphics2D)g;
    FastGradientPaint gradient = new FastGradientPaint(color1, color2,
                                                       isVertical);
    g2D.setPaint(gradient);
    g2D.fillRect(x, y, w, h);
  }

  /**
   * Fast horizontal gradient.
   *
   * @param g the graphics
   * @param color1 the first color
   * @param color2 the second color
   * @param rect the rectangle
   */
  public static void drawFastHorizontalGradient(Graphics g, Color color1,
                                                Color color2, Rectangle rect) {
    drawFastGradient(g, color1, color2, rect, false);
  }

  /**
   * Fast horizontal gradient.
   *
   * @param g the graphics
   * @param color1 the first color
   * @param color2 the second color
   * @param x the x coordinate
   * @param y the y coordinate
   * @param w the width
   * @param h the height
   */
  public static void drawFastHorizontalGradient(Graphics g, Color color1,
                                            Color color2, int x, int y, int w,
                                            int h) {
    drawFastGradient(g, color1, color2, x, y, w, h, false);
  }

  /**
   * Fast vertical gradient.
   *
   * @param g the graphics
   * @param color1 the first color
   * @param color2 the second color
   * @param rect the rectangle
   */
  public static void drawFastVerticalGradient(Graphics g, Color color1,
                                              Color color2, Rectangle rect) {
    drawFastGradient(g, color1, color2, rect, true);
  }

  /**
   * Fast vertical gradient.
   *
   * @param g the graphics
   * @param color1 the first color
   * @param color2 the second color
   * @param x the x coordinate
   * @param y the y coordinate
   * @param w the width
   * @param h the height
   */
  public static void drawFastVerticalGradient(Graphics g, Color color1,
                                          Color color2, int x, int y, int w,
                                          int h) {
    drawFastGradient(g, color1, color2, x, y, w, h, true);
  }


  /**
   * Draws two colors rectangle.
   *
   * @param g the graphics
   * @param x the rectangle x position
   * @param y the rectangle y position
   * @param w the rectangle width
   * @param h the rectangle height
   * @param topleftColor the top left color of the rectangle
   * @param bottomrightColor the bottom right color of the rectangle
   */
  public static void draw2ColorsRect(Graphics g, int x, int y, int w, int h,
                                     Color topleftColor,
                                     Color bottomrightColor) {
    Color oldColor = g.getColor();
    int wm1 = w - 1;
    int hm1 = h - 1;
    g.setColor(topleftColor);
    g.translate(x, y);
    g.fillRect(0, 0, wm1, 1);
    g.fillRect(0, 0, 1, hm1);
    g.setColor(bottomrightColor);
    g.fillRect(wm1, 0, 1, h);
    g.fillRect(0, hm1, w, 1);
    g.translate(-x, -y);
    g.setColor(oldColor);
  }

  /**
   * Draws oval rectangle.
   *
   * @param g the graphics
   * @param x the rectangle x position
   * @param y the rectangle y position
   * @param w the rectangle width
   * @param h the rectangle height
   */
  public static void draw1PixRoundRect(Graphics g, int x, int y, int w, int h) {
    int xp1 = x + 1;
    int xpw = x + w;
    int wm1 = w - 1;
    int yp1 = y + 1;
    int yph = y + h;
    int hm1 = h - 1;
    g.fillRect(xp1, y, wm1, 1);
    g.fillRect(xpw, yp1, 1, hm1);
    g.fillRect(xp1, yph, wm1, 1);
    g.fillRect(x, yp1, 1, hm1);
  }

  /**
   * Draws oval rectangle.
   *
   * @param g the graphics
   * @param r the rectangle
   */
  public static void draw1PixRoundRect(Graphics g, Rectangle r) {
    draw1PixRoundRect(g, r.x, r.y, r.width, r.height);
  }

  /**
   * Draws dashed rectangle.
   *
   * @param g the graphics
   * @param x the rectangle x position
   * @param y the rectangle y position
   * @param width the rectangle width
   * @param height the rectangle height
   */
  public static void drawDashedRect(Graphics g, int x, int y, int width,
                                    int height) {
    int dx;
    int dy;
    int maxX = x + width - 1;
    int maxY = y + height - 1;
    for (dx = x + 1; dx < maxX; dx += 2) {
      g.fillRect(dx, y, 1, 1);
      g.fillRect(dx, maxY, 1, 1);
    }
    for (dy = y + 1; dy < maxY; dy += 2) {
      g.fillRect(x, dy, 1, 1);
      g.fillRect(maxX, dy, 1, 1);
    }
  }

  /**
   * String drawing.
   *
   * @param g the graphics
   * @param text the text
   * @param underlinedChar the underlined char
   * @param x the text x position
   * @param y the text y position
   */
  public static void drawString(Graphics g, String text, int underlinedChar,
                                int x, int y) {
    int index = -1;
    if (underlinedChar != '\0') {
      index = text.indexOf(Character.toUpperCase((char)underlinedChar));
      if (index == -1) {
        index = text.indexOf(Character.toLowerCase((char)underlinedChar));
      }
    }
    drawStringUnderlineCharAt(g, text, index, x, y);
  }

  /**
   * Draws string underlined at specified position.
   *
   * @param g the graphics
   * @param text the text
   * @param underlinedIndex the underline char index
   * @param x the text x position
   * @param y the text y position
   */
  public static void drawStringUnderlineCharAt(Graphics g, String text,
                                               int underlinedIndex, int x,
                                               int y) {
    g.drawString(text, x, y);
    if ((underlinedIndex >= 0) && (underlinedIndex < text.length())) {
      FontMetrics fm = g.getFontMetrics();
      int underlineX = x + fm.stringWidth(text.substring(0, underlinedIndex));
      int underlineWidth = fm.charWidth(text.charAt(underlinedIndex));
      g.fillRect(underlineX, y + fm.getDescent() - 1, underlineWidth, 1);
    }
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-17 22:38:12 Slobodan
   Initial version

*****************************************************************************^*/
