/*
 * @(#)ImageFactory.java  1.0.0 / 2008-05-17
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.swing.graphics;

import java.awt.Component;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.Toolkit;
import java.awt.image.BufferedImage;
import java.awt.image.FilteredImageSource;
import java.awt.image.ImageProducer;

import javax.swing.GrayFilter;
import javax.swing.Icon;
import javax.swing.ImageIcon;

/**
 * Image Factory.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public final class ImageFactory {

/*^**************************************************************************^*/
/*- Private Constructor.                                                     -*/
/*^**************************************************************************^*/

  /**
   * Class has no instances.
   */
  private ImageFactory() {
    // empty constructor body
  }

/*^**************************************************************************^*/
/*- Public Static Methods.                                                   -*/
/*^**************************************************************************^*/

  /**
   * Returns gray filtered image.
   *
   * @param image the input image
   * @param brighter the brighter or darker mode
   * @param percent the percent of gray
   *
   * @return gray filtered image
   */
  public static Image createGrayFilteredImage(Image image, boolean brighter,
                                              int percent) {
    try {
      GrayFilter filter = new GrayFilter(brighter, percent);
      ImageProducer prod = new FilteredImageSource(image.getSource(), filter);
      return Toolkit.getDefaultToolkit().createImage(prod);
    } catch (Throwable t) { // NOPMD
      return image;
    }
  }

  /**
   * Returns 50% brighter gray filtered image.
   *
   * @param image the input image
   *
   * @return 50% brighter gray filtered image
   */
  public static Image createDisabledImage(Image image) {
    return createGrayFilteredImage(image, true, 50);
  }

  /**
   * Returns 50% brighter gray filtered image.
   *
   * @param icon the input icon
   * @param component the component
   *
   * @return 50% brighter gray filtered image
   */
  public static Image createDisabledImage(Icon icon, Component component) {
    try {
      Image im;
      if (icon instanceof ImageIcon) {
        im = ((ImageIcon)icon).getImage();
      } else {
        im = createImageFromIcon(icon, component);
      }
      return createDisabledImage(im);
    } catch (Throwable t) { // NOPMD
      return null;
    }
  }

  /**
   * Creates disabled ImageIcon using the specified icon.
   *
   * @param icon the input icon
   * @param component the component
   *
   * @return disabled icon
   */
  public static Icon createDisabledIcon(Icon icon, Component component) {
    Image im = createDisabledImage(icon, component);
    if (im != null) {
      return new ImageIcon(im);
    }
    return icon;
  }

  /**
   * Creates shadowed Image using the specified image.
   *
   * @param image the input Image
   *
   * @return image with shadow
   */
  public static Image createShadowedImage(Image image) {
    ImageIcon imageIcon = createShadowedImageIcon(image);
    if (imageIcon != null) {
      return imageIcon.getImage();
    }
    return image;
  }

  /**
   * Creates shadowed ImageIcon using the specified image.
   *
   * @param image the input image
   *
   * @return image with shadow
   */
  public static ImageIcon createShadowedImageIcon(Image image) {
    try {
      ImageIcon ic = new ImageIcon(
          createGrayFilteredImage(createGrayFilteredImage(image, false, 50),
                                  true, 50));
      BufferedImage bi = new BufferedImage(ic.getIconWidth(),
                                           ic.getIconHeight(),
                                           BufferedImage.TYPE_4BYTE_ABGR);
      Graphics2D big = bi.createGraphics();
      big.drawImage(ic.getImage(), 1, 1, null);
      big.drawImage(image, -1, -1, null);
      big.dispose();
      ic.setImage(bi);
      return ic;
    } catch (Throwable t) { // NOPMD
      return null;
    }
  }

  /**
   * Creates shadowed ImageIcon using the specified icon and component.
   *
   * @param icon the input icon
   * @param component the component
   *
   * @return icon with shadow
   */
  public static Icon createShadowedImageIcon(Icon icon, Component component) {
    if (icon != null) {
      Image im;
      if (icon instanceof ImageIcon) {
        im = ((ImageIcon)icon).getImage();
      } else {
        im = createImageFromIcon(icon, component);
      }
      if (im != null) {
        return createShadowedImageIcon(im);
      }
    }
    return icon;
  }

  /**
   * Creates Image using the specified icon and component.
   *
   * @param icon the icon
   * @param component the component
   *
   * @return Image
   */
  public static Image createImageFromIcon(Icon icon, Component component) {
    try {
      BufferedImage bim = new BufferedImage(icon.getIconWidth(),
                                            icon.getIconHeight(),
                                            BufferedImage.TYPE_4BYTE_ABGR);
      Graphics2D bimg = bim.createGraphics();
      icon.paintIcon(component, bimg, 0, 0);
      bimg.dispose();
      return bim;
    } catch (Throwable t) { // NOPMD
      return null;
    }
  }

  /**
   * Creates ImageIcon using the specified icon and component.
   *
   * @param icon the icon
   * @param component the component
   *
   * @return ImageIcon
   */
  public static ImageIcon createImageIconFromIcon(Icon icon,
                                                  Component component) {
    Image image = createImageFromIcon(icon, component);
    if (image != null) {
      return new ImageIcon(image);
    }
    return null;
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-17 22:27:58 Slobodan
   Initial version

*****************************************************************************^*/
