/*
 * @(#)HelpPanel.java  1.0.0 / 2008-05-17
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.swing.help;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Insets;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.Enumeration;

import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTree;
import javax.swing.LookAndFeel;
import javax.swing.SwingUtilities;
import javax.swing.border.EmptyBorder;
import javax.swing.event.HyperlinkEvent;
import javax.swing.event.HyperlinkListener;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.TreePath;
import javax.swing.tree.TreeSelectionModel;

import net.wrka.swing.components.JWxHtmlBrowser;
import net.wrka.swing.components.JWxButton;
import net.wrka.swing.components.JWxSeparator;
import net.wrka.swing.components.JWxToolBar;
import net.wrka.swing.plaf.LookAndFeelChangeListener;
import net.wrka.swing.plaf.LookAndFeelManager;
import net.wrka.swing.resources.BuiltInResources;
import net.wrka.swing.tooltips.ToolTipManager;
import net.wrka.util.CommonConstants;
import net.wrka.xml.simple.XMLLoadingException;
import net.wrka.xml.simple.sax.Parser;

/**
 * SimpleHelp Panel.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
final class HelpPanel extends JPanel implements LookAndFeelChangeListener,
                                                TreeSelectionListener,
                                                HyperlinkListener {

/*^**************************************************************************^*/
/*- Private Static Fields.                                                   -*/
/*^**************************************************************************^*/

  private static final int HELP_WIDTH;
  private static final int HELP_HEIGHT;
  private static final int DIVIDER_LOCATION;
  private static final Dimension DIMENSIONS;

  private static HelpPanel instance;
  private static URL tocurl;
  private static URL currenturl;
  private static String tochome;
  private static DefaultMutableTreeNode toctree;

/*^**************************************************************************^*/
/*- Static Initialization.                                                   -*/
/*^**************************************************************************^*/

  static {
    HELP_WIDTH = 450;
    HELP_HEIGHT = 460;
    DIVIDER_LOCATION = 150;
    DIMENSIONS = new Dimension(HELP_WIDTH, HELP_HEIGHT);
  }

/*^**************************************************************************^*/
/*- Private Fields.                                                          -*/
/*^**************************************************************************^*/

  private final JTree jtTOC;
  private final JWxHtmlBrowser htmlBrowser;

/*^**************************************************************************^*/
/*- Private Constructor.                                                     -*/
/*^**************************************************************************^*/

  /**
   * Constructs new HelpPanel.
   */
  private HelpPanel() {
    super(new BorderLayout());
    JWxToolBar toolbar = new JWxToolBar(JWxToolBar.HORIZONTAL);
    toolbar.setFloatable(false);
    JWxButton backButton = new JWxButton();
    JWxButton forwardButton = new JWxButton();
    JWxButton reloadButton = new JWxButton();
    JWxButton homeButton = new JWxButton();
    setSize(DIMENSIONS);
    setMinimumSize(DIMENSIONS);
    setPreferredSize(DIMENSIONS);
    StringBuffer urlhome = new StringBuffer(tochome);
    urlhome.append('/');
    urlhome.append(((HelpTreeNode)toctree.getUserObject()).getTarget());
    URL urlh = null;
    try {
      urlh = new URL(urlhome.toString());
    } catch (MalformedURLException e) { // NOPMD
      // empty catch block
    }
    htmlBrowser = new JWxHtmlBrowser(urlh);
    htmlBrowser.addHyperlinkListener(this);
    // TOCTree
    jtTOC = new JTree(toctree);
    jtTOC.setBorder(new EmptyBorder(1, 1, 1, 1));
    jtTOC.putClientProperty("JTree.lineStyle", //$NON-NLS-1$
                            "None"); //$NON-NLS-1$
    HelpTreeCellRenderer jtTOCRenderer = new HelpTreeCellRenderer();
    jtTOC.setCellRenderer(jtTOCRenderer);
    jtTOC.getSelectionModel().setSelectionMode(TreeSelectionModel.
                                                 SINGLE_TREE_SELECTION);
    jtTOC.addTreeSelectionListener(this);
    jtTOC.setShowsRootHandles(false);
    // Toolbar
    backButton.setAction(htmlBrowser.getBackAction());
    backButton.setIcon(BuiltInResources.INSTANCE.
                                  getImageIcon("BackArrow.png")); //$NON-NLS-1$
    backButton.setText(CommonConstants.STRING_EMPTY);
    forwardButton.setAction(htmlBrowser.getForwardAction());
    forwardButton.setIcon(BuiltInResources.INSTANCE.
                               getImageIcon("ForwardArrow.png")); //$NON-NLS-1$
    forwardButton.setText(CommonConstants.STRING_EMPTY);
    reloadButton.setAction(htmlBrowser.getReloadAction());
    reloadButton.setIcon(BuiltInResources.INSTANCE.
                                    getImageIcon("Refresh.png")); //$NON-NLS-1$
    reloadButton.setText(CommonConstants.STRING_EMPTY);
    homeButton.setAction(htmlBrowser.getHomeAction());
    homeButton.setIcon(BuiltInResources.INSTANCE.
                                       getImageIcon("Home.png")); //$NON-NLS-1$
    homeButton.setText(CommonConstants.STRING_EMPTY);
    toolbar.add(backButton);
    Dimension sepdim = new Dimension(6, 20);
    toolbar.add(JWxSeparator.createFlat(sepdim));
    toolbar.add(forwardButton);
    toolbar.add(JWxSeparator.createFlat(sepdim));
    toolbar.add(reloadButton);
    toolbar.add(JWxSeparator.createFlat(sepdim));
    toolbar.add(homeButton);
    toolbar.add(JWxSeparator.createFlat(new Dimension(Short.MAX_VALUE, 20)));
    JPanel jpTlb = new JPanel(new BorderLayout());
    jpTlb.add(toolbar, BorderLayout.NORTH);
    jpTlb.add(new JWxSeparator(new Dimension(200, 5), new Insets(0, 0, 0, 0),
              JWxSeparator.TOP, false), BorderLayout.CENTER);
    add(jpTlb, BorderLayout.NORTH);
    // Split pane
    JSplitPane splitpane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT,
                                          new JScrollPane(jtTOC), htmlBrowser);
    splitpane.setDividerSize(8);
    splitpane.setOneTouchExpandable(false);
    splitpane.setContinuousLayout(true);
    splitpane.setDividerLocation(DIVIDER_LOCATION);
    splitpane.setResizeWeight(0.1);
    JPanel jpCont = new JPanel(new BorderLayout());
    jpCont.setBorder(new EmptyBorder(1, 1, 1, 1));
    jpCont.add(splitpane);
    add(jpCont);
    LookAndFeelManager.addLnFListener(this);
    ToolTipManager.registerContainer(this);
  }

/*^**************************************************************************^*/
/*- LookAndFeelChangeListener Implementation.                                -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  public void lookAndFeelChanged(LookAndFeel oldLnF, LookAndFeel newLnF) {
    SwingUtilities.updateComponentTreeUI(this);
    jtTOC.updateUI();
    jtTOC.setBackground(htmlBrowser.getBackground());
  }

/*^**************************************************************************^*/
/*- TreeSelectionListener Implementation.                                    -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  public void valueChanged(TreeSelectionEvent e) {
    Object[] nodes = e.getPath().getPath();
    DefaultMutableTreeNode lastNode =
                               (DefaultMutableTreeNode)nodes[nodes.length - 1];
    HelpTreeNode nodeTarget = (HelpTreeNode)lastNode.getUserObject();
    String lastTarget = nodeTarget.getTarget();
    if ((lastTarget != null) && (lastTarget.length() > 0)) {
      StringBuffer urlsb = new StringBuffer(tochome);
      urlsb.append('/');
      for (int i = 1; i < nodes.length - 1; i++) {
        DefaultMutableTreeNode node = (DefaultMutableTreeNode)nodes[i];
        HelpTreeNode htnode = (HelpTreeNode)node.getUserObject();
        urlsb.append(htnode.getName());
        urlsb.append('/');
      }
      urlsb.append(lastTarget);
      try {
        htmlBrowser.navigate(new URL(urlsb.toString()));
      } catch (Throwable t) { // NOPMD
        // empty catch block
      }
    }
  }

/*^**************************************************************************^*/
/*- HyperlinkListener Implementation.                                        -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  public void hyperlinkUpdate(HyperlinkEvent e) {
    if (e.getEventType() == HyperlinkEvent.EventType.ACTIVATED) {
      URL url = e.getURL();
      if (!url.equals(currenturl)) {
        currenturl = url;
        String cu = url.toExternalForm();
        if ((tochome != null) && (tochome.length() < cu.length())) {
          selectTOCTreeNode(cu.substring(tochome.length() + 1));
        }
      }
    }
  }

/*^**************************************************************************^*/
/*- Package-Private Static Methods.                                          -*/
/*^**************************************************************************^*/

  static void init(URL tocurl) {
    if (tocurl == null) {
      throw new IllegalArgumentException(
                           "Simple Help Support: Null Help TOC"); //$NON-NLS-1$
    }
    HelpPanel.tocurl = tocurl;
    createTOCTree();
    tochome = tocurl.toExternalForm();
    int i = tochome.lastIndexOf('/');
    if (i > 0) {
      tochome = tochome.substring(0, i);
    }
    if (toctree == null) { // NOPMD
      instance = null;
      throw new ExceptionInInitializerError(
                   "SimpleHelp Support: Initialization failed."); //$NON-NLS-1$
    }
    instance = new HelpPanel();
  }

  static HelpPanel getInstance() {
    if (instance == null) {
      throw new IllegalStateException(
                "SimpleHelp Support: There is no valid HelpTOC"); //$NON-NLS-1$
    }
    return instance;
  }

  static HelpPanel getInstance(String topic) {
    if (instance == null) {
      throw new IllegalStateException(
                "SimpleHelp Support: There is no valid HelpTOC"); //$NON-NLS-1$
    } else if (setTopic(topic)) {
      return instance;
    }
    return null;
  }

  static boolean setTopic(String topic) {
    if (instance != null) {
      try {
        Enumeration te = toctree.depthFirstEnumeration();
        HelpTreeNode tn = null;
        while (te.hasMoreElements()) {
          DefaultMutableTreeNode mtn = (DefaultMutableTreeNode)te.nextElement();
          tn = (HelpTreeNode)mtn.getUserObject();
          if (tn.getName().equalsIgnoreCase(topic)) {
            StringBuffer sb = new StringBuffer(tochome);
            sb.append('/');
            sb.append(tn.getTarget());
            return instance.htmlBrowser.navigate(new URL(sb.toString()));
          }
        }
      } catch (Throwable t) { // NOPMD
        // empty catch block
      }
    }
    return false;
  }

/*^**************************************************************************^*/
/*- Private Static Methods.                                                  -*/
/*^**************************************************************************^*/

  private static void createTOCTree() {
    InputStream is = null;
    try {
      is = tocurl.openStream();
      toctree = new DefaultMutableTreeNode();
      InputStreamReader isr = new InputStreamReader(is);
      Parser.parse(isr, new HelpHandler(toctree));
    } catch (XMLLoadingException se) {
      System.out.println(// NOPMD
                "SimpleHelp Support: Could not create HelpTOC."); //$NON-NLS-1$
      toctree = null;
    } catch (Throwable t) { // NOPMD
      System.out.println(// NOPMD
                    "SimpleHelp Support: Initialization error."); //$NON-NLS-1$
      toctree = null;
    } finally {
      if (is != null) {
        try {
          is.close();
        } catch (IOException ioe) { // NOPMD
          // empty catch block
        }
      }
    }
  }

/*^**************************************************************************^*/
/*- Private Method.                                                          -*/
/*^**************************************************************************^*/

  private void selectTOCTreeNode(String addr) {
    TreePath tp = new TreePath(toctree);
    DefaultMutableTreeNode tn = toctree;
    int sindex = 0;
    int findex = 0;
    int addrlen = addr.length();
    String t;
    while ((sindex < addrlen) && (sindex >= 0)) {
      findex = addr.indexOf('/', sindex);
      if (findex < 0) {
        t = addr.substring(sindex);
        int tncc = tn.getChildCount();
        for (int i = 0; i < tncc; i++) {
          DefaultMutableTreeNode dmtn =
              (DefaultMutableTreeNode)tn.getChildAt(i);
          HelpTreeNode htn = (HelpTreeNode)dmtn.getUserObject();
          if (htn.getTarget().equals(t)) {
            tp = tp.pathByAddingChild(dmtn);
            break;
          }
        }
        break;
      }
      t = addr.substring(sindex, findex);
      int tncc = tn.getChildCount();
      for (int i = 0; i < tncc; i++) {
        DefaultMutableTreeNode dmtn = (DefaultMutableTreeNode)tn.getChildAt(i);
        HelpTreeNode htn = (HelpTreeNode)dmtn.getUserObject();
        if (htn.getName().equals(t)) {
          tp = tp.pathByAddingChild(dmtn);
          tn = dmtn;
          break;
        }
      }
      sindex = findex + 1;
      tp.pathByAddingChild(tn);
    }
    jtTOC.setSelectionPath(tp);
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-17 21:06:39 Slobodan
   Initial version

*****************************************************************************^*/
