/*
 * @(#)HelpTreeCellRenderer.java  1.0.0 / 2008-05-17
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.swing.help;

import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Rectangle;

import javax.swing.Icon;
import javax.swing.JLabel;
import javax.swing.JTree;
import javax.swing.UIManager;
import javax.swing.plaf.ColorUIResource;
import javax.swing.plaf.FontUIResource;
import javax.swing.tree.TreeCellRenderer;

import net.wrka.swing.resources.BuiltInResources;

/**
 * SimpleHelp TOC Tree Cell Renderer.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
final class HelpTreeCellRenderer extends JLabel implements TreeCellRenderer {

/*^**************************************************************************^*/
/*- Private Fields.                                                          -*/
/*^**************************************************************************^*/

  private JTree tree;
  private boolean selected;
  private boolean hasFocus;
  private final Icon closedIcon;
  private final Icon leafIcon;
  private final Icon openIcon;
  private Color textSelectionColor;
  private Color textNonSelectionColor;
  private Color backgroundSelectionColor;
  private Color borderSelectionColor;
  private Font boldFont;
  private Font planeFont;

/*^**************************************************************************^*/
/*- Public Constructor.                                                      -*/
/*^**************************************************************************^*/

  /**
   * Constructs new HelpTreeCellRenderer object.
   */
  public HelpTreeCellRenderer() {
    super();
    setOpaque(false);
    setHorizontalAlignment(JLabel.LEFT);
    openIcon = BuiltInResources.INSTANCE.
                                 getImageIcon("TOCopenBook.png"); //$NON-NLS-1$
    closedIcon = BuiltInResources.INSTANCE.
                                getImageIcon("TOCcloseBook.png"); //$NON-NLS-1$
    leafIcon = BuiltInResources.INSTANCE.
                                    getImageIcon("TOCtopic.png"); //$NON-NLS-1$
  }

/*^**************************************************************************^*/
/*- Overridden Method.                                                       -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  public Component getTreeCellRendererComponent(JTree tree, Object value,
                                                boolean sel, boolean expanded,
                                                boolean leaf, int row,
                                                boolean hasFocus) {
    String stringValue = tree.convertValueToText(value, sel, expanded, leaf,
                                                 row, hasFocus);
    this.tree = tree;
    this.hasFocus = hasFocus;
    selected = sel;
    setText(stringValue);
    if (sel) {
      setForeground(textSelectionColor);
    } else {
      setForeground(textNonSelectionColor);
    }
    if (leaf) {
      setIcon(leafIcon);
    } else if (expanded) {
      setIcon(openIcon);
    } else {
      setIcon(closedIcon);
    }
    Font font = getFont();
    if (leaf) {
      if (!font.isPlain()) {
        super.setFont(planeFont);
      }
    } else {
      if (!font.isBold()) {
        super.setFont(boldFont);
      }
    }
    return this;
  }

  /**
   * {@inheritDoc}
   */
  public void setFont(Font font) {
    if (font instanceof FontUIResource) {
      font = null;
    }
    super.setFont(font);
  }

  /**
   * {@inheritDoc}
   */
  public Font getFont() {
    Font font = super.getFont();
    if ((font == null) && (tree != null)) {
      font = tree.getFont();
    }
    return font;
  }

  /**
   * {@inheritDoc}
   */
  public void setBackground(Color color) {
    if (color instanceof ColorUIResource) {
      color = null;
    }
    super.setBackground(color);
  }

  /**
   * {@inheritDoc}
   */
  public Dimension getPreferredSize() {
    Dimension retDimension = super.getPreferredSize();
    if (retDimension != null) {
      retDimension = new Dimension(retDimension.width + 2, retDimension.height);
    }
    return retDimension;
  }

  /**
   * {@inheritDoc}
   */
  public void paint(Graphics g) {
    Color bgColor;
    if (selected) {
      bgColor = backgroundSelectionColor;
    } else {
      bgColor = tree.getBackground();
    }
    int imageOffset;
    Icon icon = getIcon();
    if ((icon == null) || (getText() == null)) {
      imageOffset = 0;
    } else {
      imageOffset = icon.getIconWidth();
      int itg = getIconTextGap();
      if (itg > 2) {
        imageOffset = imageOffset + itg - 2;
      }
    }
    if (bgColor != null) {
      g.setColor(bgColor);
      g.fillRect(imageOffset, 0, getWidth() - 1 - imageOffset, getHeight());
    }
    if (hasFocus && (borderSelectionColor != null)) {
      g.setColor(borderSelectionColor);
      g.drawRect(imageOffset, 0, getWidth() - 1 - imageOffset, getHeight() - 1);
    }
    super.paint(g);
  }

  /**
   * {@inheritDoc}
   */
  public void repaint(Rectangle r) {
    // empty method body
  }

  /**
   * {@inheritDoc}
   */
  public void repaint(long tm, int x, int y, int width, int height) {
    // empty method body
  }

  /**
   * {@inheritDoc}
   */
  public void validate() {
    // empty method body
  }

  /**
   * {@inheritDoc}
   */
  public void revalidate() {
    // empty method body
  }

  /**
   * {@inheritDoc}
   */
  protected void firePropertyChange(String propertyName, Object oldValue,
                                    Object newValue) {
    if ("text".equals(propertyName) || //$NON-NLS-1$
        "font".equals(propertyName)) { //$NON-NLS-1$
      super.firePropertyChange(propertyName, oldValue, newValue);
    }
  }

  /**
   * {@inheritDoc}
   */
  public void firePropertyChange(String propertyName, byte oldValue,
                                 byte newValue) {
    // empty method body
  }

  /**
   * {@inheritDoc}
   */
  public void firePropertyChange(String propertyName, char oldValue,
                                 char newValue) {
    // empty method body
  }

  /**
   * {@inheritDoc}
   */
  public void firePropertyChange(String propertyName, short oldValue,
                                 short newValue) {
    // empty method body
  }

  /**
   * {@inheritDoc}
   */
  public void firePropertyChange(String propertyName, int oldValue,
                                 int newValue) {
    // empty method body
  }

  /**
   * {@inheritDoc}
   */
  public void firePropertyChange(String propertyName, long oldValue,
                                 long newValue) {
    // empty method body
  }

  /**
   * {@inheritDoc}
   */
  public void firePropertyChange(String propertyName, float oldValue,
                                 float newValue) {
    // empty method body
  }

  /**
   * {@inheritDoc}
   */
  public void firePropertyChange(String propertyName, double oldValue,
                                 double newValue) {
    // empty method body
  }

  /**
   * {@inheritDoc}
   */
  public void firePropertyChange(String propertyName, boolean oldValue,
                                 boolean newValue) {
    // empty method body
  }

  /**
   * {@inheritDoc}
   */
  public void updateUI() {
    super.updateUI();
    initDefaults();
  }

/*^**************************************************************************^*/
/*- Private Method.                                                          -*/
/*^**************************************************************************^*/

  private void initDefaults() {
    Font font = UIManager.getFont("Label.font"); //$NON-NLS-1$
    if (font == null) {
      font = new Font("Dialog", Font.PLAIN, 11); //$NON-NLS-1$
    }
    planeFont = font.deriveFont(Font.PLAIN);
    boldFont = font.deriveFont(Font.BOLD);
    textSelectionColor = UIManager.getColor(
                                     "Tree.selectionForeground"); //$NON-NLS-1$
    textNonSelectionColor = UIManager.getColor(
                                          "Tree.textForeground"); //$NON-NLS-1$
    backgroundSelectionColor = UIManager.getColor(
                                     "Tree.selectionBackground"); //$NON-NLS-1$
    borderSelectionColor = UIManager.getColor(
                                    "Tree.selectionBorderColor"); //$NON-NLS-1$
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-17 21:37:11 Slobodan
   Initial version

*****************************************************************************^*/
