/*
 * @(#)SimpleHelp.java  1.0.0 / 2008-05-17
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.swing.help;

import java.awt.Container;
import java.awt.Dimension;
import java.awt.KeyboardFocusManager;
import java.awt.Rectangle;
import java.awt.Toolkit;
import java.awt.Window;
import java.awt.event.ActionEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.net.URL;

import javax.swing.Action;
import javax.swing.ImageIcon;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.KeyStroke;

import net.wrka.swing.actions.HelpAction;
import net.wrka.swing.resources.BuiltInResources;

/**
 * Simple help support.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public final class SimpleHelp {

/*^**************************************************************************^*/
/*- Private-Private Static Fields.                                           -*/
/*^**************************************************************************^*/

  static boolean initialized;
  static JDialog dialog;
  static JFrame frame;
  static ImageIcon icon;
  static Rectangle bounds;
  static HelpInitializer initializer;
  static HelpWindowListener helpWindowListener;
  static URL tocurl;
  static String title;

/*^**************************************************************************^*/
/*- Private Static Final Fields.                                             -*/
/*^**************************************************************************^*/

  private static final int NON_ICONIFIED;
  private static final Dimension SCREEN_DIMENZION;
  private static final Rectangle OWNER_BOUNDS;

/*^**************************************************************************^*/
/*- Static Initialization.                                                   -*/
/*^**************************************************************************^*/

  static {
    NON_ICONIFIED = ~JFrame.ICONIFIED;
    SCREEN_DIMENZION = Toolkit.getDefaultToolkit().getScreenSize();
    OWNER_BOUNDS = new Rectangle();
  }

/*^**************************************************************************^*/
/*- Private Constructor.                                                     -*/
/*^**************************************************************************^*/

  /**
   * Class has no instances.
   */
  private SimpleHelp() {
    // empty constructor body
  }

/*^**************************************************************************^*/
/*- Public Static Methods.                                                   -*/
/*^**************************************************************************^*/

  /**
   * Initializes help support and sets help TOC.
   *
   * @param title the help window title
   * @param tocurl the help TOC path
   * @param icon the help icon
   */
  public static void setHelp(String title, URL tocurl, ImageIcon icon) {
    if (SimpleHelp.tocurl == null) {
      if (tocurl == null) {
        throw new IllegalArgumentException("Null TOC url"); //$NON-NLS-1$
      }
      SimpleHelp.title = title;
      SimpleHelp.tocurl = tocurl;
      if (icon == null) {
        SimpleHelp.icon = BuiltInResources.INSTANCE.
                                        getImageIcon("Help.png"); //$NON-NLS-1$
      } else {
        SimpleHelp.icon = icon;
      }
      initializer = new HelpInitializer();
      initializer.start();
    } else {
      System.out.println(// NOPMD
                         "Help support is already initialized"); //$NON-NLS-1$
    }
  }

  /**
   * Initializes help support and sets help TOC.
   *
   * @param title the help window title
   * @param tocurl the help TOC path
   */
  public static void setHelp(String title, URL tocurl) {
    setHelp(title, tocurl, null);
  }

  /**
   * Returns help TOC url.
   *
   * @return help TOC url
   */
  public static URL getHelpTOC() {
    return tocurl;
  }

  /**
   * Returns help action.
   *
   * @param topic help topic
   *
   * @return help action
   */
  public static Action getHelpAction(String topic) {
    if (initializer == null) {
      System.out.println("Help Support is not initialized"); //$NON-NLS-1$ NOPMD
      return null;
    }
    return new SimpleHelpAction(topic);
  }

  /**
   * Returns help action.
   *
   * @param name the action name
   * @param tooltip the action tooltip
   * @param topic help topic
   *
   * @return help action
   */
  public static Action getHelpAction(String name, String tooltip,
                                     String topic) {
    if (initializer == null) {
      System.out.println("Help Support is not initialized"); //$NON-NLS-1$ NOPMD
      return null;
    }
    return new SimpleHelpAction(name, tooltip, topic);
  }

  /**
   * Returns help action.
   *
   * @param name the action name
   * @param tooltip the action tooltip
   * @param mnemonic mnemonic
   * @param acc accelerator
   * @param topic help topic
   *
   * @return help action
   */
  public static Action getHelpAction(String name, String tooltip, int mnemonic,
                                     KeyStroke acc, String topic) {
    if (initializer == null) {
      System.out.println("Help Support is not initialized"); //$NON-NLS-1$ NOPMD
      return null;
    }
    return new SimpleHelpAction(name, tooltip, mnemonic, acc, topic);
  }

  /**
   * Displays help.
   *
   * @param topic the help topic
   */
  public static void showHelp(String topic) { // NOPMD
    if (initializer == null) {
      System.out.println("Help Support is not initialized"); //$NON-NLS-1$ NOPMD
      return;
    }
    if (!initialized) {
      try {
        initializer.join();
      } catch (InterruptedException ie) { // NOPMD
        // empty catch block
      }
    }
    Window owner = KeyboardFocusManager.
                            getCurrentKeyboardFocusManager().getActiveWindow();
    if (owner instanceof JDialog) {
      HelpPanel hp = HelpPanel.getInstance(topic);
      if (hp == null) {
        System.out.print(// NOPMD
                         "Help Support: There is no topic: "); //$NON-NLS-1$
        System.out.println(topic); //$NON-NLS-1$ NOPMD
        return;
      }
      if ((frame != null) && (frame.isVisible())) {
        frame.setVisible(false);
      }
      if (dialog != null) { // NOPMD - not singleton
        Window oldOwner = dialog.getOwner();
        if (owner == oldOwner) { // NOPMD
          Container cont = dialog.getContentPane();
          if (cont.getComponentCount() == 0) {
            cont.add(hp);
          }
          dialog.setVisible(true);
          dialog.toFront();
          return;
        }
        bounds = dialog.getBounds();
        dialog.removeWindowListener(helpWindowListener);
        dialog.dispose();
      }
      dialog = new JDialog((JDialog)owner, title, false);
      dialog.getContentPane().add(hp);
      dialog.pack();
      if (bounds == null) { // NOPMD - not singleton
        bounds = calcPosition(owner, dialog);
      }
      dialog.setBounds(bounds);
      dialog.setDefaultCloseOperation(JDialog.DO_NOTHING_ON_CLOSE);
      dialog.addWindowListener(helpWindowListener);
      dialog.setVisible(true);
    } else {
      if ((dialog != null) && dialog.isVisible()) {
        dialog.setVisible(false);
      }
      if (!HelpPanel.setTopic(topic)) {
        System.out.print(// NOPMD
                         "Help Support: There is no topic: "); //$NON-NLS-1$
        System.out.println(topic); // NOPMD
        return;
      }
      Container cp = frame.getContentPane();
      if (cp.getComponentCount() == 0) {
        cp.add(HelpPanel.getInstance());
      }
      if (frame.isVisible()) {
        int frameState = frame.getExtendedState();
        if ((frameState & JFrame.ICONIFIED) > 0) {
          frameState = frameState & NON_ICONIFIED;
          frame.setExtendedState(frameState);
        }
        frame.toFront();
      } else {
        if (bounds == null) { // NOPMD - not singleton
          bounds = calcPosition(owner, frame);
        }
        frame.setBounds(bounds);
        int frameState = frame.getExtendedState();
        if ((frameState & JFrame.ICONIFIED) > 0) {
          frameState = frameState & NON_ICONIFIED;
          frame.setExtendedState(frameState);
        }
        frame.setVisible(true);
      }
    }
  }

/*^**************************************************************************^*/
/*- Private Methods.                                                         -*/
/*^**************************************************************************^*/

  private static Rectangle calcPosition(Window owner, Window inst) {
    Rectangle owb = SimpleHelp.OWNER_BOUNDS;
    if (owner == null) {
      owb.setLocation(0, 0);
      owb.setSize(SCREEN_DIMENZION);
    } else {
      owb.setBounds(owner.getBounds());
    }
    int xp = ((owb.width  - inst.getWidth()) >> 1)  + owb.x;
    int yp = ((owb.height - inst.getHeight()) >> 1) + owb.y;
    if (yp < 0) {
      yp = 0;
    }
    if (xp > SCREEN_DIMENZION.width) {
      xp = SCREEN_DIMENZION.width - inst.getWidth();
    }
    owb.setSize(inst.getSize());
    owb.setLocation(xp, yp);
    return owb;
  }

/*^**************************************************************************^*/
/*- Inner Classes.                                                           -*/
/*^**************************************************************************^*/

  /**
   * Simple Help Action.
   */
  static final class SimpleHelpAction extends HelpAction {
    String topic;
    SimpleHelpAction(String topic) {
      super();
      this.topic = topic;
    }
    SimpleHelpAction(String name, String tooltip, String topic) {
      super(name, icon);
      if (tooltip != null) {
        putValue(Action.SHORT_DESCRIPTION, tooltip);
      }
      this.topic = topic;
    }
    SimpleHelpAction(String name, String tooltip, int mnemonic, KeyStroke acc,
                     String topic) {
      super(name, icon);
      if (tooltip != null) {
        putValue(Action.SHORT_DESCRIPTION, tooltip);
      }
      putValue(Action.MNEMONIC_KEY, new Integer(mnemonic));
      if (acc != null) {
        putValue(Action.ACCELERATOR_KEY, acc);
      }
      this.topic = topic;
    }
    /**
     * {@inheritDoc}
     */
    public void actionPerformed(ActionEvent e) {
      showHelp(topic);
    }
  }

  /**
   * Help Initializer.
   */
  static final class HelpInitializer extends Thread {
    HelpInitializer() {
      super("HelpInitializer"); //$NON-NLS-1$
      setDaemon(true);
      setPriority(MIN_PRIORITY);
    }
    /**
     * {@inheritDoc}
     */
    public void run() {
      yield();
      try {
        HelpPanel.init(tocurl);
      } catch (Throwable t) { // NOPMD
        initializer = null;
        return;
      }
      frame = new JFrame(title);
      frame.setIconImage(icon.getImage());
      frame.setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
      helpWindowListener = new HelpWindowListener();
      frame.addWindowListener(helpWindowListener);
      frame.getContentPane().add(HelpPanel.getInstance());
      frame.pack();
      initialized = true;
    }
  }

  /**
   * HelpWindow Listener.
   */
  static final class HelpWindowListener extends WindowAdapter {
    /**
     * {@inheritDoc}
     */
    public void windowClosing(WindowEvent e) {
      Window window = e.getWindow();
      bounds = window.getBounds();
      window.setVisible(false);
    }
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-17 20:28:01 Slobodan
   Initial version

*****************************************************************************^*/
