/*
 * @(#)FileChooserFilter.java  1.0.0 / 2008-05-15
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.swing.io;

import java.io.File;
import java.util.Locale;
import javax.swing.filechooser.FileFilter;

import net.wrka.io.FileUtils;
import net.wrka.util.SimpleStringMap;
import net.wrka.util.StringIterator;

/**
 * FileFilter Implementation.
 * <p>
 *  <tt>Based on Sun's ExampleFileFilter.java</tt>
 * </p>
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public class FileChooserFilter extends FileFilter {

/*^**************************************************************************^*/
/*- Private Fields.                                                          -*/
/*^**************************************************************************^*/

  private final SimpleStringMap filters;
  private String description;
  private String fullDescription;
  private boolean useExtensionsInDescription;

/*^**************************************************************************^*/
/*- Public Constructors.                                                     -*/
/*^**************************************************************************^*/

  /**
   * Constructs new FileChooserFilter object.
   */
  public FileChooserFilter() {
    super();
    filters = new SimpleStringMap(3);
  }

  /**
   * Constructs new FileChooserFilter object.
   *
   * @param extension extension
   */
  public FileChooserFilter(String extension) {
    this(extension, null);
  }

  /**
   * Constructs new FileChooserFilter object.
   *
   * @param extension extension
   * @param description extension description
   */
  public FileChooserFilter(String extension, String description) {
    this();
    if (extension != null) {
      String lcext = extension.toLowerCase(Locale.getDefault());
      filters.put(lcext, lcext);
    }
    if (description != null) {
      this.description = description;
      useExtensionsInDescription = true;
    }
  }

  /**
   * Constructs new FileChooserFilter object.
   *
   * @param filters String[] filter extensions
   */
  public FileChooserFilter(String[] filters) {
    this(filters, null);
  }

  /**
   * Constructs new FileChooserFilter object.
   *
   * @param filters String[] filter extensions
   * @param description filter description
   */
  public FileChooserFilter(String[] filters, String description) {
    this();
    for (int i = 0; i < filters.length; i++) {
      String lcext = filters[i];
      if (lcext != null) {
        lcext = lcext.toLowerCase(Locale.getDefault());
        this.filters.put(lcext, lcext);
      }
    }
    if (description != null) {
      this.description = description;
      useExtensionsInDescription = true;
    }
  }

/*^**************************************************************************^*/
/*- Public Methods.                                                          -*/
/*^**************************************************************************^*/

  /**
   * Adds new extension.
   *
   * @param extension the extension
   */
  public void addExtension(String extension) {
    if (extension != null) {
      String lcext = extension.toLowerCase(Locale.getDefault());
      filters.put(lcext, lcext);
      if (fullDescription != null) {
        fullDescription = null;
      }
    }
  }

  /**
   * Sets filter description.
   *
   * @param description the description
   */
  public void setDescription(String description) {
    this.description = description;
    useExtensionsInDescription = (description != null);
    if (fullDescription != null) {
      fullDescription = null;
    }
  }

  /**
   * Sets whether or not extension list will be used in description.
   *
   * @param b if <tt>true</tt> extension list will be used in description
   */
  public void setExtensionListInDescription(boolean b) {
    useExtensionsInDescription = b;
    if (fullDescription != null) {
      fullDescription = null;
    }
  }

  /**
   * Returns true if extension list is used in description.
   *
   * @return <tt>true</tt> if extension list is used in description
   */
  public boolean isExtensionListInDescription() {
    return useExtensionsInDescription;
  }

/*^**************************************************************************^*/
/*- Overridden Methods.                                                      -*/
/*^**************************************************************************^*/

  /**
   * Returns <tt>true</tt> if file is acceptable.
   *
   * @param f the File
   *
   * @return <tt>true</tt> if file is acceptable
   */
  public boolean accept(File f) {
    return (f != null) && (f.isDirectory() ||
                           filters.containsKey(FileUtils.getFileExtension(f)));
  }

  /**
   * Returns filter description.
   *
   * @return filter description
   */
  public String getDescription() {
    if (fullDescription == null) {
      if ((description == null) || useExtensionsInDescription) {
        StringBuffer sb = new StringBuffer();
        if (description == null) {
          sb.append('(');
        } else {
          sb.append(description);
          sb.append(" ("); //$NON-NLS-1$
        }
        StringIterator extensions = filters.keys();
        if (extensions.hasNextString()) {
          sb.append("*."); //$NON-NLS-1$
          sb.append(extensions.nextString());
          while (extensions.hasNextString()) {
            sb.append(", *."); //$NON-NLS-1$
            sb.append(extensions.nextString());
          }
        }
        sb.append(')');
        fullDescription = sb.toString();
      } else {
        fullDescription = description;
      }
    }
    return fullDescription;
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-15 21:41:09 Slobodan
   Initial version

*****************************************************************************^*/
