/*
 * @(#)LookAndFeelManager.java  1.0.0 / 2008-05-19
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.swing.plaf;

import java.awt.Color;
import java.awt.Toolkit;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.lang.reflect.Method;
import java.util.Iterator;

import javax.swing.JComponent;
import javax.swing.LookAndFeel;
import javax.swing.UIDefaults;
import javax.swing.UIManager;
import javax.swing.plaf.ComponentUI;
import javax.swing.plaf.metal.MetalLookAndFeel;

import net.wrka.LibraryConfig;
import net.wrka.LibraryInfo;
import net.wrka.swing.plaf.smooth.SmoothLookAndFeel;
import net.wrka.util.GPTools;
import net.wrka.util.SimpleHashMap;
import net.wrka.util.ref.SimpleSoftHashMap;
import net.wrka.util.ref.WeakArrayList;

/**
 * Look and Feel Manager.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public final class LookAndFeelManager implements PropertyChangeListener {

/*^**************************************************************************^*/
/*- Public Static Fields.                                                    -*/
/*^**************************************************************************^*/

  /**
   * Default swing look and feel key.
   */
  public static final String SWING_DEFAULT_LAF =
                                              "swing.defaultlaf"; //$NON-NLS-1$

/*^**************************************************************************^*/
/*- Private Static Fields.                                                   -*/
/*^**************************************************************************^*/

  private static final SimpleHashMap INSTALLED_LOOK_AND_FEELS;
  private static final WeakArrayList LOOK_AND_FEEL_LISTENERS;
  private static final SimpleSoftHashMap UI_CACHE;
  private static final Class[] CREATE_UI_TYPES;
  private static final Object[] CREATE_UI_ARGS;
  private static LookAndFeelManager lnfListener;
  private static Color controlColor;
  private static Color controlDarkShadow;
  private static Class windowLnFClass;
  private static Method isClassicWindows;
  private static boolean winXPThemed;
  private static boolean metalLnF;

/*^**************************************************************************^*/
/*- Static Initialization.                                                   -*/
/*^**************************************************************************^*/

  static {
    CREATE_UI_TYPES = new Class[] {JComponent.class};
    CREATE_UI_ARGS = new Object[1];
    UI_CACHE = new SimpleSoftHashMap();
    LOOK_AND_FEEL_LISTENERS = new WeakArrayList(30);
    INSTALLED_LOOK_AND_FEELS = new SimpleHashMap();
  }

/*^**************************************************************************^*/
/*- Private Constructor.                                                     -*/
/*^**************************************************************************^*/

  /**
   * Class has no instances.
   */
  private LookAndFeelManager() {
    // empty constructor body
  }

/*^**************************************************************************^*/
/*- Public Static Methods.                                                   -*/
/*^**************************************************************************^*/

  /**
   * Manager initialization.
   */
  public static void initialize() {
    if (lnfListener == null) { // NOPMD - not thread safe
      lnfListener = new LookAndFeelManager();
      UIManager.addPropertyChangeListener(lnfListener);
      LookAndFeel laf = UIManager.getLookAndFeel();
      windowLnFClass = GPTools.getClassForName(
            "com.sun.java.swing.plaf.windows.WindowsLookAndFeel"); //$NON-NLS-1$
      if (windowLnFClass != null) { // NOPMD - not thread safe
        try {
          isClassicWindows = windowLnFClass.getMethod("isClassicWindows", null);
        } catch (Throwable t) { // NOPMD
          // empty catch block
        }
      }
      detectLnF(laf);
      grabInstalledLnFs();
      String lafClassName = laf.getClass().getName();
      String defLnFClassName = System.getProperty(SWING_DEFAULT_LAF);
      if (!lafClassName.equals(defLnFClassName)) {
        setLookAndFeel(defLnFClassName);
      }
    }
  }

  /**
   * Adds a <tt>WxLnFListener</tt> to the listener list.
   *
   * @param lnfl the look and feel listener
   */
  public static void addLnFListener(LookAndFeelChangeListener lnfl) {
    LOOK_AND_FEEL_LISTENERS.add(lnfl);
  }

  /**
   * Removes a <tt>WxLnFListener</tt> from the listener list.
   *
   * @param lnfl the look and feel listener
   */
  public static void removeLnFListener(LookAndFeelChangeListener lnfl) {
    LOOK_AND_FEEL_LISTENERS.remove(lnfl);
  }

  /**
   * Returns lnf class name.
   *
   * @param lnfName lnf name
   *
   * @return lnf class name
   */
  public static String getLnFClassName(String lnfName) {
    if (lnfName != null) {
      return (String)INSTALLED_LOOK_AND_FEELS.get(lnfName);
    }
    return null;
  }

  /**
   * Returns <tt>true</tt> if specified lnf is installed.
   *
   * @param lnfName lnf name
   *
   * @return <tt>true</tt> if specified lnf is installed
   */
  public static boolean isLnFInstalled(String lnfName) {
    if (lnfName != null) {
      return INSTALLED_LOOK_AND_FEELS.containsKey(lnfName);
    }
    return false;
  }

  /**
   * Installs new look and feel.
   *
   * @param name look and feel name
   * @param classname look and feel class name
   */
  public static void installLookAndFeel(String name, String classname) {
    if ((name != null) && (classname != null) &&
        (!INSTALLED_LOOK_AND_FEELS.containsKey(name))) {
      INSTALLED_LOOK_AND_FEELS.put(name, classname);
    }
  }

  /**
   * Returns enumeration over lnf names.
   *
   * @return enumeration over lnf names
   */
  public static Iterator lnfNames() {
    return INSTALLED_LOOK_AND_FEELS.keys();
  }

  /**
   * Returns enumeration over lnf classes.
   *
   * @return enumeration over lnf classes
   */
  public static Iterator lnfClasses() {
    return INSTALLED_LOOK_AND_FEELS.values();
  }

  /**
   * Returns current look and feel.
   *
   * @return current look and feel
   */
  public static LookAndFeel getCurrentLookAndFeel() {
    return UIManager.getLookAndFeel();
  }

  /**
   * Returns current look and feel name.
   *
   * @return current look and feel name
   */
  public static String getCurrentLookAndFeelName() {
    return UIManager.getLookAndFeel().getName();
  }

  /**
   * Sets installed look and feel.
   *
   * @param name lnf name
   */
  public static void setInstalledLookAndFeel(String name) {
    try {
      Object lnfname = getLnFClassName(name);
      if (lnfname != null) {
        setNewLookAndFeel((String)lnfname);
      }
    } catch (Throwable t) { // NOPMD
      if (LibraryConfig.VERBOSE) {
        // CHANGE: use logger
        System.out.println(// NOPMD
                           "Look and Feel installation failed."); //$NON-NLS-1$
      }
      if (LibraryConfig.DEBUG) {
        // CHANGE: use logger
        t.printStackTrace(); // NOPMD - debug
      }
      LookAndFeel lnf = UIManager.getLookAndFeel();
      lnf.provideErrorFeedback(null);
    }
  }

  /**
   * Sets look and feel.
   *
   * @param name lnf name
   */
  public static void setLookAndFeel(String name) {
    try {
      UIManager.setLookAndFeel(name);
    } catch (Throwable t) { // NOPMD
      if (LibraryConfig.VERBOSE) {
        // CHANGE: use logger
        System.out.println(// NOPMD
                           "Look and Feel installation failed."); //$NON-NLS-1$
      }
      if (LibraryConfig.DEBUG) {
        // CHANGE: use logger
        t.printStackTrace(); // NOPMD - debug
      }
      LookAndFeel lnf = UIManager.getLookAndFeel();
      lnf.provideErrorFeedback(null);
    }
  }

  /**
   * Sets look and feel.
   *
   * @param newlnf new look and feel
   */
  public static void setLookAndFeel(LookAndFeel newlnf) {
    try {
      UIManager.setLookAndFeel(newlnf);
    } catch (Throwable t) { // NOPMD
      if (LibraryConfig.VERBOSE) {
        // CHANGE: use logger
        System.out.println(// NOPMD
                           "Look and Feel installation failed."); //$NON-NLS-1$
      }
      if (LibraryConfig.DEBUG) {
        // CHANGE: use logger
        t.printStackTrace(); // NOPMD - debug
      }
      LookAndFeel lnf = UIManager.getLookAndFeel();
      lnf.provideErrorFeedback(null);
    }
  }

  /**
   * Sets look and feel only if current look and feel is different.
   *
   * @param name lnf name
   */
  public static void setNewLookAndFeel(String name) {
    LookAndFeel lnf = UIManager.getLookAndFeel();
    if ((name != null) && (!name.equals(lnf.getName()))) {
      setLookAndFeel(name);
    }
  }

  /**
   * Sets look and feel only if current look and feel is different.
   *
   * @param lnf look and feel
   */
  public static void setNewLookAndFeel(LookAndFeel lnf) {
    if ((lnf != null) && (!lnf.equals(UIManager.getLookAndFeel()))) {
      setLookAndFeel(lnf);
    }
  }

  /**
   * Sets look and feel only if current look and feel is different.
   *
   * @param lnf LookAndFeel
   */
  public static void setNewThemeLookAndFeel(LookAndFeel lnf) {
    setLookAndFeel(lnf);
    controlColor = null;
    controlDarkShadow = null;
    fireLnFChanged(lnf, lnf);
  }

  /**
   * Sets system look and feel.
   */
  public static void setSystemLookAndFeel() {
    try {
      UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
    } catch (Throwable t) { // NOPMD
      if (LibraryConfig.VERBOSE) {
        // CHANGE: use logger
        System.out.println(// NOPMD
                    "System look and feel installation failed."); //$NON-NLS-1$
      }
      if (LibraryConfig.DEBUG) {
        // CHANGE: use logger
        t.printStackTrace(); // NOPMD - debug
      }
      LookAndFeel lnf = UIManager.getLookAndFeel();
      lnf.provideErrorFeedback(null);
    }
  }

  /**
   * Sets cross platform look and feel.
   */
  public static void setCrossPlatformLookAndFeel() {
    try {
      String cplnfcname = UIManager.getCrossPlatformLookAndFeelClassName();
      UIManager.setLookAndFeel(cplnfcname);
    } catch (Throwable t) { // NOPMD
      if (LibraryConfig.VERBOSE) {
        // CHANGE: use logger
        System.out.println(// NOPMD
            "Cross platform look and feel installation failed."); //$NON-NLS-1$
      }
      if (LibraryConfig.DEBUG) {
        // CHANGE: use logger
        t.printStackTrace(); // NOPMD - debug
      }
      LookAndFeel lnf = UIManager.getLookAndFeel();
      lnf.provideErrorFeedback(null);
    }
  }

  /**
   * Sets cross platform look and feel.
   */
  public static void setDefaultLookAndFeel() {
    try {
      UIManager.setLookAndFeel(new SmoothLookAndFeel());
    } catch (Throwable t) { // NOPMD
      if (LibraryConfig.VERBOSE) {
        // CHANGE: use logger
        System.out.println(// NOPMD
                   "Default look and feel installation failed."); //$NON-NLS-1$
      }
      if (LibraryConfig.DEBUG) {
        // CHANGE: use logger
        t.printStackTrace(); // NOPMD - debug
      }
      LookAndFeel lnf = UIManager.getLookAndFeel();
      lnf.provideErrorFeedback(null);
    }
  }

  /**
   * Returns control color.
   *
   * @return control color
   */
  public static Color getControlColor() {
    if (controlColor == null) { // NOPMD - not thread safe
      controlColor = UIManager.getColor("control"); //$NON-NLS-1$
    }
    return controlColor;
  }

  /**
   * Returns control dark shadow.
   *
   * @return control dark shadow
   */
  public static Color getControlDarkShadow() {
    if (controlDarkShadow == null) { // NOPMD - not thread safe
      controlDarkShadow = UIManager.getColor("controlDkShadow"); //$NON-NLS-1$
    }
    return controlDarkShadow;
  }

  /**
   * Returns ComponentUI for the specified component.
   *
   * @param target the JComponent
   *
   * @return ComponentUI for the specified target
   */
  public static ComponentUI getComponentUI(JComponent target) {
    if (target != null) {
      UIDefaults uidefs = UIManager.getDefaults();
      Class c = uidefs.getUIClass(target.getUIClassID());
      if (c != null) {
        try {
          Object[] createUIArgs =
                           (Object[])LookAndFeelManager.CREATE_UI_ARGS.clone();
          Method m = (Method)UI_CACHE.get(c);
          if (m == null) {
            m = c.getMethod("createUI", CREATE_UI_TYPES); //$NON-NLS-1$
            if (m != null) {
              UI_CACHE.put(c, m);
              createUIArgs[0] = target;
              return (ComponentUI)m.invoke(null, createUIArgs);
            }
          } else {
            createUIArgs[0] = target;
            return (ComponentUI)m.invoke(null, createUIArgs);
          }
        } catch (Throwable t) { // NOPMD
          // empty catch body
        }
      }
    }
    return null;
  }

  /**
   * Returns <tt>true</tt> current lnf is Metal.
   *
   * @return <tt>true</tt> current lnf is Metal
   */
  public static boolean isMetalLookAndFeel() {
    return metalLnF;
  }

  /**
   * Returns <tt>true</tt> if WinXP theme is active.
   *
   * @return <tt>true</tt> if WinXP theme is active
   */
  public static boolean isWinXPThemed() {
    return winXPThemed;
  }

/*^**************************************************************************^*/
/*- PropertyChangeListener Implementation.                                   -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  public void propertyChange(PropertyChangeEvent evt) {
    if ("lookAndFeel".equals(evt.getPropertyName())) { //$NON-NLS-1$
      controlColor = null; // FB: OK - there is only one object.
      controlDarkShadow = null; // FB: OK - there is only one object.
      LookAndFeel lnf = (LookAndFeel)evt.getNewValue();
      installLookAndFeel(lnf.getName(), lnf.getClass().getName());
      detectLnF(lnf);
      fireLnFChanged((LookAndFeel)evt.getOldValue(),
                     (LookAndFeel)evt.getNewValue());
    }
  }

/*^**************************************************************************^*/
/*- Private Static Methods.                                                  -*/
/*^**************************************************************************^*/

  private static void grabInstalledLnFs() {
    UIManager.LookAndFeelInfo[] lnfs = UIManager.getInstalledLookAndFeels();
    boolean fwcl = LibraryInfo.IS_JAVA_15_OR_HIGHER &&
                   LibraryInfo.OS_WINDOWS && !winXPThemed;
    for (int i = lnfs.length - 1; i >= 0; i--) {
      try {
        Class lnfClass = Class.forName(lnfs[i].getClassName(), true,
                                       ClassLoader.getSystemClassLoader());
        LookAndFeel lnf = (LookAndFeel)lnfClass.newInstance();
        if (lnf.isSupportedLookAndFeel()) {
          String lnfn = lnfs[i].getName();
          if (!fwcl || !"Windows Classic".equals(lnfn)) { //$NON-NLS-1$
            INSTALLED_LOOK_AND_FEELS.put(lnfn, lnfs[i].getClassName());
          }
        }
      } catch (Throwable t) { // NOPMD
        // empty catch body
      }
    }
  }

  private static boolean winXPThemeActive() {
    Toolkit dtk = Toolkit.getDefaultToolkit();
    Boolean themeActive =
      (Boolean)dtk.getDesktopProperty("win.xpstyle.themeActive"); //$NON-NLS-1$
    if (themeActive == null) {
      return false;
    }
    if (themeActive.booleanValue() &&
        (System.getProperty("swing.noxp") == null)) { //$NON-NLS-1$
      return true;
    }
    return false;
  }

  private static void detectLnF(LookAndFeel lnf) {
    Class lnfClass = lnf.getClass();
    if (lnfClass == MetalLookAndFeel.class) {
      metalLnF = true;
      winXPThemed = false;
    } else {
      metalLnF = false;
      if (lnfClass == windowLnFClass) {  // NOPMD
        winXPThemed = winXPThemeActive();
        if (winXPThemed) { // NOPMD - not thread safe
          try {
            Boolean b = (Boolean)isClassicWindows.invoke(lnfClass, null);
            winXPThemed = !b.booleanValue();
          } catch (Throwable t) { // NOPMD
            winXPThemed = false;
          }
        }
      } else {
        winXPThemed = false;
      }
    }
  }

  private static void fireLnFChanged(LookAndFeel oldLnF, LookAndFeel newLnF) {
    boolean hasnull = false;
    for (int i = LOOK_AND_FEEL_LISTENERS.size() - 1; i >= 0; i--) {
      LookAndFeelChangeListener lnfl =
                     (LookAndFeelChangeListener)LOOK_AND_FEEL_LISTENERS.get(i);
      if (lnfl == null) {
        hasnull = true;
      } else {
        lnfl.lookAndFeelChanged(oldLnF, newLnF);
      }
    }
    if (hasnull) {
      LOOK_AND_FEEL_LISTENERS.expungeStaleReferences();
    }
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-19 22:11:16 Slobodan
   Initial version

*****************************************************************************^*/
