/*
 * @(#)BasicWxStatusBarBorder.java  1.0.0 / 2008-05-22
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.swing.plaf.basic;

import java.awt.Color;
import java.awt.Component;
import java.awt.Graphics;
import java.awt.Insets;

import javax.swing.LookAndFeel;
import javax.swing.border.AbstractBorder;
import javax.swing.plaf.UIResource;

import net.wrka.swing.graphics.GraphicUtils;
import net.wrka.swing.plaf.LookAndFeelChangeListener;
import net.wrka.swing.plaf.LookAndFeelManager;

/**
 * Basic Status Bar Border.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public class BasicWxStatusBarBorder extends AbstractBorder
                                    implements UIResource,
                                               LookAndFeelChangeListener {

/*^**************************************************************************^*/
/*- Private Static Final Fields.                                             -*/
/*^**************************************************************************^*/

  private static final int R_MARGIN = 13;
  private static final Insets INSETS = new Insets(1, 1, 0, R_MARGIN + 5);

/*^**************************************************************************^*/
/*- Private Fields.                                                          -*/
/*^**************************************************************************^*/

  private Color highlight;
  private Color shadow;
  private Color refShadow;
  private Color refDarkShadow;

/*^**************************************************************************^*/
/*- Public Constructor.                                                      -*/
/*^**************************************************************************^*/

  /**
   * Constructs new BasicWxStatusBarBorder object.
   */
  public BasicWxStatusBarBorder() {
    super();
    LookAndFeelManager.addLnFListener(this);
  }

/*^**************************************************************************^*/
/*- Overridden Methods.                                                      -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  public boolean isBorderOpaque() {
    return true;
  }

  /**
   * {@inheritDoc}
   */
  public Insets getBorderInsets(Component c) {
    return (Insets)INSETS.clone();
  }

  /**
   * {@inheritDoc}
   */
  public Insets getBorderInsets(Component c, Insets insets) {
    insets.left = INSETS.left;
    insets.top = INSETS.top;
    insets.bottom = INSETS.bottom;
    insets.right = INSETS.right;
    return insets;
  }

  /**
   * {@inheritDoc}
   */
  public void paintBorder(Component c, Graphics g, int x, int y, int w, int h) {
    if (LookAndFeelManager.isMetalLookAndFeel()) {
      paintMetalBorder(c, g, x, y, w, h);
    } else if (LookAndFeelManager.isWinXPThemed()) {
      paintWinXPBorder(c, g, x, y, w, h);
    } else {
      paintBasicBorder(c, g, x, y, w, h);
    }
  }

/*^**************************************************************************^*/
/*- LookAndFeelChangeListener Implementation.                                -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  public void lookAndFeelChanged(LookAndFeel oldLnF, LookAndFeel newLnF) {
    highlight = null;
    shadow = null;
    refShadow = null;
    refDarkShadow = null;
  }

/*^**************************************************************************^*/
/*- Protected Methods.                                                       -*/
/*^**************************************************************************^*/

  /**
   * Returns shadow color.
   *
   * @param c the color
   *
   * @return shadow color
   */
  protected Color getShadowColor(Component c) {
    if (shadow == null) {
      shadow = c.getBackground().darker();
    }
    return shadow;
  }

  /**
   * Returns highlight color.
   *
   * @param c the component
   *
   * @return highlight color
   */
  protected Color getHighlightColor(Component c) {
    if (highlight == null) {
      highlight = c.getBackground().brighter();
    }
    return highlight;
  }

  /**
   * Returns reflected shadow color.
   *
   * @param c the component
   *
   * @return reflected shadow color
   */
  protected Color getRefalctionShadow(Component c) {
    if (refShadow == null) {
      refShadow = GraphicUtils.darkerColor(c.getBackground(), 0.1f);
    }
    return refShadow;
  }

  /**
   * Returns reflected dark shadow color.
   *
   * @param c the component
   *
   * @return reflected dark shadow color
   */
  protected Color getRefalctionDarkShadow(Component c) {
    if (refDarkShadow == null) {
      refDarkShadow = GraphicUtils.darkerColor(c.getBackground(), 0.17f);
    }
    return refDarkShadow;
  }

  /**
   * Paints basic border.
   *
   * @param c the component
   * @param g the graphic
   * @param x the x coordinate
   * @param y the y coordinate
   * @param w the width
   * @param h the height
   */
  protected void paintBasicBorder(Component c, Graphics g, int x, int y, int w,
                                  int h) {
    Color oldColor = g.getColor();
    int wm1 = w - 1;
    int hm1 = h - 1;
    int hmrm = h - R_MARGIN;
    int wmrm = w - R_MARGIN;
    g.translate(x, y);
    g.setColor(getHighlightColor(c));
    g.drawLine(wm1, hmrm, wmrm, hm1);
    g.drawLine(wm1, hmrm + 4, wmrm + 4, hm1);
    g.drawLine(wm1, hmrm + 8, wmrm + 8, hm1);
    g.setColor(getShadowColor(c));
    g.drawLine(wm1, hmrm + 1, wmrm + 1, hm1);
    g.drawLine(wm1, hmrm + 5, wmrm + 5, hm1);
    g.drawLine(wm1, hmrm + 9, wmrm + 9, hm1);
    g.drawLine(wm1, hmrm + 2, wmrm + 2, hm1);
    g.drawLine(wm1, hmrm + 6, wmrm + 6, hm1);
    g.drawLine(wm1, hmrm + 10, wmrm + 10, hm1);
    g.translate(-x, -y);
    g.setColor(oldColor);
  }

  /**
   * Paints metal border.
   *
   * @param c the component
   * @param g the graphic
   * @param x the x coordinate
   * @param y the y coordinate
   * @param w the width
   * @param h the height
   */
  protected void paintMetalBorder(Component c, Graphics g, int x, int y, int w,
                                  int h) {
    Color oldColor = g.getColor();
    int wmrm = w - R_MARGIN - 1;
    int hmrm = h - R_MARGIN - 1;
    g.translate(x, y);
    g.setColor(getHighlightColor(c));
    int k = 0;
    for (int i = w - 2; i >= wmrm; i -= 4) {
      for (int j = hmrm + k; j < h; j += 4) {
        g.fillRect(i, j, 1, 1);
        g.fillRect(i + 2, j + 2, 1, 1);
      }
      k += 4;
    }
    g.setColor(getShadowColor(c));
    k = 1;
    for (int i = w - 1; i > wmrm; i -= 4) {
      for (int j = hmrm + k; j < h; j += 4) {
        g.fillRect(i, j, 1, 1);
        g.fillRect(i + 2, j + 2, 1, 1);
      }
      k += 4;
    }
    g.translate(-x, -y);
    g.setColor(oldColor);
  }

  /**
   * Paints WinXP border.
   *
   * @param c the component
   * @param g the graphic
   * @param x the x coordinate
   * @param y the y coordinate
   * @param w the width
   * @param h the height
   */
  protected void paintWinXPBorder(Component c, Graphics g, int x, int y, int w,
                                  int h) {
    Color oldColor = g.getColor();
    int hmrm = h - R_MARGIN;
    int wmrm = w - R_MARGIN;
    int wmrmp4 = wmrm + 4;
    int wmrmp5 = wmrm + 5;
    int wmrmp8 = wmrm + 8;
    int wmrmp9 = wmrm + 9;
    int hmrmp5 = hmrm + 5;
    int hmrmp6 = hmrm + 6;
    int hmrmp9 = hmrm + 9;
    int hmrmp10 = hmrm + 10;
    g.translate(x, y);
    g.setColor(getRefalctionDarkShadow(c));
    g.fillRect(0, 0, w, 1);
    g.setColor(getRefalctionShadow(c));
    g.fillRect(0, h - 1, w, 1);
    g.setColor(getHighlightColor(c));
    g.fillRect(wmrm + 1, hmrmp10, 2, 2);
    g.fillRect(wmrmp5, hmrmp10, 2, 2);
    g.fillRect(wmrmp5, hmrmp6, 2, 2);
    g.fillRect(wmrmp9, hmrmp10, 2, 2);
    g.fillRect(wmrmp9, hmrmp6, 2, 2);
    g.fillRect(wmrmp9, hmrm + 2, 2, 2);
    g.setColor(getShadowColor(c));
    g.fillRect(wmrm, hmrmp9, 2, 2);
    g.fillRect(wmrmp4, hmrmp9, 2, 2);
    g.fillRect(wmrmp4, hmrmp5, 2, 2);
    g.fillRect(wmrmp8, hmrmp9, 2, 2);
    g.fillRect(wmrmp8, hmrmp5, 2, 2);
    g.fillRect(wmrmp8, hmrm + 1, 2, 2);
    g.translate(-x, -y);
    g.setColor(oldColor);
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-22 20:56:33 Slobodan
   Initial version

*****************************************************************************^*/
