/*
 * @(#)MetalThemeManager.java  1.0.0 / 2008-05-19
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.swing.plaf.metal;

import javax.swing.plaf.metal.DefaultMetalTheme;
import javax.swing.plaf.metal.MetalLookAndFeel;
import javax.swing.plaf.metal.MetalTheme;

import net.wrka.LibraryInfo;
import net.wrka.swing.plaf.LookAndFeelManager;
import net.wrka.swing.plaf.metal.themes.AquaTheme;
import net.wrka.swing.plaf.metal.themes.EmeraldTheme;
import net.wrka.swing.plaf.metal.themes.KhakiMetalTheme;
import net.wrka.swing.plaf.metal.themes.RubyTheme;
import net.wrka.swing.plaf.metal.themes.TinyTheme;
import net.wrka.util.SimpleHashMap;

/**
 * Metal Theme Manager.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public final class MetalThemeManager {

/*^**************************************************************************^*/
/*- Private Static Fields.                                                   -*/
/*^**************************************************************************^*/

  private static final Class OCEAN_THEME;
  private static final SimpleHashMap METAL_THEMES_HASH;
  private static MetalTheme[] metalThemes;
  private static int count;

/*^**************************************************************************^*/
/*- Static Initialization.                                                   -*/
/*^**************************************************************************^*/

  static {
    Class oceanThemeClass = null;
    if (LibraryInfo.IS_JAVA_15_OR_HIGHER) {
      try {
        oceanThemeClass = Class.forName(
                            "javax.swing.plaf.metal.OceanTheme"); //$NON-NLS-1$
      } catch (Throwable t) { // NOPMD
        // empty catch block
      }
    }
    if (oceanThemeClass == null) {
      OCEAN_THEME = null;
    } else {
      OCEAN_THEME = oceanThemeClass;
    }
    metalThemes = craeteMetalThemes();
    count = metalThemes.length;
    METAL_THEMES_HASH = new SimpleHashMap(count);
    for (int i = 0; i < count; i++) {
      MetalTheme metalTheme = metalThemes[i];
      METAL_THEMES_HASH.put(metalTheme.getName(), metalTheme);
    }
  }

/*^**************************************************************************^*/
/*- Private Constructor.                                                     -*/
/*^**************************************************************************^*/

  /**
   * Class has no instances.
   */
  private MetalThemeManager() {
    // empty constructor body
  }

/*^**************************************************************************^*/
/*- Public Methods.                                                          -*/
/*^**************************************************************************^*/

  /**
   * Returns default theme.
   *
   * @return default theme
   */
  public static MetalTheme getDefaultTheme() {
    return metalThemes[0];
  }

  /**
   * Returns the number of supported themes.
   *
   * @return the number of supported themes
   */
  public static int getThemeCount() {
    return count;
  }

  /**
   * Returns theme at specified index.
   *
   * @param index the theme index
   *
   * @return theme at specified index
   *
   * @throws IllegalArgumentException if index is invalid
   */
  public static MetalTheme getTheme(int index) {
    if ((index < 0) || (index >= count)) {
      throw new IllegalArgumentException("Invalid theme index"); //$NON-NLS-1$
    }
    return metalThemes[index];
  }

  /**
   * Returns theme specified by the name.
   *
   * @param name the theme name
   *
   * @return theme specified by the name
   */
  public static MetalTheme getTheme(String name) {
    return (MetalTheme)METAL_THEMES_HASH.get(name);
  }

  /**
   * Adds new theme.
   *
   * @param theme the theme to be added
   */
  public static void addTheme(MetalTheme theme) {
    if (theme != null) {
      for (int i = 0; i < count; i++) {
        if (metalThemes[i].equals(theme)) {
          return;
        }
      }
      ensureCapacity(count + 1);
      metalThemes[count++] = theme;
      METAL_THEMES_HASH.put(theme.getName(), theme);
    }
  }

  /**
   * Removes specified theme.
   *
   * @param theme the theme to be removed
   */
  public static void removeTheme(MetalTheme theme) {
    if (theme != null) {
      for (int i = 0; i < count; i++) {
        if (metalThemes[i].equals(theme)) {
          removeTheme(i);
          return;
        }
      }
    }
  }

  /**
   * Removes theme at specified index.
   *
   * @param index the theme index
   */
  public static void removeTheme(int index) {
    if ((index >= metalThemes.length) && (index < count)) {
      METAL_THEMES_HASH.remove(metalThemes[index]);
      int numMoved = count - index - 1;
      if (numMoved > 0) {
        System.arraycopy(metalThemes, index + 1, metalThemes, index,
                         numMoved);
      }
      metalThemes[--count] = null;
    }
  }

  /**
   * Returns current metal theme.
   *
   * @return current metal theme
   */
  public static MetalTheme getCurrentMetalTheme() {
    MetalTheme metalTheme = null;
    try {
      sun.awt.AppContext context = sun.awt.AppContext.getAppContext();
      metalTheme = (MetalTheme)context.get("currentMetalTheme"); //$NON-NLS-1$
    } catch (Throwable t) { // NOPMD
      // empty catch block
    }
    if (metalTheme == null) {
      metalTheme = metalThemes[0];
    }
    return metalTheme;
  }

  // Alternative  - DO NOT REMOVE
  /*
   * Returns current metal theme.
   *
   * @return current metal theme
   *
  public static MetalTheme getCurrentMetalTheme() {
    MetalTheme metalTheme = null;
    try {
      Class classAppContext = Class.forName("sun.awt.AppContext"); //$NON-NLS-1$
      Method methodGetAppContext =
                        classAppContext.getMethod("getAppContext", //$NON-NLS-1$
                                                  null);
      Object context = methodGetAppContext.invoke(null, null);
      Method methodGet = classAppContext.getMethod("get", //$NON-NLS-1$
                                                   new Class[] {Object.class});
      metalTheme = (MetalTheme)methodGet.invoke(context,
                                           new String[] {"currentMetalTheme"});
    } catch (Throwable t) {
    }
    if (metalTheme == null) {
      metalTheme = metalThemes[0];
    }
    return metalTheme;
  }*/

  /**
   * Returns <tt>true</tt> if current theme is ocean theme.
   *
   * @return <tt>true</tt> if current theme is ocean theme
   */
  public static boolean isCurrentOcean() {
    if ((OCEAN_THEME != null) &&
        (LookAndFeelManager.getCurrentLookAndFeel() instanceof
                                                           MetalLookAndFeel)) {
      try {
        sun.awt.AppContext context = sun.awt.AppContext.getAppContext();
        MetalTheme metalTheme =
                    (MetalTheme)context.get("currentMetalTheme"); //$NON-NLS-1$
        return ((metalTheme != null) &&
                (OCEAN_THEME.isAssignableFrom(metalTheme.getClass())));
      } catch (Throwable t) { // NOPMD
        // empty catch block
      }
    }
    return false;
  }

/*^**************************************************************************^*/
/*- Private Static Method.                                                   -*/
/*^**************************************************************************^*/

  private static void ensureCapacity(int minCapacity) {
    int oldCapacity = metalThemes.length;
    if (minCapacity > oldCapacity) {
      MetalTheme[] oldThemes = metalThemes;
      int newCapacity = ((3 * oldCapacity) >> 1) + 1;
      if (newCapacity < minCapacity) {
        newCapacity = minCapacity;
      }
      metalThemes = new MetalTheme[newCapacity];
      System.arraycopy(oldThemes, 0, metalThemes, 0, count);
    }
  }

  private static MetalTheme[] craeteMetalThemes() {
    MetalTheme[] builtInThemes = craeteBuiltInMetalThemes();
    MetalTheme oceanTheme = null;
    if (OCEAN_THEME != null) {
      try {
        oceanTheme = (MetalTheme)OCEAN_THEME.newInstance();
      } catch (Throwable t) { // NOPMD
        // empty catch block
      }
    }
    if (oceanTheme == null) {
      return builtInThemes;
    }
    MetalTheme[] metalThemes = new MetalTheme[builtInThemes.length + 1];
    metalThemes[0] = oceanTheme;
    System.arraycopy(builtInThemes, 0, metalThemes, 1, builtInThemes.length);
    return metalThemes;
  }

  private static MetalTheme[] craeteBuiltInMetalThemes() {
    MetalTheme[] metalThemes = new MetalTheme[] {
      new DefaultMetalTheme(),
      new AquaTheme(),
      new KhakiMetalTheme(),
      new EmeraldTheme(),
      new RubyTheme(),
      new TinyTheme()
    };
    return metalThemes;
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-19 22:18:42 Slobodan
   Initial version

*****************************************************************************^*/
