/*
 * @(#)SmoothBorders.java  1.0.0 / 2008-05-25
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.swing.plaf.smooth;

import java.awt.Component;
import java.awt.Container;
import java.awt.Graphics;
import java.awt.Insets;

import javax.swing.AbstractButton;
import javax.swing.ButtonModel;
import javax.swing.JButton;
import javax.swing.JToggleButton;
import javax.swing.JToolBar;
import javax.swing.border.AbstractBorder;
import javax.swing.border.Border;
import javax.swing.border.EmptyBorder;
import javax.swing.plaf.UIResource;
import javax.swing.text.JTextComponent;

import net.wrka.swing.graphics.GraphicUtils;

/**
 * Smooth Borders.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public final class SmoothBorders {

/*^**************************************************************************^*/
/*- Private Static Fields.                                                   -*/
/*^**************************************************************************^*/

  private static Border popupMenuBorder;
  private static Border buttonOvalBorder;
  private static Border buttonLineBorder;
  private static Border buttonRolloverBorder;
  private static Border toggleButtonOvalBorder;
  private static Border toggleButtonRolloverBorder;
  private static Border tableHeaderBorder;
  private static Border textFieldBorder;
  private static Border internalFrameBorder;
  private static Border toolBarBorder;
  private static Border statusBarPanelBorder;
  private static Border menuItemBorder;

/*^**************************************************************************^*/
/*- Private Constructor.                                                     -*/
/*^**************************************************************************^*/

  /**
   * Class has no instances.
   */
  private SmoothBorders() {
    // empty constructor body
  }

/*^**************************************************************************^*/
/*- Public Static Methods.                                                   -*/
/*^**************************************************************************^*/

  /**
   * Returns button oval border.
   *
   * @return button oval border
   */
  public static Border getButtonOvalBorder() {
    if (buttonOvalBorder == null) { // NOPMD
      buttonOvalBorder = new ButtonOvalBorder();
    }
    return buttonOvalBorder;
  }

  /**
   * Returns button line border.
   *
   * @return button line border
   */
  public static Border getButtonLineBorder() {
    if (buttonLineBorder == null) { // NOPMD
      buttonLineBorder = new ButtonLineBorder();
    }
    return buttonLineBorder;
  }

  /**
   * Returns button rollover border.
   *
   * @return button rollover border
   */
  public static Border getButtonRolloverBorder() {
    if (buttonRolloverBorder == null) { // NOPMD
      buttonRolloverBorder = new ButtonRolloverBorder();
    }
    return buttonRolloverBorder;
  }

  /**
   * Returns toggle button oval border.
   *
   * @return toggle button oval border
   */
  public static Border getToggleButtonOvalBorder() {
    if (toggleButtonOvalBorder == null) { // NOPMD
      toggleButtonOvalBorder = new ToggleButtonOvalBorder();
    }
    return toggleButtonOvalBorder;
  }

  /**
   * Returns toggle button rollover border.
   *
   * @return toggle button rollover border
   */
  public static Border getToggleButtonRolloverBorder() {
    if (toggleButtonRolloverBorder == null) { // NOPMD
      toggleButtonRolloverBorder = new ToggleButtonRolloverBorder();
    }
    return toggleButtonRolloverBorder;
  }

  /**
   * Returns table header border.
   *
   * @return table header border
   */
  public static Border getTableHeaderBorder() {
    if (tableHeaderBorder == null) { // NOPMD
      tableHeaderBorder = new TableHeaderBorder();
    }
    return tableHeaderBorder;
  }

  /**
   * Returns text field border.
   *
   * @return text field border
   */
  public static Border getTextFieldBorder() {
    if (textFieldBorder == null) { // NOPMD
      textFieldBorder = new TextFieldBorder();
    }
    return textFieldBorder;
  }

  /**
   * Returns popup menu border.
   *
   * @return popup menu border
   */
  public static Border getPopupMenuBorder() {
    if (popupMenuBorder == null) { // NOPMD
      popupMenuBorder = new PopupMenuBorder();
    }
    return popupMenuBorder;
  }

  /**
   * Returns tool bar border.
   *
   * @return tool bar border
   */
  public static Border getToolBarBorder() {
    if (toolBarBorder == null) { // NOPMD
      toolBarBorder = new ToolBarBorder();
    }
    return toolBarBorder;
  }

  /**
   * Returns menu item border.
   *
   * @return menu item border
   */
  public static Border getMenuItemBorder() {
    if (menuItemBorder == null) { // NOPMD
      menuItemBorder = new EmptyBorderUIResource(3, 1, 3, 1);
    }
    return menuItemBorder;
  }

  /**
   * Returns internal frame border.
   *
   * @return internal frame border
   */
  public static Border getInternalFrameBorder() {
    if (internalFrameBorder == null) { // NOPMD
      internalFrameBorder = new InternalFrameBorder();
    }
    return internalFrameBorder;
  }

  /**
   * Returns status bar panel border.
   *
   * @return status bar panel border
   */
  public static Border getStatusBarPanelBorder() {
    if (statusBarPanelBorder == null) { // NOPMD
      statusBarPanelBorder = new WxStatusBarPanelBorder();
    }
    return statusBarPanelBorder;
  }

/*^**************************************************************************^*/
/*- Inner Classes.                                                           -*/
/*^**************************************************************************^*/

  /**
   * Button Line Border.
   */
  public static final class ButtonLineBorder extends AbstractBorder
                                             implements UIResource {
    private static final Insets INSETS = new Insets(3, 3, 3, 3);
    /**
     * {@inheritDoc}
     */
    public Insets getBorderInsets(Component c) {
      Insets insets = (Insets)INSETS.clone();
      Insets margin = ((AbstractButton)c).getMargin();
      if (margin != null) {
        insets.left = insets.left + margin.left;
        insets.top = insets.top + margin.top;
        insets.right = insets.right + margin.right;
        insets.bottom = insets.bottom + margin.bottom;
      }
      return insets;
    }
    /**
     * {@inheritDoc}
     */
    public Insets getBorderInsets(Component c, Insets insets) {
      insets.top = INSETS.top;
      insets.left = INSETS.left;
      insets.bottom = INSETS.bottom;
      insets.right = INSETS.right;
      Insets margin = ((AbstractButton)c).getMargin();
      if (margin != null) {
        insets.left = insets.left + margin.left;
        insets.top = insets.top + margin.top;
        insets.right = insets.right + margin.right;
        insets.bottom = insets.bottom + margin.bottom;
      }
      return insets;
    }
    /**
     * {@inheritDoc}
     */
    public void paintBorder(Component c, Graphics g, int x, int y, int w,
                            int h) {
      JButton button = (JButton)c;
      if (button.isEnabled()) {
        g.setColor(SmoothLookAndFeel.getControlDarkShadow());
      } else {
        g.setColor(SmoothLookAndFeel.getControlShadow());
      }
      g.drawRect(x, y, w - 1, h - 1);
      if (button.isDefaultButton()) {
        g.drawRect(x + 1, y + 1, w - 3, h - 3);
      }
    }
  }

  /**
   * Button Oval Border.
   */
  public static class ButtonOvalBorder extends AbstractBorder
                                       implements UIResource {
    private static final Insets INSETS = new Insets(3, 3, 3, 3);
    /**
     * {@inheritDoc}
     */
    public Insets getBorderInsets(Component c) {
      Insets insets = (Insets)INSETS.clone();
      Insets margin = ((AbstractButton)c).getMargin();
      if (margin != null) {
        insets.left = insets.left + margin.left;
        insets.top = insets.top + margin.top;
        insets.right = insets.right + margin.right;
        insets.bottom = insets.bottom + margin.bottom;
      }
      return insets;
    }
    /**
     * {@inheritDoc}
     */
    public Insets getBorderInsets(Component c, Insets insets) {
      insets.top = INSETS.top;
      insets.left = INSETS.left;
      insets.bottom = INSETS.bottom;
      insets.right = INSETS.right;
      Insets margin = ((AbstractButton)c).getMargin();
      if (margin != null) {
        insets.left = insets.left + margin.left;
        insets.top = insets.top + margin.top;
        insets.right = insets.right + margin.right;
        insets.bottom = insets.bottom + margin.bottom;
      }
      return insets;
    }
    /**
     * {@inheritDoc}
     */
    public void paintBorder(Component c, Graphics g, int x, int y, int w,
                            int h) {
      JButton button = (JButton)c;
      Container bp = button.getParent();
      int wm1 = w - 1;
      int wm2 = w - 2;
      int hm1 = h - 1;
      int hm2 = h - 2;
      if (button.isEnabled()) {
        g.setColor(SmoothLookAndFeel.getControlDarkShadow());
      } else {
        g.setColor(SmoothLookAndFeel.getControlShadow());
      }
      GraphicUtils.draw1PixRoundRect(g, x, y, wm1, hm1);
      if (bp == null) {
        g.setColor(button.getBackground());
      } else {
        g.setColor(bp.getBackground());
      }
      g.fillRect(0, 0, 1, 1);
      g.fillRect(0, hm1, 1, 1);
      g.fillRect(wm1, 0, 1, 1);
      g.fillRect(wm1, hm1, 1, 1);
      g.setColor(SmoothLookAndFeel.getControlShadow());
      g.fillRect(0, 1, 2, 1);
      g.fillRect(1, 0, 1, 1);
      g.fillRect(wm2, 1, 2, 1);
      g.fillRect(wm2, 0, 1, 1);
      g.fillRect(0, hm2, 2, 1);
      g.fillRect(1, hm1, 1, 1);
      g.fillRect(wm2, hm2, 2, 1);
      g.fillRect(wm2, hm1, 1, 1);
      if (button.isDefaultButton()) {
        g.drawRect(x + 1, y + 1, w - 3, h - 3);
      }
    }
  }

  /**
   * Toggle Button Oval Border.
   */
  public static class ToggleButtonOvalBorder extends AbstractBorder
                                             implements UIResource {
    private static final Insets INSETS = new Insets(3, 3, 3, 3);
    /**
     * {@inheritDoc}
     */
    public Insets getBorderInsets(Component c) {
      Insets insets = (Insets)INSETS.clone();
      Insets margin = ((AbstractButton)c).getMargin();
      if (margin != null) {
        insets.left = insets.left + margin.left;
        insets.top = insets.top + margin.top;
        insets.right = insets.right + margin.right;
        insets.bottom = insets.bottom + margin.bottom;
      }
      return insets;
    }
    /**
     * {@inheritDoc}
     */
    public Insets getBorderInsets(Component c, Insets insets) {
      insets.top = INSETS.top;
      insets.left = INSETS.left;
      insets.bottom = INSETS.bottom;
      insets.right = INSETS.right;
      Insets margin = ((AbstractButton)c).getMargin();
      if (margin != null) {
        insets.left = insets.left + margin.left;
        insets.top = insets.top + margin.top;
        insets.right = insets.right + margin.right;
        insets.bottom = insets.bottom + margin.bottom;
      }
      return insets;
    }
    /**
     * {@inheritDoc}
     */
    public void paintBorder(Component c, Graphics g, int x, int y, int w,
                            int h) {
      JToggleButton button = (JToggleButton)c;
      Container bp = button.getParent();
      int wm1 = w - 1;
      int wm2 = w - 2;
      int hm1 = h - 1;
      int hm2 = h - 2;
      if (button.isEnabled()) {
        g.setColor(SmoothLookAndFeel.getControlDarkShadow());
      } else {
        g.setColor(SmoothLookAndFeel.getControlShadow());
      }
      GraphicUtils.draw1PixRoundRect(g, x, y, wm1, hm1);
      if (bp == null) {
        g.setColor(button.getBackground());
      } else {
        g.setColor(bp.getBackground());
      }
      g.fillRect(0, 0, 1, 1);
      g.fillRect(0, hm1, 1, 1);
      g.fillRect(wm1, 0, 1, 1);
      g.fillRect(wm1, hm1, 1, 1);
      g.setColor(SmoothLookAndFeel.getControlShadow());
      g.fillRect(0, 1, 2, 1);
      g.fillRect(1, 0, 1, 1);
      g.fillRect(wm2, 1, 2, 1);
      g.fillRect(wm2, 0, 1, 1);
      g.fillRect(0, hm2, 2, 1);
      g.fillRect(1, hm1, 1, 1);
      g.fillRect(wm2, hm2, 2, 1);
      g.fillRect(wm2, hm1, 1, 1);
    }
  }

  /**
   * Button Rollover Border.
   *
   * NOTE: This class cannot implement UIResource!!!
   */
  public static class ButtonRolloverBorder extends AbstractBorder {
    private static final Insets INSETS = new Insets(3, 3, 3, 3);
    /**
     * {@inheritDoc}
     */
    public Insets getBorderInsets(Component c) {
      return (Insets)INSETS.clone();
    }
    /**
     * {@inheritDoc}
     */
    public Insets getBorderInsets(Component c, Insets insets) {
      insets.top = INSETS.top;
      insets.left = INSETS.left;
      insets.bottom = INSETS.bottom;
      insets.right = INSETS.right;
      return insets;
    }
    /**
     * {@inheritDoc}
     */
    public void paintBorder(Component c, Graphics g, int x, int y, int w,
                            int h) {
      AbstractButton b = (AbstractButton)c;
      ButtonModel model = b.getModel();
      if (model.isRollover() && !(model.isPressed() && !model.isArmed())) {
        if (model.isPressed()) {
          g.setColor(SmoothLookAndFeel.getRolloverInnerBorderDarkColor());
        } else {
          g.setColor(SmoothLookAndFeel.getRolloverInnerBorderLightColor());
        }
        g.drawRect(1, 1, w - 3, h - 3);
        g.setColor(SmoothLookAndFeel.getRolloverMaskColor());
        g.fillRect(1, 1, w - 2, h - 2);
        g.setColor(SmoothLookAndFeel.getRolloverOuterBorderColor());
        GraphicUtils.draw1PixRoundRect(g, 0, 0, w - 1, h - 1);
      }
    }
  }

  /**
   * Toggle Button Rollover Border.
   *
   * NOTE: This class cannot implement UIResource!!!
   */
  public static class ToggleButtonRolloverBorder extends AbstractBorder {
    private static final Insets INSETS = new Insets(3, 3, 3, 3);
    /**
     * {@inheritDoc}
     */
    public Insets getBorderInsets(Component c) {
      return (Insets)INSETS.clone();
    }
    /**
     * {@inheritDoc}
     */
    public Insets getBorderInsets(Component c, Insets insets) {
      insets.top = INSETS.top;
      insets.left = INSETS.left;
      insets.bottom = INSETS.bottom;
      insets.right = INSETS.right;
      return insets;
    }
    /**
     * {@inheritDoc}
     */
    public void paintBorder(Component c, Graphics g, int x, int y, int w,
                            int h) {
      AbstractButton b = (AbstractButton)c;
      ButtonModel model = b.getModel();
      if (model.isRollover() || model.isSelected()) {
        if (model.isPressed()) {
          g.setColor(SmoothLookAndFeel.getRolloverInnerBorderDarkColor());
          g.drawRect(1, 1, w - 3, h - 3);
        } else {
          if (!model.isSelected()) {
            g.setColor(SmoothLookAndFeel.getRolloverInnerBorderLightColor());
            g.setColor(SmoothLookAndFeel.getComponentGradientReflectionLight());
            g.drawRect(1, 1, w - 3, h - 3);
          }
        }
        g.setColor(SmoothLookAndFeel.getRolloverMaskColor());
        g.fillRect(1, 1, w - 2, h - 2);
        g.setColor(SmoothLookAndFeel.getRolloverOuterBorderColor());
        GraphicUtils.draw1PixRoundRect(g, 0, 0, w - 1, h - 1);
      }
    }
  }

  /**
   * Table Header Border.
   */
  public static class TableHeaderBorder extends AbstractBorder
                                        implements UIResource {
    private static final Insets INSETS = new Insets(1, 2, 1, 2);
    /**
     * {@inheritDoc}
     */
    public Insets getBorderInsets(Component c) {
      return (Insets)INSETS.clone();
    }
    /**
     * {@inheritDoc}
     */
    public Insets getBorderInsets(Component c, Insets insets) {
      insets.top = INSETS.top;
      insets.left = INSETS.left;
      insets.bottom = INSETS.bottom;
      insets.right = INSETS.right;
      return insets;
    }
    /**
     * {@inheritDoc}
     */
    public void paintBorder(Component c, Graphics g, int x, int y, int w,
                            int h) {
      g.translate(x, y);
      g.setColor(SmoothLookAndFeel.getControlDarkShadow());
      g.drawLine(w - 1, 0, w - 1, h - 1);
      g.drawLine(1, h - 1, w - 1, h - 1);
      g.setColor(SmoothLookAndFeel.getControlHighlight());
      g.drawLine(0, 0, w - 2, 0);
      g.drawLine(0, 0, 0, h - 2);
      g.translate(-x, -y);
    }
  }

  /**
   * Text Field Border.
   */
  public static class TextFieldBorder extends AbstractBorder
                                      implements UIResource {
    private static final Insets INSETS = new Insets(2, 3, 2, 3);
    /**
     * {@inheritDoc}
     */
    public Insets getBorderInsets(Component c) {
      return (Insets)INSETS.clone();
    }
    /**
     * {@inheritDoc}
     */
    public Insets getBorderInsets(Component c, Insets insets) {
      insets.top = INSETS.top;
      insets.left = INSETS.left;
      insets.bottom = INSETS.bottom;
      insets.right = INSETS.right;
      return insets;
    }
    /**
     * {@inheritDoc}
     */
    public void paintBorder(Component c, Graphics g, int x, int y, int w,
                            int h) {
      int wm1 = w - 1;
      int wm2 = w - 2;
      int hm1 = h - 1;
      int hm2 = h - 2;
      g.translate(x, y);
      Container parent = c.getParent();
      if (parent == null) {
        g.setColor(c.getBackground());
      } else {
        g.setColor(parent.getBackground());
      }
      g.fillRect(0, 0, 1, 1);
      g.fillRect(0, hm1, 1, 1);
      g.fillRect(wm1, 0, 1, 1);
      g.fillRect(wm1, hm1, 1, 1);
      g.setColor(SmoothLookAndFeel.getComponentGradientShadowLight());
      g.fillRect(1, 1, 1, 1);
      g.fillRect(1, hm2, 1, 1);
      g.fillRect(wm2, 1, 1, 1);
      g.fillRect(wm2, hm2, 1, 1);
      g.setColor(SmoothLookAndFeel.getControlDarkShadow());
      g.drawLine(1, 0, wm2, 0);
      g.drawLine(0, 1, 0, hm2);
      g.drawLine(1, hm1, wm2, hm1);
      g.drawLine(wm1, 1, wm1, hm2);
      g.translate(-x, -y);
    }
  }

  /**
   * EmptyBorder UIResource.
   */
  public static class EmptyBorderUIResource extends EmptyBorder
                                            implements UIResource {
    /**
     * Constructs new EmptyBorderUIResource object.
     *
     * @param insets the insets
     */
    public EmptyBorderUIResource(Insets insets) {
      super(insets);
    }
    /**
     * Constructs new EmptyBorderUIResource object.
     *
     * @param top the top insets
     * @param left the left insets
     * @param bottom the bottom insets
     * @param right the right insets
     */
    public EmptyBorderUIResource(int top, int left, int bottom, int right) {
      super(top, left, bottom, right);
    }
  }

  /**
   * PopupMenu Border.
   */
  public static class PopupMenuBorder extends AbstractBorder
                                      implements UIResource {
    private static final Insets INSETS = new Insets(2, 1, 2, 1);
    /**
     * {@inheritDoc}
     */
    public Insets getBorderInsets(Component c) {
      return (Insets)INSETS.clone();
    }
    /**
     * {@inheritDoc}
     */
    public Insets getBorderInsets(Component c, Insets insets) {
      insets.top = INSETS.top;
      insets.left = INSETS.left;
      insets.bottom = INSETS.bottom;
      insets.right = INSETS.right;
      return insets;
    }
    /**
     * {@inheritDoc}
     */
    public void paintBorder(Component c, Graphics g, int x, int y, int w,
                            int h) {
      g.translate(x, y);
      g.setColor(SmoothLookAndFeel.getPrimaryControlDarkShadow());
      g.drawRect(0, 0, w - 1, h - 1);
      g.setColor(SmoothLookAndFeel.getMenuItemRightBackground());
      int wm2 = w - 2;
      int hm2 = h - 2;
      g.drawLine(1, 1, wm2, 1);
      g.drawLine(1, hm2, wm2, hm2);
      g.translate(-x, -y);
    }
  }

  /**
   * WxStatusBar Panel Border.
   */
  public static class WxStatusBarPanelBorder extends AbstractBorder
                                             implements UIResource {
    private static final Insets INSETS = new Insets(3, 4, 3, 4);
    /**
     * {@inheritDoc}
     */
    public Insets getBorderInsets(Component c) {
      return (Insets)INSETS.clone();
    }
    /**
     * {@inheritDoc}
     */
    public Insets getBorderInsets(Component c, Insets insets) {
      insets.top = INSETS.top;
      insets.left = INSETS.left;
      insets.bottom = INSETS.bottom;
      insets.right = INSETS.right;
      return insets;
    }
    /**
     * {@inheritDoc}
     */
    public void paintBorder(Component c, Graphics g, int x, int y, int w,
                            int h) {
      g.setColor(SmoothLookAndFeel.getStatusBarHighlightColor());
      GraphicUtils.draw1PixRoundRect(g, x + 1, y + 1, w - 3, h - 3);
    }
  }

  /**
   * WxToolBar Border.
   */
  public static class ToolBarBorder extends AbstractBorder
                                    implements UIResource {
    private static final Insets INSETS = new Insets(2, 2, 2, 2);
    /**
     * {@inheritDoc}
     */
    public Insets getBorderInsets(Component c) {
      Insets insets = (Insets)INSETS.clone();
      JToolBar toolBar = (JToolBar)c;
      if (toolBar.isFloatable()) {
        if (toolBar.getOrientation() == JToolBar.HORIZONTAL) {
          insets.left = 9;
        } else {
          insets.top = 9;
        }
      }
      Insets margin = toolBar.getMargin();
      if (margin != null) {
        insets.left = insets.left + margin.left;
        insets.top = insets.top + margin.top;
        insets.right = insets.right + margin.right;
        insets.bottom = insets.bottom + margin.bottom;
      }
      return insets;
    }
    /**
     * {@inheritDoc}
     */
    public Insets getBorderInsets(Component c, Insets insets) {
      insets.top = INSETS.top;
      insets.left = INSETS.left;
      insets.bottom = INSETS.bottom;
      insets.right = INSETS.right;
      Insets margin = ((JToolBar)c).getMargin();
      if (margin != null) {
        insets.left = insets.left + margin.left;
        insets.top = insets.top + margin.top;
        insets.right = insets.right + margin.right;
        insets.bottom = insets.bottom + margin.bottom;
      }
      return insets;
    }
    /**
     * {@inheritDoc}
     */
    public void paintBorder(Component c, Graphics g, int x, int y, int w,
                            int h) {
      g.translate(x, y);
      if (((JToolBar)c).isFloatable()) {
        if (((JToolBar)c).getOrientation() == JToolBar.HORIZONTAL) {
          h = h - 2;
          for (int i = 2; i < h; i += 3) {
            g.setColor(SmoothLookAndFeel.getComponentGradientReflection());
            g.fillRect(1, i, 5, 1);
            g.setColor(SmoothLookAndFeel.getComponentGradientShadow());
            g.fillRect(2, i + 1, 5, 1);
          }
        } else {
          w = w - 2;
          for (int i = 2; i < w; i += 3) {
            g.setColor(SmoothLookAndFeel.getComponentGradientShadow());
            g.fillRect(i + 1, 2, 1, 5);
            g.setColor(SmoothLookAndFeel.getComponentGradientReflection());
            g.fillRect(i, 1, 1, 5);
          }
        }
      }
      g.translate(-x, -y);
    }
  }

  /**
   * InternalFrame Border.
   */
  public static class InternalFrameBorder extends AbstractBorder
                                          implements UIResource {
    private static final Insets INSETS = new Insets(4, 4, 4, 4);
    /**
     * {@inheritDoc}
     */
    public Insets getBorderInsets(Component c) {
      return (Insets)INSETS.clone();
    }
    /**
     * {@inheritDoc}
     */
    public Insets getBorderInsets(Component c, Insets insets) {
      insets.top = INSETS.top;
      insets.left = INSETS.left;
      insets.bottom = INSETS.bottom;
      insets.right = INSETS.right;
      return insets;
    }
    /**
     * {@inheritDoc}
     */
    public void paintBorder(Component c, Graphics g, int x, int y, int w,
                            int h) {
      g.translate(x, y);
      int wm3 = w - 3;
      int wm4 = w - 4;
      int wm6 = w - 6;
      int hm3 = h - 3;
      int hm4 = h - 4;
      int hm6 = h - 6;
      g.setColor(c.getBackground());
      g.fillRect(3, 0, wm6, 4);
      g.fillRect(3, hm4, wm6, 4);
      g.fillRect(0, 3, 4, hm6);
      g.fillRect(wm4, 3, 4, hm6);
      g.fillRect(1, 1, 2, 2);
      g.fillRect(wm3, 1, 2, 2);
      g.fillRect(1, hm3, 2, 2);
      g.fillRect(wm3, hm3, 2, 2);
      SmoothUtils.drawFastTranReflectGradient(g, true, 2, 0, wm4, 4);
      SmoothUtils.drawFastInvTranShadowGradient(g, true, 2, h - 5, wm4, 5);
      SmoothUtils.drawFastTranReflectGradient(g, false, 0, 2, 4, hm4);
      SmoothUtils.drawFastInvTranShadowGradient(g, false, w - 5, 2, 5, hm4);
      g.translate(-x, -y);
    }
  }

  /**
   * Margin Border.
   */
  public static class WxMarginBorder extends AbstractBorder
                                     implements UIResource {
    private static final Insets NULL_INSETS = new Insets(0, 0, 0, 0);
    /**
     * {@inheritDoc}
     */
    public Insets getBorderInsets(Component c) {
      Insets margin = null;
      if (c instanceof AbstractButton) {
        AbstractButton b = (AbstractButton)c;
        margin = b.getMargin();
      } else if (c instanceof JToolBar) {
        JToolBar t = (JToolBar)c;
        margin = t.getMargin();
      } else if (c instanceof JTextComponent) {
        JTextComponent t = (JTextComponent)c;
        margin = t.getMargin();
      }
      if (margin != null) {
        return margin;
      }
      return (Insets)NULL_INSETS.clone();
    }
    /**
     * {@inheritDoc}
     */
    public Insets getBorderInsets(Component c, Insets insets) {
      Insets margin = null;
      if (c instanceof AbstractButton) {
        AbstractButton b = (AbstractButton)c;
        margin = b.getMargin();
      } else if (c instanceof JToolBar) {
        JToolBar t = (JToolBar)c;
        margin = t.getMargin();
      } else if (c instanceof JTextComponent) {
        JTextComponent t = (JTextComponent)c;
        margin = t.getMargin();
      }
      if (margin == null) {
        margin = (Insets)NULL_INSETS.clone();
      }
      insets.left = margin.left;
      insets.top = margin.top;
      insets.right = margin.right;
      insets.bottom = margin.bottom;
      return insets;
    }
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-25 12:20:34 Slobodan
   Initial version

*****************************************************************************^*/
