/*
 * @(#)SmoothButtonListener.java  1.0.0 / 2008-05-25
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.swing.plaf.smooth;

import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import javax.swing.AbstractAction;
import javax.swing.AbstractButton;
import javax.swing.ActionMap;
import javax.swing.ButtonModel;
import javax.swing.InputMap;
import javax.swing.JComponent;
import javax.swing.KeyStroke;
import javax.swing.SwingUtilities;
import javax.swing.UIManager;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.plaf.ActionMapUIResource;
import javax.swing.plaf.ButtonUI;
import javax.swing.plaf.ComponentInputMapUIResource;
import javax.swing.plaf.basic.BasicButtonUI;
import javax.swing.plaf.basic.BasicHTML;

/**
 * Button listener.
 * <BR>The code is based on Sun's javax.swing.plaf.basic.BasicButtonListner.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public class SmoothButtonListener implements MouseListener,
                                             MouseMotionListener,
                                             FocusListener, ChangeListener,
                                             PropertyChangeListener {

/*^**************************************************************************^*/
/*- Package-Private Static Final Fields.                                     -*/
/*^**************************************************************************^*/

  static final String FONT_PROPERTY = "font"; //$NON-NLS-1$
  static final String FOREGROUND_PROPERTY = "foreground"; //$NON-NLS-1$
  static final String WIM_PROPERTY = "WndInpMap"; //$NON-NLS-1$
  static final String BUTTON_FOREGROUND = "Button.focusInputMap"; //$NON-NLS-1$
  static final String BPRESSED = "pressed"; //$NON-NLS-1$
  static final String BRELEASED = "released"; //$NON-NLS-1$

/*^**************************************************************************^*/
/*- Package-Private Fields.                                                  -*/
/*^**************************************************************************^*/

  long lastPressedTimestamp;
  boolean shouldDiscardRelease;

/*^**************************************************************************^*/
/*- Protected Constructor.                                                   -*/
/*^**************************************************************************^*/

  /**
   * Constructs new SmoothButtonListener object.
   */
  public SmoothButtonListener() {
    lastPressedTimestamp = -1;
    shouldDiscardRelease = false;
  }

/*^**************************************************************************^*/
/*- Protected Methods.                                                       -*/
/*^**************************************************************************^*/

  /**
   * Checks opacity.
   *
   * @param b the abstract button
   */
  protected void checkOpacity(AbstractButton b) {
    b.setOpaque(b.isContentAreaFilled());
  }

  /**
   * Installs keyboard actions.
   *
   * @param c the component
   */
  protected void installKeyboardActions(JComponent c) {
    AbstractButton b = (AbstractButton)c;
    updateMnemonicBinding(b);
    ActionMap map = getActionMap(b);
    SwingUtilities.replaceUIActionMap(c, map);
    InputMap km = getInputMap(JComponent.WHEN_FOCUSED, c);
    SwingUtilities.replaceUIInputMap(c, JComponent.WHEN_FOCUSED, km);
  }

  /**
   * Uninstalls keyboard actions.
   *
   * @param c the component
   */
  public void uninstallKeyboardActions(JComponent c) {
    if (c.getClientProperty(WIM_PROPERTY) != null) {
      SwingUtilities.replaceUIInputMap(c, JComponent.WHEN_IN_FOCUSED_WINDOW,
                                       null);
      c.putClientProperty(WIM_PROPERTY, null);
    }
    SwingUtilities.replaceUIInputMap(c, JComponent.WHEN_FOCUSED, null);
    SwingUtilities.replaceUIActionMap(c, null);
  }

  /**
   * Returns action map.
   *
   * @param b the abstract button
   *
   * @return action map
   */
  protected ActionMap getActionMap(AbstractButton b) {
    return createActionMap(b);
  }

  /**
   * Returns input map.
   *
   * @param condition the condition
   * @param c the component
   *
   * @return input map
   */
  protected InputMap getInputMap(int condition, JComponent c) {
    if (condition == JComponent.WHEN_FOCUSED) {
      ButtonUI ui = ((AbstractButton)c).getUI();
      if ((ui != null) && (ui instanceof BasicButtonUI)) {
        return (InputMap)UIManager.get(BUTTON_FOREGROUND);
      }
    }
    return null;
  }

  /**
   * Creates action map.
   *
   * @param b the button
   *
   * @return action map
   */
  protected ActionMap createActionMap(AbstractButton b) {
    ActionMap retValue = new ActionMapUIResource();
    retValue.put(BPRESSED, new PressedAction(b));
    retValue.put(BRELEASED, new ReleasedAction(b));
    return retValue;
  }

  /**
   * Updates mnemonic binding.
   *
   * @param b the button
   */
  protected void updateMnemonicBinding(AbstractButton b) {
    int m = b.getMnemonic();
    if (m != 0) { // NOPMD
      InputMap map;
      if (b.getClientProperty(WIM_PROPERTY) == null) {
        map = new ComponentInputMapUIResource(b);
        SwingUtilities.replaceUIInputMap(b, JComponent.WHEN_IN_FOCUSED_WINDOW,
                                         map);
        b.putClientProperty(WIM_PROPERTY, Boolean.TRUE);
      } else {
        map = SwingUtilities.getUIInputMap(b,
                                           JComponent.WHEN_IN_FOCUSED_WINDOW);
      }
      if (map != null) {
        map.clear();
        map.put(KeyStroke.getKeyStroke(m, ActionEvent.ALT_MASK, false),
                BPRESSED);
        map.put(KeyStroke.getKeyStroke(m, ActionEvent.ALT_MASK, true),
                BRELEASED);
        map.put(KeyStroke.getKeyStroke(m, 0, true), BRELEASED);
      }
    } else if (b.getClientProperty(WIM_PROPERTY) != null) {
      InputMap map = SwingUtilities.getUIInputMap(b,
                                                  JComponent.
                                                       WHEN_IN_FOCUSED_WINDOW);
      if (map != null) {
        map.clear();
      }
    }
  }

/*^**************************************************************************^*/
/*- Implementation of the Interfaces.                                        -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  public void propertyChange(PropertyChangeEvent e) {
    String prop = e.getPropertyName();
    if (AbstractButton.MNEMONIC_CHANGED_PROPERTY.equals(prop)) {
      updateMnemonicBinding((AbstractButton)e.getSource());
    }
    if (AbstractButton.CONTENT_AREA_FILLED_CHANGED_PROPERTY.equals(prop)) {
      checkOpacity((AbstractButton)e.getSource());
    }
    if (AbstractButton.TEXT_CHANGED_PROPERTY.equals(prop) ||
        FONT_PROPERTY.equals(prop) || FOREGROUND_PROPERTY.equals(prop)) {
      AbstractButton b = (AbstractButton)e.getSource();
      BasicHTML.updateRenderer(b, b.getText());
    }
  }

  /**
   * {@inheritDoc}
   */
  public void stateChanged(ChangeEvent e) {
    AbstractButton b = (AbstractButton)e.getSource();
    b.repaint();
  }

  /**
   * {@inheritDoc}
   */
  public void focusGained(FocusEvent e) {
    Component c = (Component)e.getSource();
    c.repaint();
  }

  /**
   * {@inheritDoc}
   */
  public void focusLost(FocusEvent e) {
    AbstractButton b = (AbstractButton)e.getSource();
    b.getModel().setArmed(false);
    b.repaint();
  }

  /**
   * {@inheritDoc}
   */
  public void mouseMoved(MouseEvent e) {
    // empty method body
  }

  /**
   * {@inheritDoc}
   */
  public void mouseDragged(MouseEvent e) {
    // empty method body
  }

  /**
   * {@inheritDoc}
   */
  public void mouseClicked(MouseEvent e) {
    // empty method body
  }

  /**
   * {@inheritDoc}
   */
  public void mousePressed(MouseEvent e) {
    if (SwingUtilities.isLeftMouseButton(e)) {
      AbstractButton b = (AbstractButton)e.getSource();
      if (b.contains(e.getX(), e.getY())) {
        long multiClickThreshhold = b.getMultiClickThreshhold();
        long lastTime = lastPressedTimestamp;
        long currentTime = lastPressedTimestamp = e.getWhen();
        if ((lastTime != -1) &&
            (currentTime - lastTime < multiClickThreshhold)) {
          shouldDiscardRelease = true;
          return;
        }
        ButtonModel model = b.getModel();
        if (!model.isEnabled()) {
          return;
        }
        if (!model.isArmed()) {
          model.setArmed(true);
        }
        model.setPressed(true);
        if (!b.hasFocus() && b.isRequestFocusEnabled()) {
          b.requestFocus();
        }
      }
    }
  }

  /**
   * {@inheritDoc}
   */
  public void mouseReleased(MouseEvent e) {
    if (SwingUtilities.isLeftMouseButton(e)) {
      if (shouldDiscardRelease) {
        shouldDiscardRelease = false;
        return;
      }
      AbstractButton b = (AbstractButton)e.getSource();
      ButtonModel model = b.getModel();
      model.setPressed(false);
    }
  }

  /**
   * {@inheritDoc}
   */
  public void mouseEntered(MouseEvent e) {
    AbstractButton b = (AbstractButton)e.getSource();
    ButtonModel model = b.getModel();
    if (b.isRolloverEnabled()) {
      model.setRollover(true);
    }
    model.setArmed(true);
    b.repaint();
  }

  /**
   * {@inheritDoc}
   */
  public void mouseExited(MouseEvent e) {
    AbstractButton b = (AbstractButton)e.getSource();
    ButtonModel model = b.getModel();
    if (b.isRolloverEnabled()) {
      model.setRollover(false);
    }
    model.setArmed(false);
    b.repaint();
  }

/*^**************************************************************************^*/
/*- Inner Classes.                                                           -*/
/*^**************************************************************************^*/

  /**
   * Pressed Action.
   */
  protected static class PressedAction extends AbstractAction {
    private final AbstractButton button;
    /**
     * Constructs new PressedAction object.
     *
     * @param button the button
     */
    public PressedAction(AbstractButton button) {
      super();
      this.button = button;
    }
    /**
     * {@inheritDoc}
     */
    public void actionPerformed(ActionEvent e) {
      ButtonModel model = button.getModel();
      model.setArmed(true);
      model.setPressed(true);
      if (!button.hasFocus()) {
        button.requestFocus();
      }
    }
    /**
     * {@inheritDoc}
     */
    public boolean isEnabled() {
      return button.getModel().isEnabled();
    }
  }

  /**
   * Released Action.
   */
  protected static class ReleasedAction extends AbstractAction {
    private final AbstractButton button;
    /**
     * Constructs new ReleasedAction object.
     *
     * @param button the button
     */
    public ReleasedAction(AbstractButton button) {
      super();
      this.button = button;
    }
    /**
     * {@inheritDoc}
     */
    public void actionPerformed(ActionEvent e) {
      ButtonModel model = button.getModel();
      model.setPressed(false);
      model.setArmed(false);
    }
    /**
     * {@inheritDoc}
     */
    public boolean isEnabled() {
      return button.getModel().isEnabled();
    }
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-25 18:31:15 Slobodan
   Initial version

*****************************************************************************^*/
