/*
 * @(#)SmoothComboBoxButton.java  1.0.0 / 2008-05-25
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.swing.plaf.smooth;

import java.awt.Component;
import java.awt.Graphics;
import java.awt.Insets;

import javax.swing.CellRendererPane;
import javax.swing.DefaultButtonModel;
import javax.swing.Icon;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JList;
import javax.swing.ListCellRenderer;

import net.wrka.awt.AwtTools;
import net.wrka.swing.graphics.GraphicUtils;
import net.wrka.util.CommonConstants;

/**
 * ComboBox Button.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public class SmoothComboBoxButton extends JButton {

/*^**************************************************************************^*/
/*- Private Static Field.                                                    -*/
/*^**************************************************************************^*/

  private static final Insets MARGIN = new Insets(1, 1, 1, 2);

/*^**************************************************************************^*/
/*- Package-Private Fields.                                                  -*/
/*^**************************************************************************^*/

  JComboBox comboBox;
  JList list;
  CellRendererPane rendererPane;
  transient Icon comboIcon;
  boolean iconOnly;

/*^**************************************************************************^*/
/*- Protected Constructor.                                                   -*/
/*^**************************************************************************^*/

  /**
   * Constructs new SmoothComboBoxButton object.
   */
  protected SmoothComboBoxButton() {
    super(CommonConstants.STRING_EMPTY);
    setMargin(MARGIN);
    setModel(new ComboBoxButtonModel());
  }

/*^**************************************************************************^*/
/*- Public Constructors.                                                     -*/
/*^**************************************************************************^*/

  /**
   * Constructs new SmoothComboBoxButton object.
   *
   * @param comboBox the combo box
   * @param comboIcon the combo icon
   * @param rendererPane the renderer pane
   * @param list the list
   */
  public SmoothComboBoxButton(JComboBox comboBox, Icon comboIcon,
                              CellRendererPane rendererPane, JList list) {
    this();
    this.comboBox = comboBox;
    this.comboIcon = comboIcon;
    this.rendererPane = rendererPane;
    this.list = list;
    setEnabled(comboBox.isEnabled());
    setRequestFocusEnabled(comboBox.isEnabled());
  }

  /**
   * Constructs new SmoothComboBoxButton object.
   *
   * @param comboBox the combo box
   * @param comboIcon the combo icon
   * @param iconOnly the icon only flag
   * @param rendererPane the renderer pane
   * @param list the list
   */
  public SmoothComboBoxButton(JComboBox comboBox, Icon comboIcon,
                              boolean iconOnly, CellRendererPane rendererPane,
                              JList list) {
    this(comboBox, comboIcon, rendererPane, list);
    this.iconOnly = iconOnly;
  }

/*^**************************************************************************^*/
/*- Public Methods.                                                          -*/
/*^**************************************************************************^*/

  /**
   * Returns combo box.
   *
   * @return combo box
   */
  public JComboBox getComboBox() {
    return comboBox;
  }

  /**
   * Returns combo icon.
   *
   * @return combo icon
   */
  public Icon getComboIcon() {
    return comboIcon;
  }

  /**
   * Returns icon only flag.
   *
   * @return icon only flag
   */
  public boolean isIconOnly() {
    return iconOnly;
  }

  /**
   * Sets icon only flag.
   *
   * @param iconOnly the flag
   */
  public void setIconOnly(boolean iconOnly) {
    this.iconOnly = iconOnly;
  }

/*^**************************************************************************^*/
/*- Overridden Methods.                                                      -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  public boolean isFocusable() {
    return false;
  }

  /**
   * {@inheritDoc}
   */
  public boolean isFocusPainted() {
    return false;
  }

  /**
   * {@inheritDoc}
   */
  public void paintBorder(Graphics g) {
    super.paintBorder(g);
    if (iconOnly) {
      if (isEnabled()) {
        g.setColor(SmoothLookAndFeel.getControlDarkShadow());
      } else {
        g.setColor(SmoothLookAndFeel.getControlShadow());
      }
      g.fillRect(0, 0, 1, 1);
      g.fillRect(0, getHeight() - 1, 1, 1);
    }
  }

  /**
   * {@inheritDoc}
   */
  public void paintComponent(Graphics g) { // NOPMD
    super.paintComponent(g);
    boolean leftToRight = AwtTools.isLeftToRight(comboBox);
    Insets insets = getInsets();
    int width = getWidth() - (insets.left + insets.right);
    int height = getHeight() - (insets.top + insets.bottom);
    if ((height <= 0) || (width <= 0)) {
      return;
    }
    int left = insets.left;
    int top = insets.top;
    int bottom = top + (height - 1);
    int iconWidth = 0;
    int iconLeft;
    // Paint the icon
    if (comboIcon != null) {
      iconWidth = comboIcon.getIconWidth();
      int iconHeight = comboIcon.getIconHeight();
      int iconTop = 0;
      if (iconOnly) {
        iconLeft = (getWidth() - iconWidth) >> 1;
        iconTop = (getHeight() - iconHeight) >> 1;
      } else {
        if (leftToRight) {
          iconLeft = (left + (width - 1)) - iconWidth;
        } else {
          iconLeft = left;
        }
        iconTop = (top + ((bottom - top) >> 1)) - (iconHeight >> 1);
      }
      comboIcon.paintIcon(this, g, iconLeft, iconTop);
      // Paint the focus
      if ((comboBox != null) && (comboBox.hasFocus())) {
        g.setColor(SmoothLookAndFeel.getFocusColor());
        GraphicUtils.drawDashedRect(g, 3, 3, getWidth() - 6, getHeight() - 6);
      }
    }
    // Let the renderer paint
    if (!iconOnly && (comboBox != null)) {
      ListCellRenderer renderer = comboBox.getRenderer();
      Component c;
      boolean renderPressed = getModel().isPressed();
      c = renderer.getListCellRendererComponent(list,
                                                comboBox.getSelectedItem(), -1,
                                                renderPressed, false);
      c.setFont(rendererPane.getFont());
      if (model.isArmed() && model.isPressed()) {
        if (isOpaque()) {
          c.setBackground(comboBox.getBackground());
        }
        c.setForeground(comboBox.getForeground());
      } else if (!comboBox.isEnabled()) { // NOPMD
        if (isOpaque()) {
          c.setBackground(SmoothLookAndFeel.getControl());
        }
        c.setForeground(SmoothLookAndFeel.getInactiveControlTextColor());
      } else {
        c.setForeground(comboBox.getForeground());
        c.setBackground(comboBox.getBackground());
      }
      int cWidth = width - (insets.right + iconWidth);
      if (leftToRight) {
        rendererPane.paintComponent(g, c, this, left, top, cWidth, height);
      } else {
        rendererPane.paintComponent(g, c, this, left + iconWidth, top, cWidth,
                                    height);
      }
    }
  }

/*^**************************************************************************^*/
/*- Inner Class.                                                             -*/
/*^**************************************************************************^*/

  /**
   * ComboBox Button Model.
   */
  protected static class ComboBoxButtonModel extends DefaultButtonModel {
    /**
     * {@inheritDoc}
     */
    public void setArmed(boolean armed) {
      if (isPressed()) {
        super.setArmed(true);
      } else {
        super.setArmed(armed);
      }
    }
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-25 18:42:46 Slobodan
   Initial version

*****************************************************************************^*/
