/*
 * @(#)SmoothIconFactory.java  1.0.0 / 2008-05-25
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.swing.plaf.smooth;

import java.awt.Color;
import java.awt.Component;
import java.awt.Container;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.RenderingHints;
import java.awt.geom.AffineTransform;
import java.awt.geom.GeneralPath;

import javax.swing.ButtonModel;
import javax.swing.Icon;
import javax.swing.JCheckBox;
import javax.swing.JComponent;
import javax.swing.JMenuItem;
import javax.swing.JRadioButton;
import javax.swing.UIManager;
import javax.swing.plaf.UIResource;

import net.wrka.swing.graphics.GraphicUtils;

/**
 * Smooth Icon Factory.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public final class SmoothIconFactory {

/*^**************************************************************************^*/
/*- Private Static Fields.                                                   -*/
/*^**************************************************************************^*/

  private static Icon checkBoxIcon;
  private static Icon radioButtonIcon;
  private static Icon checkBoxMenuItemIcon;
  private static Icon radioButtonMenuItemIcon;
  private static Icon expandedTreeIcon;
  private static Icon collapsedTreeIcon;
  private static Icon treeClosedIcon;
  private static Icon treeOpenIcon;
  private static Icon treeLeafIcon;
  private static Icon comboBoxButtonIcon;
  private static Icon frameCloseIcon;
  private static Icon frameIconifyIcon;
  private static Icon frameMaximizeIcon;
  private static Icon frameMinimizeIcon;

/*^**************************************************************************^*/
/*- Private Constructor.                                                     -*/
/*^**************************************************************************^*/

  /**
   * Class has no instances.
   */
  private SmoothIconFactory() {
    // empty constructor body
  }

/*^**************************************************************************^*/
/*- Public Static Methods.                                                   -*/
/*^**************************************************************************^*/

  /**
   * Returns check box icon.
   *
   * @return check box icon
   */
  public static Icon getCheckBoxIcon() {
    if (checkBoxIcon == null) { // NOPMD - not singleton
      checkBoxIcon = new CheckBoxIcon();
    }
    return checkBoxIcon;
  }

  /**
   * Returns radio button icon.
   *
   * @return radio button icon
   */
  public static Icon getRadioButtonIcon() {
    if (radioButtonIcon == null) { // NOPMD - not singleton
      radioButtonIcon = new RadioButtonIcon();
    }
    return radioButtonIcon;
  }

  /**
   * Returns check box menu item icon.
   *
   * @return check box menu item icon
   */
  public static Icon getCheckBoxMenuItemIcon() {
    if (checkBoxMenuItemIcon == null) { // NOPMD - not singleton
      checkBoxMenuItemIcon = new CheckBoxMenuItemIcon();
    }
    return checkBoxMenuItemIcon;
  }

  /**
   * Returns radio button menu item icon.
   *
   * @return radio button menu item icon
   */
  public static Icon getRadioButtonMenuItemIcon() {
    if (radioButtonMenuItemIcon == null) { // NOPMD - not singleton
      radioButtonMenuItemIcon = new RadioButtonMenuItemIcon();
    }
    return radioButtonMenuItemIcon;
  }

  /**
   * Returns expanded tree icon.
   *
   * @return expanded tree icon
   */
  public static Icon getExpandedTreeIcon() {
    if (expandedTreeIcon == null) { // NOPMD - not singleton
      expandedTreeIcon = new ExpandedTreeIcon();
    }
    return expandedTreeIcon;
  }

  /**
   * Returns collapsed tree icon.
   *
   * @return collapsed tree icon
   */
  public static Icon getCollapsedTreeIcon() {
    if (collapsedTreeIcon == null) { // NOPMD - not singleton
      collapsedTreeIcon = new CollapsedTreeIcon();
    }
    return collapsedTreeIcon;
  }

  /**
   * Returns tree closed icon.
   *
   * @return tree closed icon
   */
  public static Icon getTreeClosedIcon() {
    if (treeClosedIcon == null) { // NOPMD - not singleton
      treeClosedIcon = new TreeClosedIcon();
    }
    return treeClosedIcon;
  }

  /**
   * Returns tree open icon.
   *
   * @return tree open icon
   */
  public static Icon getTreeOpenIcon() {
    if (treeOpenIcon == null) { // NOPMD - not singleton
      treeOpenIcon = new TreeOpenIcon();
    }
    return treeOpenIcon;
  }

  /**
   * Returns tree leaf icon.
   *
   * @return tree leaf icon
   */
  public static Icon getTreeLeafIcon() {
    if (treeLeafIcon == null) { // NOPMD - not singleton
      treeLeafIcon = new TreeLeafIcon();
    }
    return treeLeafIcon;
  }

  /**
   * Returns combo box button icon.
   *
   * @return combo box button icon
   */
  public static Icon getComboBoxButtonIcon() {
    if (comboBoxButtonIcon == null) { // NOPMD - not singleton
      comboBoxButtonIcon = new ComboBoxButtonIcon();
    }
    return comboBoxButtonIcon;
  }

  /**
   * Returns frame close icon.
   *
   * @return frame close icon
   */
  public static Icon getFrameCloseIcon() {
    if (frameCloseIcon == null) { // NOPMD - not singleton
      frameCloseIcon = new FrameCloseIcon();
    }
    return frameCloseIcon;
  }

  /**
   * Returns frame iconify icon.
   *
   * @return frame iconify icon
   */
  public static Icon getFrameIconifyIcon() {
    if (frameIconifyIcon == null) { // NOPMD - not singleton
      frameIconifyIcon = new FrameIconifyIcon();
    }
    return frameIconifyIcon;
  }

  /**
   * Returns maximize icon.
   *
   * @return maximize icon
   */
  public static Icon getFrameMaximizeIcon() {
    if (frameMaximizeIcon == null) { // NOPMD - not singleton
      frameMaximizeIcon = new FrameMaximizeIcon();
    }
    return frameMaximizeIcon;
  }

  /**
   * Returns minimize icon.
   *
   * @return minimize icon
   */
  public static Icon getFrameMinimizeIcon() {
    if (frameMinimizeIcon == null) { // NOPMD - not singleton
      frameMinimizeIcon = new FrameMinimizeIcon();
    }
    return frameMinimizeIcon;
  }

/*^**************************************************************************^*/
/*- Inner Classes.                                                           -*/
/*^**************************************************************************^*/

  /**
   * TreeClosed Icon.
   */
  static class TreeClosedIcon implements Icon, UIResource {
    private static final GeneralPath TOP_LEFT;
    private static final GeneralPath BOTTOM_RIGHT;
    private static final GeneralPath SHAPE;
    static {
      TOP_LEFT = new GeneralPath();
      TOP_LEFT.moveTo(0f, 11f);
      TOP_LEFT.lineTo(0f, 4f);
      TOP_LEFT.lineTo(2f, 2f);
      TOP_LEFT.lineTo(5f, 2f);
      TOP_LEFT.lineTo(7f, 4f);
      TOP_LEFT.lineTo(11f, 4f);
      BOTTOM_RIGHT = new GeneralPath();
      BOTTOM_RIGHT.moveTo(12f, 5f);
      BOTTOM_RIGHT.lineTo(12f, 11f);
      BOTTOM_RIGHT.lineTo(11f, 12f);
      BOTTOM_RIGHT.lineTo(1f, 12f);
      SHAPE = new GeneralPath(TOP_LEFT);
      SHAPE.append(BOTTOM_RIGHT, true);
    }
    /**
     * {@inheritDoc}
     */
    public void paintIcon(Component c, Graphics g, int x, int y) {
      Graphics2D g2 = (Graphics2D)g;
      g2.setColor(SmoothLookAndFeel.getPrimaryControl());
      g2.fill(SHAPE);
      g2.setColor(SmoothLookAndFeel.getPrimaryControlShadow());
      g2.draw(TOP_LEFT);
      g2.setColor(SmoothLookAndFeel.getPrimaryControlDarkShadow());
      g2.draw(BOTTOM_RIGHT);
      SmoothUtils.drawFastInvTranShadowGradient(g, false, 0, 4, 13, 9);
      SmoothUtils.drawFastInvTranShadowGradient(g, true, 0, 4, 13, 9);
    }
    /**
     * {@inheritDoc}
     */
    public int getIconWidth() {
      return 16;
    }
    /**
     * {@inheritDoc}
     */
    public int getIconHeight() {
      return 16;
    }
  }

  /**
   * TreeOpen Icon.
   */
  static class TreeOpenIcon implements Icon, UIResource {
    private static final GeneralPath TOP_LEFT;
    private static final GeneralPath BOTTOM_RIGHT;
    private static final GeneralPath SHAPE;
    static {
      TOP_LEFT = new GeneralPath();
      TOP_LEFT.moveTo(0f, 11f);
      TOP_LEFT.lineTo(0f, 4f);
      TOP_LEFT.lineTo(2f, 2f);
      TOP_LEFT.lineTo(5f, 2f);
      TOP_LEFT.lineTo(7f, 4f);
      TOP_LEFT.lineTo(11f, 4f);
      BOTTOM_RIGHT = new GeneralPath();
      BOTTOM_RIGHT.moveTo(12f, 5f);
      BOTTOM_RIGHT.lineTo(12f, 11f);
      BOTTOM_RIGHT.lineTo(11f, 12f);
      BOTTOM_RIGHT.lineTo(1f, 12f);
      SHAPE = new GeneralPath(TOP_LEFT);
      SHAPE.append(BOTTOM_RIGHT, true);
    }
    /**
     * {@inheritDoc}
     */
    public void paintIcon(Component c, Graphics g, int x, int y) {
      Graphics2D g2 = (Graphics2D)g;
      g2.setColor(SmoothLookAndFeel.getPrimaryControl());
      g2.fill(SHAPE);
      g2.setColor(SmoothLookAndFeel.getPrimaryControlShadow());
      g2.draw(TOP_LEFT);
      g2.setColor(SmoothLookAndFeel.getPrimaryControlDarkShadow());
      g2.draw(BOTTOM_RIGHT);
      AffineTransform oat = g2.getTransform();
      g2.translate(10, 0);
      g2.shear(-0.707, 0);
      g2.setColor(SmoothLookAndFeel.getPrimaryControl());
      g2.fillRect(0, 7, 11, 6);
      g2.setColor(SmoothLookAndFeel.getPrimaryControlShadow());
      g2.drawLine(0, 12, 0, 7);
      g2.drawLine(1, 7, 10, 7);
      g2.setColor(SmoothLookAndFeel.getPrimaryControlDarkShadow());
      g2.drawLine(0, 12, 10, 12);
      g2.drawLine(11, 12, 11, 8);
      SmoothUtils.drawFastInvTranShadowGradient(g, false, 0, 7, 11, 6);
      SmoothUtils.drawFastInvTranShadowGradient(g, true, 0, 7, 11, 6);
      g2.setTransform(oat);
    }
    /**
     * {@inheritDoc}
     */
    public int getIconWidth() {
      return 16;
    }
    /**
     * {@inheritDoc}
     */
    public int getIconHeight() {
      return 16;
    }
  }

  /**
   * TreeLeaf Icon.
   */
  static class TreeLeafIcon implements Icon, UIResource {
    /**
     * {@inheritDoc}
     */
    public void paintIcon(Component c, Graphics g, int x, int y) {
      g.setColor(SmoothLookAndFeel.getWindowBackground());
      g.fillRect(1, 1, 8, 8);
      g.setColor(SmoothLookAndFeel.getPrimaryControlShadow());
      g.fillRect(0, 0, 9, 1);
      g.fillRect(0, 1, 1, 9);
      g.fillRect(9, 1, 1, 2);
      g.setColor(SmoothLookAndFeel.getPrimaryControl());
      g.fillRect(8, 1, 1, 1);
      g.setColor(SmoothLookAndFeel.getPrimaryControlDarkShadow());
      g.fillRect(9, 3, 1, 7);
      g.fillRect(0, 10, 10, 1);
      SmoothUtils.drawFastInvTranShadowGradient(g, false, 1, 1, 9, 9);
      SmoothUtils.drawFastInvTranShadowGradient(g, true, 0, 0, 10, 11);
    }
    /**
     * {@inheritDoc}
     */
    public int getIconWidth() {
      return 11;
    }
    /**
     * {@inheritDoc}
     */
    public int getIconHeight() {
      return 11;
    }
  }

  /**
   * ExpandedTree Icon.
   */
  static class ExpandedTreeIcon implements Icon, UIResource {
    static final int SIZE = 9;
    static final int SIZE_M1 = 8;
    static final int SIZE_M3 = 6;
    static final int HALF_SIZE = 4;
    /**
     * {@inheritDoc}
     */
    public void paintIcon(Component c, Graphics g, int x, int y) {
      Color bg = c.getBackground();
      if (bg == null) {
        g.setColor(UIManager.getColor("Tree.background")); //$NON-NLS-1$
      } else {
        g.setColor(bg);
      }
      g.fillRect(x, y, SIZE_M1, SIZE_M1);
      g.setColor(Color.gray);
      g.drawRect(x, y, SIZE_M1, SIZE_M1);
      g.setColor(Color.BLACK);
      g.drawLine(x + 2, y + HALF_SIZE, x + SIZE_M3, y + HALF_SIZE);
    }
    /**
     * {@inheritDoc}
     */
    public int getIconWidth() {
      return SIZE;
    }
    /**
     * {@inheritDoc}
     */
    public int getIconHeight() {
      return SIZE;
    }
  }

  /**
   * CollapsedTree Icon.
   */
  static class CollapsedTreeIcon extends ExpandedTreeIcon {
    /**
     * {@inheritDoc}
     */
    public void paintIcon(Component c, Graphics g, int x, int y) {
      super.paintIcon(c, g, x, y);
      g.drawLine(x + HALF_SIZE, y + 2, x + HALF_SIZE, y + SIZE_M3);
    }
  }

  /**
   * CheckBox Icon.
   */
  static class CheckBoxIcon implements Icon, UIResource {
    private static final int ICON_SIZE = 13;
    private static final int ICON_SIZE_M1 = ICON_SIZE - 1;
    private static final int ICON_SIZE_M2 = ICON_SIZE - 2;
    private static final int ICON_SIZE_M4 = ICON_SIZE - 4;
    private static final int ICON_SIZE_M5 = ICON_SIZE - 5;
    private static final int ICON_SIZE_M6 = ICON_SIZE - 6;
    private static final int ICON_SIZE_M8 = ICON_SIZE - 8;
    /**
     * {@inheritDoc}
     */
    public void paintIcon(Component c, Graphics g, int x, int y) { // NOPMD
      JCheckBox cb = (JCheckBox)c;
      ButtonModel model = cb.getModel();
      Container parent = c.getParent();
      g.translate(x, y - 1);
      if (!model.isPressed() && cb.isEnabled()) {
        g.setColor(SmoothLookAndFeel.getWindowBackground());
        g.fillRect(1, 1, ICON_SIZE_M2, ICON_SIZE_M2);
      }
      if (!(cb.isContentAreaFilled() &&  model.isArmed() &&
            !model.isPressed())) {
        SmoothUtils.drawFastInvFlat3DEffectHorizontalLight(g, 1, 1,
                                                           ICON_SIZE_M2,
                                                           ICON_SIZE_M2);
        SmoothUtils.drawFastInvFlat3DEffectVerticalLight(g, 1, 1, ICON_SIZE_M2,
                                                         ICON_SIZE_M2);
      }
      if (model.isEnabled()) {
        g.setColor(SmoothLookAndFeel.getControlDarkShadow());
      } else {
        g.setColor(SmoothLookAndFeel.getControlShadow());
      }
      g.fillRect(2, 0, ICON_SIZE_M4, 1);
      g.fillRect(0, 2, 1, ICON_SIZE_M4);
      g.fillRect(2, ICON_SIZE_M1, ICON_SIZE_M4, 1);
      g.fillRect(ICON_SIZE_M1, 2, 1, ICON_SIZE_M4);
      if (parent == null) {
        g.setColor(c.getBackground());
      } else {
        g.setColor(parent.getBackground());
      }
      g.fillRect(0, 0, 1, 1);
      g.fillRect(0, ICON_SIZE_M1, 1, 1);
      g.fillRect(ICON_SIZE_M1, 0, 1, 1);
      g.fillRect(ICON_SIZE_M1, ICON_SIZE_M1, 1, 1);
      g.setColor(SmoothLookAndFeel.getComponentGradientShadow());
      g.fillRect(1, 1, 1, 1);
      g.fillRect(1, ICON_SIZE_M2, 1, 1);
      g.fillRect(ICON_SIZE_M2, 1, 1, 1);
      g.fillRect(ICON_SIZE_M2, ICON_SIZE_M2, 1, 1);
      g.setColor(SmoothLookAndFeel.getControlShadow());
      g.fillRect(0, 1, 1, 1);
      g.fillRect(1, 0, 1, 1);
      g.fillRect(ICON_SIZE_M2, 0, 1, 1);
      g.fillRect(ICON_SIZE_M1, 1, 1, 1);
      g.fillRect(0, ICON_SIZE_M2, 1, 1);
      g.fillRect(1, ICON_SIZE_M1, 1, 1);
      g.fillRect(ICON_SIZE_M2, ICON_SIZE_M1, 1, 1);
      g.fillRect(ICON_SIZE_M1, ICON_SIZE_M2, 1, 1);
      if (model.isSelected()) {
        // paint check
        if (model.isEnabled()) {
          if (model.isArmed() && !model.isPressed()) {
            g.setColor(SmoothLookAndFeel.getRolloverForeground());
          } else {
            g.setColor(cb.getForeground());
          }
        } else {
          g.setColor(SmoothLookAndFeel.getControlShadow());
        }
        g.fillRect(3, 5, 2, ICON_SIZE_M8);
        g.drawLine(ICON_SIZE_M4, 3, 5, ICON_SIZE_M6);
        g.drawLine(ICON_SIZE_M4, 4, 5, ICON_SIZE_M5);
      }
      g.translate(-x, -y + 1);
    }
    /**
     * {@inheritDoc}
     */
    public int getIconWidth() {
      return ICON_SIZE;
    }
    /**
     * {@inheritDoc}
     */
    public int getIconHeight() {
      return ICON_SIZE;
    }
  }

  /**
   * RadioButton Icon.
   */
  static class RadioButtonIcon implements Icon, UIResource {
    private static final int ICON_SIZE = 12;
    private static final int ICON_SIZE_M2 = ICON_SIZE - 2;
    private static final int ICON_SIZE_M3 = ICON_SIZE - 3;
    /**
     * {@inheritDoc}
     */
    public void paintIcon(Component c, Graphics g, int x, int y) { // NOPMD
      JRadioButton rb = (JRadioButton)c;
      ButtonModel model = rb.getModel();
      Container parent = rb.getParent();
      Graphics2D g2 = (Graphics2D)g;
      g.translate(x, y);
      if (!model.isPressed() && rb.isEnabled()) {
        g.setColor(SmoothLookAndFeel.getWindowBackground());
        g.fillRect(1, 1, ICON_SIZE_M2, ICON_SIZE_M2);
      }
      if (!(rb.isContentAreaFilled() &&  model.isArmed() &&
            !model.isPressed())) {
        SmoothUtils.drawFastInvFlat3DEffectHorizontalLight(g, 1, 1,
                                                           ICON_SIZE_M3,
                                                           ICON_SIZE_M3);
        SmoothUtils.drawFastInvFlat3DEffectVerticalLight(g, 1, 1,
                                                           ICON_SIZE_M3,
                                                           ICON_SIZE_M3);
      }
      if (parent == null) {
        g.setColor(c.getBackground());
      } else {
        g.setColor(parent.getBackground());
      }
      g.fillRect(1, 1, 1, 1);
      g.fillRect(1, 10, 1, 1);
      g.fillRect(10, 1, 1, 1);
      g.fillRect(10, 10, 1, 1);
      if (model.isEnabled()) {
        g.setColor(SmoothLookAndFeel.getControlDarkShadow());
      } else {
        g.setColor(SmoothLookAndFeel.getControlShadow());
      }
      g.fillRect(4, 0, 4, 1);
      g.fillRect(8, 1, 2, 1);
      g.fillRect(10, 2, 1, 2);
      g.fillRect(11, 4, 1, 4);
      g.fillRect(10, 8, 1, 2);
      g.fillRect(8, 10, 2, 1);
      g.fillRect(4, 11, 4, 1);
      g.fillRect(2, 10, 2, 1);
      g.fillRect(1, 8, 1, 2);
      g.fillRect(0, 4, 1, 4);
      g.fillRect(1, 2, 1, 2);
      g.fillRect(2, 1, 2, 1);
      if (model.isSelected()) {
        // paint dot
        if (model.isEnabled()) {
          if (model.isArmed() && !model.isPressed()) {
            g.setColor(SmoothLookAndFeel.getRolloverForeground());
          } else {
            g.setColor(rb.getForeground());
          }
        } else {
          g.setColor(SmoothLookAndFeel.getControlShadow());
        }
        Object oldaa = g2.getRenderingHint(RenderingHints.KEY_ANTIALIASING);
        g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
                            RenderingHints.VALUE_ANTIALIAS_ON);
        g2.fillOval(3, 3, 6, 6);
        g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING, oldaa);
      }
      g.translate(-x, -y);
    }
    /**
     * {@inheritDoc}
     */
    public int getIconWidth() {
      return ICON_SIZE;
    }
    /**
     * {@inheritDoc}
     */
    public int getIconHeight() {
      return ICON_SIZE;
    }
  }

  /**
   * CheckBoxMenuItem Icon.
   */
  static class CheckBoxMenuItemIcon implements Icon, UIResource {
    private static final int ICON_SIZE = 15;
    private static final int ICON_SIZE_M1 = ICON_SIZE - 1;
    private static final int ICON_SIZE_M2 = ICON_SIZE - 2;
    private static final int ICON_SIZE_M5 = ICON_SIZE - 5;
    private static final int ICON_SIZE_M6 = ICON_SIZE - 6;
    private static final int ICON_SIZE_M10 = ICON_SIZE - 10;
    /**
     * {@inheritDoc}
     */
    public void paintIcon(Component c, Graphics g, int x, int y) {
      JMenuItem mi = (JMenuItem)c;
      ButtonModel model = mi.getModel();
      g.translate(x - 1, y);
      if (model.isSelected()) {
        // paint check
        if (model.isEnabled()) {
          g.setColor(mi.getForeground());
        } else {
          g.setColor(SmoothLookAndFeel.getControlShadow());
        }
        g.fillRect(4, 6, 2, ICON_SIZE_M10);
        g.drawLine(ICON_SIZE_M5, 4, 5, ICON_SIZE_M6);
        g.drawLine(ICON_SIZE_M5, 5, 5, ICON_SIZE_M5);
        g.setColor(SmoothLookAndFeel.getRolloverMaskColor());
        g.fillRect(1, 1, ICON_SIZE_M2, ICON_SIZE_M2);
        g.setColor(SmoothLookAndFeel.getRolloverOuterBorderColor());
        GraphicUtils.draw1PixRoundRect(g, 0, 0, ICON_SIZE_M1, ICON_SIZE_M1);
      }
      g.translate(-x + 1, -y);
    }
    /**
     * {@inheritDoc}
     */
    public int getIconWidth() {
      return ICON_SIZE;
    }
    /**
     * {@inheritDoc}
     */
    public int getIconHeight() {
      return ICON_SIZE;
    }
  }

  /**
   * RadioButtonMenuItem Icon.
   */
  static class RadioButtonMenuItemIcon implements Icon, UIResource {
    private static final int ICON_SIZE = 15;
    private static final int ICON_SIZE_M1 = ICON_SIZE - 1;
    private static final int ICON_SIZE_M2 = ICON_SIZE - 2;
    /**
     * {@inheritDoc}
     */
    public void paintIcon(Component c, Graphics g, int x, int y) {
      JMenuItem mi = (JMenuItem)c;
      ButtonModel model = mi.getModel();
      Graphics2D g2 = (Graphics2D)g;
      g2.translate(x - 1, y);
      if (model.isSelected()) {
        // paint dot
        if (model.isEnabled()) {
          g2.setColor(mi.getForeground());
        } else {
          g2.setColor(SmoothLookAndFeel.getControlShadow());
        }
        Object oldaa = g2.getRenderingHint(RenderingHints.KEY_ANTIALIASING);
        g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
                            RenderingHints.VALUE_ANTIALIAS_ON);
        g2.fillOval(5, 5, 5, 5);
        g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING, oldaa);
        g2.setColor(SmoothLookAndFeel.getRolloverMaskColor());
        g2.fillRect(1, 1, ICON_SIZE_M2, ICON_SIZE_M2);
        g2.setColor(SmoothLookAndFeel.getRolloverOuterBorderColor());
        GraphicUtils.draw1PixRoundRect(g2, 0, 0, ICON_SIZE_M1, ICON_SIZE_M1);
      }
      g2.translate(-x + 1, -y);
    }
    /**
     * {@inheritDoc}
     */
    public int getIconWidth() {
      return ICON_SIZE;
    }
    /**
     * {@inheritDoc}
     */
    public int getIconHeight() {
      return ICON_SIZE;
    }
  }

  /**
   * ComboBoxButton Icon.
   */
  static class ComboBoxButtonIcon implements Icon, UIResource {
    /**
     * {@inheritDoc}
     */
    public void paintIcon(Component c, Graphics g, int x, int y) {
      JComponent component = (JComponent)c;
      g.translate(x, y);
      if (component.isEnabled()) {
        g.setColor(SmoothLookAndFeel.getControlInfo());
      } else {
        g.setColor(SmoothLookAndFeel.getControlShadow());
      }
      g.fillRect(0, 0, 8, 1);
      g.fillRect(1, 1, 6, 1);
      g.fillRect(2, 2, 4, 1);
      g.fillRect(3, 3, 2, 1);
      g.translate(-x, -y);
    }
    /**
     * {@inheritDoc}
     */
    public int getIconWidth() {
      return 8;
    }
    /**
     * {@inheritDoc}
     */
    public int getIconHeight() {
      return 4;
    }
  }

  /**
   * FrameClose Icon.
   */
  static class FrameCloseIcon implements Icon, UIResource {
    /**
     * {@inheritDoc}
     */
    public void paintIcon(Component c, Graphics g, int x, int y) {
      g.setColor(SmoothLookAndFeel.getControlTextColor());
      g.translate(0, 1);
      g.drawLine(4, 3, 10, 9);
      g.drawLine(5, 3, 11, 9);
      g.drawLine(10, 3, 4, 9);
      g.drawLine(11, 3, 5, 9);
      g.translate(0, -1);
    }
    /**
     * {@inheritDoc}
     */
    public int getIconWidth() {
      return 14;
    }
    /**
     * {@inheritDoc}
     */
    public int getIconHeight() {
      return 16;
    }
  }

  /**
   * FrameIconify Icon.
   */
  static class FrameIconifyIcon implements Icon {
    /**
     * {@inheritDoc}
     */
    public void paintIcon(Component c, Graphics g, int x, int y) {
      g.setColor(SmoothLookAndFeel.getControlTextColor());
      g.drawRect(4, 9, 6, 1);
    }
    /**
     * {@inheritDoc}
     */
    public int getIconWidth() {
      return 14;
    }
    /**
     * {@inheritDoc}
     */
    public int getIconHeight() {
      return 16;
    }
  }

  /**
   * FrameMaximize Icon.
   */
  static class FrameMaximizeIcon implements Icon {
    /**
     * {@inheritDoc}
     */
    public void paintIcon(Component c, Graphics g, int x, int y) {
      g.setColor(SmoothLookAndFeel.getControlTextColor());
      g.translate(1, 1);
      g.drawRect(3, 2, 8, 8);
      g.drawLine(4, 3, 10, 3);
      g.setColor(SmoothLookAndFeel.getControl());
      g.fillRect(4, 4, 7, 5);
      g.translate(-1, -1);
    }
    /**
     * {@inheritDoc}
     */
    public int getIconWidth() {
      return 14;
    }
    /**
     * {@inheritDoc}
     */
    public int getIconHeight() {
      return 16;
    }
  }

  /**
   * FrameMinimize Icon.
   */
  static class FrameMinimizeIcon implements Icon {
    /**
     * {@inheritDoc}
     */
    public void paintIcon(Component c, Graphics g, int x, int y) {
      g.setColor(SmoothLookAndFeel.getControlTextColor());
      g.drawRect(5, 2, 5, 5);
      g.drawLine(5, 3, 10, 3);
      g.drawRect(3, 5, 5, 5);
      g.drawLine(3, 6, 7, 6);
      g.setColor(SmoothLookAndFeel.getControl());
      g.fillRect(4, 7, 4, 3);
    }
    /**
     * {@inheritDoc}
     */
    public int getIconWidth() {
      return 14;
    }
    /**
     * {@inheritDoc}
     */
    public int getIconHeight() {
      return 16;
    }
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-25 20:36:47 Slobodan
   Initial version

*****************************************************************************^*/
