/*
 * @(#)SmoothInternalFrameTitlePane.java  1.0.0 / 2008-05-25
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.swing.plaf.smooth;

import java.awt.Color;
import java.awt.Component;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.LayoutManager;

import javax.swing.JButton;
import javax.swing.JInternalFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.SwingUtilities;
import javax.swing.UIManager;
import javax.swing.plaf.basic.BasicInternalFrameTitlePane;

import net.wrka.awt.AwtTools;
import net.wrka.swing.graphics.GraphicUtils;
import net.wrka.swing.graphics.ImageFactory;
import net.wrka.swing.tooltips.ToolTipManager;

/**
 * InternalFrameTitlePane.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public class SmoothInternalFrameTitlePane extends BasicInternalFrameTitlePane {

/*^**************************************************************************^*/
/*- Packge-Private Fields.                                                   -*/
/*^**************************************************************************^*/

  Color titleABG1;
  Color titleABG2;
  Color titleIBG1;
  Color titleIBG2;
  String closeButtonToolTip;
  String iconButtonToolTip;
  String restoreButtonToolTip;
  String maxButtonToolTip;

/*^**************************************************************************^*/
/*- Public Constructor.                                                      -*/
/*^**************************************************************************^*/

  /**
   * Constructs new SmoothInternalFrameTitlePane object.
   *
   * @param frame the frame
   */
  public SmoothInternalFrameTitlePane(JInternalFrame frame) {
    super(frame);
    ToolTipManager.registerContainer(this);
  }

/*^**************************************************************************^*/
/*- Overridden Methods.                                                      -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  public void paintComponent(Graphics g)  {
    boolean isSelected = frame.isSelected();
    if (isSelected) {
      GraphicUtils.drawFastHorizontalGradient(g, titleABG1, titleABG2, 0, 0,
                                              getWidth(), getHeight());
    } else {
      GraphicUtils.drawFastHorizontalGradient(g, titleIBG1, titleIBG2, 0, 0,
                                              getWidth(), getHeight());
    }
    if (frame.getTitle() != null) {
      Font f = g.getFont();
      g.setFont(getFont());
      if (isSelected) {
        g.setColor(selectedTextColor);
      } else {
        g.setColor(notSelectedTextColor);
      }
      FontMetrics fm = g.getFontMetrics();
      int fmHeight = fm.getHeight() - fm.getLeading();
      int baseline = ((18 - fmHeight) >> 1) + fm.getAscent() + fm.getLeading();
      int titleX;
      String title = frame.getTitle();
      if (AwtTools.isLeftToRight(frame)) {
        titleX = menuBar.getX() + menuBar.getWidth() + 3;
      } else {
        titleX = menuBar.getX() - 3 -
                 SwingUtilities.computeStringWidth(fm, title);
      }
      g.drawString(title, titleX, baseline);
      g.setFont(f);
    }
  }

  /**
   * {@inheritDoc}
   */
  protected void installDefaults() {
    super.installDefaults();
    titleABG1 = SmoothLookAndFeel.getInternalFrameTitlePaneABG1();
    titleABG2 = SmoothLookAndFeel.getInternalFrameTitlePaneABG2();
    titleIBG1 = SmoothLookAndFeel.getControlShadow();
    titleIBG2 = GraphicUtils.brighterColor(
                  SmoothLookAndFeel.getControlShadow(), 0.15f);
    setFont(SmoothLookAndFeel.getWindowTitleFont());
    closeButtonToolTip =
        UIManager.getString("InternalFrame.closeButtonToolTip"); //$NON-NLS-1$
    iconButtonToolTip =
        UIManager.getString("InternalFrame.iconButtonToolTip"); //$NON-NLS-1$
    restoreButtonToolTip =
        UIManager.getString("InternalFrame.restoreButtonToolTip"); //$NON-NLS-1$
    maxButtonToolTip =
        UIManager.getString("InternalFrame.maxButtonToolTip"); //$NON-NLS-1$
  }

  /**
   * {@inheritDoc}
   */
  protected void createButtons() {
    iconButton = new TitleButton();
    iconButton.addActionListener(iconifyAction);
    maxButton = new TitleButton();
    maxButton.addActionListener(maximizeAction);
    closeButton = new TitleButton();
    closeButton.addActionListener(closeAction);
    setButtonIcons();
  }

  /**
   * {@inheritDoc}
   */
  protected LayoutManager createLayout() {
    return new TitlePaneLayout();
  }

  /**
   * {@inheritDoc}
   */
  protected void setButtonIcons() {
    if (frame.isIcon()) {
      iconButton.setIcon(minIcon);
      if ((restoreButtonToolTip != null) &&
          (restoreButtonToolTip.length() != 0)) {
        iconButton.setToolTipText(restoreButtonToolTip);
      }
      maxButton.setIcon(maxIcon);
      if ((maxButtonToolTip != null) && (maxButtonToolTip.length() != 0)) {
        maxButton.setToolTipText(maxButtonToolTip);
      }
    } else if (frame.isMaximum()) {
      iconButton.setIcon(iconIcon);
      maxButton.setIcon(minIcon);
      if ((restoreButtonToolTip != null) &&
          (restoreButtonToolTip.length() != 0)) {
        maxButton.setToolTipText(restoreButtonToolTip);
      }
    } else {
      iconButton.setIcon(iconIcon);
      if ((iconButtonToolTip != null) &&
          (iconButtonToolTip.length() != 0)) {
        iconButton.setToolTipText(iconButtonToolTip);
      }
      maxButton.setIcon(maxIcon);
      if ((maxButtonToolTip != null) && (maxButtonToolTip.length() != 0)) {
        maxButton.setToolTipText(maxButtonToolTip);
      }
    }
    closeButton.setIcon(closeIcon);
    if ((closeButtonToolTip != null) && (closeButtonToolTip.length() != 0)) {
      closeButton.setToolTipText(closeButtonToolTip);
    }
  }

  /**
   * {@inheritDoc}
   */
  protected void addSystemMenuItems(JMenu systemMenu) {
    JMenuItem mi = systemMenu.add(restoreAction);
    mi.setMnemonic('R');
    mi.setIcon(ImageFactory.createImageIconFromIcon(
                 SmoothIconFactory.getFrameMinimizeIcon(), mi));
    mi = systemMenu.add(moveAction);
    mi.setMnemonic('M');
    mi = systemMenu.add(sizeAction);
    mi.setMnemonic('S');
    mi = systemMenu.add(iconifyAction);
    mi.setIcon(ImageFactory.createImageIconFromIcon(
                 SmoothIconFactory.getFrameIconifyIcon(), mi));
    mi.setMnemonic('n');
    mi = systemMenu.add(maximizeAction);
    mi.setMnemonic('x');
    mi.setIcon(ImageFactory.createImageIconFromIcon(
                 SmoothIconFactory.getFrameMaximizeIcon(), mi));
    systemMenu.addSeparator();
    mi = systemMenu.add(closeAction);
    mi.setMnemonic('C');
    mi.setIcon(ImageFactory.createImageIconFromIcon(
                 SmoothIconFactory.getFrameCloseIcon(), mi));
  }

/*^**************************************************************************^*/
/*- Protected Methods.                                                       -*/
/*^**************************************************************************^*/

  /**
   * Returns internal frame.
   *
   * @return internal frame
   */
  protected JInternalFrame getInternalFrame() {
    return frame;
  }

  /**
   * Returns menu bar.
   *
   * @return menu bar
   */
  protected JMenuBar getMenuBar() {
    return menuBar;
  }

  /**
   * Returns close button.
   *
   * @return close button
   */
  protected JButton getCloseButton() {
    return closeButton;
  }

  /**
   * Returns max button.
   *
   * @return max button
   */
  protected JButton getMaxButton() {
    return maxButton;
  }

  /**
   * Returns icon button.
   *
   * @return icon button
   */
  protected JButton getIconButton() {
    return iconButton;
  }

/*^**************************************************************************^*/
/*- Inner Classes.                                                           -*/
/*^**************************************************************************^*/

  /**
   * Title button.
   */
  final class TitleButton extends JButton {
    /**
     * Constructs new TitleButton object.
     */
    public TitleButton() {
      super();
      setFocusPainted(false);
      setFocusable(false);
    }
    /**
     * {@inheritDoc}
     */
    public boolean isFocusable() {
      return false;
    }
    /**
     * {@inheritDoc}
     */
    public void requestFocus() {
      // empty method body
    }
    /**
     * {@inheritDoc}
     */
    public void paintBorder(Graphics g) {
      super.paintBorder(g);
      if (getInternalFrame().isSelected()) {
        g.setColor(titleABG2);
      } else {
        g.setColor(titleIBG2);
      }
      int wm1 = getWidth() - 1;
      int hm1 = getHeight() - 1;
      g.fillRect(0, 0, 1, 1);
      g.fillRect(wm1, 0, 1, 1);
      g.fillRect(0, hm1, 1, 1);
      g.fillRect(wm1, hm1, 1, 1);
    }
  }

  /**
   * TitlePane Layout.
   */
  final class TitlePaneLayout implements LayoutManager {
    /**
     * {@inheritDoc}
     */
    public void addLayoutComponent(String name, Component c) {
      // empty method body
    }
    /**
     * {@inheritDoc}
     */
    public void removeLayoutComponent(Component c) {
      // empty method body
    }
    /**
     * {@inheritDoc}
     */
    public Dimension preferredLayoutSize(Container c)  {
      return new Dimension(100, 18);
    }
    /**
     * {@inheritDoc}
     */
    public Dimension minimumLayoutSize(Container c) {
      return preferredLayoutSize(c);
    }
    /**
     * {@inheritDoc}
     */
    public void layoutContainer(Container c) {
      JInternalFrame frame = getInternalFrame();
      boolean leftToRight = AwtTools.isLeftToRight(frame);
      int w = getWidth();
      int x;
      if (leftToRight) {
        x = 2;
      } else {
        x = w - 16 - 2;
      }
      getMenuBar().setBounds(x, 1, 16, 16);
      if (leftToRight) {
        x = w - 16 - 2;
      } else {
        x = 2;
      }
      if (frame.isClosable()) {
        getCloseButton().setBounds(x, 2, 16, 14);
        if (leftToRight) {
          x += -17;
        } else {
          x += 17;
        }
      }
      if (frame.isMaximizable()) {
        getMaxButton().setBounds(x, 2, 16, 14);
        if (leftToRight) {
          x += -17;
        } else {
          x += 17;
        }
      }
      if (frame.isIconifiable()) {
        getIconButton().setBounds(x, 2, 16, 14);
      }
    }
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-25 20:15:39 Slobodan
   Initial version

*****************************************************************************^*/
