/*
 * @(#)SmoothListUI.java  1.0.0 / 2008-05-25
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.swing.plaf.smooth;

import java.awt.Component;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import javax.swing.JComponent;
import javax.swing.ListCellRenderer;
import javax.swing.ListModel;
import javax.swing.ListSelectionModel;
import javax.swing.border.Border;
import javax.swing.plaf.ComponentUI;
import javax.swing.plaf.UIResource;
import javax.swing.plaf.basic.BasicListUI;

import net.wrka.swing.graphics.GraphicUtils;

/**
 * ListUI.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public class SmoothListUI extends BasicListUI {

/*^**************************************************************************^*/
/*- Private Static Final Field.                                              -*/
/*^**************************************************************************^*/

  private static final Border CELL_BORDER;

/*^**************************************************************************^*/
/*- Static Initialization.                                                   -*/
/*^**************************************************************************^*/

  static {
    CELL_BORDER = new SmoothBorders.EmptyBorderUIResource(1, 3, 1, 3);
  }

/*^**************************************************************************^*/
/*- Private Fields.                                                          -*/
/*^**************************************************************************^*/

  private boolean oldOpaque;
  private Border oldBorder;

/*^**************************************************************************^*/
/*- Public Static Method.                                                    -*/
/*^**************************************************************************^*/

  /**
   * Factory method. Return ui for the specified component.
   *
   * @param c the component
   *
   * @return ui for the specified component
   */
  public static ComponentUI createUI(JComponent c) {
    return new SmoothListUI();
  }

/*^**************************************************************************^*/
/*- Overridden Methods.                                                      -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  public void update(Graphics g, JComponent c) {
    if (c.isOpaque()) {
      g.setColor(c.getBackground());
      g.fillRect(0, 0, c.getWidth(), c.getHeight());
      if (c.isOpaque()) {
        SmoothUtils.drawFastInvFlat3DEffectHorizontalLight(g, c);
      }
    }
    paint(g, c);
  }

  /**
   * {@inheritDoc}
   */
  public void paint(Graphics g, JComponent c) {
    SmoothLookAndFeel.getRenderingPolicy(this).apply((Graphics2D)g);
    super.paint(g, c);
  }

  /**
   * {@inheritDoc}
   */
  protected void paintCell(Graphics g, int row, Rectangle rowBounds,
                           ListCellRenderer cellRenderer, ListModel dataModel,
                           ListSelectionModel selModel, int leadIndex) {
    Object value = dataModel.getElementAt(row);
    boolean cellHasFocus = list.hasFocus() && (row == leadIndex);
    boolean isSelected = selModel.isSelectedIndex(row);
    Component rendererComponent = cellRenderer.
                                  getListCellRendererComponent(list, value, row,
                                                               isSelected,
                                                               cellHasFocus);
    if (rendererComponent instanceof JComponent) {
      JComponent rc = (JComponent)rendererComponent;
      rc.setBorder(CELL_BORDER);
    }
    rendererPane.paintComponent(g, rendererComponent, list, rowBounds.x,
                                rowBounds.y, rowBounds.width, rowBounds.height,
                                true);
    if (selModel.isSelectedIndex(row)) {
      g.setColor(SmoothLookAndFeel.getRolloverMaskColor());
      g.fillRect(rowBounds.x + 2, rowBounds.y + 1, rowBounds.width - 4,
                 rowBounds.height - 2);
      g.setColor(SmoothLookAndFeel.getRolloverOuterBorderColor());
      GraphicUtils.draw1PixRoundRect(g, rowBounds.x + 1, rowBounds.y,
                                     rowBounds.width - 3, rowBounds.height - 1);
    }
  }

  /**
   * {@inheritDoc}
   */
  protected void installDefaults() {
    super.installDefaults();
    adjustListCellRenderer(list.getCellRenderer());
  }

  /**
   * {@inheritDoc}
   */
  protected void uninstallDefaults() {
    super.uninstallDefaults();
    releaseListCellRenderer(list.getCellRenderer());
  }

  /**
   * {@inheritDoc}
   */
  protected void updateLayoutState() {
    super.updateLayoutState();
    cellWidth = cellWidth + 4;
  }

  /**
   * {@inheritDoc}
   */
  protected PropertyChangeListener createPropertyChangeListener() {
    return new SmoothPropertyChangeListener();
  }

/*^**************************************************************************^*/
/*- Protected Method.                                                        -*/
/*^**************************************************************************^*/

  /**
   * Adjusts list cell renderer.
   *
   * @param cellRenderer the cell renderer
   */
  protected void adjustListCellRenderer(ListCellRenderer cellRenderer) {
    if (cellRenderer instanceof JComponent) {
      JComponent componentCellRenderer = (JComponent)cellRenderer;
      oldBorder = componentCellRenderer.getBorder();
      oldOpaque = componentCellRenderer.isOpaque();
      componentCellRenderer.setBorder(CELL_BORDER);
      if (componentCellRenderer.getBackground() instanceof UIResource) {
        componentCellRenderer.setOpaque(false);
      }
    }
  }

  /**
   * Releses list cell renderer.
   *
   * @param cellRenderer the renderer
   */
  protected void releaseListCellRenderer(ListCellRenderer cellRenderer) {
    if (cellRenderer instanceof JComponent) {
      JComponent componentCellRenderer = (JComponent)cellRenderer;
      componentCellRenderer.setOpaque(oldOpaque);
      componentCellRenderer.setBorder(oldBorder);
    }
  }

/*^**************************************************************************^*/
/*- Inner Class.                                                             -*/
/*^**************************************************************************^*/

  /**
   * Smooth Property Change Listener.
   */
  protected class SmoothPropertyChangeListener extends PropertyChangeHandler {
    /**
     * {@inheritDoc}
     */
    public void propertyChange(PropertyChangeEvent e) {
      super.propertyChange(e);
      if ("cellRenderer".equals(e.getPropertyName())) { //$NON-NLS-1$
        releaseListCellRenderer((ListCellRenderer)e.getOldValue());
        adjustListCellRenderer((ListCellRenderer)e.getNewValue());
      }
    }
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-25 13:28:42 Slobodan
   Initial version

*****************************************************************************^*/
