/*
 * @(#)SmoothLookAndFeel.java  1.0.0 / 2008-05-25
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.swing.plaf.smooth;

import javax.swing.LookAndFeel;
import javax.swing.UIDefaults;
import javax.swing.UIManager;
import javax.swing.plaf.ComponentUI;
import javax.swing.plaf.InsetsUIResource;
import javax.swing.plaf.metal.MetalLookAndFeel;
import javax.swing.plaf.metal.MetalTheme;

import net.wrka.swing.borders.ThinBevelBorder;
import net.wrka.swing.graphics.GraphicUtils;
import net.wrka.swing.plaf.AlphaColorUIResource;
import net.wrka.swing.plaf.RenderingPolicy;
import net.wrka.swing.plaf.metal.MetalThemeManager;

/**
 * Smooth LookAndFeel.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public final class SmoothLookAndFeel extends MetalLookAndFeel {

/*^**************************************************************************^*/
/*- Static Fields.                                                           -*/
/*^**************************************************************************^*/

  /**
   * Look and feel name.
   */
  public static final String NAME = "Smooth"; //$NON-NLS-1$

  /**
   * Packge name.
   */
  public static final String PACKAGE_NAME =
                                   "net.wrka.swing.plaf.smooth."; //$NON-NLS-1$

/*^**************************************************************************^*/
/*- Private Static Fields.                                                   -*/
/*^**************************************************************************^*/

  private static SmoothTheme currentSmoothTheme;
  private static MetalTheme lastMetalTheme;
  private static boolean installed; // NOPMD

/*^**************************************************************************^*/
/*- Public Constructor.                                                      -*/
/*^**************************************************************************^*/

  /**
   * Constructs new SmoothLookAndFeel object.
   */
  public SmoothLookAndFeel() {
    super();
    if (!installed) {
      installed = true;
      UIManager.installLookAndFeel(
          new UIManager.LookAndFeelInfo(getName(), getClass().getName()));
    }
  }

/*^**************************************************************************^*/
/*- Public Static Methods.                                                   -*/
/*^**************************************************************************^*/

  /**
   * Returns previous metal theme.
   *
   * @return previous metal theme
   */
  public static MetalTheme getPreviousMetalTheme() {
    return lastMetalTheme;
  }

  /**
   * Returns current smooth theme.
   *
   * @return current shooth theme
   */
  public static SmoothTheme getCurrentSmoothTheme() {
    return currentSmoothTheme;
  }

  /**
   * Sets current theme.
   *
   * @param theme the theme
   */
  public static void setCurrentTheme(MetalTheme theme) {
    if (theme instanceof SmoothTheme) {
      setCurrentTheme((SmoothTheme)theme);
    } else {
      throw new IllegalArgumentException(
                  "Do not use MetalTheme; use SmoothTheme!"); //$NON-NLS-1$
    }
  }

  /**
   * Sets current theme.
   *
   * @param theme the theme
   */
  public static void setCurrentTheme(SmoothTheme theme) {
    if (theme == null) {
      currentSmoothTheme = new DefaultSmoothTheme();
      MetalLookAndFeel.setCurrentTheme(currentSmoothTheme);
    } else {
      MetalLookAndFeel.setCurrentTheme(theme);
      currentSmoothTheme = theme;
    }
  }

  /**
   * Returns rollover foreground.
   *
   * @return rollover foreground
   */
  public static AlphaColorUIResource getRolloverForeground() {
    return currentSmoothTheme.getRolloverForeground();
  }

  /**
   * Returns rollover background.
   *
   * @return rollover background
   */
  public static AlphaColorUIResource getRolloverBackground() {
    return GraphicUtils.brighterColorUIResource(getControl(),
                                                getRolloverBackgroundFactor());
  }

  /**
   * Returns rollover backgournd factor.
   *
   * @return rollover backgournd factor
   */
  public static float getRolloverBackgroundFactor() {
    return currentSmoothTheme.getRollOverBackgroundFactor();
  }

  /**
   * Returns component gradient color.
   *
   * @return component gradient color
   */
  public static AlphaColorUIResource getComponentGradientColor() {
    return currentSmoothTheme.getComponentGradientColor();
  }

  /**
   * Returns component gradient color reflection.
   *
   * @return component gradient color reflection
   */
  public static AlphaColorUIResource getComponentGradientReflection() {
    return currentSmoothTheme.getComponentGradientReflection();
  }

  /**
   * Returns component gradient color shadow.
   *
   * @return component gradient color shadow
   */
  public static AlphaColorUIResource getComponentGradientShadow() {
    return currentSmoothTheme.getComponentGradientShadow();
  }

  /**
   * Returns component gradient color light.
   *
   * @return component gradient color light
   */
  public static AlphaColorUIResource getComponentGradientLight() {
    return currentSmoothTheme.getComponentGradientLight();
  }

  /**
   * Returns component gradient color reflection light.
   *
   * @return component gradient color reflection light
   */
  public static AlphaColorUIResource getComponentGradientReflectionLight() {
    return currentSmoothTheme.getComponentGradientReflectionLight();
  }

  /**
   * Returns component gradient color shadow light.
   *
   * @return component gradient color shadow light
   */
  public static AlphaColorUIResource getComponentGradientShadowLight() {
    return currentSmoothTheme.getComponentGradientShadowLight();
  }

  /**
   * Returns the left backgournd color of the menu items.
   *
   * @return the left backgournd color of the menu items
   */
  public static AlphaColorUIResource getMenuItemLeftBackground() {
    return currentSmoothTheme.getMenuItemLeftBackground();
  }

  /**
   * Returns menu item right background.
   *
   * @return menu item right background.
   */
  public static AlphaColorUIResource getMenuItemRightBackground() {
    return currentSmoothTheme.getMenuItemRightBackground();
  }

  /**
   * Returns the width of the menu left part.
   *
   * @return the width of the menu left part
   */
  public static int getMenuItemLeftBackgroundWidth() {
    return currentSmoothTheme.getMenuItemLeftBackgroundWidth();
  }

  /**
   * Returns rollover mask color.
   *
   * @return rollover mask color
   */
  public static AlphaColorUIResource getRolloverMaskColor() {
    return currentSmoothTheme.getRolloverMaskColor();
  }

  /**
   * Returns rollover border color.
   *
   * @return rollover border color
   */
  public static AlphaColorUIResource getRolloverOuterBorderColor() {
    return GraphicUtils.darkerColorUIResource(
             currentSmoothTheme.getRolloverMaskColor(), 0.65f, 255);
  }

  /**
   * Returns rollover inner border dark color.
   *
   * @return rollover inner border dark color
   */
  public static AlphaColorUIResource getRolloverInnerBorderDarkColor() {
    return GraphicUtils.darkerColorUIResource(getControl(), 0.35f);
  }

  /**
   * Returns rollover inner border light color.
   *
   * @return rollover inner border light color
   */
  public static AlphaColorUIResource getRolloverInnerBorderLightColor() {
    return GraphicUtils.brighterColorUIResource(getControl(), 0.05f);
  }

  /**
   * Returns status bar highligh color.
   *
   * @return status bar highligh color
   */
  public static AlphaColorUIResource getStatusBarHighlightColor() {
    return GraphicUtils.darkerColorUIResource(getControlHighlight(), 0.1f);
  }

  /**
   * Returns internal frame title pane active background 1.
   *
   * @return internal frame title pane active background 1
   */
  public static AlphaColorUIResource getInternalFrameTitlePaneABG1() {
    return currentSmoothTheme.getInternalFrameTitlePaneABG1();
  }

  /**
   * Returns internal frame title pane active background 2.
   *
   * @return internal frame title pane active background 2
   */
  public static AlphaColorUIResource getInternalFrameTitlePaneABG2() {
    return currentSmoothTheme.getInternalFrameTitlePaneABG2();
  }

  /**
   * Returns rendering policy.
   *
   * @param cui the component
   *
   * @return  rendering policy
   */
  public static RenderingPolicy getRenderingPolicy(ComponentUI cui) {
    return currentSmoothTheme.getRenderingPolicy(cui);
  }

/*^**************************************************************************^*/
/*- Overridden Methods.                                                      -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  public String getID() {
    return NAME;
  }

  /**
   * {@inheritDoc}
   */
  public String getName() {
    return NAME;
  }

  /**
   * {@inheritDoc}
   */
  public String getDescription() {
    return "Smooth Look and Feel"; //$NON-NLS-1$
  }

  /**
   * {@inheritDoc}
   */
  public boolean isNativeLookAndFeel() {
    return false;
  }

  /**
   * {@inheritDoc}
   */
  public boolean isSupportedLookAndFeel() {
    return true;
  }

  /**
   * {@inheritDoc}
   */
  public void initialize() {
    lastMetalTheme = MetalThemeManager.getCurrentMetalTheme();
    //net.wrka.swing.popup.ShadowPopupFactory.install();
  }

  /**
   * {@inheritDoc}
   */
  public void uninitialize() {
    if (lastMetalTheme != null) {
      MetalLookAndFeel.setCurrentTheme(lastMetalTheme);
    }
    //net.wrka.swing.popup.ShadowPopupFactory.uninstall();
  }

  /**
   * {@inheritDoc}
   */
  protected void createDefaultTheme() {
    if (currentSmoothTheme == null) {
      currentSmoothTheme = new DefaultSmoothTheme();
    }
    setCurrentTheme(currentSmoothTheme);
  }

  /**
   * {@inheritDoc}
   */
  protected void initClassDefaults(UIDefaults table) {
    super.initClassDefaults(table);
    // CS:OFF
    Object[] uiDefaults = {
                  "ButtonUI", PACKAGE_NAME + "SmoothButtonUI",
                "CheckBoxUI", PACKAGE_NAME + "SmoothCheckBoxUI",
             "RadioButtonUI", PACKAGE_NAME + "SmoothRadioButtonUI",
            "ToggleButtonUI", PACKAGE_NAME + "SmoothToggleButtonUI",
                   "LabelUI", PACKAGE_NAME + "SmoothLabelUI",
               "TextFieldUI", PACKAGE_NAME + "SmoothTextFieldUI",
      "FormattedTextFieldUI", PACKAGE_NAME + "SmoothFormattedTextFieldUI",
                "TextAreaUI", PACKAGE_NAME + "SmoothTextAreaUI",
           "PasswordFieldUI", PACKAGE_NAME + "SmoothPasswordFieldUI",
                "ComboBoxUI", PACKAGE_NAME + "SmoothComboBoxUI",
                    "ListUI", PACKAGE_NAME + "SmoothListUI",
             "TableHeaderUI", PACKAGE_NAME + "SmoothTableHeaderUI",
                    "TreeUI", PACKAGE_NAME + "SmoothTreeUI",
                 "ToolBarUI", PACKAGE_NAME + "SmoothToolBarUI",
        "ToolBarSeparatorUI", PACKAGE_NAME + "SmoothToolBarSeparatorUI",
                 "MenuBarUI", PACKAGE_NAME + "SmoothMenuBarUI",
                "MenuItemUI", PACKAGE_NAME + "SmoothMenuItemUI",
     "RadioButtonMenuItemUI", PACKAGE_NAME + "SmoothButtonMenuItemUI",
                  "SliderUI", PACKAGE_NAME + "SmoothSliderUI",
               "SplitPaneUI", PACKAGE_NAME + "SmoothSplitPaneUI",
               "ScrollBarUI", PACKAGE_NAME + "SmoothScrollBarUI",
        "CheckBoxMenuItemUI", PACKAGE_NAME + "SmoothCheckBoxMenuItemUI",
                    "MenuUI", PACKAGE_NAME + "SmoothMenuUI",
      "PopupMenuSeparatorUI", PACKAGE_NAME + "SmoothPopupMenuSeparatorUI",
              "OptionPaneUI", PACKAGE_NAME + "SmoothOptionPaneUI",
              "TabbedPaneUI", PACKAGE_NAME + "SmoothTabbedPaneUI",
           "InternalFrameUI", PACKAGE_NAME + "SmoothInternalFrameUI",
             "DesktopIconUI", PACKAGE_NAME + "SmoothDesktopIconUI"
    };
    // CS:ON
    table.putDefaults(uiDefaults);
    //if (LibraryInfo.OS_WINDOWS) {
      //table.put("FileChooserUI", PACKAGE_NAME + "SmoothFileChooserUI");
    //}
  }

  /**
   * {@inheritDoc}
   */
  protected void initComponentDefaults(UIDefaults table) {
    super.initComponentDefaults(table);
    String iconFactoryClassName = SmoothIconFactory.class.getName();
    String bordersClassName = SmoothBorders.class.getName();
    // CS:OFF
    Object[] defaults = {
      "Button.border", new UIDefaults.ProxyLazyValue(bordersClassName,
                                                     "getButtonOvalBorder"),
      "Button.focusInputMap", new UIDefaults.LazyInputMap(new Object[] {
                   "SPACE", "pressed",
          "released SPACE", "released",
                   "ENTER", "pressed",
          "released ENTER", "released"
        }),
//      "CheckBox.border", new UIDefaults.ProxyLazyValue(bordersClassName,
//                                          "getCheckBoxBorder"),
      "CheckBox.icon", new UIDefaults.ProxyLazyValue(iconFactoryClassName,
                                                     "getCheckBoxIcon"),
      "RadioButton.icon", new UIDefaults.ProxyLazyValue(iconFactoryClassName,
                                                        "getRadioButtonIcon"),
      "ToggleButton.border", new UIDefaults.ProxyLazyValue(bordersClassName,
                                              "getToggleButtonOvalBorder"),
      "DesktopIcon.width", new Integer(130),
      "InternalFrame.maximizeIcon", new UIDefaults.ProxyLazyValue(
                                                     iconFactoryClassName,
                                                     "getFrameMaximizeIcon"),
      "InternalFrame.minimizeIcon", new UIDefaults.ProxyLazyValue(
                                                     iconFactoryClassName,
                                                     "getFrameMinimizeIcon"),
      "InternalFrame.iconifyIcon", new UIDefaults.ProxyLazyValue(
                                                    iconFactoryClassName,
                                                    "getFrameIconifyIcon"),
      "InternalFrame.closeIcon", new UIDefaults.ProxyLazyValue(
                                                  iconFactoryClassName,
                                                  "getFrameCloseIcon"),
      "InternalFrame.icon", new UIDefaults.LazyValue() {
        public Object createValue(UIDefaults table) {
          return LookAndFeel.makeIcon(getClass(), "icons/JavaCup.png");
      }},
      "InternalFrame.border", new UIDefaults.ProxyLazyValue(
          bordersClassName, "getInternalFrameBorder"),
      "TextField.border", new UIDefaults.ProxyLazyValue(bordersClassName,
                                                        "getTextFieldBorder"),
      "FormattedTextField.border", new UIDefaults.ProxyLazyValue(
                                        bordersClassName, "getTextFieldBorder"),
      "PasswordField.border", new UIDefaults.ProxyLazyValue(bordersClassName,
                                               "getTextFieldBorder"),
      "Tree.closedIcon", new UIDefaults.ProxyLazyValue(iconFactoryClassName,
                                                       "getTreeClosedIcon"),
      "Tree.openIcon", new UIDefaults.ProxyLazyValue(iconFactoryClassName,
                                                     "getTreeOpenIcon"),
      "Tree.leafIcon", new UIDefaults.ProxyLazyValue(iconFactoryClassName,
                                                     "getTreeLeafIcon"),
      "Tree.expandedIcon", new UIDefaults.ProxyLazyValue(iconFactoryClassName,
                                                         "getExpandedTreeIcon"),
      "Tree.collapsedIcon", new UIDefaults.ProxyLazyValue(
                                  iconFactoryClassName, "getCollapsedTreeIcon"),
      "TabbedPane.focus", getFocusColor(),
      "TabbedPane.tabInsets", new InsetsUIResource(0, 4, 1, 4),
      "StatusBarPanelUI.statusBorder", new UIDefaults.ProxyLazyValue(
          bordersClassName, "getStatusBarPanelBorder"),
      "ScrollBar.width", new Integer(16),
      "ScrollPane.border", new UIDefaults.LazyValue() {
        public Object createValue(UIDefaults table) {
          return ThinBevelBorder.getSharedLoweredInstance();
      }},
      "Slider.thumb", getPrimaryControlShadow(),
      //"TitledBorder.border", ...,
      "Menu.submenuPopupOffsetX", new Integer(-2),
      "Menu.submenuPopupOffsetY", new Integer(2),
      "MenuItem.acceleratorDelimiter", "+",
      "MenuItem.border", new UIDefaults.ProxyLazyValue(bordersClassName,
                                                       "getMenuItemBorder"),
      "CheckBoxMenuItem.checkIcon", new UIDefaults.ProxyLazyValue(
                          iconFactoryClassName, "getCheckBoxMenuItemIcon"),
      "RadioButtonMenuItem.checkIcon", new UIDefaults.ProxyLazyValue(
                            iconFactoryClassName, "getRadioButtonMenuItemIcon"),
      "List.selectionBackground", table.get("textHighlight"),
      "List.selectionForeground", table.get("textHighlightText"),
      "List.focusCellHighlightBorder", null,
      "PopupMenu.border", new UIDefaults.ProxyLazyValue(bordersClassName,
                                                        "getPopupMenuBorder"),
      "Table.gridColor", SmoothLookAndFeel.getControl(),
      "TableHeader.cellBorder", new UIDefaults.ProxyLazyValue(bordersClassName,
                                                 "getTableHeaderBorder"),
      "ToolBar.border", new UIDefaults.ProxyLazyValue(bordersClassName,
                                                      "getToolBarBorder"),
      "ToolBar.separatorSize", null,
      "ToolTip.hideAccelerator", Boolean.TRUE,
    };
    // CS:ON
    table.putDefaults(defaults);
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-25 20:54:36 Slobodan
   Initial version

*****************************************************************************^*/
