/*
 * @(#)SmoothMenuItemUI.java  1.0.0 / 2008-05-25
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.swing.plaf.smooth;

import java.awt.Color;
import java.awt.Component;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Insets;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.InputEvent;
import java.awt.event.KeyEvent;
import java.awt.event.MouseEvent;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.lang.reflect.Method;

import javax.swing.AbstractAction;
import javax.swing.ActionMap;
import javax.swing.ButtonModel;
import javax.swing.Icon;
import javax.swing.InputMap;
import javax.swing.JComponent;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.KeyStroke;
import javax.swing.LookAndFeel;
import javax.swing.MenuElement;
import javax.swing.MenuSelectionManager;
import javax.swing.SwingUtilities;
import javax.swing.UIManager;
import javax.swing.event.MenuDragMouseEvent;
import javax.swing.event.MenuDragMouseListener;
import javax.swing.event.MenuKeyEvent;
import javax.swing.event.MenuKeyListener;
import javax.swing.event.MouseInputListener;
import javax.swing.plaf.ActionMapUIResource;
import javax.swing.plaf.ComponentInputMapUIResource;
import javax.swing.plaf.ComponentUI;
import javax.swing.plaf.InsetsUIResource;
import javax.swing.plaf.MenuItemUI;
import javax.swing.plaf.UIResource;
import javax.swing.plaf.basic.BasicHTML;
import javax.swing.text.View;

import net.wrka.awt.AwtTools;
import net.wrka.swing.graphics.GraphicUtils;
import net.wrka.swing.graphics.ImageFactory;
import net.wrka.util.CommonConstants;

/**
 * MenuItemUI.
 * <p>
 *  <tt>The code is based on Sun's javax.swing.plaf.basic.BasicMenuItem</tt>
 * </p>
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public class SmoothMenuItemUI extends MenuItemUI {

  // CHANGE: Optimise and clean.

/*^**************************************************************************^*/
/*- Package-Private Static Final Fields.                                     -*/
/*^**************************************************************************^*/

  static final String MAX_TEXT_WIDTH;
  static final String MAX_ACC_WIDTH;
  static final int BUTTON_MASK;
  static final Method INSTALL_PROPERTY_METHOD;
  static final Rectangle ICON_RECT;
  static final Rectangle TEXT_RECT;
  static final Rectangle ACCELERATOR_RECT;
  static final Rectangle CHECK_ICON_RECT;
  static final Rectangle ARROW_ICON_RECT;
  static final Rectangle VIEW_RECT;
  static final Rectangle RECT;

/*^**************************************************************************^*/
/*- Static Initialization.                                                   -*/
/*^**************************************************************************^*/

  static {
    MAX_TEXT_WIDTH = "maxTextWidth"; //$NON-NLS-1$
    MAX_ACC_WIDTH = "maxAccWidth"; //$NON-NLS-1$
    BUTTON_MASK = InputEvent.BUTTON1_MASK | InputEvent.BUTTON2_MASK |
                  InputEvent.BUTTON3_MASK;
    Method m = null;
    try {
      Class[] argsClass = new Class[] {
        JComponent.class, String.class, Object.class
      };
      m = LookAndFeel.class.getMethod("installProperty", //$NON-NLS-1$
                                      argsClass);
    } catch (Throwable t) { // NOPMD
      // empty catch block
    }
    INSTALL_PROPERTY_METHOD = m;
    ICON_RECT = new Rectangle();
    TEXT_RECT = new Rectangle();
    ACCELERATOR_RECT = new Rectangle();
    CHECK_ICON_RECT = new Rectangle();
    ARROW_ICON_RECT = new Rectangle();
    VIEW_RECT = new Rectangle(Short.MAX_VALUE, Short.MAX_VALUE);
    RECT = new Rectangle();
  }

/*^**************************************************************************^*/
/*- Packge-Private Fields.                                                   -*/
/*^**************************************************************************^*/

  JMenuItem menuItem;
  Color selectionBackground;
  Color selectionForeground;
  Color disabledForeground;
  Color acceleratorForeground;
  Color acceleratorSelectionForeground;
  String acceleratorDelimiter;
  int defaultTextIconGap;
  Font acceleratorFont;
  MouseInputListener mouseInputListener;
  MenuDragMouseListener menuDragMouseListener;
  MenuKeyListener menuKeyListener;
  PropertyChangeListener propertyChangeListener;
  Icon arrowIcon;
  Icon checkIcon;
  InputMap windowInputMap;
  int menuItemLeftBgdWidth;
  boolean oldBorderPainted;

/*^**************************************************************************^*/
/*- Public Static Method.                                                    -*/
/*^**************************************************************************^*/

  /**
   * Factory method. Return ui for the specified component.
   *
   * @param c the component
   *
   * @return ui for the specified component
   */
  public static ComponentUI createUI(JComponent c) {
    return new SmoothMenuItemUI();
  }

/*^**************************************************************************^*/
/*- Overridden Methods.                                                      -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  public void installUI(JComponent c) {
    menuItem = (JMenuItem)c;
    installDefaults();
    installComponents(menuItem);
    installListeners();
    installKeyboardActions();
  }

  /**
   * {@inheritDoc}
   */
  public void uninstallUI(JComponent c) {
    uninstallDefaults();
    uninstallComponents(menuItem);
    uninstallListeners();
    uninstallKeyboardActions();
    Container parent = menuItem.getParent();
    if ((parent instanceof JComponent) && useCheckAndArrow()) {
      JComponent p = (JComponent)parent;
      p.putClientProperty(MAX_ACC_WIDTH, null);
      p.putClientProperty(MAX_TEXT_WIDTH, null);
    }
  }

  /**
   * {@inheritDoc}
   */
  public Dimension getPreferredSize(JComponent c) {
    return getPreferredMenuItemSize(c, checkIcon, arrowIcon,
                                    defaultTextIconGap);
  }

  /**
   * {@inheritDoc}
   */
  public Dimension getMinimumSize(JComponent c) {
    Dimension d = null;
    View v = (View)c.getClientProperty(BasicHTML.propertyKey);
    if (v != null) {
      d = getPreferredSize(c);
      d.width -= v.getPreferredSpan(View.X_AXIS) -
                 v.getMinimumSpan(View.X_AXIS);
    }
    return d;
  }

  /**
   * {@inheritDoc}
   */
  public Dimension getMaximumSize(JComponent c) {
    Dimension d = null;
    View v = (View)c.getClientProperty(BasicHTML.propertyKey);
    if (v != null) {
      d = getPreferredSize(c);
      d.width += v.getMaximumSpan(View.X_AXIS) -
                 v.getPreferredSpan(View.X_AXIS);
    }
    return d;
  }

  /**
   * {@inheritDoc}
   */
  public void update(Graphics g, JComponent c) {
    paint(g, c);
  }

  /**
   * {@inheritDoc}
   */
  public void paint(Graphics g, JComponent c) {
    SmoothLookAndFeel.getRenderingPolicy(this).apply((Graphics2D)g);
    paintMenuItem(g, c, checkIcon, arrowIcon, selectionForeground,
                  defaultTextIconGap);
  }

/*^**************************************************************************^*/
/*- Protected Methods.                                                       -*/
/*^**************************************************************************^*/

  /**
   * Returns property prefix.
   *
   * @return  property prefix
   */
  protected String getPropertyPrefix() {
    return "MenuItem"; //$NON-NLS-1$
  }

  /**
   * Installs defaults.
   */
  protected void installDefaults() { // NOPMD
    String prefix = getPropertyPrefix();
    acceleratorFont = UIManager.getFont(
                                     "MenuItem.acceleratorFont"); //$NON-NLS-1$
    Insets margin = menuItem.getMargin();
    if ((margin == null) || (margin instanceof UIResource)) {
      menuItem.setMargin(new InsetsUIResource(0, 0, 0, 0));
    }
    defaultTextIconGap = 4;
    if (menuItem.isBorderPainted()) {
      if (INSTALL_PROPERTY_METHOD == null) {
        oldBorderPainted = true;
        menuItem.setOpaque(false);
      } else {
        installProperty(menuItem, "borderPainted", Boolean.FALSE); //$NON-NLS-1$
      }
    }
    LookAndFeel.installBorder(menuItem, prefix + ".border"); //$NON-NLS-1$
    LookAndFeel.installColorsAndFont(menuItem,
                                     prefix.concat(".background"), //$NON-NLS-1$
                                     prefix.concat(".foreground"), //$NON-NLS-1$
                                     prefix.concat(".font")); //$NON-NLS-1$
    menuItemLeftBgdWidth = SmoothLookAndFeel.getMenuItemLeftBackgroundWidth();
    if (menuItemLeftBgdWidth == 0) {
      menuItemLeftBgdWidth = 24;
    }
    if ((selectionBackground == null) ||
        (selectionBackground instanceof UIResource)) {
      selectionBackground =
        UIManager.getColor(prefix.concat(".selectionBackground")); //$NON-NLS-1$
    }
    if ((selectionForeground == null) ||
        (selectionForeground instanceof UIResource)) {
      selectionForeground =
        UIManager.getColor(prefix.concat(".selectionForeground")); //$NON-NLS-1$
    }
    if ((disabledForeground == null) ||
        (disabledForeground instanceof UIResource)) {
      disabledForeground =
        UIManager.getColor(prefix.concat(".disabledForeground")); //$NON-NLS-1$
    }
    if ((acceleratorForeground == null) ||
        (acceleratorForeground instanceof UIResource)) {
      acceleratorForeground =
        UIManager.getColor(prefix.concat(
                                      ".acceleratorForeground")); //$NON-NLS-1$
    }
    if ((acceleratorSelectionForeground == null) ||
        (acceleratorSelectionForeground instanceof UIResource)) {
      acceleratorSelectionForeground =
        UIManager.getColor(prefix.concat(
                             ".acceleratorSelectionForeground")); //$NON-NLS-1$
    }
    acceleratorDelimiter =
        UIManager.getString("MenuItem.acceleratorDelimiter"); //$NON-NLS-1$
    if (acceleratorDelimiter == null) {
      acceleratorDelimiter = "+"; //$NON-NLS-1$
    }
    if ((arrowIcon == null) || (arrowIcon instanceof UIResource)) {
      arrowIcon = UIManager.getIcon(prefix.concat(".arrowIcon")); //$NON-NLS-1$
    }
    if ((checkIcon == null) || (checkIcon instanceof UIResource)) {
      checkIcon = UIManager.getIcon(prefix.concat(".checkIcon")); //$NON-NLS-1$
    }
  }

  /**
   * Installs comonents.
   *
   * @param menuItem the menu item
   */
  protected void installComponents(JMenuItem menuItem) {
    BasicHTML.updateRenderer(menuItem, menuItem.getText());
  }

  /**
   * Installs listeners.
   */
  protected void installListeners() {
    mouseInputListener = createMouseInputListener(menuItem);
    if (mouseInputListener != null) {
      menuItem.addMouseListener(mouseInputListener);
      menuItem.addMouseMotionListener(mouseInputListener);
    }
    menuDragMouseListener = createMenuDragMouseListener(menuItem);
    if (menuDragMouseListener != null) {
      menuItem.addMenuDragMouseListener(menuDragMouseListener);
    }
    menuKeyListener = createMenuKeyListener(menuItem);
    if (menuKeyListener != null) {
      menuItem.addMenuKeyListener(menuKeyListener);
    }
    propertyChangeListener = createPropertyChangeListener(menuItem);
    if (propertyChangeListener != null) {
      menuItem.addPropertyChangeListener(propertyChangeListener);
    }
  }

  /**
   * Installs keyboard actions.
   */
  protected void installKeyboardActions() {
    ActionMap actionMap = getActionMap();
    SwingUtilities.replaceUIActionMap(menuItem, actionMap);
    updateAcceleratorBinding();
  }

  /**
   * Uninstalls defautls.
   */
  protected void uninstallDefaults() {
    LookAndFeel.uninstallBorder(menuItem);
    if (menuItem.getMargin() instanceof UIResource) {
      menuItem.setMargin(null);
    }
    if (arrowIcon instanceof UIResource) {
      arrowIcon = null;
    }
    if (checkIcon instanceof UIResource) {
      checkIcon = null;
    }
    if (oldBorderPainted) {
      menuItem.setBorderPainted(true);
    }
  }

  /**
   * Uninstalls components.
   *
   * @param menuItem the menu item
   */
  protected void uninstallComponents(JMenuItem menuItem) {
    BasicHTML.updateRenderer(menuItem, CommonConstants.STRING_EMPTY);
  }

  /**
   * Uninstalls listeners.
   */
  protected void uninstallListeners() {
    if (mouseInputListener != null) {
      menuItem.removeMouseListener(mouseInputListener);
      menuItem.removeMouseMotionListener(mouseInputListener);
    }
    if (menuDragMouseListener != null) {
      menuItem.removeMenuDragMouseListener(menuDragMouseListener);
    }
    if (menuKeyListener != null) {
      menuItem.removeMenuKeyListener(menuKeyListener);
    }
    if (propertyChangeListener != null) {
      menuItem.removePropertyChangeListener(propertyChangeListener);
    }
    mouseInputListener = null;
    menuDragMouseListener = null;
    menuKeyListener = null;
    propertyChangeListener = null;
  }

  /**
   * Uninstalls keyboard actions.
   */
  protected void uninstallKeyboardActions() {
    SwingUtilities.replaceUIActionMap(menuItem, null);
    if (windowInputMap != null) {
      SwingUtilities.replaceUIInputMap(menuItem,
                                       JComponent.WHEN_IN_FOCUSED_WINDOW, null);
      windowInputMap = null;
    }
  }

  /**
   * Creates mouse input listener.
   *
   * @param c the component
   *
   * @return mouse input listener
   */
  protected MouseInputListener createMouseInputListener(JComponent c) {
    return new MouseInputHandler();
  }

  /**
   * Creates menu drag mouse listener.
   *
   * @param c the component
   *
   * @return menu drag mouse listener
   */
  protected MenuDragMouseListener createMenuDragMouseListener(JComponent c) {
    return new MenuDragMouseHandler();
  }

  /**
   * Creates menu key listener.
   *
   * @param c the component
   *
   * @return menu key listener
   */
  protected MenuKeyListener createMenuKeyListener(JComponent c) {
    return new MenuKeyHandler();
  }

  /**
   * Creates property change listener.
   *
   * @param c the component
   *
   * @return property change listener
   */
  protected PropertyChangeListener createPropertyChangeListener(JComponent c) {
    return new PropertyChangeHandler();
  }

  /**
   * Returns action map.
   *
   * @return acton map
   */
  protected ActionMap getActionMap() {
    String propertyPrefix = getPropertyPrefix();
    String uiKey = propertyPrefix.concat(".actionMap"); //$NON-NLS-1$
    ActionMap am = (ActionMap)UIManager.get(uiKey);
    if (am == null) {
      am = createActionMap();
      UIManager.getLookAndFeelDefaults().put(uiKey, am);
    }
    return am;
  }

  /**
   * Creates action map.
   *
   * @return action map
   */
  protected ActionMap createActionMap() {
    ActionMap map = new ActionMapUIResource();
    map.put("doClick", new ClickAction()); //$NON-NLS-1$
    return map;
  }

  /**
   * Creates input map.
   *
   * @param condition the condition
   *
   * @return input map
   */
  protected InputMap createInputMap(int condition) {
    if (condition == JComponent.WHEN_IN_FOCUSED_WINDOW) {
      return new ComponentInputMapUIResource(menuItem);
    }
    return null;
  }

  /**
   * Updates accelerator binding.
   */
  protected void updateAcceleratorBinding() {
    KeyStroke accelerator = menuItem.getAccelerator();
    if (windowInputMap != null) {
      windowInputMap.clear();
    }
    if (accelerator != null) {
      if (windowInputMap == null) {
        windowInputMap = createInputMap(JComponent.WHEN_IN_FOCUSED_WINDOW);
        SwingUtilities.replaceUIInputMap(menuItem,
                                         JComponent.WHEN_IN_FOCUSED_WINDOW,
                                         windowInputMap);
      }
      windowInputMap.put(accelerator, "doClick"); //$NON-NLS-1$
    }
  }

  /**
   * Returns <tt>true</tt> if uses check and arror mark.
   *
   * @return <tt>true</tt> if uses check and arror mark
   */
  protected boolean useCheckAndArrow() {
    return !((menuItem instanceof JMenu) && ((JMenu)menuItem).isTopLevelMenu());
  }

  /**
   * Returns menu path.
   *
   * @return menu path
   */
  protected MenuElement[] getPath() {
    MenuSelectionManager m = MenuSelectionManager.defaultManager();
    MenuElement[] oldPath = m.getSelectedPath();
    MenuElement[] newPath;
    int i = oldPath.length;
    if (i == 0) {
      return new MenuElement[0];
    }
    Component parent = menuItem.getParent();
    if (oldPath[i - 1].getComponent() == parent) {
      newPath = new MenuElement[i + 1];
      System.arraycopy(oldPath, 0, newPath, 0, i);
      newPath[i] = menuItem;
    } else {
      int j;
      for (j = oldPath.length - 1; j >= 0; j--) {
        if (oldPath[j].getComponent() == parent) {
          break;
        }
      }
      newPath = new MenuElement[j + 2];
      System.arraycopy(oldPath, 0, newPath, 0, j + 1);
      newPath[j + 1] = menuItem;
    }
    return newPath;
  }

  /**
   * Perform menu item layout.
   *
   * @param fm the font metrics
   * @param text the text
   * @param fmAccel the accel font metrics
   * @param acceleratorText the accelerator text
   * @param icon the icon
   * @param checkIcon the check icon
   * @param arrowIcon the arrow icon
   * @param verticalAlignment the vertical alignment
   * @param horizontalAlignment the horizontal alignment
   * @param verticalTextPosition the vertical text position
   * @param horizontalTextPosition the horizontal text position
   * @param viewRect the view rectangle
   * @param iconRect the icon rectangle
   * @param textRect the text rectangle
   * @param acceleratorRect the accelerator rectangle
   * @param checkIconRect the check icon rectangle
   * @param arrowIconRect the arrow icon rectangle
   * @param textIconGap the text icon gap
   * @param menuItemGap the menu  tem gap
   *
   * @return menu item string
   */
  protected String layoutMenuItem(FontMetrics fm, String text, // NOPMD
                                  FontMetrics fmAccel, String acceleratorText,
                                  Icon icon, Icon checkIcon, Icon arrowIcon,
                                  int verticalAlignment,
                                  int horizontalAlignment,
                                  int verticalTextPosition,
                                  int horizontalTextPosition,
                                  Rectangle viewRect, Rectangle iconRect,
                                  Rectangle textRect, Rectangle acceleratorRect,
                                  Rectangle checkIconRect,
                                  Rectangle arrowIconRect, int textIconGap,
                                  int menuItemGap) {
    SwingUtilities.layoutCompoundLabel(menuItem, fm, text, icon,
                                       verticalAlignment, horizontalAlignment,
                                       verticalTextPosition,
                                       horizontalTextPosition, viewRect,
                                       iconRect, textRect, textIconGap);
    if ((acceleratorText == null) || (acceleratorText.length() == 0)) {
      acceleratorRect.width = 0;
      acceleratorRect.height = 0;
    } else {
      acceleratorRect.width = SwingUtilities.computeStringWidth(fmAccel,
                                              acceleratorText);
      acceleratorRect.height = fmAccel.getHeight();
    }
    if (useCheckAndArrow()) {
      if (checkIcon == null) {
        checkIconRect.width = 0;
        checkIconRect.height = 0;
      } else {
        checkIconRect.width = checkIcon.getIconWidth();
        checkIconRect.height = checkIcon.getIconHeight();
      }
      if (arrowIcon == null) {
        arrowIconRect.width = 0;
        arrowIconRect.height = 0;
      } else {
        arrowIconRect.width = arrowIcon.getIconWidth();
        arrowIconRect.height = arrowIcon.getIconHeight();
      }
    }
    Rectangle labelRect = iconRect.union(textRect);
    if (AwtTools.isLeftToRight(menuItem)) {
      textRect.x += menuItemGap;
      iconRect.x += menuItemGap;
      acceleratorRect.x = viewRect.x + viewRect.width - arrowIconRect.width -
                          menuItemGap - acceleratorRect.width;
      if (useCheckAndArrow()) {
        checkIconRect.x = viewRect.x + menuItemGap;
        textRect.x += menuItemGap + checkIconRect.width;
        iconRect.x += menuItemGap + checkIconRect.width;
        arrowIconRect.x = viewRect.x + viewRect.width - menuItemGap -
                          arrowIconRect.width;
      }
    } else {
      textRect.x -= menuItemGap;
      iconRect.x -= menuItemGap;
      acceleratorRect.x = viewRect.x + arrowIconRect.width + menuItemGap;
      if (useCheckAndArrow()) {
        checkIconRect.x = viewRect.x + viewRect.width - menuItemGap -
                          checkIconRect.width;
        textRect.x -= menuItemGap + checkIconRect.width;
        iconRect.x -= menuItemGap + checkIconRect.width;
        arrowIconRect.x = viewRect.x + menuItemGap;
      }
    }
    acceleratorRect.y = labelRect.y + (labelRect.height >> 1) -
                        (acceleratorRect.height >> 1);
    if (useCheckAndArrow()) {
      arrowIconRect.y = labelRect.y + (labelRect.height >> 1) -
                        (arrowIconRect.height >> 1);
      checkIconRect.y = labelRect.y + (labelRect.height >> 1) -
                        (checkIconRect.height >> 1);
    }
    Component miParent = menuItem.getParent();
    if ((miParent instanceof JComponent) && useCheckAndArrow()) {
      textRect.x = menuItemLeftBgdWidth + defaultTextIconGap;
      iconRect.x = ((menuItemLeftBgdWidth - iconRect.width) >> 1) + 1;
    }
    return text;
  }

  /**
   * Returns preferred menu item size.
   *
   * @param c the component
   * @param checkIcon the check icon
   * @param arrowIcon the arrow icon
   * @param defaultTextIconGap the default text icon gap
   *
   * @return preferred menu item size
   */
  protected Dimension getPreferredMenuItemSize(JComponent c, // NOPMD
                                               Icon checkIcon, Icon arrowIcon,
                                               int defaultTextIconGap) {
    JMenuItem b = (JMenuItem)c;
    Icon icon = b.getIcon();
    String text = b.getText();
    KeyStroke accelerator = b.getAccelerator();
    StringBuffer sb = new StringBuffer();
    if (accelerator != null) {
      int modifiers = accelerator.getModifiers();
      if (modifiers > 0) {
        sb.append(KeyEvent.getKeyModifiersText(modifiers));
        sb.append(acceleratorDelimiter);
      }
      int keyCode = accelerator.getKeyCode();
      if (keyCode == 0) {
        sb.append(accelerator.getKeyChar());
      } else {
        sb.append(KeyEvent.getKeyText(keyCode));
      }
    }
    String acceleratorText = sb.toString();
    Font font = b.getFont();
    FontMetrics fm = b.getFontMetrics(font);
    FontMetrics fmAccel = b.getFontMetrics(acceleratorFont);
    resetRects();
    int textIconGap;
    if (text == null) {
      textIconGap = 0;
    } else {
      textIconGap = defaultTextIconGap;
    }
    layoutMenuItem(fm, text, fmAccel, acceleratorText, icon, checkIcon,
                   arrowIcon, b.getVerticalAlignment(),
                   b.getHorizontalAlignment(), b.getVerticalTextPosition(),
                   b.getHorizontalTextPosition(), VIEW_RECT, ICON_RECT,
                   TEXT_RECT, ACCELERATOR_RECT, CHECK_ICON_RECT,
                   ARROW_ICON_RECT, textIconGap, defaultTextIconGap);
    RECT.setBounds(TEXT_RECT);
    SwingUtilities.computeUnion(ICON_RECT.x, ICON_RECT.y, ICON_RECT.width,
                                ICON_RECT.height, RECT);
    Container parent = menuItem.getParent();
    if ((parent instanceof JComponent) && useCheckAndArrow()) {
      JComponent p = (JComponent)parent;
      Integer maxTextWidth = (Integer)p.getClientProperty(MAX_TEXT_WIDTH);
      Integer maxAccWidth = (Integer)p.getClientProperty(MAX_ACC_WIDTH);
      int maxTextValue;
      if (maxTextWidth == null) {
        maxTextValue = 0;
      } else {
        maxTextValue = maxTextWidth.intValue();
      }
      int maxAccValue;
      if (maxAccWidth == null) {
        maxAccValue = 0;
      } else {
        maxAccValue = maxAccWidth.intValue();
      }
      if (RECT.width < maxTextValue) {
        RECT.width = maxTextValue;
      } else {
        p.putClientProperty(MAX_TEXT_WIDTH, new Integer(RECT.width));
      }
      if (ACCELERATOR_RECT.width > maxAccValue) {
        maxAccValue = ACCELERATOR_RECT.width;
        p.putClientProperty(MAX_ACC_WIDTH, new Integer(ACCELERATOR_RECT.width));
      }
      RECT.width += maxAccValue;
      RECT.width += defaultTextIconGap;
    }
    if (useCheckAndArrow()) {
      RECT.width += CHECK_ICON_RECT.width;
      RECT.width += defaultTextIconGap;
      RECT.width += defaultTextIconGap;
      RECT.width += ARROW_ICON_RECT.width;
    }
    RECT.width += (defaultTextIconGap << 1);
    Insets insets = b.getInsets();
    if (insets != null) {
      RECT.width += insets.left + insets.right;
      RECT.height += insets.top + insets.bottom;
    }
    if ((RECT.width % 2) == 0) {
      RECT.width++;
    }
    if ((RECT.height % 2) == 0) {
      RECT.height++;
    }
    if ((RECT.height < 23) && (parent instanceof JComponent) &&
        useCheckAndArrow()) {
      RECT.height = 23;
    }
    return RECT.getSize();
  }

  /**
   * Paints menu item.
   *
   * @param g the graphics
   * @param c the component
   * @param checkIcon the chek icon
   * @param arrowIcon the arrow icon
   * @param foreground the foreground
   * @param defaultTextIconGap the default text icon gap
   */
  protected void paintMenuItem(Graphics g, JComponent c, // NOPMD
                               Icon checkIcon, Icon arrowIcon, Color foreground,
                               int defaultTextIconGap) {
    JMenuItem b = (JMenuItem)c;
    ButtonModel model = b.getModel();
    int menuWidth = b.getWidth();
    int menuHeight = b.getHeight();
    Insets i = c.getInsets();
    resetRects();
    VIEW_RECT.setBounds(0, 0, menuWidth, menuHeight);
    VIEW_RECT.x += i.left;
    VIEW_RECT.y += i.top;
    VIEW_RECT.width -= (i.right + VIEW_RECT.x);
    VIEW_RECT.height -= (i.bottom + VIEW_RECT.y);
    Font holdf = g.getFont();
    Font f = c.getFont();
    g.setFont(f);
    FontMetrics fm = g.getFontMetrics(f);
    FontMetrics fmAccel = g.getFontMetrics(acceleratorFont);
    KeyStroke accelerator = b.getAccelerator();
    StringBuffer sb = new StringBuffer();
    if (accelerator != null) {
      int modifiers = accelerator.getModifiers();
      if (modifiers > 0) {
        sb.append(KeyEvent.getKeyModifiersText(modifiers));
        sb.append(acceleratorDelimiter);
      }
      int keyCode = accelerator.getKeyCode();
      if (keyCode == 0) {
        sb.append(accelerator.getKeyChar());
      } else {
        sb.append(KeyEvent.getKeyText(keyCode));
      }
    }
    String acceleratorText = sb.toString();
    int textIconGap;
    String buttonText = b.getText();
    if (buttonText == null) {
      textIconGap = 0;
    } else {
      textIconGap = defaultTextIconGap;
    }
    String text = layoutMenuItem(fm, buttonText, fmAccel, acceleratorText,
                                 b.getIcon(), checkIcon, arrowIcon,
                                 b.getVerticalAlignment(),
                                 b.getHorizontalAlignment(),
                                 b.getVerticalTextPosition(),
                                 b.getHorizontalTextPosition(),
                                 VIEW_RECT, ICON_RECT, TEXT_RECT,
                                 ACCELERATOR_RECT, CHECK_ICON_RECT,
                                 ARROW_ICON_RECT, textIconGap,
                                 defaultTextIconGap);
    paintBackground(g, b);
    Color holdc = g.getColor();
    if (checkIcon != null) {
      if (model.isArmed() || ((c instanceof JMenu) && model.isSelected())) {
        g.setColor(foreground);
      } else {
        g.setColor(holdc);
      }
      if (useCheckAndArrow()) {
        checkIcon.paintIcon(c, g, CHECK_ICON_RECT.x, CHECK_ICON_RECT.y);
      }
      g.setColor(holdc);
    }
    if (b.getIcon() != null) {
      Icon icon;
      if (!model.isEnabled()) { // NOPMD
        icon = b.getDisabledIcon();
        if (icon == null) {
          icon = ImageFactory.createDisabledIcon(b.getIcon(), b);
        }
      } else if (model.isPressed() && model.isArmed()) {
        icon = b.getPressedIcon();
        if (icon == null) {
          icon = b.getIcon();
        }
      } else if (model.isArmed() ||
                 ((menuItem instanceof JMenu) && model.isSelected())) {
        icon = b.getRolloverIcon();
        if (icon == null) {
          icon = ImageFactory.createShadowedImageIcon(b.getIcon(), b);
          b.setRolloverIcon(icon);
        }
      } else {
        icon = b.getIcon();
      }
      if (icon != null) {
        icon.paintIcon(c, g, ICON_RECT.x, ICON_RECT.y);
      }
    }
    if (text != null) {
      View v = (View)c.getClientProperty(BasicHTML.propertyKey);
      if (v == null) {
        paintText(g, b, TEXT_RECT, text);
      } else {
        v.paint(g, TEXT_RECT);
      }
    }
    if ((acceleratorText != null) && (acceleratorText.length() != 0)) {
      int accOffset = 0;
      Container parent = menuItem.getParent();
      if (parent instanceof JComponent) {
        JComponent p = (JComponent)parent;
        Integer maxValueInt = (Integer)p.getClientProperty(MAX_ACC_WIDTH);
        int maxValue;
        if (maxValueInt == null) {
          maxValue = ACCELERATOR_RECT.width;
        } else {
          maxValue = maxValueInt.intValue();
        }
        accOffset = maxValue - ACCELERATOR_RECT.width;
      }
      g.setFont(acceleratorFont);
      if (model.isEnabled()) {
        if (model.isArmed() || (c instanceof JMenu && model.isSelected())) {
          g.setColor(acceleratorSelectionForeground);
        } else {
          g.setColor(acceleratorForeground);
        }
        g.drawString(acceleratorText, ACCELERATOR_RECT.x - accOffset,
                     ACCELERATOR_RECT.y + fmAccel.getAscent());
      } else {
        if (disabledForeground == null) {
          g.setColor(b.getBackground().brighter());
          g.drawString(acceleratorText, ACCELERATOR_RECT.x - accOffset,
                       ACCELERATOR_RECT.y + fmAccel.getAscent());
          g.setColor(b.getBackground().darker());
          g.drawString(acceleratorText, ACCELERATOR_RECT.x - accOffset - 1,
                       ACCELERATOR_RECT.y + fmAccel.getAscent() - 1);
        } else {
          g.setColor(disabledForeground);
          g.drawString(acceleratorText, ACCELERATOR_RECT.x - accOffset,
                       ACCELERATOR_RECT.y + fmAccel.getAscent());
        }
      }
    }
    if (arrowIcon != null) {
      if (model.isArmed() || (c instanceof JMenu && model.isSelected())) {
        g.setColor(foreground);
      }
      if (useCheckAndArrow()) {
        arrowIcon.paintIcon(c, g, ARROW_ICON_RECT.x, ARROW_ICON_RECT.y);
      }
    }
    if (model.isArmed() ||
        ((c instanceof JMenu) && (model.isSelected() || model.isRollover()))) {
      g.setColor(SmoothLookAndFeel.getRolloverMaskColor());
      g.fillRect(2, 2, menuWidth - 4, menuHeight - 4);
      g.setColor(SmoothLookAndFeel.getRolloverOuterBorderColor());
      GraphicUtils.draw1PixRoundRect(g, 1, 1, menuWidth - 3, menuHeight - 3);
    }
    g.setColor(holdc);
    g.setFont(holdf);
  }

  /**
   * Paints background.
   *
   * @param g the graphics
   * @param menuItem the menu item
   */
  protected void paintBackground(Graphics g, JMenuItem menuItem) {
    Color oldColor = g.getColor();
    int menuWidth = menuItem.getWidth();
    int menuHeight = menuItem.getHeight();
    g.setColor(SmoothLookAndFeel.getMenuItemRightBackground());
    g.fillRect(menuItemLeftBgdWidth, 0, menuWidth - menuItemLeftBgdWidth,
               menuHeight);
    g.setColor(SmoothLookAndFeel.getMenuItemLeftBackground());
    g.fillRect(0, 0, menuItemLeftBgdWidth, menuHeight);
    g.setColor(oldColor);
  }

  /**
   * Paints text.
   *
   * @param g the graphics
   * @param menuItem the menu item
   * @param textRect the text tectangle
   * @param text the text
   */
  protected void paintText(Graphics g, JMenuItem menuItem, Rectangle textRect,
                           String text) {
    ButtonModel model = menuItem.getModel();
    FontMetrics fm = g.getFontMetrics();
    int mnemIndex = menuItem.getDisplayedMnemonicIndex();
    if (!model.isEnabled()) {
      g.setColor(SmoothLookAndFeel.getInactiveControlTextColor());
    }
    GraphicUtils.drawStringUnderlineCharAt(g, text, mnemIndex, textRect.x,
                                             textRect.y + fm.getAscent());
  }

  /*protected void paintText(Graphics g, JMenuItem menuItem,
                             Rectangle TEXT_RECT, String text) {
    ButtonModel model = menuItem.getModel();
    FontMetrics fm = g.getFontMetrics();
    int mnemIndex = menuItem.getDisplayedMnemonicIndex();
    if (!model.isEnabled()) {
      g.setColor(SmoothLookAndFeel.getInactiveControlTextColor());
    }
    if (model.isArmed()) {
      GraphicUtils.drawStringUnderlineCharAt(g, text, mnemIndex,
                                             TEXT_RECT.x - 1,
                                             TEXT_RECT.y + fm.getAscent() - 1);
    } else {
      GraphicUtils.drawStringUnderlineCharAt(g, text, mnemIndex, TEXT_RECT.x,
                                             TEXT_RECT.y + fm.getAscent());
    }
  }*/

  /**
   * Performs click.
   *
   * @param msm the menu selection manager
   */
  protected void doClick(MenuSelectionManager msm) {
    if (msm == null) {
      msm = MenuSelectionManager.defaultManager();
    }
    msm.clearSelectedPath();
    menuItem.doClick(0);
  }

  /**
   * Installs property.
   *
   * @param c the component
   * @param property the property
   * @param value the value
   */
  protected static final void installProperty(JComponent c, String property,
                                              Object value) {
    try {
      INSTALL_PROPERTY_METHOD.invoke(null, new Object[] {c, property, value});
    } catch (Throwable t) { // NOPMD
      // empty catch block
    }
  }

/*^**************************************************************************^*/
/*- Private Method.                                                          -*/
/*^**************************************************************************^*/

  private void resetRects() {
    ICON_RECT.setBounds(0, 0, 0, 0);
    TEXT_RECT.setBounds(0, 0, 0, 0);
    ACCELERATOR_RECT.setBounds(0, 0, 0, 0);
    CHECK_ICON_RECT.setBounds(0, 0, 0, 0);
    ARROW_ICON_RECT.setBounds(0, 0, 0, 0);
    RECT.setBounds(0, 0, 0, 0);
    VIEW_RECT.setBounds(0, 0, Short.MAX_VALUE, Short.MAX_VALUE);
  }

/*^**************************************************************************^*/
/*- Inner Classes.                                                           -*/
/*^**************************************************************************^*/

  /**
   * MouseInput Handler.
   */
  protected class MouseInputHandler implements MouseInputListener {
    /**
     * {@inheritDoc}
     */
    public void mouseClicked(MouseEvent e) {
      // empty method body
    }
    /**
     * {@inheritDoc}
     */
    public void mousePressed(MouseEvent e) {
      // empty method body
    }
    /**
     * {@inheritDoc}
     */
    public void mouseReleased(MouseEvent e) {
      MenuSelectionManager manager = MenuSelectionManager.defaultManager();
      Point p = e.getPoint();
      if ((p.x >= 0) && (p.x < menuItem.getWidth()) &&
          (p.y >= 0) && (p.y < menuItem.getHeight())) {
        doClick(manager);
      } else {
        manager.processMouseEvent(e);
      }
    }
    /**
     * {@inheritDoc}
     */
    public void mouseEntered(MouseEvent e) {
      MenuSelectionManager manager = MenuSelectionManager.defaultManager();
      int modifiers = e.getModifiers();
      if ((modifiers & BUTTON_MASK) == 0) {
        manager.setSelectedPath(getPath());
      } else {
        MenuSelectionManager.defaultManager().processMouseEvent(e);
      }
    }
    /**
     * {@inheritDoc}
     */
    public void mouseExited(MouseEvent e) {
      MenuSelectionManager manager = MenuSelectionManager.defaultManager();
      int modifiers = e.getModifiers();
      if ((modifiers & BUTTON_MASK) == 0) {
        MenuElement[] path = manager.getSelectedPath();
        if (path.length > 1) {
          MenuElement[] newPath = new MenuElement[path.length - 1];
          System.arraycopy(path, 0, newPath, 0, newPath.length);
          manager.setSelectedPath(newPath);
        }
      } else {
        MenuSelectionManager.defaultManager().processMouseEvent(e);
      }
    }
    /**
     * {@inheritDoc}
     */
    public void mouseDragged(MouseEvent e) {
      MenuSelectionManager.defaultManager().processMouseEvent(e);
    }
    /**
     * {@inheritDoc}
     */
    public void mouseMoved(MouseEvent e) {
      // empty method body
    }
  }

  /**
   * MenuDragMouse Handler.
   */
  protected class MenuDragMouseHandler implements MenuDragMouseListener {
    /**
     * {@inheritDoc}
     */
    public void menuDragMouseEntered(MenuDragMouseEvent e) {
      MenuSelectionManager manager = e.getMenuSelectionManager();
      MenuElement[] path = e.getPath();
      manager.setSelectedPath(path);
    }
    /**
     * {@inheritDoc}
     */
    public void menuDragMouseDragged(MenuDragMouseEvent e) {
      MenuSelectionManager manager = e.getMenuSelectionManager();
      MenuElement[] path = e.getPath();
      manager.setSelectedPath(path);
    }
    /**
     * {@inheritDoc}
     */
    public void menuDragMouseExited(MenuDragMouseEvent e) {
      // empty method body
    }
    /**
     * {@inheritDoc}
     */
    public void menuDragMouseReleased(MenuDragMouseEvent e) {
      MenuSelectionManager manager = e.getMenuSelectionManager();
      Point p = e.getPoint();
      if ((p.x >= 0) && (p.x < menuItem.getWidth()) && (p.y >= 0) &&
          (p.y < menuItem.getHeight())) {
        doClick(manager);
      } else {
        manager.clearSelectedPath();
      }
    }
  }

  /**
   * MenuKey Handler.
   */
  protected class MenuKeyHandler implements MenuKeyListener {
    /**
     * {@inheritDoc}
     */
    public void menuKeyTyped(MenuKeyEvent e) {
      int key = menuItem.getMnemonic();
      if ((key == 0) || (e.getPath().length != 2)) {
        return;
      } // Only proceed if in a JPopupMenu
      if (Character.toLowerCase((char)key) ==
          Character.toLowerCase(e.getKeyChar())) {
        MenuSelectionManager manager = e.getMenuSelectionManager();
        doClick(manager);
        e.consume();
      }
    }
    /**
     * {@inheritDoc}
     */
    public void menuKeyPressed(MenuKeyEvent e) {
      // empty method body
    }
    /**
     * {@inheritDoc}
     */
    public void menuKeyReleased(MenuKeyEvent e) {
      // empty method body
    }
  }

  /**
   * PropertyChange Handler.
   */
  protected class PropertyChangeHandler implements PropertyChangeListener {
    /**
     * {@inheritDoc}
     */
    public void propertyChange(PropertyChangeEvent e) {
      String name = e.getPropertyName();
      if ("labelFor".equals(name) || //$NON-NLS-1$
          "displayedMnemonic".equals(name) || //$NON-NLS-1$
          "accelerator".equals(name)) { //$NON-NLS-1$
        updateAcceleratorBinding();
      } else if ("text".equals(name) || //$NON-NLS-1$
                 "font".equals(name) || //$NON-NLS-1$
                 "foreground".equals(name)) { //$NON-NLS-1$
        JMenuItem lbl = ((JMenuItem)e.getSource());
        String text = lbl.getText();
        BasicHTML.updateRenderer(lbl, text);
      }
    }
  }

  /**
   * Click Action.
   */
  protected static class ClickAction extends AbstractAction {
    /**
     * {@inheritDoc}
     */
    public void actionPerformed(ActionEvent e) {
      JMenuItem mi = (JMenuItem)e.getSource();
      MenuSelectionManager.defaultManager().clearSelectedPath();
      mi.doClick();
    }
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-25 21:06:43 Slobodan
   Initial version

*****************************************************************************^*/
