/*
 * @(#)SmoothRadioButtonUI.java  1.0.0 / 2008-05-25
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.swing.plaf.smooth;

import java.awt.Dimension;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Insets;
import java.awt.Rectangle;

import javax.swing.AbstractButton;
import javax.swing.ButtonModel;
import javax.swing.Icon;
import javax.swing.JComponent;
import javax.swing.SwingUtilities;
import javax.swing.plaf.ComponentUI;
import javax.swing.plaf.basic.BasicHTML;
import javax.swing.plaf.metal.MetalRadioButtonUI;
import javax.swing.text.View;

import net.wrka.swing.graphics.GraphicUtils;

/**
 * RadioButtonUI.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public class SmoothRadioButtonUI extends MetalRadioButtonUI {

/*^**************************************************************************^*/
/*- Private Static Final Field.                                              -*/
/*^**************************************************************************^*/

  private static final SmoothRadioButtonUI SHARED_INSTANCE;

/*^**************************************************************************^*/
/*- Static Initialization.                                                   -*/
/*^**************************************************************************^*/

  static {
    SHARED_INSTANCE = new SmoothRadioButtonUI();
  }

/*^**************************************************************************^*/
/*- Private Field.                                                           -*/
/*^**************************************************************************^*/

  private final SmoothButtonListener buttonListener;

/*^**************************************************************************^*/
/*- Public Constructor.                                                      -*/
/*^**************************************************************************^*/

  /**
   * Constructs new SmoothRadioButtonUI object.
   */
  public SmoothRadioButtonUI() {
    super();
    buttonListener = new SmoothButtonListener();
  }

/*^**************************************************************************^*/
/*- Public Static Field.                                                     -*/
/*^**************************************************************************^*/

  /**
   * Factory method. Return ui for the specified component.
   *
   * @param c the component
   *
   * @return ui for the specified component
   */
  public static ComponentUI createUI(JComponent c) {
    return SHARED_INSTANCE;
  }

/*^**************************************************************************^*/
/*- Overridden Methods.                                                      -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  public void paint(Graphics g, JComponent c) { // NOPMD
    SmoothLookAndFeel.getRenderingPolicy(this).apply((Graphics2D)g);
    AbstractButton b = (AbstractButton) c;
    ButtonModel model = b.getModel();
    Dimension size = c.getSize();
    Font f = c.getFont();
    g.setFont(f);
    FontMetrics fm = g.getFontMetrics();
    Rectangle viewRect = new Rectangle(size);
    Rectangle iconRect = new Rectangle();
    Rectangle textRect = new Rectangle();
    Insets i = c.getInsets();
    viewRect.x += i.left;
    viewRect.y += i.top;
    viewRect.width -= (i.right + viewRect.x);
    viewRect.height -= (i.bottom + viewRect.y);
    Icon altIcon = b.getIcon();
    Icon icon;
    if (altIcon == null) {
      icon = getDefaultIcon();
    } else {
      icon = altIcon;
    }
    String text = SwingUtilities.layoutCompoundLabel(
                    c, fm, b.getText(), icon, b.getVerticalAlignment(),
                    b.getHorizontalAlignment(), b.getVerticalTextPosition(),
                    b.getHorizontalTextPosition(), viewRect, iconRect, textRect,
                    b.getIconTextGap());
    // fill background
    if (c.isOpaque()) {
      g.setColor(b.getBackground());
      g.fillRect(0, 0, size.width, size.height);
    }
    // Paint the radio button
    if (altIcon == null) {
      getDefaultIcon().paintIcon(c, g, iconRect.x, iconRect.y);
    } else {
      if (!model.isEnabled()) { // NOPMD
        if (model.isSelected()) {
          altIcon = b.getDisabledSelectedIcon();
        } else {
          altIcon = b.getDisabledIcon();
        }
      } else if (model.isPressed() && model.isArmed()) {
        altIcon = b.getPressedIcon();
        if (altIcon == null) {
          // Use selected icon
          altIcon = b.getSelectedIcon();
        }
      } else if (model.isSelected()) {
        if (b.isRolloverEnabled() && model.isRollover()) {
          altIcon = b.getRolloverSelectedIcon();
          if (altIcon == null) {
            altIcon = b.getSelectedIcon();
          }
        } else {
          altIcon = b.getSelectedIcon();
        }
      } else if (b.isRolloverEnabled() && model.isRollover()) {
        altIcon = b.getRolloverIcon();
      }
      if (altIcon == null) {
        altIcon = b.getIcon();
      }
      altIcon.paintIcon(c, g, iconRect.x, iconRect.y);
    }
    // Draw the Text
    if (text != null) {
      View v = (View) c.getClientProperty(BasicHTML.propertyKey);
      if (v == null) {
        int mnemIndex = b.getDisplayedMnemonicIndex();
        if (model.isEnabled()) {
          // *** paint the text normally
          if (b.isContentAreaFilled() &&  model.isArmed() &&
              !model.isPressed()) {
            g.setColor(SmoothLookAndFeel.getRolloverForeground());
          } else {
            g.setColor(b.getForeground());
          }
          GraphicUtils.drawStringUnderlineCharAt(g, text, mnemIndex, textRect.x,
                                                 textRect.y + fm.getAscent());
        } else {
          // *** paint the text disabled
          g.setColor(getDisabledTextColor());
          GraphicUtils.drawStringUnderlineCharAt(g, text, mnemIndex,
                                                 textRect.x,
                                                 textRect.y + fm.getAscent());
        }
        if (b.hasFocus() && b.isFocusPainted() && (textRect.width > 0) &&
            (textRect.height > 0)) {
          g.setColor(getFocusColor());
          GraphicUtils.drawDashedRect(g, textRect.x - 2, textRect.y - 1,
                                      textRect.width + 3, textRect.height + 1);
        }
      } else {
        v.paint(g, textRect);
      }
    }
  }

  /**
   * {@inheritDoc}
   */
  protected void installListeners(AbstractButton b) {
    b.addMouseListener(buttonListener);
    b.addMouseMotionListener(buttonListener);
    b.addFocusListener(buttonListener);
    b.addPropertyChangeListener(buttonListener);
    b.addChangeListener(buttonListener);
  }

  /**
   * {@inheritDoc}
   */
  protected void installKeyboardActions(AbstractButton b) {
    buttonListener.installKeyboardActions(b);
  }

  /**
   * {@inheritDoc}
   */
  protected void uninstallListeners(AbstractButton b) {
    b.removeMouseListener(buttonListener);
    b.removeMouseListener(buttonListener);
    b.removeMouseMotionListener(buttonListener);
    b.removeFocusListener(buttonListener);
    b.removeChangeListener(buttonListener);
    b.removePropertyChangeListener(buttonListener);
  }

  /**
   * {@inheritDoc}
   */
  protected void uninstallKeyboardActions(AbstractButton b) {
    buttonListener.uninstallKeyboardActions(b);
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-25 20:00:14 Slobodan
   Initial version

*****************************************************************************^*/
