/*
 * @(#)SmoothScrollBarUI.java  1.0.0 / 2008-05-25
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.swing.plaf.smooth;

import java.awt.Adjustable;
import java.awt.Color;
import java.awt.Component;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.awt.event.MouseEvent;

import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.UIManager;
import javax.swing.plaf.ComponentUI;
import javax.swing.plaf.basic.BasicScrollBarUI;

import net.wrka.swing.graphics.GraphicUtils;

/**
 * ScrollBarUI.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public class SmoothScrollBarUI extends BasicScrollBarUI {

/*^**************************************************************************^*/
/*- Private Static Final Field.                                              -*/
/*^**************************************************************************^*/

  private static final int MAX_TUMB_LINE_WIDTH = 14;

/*^**************************************************************************^*/
/*- Packge-Private Fields.                                                   -*/
/*^**************************************************************************^*/

  Color shadowColor;
  Color darkShadowColor;
  Color thumbRolloverBackground;
  Color currentThumbColor;
  int scrollBarWidth;

/*^**************************************************************************^*/
/*- Public Static Method.                                                    -*/
/*^**************************************************************************^*/

  /**
   * Factory method. Return ui for the specified component.
   *
   * @param c the component
   *
   * @return ui for the specified component
   */
  public static ComponentUI createUI(JComponent c) {
    return new SmoothScrollBarUI();
  }

/*^**************************************************************************^*/
/*- Overridden Methods.                                                      -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  public void paint(Graphics g, JComponent c) {
    SmoothLookAndFeel.getRenderingPolicy(this).apply((Graphics2D)g);
    super.paint(g, c);
  }

  /**
   * {@inheritDoc}
   */
  protected void installDefaults() {
    scrollBarWidth = UIManager.getInt("ScrollBar.width"); //$NON-NLS-1$
    super.installDefaults();
  }

  /**
   * {@inheritDoc}
   */
  protected void configureScrollBarColors() {
    super.configureScrollBarColors();
    shadowColor = UIManager.getColor("ScrollBar.shadow"); //$NON-NLS-1$
    darkShadowColor = UIManager.getColor("ScrollBar.darkShadow"); //$NON-NLS-1$
    thumbHighlightColor = GraphicUtils.darkerColor(thumbHighlightColor, 0.1f);
    thumbRolloverBackground =
        GraphicUtils.brighterColor(thumbColor, SmoothLookAndFeel.
                                                 getRolloverBackgroundFactor());
    currentThumbColor = thumbColor;
  }

  /**
   * {@inheritDoc}
   */
  protected JButton createDecreaseButton(int orientation) {
    return new SmoothSrollBarArrowButton(orientation, scrollBarWidth);
  }

  /**
   * {@inheritDoc}
   */
  protected JButton createIncreaseButton(int orientation) {
    return new SmoothSrollBarArrowButton(orientation, scrollBarWidth);
  }

  /**
   * {@inheritDoc}
   */
  protected TrackListener createTrackListener() {
    return new SmoothTrackListener();
  }

  /**
   * {@inheritDoc}
   */
  protected Rectangle getThumbBounds() { // optimization
    return thumbRect;
  }

  /**
   * {@inheritDoc}
   */
  protected void setThumbBounds(int x, int y, int width, int height) {
    if ((thumbRect.x == x) && (thumbRect.y == y) &&
        (thumbRect.width == width) && (thumbRect.height == height)) {
      return;
    }
    int minX = Math.min(x, thumbRect.x);
    int minY = Math.min(y, thumbRect.y);
    int maxX = Math.max(x + width + 1, thumbRect.x + thumbRect.width + 1);
    int maxY = Math.max(y + height + 1, thumbRect.y + thumbRect.height + 1);
    thumbRect.setBounds(x, y, width, height);
    scrollbar.repaint(minX, minY, maxX - minX, maxY - minY);
  }

  /**
   * {@inheritDoc}
   */
  protected void paintTrack(Graphics g, JComponent c, Rectangle trackBounds) {
    g.translate(trackBounds.x, trackBounds.y);
    if (c.isEnabled()) {
      if (scrollbar.getOrientation() == Adjustable.VERTICAL) { // VERTICAL
        g.setColor(darkShadowColor);
        g.drawLine(0, 0, 0, trackBounds.height - 1);
        g.drawLine(trackBounds.width - 1, 0, trackBounds.width - 1,
                   trackBounds.height - 1);
        g.setColor(shadowColor);
        g.drawLine(1, 0, 1, trackBounds.height - 1);
        g.drawLine(1, 0, trackBounds.width - 2, 0);
      } else { // HORIZONTAL
        g.setColor(darkShadowColor);
        g.drawLine(0, 0, trackBounds.width - 1, 0);
        g.drawLine(0, trackBounds.height - 1, trackBounds.width - 1,
                   trackBounds.height - 1);
        g.setColor(shadowColor);
        g.drawLine(0, 1, trackBounds.width - 1, 1);
        g.drawLine(0, 2, 0, trackBounds.height - 2);
      }
    } else {
      SmoothUtils.drawDisabledBorder(g, 0, 0, trackBounds.width,
                                       trackBounds.height);
    }
    g.translate(-trackBounds.x, -trackBounds.y);
  }

  /**
   * {@inheritDoc}
   */
  protected void paintThumb(Graphics g, JComponent c, Rectangle thumbBounds) {
    if (c.isEnabled()) {
      g.translate(thumbBounds.x, thumbBounds.y);
      g.setColor(currentThumbColor);
      if (scrollbar.getOrientation() == Adjustable.VERTICAL) {
        g.fillRect(2, 2, thumbBounds.width - 3, thumbBounds.height - 3);
        g.setColor(thumbLightShadowColor);
        g.drawRect(0, 0, thumbBounds.width - 1, thumbBounds.height - 1);
        g.setColor(thumbHighlightColor);
        g.drawLine(1, 1, thumbBounds.width - 2, 1);
        g.drawLine(1, 1, 1, thumbBounds.height - 2);
        paintThumbLines(g, c, 3, 4, thumbBounds.width - 5,
                        thumbBounds.height - 6, false);
        g.setColor(shadowColor);
        g.fillRect(2, thumbRect.height, thumbRect.width - 3, 1);
        g.translate(1, 1);
        SmoothUtils.drawFast3DEffectHorizontalLight(g, thumbBounds.width - 2,
                                                    thumbBounds.height - 2);
      } else { // HORIZONTAL
        g.fillRect(2, 2, thumbBounds.width - 3, thumbBounds.height - 3);
        g.setColor(thumbLightShadowColor);
        g.drawRect(0, 0, thumbBounds.width - 1, thumbBounds.height - 1);
        g.setColor(thumbHighlightColor);
        g.drawLine(1, 1, thumbBounds.width - 2, 1);
        g.drawLine(1, 1, 1, thumbBounds.height - 2);
        paintThumbLines(g, c, 3, 4, thumbBounds.width - 5,
                        thumbBounds.height - 6, true);
        g.setColor(shadowColor);
        g.fillRect(thumbRect.width, 2, 1, thumbRect.height - 3);
        g.translate(1, 1);
        SmoothUtils.drawFast3DEffectVerticalLight(g, thumbBounds.width - 2,
                                                  thumbBounds.height - 2);
      }
      g.translate(-thumbBounds.x - 1, -thumbBounds.y - 1);
    }
  }

/*^**************************************************************************^*/
/*- Protected Methods.                                                       -*/
/*^**************************************************************************^*/

  /**
   * Returns thumb color.
   *
   * @return thumb color
   */
  protected Color getThumbColor() { // optimization
    return thumbColor;
  }

  /**
   * Paints thumb lines.
   *
   * @param g the graphics
   * @param c the comonent
   * @param x the x cooridnate
   * @param y the y coordinate
   * @param width the width
   * @param height the height
   * @param vertical the <tt>true</tt> if vertical
   */
  protected void paintThumbLines(Graphics g, JComponent c, int x, int y,
                                 int width, int height, boolean vertical) {
    int w;
    int h;
    if (MAX_TUMB_LINE_WIDTH < width) {
      w = MAX_TUMB_LINE_WIDTH;
    } else {
      w = width;
    }
    if (MAX_TUMB_LINE_WIDTH < height) {
      h = MAX_TUMB_LINE_WIDTH;
    } else {
      h = height;
    }
    if (vertical) {
      x = x + ((width - w) >> 1);
      y = 3;
      for (int i = 0; i < w; i += 2) {
        g.setColor(thumbHighlightColor);
        g.fillRect(x + i++, y, 1, h);
        g.setColor(thumbLightShadowColor);
        g.fillRect(x + i, y + 1, 1, h);
      }
    } else {
      x = 3;
      y = y + ((height - h) >> 1);
      w -= 1;
      for (int i = 0; i < h; i += 2) {
        g.setColor(thumbHighlightColor);
        g.fillRect(x, y + i++, w, 1);
        g.setColor(thumbLightShadowColor);
        g.fillRect(x + 1, y + i, w, 1);
      }
    }

  }

/*^**************************************************************************^*/
/*- Inner Class.                                                             -*/
/*^**************************************************************************^*/

  /**
   * Smooth Track Listener.
   */
  protected class SmoothTrackListener extends BasicScrollBarUI.TrackListener {
    private boolean entered;
    /**
     * {@inheritDoc}
     */
    public void mouseExited(MouseEvent e) {
      super.mouseExited(e);
      Component c = (Component)e.getSource();
      if (c.isEnabled()) {
        currentThumbColor = getThumbColor();
        entered = false;
        c.repaint();
      }
    }
    /**
     * {@inheritDoc}
     */
    public void mouseMoved(MouseEvent e) {
      super.mouseMoved(e);
      Rectangle thumbR = getThumbBounds();
      Component c = (Component)e.getSource();
      if (thumbR.contains(e.getPoint())) {
        if (!entered) {
          currentThumbColor = thumbRolloverBackground;
          entered = true;
          c.repaint();
        }
      } else {
        if (entered) {
          currentThumbColor = getThumbColor();
          entered = false;
          c.repaint();
        }
      }
    }
    /**
     * {@inheritDoc}
     */
    public void mousePressed(MouseEvent e) {
      super.mousePressed(e);
      Component c = (Component)e.getSource();
      if (c.isEnabled()) {
        currentThumbColor = getThumbColor();
        c.repaint();
      }
    }
    /**
     * {@inheritDoc}
     */
    public void mouseReleased(MouseEvent e) {
      super.mouseReleased(e);
      Component c = (Component)e.getSource();
      if (c.isEnabled() && entered) {
        currentThumbColor = thumbRolloverBackground;
        c.repaint();
      }
    }
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-25 13:34:39 Slobodan
   Initial version

*****************************************************************************^*/
