/*
 * @(#)SmoothSliderUI.java  1.0.0 / 2008-05-25
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.swing.plaf.smooth;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Rectangle;
import java.awt.event.MouseEvent;

import javax.swing.JComponent;
import javax.swing.JSlider;
import javax.swing.plaf.ComponentUI;
import javax.swing.plaf.basic.BasicSliderUI;
import javax.swing.plaf.metal.MetalSliderUI;

import net.wrka.swing.graphics.GraphicUtils;

/**
 * SliderUI.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public class SmoothSliderUI extends MetalSliderUI {

/*^**************************************************************************^*/
/*- Package-Private Fields.                                                  -*/
/*^**************************************************************************^*/

  Color thumbRolloverColor;
  boolean rollover;

/*^**************************************************************************^*/
/*- Public Static Method.                                                    -*/
/*^**************************************************************************^*/

  /**
   * Factory method. Return ui for the specified component.
   *
   * @param c the component
   *
   * @return ui for the specified component
   */
  public static ComponentUI createUI(JComponent c)    {
    return new SmoothSliderUI();
  }

/*^**************************************************************************^*/
/*- Overridden Methods.                                                      -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  public void installUI(JComponent c) {
    super.installUI(c);
    thumbRolloverColor =
        GraphicUtils.brighterColor(thumbColor, SmoothLookAndFeel.
                                                 getRolloverBackgroundFactor());
  }

  /**
   * {@inheritDoc}
   */
  protected TrackListener createTrackListener(JSlider slider) {
    return new SmoothTrackListener(slider);
  }

  /**
   * {@inheritDoc}
   */
  public void paintThumb(Graphics g)  {
    g.translate(thumbRect.x, thumbRect.y);
    if (slider.getOrientation() == JSlider.HORIZONTAL) {
      if (slider.isEnabled()) {
        g.setColor(SmoothLookAndFeel.getPrimaryControlInfo());
      } else {
        g.setColor(SmoothLookAndFeel.getControlDarkShadow());
      }
      g.drawLine(4, 0, 10, 0);
      g.drawLine(3, 1, 3, 10);
      g.drawLine(11, 1, 11, 10);
      g.drawLine(3, 11, 7, 15);
      g.drawLine(7, 15, 11, 11);
      if (slider.hasFocus() && slider.isEnabled()) {
        if (rollover) {
          g.setColor(thumbRolloverColor);
        } else {
          g.setColor(thumbColor);
        }
      } else {
        if (rollover) {
          g.setColor(SmoothLookAndFeel.getRolloverBackground());
        } else {
          g.setColor(SmoothLookAndFeel.getControl());
        }
      }
      g.fillRect(4, 1, 7, 11);
      g.fillRect(5, 12, 5, 1);
      g.fillRect(6, 13, 3, 1);
      g.fillRect(7, 14, 1, 1);
      if (slider.isEnabled()) {
        g.translate(4, 1);
        SmoothUtils.drawFast3DEffectHorizontalLight(g, 8, 11);
        SmoothUtils.drawFast3DEffectVerticalLight(g, 7, 12);
        g.translate(-4, -1);
      }
    } else {
      if (slider.isEnabled()) {
        g.setColor(SmoothLookAndFeel.getPrimaryControlInfo());
      } else {
        g.setColor(SmoothLookAndFeel.getControlDarkShadow());
      }
      g.drawLine(0, 4, 0, 10);
      g.drawLine(1, 3, 10, 3);
      g.drawLine(1, 11, 10, 11);
      g.drawLine(11, 3, 15, 7);
      g.drawLine(15, 7, 11, 11);
      if (slider.hasFocus() && slider.isEnabled()) {
        if (rollover) {
          g.setColor(thumbRolloverColor);
        } else {
          g.setColor(thumbColor);
        }
      } else {
        if (rollover) {
          g.setColor(SmoothLookAndFeel.getRolloverBackground());
        } else {
          g.setColor(SmoothLookAndFeel.getControl());
        }
      }
      g.fillRect(1, 4, 11, 7);
      g.fillRect(12, 5, 1, 5);
      g.fillRect(13, 6, 1, 3);
      g.fillRect(14, 7, 1, 1);
      if (slider.isEnabled()) {
        g.translate(1, 4);
        SmoothUtils.drawFast3DEffectHorizontalLight(g, 11, 8);
        SmoothUtils.drawFast3DEffectVerticalLight(g, 12, 7);
        g.translate(-1, -4);
      }
    }
    g.translate(-thumbRect.x, -thumbRect.y);
  }

/*^**************************************************************************^*/
/*- Protected Methods.                                                       -*/
/*^**************************************************************************^*/

  /**
   * Returns thumb bounds.
   *
   * @return thumb bounds
   */
  protected Rectangle getThumbBounds() { // optimization
    return thumbRect;
  }

/*^**************************************************************************^*/
/*- Inner Class.                                                             -*/
/*^**************************************************************************^*/

  /**
   * SmoothTrackListener Class.
   */
  protected class SmoothTrackListener extends BasicSliderUI.TrackListener {
    private final JSlider slider;
    /**
     * Constructs new SmoothTrackListener object.
     *
     * @param slider the slider
     */
    public SmoothTrackListener(JSlider slider) {
      super();
      this.slider = slider;
    }
    /**
     * {@inheritDoc}
     */
    public void mouseExited(MouseEvent e) {
      super.mouseExited(e);
      if (rollover && slider.isEnabled()) {
        rollover = false;
        slider.repaint(getThumbBounds());
      }
    }
    /**
     * {@inheritDoc}
     */
    public void mouseMoved(MouseEvent e) {
      super.mouseMoved(e);
      if (slider.isEnabled()) {
        final Rectangle thumbRect = getThumbBounds();
        if (thumbRect.contains(e.getPoint())) {
          if (!rollover) {
            rollover = true;
            slider.repaint(thumbRect);
          }
        } else {
          if (rollover) {
            rollover = false;
            slider.repaint(thumbRect);
          }
        }
      }
    }
    /**
     * {@inheritDoc}
     */
    public void mousePressed(MouseEvent e) {
      super.mousePressed(e);
      if (slider.isEnabled()) {
        rollover = false;
        slider.repaint(getThumbBounds());
      }
    }
    /**
     * {@inheritDoc}
     */
    public void mouseReleased(MouseEvent e) {
      super.mouseReleased(e);
      final Rectangle thumbRect = getThumbBounds();
      if (slider.isEnabled() && thumbRect.contains(e.getPoint())) {
        rollover = true;
        slider.repaint(thumbRect);
      }
    }
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-25 13:43:19 Slobodan
   Initial version

*****************************************************************************^*/
