/*
 * @(#)SmoothSrollBarArrowButton.java  1.0.0 / 2008-05-25
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.swing.plaf.smooth;

import java.awt.AWTEvent;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.event.MouseEvent;

import javax.swing.JButton;
import javax.swing.UIManager;

/**
 * Scroll Bar Arrow Button.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public class SmoothSrollBarArrowButton extends JButton {

/*^**************************************************************************^*/
/*- Private Fields.                                                          -*/
/*^**************************************************************************^*/

  private int direction;
  private final int scrollBarWidth;
  private final Color shadowColor;
  private final Color highlightColor;
  private boolean rollover;

/*^**************************************************************************^*/
/*- Public Constructor.                                                      -*/
/*^**************************************************************************^*/

  /**
   * Constructs new SmoothSrollBarArrowButton object.
   *
   * @param direction the arrow direction
   * @param width the scroll bar width
   */
  public SmoothSrollBarArrowButton(int direction, int width) {
    super();
    setRequestFocusEnabled(false);
    setDefaultCapable(false);
    enableEvents(AWTEvent.MOUSE_EVENT_MASK);
    disableEvents(AWTEvent.FOCUS_EVENT_MASK);
    setDirection(direction);
    scrollBarWidth = width;
    shadowColor = UIManager.getColor("ScrollBar.darkShadow"); //$NON-NLS-1$
    highlightColor = UIManager.getColor("ScrollBar.highlight"); //$NON-NLS-1$
  }

/*^**************************************************************************^*/
/*- Public Methods.                                                          -*/
/*^**************************************************************************^*/

  /**
   * Returns direction.
   *
   * @return direction
   */
  public int getDirection() {
    return direction;
  }

  /**
   * Sets direction.
   *
   * @param dir the new direction
   */
  public void setDirection(int dir) {
    direction = dir;
  }

/*^**************************************************************************^*/
/*- Overridden Methods.                                                      -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  public Dimension getPreferredSize() {
    if ((direction == NORTH) || (direction == SOUTH)) {
      return new Dimension(scrollBarWidth, 16);
    }
    return new Dimension(16, scrollBarWidth);
  }

  /**
   * {@inheritDoc}
   */
  public Dimension getMinimumSize() {
    return getPreferredSize();
  }

  /**
   * {@inheritDoc}
   */
  public Dimension getMaximumSize() {
    return new Dimension(Integer.MAX_VALUE, Integer.MAX_VALUE);
  }

  /**
   * {@inheritDoc}
   */
  public void paint(Graphics g) {
    boolean enabled = getParent().isEnabled();
    boolean pressed = getModel().isPressed();
    Color arrowColor;
    if (enabled) {
      arrowColor = SmoothLookAndFeel.getControlInfo();
    } else {
      arrowColor = SmoothLookAndFeel.getControlDisabled();
    }
    int w = getWidth();
    int h = getHeight();
    int wm2 = w - 2;
    int hm2 = h - 2;
    int arrowHeight = (scrollBarWidth + 1) / 4;
    if (pressed) {
      g.setColor(SmoothLookAndFeel.getControlShadow());
    } else if (rollover) {
      g.setColor(SmoothLookAndFeel.getRolloverBackground());
    } else {
      g.setColor(SmoothLookAndFeel.getControl());
    }
    g.fillRect(1, 1, wm2, hm2);
    g.setColor(shadowColor);
    g.drawRect(0, 0, w - 1, h - 1);
    int startX;
    int startX1;
    int startY;
    int startY1;
    switch (direction) { // NOPMD
      case NORTH :
        startY = ((h + 1) - arrowHeight) >> 1;
        startX = (w >> 1) - 1;
        startX1 = startX + 1;
        g.setColor(arrowColor);
        for (int line = 0; line < arrowHeight; line++) {
          g.drawLine(startX - line, startY + line, startX1 + line,
                     startY + line);
        }
        break;
      case SOUTH :
        startY = (((h + 1) - arrowHeight) >> 1) + arrowHeight - 1;
        startX = (w >> 1) - 1;
        startX1 = startX + 1;
        g.setColor(arrowColor);
        for (int line = 0; line < arrowHeight; line++) {
          g.drawLine(startX - line, startY - line, startX1 + line,
                     startY - line);
        }
        break;
      case EAST :
        startX = ((w - arrowHeight) >> 1) + arrowHeight - 1;
        startY = (h >> 1) - 1;
        g.setColor(arrowColor);
        startY1 = startY + 1;
        for (int line = 0; line < arrowHeight; line++) {
          g.drawLine(startX - line, startY - line, startX - line,
                     startY1 + line);
        }
        break;
      case WEST :
        startX = (((w + 1) - arrowHeight) >> 1);
        startY = ((h >> 1) - 1);
        startY1 = startY + 1;
        g.setColor(arrowColor);
        for (int line = 0; line < arrowHeight; line++) {
          g.drawLine(startX + line, startY - line, startX + line,
                     startY1 + line);
        }
        break;
    }
    if (!pressed) {
      g.setColor(highlightColor);
      g.fillRect(1, 1, wm2, 1);
      g.fillRect(1, 1, 1, hm2);
      g.translate(1, 1);
      if ((direction == NORTH) || (direction == SOUTH)) {
        SmoothUtils.drawFast3DEffectHorizontalLight(g, wm2, hm2);
      } else {
        SmoothUtils.drawFast3DEffectVerticalLight(g, wm2, hm2);
      }
      g.translate(-1, -1);
    }
  }

  /**
   * {@inheritDoc}
   */
  protected void processMouseEvent(MouseEvent e) {
    super.processMouseEvent(e);
    switch (e.getID()) { // NOPMD
      case MouseEvent.MOUSE_ENTERED:
        rollover = true;
        repaint();
        break;
      case MouseEvent.MOUSE_EXITED:
        rollover = false;
        repaint();
        break;
    }
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-25 16:01:03 Slobodan
   Initial version

*****************************************************************************^*/
