/*
 * @(#)SmoothTabbedPaneUI.java  1.0.0 / 2008-05-25
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.swing.plaf.smooth;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;

import javax.swing.JComponent;
import javax.swing.JTabbedPane;
import javax.swing.plaf.ComponentUI;
import javax.swing.plaf.basic.BasicTabbedPaneUI;

import net.wrka.swing.graphics.GraphicUtils;

/**
 * TabbedPaneUI.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public class SmoothTabbedPaneUI extends BasicTabbedPaneUI {

/*^**************************************************************************^*/
/*- Private Static Final Fields.                                             -*/
/*^**************************************************************************^*/

  private static final int SHADOW_DEPTH = 10;
  private static final int MIN_HEIGHT = 22;

/*^**************************************************************************^*/
/*- Packge-Protected Fields.                                                 -*/
/*^**************************************************************************^*/

  Color activeTextColor;
  Color tabActiveBackground;
  Color tabNormalBackground;
  Color tabNormalForeground;
  TabbedPaneMouseListener tabbedPaneMouseListener;
  int chTabIndex;

/*^**************************************************************************^*/
/*- Public Constructor.                                                      -*/
/*^**************************************************************************^*/

  /**
   * Constructs new SmoothTabbedPaneUI object.
   */
  public SmoothTabbedPaneUI() {
    super();
    chTabIndex = -1;
  }

/*^**************************************************************************^*/
/*- Public Static Method.                                                    -*/
/*^**************************************************************************^*/

  /**
   * Factory method. Return ui for the specified component.
   *
   * @param c the component
   *
   * @return ui for the specified component
   */
  public static ComponentUI createUI(JComponent c) {
    return new SmoothTabbedPaneUI();
  }

/*^**************************************************************************^*/
/*- Overridden Methods.                                                      -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  public void installUI(JComponent c) {
    super.installUI(c);
    activeTextColor = SmoothLookAndFeel.getRolloverForeground();
    tabActiveBackground =
        GraphicUtils.brighterColorUIResource(c.getBackground(),
                                             SmoothLookAndFeel.
                                               getRolloverBackgroundFactor());
  }

  /**
   * {@inheritDoc}
   */
  protected void installListeners() {
    super.installListeners();
    tabbedPaneMouseListener = new TabbedPaneMouseListener();
    tabPane.addMouseMotionListener(tabbedPaneMouseListener);
    tabPane.addMouseListener(tabbedPaneMouseListener);
  }

  /**
   * {@inheritDoc}
   */
  protected void uninstallListeners() {
    super.uninstallListeners();
    if (tabbedPaneMouseListener != null) {
      tabPane.removeMouseMotionListener(tabbedPaneMouseListener);
      tabPane.removeMouseListener(tabbedPaneMouseListener);
      for (int i = 0; i < tabPane.getTabCount(); i++) {
        tabPane.setBackgroundAt(i, null);
        tabPane.setForegroundAt(i, null);
      }
    }
  }

  /**
   * {@inheritDoc}
   */
  public void paint(Graphics g, JComponent c) {
    SmoothLookAndFeel.getRenderingPolicy(this).apply((Graphics2D)g);
    super.paint(g, c);
  }

  /**
   * {@inheritDoc}
   */
  protected void paintTabBackground(Graphics g, int tabPlacement, int tabIndex,
                                    int x, int y, int w, int h,
                                    boolean isSelected) {
    // empty method body
  }

  /**
   * {@inheritDoc}
   */
  protected void paintTabBorder(Graphics g, int tabPlacement, int tabIndex,
                                int x, int y, int w, int h,
                                boolean isSelected) {

    if (isSelected) {
      g.setColor(SmoothLookAndFeel.getControl());
    } else {
      g.setColor(tabPane.getBackgroundAt(tabIndex));
    }
    int selectedIndex = tabPane.getSelectedIndex();
    switch(tabPlacement) {
      case LEFT:
        g.fillRect(x + 1, y + 1, w - 2, h - 3);
        SmoothUtils.drawFastTranReflectGradient(g, true, x + 1, y, w - 2,
                                                SHADOW_DEPTH);
        super.paintTabBorder(g, tabPlacement, tabIndex, x, y, w, h, isSelected);
        break;
      case RIGHT:
        g.fillRect(x, y + 1, w - 2, h - 3);
        SmoothUtils.drawFastTranReflectGradient(g, true, x, y, w - 2,
                                                SHADOW_DEPTH);
        if (tabIndex != selectedIndex) {
          SmoothUtils.drawFastTranShadowGradient(g, false, x, y, SHADOW_DEPTH,
                                                 h);
        }
        super.paintTabBorder(g, tabPlacement, tabIndex, x, y, w, h, isSelected);
        break;
      case BOTTOM:
        g.fillRect(x + 1, y, w - 3, h - 1);
        super.paintTabBorder(g, tabPlacement, tabIndex, x, y, w, h, isSelected);
        if (tabIndex != selectedIndex) {
          SmoothUtils.drawFastTranShadowGradient(g, true, x, y, w,
                                                 SHADOW_DEPTH);
          if (tabIndex == (selectedIndex + 1)) {
            SmoothUtils.drawFastTranShadowGradient(g, false, x, y,
                                                   SHADOW_DEPTH, h);
          }
        }
        SmoothUtils.drawFastInvTranShadowGradient(g, true, x + 1,
                                                  y + h - SHADOW_DEPTH, w - 3,
                                                  SHADOW_DEPTH);
        break;
      case TOP:
      default:
        g.fillRect(x + 1, y + 1, w - 3, h - 1);
        SmoothUtils.drawFastTranReflectGradient(g, true, x + 1, y, w - 2,
                                                SHADOW_DEPTH);
        if (tabIndex == (selectedIndex + 1)) {
          SmoothUtils.drawFastTranShadowGradient(g, false, x, y,
                                                 SHADOW_DEPTH, h);
        }
        super.paintTabBorder(g, tabPlacement, tabIndex, x, y, w, h, isSelected);
        break;
    }
  }

  /**
   * {@inheritDoc}
   */
  protected int calculateTabHeight(int tabPlacement, int tabIndex,
                                   int fontHeight) {
    int sh = super.calculateTabHeight(tabPlacement, tabIndex, fontHeight);
    if ((tabPlacement == TOP) || (tabPlacement == BOTTOM)) {
      return sh;
    }
    if (MIN_HEIGHT > sh) {
      return MIN_HEIGHT;
    }
    return sh;
  }

/*^**************************************************************************^*/
/*- Protected Methods.                                                       -*/
/*^**************************************************************************^*/

  /**
   * Sets highlight mode for the specified tab.
   *
   * @param tabIndex the tab index
   */
  protected void setHighLights(int tabIndex) {
    tabNormalBackground = tabPane.getBackgroundAt(tabIndex);
    tabNormalForeground = tabPane.getForegroundAt(tabIndex);
    tabPane.setBackgroundAt(tabIndex, tabActiveBackground);
    tabPane.setForegroundAt(tabIndex, activeTextColor);
  }

  /**
   * Clears highlight mode for the specified tab.
   *
   * @param tabIndex tabIndex the tab index
   */
  protected void clearHighLights(int tabIndex) {
    if (tabIndex < tabPane.getTabCount()) {
      tabPane.setBackgroundAt(tabIndex, tabNormalBackground);
      tabPane.setForegroundAt(tabIndex, tabNormalForeground);
    }
  }

/*^**************************************************************************^*/
/*- Inner Classes.                                                           -*/
/*^**************************************************************************^*/

  /**
   * TabPane Mouse Listener.
   */
  protected class TabbedPaneMouseListener implements MouseMotionListener,
                                                     MouseListener {
    /**
     * {@inheritDoc}
     */
    public void mouseMoved(MouseEvent e) {
      JTabbedPane tabPane = (JTabbedPane)e.getSource();
      if (tabPane.isEnabled()) {
        int tabIndex = tabForCoordinate(tabPane, e.getX(), e.getY());
        if ((tabIndex >= 0) && (chTabIndex != tabIndex)) {
          setHighLights(tabIndex);
          if (chTabIndex >= 0) {
            clearHighLights(chTabIndex);
          }
        } else {
          if ((tabIndex < 0) && (chTabIndex >= 0)) {
            clearHighLights(chTabIndex);
          }
        }
        chTabIndex = tabIndex;
      }
    }
    /**
     * {@inheritDoc}
     */
    public void mouseExited(MouseEvent e) {
      if (chTabIndex >= 0) {
        clearHighLights(chTabIndex);
      }
      chTabIndex = -1;
    }
    /**
     * {@inheritDoc}
     */
    public void mouseDragged(MouseEvent e) {
      // empty method body
    }
    /**
     * {@inheritDoc}
     */
    public void mouseEntered(MouseEvent e) {
      // empty method body
    }
    /**
     * {@inheritDoc}
     */
    public void mouseClicked(MouseEvent e) {
      // empty method body
    }
    /**
     * {@inheritDoc}
     */
    public void mousePressed(MouseEvent e) {
      // empty method body
    }
    /**
     * {@inheritDoc}
     */
    public void mouseReleased(MouseEvent e) {
      // empty method body
    }
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-25 16:09:30 Slobodan
   Initial version

*****************************************************************************^*/
