/*
 * @(#)SmoothThemeManager.java  1.0.0 / 2008-05-24
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.swing.plaf.smooth;

import net.wrka.swing.plaf.smooth.themes.DesertTheme;
import net.wrka.swing.plaf.smooth.themes.LightGrayTheme;
import net.wrka.swing.plaf.smooth.themes.NativeTheme;
import net.wrka.swing.plaf.smooth.themes.RedmondLargeTheme;
import net.wrka.swing.plaf.smooth.themes.RedmondTheme;
import net.wrka.util.SimpleHashMap;

/**
 * Smooth Theme Manager.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public final class SmoothThemeManager {

/*^**************************************************************************^*/
/*- Private Fields.                                                          -*/
/*^**************************************************************************^*/

  private static final SimpleHashMap SMOOTH_THEMES_HASH;
  private static SmoothTheme[] smoothThemes;
  private static int count;

/*^**************************************************************************^*/
/*- Static Initialization.                                                   -*/
/*^**************************************************************************^*/

  static {
    smoothThemes = craeteSmoothThemes();
    count = smoothThemes.length;
    SMOOTH_THEMES_HASH = new SimpleHashMap(count);
    for (int i = 0; i < count; i++) {
      SmoothTheme smoothTheme = smoothThemes[i];
      SMOOTH_THEMES_HASH.put(smoothTheme.getName(), smoothTheme);
    }
  }

/*^**************************************************************************^*/
/*- Private Constructor.                                                     -*/
/*^**************************************************************************^*/

  /**
   * Class has no instances.
   */
  private SmoothThemeManager() {
    // empty constructor body
  }

/*^**************************************************************************^*/
/*- Public Static Methods.                                                   -*/
/*^**************************************************************************^*/

  /**
   * Returns default theme.
   *
   * @return default theme
   */
  public static SmoothTheme getDefaultTheme() {
    return smoothThemes[0];
  }

  /**
   * Returns the number of supported themes.
   *
   * @return the number of supported themes
   */
  public static int getThemeCount() {
    return count;
  }

  /**
   * Returns theme at specified index.
   *
   * @param index the theme index
   *
   * @return theme at specified index
   *
   * @throws IllegalArgumentException if index is invalid
   */
  public static SmoothTheme getTheme(int index) {
    if ((index < 0) || (index >= count)) {
      throw new IllegalArgumentException("Invalid theme index"); //$NON-NLS-1$
    }
    return smoothThemes[index];
  }

  /**
   * Returns theme specified by the name.
   *
   * @param name the theme name
   *
   * @return theme specified by the name
   */
  public static SmoothTheme getTheme(String name) {
    return (SmoothTheme)SMOOTH_THEMES_HASH.get(name);
  }

  /**
   * Adds new theme.
   *
   * @param theme the theme to be added
   */
  public static void addTheme(SmoothTheme theme) {
    if ((theme != null) && (theme.isSupported())) {
      for (int i = 0; i < count; i++) {
        if (smoothThemes[i].equals(theme)) {
          return;
        }
      }
      ensureCapacity(count + 1);
      smoothThemes[count++] = theme;
      SMOOTH_THEMES_HASH.put(theme.getName(), theme);
    }
  }

  /**
   * Removes specified theme.
   *
   * @param theme the theme to be removed
   */
  public static void removeTheme(SmoothTheme theme) {
    if (theme != null) {
      for (int i = 0; i < count; i++) {
        if (smoothThemes[i].equals(theme)) {
          removeTheme(i);
          return;
        }
      }
    }
  }

  /**
   * Removes theme at specified index.
   *
   * @param index the theme index
   */
  public static void removeTheme(int index) {
    if ((index >= smoothThemes.length) && (index < count)) {
      SMOOTH_THEMES_HASH.remove(smoothThemes[index]);
      int numMoved = count - index - 1;
      if (numMoved > 0) {
        System.arraycopy(smoothThemes, index + 1, smoothThemes, index,
                         numMoved);
      }
      smoothThemes[--count] = null;
    }
  }

/*^**************************************************************************^*/
/*- Private Static Methods.                                                  -*/
/*^**************************************************************************^*/

  private static void ensureCapacity(int minCapacity) {
    int oldCapacity = smoothThemes.length;
    if (minCapacity > oldCapacity) {
      SmoothTheme[] oldThemes = smoothThemes;
      int newCapacity = ((3 * oldCapacity) >> 1) + 1;
      if (newCapacity < minCapacity) {
        newCapacity = minCapacity;
      }
      smoothThemes = new SmoothTheme[newCapacity];
      System.arraycopy(oldThemes, 0, smoothThemes, 0, count);
    }
  }

  private static SmoothTheme[] craeteSmoothThemes() {
    SmoothTheme[] builtInSmoothThemes = craeteBuiltInSmoothThemes();
    int count = 0;
    for (int i = builtInSmoothThemes.length - 1; i >= 0; i--) {
      if (builtInSmoothThemes[i].isSupported()) {
        count++;
      } else {
        builtInSmoothThemes[i] = null;
      }
    }
    if (count == builtInSmoothThemes.length) {
      return builtInSmoothThemes;
    }
    SmoothTheme[] smoothThemes = new SmoothTheme[count];
    int j = builtInSmoothThemes.length - 1;
    for (int i = count - 1; i >= 0; i--) {
      while ((j > 0) && (builtInSmoothThemes[j] == null)) {
        j--;
      }
      smoothThemes[i] = builtInSmoothThemes[j--];
    }
    return smoothThemes;
  }

  private static SmoothTheme[] craeteBuiltInSmoothThemes() {
    SmoothTheme[] builtInSmoothThemes = new SmoothTheme[] {
      new DefaultSmoothTheme(),
      new LightGrayTheme(),
      new DesertTheme(),
      new RedmondTheme(),
      new RedmondLargeTheme(),
      new NativeTheme()
    };
    return builtInSmoothThemes;
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-24 11:25:30 Slobodan
   Initial version

*****************************************************************************^*/
