/*
 * @(#)SmoothToggleButtonUI.java  1.0.0 / 2008-05-25
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.swing.plaf.smooth;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Insets;
import java.awt.Rectangle;
import java.beans.PropertyChangeEvent;

import javax.swing.AbstractButton;
import javax.swing.ButtonModel;
import javax.swing.JComponent;
import javax.swing.SwingUtilities;
import javax.swing.plaf.ComponentUI;
import javax.swing.plaf.basic.BasicHTML;
import javax.swing.plaf.basic.BasicToggleButtonUI;
import javax.swing.text.View;

import net.wrka.swing.graphics.GraphicUtils;

/**
 * ToggleButtonUI.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public class SmoothToggleButtonUI extends BasicToggleButtonUI {

/*^**************************************************************************^*/
/*- Package-Private Static Fields.                                           -*/
/*^**************************************************************************^*/

  static final String PROPERTY_BACKGROUND;
  static final String ACTIVE_BACKGROUND;
  static final String PRESSED_BACKGROUND;

/*^**************************************************************************^*/
/*- Private Static Fields.                                                   -*/
/*^**************************************************************************^*/

  private static final SmoothToggleButtonUI SHARED_INSTANCE;

/*^**************************************************************************^*/
/*- Static Initialization.                                                   -*/
/*^**************************************************************************^*/

  static {
    PROPERTY_BACKGROUND = "background"; //$NON-NLS-1$
    ACTIVE_BACKGROUND = "smoothActiveBackground"; //$NON-NLS-1$
    PRESSED_BACKGROUND = "smoothPressedBackground"; //$NON-NLS-1$
    SHARED_INSTANCE = new SmoothToggleButtonUI();
  }

/*^**************************************************************************^*/
/*- Private Fields.                                                          -*/
/*^**************************************************************************^*/

  private boolean deftsInited;
  private Color normalBackground;
  private Color activeBackground;
  private Color pressedBackground;
  private final ToggleButtonListener buttonListener;

/*^**************************************************************************^*/
/*- Public Constructor.                                                      -*/
/*^**************************************************************************^*/

  /**
   * Constructs new SmoothToggleButtonUI object.
   */
  public SmoothToggleButtonUI() {
    super();
    buttonListener = new ToggleButtonListener();
  }

/*^**************************************************************************^*/
/*- Public Static Methods.                                                   -*/
/*^**************************************************************************^*/

  /**
   * Factory method. Return ui for the specified component.
   *
   * @param c the component
   *
   * @return ui for the specified component
   */
  public static ComponentUI createUI(JComponent c) {
    return SHARED_INSTANCE;
  }

/*^**************************************************************************^*/
/*- Overridden Methods.                                                      -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  public void uninstallUI(JComponent c) {
    c.putClientProperty(ACTIVE_BACKGROUND, null);
    c.putClientProperty(PRESSED_BACKGROUND, null);
    super.uninstallUI(c);
  }

  /**
   * {@inheritDoc}
   */
  public void update(Graphics g, JComponent c) {
    if (c.isOpaque()) {
      ButtonModel model = ((AbstractButton)c).getModel();
      if (model.isArmed() && model.isEnabled()) {
        g.setColor((Color)c.getClientProperty(ACTIVE_BACKGROUND));
      } else {
        g.setColor(c.getBackground());
      }
      g.fillRect(0, 0, c.getWidth(), c.getHeight());
    }
    paint(g, c);
  }

  /**
   * {@inheritDoc}
   */
  public void paint(Graphics g, JComponent c) { // NOPMD
    SmoothLookAndFeel.getRenderingPolicy(this).apply((Graphics2D)g);
    AbstractButton button = (AbstractButton)c;
    ButtonModel model = button.getModel();
    Dimension size = button.getSize();
    FontMetrics fm = g.getFontMetrics();
    Insets i = button.getInsets();
    Rectangle viewRect = new Rectangle(size);
    viewRect.x += i.left;
    viewRect.y += i.top;
    viewRect.width -= (i.right + viewRect.x);
    viewRect.height -= (i.bottom + viewRect.y);
    Rectangle iconRect = new Rectangle();
    Rectangle textRect = new Rectangle();
    Font f = c.getFont();
    g.setFont(f);
    String buttonText = button.getText();
    int textIconGap;
    if (buttonText == null) {
      textIconGap = getDefaultTextIconGap(button);
    } else {
      textIconGap = 0;
    }
    String text = SwingUtilities.layoutCompoundLabel(c, fm, buttonText,
        button.getIcon(), button.getVerticalAlignment(),
        button.getHorizontalAlignment(), button.getVerticalTextPosition(),
        button.getHorizontalTextPosition(), viewRect, iconRect, textRect,
        textIconGap);
    g.setColor(button.getBackground());
    if (model.isSelected() || model.isArmed() && model.isPressed()) {
      paintButtonPressed(g, button);
    } else if (button.isOpaque() && button.isContentAreaFilled() &&
               button.isEnabled()) {
      SmoothUtils.drawFast3DEffectVertical(g, c);
    }
    if (button.getIcon() != null) {
      paintIcon(g, button, iconRect);
    }
    if ((text != null) && (text.length() != 0)) {
      View v = (View)c.getClientProperty(BasicHTML.propertyKey);
      if (v == null) {
        paintText(g, c, textRect, text);
      } else {
        v.paint(g, textRect);
      }
    }
    if (button.isFocusPainted() && button.hasFocus()) {
      paintFocus(g, button, viewRect, textRect, iconRect);
    }
  }

  /**
   * {@inheritDoc}
   */
  protected void installDefaults(AbstractButton b) {
    if (!deftsInited) {
      normalBackground = SmoothLookAndFeel.getControl();
      activeBackground = GraphicUtils.brighterColor(normalBackground,
                              SmoothLookAndFeel.getRolloverBackgroundFactor());
      pressedBackground = SmoothLookAndFeel.getControlShadow();
      deftsInited = true;
    }
    initButton(b);
    super.installDefaults(b);
  }

  /**
   * {@inheritDoc}
   */
  protected void installListeners(AbstractButton b) {
    b.addMouseListener(buttonListener);
    b.addMouseMotionListener(buttonListener);
    b.addFocusListener(buttonListener);
    b.addPropertyChangeListener(buttonListener);
    b.addChangeListener(buttonListener);
  }

  /**
   * {@inheritDoc}
   */
  protected void installKeyboardActions(AbstractButton b) {
    buttonListener.installKeyboardActions(b);
  }

  /**
   * {@inheritDoc}
   */
  protected void uninstallDefaults(AbstractButton b) {
    deftsInited = false;
    super.uninstallDefaults(b);
  }

  /**
   * {@inheritDoc}
   */
  protected void uninstallListeners(AbstractButton b) {
    b.removeMouseListener(buttonListener);
    b.removeMouseListener(buttonListener);
    b.removeMouseMotionListener(buttonListener);
    b.removeFocusListener(buttonListener);
    b.removeChangeListener(buttonListener);
    b.removePropertyChangeListener(buttonListener);
  }

  /**
   * {@inheritDoc}
   */
  protected void uninstallKeyboardActions(AbstractButton b) {
    buttonListener.uninstallKeyboardActions(b);
  }

  /**
   * {@inheritDoc}
   */
  protected void paintButtonPressed(Graphics g, AbstractButton b) {
    g.setColor((Color)b.getClientProperty(PRESSED_BACKGROUND));
    g.fillRect(1, 1, b.getWidth() - 2, b.getHeight() - 2);
  }

  /**
   * {@inheritDoc}
   */
  protected void paintFocus(Graphics g, AbstractButton b, Rectangle viewRect,
                            Rectangle textRect, Rectangle iconRect) {
    if (b.isFocusPainted() && !b.getModel().isPressed()) {
      Rectangle focusRect = b.getBounds();
      focusRect.x = 4;
      focusRect.y = 4;
      focusRect.width = focusRect.width - 8;
      focusRect.height = focusRect.height - 8;
      focusRect = focusRect.union(iconRect);
      g.setColor(SmoothLookAndFeel.getFocusColor());
      GraphicUtils.drawDashedRect(g, focusRect.x, focusRect.y,
                                  focusRect.width, focusRect.height);
    }
  }

  /**
   * {@inheritDoc}
   */
  protected void paintText(Graphics g, JComponent c, Rectangle textRect,
                           String text) {
    AbstractButton button = (AbstractButton)c;
    ButtonModel model = button.getModel();
    FontMetrics fm = g.getFontMetrics();
    if (model.isEnabled()) {
      if (button.isContentAreaFilled() &&  model.isArmed() &&
          !model.isPressed() && !model.isSelected()) {
        g.setColor(SmoothLookAndFeel.getRolloverForeground());
      } else {
        g.setColor(button.getForeground());
      }
    } else {
      g.setColor(SmoothLookAndFeel.getInactiveControlTextColor());
    }
    GraphicUtils.drawStringUnderlineCharAt(g, text,
                                           button.getDisplayedMnemonicIndex(),
                                           textRect.x,
                                           textRect.y + fm.getAscent());
  }

/*^**************************************************************************^*/
/*- Protected Method.                                                        -*/
/*^**************************************************************************^*/

  /**
   * Initializes the toggle button UI.
   *
   * @param button the button
   */
  protected void initButton(AbstractButton button) {
    Color buttonBackground = button.getBackground();
    if ((buttonBackground == null) ||
        buttonBackground.equals(normalBackground)) {
      button.putClientProperty(ACTIVE_BACKGROUND, activeBackground);
      button.putClientProperty(PRESSED_BACKGROUND, pressedBackground);
    } else {
      button.putClientProperty(ACTIVE_BACKGROUND,
                 GraphicUtils.brighterColor(buttonBackground,
                                            SmoothLookAndFeel.
                                              getRolloverBackgroundFactor()));
      button.putClientProperty(PRESSED_BACKGROUND,
                 GraphicUtils.darkerColor(buttonBackground, 0.3f));
    }
  }

/*^**************************************************************************^*/
/*- Inner Class.                                                             -*/
/*^**************************************************************************^*/

  /**
   * ToggleButtonListener.
   */
  protected class ToggleButtonListener extends SmoothButtonListener {
    /**
     * {@inheritDoc}
     */
    public void propertyChange(PropertyChangeEvent e) {
      String prop = e.getPropertyName();
      if (PROPERTY_BACKGROUND.equals(prop)) {
        initButton((AbstractButton)e.getSource());
      }
      super.propertyChange(e);
    }
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-25 17:15:53 Slobodan
   Initial version

*****************************************************************************^*/
