/*
 * @(#)SmoothUtils.java  1.0.0 / 2008-05-24
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.swing.plaf.smooth;

import java.awt.Color;
import java.awt.Component;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Paint;

import net.wrka.swing.graphics.FastGradientPaint;

/**
 * Look and Feel Helper Class.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public final class SmoothUtils {

/*^**************************************************************************^*/
/*- Private Static Final Fields.                                             -*/
/*^**************************************************************************^*/

  private static final Color TRANSLUCENT_COLOR = new Color(255, 255, 255, 0);

/*^**************************************************************************^*/
/*- Private Constructor.                                                     -*/
/*^**************************************************************************^*/

  /**
   * Class has no instances.
   */
  private SmoothUtils() {
    // empty constructor body
  }

/*^**************************************************************************^*/
/*- Public Static Methods.                                                   -*/
/*^**************************************************************************^*/

  /**
   * Fast transluct shadow gradient.
   *
   * @param g the graphics
   * @param isVertical <tt>true</tt> if gradient is vertical
   * @param x the x coordinate
   * @param y the y coordinate
   * @param w the width
   * @param h the height
   */
  public static void drawFastTranShadowGradient(Graphics g, boolean isVertical,
                                                int x, int y, int w, int h) {
    Graphics2D g2D = (Graphics2D)g;
    Paint oldPaint = g2D.getPaint();
    Color tb = SmoothLookAndFeel.getComponentGradientShadowLight();
    FastGradientPaint gradient = new FastGradientPaint(tb, TRANSLUCENT_COLOR,
                                                       isVertical);
    g2D.setPaint(gradient);
    g2D.fillRect(x, y, w, h);
    g2D.setPaint(oldPaint);
  }

  /**
   * Fast inverse transluct shadow gradient.
   *
   * @param g the graphics
   * @param isVertical <tt>true</tt> if the gradient is vertical
   * @param x the x coordinate
   * @param y the y coordinate
   * @param w the width
   * @param h the height
   */
  public static void drawFastInvTranShadowGradient(Graphics g,
                                                   boolean isVertical, int x,
                                                   int y, int w, int h) {
    Graphics2D g2D = (Graphics2D)g;
    Paint oldPaint = g2D.getPaint();
    Color tb = SmoothLookAndFeel.getComponentGradientShadow();
    FastGradientPaint gradient = new FastGradientPaint(TRANSLUCENT_COLOR, tb,
                                                       isVertical);
    g2D.setPaint(gradient);
    g2D.fillRect(x, y, w, h);
    g2D.setPaint(oldPaint);
  }

  /**
   * Fast transluct reflection gradient.
   *
   * @param g the graphics
   * @param isVertical <tt>true</tt> if the gradient is vertical
   * @param x the x coordinate
   * @param y the y coordinate
   * @param w the width
   * @param h the height
   */
  public static void drawFastTranReflectGradient(Graphics g, boolean isVertical,
                                                 int x, int y, int w, int h) {
    Graphics2D g2D = (Graphics2D)g;
    Paint oldPaint = g2D.getPaint();
    Color tw = SmoothLookAndFeel.getComponentGradientReflection();
    FastGradientPaint gradient = new FastGradientPaint(tw, TRANSLUCENT_COLOR,
                                                       isVertical);
    g2D.setPaint(gradient);
    g2D.fillRect(x, y, w, h);
    g2D.setPaint(oldPaint);
  }

  /**
   * Fast 3D vertical effect.
   *
   * @param g the graphics
   * @param c the component
   */
  public static void drawFast3DEffectVertical(Graphics g, Component c) {
    int w = c.getWidth();
    int h = c.getHeight();
    int hd2 = (h >> 1);
    Color tw = SmoothLookAndFeel.getComponentGradientReflection();
    Color tt = SmoothLookAndFeel.getComponentGradientColor();
    Color tb = SmoothLookAndFeel.getComponentGradientShadow();
    Graphics2D g2D = (Graphics2D)g;
    Paint oldPaint = g2D.getPaint();
    FastGradientPaint gradient = new FastGradientPaint(tw, tt, true);
    g2D.setPaint(gradient);
    g2D.fillRect(0, 0, w, hd2);
    gradient = new FastGradientPaint(tt, tb, true);
    g2D.setPaint(gradient);
    g2D.fillRect(0, hd2, w, hd2);
    g2D.setPaint(oldPaint);
  }

  /**
   * Fast 3D vertical effect.
   *
   * @param g the graphics
   * @param w the width
   * @param h the height
   */
  public static void drawFast3DEffectVertical(Graphics g, int w, int h) {
    Color tw = SmoothLookAndFeel.getComponentGradientReflection();
    Color tt = SmoothLookAndFeel.getComponentGradientColor();
    Color tb = SmoothLookAndFeel.getComponentGradientShadow();
    Graphics2D g2D = (Graphics2D)g;
    Paint oldPaint = g2D.getPaint();
    FastGradientPaint gradient = new FastGradientPaint(tw, tt, true);
    int h2 = (h >> 1);
    g2D.setPaint(gradient);
    g2D.fillRect(0, 0, w, h2);
    gradient = new FastGradientPaint(tt, tb, true);
    g2D.setPaint(gradient);
    g2D.fillRect(0, h2, w, h2);
    g2D.setPaint(oldPaint);
  }

  /**
   * Fast inverse 3D vertical effect.
   *
   * @param g the graphics
   * @param c the component
   */
  public static void drawInv3DEffectVertical(Graphics g, Component c) {
    int w = c.getWidth();
    int h = c.getHeight();
    Color tw = SmoothLookAndFeel.getComponentGradientReflection();
    Color tt = SmoothLookAndFeel.getComponentGradientColor();
    Color tb = SmoothLookAndFeel.getComponentGradientShadow();
    Graphics2D g2D = (Graphics2D)g;
    Paint oldPaint = g2D.getPaint();
    FastGradientPaint gradient = new FastGradientPaint(tb, tt, true);
    int h2 = (h >> 1);
    g2D.setPaint(gradient);
    g2D.fillRect(0, 0, w, h2);
    gradient = new FastGradientPaint(tt, tw, true);
    g2D.setPaint(gradient);
    g2D.fillRect(0, h2, w, h2);
    g2D.setPaint(oldPaint);
  }

  /**
   * Fast inverse 3D vertical effect.
   *
   * @param g the graphics
   * @param w the width
   * @param h the height
   */
  public static void drawFastInv3DEffectVertical(Graphics g, int w, int h) {
    Color tw = SmoothLookAndFeel.getComponentGradientReflection();
    Color tt = SmoothLookAndFeel.getComponentGradientColor();
    Color tb = SmoothLookAndFeel.getComponentGradientShadow();
    Graphics2D g2D = (Graphics2D)g;
    Paint oldPaint = g2D.getPaint();
    FastGradientPaint gradient = new FastGradientPaint(tb, tt, true);
    int h2 = (h >> 1);
    g2D.setPaint(gradient);
    g2D.fillRect(0, 0, w, h2);
    gradient = new FastGradientPaint(tw, tb, true);
    g2D.setPaint(gradient);
    g2D.fillRect(0, h2, w, h2);
    g2D.setPaint(oldPaint);
  }

  /**
   * Fast 3D horizontal effect.
   *
   * @param g the graphics
   * @param c the component
   */
  public static void drawFast3DEffectHorizontal(Graphics g, Component c) {
    int w = c.getWidth();
    int h = c.getHeight();
    Color tw = SmoothLookAndFeel.getComponentGradientReflection();
    Color tt = SmoothLookAndFeel.getComponentGradientColor();
    Color tb = SmoothLookAndFeel.getComponentGradientShadow();
    Graphics2D g2D = (Graphics2D)g;
    Paint oldPaint = g2D.getPaint();
    FastGradientPaint gradient = new FastGradientPaint(tw, tt, false);
    int w2 = (w >> 1);
    g2D.setPaint(gradient);
    g2D.fillRect(0, 0, w2, h);
    gradient = new FastGradientPaint(tt, tb, false);
    g2D.setPaint(gradient);
    g2D.fillRect(w2, 0, w2, h);
    g2D.setPaint(oldPaint);
  }

  /**
   * Fast 3D horizontal effect.
   *
   * @param g the graphics
   * @param w the width
   * @param h the height
   */
  public static void drawFast3DEffectHorizontal(Graphics g, int w, int h) {
    Color tw = SmoothLookAndFeel.getComponentGradientReflection();
    Color tt = SmoothLookAndFeel.getComponentGradientColor();
    Color tb = SmoothLookAndFeel.getComponentGradientShadow();
    Graphics2D g2D = (Graphics2D)g;
    Paint oldPaint = g2D.getPaint();
    FastGradientPaint gradient = new FastGradientPaint(tw, tt, false);
    int w2 = (w >> 1);
    g2D.setPaint(gradient);
    g2D.fillRect(0, 0, w2, h);
    gradient = new FastGradientPaint(tt, tb, false);
    g2D.setPaint(gradient);
    g2D.fillRect(w2, 0, w2, h);
    g2D.setPaint(oldPaint);
  }

  /**
   * Fast 3D light vertical effect.
   *
   * @param g the graphics
   * @param c the component
   */
  public static void drawFast3DEffectVerticalLight(Graphics g, Component c) {
    int w = c.getWidth();
    int h = c.getHeight();
    Color tw = SmoothLookAndFeel.getComponentGradientReflectionLight();
    Color tt = SmoothLookAndFeel.getComponentGradientLight();
    Color tb = SmoothLookAndFeel.getComponentGradientShadowLight();
    Graphics2D g2D = (Graphics2D)g;
    Paint oldPaint = g2D.getPaint();
    FastGradientPaint gradient = new FastGradientPaint(tw, tt, true);
    int h2 = (h >> 1);
    g2D.setPaint(gradient);
    g2D.fillRect(0, 0, w, h2);
    gradient = new FastGradientPaint(tt, tb, true);
    g2D.setPaint(gradient);
    g2D.fillRect(0, h2, w, h2);
    g2D.setPaint(oldPaint);
  }

  /**
   * Fast 3D light vertical effect.
   *
   * @param g the graphics
   * @param w the width
   * @param h the height
   */
  public static void drawFast3DEffectVerticalLight(Graphics g, int w, int h) {
    Color tw = SmoothLookAndFeel.getComponentGradientReflectionLight();
    Color tt = SmoothLookAndFeel.getComponentGradientLight();
    Color tb = SmoothLookAndFeel.getComponentGradientShadowLight();
    Graphics2D g2D = (Graphics2D)g;
    Paint oldPaint = g2D.getPaint();
    FastGradientPaint gradient = new FastGradientPaint(tw, tt, true);
    int h2 = (h >> 1);
    g2D.setPaint(gradient);
    g2D.fillRect(0, 0, w, h2);
    gradient = new FastGradientPaint(tt, tb, true);
    g2D.setPaint(gradient);
    g2D.fillRect(0, h2, w, h2);
    g2D.setPaint(oldPaint);
  }

  /**
   * Fast inverse 3D light vertical effect.
   *
   * @param g the graphics
   * @param c the component
   */
  public static void drawFastInv3DEffectVerticalLight(Graphics g, Component c) {
    int w = c.getWidth();
    int h = c.getHeight();
    Color tw = SmoothLookAndFeel.getComponentGradientReflectionLight();
    Color tt = SmoothLookAndFeel.getComponentGradientLight();
    Color tb = SmoothLookAndFeel.getComponentGradientShadowLight();
    Graphics2D g2D = (Graphics2D)g;
    Paint oldPaint = g2D.getPaint();
    FastGradientPaint gradient = new FastGradientPaint(tb, tt, true);
    int h2 = (h >> 1);
    g2D.setPaint(gradient);
    g2D.fillRect(0, 0, w, h2);
    gradient = new FastGradientPaint(tt, tw, true);
    g2D.setPaint(gradient);
    g2D.fillRect(0, h2, w, h2);
    g2D.setPaint(oldPaint);
  }

  /**
   * Fast inverse flat 3D vertical light effect.
   *
   * @param g the graphics
   * @param c the component
   */
  public static void drawFastInvFlat3DEffectVerticalLight(Graphics g,
                                                          Component c) {
    int w = c.getWidth();
    int h = c.getHeight();
    int h2 = (h >> 1);
    int dh;
    if (20 < h2) {
      dh = 20;
    } else {
      dh = h2;
    }
    Color tw = SmoothLookAndFeel.getComponentGradientReflectionLight();
    Color tt = SmoothLookAndFeel.getComponentGradientLight();
    Color tb = SmoothLookAndFeel.getComponentGradientShadowLight();
    Graphics2D g2D = (Graphics2D)g;
    Paint oldPaint = g2D.getPaint();
    FastGradientPaint gradient = new FastGradientPaint(tb, tt, true);
    g2D.setPaint(gradient);
    g2D.fillRect(0, 0, w, dh);
    g2D.setColor(tt);
    g2D.fillRect(0, dh, w, h - dh);
    gradient = new FastGradientPaint(tt, tw, true);
    g2D.setPaint(gradient);
    g2D.fillRect(0, h - dh, w, dh + 1);
    g2D.setPaint(oldPaint);
  }

  /**
   * Fast inverse flat 3D vertical light effect.
   *
   * @param g the graphics
   * @param x the x coordinate
   * @param y the y coordinate
   * @param w the width
   * @param h the height
   */
  public static void drawFastInvFlat3DEffectVerticalLight(Graphics g, int x,
                                                          int y, int w, int h) {
    Color tw = SmoothLookAndFeel.getComponentGradientReflectionLight();
    Color tt = SmoothLookAndFeel.getComponentGradientLight();
    Color tb = SmoothLookAndFeel.getComponentGradientShadowLight();
    Graphics2D g2D = (Graphics2D)g;
    Paint oldPaint = g2D.getPaint();
    FastGradientPaint gradient = new FastGradientPaint(tb, tt, true);
    int h2 = (h >> 1);
    g2D.translate(x, y);
    g2D.setPaint(gradient);
    g2D.fillRect(0, 0, w, h2);
    gradient = new FastGradientPaint(tt, tw, true);
    g2D.setPaint(gradient);
    g2D.fillRect(0, h2, w, h2 + 1);
    g2D.setPaint(oldPaint);
    g2D.translate(-x, -y);
  }

  /**
   * Fast inverse flat 3D vertical light effect.
   *
   * @param g the graphics
   * @param x the x coordinate
   * @param y the y coordinate
   * @param w the width
   * @param h the height
   */
  public static void drawFastInvFlat3DEffectVertical(Graphics g, int x, int y,
                                                     int w, int h) {
    int h2 = (h >> 1);
    int dh;
    if (20 < h2) {
      dh = 20;
    } else {
      dh = h2;
    }
    Color tw = SmoothLookAndFeel.getComponentGradientReflection();
    Color tt = SmoothLookAndFeel.getComponentGradientColor();
    Color tb = SmoothLookAndFeel.getComponentGradientShadow();
    Graphics2D g2D = (Graphics2D)g;
    Paint oldPaint = g2D.getPaint();
    FastGradientPaint gradient = new FastGradientPaint(tb, tt, true);
    g2D.setPaint(gradient);
    g2D.fillRect(0, 0, w, dh);
    g2D.setColor(tt);
    g2D.fillRect(0, dh, w, h - dh);
    gradient = new FastGradientPaint(tt, tw, true);
    g2D.setPaint(gradient);
    g2D.fillRect(0, h - dh, w, dh + 1);
    g2D.setPaint(oldPaint);
  }

  /**
   * Fast 3D light horizontal effect.
   *
   * @param g the graphics
   * @param c the component
   */
  public static void drawFast3DEffectHorizontalLight(Graphics g, Component c) {
    int w = c.getWidth();
    int h = c.getHeight();
    Color tw = SmoothLookAndFeel.getComponentGradientReflectionLight();
    Color tt = SmoothLookAndFeel.getComponentGradientLight();
    Color tb = SmoothLookAndFeel.getComponentGradientShadowLight();
    Graphics2D g2D = (Graphics2D)g;
    Paint oldPaint = g2D.getPaint();
    FastGradientPaint gradient = new FastGradientPaint(tw, tt, false);
    int w2 = (w >> 1);
    g2D.setPaint(gradient);
    g2D.fillRect(0, 0, w2, h);
    gradient = new FastGradientPaint(tt, tb, false);
    g2D.setPaint(gradient);
    g2D.fillRect(w2, 0, w2, h);
    g2D.setPaint(oldPaint);
  }

  /**
   * Fast 3D horizontal light effect.
   *
   * @param g the graphics
   * @param w the width
   * @param h the height
   */
  public static void drawFast3DEffectHorizontalLight(Graphics g, int w, int h) {
    Color tw = SmoothLookAndFeel.getComponentGradientReflectionLight();
    Color tt = SmoothLookAndFeel.getComponentGradientLight();
    Color tb = SmoothLookAndFeel.getComponentGradientShadowLight();
    Graphics2D g2D = (Graphics2D)g;
    Paint oldPaint = g2D.getPaint();
    FastGradientPaint gradient = new FastGradientPaint(tw, tt, false);
    int w2 = (w >> 1);
    g2D.setPaint(gradient);
    g2D.fillRect(0, 0, w2, h);
    gradient = new FastGradientPaint(tt, tb, false);
    g2D.setPaint(gradient);
    g2D.fillRect(w2, 0, w2, h);
    g2D.setPaint(oldPaint);
  }

  /**
   * Fast flat 3D vertical light effect.
   *
   * @param g the graphics
   * @param c the component
   */
  public static void drawFastInvFlat3DEffectHorizontalLight(Graphics g,
                                                            Component c) {
    int w = c.getWidth();
    int h = c.getHeight();
    Color tw = SmoothLookAndFeel.getComponentGradientReflectionLight();
    Color tt = SmoothLookAndFeel.getComponentGradientLight();
    Color tb = SmoothLookAndFeel.getComponentGradientShadowLight();
    Graphics2D g2D = (Graphics2D)g;
    Paint oldPaint = g2D.getPaint();
    FastGradientPaint gradient = new FastGradientPaint(tw, tt, false);
    int w2 = (w >> 1);
    g2D.setPaint(gradient);
    g2D.fillRect(0, 0, w2, h);
    gradient = new FastGradientPaint(tt, tb, false);
    g2D.setPaint(gradient);
    g2D.fillRect(w2, 0, w2 + 1, h);
    g2D.setPaint(oldPaint);
  }

  /**
   * Fast flat 3D vertical light effect.
   *
   * @param g the graphics
   * @param x the x coordinate
   * @param y the y coordinate
   * @param w the width
   * @param h the height
   */
  public static void drawFastInvFlat3DEffectHorizontalLight(Graphics g, int x,
                                                            int y, int w,
                                                            int h) {
    //Color tw = SmoothLookAndFeel.getComponentGradientColorReflectionLight();
    //Color tt = SmoothLookAndFeel.getComponentGradientColorLight();
    //Color tb = SmoothLookAndFeel.getComponentGradientColorShadowLight();
    Color tw = SmoothLookAndFeel.getComponentGradientShadowLight();
    Color tt = SmoothLookAndFeel.getComponentGradientLight();
    Color tb = SmoothLookAndFeel.getComponentGradientReflectionLight();
    Graphics2D g2D = (Graphics2D)g;
    Paint oldPaint = g2D.getPaint();
    FastGradientPaint gradient = new FastGradientPaint(tw, tt, false);
    int w2 = w >> 1;
    g2D.translate(x, y);
    g2D.setPaint(gradient);
    g2D.fillRect(0, 0, w2, h);
    gradient = new FastGradientPaint(tt, tb, false);
    g2D.setPaint(gradient);
    g2D.fillRect(w2, 0, w2 + 1, h);
    g2D.setPaint(oldPaint);
    g2D.translate(-x, -y);
  }

  /**
   * Draws pressed 3D border.
   *
   * @param g the graphics
   * @param x the x coordinate
   * @param y the y coordinate
   * @param w the width
   * @param h the height
   */
  public static void drawPressed3DBorder(Graphics g, int x, int y, int w,
                                         int h) {
    g.translate(x, y);
    drawFlush3DBorder(g, 0, 0, w, h);
    g.setColor(SmoothLookAndFeel.getControlShadow());
    g.drawLine(1, 1, 1, h - 2);
    g.drawLine(1, 1, w - 2, 1);
    g.translate(-x, -y);
  }

  /**
   * Draws flush 3D border.
   *
   * @param g the graphics
   * @param x the x coordinate
   * @param y the y coordinate
   * @param w the width
   * @param h the height
   */
  public static void drawFlush3DBorder(Graphics g, int x, int y, int w, int h) {
    g.translate(x, y);
    g.setColor(SmoothLookAndFeel.getControlDarkShadow());
    int wm2 = w - 2;
    int hm2 = h - 2;
    g.drawRect(0, 0, wm2, hm2);
    g.setColor(SmoothLookAndFeel.getControlHighlight());
    g.drawRect(1, 1, wm2, hm2);
    g.setColor(SmoothLookAndFeel.getControl());
    g.drawLine(0, h - 1, 1, hm2);
    g.drawLine(w - 1, 0, wm2, 1);
    g.translate(-x, -y);
  }

  /**
   * Draws disabled border.
   *
   * @param g the graphics
   * @param x the x coordinate
   * @param y the y coordinate
   * @param w the width
   * @param h the height
   */
  public static void drawDisabledBorder(Graphics g, int x, int y, int w,
                                        int h) {
    g.setColor(SmoothLookAndFeel.getControlShadow());
    g.drawRect(x, y, w - 1, h - 1);
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-24 09:15:07 Slobodan
   Initial version

*****************************************************************************^*/
