/*
 * @(#)AbstractPreferencesGroup.java  1.0.0 / 2008-05-18
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.swing.preferences;

import java.util.prefs.BackingStoreException;
import java.util.prefs.Preferences;

import net.wrka.util.SimpleStringMap;

/**
 * Abstract Preferences Group.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public abstract class AbstractPreferencesGroup implements PreferencesGroup {

/*^**************************************************************************^*/
/*- Protected Fields.                                                        -*/
/*^**************************************************************************^*/

  /**
   * Group name.
   */
  protected final String groupName;

  /**
   * Node name.
   */
  protected final String nodeName;

  /**
   * Preferences.
   */
  protected final Preferences preferences;

/*^**************************************************************************^*/
/*- Protected Constructor.                                                   -*/
/*^**************************************************************************^*/

  /**
   * Constructs new AboutPanel object.
   *
   * @param nodeName the name of the group node
   * @param groupName the name of the group
   */
  protected AbstractPreferencesGroup(String nodeName, String groupName) {
    if (nodeName == null) {
      throw new IllegalArgumentException("Invalid node name"); //$NON-NLS-1$
    }
    this.nodeName = nodeName;
    this.groupName = groupName;
    preferences = PersistentPreferences.getUserPreferences().node(nodeName);
  }

/*^**************************************************************************^*/
/*- PreferencesGroup Implementation.                                         -*/
/*^**************************************************************************^*/

  /**
   * Returns the name of the group.
   *
   * @return the name of the group
   */
  public String getGroupName() {
    return groupName;
  }

  /**
   * Returns preferences.
   *
   * @return preferences
   */
  public Preferences getPreferences() {
    return preferences;
  }

  /**
   * Installs preferences.
   *
   * @return <tt>true</tt> if succeed
   */
  public boolean install() {
    String[] defprefs = getDefaultPreferences();
    if ((defprefs == null) || (defprefs.length <= 0) ||
        ((defprefs.length % 2) != 0)) {
      throw new IllegalStateException("Invalid default preferences");
    }
    try {
      String[] keys = preferences.keys();
      if (keys.length == 0) {
        for (int i = defprefs.length - 2; i >= 0; i -= 2) {
          preferences.put(defprefs[i], defprefs[i + 1]);
        }
      } else {
        SimpleStringMap hashedkeys = new SimpleStringMap(keys);
        for (int i = defprefs.length - 2; i >= 0; i -= 2) {
          String defkey = defprefs[i];
          if (!hashedkeys.containsKey(defkey)) {
            preferences.put(defkey, defprefs[i + 1]);
          }
        }
      }
      return true;
    } catch (BackingStoreException bse) {
      return false;
    }
  }

  /**
   * Uninstalls preferences.
   *
   * @return <tt>true</tt> if succeed
   */
  public boolean uninstall() {
    String[] defprefs = getDefaultPreferences();
    try {
      String[] keys = preferences.keys();
      if (keys.length != 0) {
        SimpleStringMap hashedkeys = new SimpleStringMap(keys);
        for (int i = defprefs.length - 2; i >= 0; i -= 2) {
          String defkey = defprefs[i];
          if (hashedkeys.containsKey(defkey)) {
            preferences.remove(defkey);
          }
        }
      }
      preferences.removeNode();
      return true;
    } catch (BackingStoreException bse) {
      return false;
    }
  }

/*^**************************************************************************^*/
/*- Protected Method.                                                        -*/
/*^**************************************************************************^*/

  /**
   * Returns default preferences.
   *
   * @return default preferences
   */
  protected abstract String[] getDefaultPreferences();

}

/*^*****************************************************************************
  File History:

 - 2008-05-18 17:38:00 Slobodan
   Initial version

*****************************************************************************^*/
