/*
 * @(#)ApplicationPreferences.java  1.0.0 / 2008-05-18
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.swing.preferences;

import net.wrka.util.SimpleHashMap;
import net.wrka.util.ref.WeakArrayList;

/**
 * Application non-storeable preferences.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public final class ApplicationPreferences {

/*^**************************************************************************^*/
/*- Private Static Fields.                                                   -*/
/*^**************************************************************************^*/

  private static final SimpleHashMap PREFERENCES = new SimpleHashMap();
  private static final WeakArrayList LISTENERS = new WeakArrayList();
  private static boolean argsSet;

/*^**************************************************************************^*/
/*- Private Constructor.                                                     -*/
/*^**************************************************************************^*/

  /**
   * Class has no instances.
   */
  private ApplicationPreferences() {
    // empty constructor body
  }

/*^**************************************************************************^*/
/*- Public Static Methods.                                                   -*/
/*^**************************************************************************^*/

  /**
   * Adds a <tt>ApplicationPreferencesListener</tt> to the listener list.
   *
   * @param apl the preferences listener
   */
  public static void addListener(ApplicationPreferencesListener apl) {
    LISTENERS.add(apl);
  }

  /**
   * Removes a <tt>ApplicationPreferencesListener</tt> from the listener list.
   *
   * @param apl the preferences listener
   */
  public static void removeListener(ApplicationPreferencesListener apl) {
    LISTENERS.remove(apl);
  }

  /**
   * Returns the number of preferences.
   *
   * @return the number of preferences
   */
  public static int count() {
    return PREFERENCES.size();
  }

  /**
   * Returns <tt>true</tt> if there are no application preferences.
   *
   * @return <tt>true</tt> if there are no application preferences
   */
  public static boolean empty() {
    return PREFERENCES.empty();
  }

  /**
   * Returns <tt>true</tt> if preferences table contains specified preference.
   *
   * @param name the preference name
   *
   * @return <tt>true</tt> if preferences table contains specified preference
   */
  public static boolean contains(String name) {
    return PREFERENCES.containsKey(name);
  }

  /**
   * Returns the value of specified preference.
   *
   * @param name the preference name
   *
   * @return the value of specified preference
   */
  public static Object get(String name) {
    return PREFERENCES.get(name);
  }

  /**
   * Returns string value of the specified preference.
   *
   * @param name the preference name
   *
   * @return string value of the specified preference
   */
  public static String getString(String name) {
    Object value = PREFERENCES.get(name);
    if (value instanceof String) {
      return (String)value;
    }
    return null;
  }

  /**
   * Returns string value of the specified preference or default value if there
   * is no such preference.
   *
   * @param name the preference name
   * @param defval default value
   *
   * @return string value of the specified preference or default value if there
   *         is no such preference
   */
  public static String getString(String name, String defval) {
    Object value = PREFERENCES.get(name);
    if (value instanceof String) {
      return (String)value;
    }
    return defval;
  }

  /**
   * Returns boolean value of the specified preference or default value if there
   * is no such preference.
   *
   * @param name the preference name
   * @param defval default value
   *
   * @return boolean value of the specified preference or default value if there
   *         is no such preference
   */
  public static boolean getBoolean(String name, boolean defval) {
    Object value = PREFERENCES.get(name);
    if (value instanceof Boolean) {
      return ((Boolean)value).booleanValue();
    }
    return defval;
  }

  /**
   * Returns integer value of the specified preference or default value if there
   * is no such preference.
   *
   * @param name the preference name
   * @param defval default value
   *
   * @return integer value of the specified preference or default value if there
   *         is no such preference
   */
  public static int getInteger(String name, int defval) {
    Object value = PREFERENCES.get(name);
    if (value instanceof Integer) {
      return ((Integer)value).intValue();
    }
    return defval;
  }

  /**
   * Returns float value of the specified preference or default value if there
   * is no such preference.
   *
   * @param name the preference name
   * @param defval default value
   *
   * @return float value of the specified preference or default value if there
   *         is no such preference
   */
  public static float getFloat(String name, float defval) {
    Object value = PREFERENCES.get(name);
    if (value instanceof Float) {
      return ((Float)value).floatValue();
    }
    return defval;
  }

  /**
   * Returns double value of the specified preference or default value if there
   * is no such preference.
   *
   * @param name the preference name
   * @param defval default value
   *
   * @return double value of the specified preference or default value if there
   *         is no such preference
   */
  public static double getDouble(String name, double defval) {
    Object value = PREFERENCES.get(name);
    if (value instanceof Double) {
      return ((Double)value).floatValue();
    }
    return defval;
  }

  /**
   * Puts new preference.
   *
   * @param name the preference name
   * @param value the preference value
   */
  public static void put(String name, Object value) {
    Object oldval;
    if (value == null) {
      oldval = PREFERENCES.remove(name);
    } else {
      oldval = PREFERENCES.put(name, value);
    }
    if (oldval != value) { // NOPMD
      firePreferenceChanged(name, oldval, value);
    }
  }

  /**
   * Puts main function arguments.
   *
   * @param args the main function arguments
   */
  public static void putArgs(String[] args) {
    if (!argsSet && (args != null)) { // NOPMD - not singleton
      SimpleHashMap preferences = PREFERENCES;
      for (int i = args.length - 1; i >= 0; i--) {
        String arg = args[i];
        if (arg.charAt(0) == '-') {
          if (arg.charAt(1) == '-') {
            if (arg.charAt(2) != '-') {
              arg = arg.substring(2, arg.length());
              String[] kvp = arg.split("="); //$NON-NLS-1$
              if (kvp.length == 1) {
                preferences.put(kvp[0].trim(), Boolean.TRUE);
              } else if (kvp.length == 2) {
                preferences.put(kvp[0].trim(), kvp[1].trim());
              }
            }
          } else {
            for (int j = 1; j < arg.length(); j++) {
              char c = arg.charAt(j);
              preferences.put(String.valueOf(c), Boolean.TRUE);
            }
          }
        }
      }
      argsSet = true;
    }
  }

/*^**************************************************************************^*/
/*- Private Static Method.                                                   -*/
/*^**************************************************************************^*/

  private static void firePreferenceChanged(String name, Object oldValue,
                                            Object newValue) {
    boolean hasnull = false;
    WeakArrayList listeners = LISTENERS;
    for (int i = listeners.size() - 1; i >= 0; i--) {
      ApplicationPreferencesListener apl =
                              (ApplicationPreferencesListener)listeners.get(i);
      if (apl == null) {
        hasnull = true;
      } else {
        apl.preferenceChanged(name, oldValue, newValue);
      }
    }
    if (hasnull) {
      listeners.expungeStaleReferences();
    }
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-18 17:50:42 Slobodan
   Initial version

*****************************************************************************^*/
