/*
 * @(#)PersistentPreferences.java  1.0.0 / 2008-05-18
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.swing.preferences;

import java.awt.Dialog;
import java.awt.Frame;
import java.awt.KeyboardFocusManager;
import java.awt.Window;
import java.awt.event.ActionEvent;
import java.io.File;
import java.util.prefs.BackingStoreException;
import java.util.prefs.Preferences;

import javax.swing.Action;
import javax.swing.LookAndFeel;
import javax.swing.SwingUtilities;

import net.wrka.swing.actions.PreferencesAction;
import net.wrka.swing.plaf.LookAndFeelChangeListener;
import net.wrka.swing.plaf.LookAndFeelManager;
import net.wrka.util.SimpleArrayList;
import net.wrka.util.ref.SimpleSoftValueHashMap;
import net.wrka.xml.preferences.XMLPreferencesFactory;

/**
 * Persistent Preferences Support.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public final class PersistentPreferences implements LookAndFeelChangeListener {

/*^**************************************************************************^*/
/*- Public Static Final Fields.                                              -*/
/*^**************************************************************************^*/

  /**
   * Resources.
   */
  public static final String RESOURCES = "PreferenceResources"; //$NON-NLS-1$

  /**
   * Resource key <i>title</i>.
   */
  public static final String RK_TITLE =
                                   "PersistentPreferences.title"; //$NON-NLS-1$

  /**
   * Resource key <i>apply button text</i>.
   */
  public static final String RK_APPLY_BUTTON_TEXT =
                         "PersistentPreferences.applyButtonText"; //$NON-NLS-1$

  /**
   * Resource key <i>apply button tooltip</i>.
   */
  public static final String RK_APPLY_BUTTON_TOOLTIP =
                      "PersistentPreferences.applyButtonToolTip"; //$NON-NLS-1$

  /**
   * Resource key <i>cancel button text</i>.
   */
  public static final String RK_CANCEL_BUTTON_TEXT =
                        "PersistentPreferences.cancelButtonText"; //$NON-NLS-1$

  /**
   * Resource key <i>cancel button tooltip</i>.
   */
  public static final String RK_CANCEL_BUTTON_TOOLTIP =
                     "PersistentPreferences.cancelButtonToolTip"; //$NON-NLS-1$

  /**
   * Resource key <i>help button title</i>.
   */
  public static final String RK_HELP_BUTTON_TEXT =
                          "PersistentPreferences.helpButtonText"; //$NON-NLS-1$

  /**
   * Resource key <i>help button tooltip</i>.
   */
  public static final String RK_HELP_BUTTON_TOOLTIP =
                       "PersistentPreferences.helpButtonToolTip"; //$NON-NLS-1$

  /**
   * Resource key <i>help topic</i>.
   */
  public static final String RK_HELP_TOPIC =
                                 "PersistentPreferences.helpTOC"; //$NON-NLS-1$

  /**
   * Resource key <i>remember changes</i>.
   */
  public static final String RK_REMEMBER_CHANGES =
                         "PersistentPreferences.rememberChanges"; //$NON-NLS-1$

  /**
   * Resource key <i>show splash</i>.
   */
  public static final String RK_SHOW_SPLASH =
                           "PersistentPreferences.turnOffSplash"; //$NON-NLS-1$

/*^**************************************************************************^*/
/*- Private-Private Static Fields.                                           -*/
/*^**************************************************************************^*/

  static String title;
  static Preferences userPreferences;
  static Preferences systemPreferences;
  static SimpleArrayList prefGroups;
  static boolean lnfChanged;
  static boolean useNatevePreferences;

/*^**************************************************************************^*/
/*- Private Static Fields.                                                   -*/
/*^**************************************************************************^*/

  private static PersistentPreferences lnfListener;
  private static PreferencesShutdownHook shutdownHook;
  private static SimpleSoftValueHashMap dialogCache;

/*^**************************************************************************^*/
/*- Static Initialization.                                                   -*/
/*^**************************************************************************^*/

  static {
    prefGroups = new SimpleArrayList(4);
    dialogCache = new SimpleSoftValueHashMap(5);
    lnfListener = new PersistentPreferences();
  }

/*^**************************************************************************^*/
/*- Private Constructor.                                                     -*/
/*^**************************************************************************^*/

  /**
   * Class has no instances.
   */
  private PersistentPreferences() {
    // empty constructor body
  }

/*^**************************************************************************^*/
/*- LookAndFeelChangeListener Implementation.                                -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  public void lookAndFeelChanged(LookAndFeel oldLnF, LookAndFeel newLnF) {
    lnfChanged = true; // FB: OK - there is only one object.
  }

/*^**************************************************************************^*/
/*- Public Static Methods.                                                   -*/
/*^**************************************************************************^*/

  /**
   * Initialize preferences.
   *
   * @param useNative use native preferences
   * @param name preferences name
   * @param userPrefsPath String
   */
  public static void initialize(boolean useNative, String name,
                                String userPrefsPath) {
    initialize(useNative, name, userPrefsPath, null);
  }

  /**
   * Initialize preferences.
   *
   * @param useNative use native preferences
   * @param name the preferences name
   * @param userPrefsPath the user preferences path
   * @param systemPrefsPath the user preferences path
   */
  public static void initialize(boolean useNative, String name,
                                String userPrefsPath, String systemPrefsPath) {
    if (title != null) {
      throw new IllegalArgumentException(
                 "PersistentPreferences are already initialized"); //$NON-NLS-1$
    }
    if (name == null) {
      throw new IllegalArgumentException("Null preferences name"); //$NON-NLS-1$
    }
    title = name;
    useNatevePreferences = useNative;
    if (!useNatevePreferences) {
      try {
        XMLPreferencesFactory.initialize(null,
            new File(name.toLowerCase().concat(".cfg"))); //$NON-NLS-1$ NOPMD
      } catch (BackingStoreException e) {
        // CHANGE: log error
        System.out.println(// NOPMD
                             "Could not initialize preferences"); //$NON-NLS-1$
      }
    }
    if (userPrefsPath != null) {
      userPreferences = Preferences.userRoot().node(userPrefsPath);
    }
    if (systemPrefsPath != null) {
      systemPreferences = Preferences.systemRoot().node(systemPrefsPath);
    }
    LookAndFeelManager.addLnFListener(lnfListener);
  }

  /**
   * Returns user preferences.
   *
   * @return user preferences
   */
  public static Preferences getUserPreferences() {
    checkInit();
    return userPreferences;
  }

  /**
   * Returns system preferences.
   *
   * @return system preferences
   */
  public static Preferences getSystemPreferences() {
    checkInit();
    return systemPreferences;
  }

  /**
   * Returns preferences action.
   *
   * @return preferences action
   */
  public static Action getPreferencesAction() {
    checkInit();
    return new PreferencesActionShow();
  }

  /**
   * Adds preferences group.
   *
   * @param pg the preferences group
   */
  public static void addPreferenceGroup(PreferencesGroup pg) {
    if (pg == null) {
      throw new IllegalArgumentException(
                                       "Null preferences group"); //$NON-NLS-1$
    }
    checkInit();
    pg.install();
    prefGroups.add(pg);
  }

  /**
   * Adds preferences group.
   *
   * @param pg the preferences group
   * @param index the index
   */
  public static void addPreferenceGroup(PreferencesGroup pg, int index) {
    if (pg == null) {
      throw new IllegalArgumentException(
                                       "Null preferences group"); //$NON-NLS-1$
    }
    checkInit();
    pg.install();
    prefGroups.insert(index, pg);
  }

  /**
   * Removes preferences group.
   *
   * @param pg the preferences group
   */
  public static void removePreferenceGroup(PreferencesGroup pg) {
    if (pg == null) {
      throw new IllegalArgumentException(
                                       "Null preferences group"); //$NON-NLS-1$
    }
    checkInit();
    if (!prefGroups.contains(pg)) {
      throw new IllegalArgumentException(
                           "Preferences group is not installed"); //$NON-NLS-1$
    }
    prefGroups.remove(pg);
    pg.uninstall();
  }

  /**
   * Displays preferences dialog.
   */
  public static void show() {
    checkInit();
    KeyboardFocusManager ckfm = KeyboardFocusManager.
                                              getCurrentKeyboardFocusManager();
    Window invoker = ckfm.getActiveWindow();
    PreferencesDialog dialog = null;
    if (invoker != null) {
      dialog = (PreferencesDialog)dialogCache.get(invoker);
    }
    if (dialog == null) {
      if (invoker instanceof Dialog) {
        dialog = new PreferencesDialog((Dialog)invoker, title);
      } else if (invoker instanceof Frame) {
        dialog = new PreferencesDialog((Frame)invoker, title);
      } else {
        return;
      }
      dialogCache.put(invoker, dialog);
    } else {
      if (lnfChanged) {
        SwingUtilities.updateComponentTreeUI(dialog);
        lnfChanged = false;
      }
      dialog.updatePanels();
    }
    dialog.setVisible(true);
  }

  /**
   * Installs shutdown hook.
   */
  public static void installShutdownHook() {
    checkInit();
    if (shutdownHook == null) { // NOPMD - not thread safe
      shutdownHook = new PreferencesShutdownHook();
      Runtime.getRuntime().addShutdownHook(shutdownHook);
    }
  }

  /**
   * Uninstalls shutdown hook.
   */
  public static void removeShutdownHook() {
    checkInit();
    if (shutdownHook != null) { // NOPMD - not thread safe
      Runtime.getRuntime().removeShutdownHook(shutdownHook);
      shutdownHook = null;
    }
  }

/*^**************************************************************************^*/
/*- Private Static Methods.                                                  -*/
/*^**************************************************************************^*/

  /**
   * Initialization checking.
   */
  private static void checkInit() {
    if (title == null) {
      throw new IllegalStateException(
                       "Preferences dialog is not initialized."); //$NON-NLS-1$
    }
  }

/*^**************************************************************************^*/
/*- Inner Classes.                                                           -*/
/*^**************************************************************************^*/

  /**
   * Preferences Action.
   */
  static final class PreferencesActionShow extends PreferencesAction {
    /**
     * {@inheritDoc}
     */
    public void actionPerformed(ActionEvent e) {
      show();
    }
  }

  /**
   * Shutdown Hook.
   */
  static final class PreferencesShutdownHook extends Thread {
    /**
     * Constructs new PreferencesShutdownHook object.
     */
    public PreferencesShutdownHook() {
      super("PreferencesShutdownHook"); //$NON-NLS-1$
    }
    /**
     * {@inheritDoc}
     */
    public void run() {
      storePreferences(userPreferences);
      storePreferences(systemPreferences);
    }
    /**
     * Stores the preferences.
     *
     * @param p the preferences
     */
    public void storePreferences(Preferences p) {
      if (p != null) {
        try {
          boolean sp = p.getBoolean(GeneralPreferences.STORE_PREFERENCES,
                                    false);
          if (useNatevePreferences) {
            if (!sp) {
              p.removeNode();
            }
          } else {
            if (sp) {
              p.flush();
            } else {
              File f;
              if (p.isUserNode()) {
                f = XMLPreferencesFactory.getUserPreferencesFile();
              } else {
                f = XMLPreferencesFactory.getSystemPreferencesFile();
              }
              if ((f != null) && f.exists()) {
                f.delete();
              }
            }
          }
        } catch (BackingStoreException bse) {
          // CHANGE: use logger
          if (p.isUserNode()) {
            System.out.println(// NOPMD
                             "Could not store user preferences"); //$NON-NLS-1$
          } else {
            System.out.println(// NOPMD
                            "Could not system user preferences"); //$NON-NLS-1$
          }
        }
      }
    }
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-18 21:40:19 Slobodan
   Initial version

*****************************************************************************^*/
