/*
 * @(#)PlafPreferences.java  1.0.0 / 2008-05-18
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.swing.preferences;

import java.awt.Component;
import java.util.Iterator;

import javax.swing.JComponent;
import javax.swing.LookAndFeel;
import javax.swing.SwingUtilities;
import javax.swing.plaf.metal.MetalLookAndFeel;

import net.wrka.swing.plaf.LookAndFeelChangeListener;
import net.wrka.swing.plaf.LookAndFeelManager;
import net.wrka.swing.plaf.metal.MetalThemeManager;
import net.wrka.swing.plaf.smooth.SmoothLookAndFeel;
import net.wrka.swing.plaf.smooth.SmoothThemeManager;
import net.wrka.swing.resources.BuiltInResources;
import net.wrka.util.ref.WeakCache;

/**
 * Look and Feel Preferences.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public final class PlafPreferences extends AbstractPreferencesGroup
                                   implements LookAndFeelChangeListener {

/*^**************************************************************************^*/
/*- Public Static Final Fields.                                              -*/
/*^**************************************************************************^*/

  /**
   * PLaF group name string key.
   */
  public static final String RK_GROUP_NAME = "LnFGroupName"; //$NON-NLS-1$

  /**
   * PLaF name string key.
   */
  public static final String PLAF_NAME = "plafname"; //$NON-NLS-1$

  /**
   * PLaF class string key.
   */
  public static final String PLAF_CLASS_NAME = "plafclassname"; //$NON-NLS-1$

  /**
   * PLaF theme string key.
   */
  public static final String PLAF_THEME = "plaftheme"; //$NON-NLS-1$

/*^**************************************************************************^*/
/*- Package-Private Static Field.                                            -*/
/*^**************************************************************************^*/

  static PlafPreferences instance;

/*^**************************************************************************^*/
/*- Static Fields.                                                           -*/
/*^**************************************************************************^*/

  private final WeakCache lnfTargets;
  private boolean canUpdateLnFTargets;

/*^**************************************************************************^*/
/*- Private Constructor.                                                     -*/
/*^**************************************************************************^*/

  /**
   * Constructs new PlafPreferences object.
   */
  private PlafPreferences() {
    super("lookandfeel", BuiltInResources.INSTANCE.//$NON-NLS-1$
                           getResourceBundle(PersistentPreferences.RESOURCES).
                                                      getString(RK_GROUP_NAME));
    lnfTargets = new WeakCache(5);
    canUpdateLnFTargets = true;
    LookAndFeelManager.addLnFListener(this);
  }

/*^**************************************************************************^*/
/*- Static Methods.                                                          -*/
/*^**************************************************************************^*/

  /**
   * Returns singleton instance of this class.
   *
   * @return singleton instance of this class
   */
  public static PlafPreferences getInstance() {
    if (instance == null) { // NOPMD not thread safe
      instance = new PlafPreferences();
    }
    return instance;
  }

  /**
   * Returns <tt>true</tt> if singleton instance is initialized.
   *
   * @return <tt>true</tt> if singleton instance is initialized
   */
  public static boolean initialized() {
    return (instance != null);
  }

/*^**************************************************************************^*/
/*- Public Methods.                                                          -*/
/*^**************************************************************************^*/

  /**
   * Adds component to be updated when lnf changes.
   *
   * @param comp the target component
   */
  public void addLnFChangeComponent(Component comp) {
    lnfTargets.put(comp);
  }

  /**
   * Returns <tt>true</tt> if contains specified lnf target component.
   *
   * @param target target component
   *
   * @return <tt>true</tt> if contains specified lnf target component
   */
  public boolean hasLnFChangeComponent(Component target) {
    return lnfTargets.contains(target);
  }
/*^**************************************************************************^*/
/*- Overridden Methods.                                                      -*/
/*^**************************************************************************^*/

  /**
   * Returns default preferences.
   *
   * @return default preferences
   */
  public String[] getDefaultPreferences() {
    return new String[] {
      PLAF_NAME, SmoothLookAndFeel.NAME,
      PLAF_CLASS_NAME, SmoothLookAndFeel.class.getName(),
      PLAF_THEME, String.valueOf(-1)
    };
  }

  /**
   * Returns group title.
   *
   * @return group title
   */
  public String getGroupTitle() {
    return "Look and Feel Preferences"; //$NON-NLS-1$
  }

  /**
   * Creates panel.
   *
   * @param parent panel's parent.
   *
   * @return JPanel
   */
  public PreferencesPanel getGroupPanel(JComponent parent) {
    return new PlafPreferencesPanel(parent, preferences);
  }

  /**
   * Installs this preferences.
   *
   * @return <tt>false</tt> if failed
   */
  public boolean install() {
    boolean res = super.install();
    if (res &&
          (System.getProperty(LookAndFeelManager.SWING_DEFAULT_LAF) == null)) {
      String className = preferences.get(PLAF_CLASS_NAME, null);
      if (className != null) {
        int themeIndex = preferences.getInt(PlafPreferences.PLAF_THEME, -1);
        if (themeIndex != -1) {
          if (className.equals(SmoothLookAndFeel.class.getName())) {
            SmoothLookAndFeel.setCurrentTheme(SmoothThemeManager.
                                                          getTheme(themeIndex));
          } else if (className.equals(MetalLookAndFeel.class.getName())) {
            MetalLookAndFeel.setCurrentTheme(MetalThemeManager.
                                                          getTheme(themeIndex));
          }
        }
        System.setProperty(LookAndFeelManager.SWING_DEFAULT_LAF, className);
      }
    }
    return res;
  }

/*^**************************************************************************^*/
/*- LookAndFeelChangeListener Implementation.                                -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  public void lookAndFeelChanged(LookAndFeel oldLnF, LookAndFeel newLnF) {
    updateTargets();
  }

/*^**************************************************************************^*/
/*- Package-Private Methods.                                                 -*/
/*^**************************************************************************^*/

  void setCanUpdateLnFTargets(boolean b) {
    canUpdateLnFTargets = b;
  }

  void updateTargets() {
    if (canUpdateLnFTargets && !lnfTargets.empty()) {
      Iterator it = lnfTargets.iterator();
      while (it.hasNext()) {
        SwingUtilities.updateComponentTreeUI((Component)it.next());
      }
    }
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-18 17:24:21 Slobodan
   Initial version

*****************************************************************************^*/
