/*
 * @(#)BuiltInResources.java  1.0.0 / 2008-05-15
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.swing.resources;

import java.net.URL;
import java.util.HashMap;
import java.util.Map;
import java.util.MissingResourceException;
import java.util.ResourceBundle;

import javax.swing.ImageIcon;

/**
 * ApplicationUtils resource manager.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public final class BuiltInResources extends GUIResourceManager {

/*^**************************************************************************^*/
/*- Public Static Field.                                                     -*/
/*^**************************************************************************^*/

  /**
   * Singleton instance.
   */
  public static final BuiltInResources INSTANCE;

/*^**************************************************************************^*/
/*- Static Initializer.                                                      -*/
/*^**************************************************************************^*/

  static {
    INSTANCE = new BuiltInResources();
  }

/*^**************************************************************************^*/
/*- Private Fields.                                                          -*/
/*^**************************************************************************^*/

  private String userImages;
  private String userBundles;
  private final Map userIconMap;
  private final Map userBundleMap;

/*^**************************************************************************^*/
/*- Private Constructor.                                                     -*/
/*^**************************************************************************^*/

  private BuiltInResources() {
    super(null, "bundles", "images", null, null, //$NON-NLS-1$ //$NON-NLS-2$
          null);
    userIconMap = new HashMap();
    userBundleMap = new HashMap();
  }

/*^**************************************************************************^*/
/*- Public Methods.                                                          -*/
/*^**************************************************************************^*/

  /**
   * Returns user images path.
   *
   * @return user images path
   */
  public String getUserImagesPath() {
    return userImages;
  }

  /**
   * Sets user images path.
   *
   * @param userImages images path
   */
  public void setUserImagesPath(String userImages) {
    this.userImages = userImages;
  }

  /**
   * Returns user bundles path.
   *
   * @return user bundles path
   */
  public String getUserBundlesPath() {
    return userBundles;
  }

  /**
   * Sets user bundles.
   *
   * @param userBundles user bundles
   */
  public void setUserBundles(String userBundles) {
    this.userBundles = userBundles;
  }

/*^**************************************************************************^*/
/*- Overridden Methods.                                                      -*/
/*^**************************************************************************^*/

  /**
   * Creates ImageIcon from resource repository.
   *
   * @param imageIconName the image icon name
   *
   * @return ImageIcon
   *
   * @throws IllegalArgumentException if image icon name is <tt>null</tt>
   */
  public ImageIcon getImageIcon(String imageIconName) {
    if (imageIconName == null) {
      throw new IllegalArgumentException("Null image icon name"); //$NON-NLS-1$
    }
    if (userImages != null) {
      URL url = classLoader.getResource(concatResource(userImages,
                                                       imageIconName));
      if (url != null) {
        return new ImageIcon(url);
      }
    }
    return super.getImageIcon(imageIconName);
  }

  /**
   * Returns cached ImageIcon from resource repository.
   *
   * @param imageName resource image name
   *
   * @return ImageIcon
   *
   * @throws IllegalArgumentException if image icon name is <tt>null</tt>
   */
  public ImageIcon getCachedImageIcon(String imageName) {
    if (imageName == null) {
      throw new IllegalArgumentException("Null image icon name"); //$NON-NLS-1$
    }
    if (userImages != null) {
      ImageIcon userImageIcon = (ImageIcon)userIconMap.get(imageName);
      if (userImageIcon == null) {
        URL url = classLoader.getResource(concatResource(userImages,
                                                         imageName));
        if (url != null) {
          userImageIcon = new ImageIcon(url);
          userIconMap.put(imageName, userImageIcon);
          return userImageIcon;
        }
      }
    }
    return super.getCachedImageIcon(imageName);
  }

  /**
   * Returns resource bundle specified by the name.
   *
   * @param bundleName the bundle name
   *
   * @return resource bundle specified by the name
   *
   * @throws IllegalArgumentException if bundle name is <tt>null</tt>
   */
  public ResourceBundle getResourceBundle(String bundleName) {
    if (bundleName == null) {
      throw new IllegalArgumentException("Null bundle name"); //$NON-NLS-1$
    }
    if (userBundles != null) {
      ResourceBundle rb = (ResourceBundle)userBundleMap.get(bundleName);
      if (rb == null) {
        try {
          rb = ResourceBundle.getBundle(concatBundle(userBundles, bundleName));
          userBundleMap.put(bundleName, rb);
        } catch (MissingResourceException mre) { // NOPMD
          // empty catch block
        }
      }
    }
    return super.getResourceBundle(bundleName);
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-15 01:22:24 Slobodan
   Initial version

*****************************************************************************^*/
