/*
 * @(#)GUIResourceManager.java  1.0.0 / 2008-05-15
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.swing.resources;

import java.awt.Image;
import java.io.File;
import java.io.InputStream;
import java.net.URL;
import java.util.HashMap;
import java.util.Map;

import javax.swing.ImageIcon;

/**
 * GUI Resource Manager.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public class GUIResourceManager extends ResourceManager {

/*^**************************************************************************^*/
/*- Protected Static Final Fields.                                           -*/
/*^**************************************************************************^*/

  /**
   * Error string <tt>icon not found</tt>.
   */
  protected static final String ERROR_ICON_NOT_FOUND;

  /**
   * Error string <tt>help not found</tt>.
   */
  protected static final String ERROR_HELP_NOT_FOUND;

  /**
   * Error string <tt>about not found</tt>.
   */
  protected static final String ERROR_ABOUT_NOT_FOUND;

  /**
   * Error string <tt>font not found</tt>.
   */
  protected static final String ERROR_FONT_NOT_FOUND;

/*^**************************************************************************^*/
/*- Static Initializer.                                                      -*/
/*^**************************************************************************^*/

  static {
    ERROR_ICON_NOT_FOUND =
                  "Resource Manager cannot find image icon file"; //$NON-NLS-1$
    ERROR_HELP_NOT_FOUND =
                        "Resource Manager cannot find help file"; //$NON-NLS-1$
    ERROR_ABOUT_NOT_FOUND =
                       "Resource Manager cannot find about file"; //$NON-NLS-1$
    ERROR_FONT_NOT_FOUND =
                 "Resource Manager cannot find stream font file"; //$NON-NLS-1$
  }

/*^**************************************************************************^*/
/*- Protected Fields.                                                        -*/
/*^**************************************************************************^*/

  /**
   * Image folder.
   */
  protected final String imageFolder;

  /**
   * Help folder.
   */
  protected final String helpFolder;

  /**
   * About folder.
   */
  protected final String aboutFolder;

  /**
   * Font folder.
   */
  protected final String fontFolder;

  /**
   * Image map.
   */
  protected final Map imageMap;

  /**
   * Icon map.
   */
  protected final Map iconMap;

  /**
   * Help map.
   */
  protected final Map helpMap;

  /**
   * About map.
   */
  protected final Map aboutMap;

/*^**************************************************************************^*/
/*- Protected Constructors.                                                  -*/
/*^**************************************************************************^*/

  /**
   * Constructs new GUIResourceManager object.
   *
   * @param repositoryFolder the resource home folder
   * @param imageFolder the folder that contains images
   */
  protected GUIResourceManager(String repositoryFolder, String imageFolder) {
    this(repositoryFolder, null, imageFolder, null, null, null);
  }

  /**
   * Constructs new GUIResourceManager object.
   *
   * @param repositoryFolder the repository home folder
   * @param imageFolder the folder that contains images
   * @param helpFolder the folder that contains help files
   * @param aboutFolder the folder that contains about files
   */
  protected GUIResourceManager(String repositoryFolder, String imageFolder,
                               String helpFolder, String aboutFolder) {
    this(repositoryFolder, null, imageFolder, helpFolder, aboutFolder, null);
  }

  /**
   * Constructs new GUIResourceManager object.
   *
   * @param repositoryFolder the repository home folder
   * @param bundlesFolder the folder that contains resource budles
   * @param imageFolder the folder that contains images
   * @param helpFolder the folder that contains help files
   * @param aboutFolder the folder that contains about files
   */
  protected GUIResourceManager(String repositoryFolder, String bundlesFolder,
                               String imageFolder, String helpFolder,
                               String aboutFolder) {
    this(repositoryFolder, bundlesFolder, imageFolder, helpFolder, aboutFolder,
         null);
  }

  /**
   * Constructs new GUIResourceManager object.
   *
   * @param repositoryFolder the repository home folder
   * @param bundlesFolder the folder that contains resource bundles
   * @param imageFolder the folder that contains images
   * @param helpFolder the folder that contains help files
   * @param aboutFolder the folder that contains about files
   * @param fontFolder the folder that contains font files
   */
  protected GUIResourceManager(String repositoryFolder, String bundlesFolder,
                               String imageFolder, String helpFolder,
                               String aboutFolder, String fontFolder) {
    super(repositoryFolder, bundlesFolder);
    StringBuffer sb = new StringBuffer(homeFolder);
    int t = sb.length();
    if (imageFolder == null) {
      imageMap = repositoryMap;
      iconMap = repositoryMap;
      this.imageFolder = homeFolder;
    } else {
      sb.append(imageFolder);
      imageMap = new HashMap();
      iconMap = new HashMap();
      this.imageFolder = sb.toString();
      sb.setLength(t);
    }
    if (helpFolder == null) {
      helpMap = repositoryMap;
      this.helpFolder = homeFolder;
    } else {
      sb.append(helpFolder);
      helpMap = new HashMap();
      this.helpFolder = sb.toString();
      sb.setLength(t);
    }
    if (aboutFolder == null) {
      aboutMap = repositoryMap;
      this.aboutFolder = homeFolder;
    } else {
      sb.append(aboutFolder);
      aboutMap = new HashMap();
      this.aboutFolder = sb.toString();
    }
    if (fontFolder == null) {
      this.fontFolder = homeFolder;
    } else {
      sb.setLength(t);
      sb.append(fontFolder);
      this.fontFolder = sb.toString();
    }
  }

/*^**************************************************************************^*/
/*- Public Methods.                                                          -*/
/*^**************************************************************************^*/

  /**
   * Creates ImageIcon from resource repository.
   *
   * @param imageIconName the image icon name
   *
   * @return ImageIcon
   *
   * @throws IllegalArgumentException if image icon name is <tt>null</tt>
   */
  public ImageIcon getImageIcon(String imageIconName) {
    if (imageIconName == null) {
      throw new IllegalArgumentException("Null image icon name"); //$NON-NLS-1$
    }
    URL url = classLoader.getResource(concatResource(imageFolder,
                                                     imageIconName));
    if (url != null) {
      return new ImageIcon(url);
    }
    logError(ERROR_ICON_NOT_FOUND, imageIconName, imageFolder);
    return null;
  }

  /**
   * Returns cached ImageIcon from resource repository.
   *
   * @param imageIconName the image icon name
   *
   * @return ImageIcon
   *
   * @throws IllegalArgumentException if image name is <tt>null</tt>
   */
  public ImageIcon getCachedImageIcon(String imageIconName) {
    if (imageIconName == null) {
      throw new IllegalArgumentException("Null image icon name"); //$NON-NLS-1$
    }
    ImageIcon imageIcon = null;
    Object ro = iconMap.get(imageIconName);
    if (ro instanceof ImageIcon) {
      imageIcon = (ImageIcon)ro;
    } else {
      if (ro == null) {
        URL url = classLoader.getResource(
                                concatResource(imageFolder, imageIconName));
        if (url != null) {
          imageIcon = new ImageIcon(url);
          iconMap.put(imageIconName, imageIcon);
        }
      }
    }
    if (imageIcon == null) {
      logError(ERROR_ICON_NOT_FOUND, imageIconName, imageFolder);
    }
    return imageIcon;
  }

  /**
   * Creates Image from resource repository.
   *
   * @param imageName resource image name
   *
   * @return Image
   *
   * @throws IllegalArgumentException if image name is <tt>null</tt>
   */
  public Image getImage(String imageName) {
    if (imageName == null) {
      throw new IllegalArgumentException("Null image name"); //$NON-NLS-1$
    }
    Image im = null;
    ImageIcon ii = getImageIcon(imageName);
    if (ii != null) {
      im = ii.getImage();
    }
    return im;
  }

  /**
   * Returns cached Image from resource repository.
   *
   * @param imageName resource image name
   *
   * @return Image
   *
   * @throws IllegalArgumentException if image name is <tt>null</tt>
   */
  public Image getCachedImage(String imageName) {
    if (imageName == null) {
      throw new IllegalArgumentException("Null image name"); //$NON-NLS-1$
    }
    Image image = null;
    Object ro = imageMap.get(imageName);
    if (ro instanceof Image) {
      image = (Image)ro;
    } else {
      if (ro == null) {
        ImageIcon ii = getImageIcon(imageName);
        if (ii != null) {
          image = ii.getImage();
          imageMap.put(imageName, image);
        }
      }
    }
    return image;
  }

  /**
   * Creates help URL from resource repository.
   *
   * @param helpFileName the help file name
   *
   * @return SimpleHelp URL
   *
   * @throws IllegalArgumentException if help file name is <tt>null</tt>
   */
  public URL getHelpURL(String helpFileName) {
    if (helpFileName == null) {
      throw new IllegalArgumentException("Null help file name"); //$NON-NLS-1$
    }
    URL helpURL = null;
    Object ro = helpMap.get(helpFileName);
    if (ro instanceof URL) {
      helpURL = (URL)ro;
    } else {
      if (ro == null) {
        helpURL = classLoader.getResource(concatResource(helpFolder,
                                                         helpFileName));
        if (helpURL != null) {
          helpMap.put(helpFileName, helpURL);
        }
      }
    }
    if (helpURL == null) {
      logError(ERROR_HELP_NOT_FOUND, helpFileName, helpFolder);
    }
    return helpURL;
  }

  /**
   * Creates about URL from resource repository.
   *
   * @param aboutFileName the about file name
   *
   * @return about URL
   *
   * @throws IllegalArgumentException if about file name is <tt>null</tt>
   */
  public URL getAboutURL(String aboutFileName) {
    if (aboutFileName == null) {
      throw new IllegalArgumentException("Null aboutFileName"); //$NON-NLS-1$
    }
    URL aboutURL = null;
    Object ro = aboutMap.get(aboutFileName);
    if (ro instanceof URL) {
      aboutURL = (URL)ro;
    } else {
      if (ro == null) {
        aboutURL = classLoader.getResource(
                                 concatResource(aboutFolder, aboutFileName));
        if (aboutURL != null) {
          aboutMap.put(aboutFileName, aboutURL);
        }
      }
    }
    if (aboutURL == null) {
      logError(ERROR_ABOUT_NOT_FOUND, aboutFileName, aboutFolder);
    }
    return aboutURL;
  }

  /**
   * Creates font input stream form resource repository.
   *
   * @param fontFileName the font file name
   *
   * @return font input stream
   *
   * @throws IllegalArgumentException if font file name is <tt>null</tt>
   */
  public InputStream getFontInputStream(String fontFileName) {
    if (fontFileName == null) {
      throw new IllegalArgumentException("Null fileName"); //$NON-NLS-1$
    }
    return classLoader.getResourceAsStream(concatResource(fontFolder,
                                                          fontFileName));
  }

  /**
   * Creates font file form resource repository.
   *
   * @param fileName the font file name
   *
   * @return font resource file
   *
   * @throws IllegalArgumentException if font file name is <tt>null</tt>
   */
  public File getFontFile(String fileName) {
    if (fileName == null) {
      throw new IllegalArgumentException("Null fileName"); //$NON-NLS-1$
    }
    URL url = classLoader.getResource(concatResource(fontFolder, fileName));
    if (url != null) {
      // FIX: Check url decoding problem (like space %20)
      return new File(url.getFile());
    }
    logError(ERROR_FONT_NOT_FOUND, fileName, fontFolder);
    return null;
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-15 00:47:00 Slobodan
   Initial version

*****************************************************************************^*/
