/*
 * @(#)ResourceManager.java  1.0.0 / 2008-05-14
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.swing.resources;

import java.io.File;
import java.io.InputStream;
import java.net.URL;
import java.util.HashMap;
import java.util.Map;
import java.util.MissingResourceException;
import java.util.ResourceBundle;

import net.wrka.util.CommonConstants;

/**
 * Resource Manager.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public class ResourceManager {

/*^**************************************************************************^*/
/*- Protected Static Final Fields.                                           -*/
/*^**************************************************************************^*/

  /**
   * Error string <tt>resource not found</tt>.
   */
  protected static final String ERROR_RESOURCE_NOT_FOUND;

  /**
   * Error string <tt>file not found</tt>.
   */
  protected static final String ERROR_FILE_NOT_FOUND;

  /**
   * Error string <tt>resource bundle not found</tt>.
   */
  protected static final String ERROR_RESOURCE_BUNDLE_NOT_FOUND;

/*^**************************************************************************^*/
/*- Static Initialization.                                                   -*/
/*^**************************************************************************^*/

  static {
    ERROR_RESOURCE_NOT_FOUND =
                      "Resource Manager could not find resource"; //$NON-NLS-1$
    ERROR_FILE_NOT_FOUND =
                          "Resource Manager could not find file"; //$NON-NLS-1$
    ERROR_RESOURCE_BUNDLE_NOT_FOUND =
               "Resource Manager could not find resource bundle"; //$NON-NLS-1$
  }

/*^**************************************************************************^*/
/*- Protected Fields.                                                        -*/
/*^**************************************************************************^*/

  /**
   * Class loader field.
   */
  protected final ClassLoader classLoader;

  /**
   * Home folder of the resource manager.
   */
  protected final String homeFolder;

  /**
   * Repository folder.
   */
  protected final String repositoryFolder;

  /**
   * Bundles folder.
   */
  protected final String bundlesFolder;

  /**
   * Repository map.
   */
  protected final Map repositoryMap;

  /**
   * Bundles map.
   */
  protected final Map bundlesMap;

/*^**************************************************************************^*/
/*- Protected Constructors.                                                  -*/
/*^**************************************************************************^*/

  /**
   * Constructs new ResourceManager object.
   *
   * @param repositoryFolder the repository folder
   */
  protected ResourceManager(String repositoryFolder) {
    this(repositoryFolder, null);
  }

  /**
   * Constructs new ResourceManager object.
   *
   * @param repositoryFolder the repository folder
   * @param bundlesFolder the folder that contains resource bundles
   */
  protected ResourceManager(String repositoryFolder, String bundlesFolder) {
    Class c = getClass();
    classLoader = c.getClassLoader();
    StringBuffer sb = new StringBuffer(c.getName());
    int t = sb.lastIndexOf(CommonConstants.STRING_DOT);
    if (t != -1) {
      sb.delete(t + 1, sb.length());
      t = sb.length() - 1;
      for (int i = t; i >= 0; i--) {
        if (sb.charAt(i) == '.') {
          sb.setCharAt(i, '/');
        }
      }
    }
    if (sb.charAt(t) !=  '/') {
      sb.append(CommonConstants.STRING_SLASH);
    }
    t = sb.length();
    homeFolder = sb.toString();
    repositoryMap = new HashMap();
    if (repositoryFolder == null) {
      this.repositoryFolder = homeFolder;
    } else {
      sb.append(repositoryFolder);
      this.repositoryFolder = sb.toString();
    }
    if (bundlesFolder == null) {
      bundlesMap = repositoryMap;
      this.bundlesFolder = homeFolder.replaceAll(CommonConstants.STRING_SLASH,
                                                 CommonConstants.STRING_DOT);
    } else {
      sb.setLength(t);
      sb.append(bundlesFolder);
      bundlesMap = new HashMap();
      String sbs = sb.toString();
      this.bundlesFolder = sbs.replaceAll(CommonConstants.STRING_SLASH,
                                          CommonConstants.STRING_DOT);
    }
  }

/*^**************************************************************************^*/
/*- Public Methods.                                                          -*/
/*^**************************************************************************^*/

  /**
   * Creates input stream for reading the specified resource.
   *
   * @param fileName the resource file name
   *
   * @return input stream for reading the specified resource
   *
   * @throws IllegalArgumentException if the file name is <tt>null</tt>
   */
  public InputStream getInputStream(String fileName) {
    if (fileName == null) {
      throw new IllegalArgumentException("Null fileName"); //$NON-NLS-1$
    }
    return classLoader.getResourceAsStream(concatResource(repositoryFolder,
                                                          fileName));
  }

  /**
   * Returns resource form resource repository.
   *
   * @param name the resource name
   *
   * @return the resource with the given name
   *
   * @throws IllegalArgumentException if the name is <tt>null</tt>
   */
  public URL getResource(String name) {
    if (name == null) {
      throw new IllegalArgumentException("Null resource name"); //$NON-NLS-1$
    }
    URL url = null;
    Object ro = repositoryMap.get(name);
    if (ro instanceof URL) {
      url = (URL)ro;
    } else {
      if (ro == null) {
        url = classLoader.getResource(concatResource(repositoryFolder, name));
        if (url != null) {
          repositoryMap.put(name, url);
        }
      }
    }
    if (url == null) {
      logError(ERROR_RESOURCE_NOT_FOUND, name, null);
    }
    return url;
  }

  /**
   * Creates file form resource repository.
   *
   * @param fileName the file name
   *
   * @return resource file
   *
   * @throws IllegalArgumentException if the file name is <tt>null</tt>
   */
  public File getFile(String fileName) {
    if (fileName == null) {
      throw new IllegalArgumentException("Null fileName"); //$NON-NLS-1$
    }
    File file = null;
    Object ro = repositoryMap.get(fileName);
    if (ro instanceof File) {
      file = (File)ro;
    } else {
      if (ro == null) {
        URL url = classLoader.getResource(concatResource(repositoryFolder,
                                                         fileName));
        if (url != null) {
          // FIX: Check url decoding problem (like space %20)
          file = new File(url.getFile());
          repositoryMap.put(fileName, file);
        }
      }
    }
    if (file == null) {
      logError(ERROR_FILE_NOT_FOUND, fileName, null);
    }
    return file;
  }

  /**
   * Returns resource bundle specified by the name.
   *
   * @param bundleName the bundle name
   *
   * @return resource bundle specified by the name
   *
   * @throws IllegalArgumentException if the bundle name is <tt>null</tt>
   */
  public ResourceBundle getResourceBundle(String bundleName) {
    if (bundleName == null) {
      throw new IllegalArgumentException("Null bundle name"); //$NON-NLS-1$
    }
    ResourceBundle rb = null;
    Object ro = bundlesMap.get(bundleName);
    if (ro instanceof ResourceBundle) {
      rb = (ResourceBundle)ro;
    } else {
      if (ro == null) {
        try {
          rb = ResourceBundle.getBundle(concatBundle(bundlesFolder,
                                                     bundleName));
          bundlesMap.put(bundleName, rb);
        } catch (MissingResourceException mre) { // NOPMD
          // empty catch block
        }
      }
    }
    if (rb == null) {
      logError(ERROR_RESOURCE_BUNDLE_NOT_FOUND, bundleName, bundlesFolder);
    }
    return rb;
  }

  /**
   * Checks if the resource manager contains specified bundle.
   *
   * @param bundleName the bundle name
   *
   * @return <code>true</code> if contains
   */
  public boolean containsResourceBundle(String bundleName) {
    if (bundleName != null) {
      if (bundlesMap.containsKey(bundleName)) {
        return true;
      }
      try {
        ResourceBundle rb = ResourceBundle.getBundle(
                              concatBundle(bundlesFolder, bundleName));
        bundlesMap.put(bundleName, rb);
        return true;
      } catch (MissingResourceException mre) { // NOPMD
        // empty catch block
      }
    }
    return false;
  }

  /**
   * Registers new resource bundle.
   *
   * @param bundleName the resource bundle name
   * @param resourceBundle the resource bundle
   */
  public void registerResourceBundle(String bundleName,
                                     ResourceBundle resourceBundle) {
    if ((bundleName != null) && (bundleName.length() > 0) &&
        (resourceBundle != null)) {
      bundlesMap.put(bundleName, resourceBundle);
    }
  }

  /**
   * Unregisters resource bundle.
   *
   * @param bundleName the resource bundle name
   */
  public void unregisterResourceBundle(String bundleName) {
    if ((bundleName != null) && bundlesMap.containsKey(bundleName)) {
      bundlesMap.remove(bundleName);
    }
  }

/*^**************************************************************************^*/
/*- Protected Methods.                                                       -*/
/*^**************************************************************************^*/

  /**
   * Concatenates the specified resource path and resource name.
   *
   * @param resourcePath resource path
   * @param resourceName resource name
   *
   * @return resource path + "/" + resource name
   */
  protected String concatResource(String resourcePath, String resourceName) {
    if (resourcePath == null) {
      return resourceName;
    }
    return resourcePath + CommonConstants.STRING_SLASH + resourceName;
  }

  /**
   * Concatenates the specified resource path and resource name.
   *
   * @param bundlesPath bundles path
   * @param bundleName bundle name
   *
   * @return resource path + "/" + resource name
   */
  protected String concatBundle(String bundlesPath, String bundleName) {
    if (bundlesPath == null) {
      return bundleName;
    }
    return bundlesPath + CommonConstants.STRING_DOT + bundleName;
  }

  /**
   * Logs Error.
   *
   * @param errorMessage error message
   * @param resourceName name of required resource
   * @param resourcePath resource path
   */
  protected void logError(String errorMessage, String resourceName,
                          String resourcePath) {
    // CHANGE: replace with logger
    System.out.print(errorMessage); // NOPMD
    System.out.print(": \""); //$NON-NLS-1$ NOPMD
    System.out.print(resourceName); // NOPMD
    System.out.println('"'); // NOPMD
    if (resourcePath != null) {
      System.out.println("in: \""); //$NON-NLS-1$NOPMD
      System.out.print(resourcePath); // NOPMD
      System.out.println('"'); // NOPMD
    }
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-14 20:48:21 Slobodan
   Initial version

*****************************************************************************^*/
