/*
 * @(#)Splash.java  1.0.0 / 2008-05-15
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.swing.splash;

import javax.swing.ImageIcon;

import net.wrka.util.CommonConstants;
import net.wrka.util.SyncMonitor;
import net.wrka.util.queue.SyncBlockingQueue;

/**
 * Simple Splash.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public final class Splash {

/*^**************************************************************************^*/
/*- Public Static Fields.                                                    -*/
/*^**************************************************************************^*/

  /**
   * Splash resource string.
   */
  public static final String RESOURCES = "SplashResources"; //$NON-NLS-1$

  /**
   * Splash resource <tt>load message</tt>.
   */
  public static final String RK_MESSAGE_START =
                                            "Splash.messageLoad"; //$NON-NLS-1$

  /**
   * Splash resource <tt>initializing message</tt>.
   */
  public static final String RK_MESSAGE_INITIALIZING =
                                    "Splash.messageInitializing"; //$NON-NLS-1$

  /**
   * Splash resource <tt>finished message</tt>.
   */
  public static final String RK_MESSAGE_FINISH =
                                        "Splash.messageFinished"; //$NON-NLS-1$

  /**
   * Splash slide time.
   */
  public static final int SLIDE_TIME = 850;

/*^**************************************************************************^*/
/*- Private Static Fields.                                                   -*/
/*^**************************************************************************^*/

  static SplashWindow splash;
  static long duration;
  static Thread user;
  static SplashThread splashThread;
  static boolean usersleep;
  static SyncBlockingQueue mqueue;
  static SyncMonitor closeSync;

/*^**************************************************************************^*/
/*- Private Constructor.                                                     -*/
/*^**************************************************************************^*/

  /**
   * Class has no instances.
   */
  private Splash() {
    // empty constructor body
  }

/*^**************************************************************************^*/
/*- Public Static Methods.                                                   -*/
/*^**************************************************************************^*/

  /**
   * Creates and shows splash screen for the specified image icon.
   *
   * @param i the splash image icon
   * @param steps the number of splash updates
   * @param duration the minimum splash duration
   * @param startmessage the start message
   */
  public static void show(ImageIcon i, int steps, long duration,
                          String startmessage) {
    if ((i != null) && (splash == null)) {
      splash = new SplashWindow(i, steps, startmessage);
      Splash.duration = duration;
      user = null;
      usersleep = false;
      mqueue = new SyncBlockingQueue();
      splashThread = new SplashThread();
      closeSync = new SyncMonitor();
      splashThread.start();
    }
  }

  /**
   * Creates and shows splash screen for the specified image icon.
   *
   * @param i the splash image icon
   * @param steps the number of splash updates
   * @param startmessage the start message
   */
  public static void show(ImageIcon i, int steps, String startmessage) {
    show(i, steps, 1700, startmessage);
  }

  /**
   * Creates and shows splash screen for the specified image icon.
   *
   * @param i the splash image icon
   * @param startmessage the start message
   */
  public static void show(ImageIcon i,  String startmessage)  {
    show(i, 0, 1700, startmessage);
  }

  /**
   * Updates splash.
   *
   * @param msg the update message
   */
  public static void update(String msg) {
    if (splash != null) {
      mqueue.put(msg);
    }
  }

  /**
   * Updates splash.
   */
  public static void update() {
    if (splash != null) {
      update(CommonConstants.STRING_EMPTY);
    }
  }

  /**
   * Closes splash.
   *
   * @param closemessage the close message
   */
  public static void close(String closemessage) {
    if (splash != null) { // NOPMD - not thread safe
      if (closemessage != null) {
        update(closemessage);
      }
      update(null);
      closeSync.waitFor();
      try {
        user = Thread.currentThread();
        usersleep = true;
        if (duration - SLIDE_TIME > 0) {
          Thread.sleep(duration - SLIDE_TIME);
        }
      } catch (InterruptedException ie) { // NOPMD
        // empty catch block
      }
      splashThread.close();
    }
  }

/*^**************************************************************************^*/
/*- Package-Private Method.                                                  -*/
/*^**************************************************************************^*/

  static void hide() {
    if (splash != null) { // NOPMD - not thread safe
      duration = SLIDE_TIME;
      if (usersleep) {
        user.interrupt();
      }
    }
  }

/*^**************************************************************************^*/
/*- Private Inner Class.                                                     -*/
/*^**************************************************************************^*/

  /**
   * Splash thread.
   */
  static final class SplashThread extends Thread {

    private boolean close;

    SplashThread() {
      super("SplashThread"); //$NON-NLS-1$
    }

    /**
     * {@inheritDoc}
     */
    public void run() {
      while (true) {
        Object mqi = mqueue.get();
        if (close) {
          splash.close();
          // release memory
          splash = null;
          splashThread = null;
          user = null;
          mqueue = null;
          closeSync = null;
          return;
        }
        if (mqi == null) {
          closeSync.signal();
        } else {
          try {
            Thread.sleep(SLIDE_TIME);
          } catch (InterruptedException ie) {
            return;
          }
          splash.updateSplash((String)mqi);
        }
      }
    }

    /**
     * Closes the splash.
     */
    public void close() {
      close = true;
      interrupt();
    }

  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-15 20:42:16 Slobodan
   Initial version

*****************************************************************************^*/
