/*
 * @(#)SplashImage.java  1.0.0 / 2008-05-15
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.swing.splash;

import java.awt.Color;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.ImageIcon;
import javax.swing.Timer;

import net.wrka.swing.components.JWxImageIconComponent;
import net.wrka.swing.graphics.GraphicUtils;

/**
 * Splash Image.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
final class SplashImage extends JWxImageIconComponent
                        implements ActionListener {

/*^**************************************************************************^*/
/*- Private Static Final Fields.                                             -*/
/*^**************************************************************************^*/

  private static final long serialVersionUID = -871787574378132610L;

/*^**************************************************************************^*/
/*- Private Static Final Fields.                                             -*/
/*^**************************************************************************^*/

  private static final int PROGRESS_WIDTH = 100;
  private static final int PROGRESS_HEIGHT = 10;
  private static final int PROGRESS_MESSAGE_OFFSET = 13;

/*^**************************************************************************^*/
/*- Private Final Fields.                                                    -*/
/*^**************************************************************************^*/

  // Non static because GC.

  private final Color PROGRESS_COLOR = new Color(85, 85, 234);
  private final Color PROGRESS_RECT_COLOR = Color.BLACK;
  private final Color PROGRESS_RECT_INNER_COLOR = Color.lightGray;
  private final Color MESSAGE_COLOR = Color.BLACK;
  private final Font MESSAGE_FONT = new Font("Dialog", Font.PLAIN, //$NON-NLS-1$
                                             12);

/*^**************************************************************************^*/
/*- Private Fields.                                                          -*/
/*^**************************************************************************^*/

  private final Rectangle messagebounds;
  private final Rectangle progressbounds;
  private String message;
  private final FontMetrics fdm;
  private int progresslenght;
  private int currentstepnumber;
  private int progressstep;
  private Timer timer;
  private boolean continuous;

/*^**************************************************************************^*/
/*- Public Constructor.                                                      -*/
/*^**************************************************************************^*/

  /**
   * Constructs new SplashImage object.
   *
   * @param imageIcon the image icon
   * @param steps the number of splash steps
   * @param startmessage the start message
   */
  public SplashImage(ImageIcon imageIcon, int steps, String startmessage) {
    super(imageIcon);
    continuous = true;
    message = startmessage;
    setFont(MESSAGE_FONT);
    fdm = getFontMetrics(MESSAGE_FONT);
    messagebounds = new Rectangle(getPreferredSize());
    progressbounds = (Rectangle)messagebounds.clone();
    messagebounds.x = 10;
    messagebounds.y = messagebounds.height - fdm.getHeight() - 7;
    messagebounds.height = fdm.getHeight();
    messagebounds.width -= 20;
    progressbounds.x = ((progressbounds.width - PROGRESS_WIDTH) >> 1) - 2;
    progressbounds.y = messagebounds.y - PROGRESS_HEIGHT -
                         PROGRESS_MESSAGE_OFFSET;
    progressbounds.height = PROGRESS_HEIGHT;
    progressbounds.width = PROGRESS_WIDTH;
    if (steps == 0) {
      continuous = false;
      progressstep = 1;
      timer = new Timer(22, this);
      timer.start();
    } else {
      progressstep = PROGRESS_WIDTH / (steps + 1);
    }
  }

/*^**************************************************************************^*/
/*- Public Methods.                                                          -*/
/*^**************************************************************************^*/

  /**
   * Updates image.
   */
  public void updateSplash() {
    if (continuous) {
      currentstepnumber++;
    } else {
      currentstepnumber = (currentstepnumber + progressstep) %
                          (PROGRESS_WIDTH + 22);
    }
    progresslenght = Math.min(currentstepnumber * progressstep,
                              progressbounds.width - 3);
    repaint(messagebounds.x, messagebounds.y, messagebounds.width,
            messagebounds.height);
    repaint(progressbounds.x + 2, progressbounds.y + 2,
            progressbounds.width - 3, progressbounds.height - 3);
  }

  /**
   * Updates image.
   *
   * @param message the message
   */
  public void updateSplash(String message) {
    this.message = message;
    updateSplash();
  }

  /**
   * Disposes this image.
   */
  public void dispose() {
    if (timer != null) {
      timer.stop();
      timer = null;
    }
  }

  /**
   * ActionListener implementation.
   *
   * @param e the ActionEvent
   */
  public void actionPerformed(ActionEvent e) {
    updateSplash();
  }

/*^**************************************************************************^*/
/*- Overridden Method.                                                       -*/
/*^**************************************************************************^*/

  /**
   * Paints the component.
   *
   * @param g the graphics
   */
  public void paintComponent(Graphics g) {
    super.paintComponent(g);
    // Paint message
    Graphics2D g2 = (Graphics2D)g;
    g2.setColor(MESSAGE_COLOR);
    g2.drawString(message, messagebounds.x, messagebounds.y + fdm.getAscent());
    // Paint progress
    g2.setColor(PROGRESS_RECT_COLOR);
    GraphicUtils.draw1PixRoundRect(g, progressbounds);
    g2.setColor(PROGRESS_RECT_INNER_COLOR);
    g2.drawRect(progressbounds.x + 1, progressbounds.y + 1,
                progressbounds.width - 2, progressbounds.height - 2);
    g2.setColor(PROGRESS_COLOR);
    if (continuous) {
      g2.fillRect(progressbounds.x + 2, progressbounds.y + 2,
                  progresslenght, progressbounds.height - 3);
    } else {
      g.setClip(progressbounds.x + 2, progressbounds.y + 2,
                progresslenght, progressbounds.height - 3);
      g2.fillRect(progressbounds.x - 20 + currentstepnumber,
                  progressbounds.y + 2, 6, progressbounds.height - 3);
      g2.fillRect(progressbounds.x - 20 + currentstepnumber + 8,
                  progressbounds.y + 2, 6, progressbounds.height - 3);
      g2.fillRect(progressbounds.x - 20 + currentstepnumber + 16,
                  progressbounds.y + 2, 6, progressbounds.height - 3);
    }
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-15 20:19:06 Slobodan
   Initial version

*****************************************************************************^*/
