/*
 * @(#)ToolTipIcon.java  1.0.0 / 2008-05-15
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.swing.tooltips;

import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Shape;
import java.awt.geom.AffineTransform;
import java.awt.geom.GeneralPath;

import net.wrka.swing.graphics.GraphicUtils;

/**
 * Tooltip Arrow Icon.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
final class ToolTipIcon extends Component {

/*^**************************************************************************^*/
/*- Private Static  Final Fields.                                            -*/
/*^**************************************************************************^*/

  private static final long serialVersionUID = -7186206828455313242L;

/*^**************************************************************************^*/
/*- Private Static  Final Fields.                                            -*/
/*^**************************************************************************^*/

  private static final int TTI_BOTTOM_LEFT = 1;
  private static final int TTI_TOP_RIGHT = 2;
  private static final int TTI_TOP_LEFT = 3;
  private static final int ICON_WIDTH = 11;
  private static final int ICON_HEIGHT = 18;

/*^**************************************************************************^*/
/*- Private Fields.                                                          -*/
/*^**************************************************************************^*/

  private final Shape topLeftArrow;
  private final Shape topRightArrow;
  private final Shape bottomLeftArrow;
  private final Shape bottomRightArrow;
  private final Dimension prefSize;
  private Color lastFG;
  private Color foregroundLight;
  private int position;

/*^**************************************************************************^*/
/*- Package-Private Constructor.                                             -*/
/*^**************************************************************************^*/

  ToolTipIcon(boolean leftArrow) {
    super();
    prefSize = new Dimension(ICON_WIDTH, ICON_HEIGHT);
    GeneralPath arrow = new GeneralPath();
    arrow.moveTo(8f, 6f);
    arrow.lineTo(5f, 3f);
    arrow.lineTo(5f, 5f);
    arrow.lineTo(1f, 5f);
    arrow.lineTo(1f, 0f);
    arrow.lineTo(0f, 0f);
    arrow.lineTo(0f, 7f);
    arrow.lineTo(1f, 8f);
    arrow.lineTo(5f, 8f);
    arrow.lineTo(5f, 10f);
    arrow.lineTo(8f, 7f);
    arrow.lineTo(8f, 6f);
    setHorizontalArrow(leftArrow);
    bottomRightArrow = arrow;
    AffineTransform vflip = new AffineTransform(1f, 0f, 0f, -1f, 0f, 0f);
    AffineTransform hflip = new AffineTransform(-1f, 0f, 0f, 1f, 0f, 0f);
    bottomLeftArrow = hflip.createTransformedShape(arrow);
    topRightArrow = vflip.createTransformedShape(arrow);
    topLeftArrow = hflip.createTransformedShape(topRightArrow);
  }

/*^**************************************************************************^*/
/*- Public Methods.                                                          -*/
/*^**************************************************************************^*/

  /**
   * Sets left arrow.
   *
   * @param left <tt>true</tt> if the arrow is left
   */
  public void setHorizontalArrow(boolean left) {
    if (left) {
      position = (position | 0x00000001);
    } else {
      position = (position & 0xFFFFFFFE);
    }
  }

  /**
   * Sets top arrow.
   *
   * @param top <tt>true</tt> if the arrow is top
   */
  public void setTopArrow(boolean top) {
    if (top) {
      position = (position | 0x00000002);
    } else {
      position = (position & 0xFFFFFFFD);
    }
  }

/*^**************************************************************************^*/
/*- Overridden Methods.                                                      -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  public void paint(Graphics g) {
    if (g instanceof Graphics2D) {
      Graphics2D g2 = (Graphics2D)g;
      Color foreground;
      Component p = getParent();
      if (p == null) {
        foreground = Color.BLACK;
      } else {
        foreground = p.getForeground();
      }
      if (!foreground.equals(lastFG)) {
        foregroundLight = GraphicUtils.brighterColor(foreground, 0.25f);
        lastFG = foreground;
      }
      Shape s;
      int xofs;
      int yofs;
      switch (position) {
        case TTI_BOTTOM_LEFT:
          s = bottomLeftArrow;
          xofs = 1;
          yofs = (getHeight() >> 1) - 8;
          g2.translate(8, 0);
          break;
        case TTI_TOP_RIGHT:
          s = topRightArrow;
          xofs = 3;
          yofs = (getHeight() >> 1) - 2;
          g2.translate(0, 8);
          break;
        case TTI_TOP_LEFT:
          s = topLeftArrow;
          xofs = 1;
          yofs = (getHeight() >> 1) - 2;
          g2.translate(8, 8);
          break;
        default:
          s = bottomRightArrow;
          xofs = 3;
          yofs = (getHeight() >> 1) - 8;
          break;
      }
      g2.translate(xofs, yofs);
      g2.setColor(foregroundLight);
      g2.fill(s);
      g2.translate(-1, 0);
      g2.setColor(foreground);
      g2.draw(s);
    }
  }

  /**
   * {@inheritDoc}
   */
  public Dimension getPreferredSize() {
    return new Dimension(prefSize);
  }

  /**
   * {@inheritDoc}
   */
  public Dimension getMinimumSize() {
    return getPreferredSize();
  }

  /**
   * {@inheritDoc}
   */
  public Dimension getMaximumSize()   {
    return getPreferredSize();
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-15 19:05:03 Slobodan
   Initial version

*****************************************************************************^*/
