/*
 * @(#)ToolTipManager.java  1.0.0 / 2008-05-15
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.swing.tooltips;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.Window;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;

import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JToolTip;
import javax.swing.Popup;
import javax.swing.PopupFactory;
import javax.swing.Timer;
import javax.swing.border.Border;
import javax.swing.border.EmptyBorder;

import net.wrka.awt.AwtTools;

/**
 * Tooltip manager. Shows tool tips outside the window.
 * <p>
 *   <tt>NOTE: As of Java 1.5+, this class is not so use full.</tt>
 * </p>
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public final class ToolTipManager implements ActionListener, MouseListener,
                                             MouseMotionListener {

/*^**************************************************************************^*/
/*- Private Static Field.                                                    -*/
/*^**************************************************************************^*/

  private static final ToolTipManager INSTANCE;

/*^**************************************************************************^*/
/*- Static Initialization.                                                   -*/
/*^**************************************************************************^*/

  static {
    INSTANCE = new ToolTipManager();
  }

/*^**************************************************************************^*/
/*- Private Fields.                                                          -*/
/*^**************************************************************************^*/

  private Popup popup;
  private JToolTip tip;
  private final ToolTipIcon tipIconLeft;
  private final ToolTipIcon tipIconRight;
  private final JPanel popupContent;
  private final int desktopRight;
  private final int desktopBottom;
  private final EmptyBorder tipEmptyBorder;
  private final PopupFactory popupFactory;
  private JComponent insideComponent;
  private MouseEvent mouseEvent;
  private String toolTipText;
  private boolean enabled;
  private boolean showToolTipIcon;
  private final Timer enterTimer;
  private final Timer insideTimer;

/*^**************************************************************************^*/
/*- Private Constructor.                                                     -*/
/*^**************************************************************************^*/

  private ToolTipManager() {
    popupFactory = PopupFactory.getSharedInstance();
    insideComponent = null;
    mouseEvent = null;
    tipIconLeft = new ToolTipIcon(false);
    tipIconRight = new ToolTipIcon(true);
    tipIconRight.setVisible(false);
    tipIconLeft.setVisible(false);
    popupContent = new JPanel(new BorderLayout());
    tipEmptyBorder = new EmptyBorder(2, 1, 1, 1);
    popupContent.add(tipIconLeft, BorderLayout.WEST);
    popupContent.add(tipIconRight, BorderLayout.EAST);
    enterTimer = new Timer(750, this);
    enterTimer.setRepeats(false);
    insideTimer = new Timer(4000, this);
    insideTimer.setRepeats(false);
    Rectangle desktopRect = AwtTools.getMaximumWindowBounds();
    desktopRight = desktopRect.x + desktopRect.width;
    desktopBottom = desktopRect.y + desktopRect.height;
  }

/*^**************************************************************************^*/
/*- Public Static Methods.                                                   -*/
/*^**************************************************************************^*/

  /**
   * Returns <tt>true</tt> if the manager is enabled.
   *
   * @return <tt>true</tt> if the manager is enabled
   */
  public static boolean isEnabled() {
    return INSTANCE.enabled;
  }

  /**
   * Enables/disables the manager.
   *
   * @param b if <tt>true</tt> the manager is enabled.
   */
  public static void setEnabled(boolean b) {
    INSTANCE.enabled = b;
    javax.swing.ToolTipManager.sharedInstance().setEnabled(!b);
  }

  /**
   * Returns initial delay.
   *
   * @return initial delay
   */
  public static int getInitialDelay() {
    return INSTANCE.enterTimer.getInitialDelay();
  }

  /**
   * Sets initial delay.
   *
   * @param milliseconds the initial delay
   */
  public static void setInitialDelay(int milliseconds) {
    INSTANCE.enterTimer.setInitialDelay(milliseconds);
  }

  /**
   * Returns dismiss delay.
   *
   * @return dismiss delay
   */
  public static int getDismissDelay() {
    return INSTANCE.insideTimer.getInitialDelay();
  }

  /**
   * Sets dismiss delay.
   *
   * @param milliseconds the dismiss delay
   */
  public static void setDismissDelay(int milliseconds) {
    INSTANCE.insideTimer.setInitialDelay(milliseconds);
  }

  /**
   * Sets whether or not the tooltip icon should be displayed.
   *
   * @param b if <tt>true</tt> the tooltip icon will be displayed
   */
  public static void setShowToolTipIcon(boolean b) {
    INSTANCE.showToolTipIcon = b;
  }

  /**
   * Registers the specified component.
   *
   * @param component the component
   */
  public static void registerComponent(JComponent component) {
    if (component != null) {
      INSTANCE.registerComp(component);
    }
  }

  /**
   * Unregisters the specified component.
   *
   * @param component the component
   */
  public static void unregisterComponent(JComponent component) {
    if (component != null) {
      INSTANCE.unregisterComp(component);
    }
  }

  /**
   * Registers the specified component and it's all children.
   *
   * @param component the component
   */
  public static void registerContainer(JComponent component) {
    if (component != null) {
      INSTANCE.registerCont(component);
    }
  }

  /**
   * Unregisters the specified component and it's all children.
   *
   * @param component the component
   */
  public static void unregisterContainer(JComponent component) {
    if (component != null) {
      INSTANCE.unregisterCont(component);
    }
  }

  /**
   * Registers the specified frame and it's all children.
   *
   * @param frame the frame
   */
  public static void registerFrame(JFrame frame) {
    if (frame != null) {
      Container cp = frame.getContentPane();
      if (cp instanceof JComponent) {
        INSTANCE.registerCont((JComponent)cp);
      }
    }
  }

  /**
   * Unregisters the specified frame and it's all children.
   *
   * @param frame the frame
   */
  public static void unregisterFrame(JFrame frame) {
    if (frame != null) {
      Container cp = frame.getContentPane();
      if (cp instanceof JComponent) {
        INSTANCE.unregisterCont((JComponent)cp);
      }
    }
  }

  /**
   * Registers the specified dialog and it's all children.
   *
   * @param dialog the dialog
   */
  public static void registerDialog(JDialog dialog) {
    if (dialog != null) {
      Container cp = dialog.getContentPane();
      if (cp instanceof JComponent) {
        INSTANCE.registerCont((JComponent)cp);
      }
    }
  }

  /**
   * Unregisters the specified dialog and it's all children.
   *
   * @param dialog the dialog
   */
  public static void unregisterDialog(JDialog dialog) {
    if (dialog != null) {
      Container cp = dialog.getContentPane();
      if (cp instanceof JComponent) {
        INSTANCE.unregisterCont((JComponent)cp);
      }
    }
  }

/*^**************************************************************************^*/
/*- Package-Private Methods.                                                 -*/
/*^**************************************************************************^*/

  void registerComp(JComponent component) {
    component.removeMouseListener(this);
    component.removeMouseMotionListener(this);
    component.addMouseListener(this);
    component.addMouseMotionListener(this);
  }

  void unregisterComp(JComponent component) {
    component.removeMouseListener(this);
    component.removeMouseMotionListener(this);
  }

  void registerCont(JComponent component) {
    if (enabled) {
      registerComp(component);
      Component[] children = component.getComponents();
      for (int i = children.length - 1; i >= 0; i--) {
        if (children[i] instanceof JComponent) {
          registerCont((JComponent)children[i]);
        }
      }
    }
  }

  void unregisterCont(JComponent component) {
    if (enabled) {
      unregisterComp(component);
      Component[] children = component.getComponents();
      for (int i = children.length - 1; i >= 0; i--) {
        if (children[i] instanceof JComponent) {
          unregisterCont((JComponent)children[i]);
        }
      }
    }
  }

  void showTipWindow() { // NOPMD
    if (enabled) {
      hideTipWindow();
      for (Container p = insideComponent.getParent(); p != null;
           p = p.getParent()) {
        if (p instanceof JPopupMenu) {
          break;
        } else if (p instanceof Window) {
          if (((Window)p).isFocused()) {
            break;
          }
          return;
        }
      }
      String ts = insideComponent.getToolTipText(mouseEvent);
      if ((ts != null) && (ts.length() > 0)) {
        tip = insideComponent.createToolTip();
        tip.setTipText(ts);
        if (showToolTipIcon) {
          Border tipBorder = tip.getBorder();
          tip.setBorder(tipEmptyBorder);
          popupContent.setBorder(tipBorder);
          popupContent.setBackground(tip.getBackground());
          popupContent.setForeground(tip.getForeground());
        }
        popupContent.add(tip);
        if (showToolTipIcon) {
          tipIconLeft.setVisible(true);
          tipIconRight.setVisible(false);
        }
        Point cloc = insideComponent.getLocationOnScreen();
        Point mloc = insideComponent.getToolTipLocation(mouseEvent);
        if (mloc == null) {
          mloc = mouseEvent.getPoint();
        }
        int x = cloc.x + mloc.x;
        int y = cloc.y + mloc.y;
        int pcw = popupContent.getWidth();
        int pch = popupContent.getHeight();
        Dimension ps = popupContent.getPreferredSize();
        if (pcw != ps.width) {
          pcw = ps.width;
          pch = ps.height;
        }
        x = x + 4;
        if ((x + pcw) > (desktopRight)) {
          x = x - pcw - 7;
          if (showToolTipIcon) {
            tipIconRight.setVisible(true);
            tipIconLeft.setVisible(false);
          }
        }
        y = y + 23;
        if ((y + pch) > (desktopBottom)) {
          y = y - pch - 33;
          tipIconLeft.setTopArrow(true);
          tipIconRight.setTopArrow(true);
        } else {
          tipIconLeft.setTopArrow(false);
          tipIconRight.setTopArrow(false);
        }
        popup = popupFactory.getPopup(insideComponent, popupContent, x, y);
        popup.show();
        insideTimer.start();
      }
    }
  }

  void hideTipWindow() {
    if (enterTimer.isRunning()) {
      enterTimer.stop();
    }
    if (insideTimer.isRunning()) {
      insideTimer.stop();
    }
    if (popup != null) {
      popup.hide();
      popupContent.remove(tip);
      popup = null;
      tip = null;
    }
  }

/*^**************************************************************************^*/
/*- MouseListener Implementation.                                            -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  public void mousePressed(MouseEvent e) {
    hideTipWindow();
  }

  /**
   * {@inheritDoc}
   */
  public void mouseEntered(MouseEvent e) {
    insideComponent = (JComponent)e.getSource();
    mouseEvent = e;
    enterTimer.start();
  }

  /**
   * {@inheritDoc}
   */
  public void mouseExited(MouseEvent e) {
    hideTipWindow();
  }

  /**
   * {@inheritDoc}
   */
  public void mouseDragged(MouseEvent event) {
    hideTipWindow();
  }

  /**
   * {@inheritDoc}
   */
  public void mouseClicked(MouseEvent e) {
    // empty method
  }

  /**
   * {@inheritDoc}
   */
  public void mouseReleased(MouseEvent e) {
    // empty method
  }

  /**
   * {@inheritDoc}
   */
  public void mouseMoved(MouseEvent e) {
    String newTText = insideComponent.getToolTipText(e);
    if ((newTText != null) && (!newTText.equals(toolTipText))) {
      toolTipText = newTText;
      hideTipWindow();
      insideComponent = (JComponent)e.getSource();
      mouseEvent = e;
      enterTimer.start();
      return;
    }
    if (popup == null) {
      mouseEvent = e;
    } else {
      int dx = e.getX() - mouseEvent.getX();
      int dy = e.getY() - mouseEvent.getY();
      if (dx < 0) {
        dx = -dx;
      }
      if (dy < 0) {
        dy = -dy;
      }
      if ((dx > 10) || (dy > 10)) {
        hideTipWindow();
      }
    }
  }

/*^**************************************************************************^*/
/*- Timer action.                                                            -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  public void actionPerformed(ActionEvent e) {
    if (e.getSource() == enterTimer) {
      showTipWindow();
    } else {
      hideTipWindow();
    }
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-15 19:16:15 Slobodan
   Initial version

*****************************************************************************^*/
