/*
 * @(#)GPTools.java  1.0.0 / 2008-05-01
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.util;

import java.util.Locale;
import java.util.StringTokenizer;

/**
 * General Purpose Tools.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public final class GPTools {

/*^**************************************************************************^*/
/*- Public Static Methods.                                                   -*/
/*^**************************************************************************^*/

  /**
   * Terminates JVM.
   *
   * @param status exit status
   */
  public static void shutdownVM(int status) {
    System.exit(status); // NOPMD NOFB
  }

  /**
   * Terminates JVM.
   */
  public static void shutdownVM() {
    System.exit(0); // NOPMD NOFB
  }

  /**
   * Returns the <code>Class</code> object associated with the class or
   * interface with the given string name or <tt>null</tt> if the class cannot
   * be located.
   *
   * @param className the fully qualified name of the desired class
   *
   * @return the <code>Class</code> object for the class with the specified name
   */
  public static Class getClassForName(String className) {
    try {
      return Class.forName(className);
    } catch (Throwable e) { // NOPMD
      return null;
    }
  }

  /**
   * Extracts class name.
   *
   * @param cls the class
   *
   * @return the class name
   */
  public static String extractClassName(Class cls) {
    String cn = cls.getName();
    return cn.substring(cn.lastIndexOf(CommonConstants.STRING_DOT) + 1);
  }

  /**
   * Returns system property.
   *
   * @param key the property key
   *
   * @return system property
   */
  public static String getSystemProperty(String key) {
    try {
      return System.getProperty(key);
    } catch (SecurityException e) {
      return null;
    }
  }

  /**
   * Returns <tt>true</tt> if str starts with prefix ignoring case.
   *
   * @param str the string
   * @param prefix the string prefix
   *
   * @return <tt>true</tt> if str starts with prefix ignoring case
   */
  public static boolean startsWithIgnoreCase(String str, String prefix) {
    Locale dl = Locale.getDefault();
    return ((str != null) && (prefix != null) &&
            str.toUpperCase(dl).startsWith(prefix.toUpperCase(dl)));
  }

  /**
   * Locale parsing.
   *
   * @param locale the locale string
   *
   * @return Locale
   */
  public static Locale parseLocale(String locale) {
    if ((locale == null) || (locale.length() == 0)) {
      throw new IllegalArgumentException("Invalid locale"); //$NON-NLS-1$
    }
    String[] tokens = {
      CommonConstants.STRING_EMPTY,
      CommonConstants.STRING_EMPTY,
      CommonConstants.STRING_EMPTY
    };
    int position;
    int currentToken = 0;
    while (currentToken < 2  && (position = locale.indexOf('_')) != -1) {
      tokens[currentToken++] = locale.substring(0, position);
      locale = locale.substring(position + 1);
    }
    tokens[currentToken] = locale;
    return new Locale(tokens[0], tokens[1], tokens[2]);
  }

  /**
   * Returns source for the specified class.
   *
   * @param c the class
   *
   * @return source for the specified class
   */
  public static String getCodeSource(Class c) {
    if (c != null) {
      try {
        String enc = System.getProperty("file.encoding"); //$NON-NLS-1$
        String loc = c.getProtectionDomain().getCodeSource().getLocation().
                                                               getFile();
        int loclen = loc.length();
        StringBuffer sb = new StringBuffer(loclen);
        boolean changed = false;
        int i = 0;
        char ch;
        byte[] bytes = null;
        while (i < loclen) {
          ch = loc.charAt(i);
          if (ch == '%') {
            if (bytes == null) {
              bytes = new byte[(loclen - i) / 3];
            }
            int pos = 0;
            while (((i + 2) < loclen) && (ch == '%')) {
              bytes[pos++] = (byte)Integer.parseInt(loc.substring(i + 1, i + 3),
                                                    16);
              i += 3;
              if (i < loclen) {
                ch = loc.charAt(i);
              }
            }
            if ((i < loclen) && (ch == '%')) {
              return null;
            }
            sb.append(new String(bytes, 0, pos, enc));
            changed = true;
          } else {
            sb.append(ch);
            i++;
          }
        }
        if (changed) {
          return sb.toString();
        }
        return loc;
      } catch (Throwable t) { // NOPMD
        // empty catch block
      }
    }
    return null;
  }

  /**
   * Compares integer versions fa and fb.
   *
   * @param fa1 fa first version number
   * @param fa2 fa second version number
   * @param fa3 fa third version number
   * @param fa4 fa fourth version number
   * @param fb1 fb first version number
   * @param fb2 fb second version number
   * @param fb3 fb third version number
   * @param fb4 fb fourth version number
   *
   * @return 0 if equal, 1 if fa > fb or -1 if fa < fb
   */
  public static int compareVersions(int fa1, int fa2, int fa3, int fa4,
                                    int fb1, int fb2, int fb3, int fb4) {
    if ((fa1 == fb1) && (fa2 == fb2) && (fa3 == fb3) && (fa4 == fb4)) {
      return 0;
    }
    if ((fa1 > fb1) || ((fa1 == fb1) && ((fa2 > fb2) ||
        ((fa2 == fb2) && ((fa3 > fb3) || ((fa3 == fb3) && (fa4 >= fb4))))))) {
      return 1;
    }
    return -1;
  }

  /**
   * Compares versions.
   *
   * @param va the version a
   * @param vb the version b
   * @param delimiter the delimiter of the version's parts
   *
   * @return 0 if (va = vb), 1 if (va > vb) and -1 if (va < vb)
   */
  public static int compareVersions(String va, String vb, char delimiter) {
    String[] vas = new String[4];
    String[] vbs = new String[4];
    StringTokenizer st = new StringTokenizer(va, String.valueOf(delimiter));
    for (int i = 0; (i < 4) && st.hasMoreTokens(); i++) {
      vas[i] = st.nextToken();
    }
    st = new StringTokenizer(vb, String.valueOf(delimiter));
    for (int i = 0; (i < 4) && st.hasMoreTokens(); i++) {
      vbs[i] = st.nextToken();
    }
    int result = 0;
    for (int i = 0; i < 4; i++) {
      if (vas[i] == null) {
        vas[i] = CommonConstants.STRING_EMPTY;
      }
      if (vbs[i] == null) {
        vbs[i] = CommonConstants.STRING_EMPTY;
      }
      result = vas[i].compareTo(vbs[i]);
      if (result != 0) {
        break;
      }
    }
    return result;
  }

  /**
   * Compares dot delimited versions.
   *
   * @param va the version a
   * @param vb the version b
   *
   * @return 0 if (va = vb), 1 if (va > vb) and -1 if (va < vb)
   */
  public static int compareVersions(String va, String vb) {
    return compareVersions(va, vb, '.');
  }

  /**
   * Checks if current version is valid version.
   *
   * @param curver the current version
   * @param expver the expected version
   *
   * @return <code>true</code> if valid
   */
  public static boolean isValidVersion(String curver, String expver) {
    return compareVersions(curver, expver) >= 0;
  }

  /**
   * Returns maximum integer.
   *
   * @param x the first integer
   * @param y the second integer
   * @param z the third integer
   *
   * @return the maximum integer
   */
  public static int max(int x, int y, int z) {
    int t = x;
    if (t < y) {
      t = y;
    }
    if (t < z) {
      t = z;
    }
    return t;
  }

  /**
   * Returns minimum integer.
   *
   * @param x the first integer
   * @param y the second integer
   * @param z the third integer
   *
   * @return the minimum integer
   */
  public static int min(int x, int y, int z) {
    int t = x;
    if (t > y) {
      t = y;
    }
    if (t > z) {
      t = z;
    }
    return t;
  }

/*^**************************************************************************^*/
/*- Private Constructor.                                                     -*/
/*^**************************************************************************^*/

  /**
   * Class has no instances.
   */
  private GPTools() {
    // empty constructor body
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-01 20:29:34 Slobodan
   Initial version

*****************************************************************************^*/
