/*
 * @(#)SimpleArrayList.java  1.0.0 / 2008-05-11
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.util;

import java.lang.reflect.Array;
import java.util.Arrays;
import java.util.Comparator;
import java.util.ConcurrentModificationException;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.NoSuchElementException;

/**
 * Simple Array List.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public final class SimpleArrayList implements Iterator, Enumeration { // NOPMD

/*^**************************************************************************^*/
/*- Private Static Field.                                                    -*/
/*^**************************************************************************^*/

  private static final int MAXIMUM_CAPACITY = 1 << 30;

/*^**************************************************************************^*/
/*- Private Fields.                                                          -*/
/*^**************************************************************************^*/

  private Object[] elements;
  private int size;
  private int initialCapacity;
  private int modCount;
  private int expectedModCount;
  private int cursor;

/*^**************************************************************************^*/
/*- Public Constructors.                                                     -*/
/*^**************************************************************************^*/

  /**
   * Constructs new SimpleArrayList object.
   */
  public SimpleArrayList() {
    this(10);
  }

  /**
   * Constructs new SimpleArrayList object.
   *
   * @param o an object array
   */
  public SimpleArrayList(Object[] o) {
    if (o == null) {
      throw new IllegalArgumentException("Null array"); //$NON-NLS-1$
    }
    size = o.length;
    elements = new Object[(3 * size) >> 1];
    System.arraycopy(o, 0, elements, 0, size);
    initialCapacity = elements.length;
  }

  /**
   * Constructs new SimpleArrayList object.
   *
   * @param initialCapacity the initial capacity
   */
  public SimpleArrayList(int initialCapacity) {
    if (initialCapacity <= 0) {
      throw new IllegalArgumentException("Invalid capacity"); //$NON-NLS-1$
    }
    if (initialCapacity > MAXIMUM_CAPACITY) {
      initialCapacity = MAXIMUM_CAPACITY;
    }
    this.initialCapacity = initialCapacity;
    elements = new Object[initialCapacity];
  }

/*^**************************************************************************^*/
/*- Public Methods.                                                          -*/
/*^**************************************************************************^*/

  /**
   * Trims the capacity of this list.
   */
  public void trimToSize() {
    int oldCapacity = elements.length;
    if (size < oldCapacity) {
      Object[] oldElements = elements;
      elements = new Object[size];
      System.arraycopy(oldElements, 0, elements, 0, size);
    }
  }

  /**
   * Increases the capacity of this list.
   *
   * @param minCapacity the minimum capacity
   */
  public void ensureCapacity(int minCapacity) {
    int oldCapacity = elements.length;
    if (minCapacity > oldCapacity) {
      Object[] oldElements = elements;
      int newCapacity = ((3 * oldCapacity) >> 1) + 1;
      if (newCapacity < minCapacity) {
        newCapacity = minCapacity;
      }
      elements = new Object[newCapacity];
      System.arraycopy(oldElements, 0, elements, 0, size);
    }
  }

  /**
   * Returns the number of elements in this list.
   *
   * @return the number of elements in this list
   */
  public int size() {
    return size;
  }

  /**
   * Returns capacity of this list.
   *
   * @return capacity of this list
   */
  public int capacity() {
    return elements.length;
  }

  /**
   * Tests if this list contains elements.
   *
   * @return <tt>true</tt> if this list has no elements;
   *         <tt>false</tt> otherwise
   */
  public boolean isEmpty() {
    return (size == 0);
  }

  /**
   * Searches for the first occurrence of the given argument.
   *
   * @param o an object
   *
   * @return the index of the first occurrence of the argument in this list;
   *         returns <tt>-1</tt> if the object is not found
   */
  public int indexOf(Object o) {
    final Object[] elems = elements;
    if (o == null) {
      for (int i = 0; i < size; i++) {
        if (elems[i] == null) {
          return i;
        }
      }
    } else {
      for (int i = 0; i < size; i++) {
        if (o.equals(elems[i])) {
          return i;
        }
      }
    }
    return -1;
  }

  /**
   * Searches for the last occurrence of the given argument.
   *
   * @param o an object
   *
   * @return the index of the last occurrence of the argument in this list;
   *         returns <tt>-1</tt> if the object is not found
   */
  public int lastIndexOf(Object o) {
    Object[] elems = elements;
    if (o == null) {
      for (int i = size - 1; i >= 0; i--) {
        if (elems[i] == null) {
          return i;
        }
      }
    } else {
      for (int i = size - 1; i >= 0; i--) {
        if (o.equals(elems[i])) {
          return i;
        }
      }
    }
    return -1;
  }

  /**
   * Returns <tt>true</tt> if this list contains specified object.
   *
   * @param o the object
   *
   * @return <tt>true</tt> if this list contains specified object
   */
  public boolean contains(Object o) {
    return (indexOf(o) != -1);
  }

  /**
   * Returns the object at the specified position in this list.
   *
   * @param index index of object to return
   *
   * @return the object at the specified position in this list.
   *
   * @throws IndexOutOfBoundsException if index is out of range <tt>
   *         (index &lt; 0 || index &gt;= size())</tt>.
   */
  public Object get(int index) {
    if ((index < 0) || (index >= size)) {
      throwIndexException(index);
    }
    return elements[index];
  }

  /**
   * Returns an array containing all of the objects in this list.
   *
   * @return an array containing all of the objects in this list
   */
  public Object[] toArray() {
    Object[] result = new Object[size];
    System.arraycopy(elements, 0, result, 0, size);
    return result;
  }

  /**
   * Returns an array containing all of the elements in this list.
   *
   * @param a the array into which the elements of the collection are to be
   *          stored
   *
   * @return an array containing all of the elements in this list
   */
  public Object[] toArray(Object[] a) {
    if (a.length < size) {
      a = (Object[])Array.newInstance(a.getClass().getComponentType(), size);
    }
    System.arraycopy(elements, 0, a, 0, size);
    for (int i = size; i < a.length; i++) {
      a[i] = null;
    }
    return a;
  }

  /**
   * Appends the specified object to the end of this list.
   *
   * @param o the object to be appended to this list
   */
  public void add(Object o) {
    if (o == this) {
      throw new IllegalArgumentException("Adding list to itself"); //$NON-NLS-1$
    }
    ensureCapacity(size + 1);
    elements[size++] = o;
    modCount++;
  }

  /**
   * Replaces the object at the specified position in this list with
   * the specified element.
   *
   * @param index index of object to replace
   * @param o the object to be stored at the specified position
   *
   * @return the object previously at the specified position
   *
   * @throws IndexOutOfBoundsException if index is out of range <tt>
   *         (index &lt; 0 || index &gt;= size())</tt>
   */
  public Object set(int index, Object o) {
    if ((index < 0) || (index >= size)) {
      throwIndexException(index);
    }
    if (o == this) {
      throw new IllegalArgumentException("Adding list to itself"); //$NON-NLS-1$
    }
    Object old = elements[index];
    elements[index] = o;
    modCount++;
    return old;
  }

  /**
   * Inserts the specified object at the specified position in this list.
   *
   * @param index index at which the specified object is to be inserted
   * @param o object to be inserted
   *
   * @throws IndexOutOfBoundsException if index is out of range <tt>
   *         (index &lt; 0 || index &gt;= size())</tt>
   */
  public void insert(int index, Object o) {
    if ((index < 0) || (index >= size)) {
      throwIndexException(index);
    }
    if (o == this) {
      throw new IllegalArgumentException("Adding list to itself"); //$NON-NLS-1$
    }
    ensureCapacity(size + 1);
    System.arraycopy(elements, index, elements, index + 1, size - index);
    elements[index] = o;
    size++;
    modCount++;
  }

  /**
   * Removes the object at the specified position in this list.
   *
   * @param index the index of the object to remove
   *
   * @return the object that was removed from the list
   *
   * @throws IndexOutOfBoundsException if index is out of range <tt>
   *         (index &lt; 0 || index &gt;= size())</tt>
   */
  public Object remove(int index) {
    if ((index < 0) || (index >= size)) {
      throwIndexException(index);
    }
    Object old = elements[index];
    int numMoved = size - index - 1;
    if (numMoved > 0) {
      System.arraycopy(elements, index + 1, elements, index, numMoved);
    }
    elements[--size] = null;
    modCount++;
    return old;
  }

  /**
   * Removes specified object.
   *
   * @param o the object to be removed
   *
   * @return <tt>true</tt> if object exists; <tt>false</tt> otherwise
   */
  public boolean remove(Object o) {
    int i = indexOf(o);
    if (i != -1) {
      remove(i);
      return true;
    }
    return false;
  }

  /**
   * Removes all objects from this list.
   */
  public void clear() {
    if (elements.length > (2 * initialCapacity)) {
      elements = new Object[initialCapacity];
    } else {
      final Object[] elems = elements;
      for (int i = size - 1; i >= 0; i--) {
        elems[i] = null;
      }
    }
    size = 0;
    modCount++;
  }

  /**
   * Sorts this list.
   *
   * @throws ClassCastException if the array contains elements that are not
   *                            <i>mutually comparable</i>
   */
  public void sort() {
    Arrays.sort(elements, 0, size);
  }

  /**
   * Sorts this list using specified comparator.
   *
   * @param c the comparator
   *
   * @throws ClassCastException if the array contains elements that are not
   *                            <i>mutually comparable</i>
   */
  public void sort(Comparator c) {
    Arrays.sort(elements, 0, size, c);
  }

  /**
   * Returns iterator over the elements.
   *
   * @return list iterator
   */
  public Iterator iterator() {
    cursor = 0;
    expectedModCount = modCount;
    return this;
  }

  /**
   * Iterator hasNext method.
   *
   * @return <tt>true</tt> if the iterator has more elements
   */
  public boolean hasNext() {
    if (modCount != expectedModCount) {
      throw new ConcurrentModificationException();
    }
    return (cursor != size);
  }

  /**
   * Iterator's next method.
   *
   * @return the next element in the iteration
   *
   * @exception NoSuchElementException if iteration has no more elements
   */
  public Object next() {
    if (modCount != expectedModCount) {
      throw new ConcurrentModificationException();
    }
    if (cursor == size) {
      throw new NoSuchElementException();
    }
    return elements[cursor++];
  }

  /**
   * Iterator's remove method.
   */
  public void remove() {
    throw new UnsupportedOperationException();
  }

  /**
   * Returns enumeration over the elements.
   *
   * @return list enumeration
   */
  public Enumeration enumeration() {
    cursor = 0;
    expectedModCount = modCount;
    return this;
  }

  /**
   * Enumeration's hasMoreElements method.
   *
   * @return <tt>true</tt> if the enumeration has more elements
   */
  public boolean hasMoreElements() {
    if (modCount != expectedModCount) {
      throw new ConcurrentModificationException();
    }
    return (cursor != size);
  }

  /**
   * Enumeration's nextElement method.
   *
   * @return the next element in the enumeration
   *
   * @exception NoSuchElementException if enumeration has no more elements
   */
  public Object nextElement() {
    if (modCount != expectedModCount) {
      throw new ConcurrentModificationException();
    }
    if (cursor == size) {
      throw new NoSuchElementException();
    }
    return elements[cursor++];
  }

/*^**************************************************************************^*/
/*- Overridden Methods.                                                      -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  public boolean equals(Object o) {
    if (o == this) {
      return true;
    }
    if (!(o instanceof SimpleArrayList)) {
      return false;
    }
    SimpleArrayList ol = (SimpleArrayList)o;
    if (size != ol.size) {
      return false;
    }
    final Object[] elems = elements;
    final Object[] oelems = ol.elements;
    for (int i = size - 1; i >= 0; i--) {
      if (((elems[i] == null) && (oelems[i] != null)) ||
          (elems[i] != null) && (!elems[i].equals(oelems[i]))) {
        return false;
      }
    }
    return true;
  }

  /**
   * {@inheritDoc}
   */
  public int hashCode() {
    int hashCode = 1;
    final Object[] elems = elements;
    for (int i = size - 1; i >= 0; i--) {
      Object o = elems[i];
      if (o != null) {
        hashCode = 31 * hashCode + o.hashCode();
      }
    }
    return hashCode;
  }

  /**
   * {@inheritDoc}
   */
  public String toString() {
    StringBuffer sb = new StringBuffer(10 * size);
    sb.append('[');
    final Object[] elements = this.elements;
    for (int i = 0; i < size; i++) {
      if (i > 0) {
        sb.append(", "); //$NON-NLS-1$
      }
      sb.append(elements[i]);
    }
    sb.append(']');
    return sb.toString();
  }

/*^**************************************************************************^*/
/*- Private Method.                                                          -*/
/*^**************************************************************************^*/

  private void throwIndexException(int index) {
    throw new IndexOutOfBoundsException("Index: " + index + //$NON-NLS-1$
                                        ", Size: " + size); //$NON-NLS-1$
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-11 21:20:58 Slobodan
   Initial version

*****************************************************************************^*/
