/*
 * @(#)SimpleHashMap.java  1.0.0 / 2008-05-12
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.util;

import java.util.ConcurrentModificationException;
import java.util.Iterator;
import java.util.NoSuchElementException;

/**
 * Simple Hash Map.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public final class SimpleHashMap implements Iterator {

/*^**************************************************************************^*/
/*- Private Static Field.                                                    -*/
/*^**************************************************************************^*/

  private static final int MAXIMUM_CAPACITY = 1 << 30;

/*^**************************************************************************^*/
/*- Private Fields.                                                          -*/
/*^**************************************************************************^*/

  private ObjectEntry[] table;
  private int size;
  private int initialCapacity;
  private float enlargeFactor;
  private float shrinkFactor;
  private int enlargeThreshold;
  private int shrinkThreshold;
  private int modCount;
  private int expectedModCount;
  private boolean itKeys;
  private ObjectEntry currentEntry;
  private int currentBucket;

/*^**************************************************************************^*/
/*- Public Constructors.                                                     -*/
/*^**************************************************************************^*/

  /**
   * Constructs new SimpleHashMap object.
   */
  public SimpleHashMap() {
    this(11, 0.75f, 0.3f);
  }

  /**
   * Constructs new SimpleHashMap object.
   *
   * @param initialCapacity the initial capacity
   */
  public SimpleHashMap(int initialCapacity) {
    this(initialCapacity, 0.75f, 0.3f);
  }

  /**
   * Constructs new SimpleHashMap object.
   *
   * @param initialCapacity the initial capacity
   * @param loadFactor the load factor
   */
  public SimpleHashMap(int initialCapacity, float loadFactor) {
    this(initialCapacity, loadFactor, 0.3f);
  }

  /**
   * Constructs new SimpleHashMap object.
   *
   * @param o the object array
   */
  public SimpleHashMap(Object[] o) {
    this((int)(o.length * 1.8), 0.75f);
    for (int i = o.length - 1; i >= 0; i--) {
      put(o[i], o[i]);
    }
  }

  /**
   * Constructs new SimpleHashMap object.
   *
   * @param initialCapacity the initial capacity
   * @param enlargeFactor the enlarge factor
   * @param shrinkFactor the shrink factor
   */
  public SimpleHashMap(int initialCapacity, float enlargeFactor,
                         float shrinkFactor) {
    if (initialCapacity <= 0) {
      throw new IllegalArgumentException("Illegal capacity"); //$NON-NLS-1$
    }
    if ((enlargeFactor <= 0) || (enlargeFactor > 1) ||
        Float.isNaN(enlargeFactor)) {
      throw new IllegalArgumentException(
                                       "Illegal enlarge factor"); //$NON-NLS-1$
    }
    if ((shrinkFactor <= 0) || (shrinkFactor > 1) ||
        Float.isNaN(shrinkFactor)) {
      throw new IllegalArgumentException("Illegal shrink factor"); //$NON-NLS-1$
    }
    if (initialCapacity > MAXIMUM_CAPACITY) {
      initialCapacity = MAXIMUM_CAPACITY;
    }
    this.initialCapacity = initialCapacity;
    this.enlargeFactor = enlargeFactor;
    this.shrinkFactor = shrinkFactor;
    table = new ObjectEntry[initialCapacity];
    enlargeThreshold = (int)(initialCapacity * enlargeFactor);
    shrinkThreshold = (int)(initialCapacity * shrinkFactor);
  }

/*^**************************************************************************^*/
/*- Public Methods.                                                          -*/
/*^**************************************************************************^*/

  /**
   * Returns number of objects in this table.
   *
   * @return number of objects in this table
   */
  public int size() {
    return size;
  }

  /**
   * Returns capacity of this map.
   *
   * @return capacity of this map
   */
  public int capacity() {
    return table.length;
  }

  /**
   * Tests if this table contains entries.
   *
   * @return <tt>true</tt> if this table has no entries;
   *         <tt>false</tt> otherwise.
   */
  public boolean empty() {
    return (size == 0);
  }

  /**
   * Returns <tt>true</tt> if this table contains specified key.
   *
   * @param key the key
   *
   * @return <tt>true</tt> if this table contains specified key
   */
  public boolean containsKey(Object key) {
    if (key != null) {
      int hash = key.hashCode();
      final ObjectEntry[] t = table;
      int index = (hash & 0x7FFFFFFF) % t.length;
      for (ObjectEntry e = t[index]; e != null; e = e.next) {
        if ((e.hash == hash) && e.key.equals(key)) {
          return true;
        }
      }
    }
    return false;
  }

  /**
   * Returns <tt>true</tt> if this table contains specified value.
   *
   * @param value the value
   *
   * @return <tt>true</tt> if this table contains specified value
   */
  public boolean containsValue(Object value) {
    if (value != null) {
      final ObjectEntry[] t = table;
      for (int i = t.length - 1; i >= 0; i--) {
        for (ObjectEntry e = t[i]; e != null; e = e.next) {
          if (e.value.equals(value)) {
            return true;
          }
        }
      }
    }
    return false;
  }

  /**
   * Returns the value of specified key or <tt>null</tt> if there is no such
   * key.
   *
   * @param key a key in the table.
   *
   * @return the value or <tt>null</tt> if there is no such key
   */
  public Object get(Object key) {
    if (key != null) {
      int hash = key.hashCode();
      final ObjectEntry[] t = table;
      int index = (hash & 0x7FFFFFFF) % t.length;
      for (ObjectEntry e = t[index]; e != null; e = e.next) {
        if ((e.hash == hash) && e.key.equals(key)) {
          return e.value;
        }
      }
    }
    return null;
  }

  /**
   * Puts new key-value pair.
   *
   * @param key the key
   * @param value the value
   *
   * @return old value
   */
  public Object put(Object key, Object value) {
    if (key == null) {
      throw new IllegalArgumentException("Null key"); //$NON-NLS-1$
    }
    if (value == null) {
      throw new IllegalArgumentException("Null value"); //$NON-NLS-1$
    }
    if ((key == this) || (value == this)) {
      throw new IllegalArgumentException("Adding map to itself"); //$NON-NLS-1$
    }
    ObjectEntry[] t = table;
    int hash = key.hashCode();
    int index = (hash & 0x7FFFFFFF) % t.length;
    for (ObjectEntry e = t[index]; e != null; e = e.next) {
      if ((e.hash == hash) && e.key.equals(key)) {
        Object old = e.value;
        e.value = value;
        modCount++;
        return old;
      }
    }
    if (size >= enlargeThreshold) {
      resize(true);
      t = table;
      index = (hash & 0x7FFFFFFF) % t.length;
    }
    ObjectEntry e = new ObjectEntry(hash, key, value, t[index]);
    t[index] = e;
    size++;
    modCount++;
    return null;
  }

  /**
   * Removes specified key from the table.
   *
   * @param key the key
   *
   * @return value
   */
  public Object remove(Object key) {
    final ObjectEntry[] t = table;
    int hash = key.hashCode();
    int index = (hash & 0x7FFFFFFF) % t.length;
    for (ObjectEntry e = t[index], prev = null; e != null;
        prev = e, e = e.next) {
      if ((e.hash == hash) && e.key.equals(key)) {
        if (prev == null) {
          t[index] = e.next;
        } else {
          prev.next = e.next;
        }
        size--;
        Object oldValue = e.value;
        e.value = null;
        if ((t.length > initialCapacity) && (size < shrinkThreshold)) {
          resize(false);
        }
        modCount++;
        return oldValue;
      }
    }
    modCount++;
    return null;
  }

  /**
   * Clears this table.
   */
  public void clear() {
    final ObjectEntry[] t = table;
    if (t.length > (2 * initialCapacity)) {
      table = new ObjectEntry[initialCapacity];
      enlargeThreshold = (int)(initialCapacity * enlargeFactor);
      shrinkThreshold = (int)(initialCapacity * shrinkFactor);
    } else {
      for (int index = t.length; --index >= 0;) {
        t[index] = null;
      }
    }
    size = 0;
    modCount++;
    currentEntry = null;
  }

  /**
   * Iterator over the keys.
   *
   * @return key iterator
   */
  public Iterator keys() {
    expectedModCount = modCount;
    currentEntry = null;
    currentBucket = table.length;
    itKeys = true;
    return this;
  }

  /**
   * Iterator over the values.
   *
   * @return value iterator
   */
  public Iterator values() {
    expectedModCount = modCount;
    currentEntry = null;
    currentBucket = table.length;
    itKeys = false;
    return this;
  }

/*^**************************************************************************^*/
/*- Iterator Interface Implementation.                                       -*/
/*^**************************************************************************^*/

  /**
   * Iterator hasNext method.
   *
   * @return <tt>true</tt> if the iterator has more elements
   */
  public boolean hasNext() {
    if (modCount != expectedModCount) {
      throw new ConcurrentModificationException();
    }
    ObjectEntry he = currentEntry;
    int i = currentBucket;
    final ObjectEntry[] t = table;
    while ((he == null) && (i > 0)) {
      he = t[--i];
    }
    currentEntry = he;
    currentBucket = i;
    return (currentEntry != null);
  }

  /**
   * Iterator next method.
   *
   * @return the next element in the iteration.
   *
   * @exception NoSuchElementException iteration has no more elements.
   */
  public Object next() {
    if (modCount != expectedModCount) {
      throw new ConcurrentModificationException();
    }
    ObjectEntry he = currentEntry;
    int i = currentBucket;
    final ObjectEntry[] t = table;
    while ((he == null) && (i > 0)) {
      he = t[--i];
    }
    currentEntry = he;
    currentBucket = i;
    if (he != null) {
      ObjectEntry e = currentEntry;
      currentEntry = e.next;
      if (itKeys) {
        return e.key;
      }
      return e.value;
    }
    throw new NoSuchElementException();
  }

  /**
   * Iterator remove method.
   */
  public void remove() {
    throw new UnsupportedOperationException();
  }

/*^**************************************************************************^*/
/*- Overridden Methods.                                                      -*/
/*^**************************************************************************^*/

  /**
   * Compares the specified object with this map for equality.
   *
   * @param o the object to be compared for equality with this map
   *
   * @return <tt>true</tt> if the specified object is equal to this map
   */
  public boolean equals(Object o) {
    if (o == this) {
      return true;
    }
    if (!(o instanceof SimpleHashMap)) {
      return false;
    }
    SimpleHashMap om = (SimpleHashMap)o;
    if (size != om.size) {
      return false;
    }
    final ObjectEntry[] t = table;
    for (int i = t.length - 1; i >= 0; i--) {
      for (ObjectEntry e = t[i]; e != null; e = e.next) {
        if (!e.value.equals(om.get(e.key))) {
          return false;
        }
      }
    }
    return true;
  }

  /**
   * Returns the hash code value for this map.
   *
   * @return the hash code value for this map
   */
  public int hashCode() {
    int hashCode = 0;
    final ObjectEntry[] t = table;
    for (int i = t.length - 1; i >= 0; i--) {
      for (ObjectEntry e = t[i]; e != null; e = e.next) {
        hashCode += e.key.hashCode() ^ e.value.hashCode();
      }
    }
    return hashCode;
  }

/*^**************************************************************************^*/
/*- Private Method.                                                          -*/
/*^**************************************************************************^*/

  private void resize(boolean enlarge) {
    int oldCapacity = table.length;
    ObjectEntry[] oldTable = table;
    int newCapacity;
    if (enlarge) {
      newCapacity = ((3 * oldCapacity) >> 1) + 1;
    } else {
      newCapacity = oldCapacity / 2;
      if (newCapacity < initialCapacity) {
        newCapacity = initialCapacity;
      }
    }
    ObjectEntry[] newTable = new ObjectEntry[newCapacity];
    enlargeThreshold = (int)(newCapacity * enlargeFactor);
    shrinkThreshold = (int)(newCapacity * shrinkFactor);
    table = newTable;
    for (int i = oldCapacity - 1; i >= 0; i--) {
      ObjectEntry old = oldTable[i];
      while (old != null) {
        ObjectEntry e = old;
        old = old.next;
        int index = (e.hash & 0x7FFFFFFF) % newCapacity;
        e.next = newTable[index];
        newTable[index] = e;
      }
    }
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-12 21:14:46 Slobodan
   Initial version

*****************************************************************************^*/
