/*
 * @(#)SimpleStringList.java  1.0.0 / 2008-05-12
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.util;

import java.util.Arrays;
import java.util.Comparator;
import java.util.ConcurrentModificationException;
import java.util.NoSuchElementException;

/**
 * Simple String List.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public final class SimpleStringList implements StringIterator {

/*^**************************************************************************^*/
/*- Private Static Field.                                                    -*/
/*^**************************************************************************^*/

  private static final int MAXIMUM_CAPACITY = 1 << 30;

/*^**************************************************************************^*/
/*- Private Fields.                                                          -*/
/*^**************************************************************************^*/

  private String[] strings;
  private int size;
  private int initialCapacity;
  private int modCount;
  private int expectedModCount;
  private int cursor;

/*^**************************************************************************^*/
/*- Public Constructors.                                                     -*/
/*^**************************************************************************^*/

  /**
   * Constructs new SimpleStringList object.
   */
  public SimpleStringList() {
    this(10);
  }

  /**
   * Constructs new SimpleStringList object.
   *
   * @param initialCapacity the initial capacity
   */
  public SimpleStringList(int initialCapacity) {
    if (initialCapacity <= 0) {
      throw new IllegalArgumentException("Illegal capacity"); //$NON-NLS-1$
    }
    if (initialCapacity > MAXIMUM_CAPACITY) {
      initialCapacity = MAXIMUM_CAPACITY;
    }
    this.initialCapacity = initialCapacity;
    strings = new String[initialCapacity];
  }

  /**
   * Constructs new SimpleStringList object.
   *
   * @param s a string array
   */
  public SimpleStringList(String[] s) {
    if (s == null) {
      throw new IllegalArgumentException("Null array"); //$NON-NLS-1$
    }
    size = s.length;
    strings = new String[(3 * size) >> 1];
    System.arraycopy(s, 0, strings, 0, size);
    initialCapacity = strings.length;
  }

/*^**************************************************************************^*/
/*- Public Methods.                                                          -*/
/*^**************************************************************************^*/

  /**
   * Trims the capacity of this list.
   */
  public void trimToSize() {
    int oldCapacity = strings.length;
    if (size < oldCapacity) {
      String[] oldElements = strings;
      strings = new String[size];
      System.arraycopy(oldElements, 0, strings, 0, size);
    }
  }

  /**
   * Increases the capacity of this list.
   *
   * @param minCapacity the minimum capacity
   */
  public void ensureCapacity(int minCapacity) {
    int oldCapacity = strings.length;
    if (minCapacity > oldCapacity) {
      String[] oldElements = strings;
      int newCapacity = (oldCapacity * 3) / 2 + 1;
      if (newCapacity < minCapacity) {
        newCapacity = minCapacity;
      }
      strings = new String[newCapacity];
      System.arraycopy(oldElements, 0, strings, 0, size);
    }
  }

  /**
   * Returns the number of elements in this list.
   *
   * @return the number of elements in this list
   */
  public int size() {
    return size;
  }

  /**
   * Returns capacity of this list.
   *
   * @return capacity of this list
   */
  public int capacity() {
    return strings.length;
  }

  /**
   * Tests if this list contains elements.
   *
   * @return <tt>true</tt> if this list has no elements;
   *         <tt>false</tt> otherwise
   */
  public boolean isEmpty() {
    return (size == 0);
  }

  /**
   * Searches for the first occurrence of the given argument.
   *
   * @param s a string
   *
   * @return the index of the first occurrence of the argument in this list;
   *         returns <tt>-1</tt> if the string is not found
   */
  public int indexOf(String s) {
    final String[] strs = strings;
    if (s == null) {
      for (int i = 0; i < size; i++) {
        if (strs[i] == null) {
          return i;
        }
      }
    } else {
      for (int i = 0; i < size; i++) {
        if (s.equals(strs[i])) {
          return i;
        }
      }
    }
    return -1;
  }

  /**
   * Searches for the last occurrence of the given argument.
   *
   * @param s a string
   *
   * @return the index of the last occurrence of the argument in this list;
   *         returns <tt>-1</tt> if the string is not found
   */
  public int lastIndexOf(String s) {
    final String[] strs = strings;
    if (s == null) {
      for (int i = size - 1; i >= 0; i--) {
        if (strs[i] == null) {
          return i;
        }
      }
    } else {
      for (int i = size - 1; i >= 0; i--) {
        if (s.equals(strs[i])) {
          return i;
        }
      }
    }
    return -1;
  }

  /**
   * Returns <tt>true</tt> if this list contains specified object.
   *
   * @param s the string
   *
   * @return <tt>true</tt> if this list contains specified object
   */
  public boolean contains(String s) {
    return (indexOf(s) != -1);
  }

  /**
   * Returns the string at the specified position in this list.
   *
   * @param index index of string to return
   *
   * @return the string at the specified position in this list
   *
   * @throws IndexOutOfBoundsException if index is out of range <tt>
   *         (index &lt; 0 || index &gt;= size())</tt>
   */
  public String get(int index) {
    if ((index < 0) || (index >= size)) {
      throwIndexException(index);
    }
    return strings[index];
  }

  /**
   * Returns an array containing all of the strings in this list.
   *
   * @return an array containing all of the strings in this list
   */
  public String[] toArray() {
    String[] result = new String[size];
    System.arraycopy(strings, 0, result, 0, size);
    return result;
  }

  /**
   * Returns an array containing all of the strings in this list.
   *
   * @param a the array into which the elements of the collection are to be
   *          stored
   *
   * @return an array containing all of the strings in this list
   */
  public String[] toArray(String[] a) {
    if (a.length < size) {
      a = new String[size];
    }
    System.arraycopy(strings, 0, a, 0, size);
    for (int i = size; i < a.length; i++) {
      a[i] = null;
    }
    return a;
  }

  /**
   * Appends the specified string to the end of this list.
   *
   * @param s string to be appended to this list
   */
  public void add(String s) {
    ensureCapacity(size + 1);
    strings[size++] = s;
    modCount++;
  }

  /**
   * Replaces the string at the specified position in this list with
   * the specified element.
   *
   * @param index index of string to replace
   * @param s string to be stored at the specified position
   *
   * @return the string previously at the specified position
   *
   * @throws IndexOutOfBoundsException if index is out of range <tt>
   *         (index &lt; 0 || index &gt;= size())</tt>
   */
  public String set(int index, String s) {
    if ((index < 0) || (index >= size)) {
      throwIndexException(index);
    }
    String old = strings[index];
    strings[index] = s;
    modCount++;
    return old;
  }

  /**
   * Inserts the specified string at the specified position in this list.
   *
   * @param index index at which the specified string is to be inserted.
   * @param s string to be inserted
   *
   * @throws IndexOutOfBoundsException if index is out of range <tt>
   *         (index &lt; 0 || index &gt;= size())</tt>
   */
  public void insert(int index, String s) {
    if ((index < 0) || (index >= size)) {
      throwIndexException(index);
    }
    ensureCapacity(size + 1);
    System.arraycopy(strings, index, strings, index + 1, size - index);
    strings[index] = s;
    size++;
    modCount++;
  }

  /**
   * Removes the string at the specified position in this list.
   *
   * @param index the index of the string to remove
   *
   * @return the string that was removed from the list
   *
   * @throws IndexOutOfBoundsException if index is out of range <tt>
   *         (index &lt; 0 || index &gt;= size())</tt>
   */
  public String remove(int index) {
    if ((index < 0) || (index >= size)) {
      throwIndexException(index);
    }
    String old = strings[index];
    int numMoved = size - index - 1;
    if (numMoved > 0) {
      System.arraycopy(strings, index + 1, strings, index, numMoved);
    }
    strings[--size] = null;
    modCount++;
    return old;
  }

  /**
   * Removes specified string.
   *
   * @param s the string to be removed
   *
   * @return <tt>true</tt> if object exists; <tt>false</tt> otherwise
   */
  public boolean remove(String s) {
    int i = indexOf(s);
    if (i != -1) {
      remove(i);
      return true;
    }
    return false;
  }

  /**
   * Removes all strings from this list.
   */
  public void clear() {
    if (strings.length > (2 * initialCapacity)) {
      strings = new String[initialCapacity];
    } else {
      String[] strs = strings;
      for (int i = size - 1; i >= 0; i--) {
        strs[i] = null;
      }
    }
    size = 0;
    modCount++;
  }

  /**
   * Returns iterator over the strings.
   *
   * @return iterator over the strings
   */
  public StringIterator stringIterator() {
    cursor = 0;
    expectedModCount = modCount;
    return this;
  }

  /**
   * Iterator's hasNextString method.
   *
   * @return <tt>true</tt> if the iterator has more elements
   */
  public boolean hasNextString() {
    if (modCount != expectedModCount) {
      throw new ConcurrentModificationException();
    }
    return (cursor != size);
  }

  /**
   * Iterator's next method.
   *
   * @return the next element in the iteration
   *
   * @exception NoSuchElementException iteration has no more elements
   */
  public String nextString() {
    if (modCount != expectedModCount) {
      throw new ConcurrentModificationException();
    }
    if (cursor == size) {
      throw new NoSuchElementException();
    }
    return strings[cursor++];
  }

  /**
   * Iterator's removeString method.
   */
  public void removeString() {
    throw new UnsupportedOperationException();
  }

  /**
   * Sorts this list.
   */
  public void sort() {
    Arrays.sort(strings, 0, size);
  }

  /**
   * Sorts this list using specified comparator.
   *
   * @param c the comparator
   */
  public void sort(Comparator c) {
    Arrays.sort(strings, 0, size, c);
  }

/*^**************************************************************************^*/
/*- Overridden Methods.                                                      -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  public boolean equals(Object o) {
    if (o == this) {
      return true;
    }
    if (!(o instanceof SimpleStringList)) {
      return false;
    }
    SimpleStringList sl = (SimpleStringList)o;
    if (size != sl.size) {
      return false;
    }
    final String[] strs = strings;
    final String[] slstrs = sl.strings;
    for (int i = size - 1; i >= 0; i--) {
      String e = strs[i];
      if (((e == null) && (slstrs[i] != null)) ||
          (e != null) && (!e.equals(slstrs[i]))) {
        return false;
      }
    }
    return true;
  }

  /**
   * {@inheritDoc}
   */
  public int hashCode() {
    int hashCode = 1;
    final String[] strs = strings;
    for (int i = size - 1; i >= 0; i--) {
      String s = strs[i];
      if (s != null) {
        hashCode = 31 * hashCode + s.hashCode();
      }
    }
    return hashCode;
  }

  /**
   * {@inheritDoc}
   */
  public String toString() {
    StringBuffer sb = new StringBuffer(10 * size);
    sb.append('[');
    final Object[] strings = this.strings;
    for (int i = 0; i < size; i++) {
      if (i > 0) {
        sb.append(", "); //$NON-NLS-1$
      }
      sb.append(strings[i]);
    }
    sb.append(']');
    return sb.toString();
  }

/*^**************************************************************************^*/
/*- Private Method.                                                          -*/
/*^**************************************************************************^*/

  private void throwIndexException(int index) {
    throw new IndexOutOfBoundsException("Index: " + index + //$NON-NLS-1$
                                        ", Size: " + size); //$NON-NLS-1$
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-12 20:28:20 Slobodan
   Initial version

*****************************************************************************^*/
