/*
 * @(#)SimpleStringMap.java  1.0.0 / 2008-05-12
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.util;

import java.util.ConcurrentModificationException;
import java.util.NoSuchElementException;

/**
 * Simple string hash map.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public final class SimpleStringMap implements StringIterator {

/*^**************************************************************************^*/
/*- Private Static Field.                                                    -*/
/*^**************************************************************************^*/

  private static final int MAXIMUM_CAPACITY = 1 << 30;

/*^**************************************************************************^*/
/*- Private Fields.                                                          -*/
/*^**************************************************************************^*/

  private StringEntry[] table;
  private int size;
  private int initialCapacity;
  private float enlargeFactor;
  private float shrinkFactor;
  private int enlargeThreshold;
  private int shrinkThreshold;
  private int modCount;
  private int expectedModCount;
  private boolean intKeys;
  private StringEntry currentEntry;
  private int currentBucket;

/*^**************************************************************************^*/
/*- Public Constructors.                                                     -*/
/*^**************************************************************************^*/

  /**
   * Constructs new SimpleStringMap object.
   */
  public SimpleStringMap() {
    this(11, 0.75f,  0.3f);
  }

  /**
   * Constructs new SimpleStringMap object.
   *
   * @param initialCapacity the initial capacity
   */
  public SimpleStringMap(int initialCapacity) {
    this(initialCapacity, 0.75f,  0.3f);
  }

  /**
   * Constructs new SimpleStringMap object.
   *
   * @param initialCapacity the initial capacity
   * @param loadFactor the load factor
   */
  public SimpleStringMap(int initialCapacity, float loadFactor) {
    this(initialCapacity, loadFactor, 0.3f);
  }

  /**
   * Constructs new SimpleStringMap object.
   *
   * @param sa the string array
   */
  public SimpleStringMap(String[] sa) {
    this((int)(sa.length * 1.8), 0.75f);
    for (int i = sa.length - 1; i >= 0; i--) {
      put(sa[i], sa[i]);
    }
  }

  /**
   * Constructs new SimpleStringMap object.
   *
   * @param initialCapacity the initial capacity
   * @param enlargeFactor the enlarge factor
   * @param shrinkFactor the shrink factor
   */
  public SimpleStringMap(int initialCapacity, float enlargeFactor,
                           float shrinkFactor) {
    if (initialCapacity <= 0) {
      throw new IllegalArgumentException("Illegal capacity"); //$NON-NLS-1$
    }
    if ((enlargeFactor <= 0) || (enlargeFactor > 1) ||
        Float.isNaN(enlargeFactor)) {
      throw new IllegalArgumentException(
                                        "Illegal enlarge factor"); //$NON-NLS-1$
    }
    if ((shrinkFactor <= 0) || (shrinkFactor > 1) ||
        Float.isNaN(shrinkFactor)) {
      throw new IllegalArgumentException("Illegal shrink factor"); //$NON-NLS-1$
    }
    if (initialCapacity > MAXIMUM_CAPACITY) {
      initialCapacity = MAXIMUM_CAPACITY;
    }
    this.initialCapacity = initialCapacity;
    this.enlargeFactor = enlargeFactor;
    this.shrinkFactor = shrinkFactor;
    table = new StringEntry[initialCapacity];
    enlargeThreshold = (int)(initialCapacity * enlargeFactor);
    shrinkThreshold = (int)(initialCapacity * shrinkFactor);
  }

/*^**************************************************************************^*/
/*- Public Methods.                                                          -*/
/*^**************************************************************************^*/

  /**
   * Returns number of strings in this table.
   *
   * @return number of strings in this table
   */
  public int size() {
    return size;
  }

  /**
   * Returns capacity of this map.
   *
   * @return capacity of this map
   */
  public int capacity() {
    return table.length;
  }

  /**
   * Tests if this table has no entries.
   *
   * @return <tt>true</tt> if this table has no entries;
   *         <tt>false</tt> otherwise.
   */
  public boolean empty() {
    return (size == 0);
  }

  /**
   * Returns <tt>true</tt> if this table contains specified key.
   *
   * @param key the key
   *
   * @return <tt>true</tt> if this table contains specified key
   */
  public boolean containsKey(String key) {
    if (key != null) {
      int hash = key.hashCode();
      final StringEntry[] t = table;
      int index = (hash & 0x7FFFFFFF) % t.length;
      for (StringEntry e = t[index]; e != null; e = e.next) {
        if ((e.hash == hash) && e.key.equals(key)) {
          return true;
        }
      }
    }
    return false;
  }

  /**
   * Returns <tt>true</tt> if this table contains specified value.
   *
   * @param value the value
   *
   * @return <tt>true</tt> if this table contains specified value
   */
  public boolean containsValue(String value) {
    if (value != null) {
      final StringEntry[] t = table;
      for (int i = t.length - 1; i >= 0; i--) {
        for (StringEntry e = t[i]; e != null; e = e.next) {
          if (e.value.equals(value)) {
            return true;
          }
        }
      }
    }
    return false;
  }

  /**
   * Returns the value of specified key or <tt>null</tt> if there is no such
   * key.
   *
   * @param key a key in the table.
   *
   * @return the value or <tt>null</tt> if there is no such key
   */
  public String get(String key) {
    if (key != null) {
      int hash = key.hashCode();
      final StringEntry[] t = table;
      int index = (hash & 0x7FFFFFFF) % t.length;
      for (StringEntry e = t[index]; e != null; e = e.next) {
        if ((e.hash == hash) && e.key.equals(key)) {
          return e.value;
        }
      }
    }
    return null;
  }

  /**
   * Puts new key.
   *
   * @param key the key
   * @param value the value
   *
   * @return old value
   */
  public String put(String key, String value) {
    if (key == null) {
      throw new IllegalArgumentException("Key value"); //$NON-NLS-1$
    }
    if (value == null) {
      throw new IllegalArgumentException("Null value"); //$NON-NLS-1$
    }
    StringEntry[] t = table;
    int hash = key.hashCode();
    int index = (hash & 0x7FFFFFFF) % t.length;
    for (StringEntry e = t[index]; e != null; e = e.next) {
      if ((e.hash == hash) && e.key.equals(key)) {
        String old = e.value;
        e.value = value;
        modCount++;
        return old;
      }
    }
    if (size >= enlargeThreshold) {
      resize(true);
      t = table;
      index = (hash & 0x7FFFFFFF) % t.length;
    }
    StringEntry e = new StringEntry(hash, key, value, t[index]);
    t[index] = e;
    size++;
    modCount++;
    return null;
  }

  /**
   * Removes specified key from the table.
   *
   * @param key the key
   *
   * @return value
   */
  public String remove(String key) {
    final StringEntry[] t = table;
    int hash = key.hashCode();
    int index = (hash & 0x7FFFFFFF) % t.length;
    for (StringEntry e = t[index], prev = null; e != null;
        prev = e, e = e.next) {
      if ((e.hash == hash) && e.key.equals(key)) {
        if (prev == null) {
          t[index] = e.next;
        } else {
          prev.next = e.next;
        }
        size--;
        String oldValue = e.value;
        e.value = null;
        if ((t.length > initialCapacity) && (size < shrinkThreshold)) {
          resize(false);
        }
        modCount++;
        return oldValue;
      }
    }
    modCount++;
    return null;
  }

  /**
   * Clears this table.
   */
  public void clear() {
    final StringEntry[] t = table;
    if (t.length > (2 * initialCapacity)) {
      table = new StringEntry[initialCapacity];
      enlargeThreshold = (int)(initialCapacity * enlargeFactor);
      shrinkThreshold = (int)(initialCapacity * shrinkFactor);
    } else {
      for (int index = t.length; --index >= 0;) {
        t[index] = null;
      }
    }
    size = 0;
    modCount++;
    currentEntry = null;
  }

  /**
   * Iterator over the keys.
   *
   * @return key iterator
   */
  public StringIterator keys() {
    expectedModCount = modCount;
    currentEntry = null;
    currentBucket = table.length;
    intKeys = true;
    return this;
  }

  /**
   * Iterator over the values.
   *
   * @return value iterator
   */
  public StringIterator values() {
    expectedModCount = modCount;
    currentEntry = null;
    currentBucket = table.length;
    intKeys = false;
    return this;
  }

/*^**************************************************************************^*/
/*- Iterator Interface Implementation.                                       -*/
/*^**************************************************************************^*/

  /**
   * Iterator's hasNextString method.
   *
   * @return <tt>true</tt> if the iterator has more elements
   */
  public boolean hasNextString() {
    if (modCount != expectedModCount) {
      throw new ConcurrentModificationException();
    }
    StringEntry he = currentEntry;
    int i = currentBucket;
    final StringEntry[] t = table;
    while ((he == null) && (i > 0)) {
      he = t[--i];
    }
    currentEntry = he;
    currentBucket = i;
    return (currentEntry != null);
  }

  /**
   * Iterator's nextString method.
   *
   * @return the next element in the iteration.
   *
   * @exception NoSuchElementException iteration has no more elements.
   */
  public String nextString() {
    if (modCount != expectedModCount) {
      throw new ConcurrentModificationException();
    }
    StringEntry he = currentEntry;
    int i = currentBucket;
    final StringEntry[] t = table;
    while ((he == null) && (i > 0)) {
      he = t[--i];
    }
    currentEntry = he;
    currentBucket = i;
    if (he != null) {
      StringEntry e = currentEntry;
      currentEntry = e.next;
      if (intKeys) {
        return e.key;
      }
      return e.value;
    }
    throw new NoSuchElementException();
  }

  /**
   * Iterator's removeString method.
   */
  public void removeString() {
    throw new UnsupportedOperationException();
  }

/*^**************************************************************************^*/
/*- Overridden Methods.                                                      -*/
/*^**************************************************************************^*/

  /**
   * Compares the specified object with this map for equality.
   *
   * @param o the object to be compared for equality with this map
   *
   * @return <tt>true</tt> if the specified object is equal to this map
   */
  public boolean equals(Object o) {
    if (o == this) {
      return true;
    }
    if (!(o instanceof SimpleStringMap)) {
      return false;
    }
    SimpleStringMap sm = (SimpleStringMap)o;
    if (size != sm.size) {
      return false;
    }
    final StringEntry[] t = table;
    for (int i = t.length - 1; i >= 0; i--) {
      for (StringEntry e = t[i]; e != null; e = e.next) {
        if (!e.value.equals(sm.get(e.key))) {
          return false;
        }
      }
    }
    return true;
  }

  /**
   * Returns the hash code value for this map.
   *
   * @return the hash code value for this map
   */
  public int hashCode() {
    int hashCode = 0;
    final StringEntry[] t = table;
    for (int i = t.length - 1; i >= 0; i--) {
      for (StringEntry e = t[i]; e != null; e = e.next) {
        hashCode += e.key.hashCode() ^ e.value.hashCode();
      }
    }
    return hashCode;
  }

/*^**************************************************************************^*/
/*- Private Method.                                                          -*/
/*^**************************************************************************^*/

  private void resize(boolean enlarge) {
    int oldCapacity = table.length;
    final StringEntry[] oldTable = table;
    int newCapacity;
    if (enlarge) {
      newCapacity = ((3 * oldCapacity) >> 1) + 1;
    } else {
      newCapacity = oldCapacity / 2;
      if (newCapacity < initialCapacity) {
        newCapacity = initialCapacity;
      }
    }
    StringEntry[] newTable = new StringEntry[newCapacity];
    enlargeThreshold = (int)(newCapacity * enlargeFactor);
    shrinkThreshold = (int)(newCapacity * shrinkFactor);
    table = newTable;
    for (int i = oldCapacity - 1; i >= 0; i--) {
      StringEntry old = oldTable[i];
      while (old != null) {
        StringEntry e = old;
        old = old.next;
        int index = (e.hash & 0x7FFFFFFF) % newCapacity;
        e.next = newTable[index];
        newTable[index] = e;
      }
    }
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-12 21:20:47 Slobodan
   Initial version

*****************************************************************************^*/
