/*
 * @(#)SyncBlockingQueue.java  1.0.0 / 2008-05-07
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.util.queue;

/**
 * Synchronized Blocking Queue of Objects.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public final class SyncBlockingQueue {

/*^**************************************************************************^*/
/*- Private Fields.                                                          -*/
/*^**************************************************************************^*/

  private QueueNode front;
  private QueueNode rear;
  private final Object lock;

/*^**************************************************************************^*/
/*- Public Constructor.                                                      -*/
/*^**************************************************************************^*/

  /**
   * Constructs new SyncBlockingQueue object.
   */
  public SyncBlockingQueue() {
    lock = new Object();
  }

/*^**************************************************************************^*/
/*- Public Methods.                                                          -*/
/*^**************************************************************************^*/

  /**
   * Return <code>true</code> if the queue is empty.
   *
   * @return <code>true</code> if the queue is empty
   */
  public boolean isEmpty() {
    synchronized (lock) {
      return (front == null);
    }
  }

  /**
   * Puts object into the queue.
   *
   * @param o the object
   */
  public void put(Object o) {
    synchronized (lock) {
      QueueNode p = new QueueNode(o);
      if (rear == null) {
        front = p;
      } else {
        rear.next = p;
      }
      rear = p;
      lock.notifyAll();
    }
  }

  /**
   * Returns and removes the first object in the queue.
   * Blocks if there are no objects in the queue.
   *
   * @return the first object in this queue queue
   */
  public Object get() {
    synchronized (lock) {
      try {
        while (front == null) {
          lock.wait();
        }
      } catch (InterruptedException e) {
        return null;
      }
      QueueNode p = front;
      front = p.next;
      if (front == null) {
        rear = null;
      }
      return p.item;
    }
  }

  /**
   * Returns the first object in the queue.
   *
   * @return the first object in the queue
   */
  public Object front() {
    synchronized (lock) {
      if (front != null) {
        return front.item;
      }
      return null;
    }
  }

  /**
   * Returns the last object in the queue.
   *
   * @return the last object in the queue
   */
  public Object rear() {
    synchronized (lock) {
      if (rear != null) {
        return rear.item;
      }
      return null;
    }
  }

/*^**************************************************************************^*/
/*- Private Inner Class.                                                     -*/
/*^**************************************************************************^*/

  /**
   * Queue Node.
   */
  static final class QueueNode {
    Object item;
    QueueNode next;
    QueueNode(Object item) {
      this.item = item;
    }
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-07 19:10:52 Slobodan
   Initial version

*****************************************************************************^*/
