/*
 * @(#)SimpleSoftValueHashMap.java  1.0.0 / 2008-05-10
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.util.ref;

import java.lang.ref.Reference;
import java.lang.ref.ReferenceQueue;
import java.lang.ref.SoftReference;
import java.util.ConcurrentModificationException;
import java.util.Iterator;
import java.util.NoSuchElementException;

/**
 * Simple SoftReference Value Hash Map.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public final class SimpleSoftValueHashMap implements Iterator {

/*^**************************************************************************^*/
/*- Private Static Field.                                                    -*/
/*^**************************************************************************^*/

  private static final int MAXIMUM_CAPACITY = 1 << 30;

/*^**************************************************************************^*/
/*- Private Fields.                                                          -*/
/*^**************************************************************************^*/

  private SoftValueEntry[] table;
  private int size;
  private int initialCapacity;
  private float enlargeFactor;
  private float shrinkFactor;
  private int enlargeThreshold;
  private int shrinkThreshold;
  private int modCount;
  private int expectedModCount;
  private boolean intKeys;
  private SoftValueEntry currentEntry;
  private int currentBucket;
  private ReferenceQueue queue;

/*^**************************************************************************^*/
/*- Public Constructors.                                                     -*/
/*^**************************************************************************^*/

  /**
   * Constructs new SimpleSoftValueHashMap object.
   */
  public SimpleSoftValueHashMap() {
    this(11, 0.75f, 0.3f);
  }

  /**
   * Constructs new SimpleSoftValueHashMap object.
   *
   * @param initialCapacity initial capacity
   */
  public SimpleSoftValueHashMap(int initialCapacity) {
    this(initialCapacity, 0.75f, 0.3f);
  }

  /**
   * Constructs new SimpleSoftValueHashMap object.
   *
   * @param o object array
   */
  public SimpleSoftValueHashMap(Object[] o) {
    this((int)(o.length * 1.8), 0.75f);
    for (int i = o.length - 1; i >= 0; i--) {
      put(o[i], o[i]);
    }
  }

  /**
   * Constructs new SimpleSoftValueHashMap object.
   *
   * @param initialCapacity initial capacity
   * @param loadFactor load factor
   */
  public SimpleSoftValueHashMap(int initialCapacity, float loadFactor) {
    this(initialCapacity, loadFactor, 0.3f);
  }

  /**
   * Constructs new SimpleSoftValueHashMap object.
   *
   * @param initialCapacity initial capacity
   * @param enlargeFactor enlarge factor
   * @param shrinkFactor shrink factor
   */
  public SimpleSoftValueHashMap(int initialCapacity, float enlargeFactor,
                                float shrinkFactor) {
    if (initialCapacity <= 0) {
      throw new IllegalArgumentException("Illegal capacity"); //$NON-NLS-1$
    }
    if ((enlargeFactor <= 0) || (enlargeFactor > 1) ||
        Float.isNaN(enlargeFactor)) {
      throw new IllegalArgumentException(
                                       "Illegal enlarge factor"); //$NON-NLS-1$
    }
    if ((shrinkFactor <= 0) || (shrinkFactor > 1) ||
        Float.isNaN(shrinkFactor)) {
      throw new IllegalArgumentException("Illegal shrink factor"); //$NON-NLS-1$
    }
    if (initialCapacity > MAXIMUM_CAPACITY) {
      initialCapacity = MAXIMUM_CAPACITY;
    }
    this.initialCapacity = initialCapacity;
    this.enlargeFactor = enlargeFactor;
    this.shrinkFactor = shrinkFactor;
    queue = new ReferenceQueue();
    table = new SoftValueEntry[initialCapacity];
    enlargeThreshold = (int)(initialCapacity * enlargeFactor);
    shrinkThreshold = (int)(initialCapacity * shrinkFactor);
  }

/*^**************************************************************************^*/
/*- Public Methods.                                                          -*/
/*^**************************************************************************^*/

  /**
   * Compares the specified object with this map for equality.
   *
   * @param o the object to be compared for equality with this map
   *
   * @return <tt>true</tt> if the specified object is equal to this map
   */
  public boolean equals(Object o) {
    if (o == this) {
      return true;
    }
    if (!(o instanceof SimpleSoftValueHashMap)) {
      return false;
    }
    SimpleSoftValueHashMap om = (SimpleSoftValueHashMap)o;
    if (size != om.size) {
      return false;
    }
    final SoftValueEntry[] t = table;
    for (int i = t.length - 1; i >= 0; i--) {
      for (SoftValueEntry e = t[i]; e != null; e = e.next) {
        Object value = e.get();
        if (((value == null) && (om.get(e.key) != null)) ||
            ((value != null) && !value.equals(om.get(e.key)))) {
          return false;
        }
      }
    }
    return true;
  }

  /**
   * Returns the hash code value for this map.
   *
   * @return the hash code value for this map
   */
  public int hashCode() {
    int hashCode = 0;
    final SoftValueEntry[] t = table;
    for (int i = t.length - 1; i >= 0; i--) {
      for (SoftValueEntry e = t[i]; e != null; e = e.next) {
        Object value = e.get();
        if (value != null) {
          hashCode += e.key.hashCode() ^ value.hashCode();
        }
      }
    }
    return hashCode;
  }

  /**
   * Returns number of objects in this table.
   *
   * @return number of objects in this table
   */
  public int size() {
    if (size == 0) {
      return 0;
    }
    expungeStaleEntries();
    return size;
  }

  /**
   * Returns capacity of this map.
   *
   * @return capacity of this map
   */
  public int capacity() {
    return table.length;
  }

  /**
   * Tests if this table has no entries.
   *
   * @return <tt>true</tt> if this table has no entries;
   *         <tt>false</tt> otherwise.
   */
  public boolean empty() {
    return (size == 0);
  }

  /**
   * Returns <tt>true</tt> if this table contains specified key.
   *
   * @param key the key
   *
   * @return <tt>true</tt> if this table contains specified key
   */
  public boolean containsKey(Object key) {
    if (key != null) {
      expungeStaleEntries();
      int hash = key.hashCode();
      final SoftValueEntry[] t = table;
      int index = (hash & 0x7FFFFFFF) % t.length;
      for (SoftValueEntry e = t[index]; e != null; e = e.next) {
        if ((e.hash == hash) && e.key.equals(key)) {
          return true;
        }
      }
    }
    return false;
  }

  /**
   * Returns <tt>true</tt> if this table contains specified value.
   *
   * @param value the value
   *
   * @return <tt>true</tt> if this table contains specified value
   */
  public boolean containsValue(Object value) {
    if (value != null) {
      expungeStaleEntries();
      final SoftValueEntry[] t = table;
      for (int i = t.length - 1; i >= 0; i--) {
        for (SoftValueEntry e = t[i]; e != null; e = e.next) {
          Object wv = e.get();
          if ((wv != null) && (wv.equals(value))) {
            return true;
          }
        }
      }
    }
    return false;
  }

  /**
   * Returns the value of specified key or <tt>null</tt> if there is no such
   * key.
   *
   * @param key a key in the table.
   *
   * @return the value or <tt>null</tt> if there is no such key
   */
  public Object get(Object key) {
    if (key != null) {
      expungeStaleEntries();
      int hash = key.hashCode();
      final SoftValueEntry[] t = table;
      int index = (hash & 0x7FFFFFFF) % t.length;
      for (SoftValueEntry e = t[index]; e != null; e = e.next) {
        if ((e.hash == hash) && e.key.equals(key)) {
          return e.get();
        }
      }
    }
    return null;
  }

  /**
   * Puts new key.
   *
   * @param key the key
   * @param value the valuse
   *
   * @return old value
   */
  public Object put(Object key, Object value) {
    if (value == null) {
      throw new IllegalArgumentException("Null value"); //$NON-NLS-1$
    }
    expungeStaleEntries();
    SoftValueEntry[] t = table;
    int hash = key.hashCode();
    int index = (hash & 0x7FFFFFFF) % t.length;
    for (SoftValueEntry ce = t[index], pe = null; ce != null;
         pe = ce, ce = ce.next) {
      if ((ce.hash == hash) && ce.key.equals(key)) {
        Object old = ce.get();
        SoftValueEntry ne = new SoftValueEntry(hash, key, value, queue,
                                               ce.next);
        if (pe == null) {
          t[index] = ne;
        } else {
          pe.next = ne;
        }
        ce.key = null;
        return old;
      }
    }
    if (size >= enlargeThreshold) {
      resize(true);
      t = table;
      index = (hash & 0x7FFFFFFF) % t.length;
    }
    t[index] = new SoftValueEntry(hash, key, value, queue, t[index]);
    size++;
    modCount++;
    return null;
  }

  /**
   * Removes specified key from the table.
   *
   * @param key the key
   *
   * @return value
   */
  public Object remove(Object key) {
    final SoftValueEntry[] t = table;
    int hash = key.hashCode();
    expungeStaleEntries();
    int index = (hash & 0x7FFFFFFF) % t.length;
    for (SoftValueEntry e = t[index], prev = null; e != null;
        prev = e, e = e.next) {
      if ((e.hash == hash) && e.key.equals(key)) {
        if (prev == null) {
          t[index] = e.next;
        } else {
          prev.next = e.next;
        }
        size--;
        Object oldValue = e.get();
        e.key = null;
        if ((t.length > initialCapacity) && (size < shrinkThreshold)) {
          resize(false);
        }
        return oldValue;
      }
    }
    modCount++;
    return null;
  }

  /**
   * Clears this table.
   */
  public void clear() {
    Reference r;
    do {
      r = queue.poll();
    } while (r != null);
    final SoftValueEntry[] t = table;
    if (t.length > (2 * initialCapacity)) {
      table = new SoftValueEntry[initialCapacity];
      enlargeThreshold = (int)(initialCapacity * enlargeFactor);
      shrinkThreshold = (int)(initialCapacity * shrinkFactor);
    } else {
      for (int index = t.length; --index >= 0;) {
        t[index] = null;
      }
    }
    size = 0;
    modCount++;
    currentEntry = null;
    do {
      r = queue.poll();
    } while (r != null);
  }

  /**
   * Iterator over the keys.
   *
   * @return key iterator
   */
  public Iterator keys() {
    expungeStaleEntries();
    expectedModCount = modCount;
    currentEntry = null;
    currentBucket = table.length;
    intKeys = true;
    return this;
  }

  /**
   * Iterator over the values.
   *
   * @return value iterator
   */
  public Iterator values() {
    expungeStaleEntries();
    expectedModCount = modCount;
    currentEntry = null;
    currentBucket = table.length;
    intKeys = false;
    return this;
  }

  /**
   * Iterator hasNext method.
   *
   * @return <tt>true</tt> if the iterator has more elements
   */
  public boolean hasNext() {
    if (modCount != expectedModCount) {
      throw new ConcurrentModificationException();
    }
    SoftValueEntry he = currentEntry;
    int i = currentBucket;
    final SoftValueEntry[] t = table;
    while ((he == null) && (i > 0)) {
      he = t[--i];
    }
    currentEntry = he;
    currentBucket = i;
    return (currentEntry != null);
  }

  /**
   * Iterator next method.
   *
   * @return the next element in the iteration.
   *
   * @exception NoSuchElementException iteration has no more elements.
   */
  public Object next() {
    if (modCount != expectedModCount) {
      throw new ConcurrentModificationException();
    }
    SoftValueEntry he = currentEntry;
    int i = currentBucket;
    final SoftValueEntry[] t = table;
    while ((he == null) && (i > 0)) {
      he = t[--i];
    }
    currentEntry = he;
    currentBucket = i;
    if (he != null) {
      SoftValueEntry e = currentEntry;
      currentEntry = e.next;
      if (intKeys) {
        return e.key;
      }
      return e.get();
    }
    throw new NoSuchElementException();
  }

  /**
   * Iterator remove method.
   */
  public void remove() {
    throw new UnsupportedOperationException();
  }

/*^**************************************************************************^*/
/*- Private Methods.                                                         -*/
/*^**************************************************************************^*/

  private void resize(boolean enlarge) {
    int oldCapacity = table.length;
    SoftValueEntry[] oldTable = table;
    int newCapacity;
    if (enlarge) {
      newCapacity = (oldCapacity * 5) / 3 + 1;
    } else {
      newCapacity = oldCapacity / 2;
      if (newCapacity < initialCapacity) {
        newCapacity = initialCapacity;
      }
    }
    SoftValueEntry[] newTable = new SoftValueEntry[newCapacity];
    enlargeThreshold = (int)(newCapacity * enlargeFactor);
    shrinkThreshold = (int)(newCapacity * shrinkFactor);
    table = newTable;
    for (int i = oldCapacity - 1; i >= 0; i--) {
      SoftValueEntry old = oldTable[i];
      while (old != null) {
        SoftValueEntry e = old;
        old = old.next;
        int index = (e.hash & 0x7FFFFFFF) % newCapacity;
        e.next = newTable[index];
        newTable[index] = e;
      }
    }
  }

  private void expungeStaleEntries() {
    Object r;
    SoftValueEntry[] t = table;
    while ((r = queue.poll()) != null) {
      SoftValueEntry e = (SoftValueEntry)r;
      int h = e.hash;
      int i = (h & 0x7FFFFFFF) % t.length;
      SoftValueEntry prev = t[i];
      SoftValueEntry p = prev;
      while (p != null) {
        SoftValueEntry next = p.next;
        if (p == e) { // NOPMD
          if (prev == e) { // NOPMD
            t[i] = next;
          } else {
            prev.next = next;
          }
          e.next = null;
          e.key = null;
          size--;
          break;
        }
        prev = p;
        p = next;
      }
    }
  }

/*^**************************************************************************^*/
/*- Private Inner Class.                                                     -*/
/*^**************************************************************************^*/

  /**
   * Soft Value Entry.
   */
  static final class SoftValueEntry extends SoftReference {
    int hash;
    Object key;
    SoftValueEntry next;
    SoftValueEntry(int hash, Object key, Object value, ReferenceQueue queue,
                   SoftValueEntry next) {
      super(value, queue);
      this.hash = hash;
      this.key = key;
      this.next = next;
    }
  }

}
