/*
 * @(#)SimpleWeakArrayList.java  1.0.0 / 2008-05-10
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.util.ref;

import java.lang.ref.WeakReference;
import java.util.ConcurrentModificationException;
import java.util.Iterator;
import java.util.NoSuchElementException;

/**
 * Simple List of WeakReferences.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public final class SimpleWeakArrayList implements Iterator {

/*^**************************************************************************^*/
/*- Private Static Field.                                                    -*/
/*^**************************************************************************^*/

  private static final int MAXIMUM_CAPACITY = 1 << 30;

/*^**************************************************************************^*/
/*- Private Fields.                                                          -*/
/*^**************************************************************************^*/

  private WeakReference[] references;
  private int size;
  private int initialCapacity;
  private int modCount;
  private int expectedModCount;
  private int cursor;
  private Object nextObject;

/*^**************************************************************************^*/
/*- Public Constructors.                                                     -*/
/*^**************************************************************************^*/

  /**
   * Constructs an empty list with an initial capacity of ten.
   */
  public SimpleWeakArrayList() {
    this(10);
  }

  /**
   * Constructs an empty list with the specified initial capacity.
   *
   * @param initialCapacity the initial capacity of the list.
   *
   * @throws IllegalArgumentException if the specified initial capacity is
   *         negative
   */
  public SimpleWeakArrayList(int initialCapacity) {
    if (initialCapacity <= 0) {
      throw new IllegalArgumentException("Illegal capacity"); //$NON-NLS-1$
    }
    if (initialCapacity > MAXIMUM_CAPACITY) {
      initialCapacity = MAXIMUM_CAPACITY;
    }
    this.initialCapacity = initialCapacity;
    references = new WeakReference[initialCapacity];
  }

/*^**************************************************************************^*/
/*- Public Methods.                                                          -*/
/*^**************************************************************************^*/

  /**
   * Expunges stale references.
   */
  public void expungeStaleReferences() {
    int d = 0;
    final WeakReference[] refs = references;
    for (int i = 0; i < size; i++) {
      if ((refs[i] == null) || (refs[i].get() == null)) {
        d++;
      } else {
        if (d > 0) {
          refs[i - d] = refs[i];
        }
      }
    }
    if (d > 0) {
      for (int i = size - d; i < size; i++) {
        refs[i] = null;
      }
      size -= d;
      modCount++;
    }
  }

  /**
   * Increases the capacity of this <tt>WeakArrayList</tt> instance, if
   * necessary, to ensure  that it can hold at least the number of elements
   * specified by the minimum capacity argument.
   *
   * @param minCapacity the desired minimum capacity.
   */
  public void ensureCapacity(int minCapacity) {
    modCount++;
    int oldCapacity = references.length;
    if (minCapacity > oldCapacity) {
      final WeakReference[] oldReferences = references;
      int newCapacity = ((3 * oldCapacity) >> 1) + 1;
      if (newCapacity < minCapacity) {
        newCapacity = minCapacity;
      }
      WeakReference[] newReferences = new WeakReference[newCapacity];
      int d = 0;
      for (int i = 0; i < size; i++) {
        if ((oldReferences[i] == null) || (oldReferences[i].get() == null)) {
          d++;
        } else {
          newReferences[i - d] = oldReferences[i];
        }
      }
      references = newReferences;
      if (d > 0) {
        size -= d;
      }
    }
  }

  /**
   * Trims the capacity of this <tt>WeakArrayList</tt> instance to be the
   * list's current size.  An application can use this operation to minimize
   * the storage of an <tt>WeakArrayList</tt> instance.
   */
  public void trimToSize() {
    modCount++;
    int d = 0;
    final WeakReference[] refs = references;
    for (int i = 0; i < size; i++) {
      if ((refs[i] == null) || (refs[i].get() == null)) {
        d++;
      } else {
        if (d > 0) {
          refs[i - d] = refs[i];
        }
      }
    }
    if (d > 0) {
      size -= d;
    }
    if (size < refs.length) {
      references = new WeakReference[size];
      System.arraycopy(refs, 0, references, 0, size);
    }
  }

  /**
   * Returns the number of elements in this list.
   *
   * @return the number of elements in this list
   */
  public int size() {
    if (size == 0) {
      return 0;
    }
    expungeStaleReferences();
    return size;
  }

  /**
   * Returns capacity of this list.
   *
   * @return capacity of this list
   */
  public int capacity() {
    expungeStaleReferences();
    return references.length;
  }

  /**
   * Returns <tt>true</tt> if this list contains no elements.
   *
   * @return <tt>true</tt> if this list contains no elements.
   */
  public boolean isEmpty() {
    expungeStaleReferences();
    return size == 0;
  }

  /**
   * Returns <tt>true</tt> if this list contains the specified element.
   *
   * @param o element whose presence in this list is to be tested
   *
   * @return <tt>true</tt> if this list contains the specified element
   */
  public boolean contains(Object o) {
    return indexOf(o) >= 0;
  }

  /**
   * Returns the index in this list of the first occurrence of the specified
   * element, or -1 if this list does not contain this element.
   *
   * @param o element to search for
   *
   * @return the index in this list of the first occurrence of the specified
   *         element, or -1 if this list does not contain this element
   */
  public int indexOf(Object o) {
    if (o == null) {
      return -1;
    }
    int d = 0;
    int ind = -1;
    Object ro = null;
    final WeakReference[] refs = references;
    for (int i = 0; i < size; i++) {
      if (refs[i] == null) {
        ro = null;
      } else {
        ro = refs[i].get();
      }
      if (ro == null) {
        d++;
      } else {
        if ((ind < 0) && ro.equals(o)) {
          ind = i;
        }
        if (d > 0) {
          refs[i - d] = refs[i];
        }
      }
    }
    if (d > 0) {
      for (int i = size - d; i < size; i++) {
        refs[i] = null;
      }
      size -= d;
      modCount++;
    }
    return ind;
  }

  /**
   * Returns the index in this list of the last occurrence of the specified
   * element, or -1 if this list does not contain this element.
   *
   * @param o element to search for
   *
   * @return the index in this list of the last occurrence of the specified
   *         element, or -1 if this list does not contain this element
   */
  public int lastIndexOf(Object o) {
    if (o == null) {
      return -1;
    }
    int d = 0;
    int ind = -1;
    Object ro = null;
    final WeakReference[] refs = references;
    for (int i = 0; i < size; i++) {
      if (refs[i] == null) {
        ro = null;
      } else {
        ro = refs[i].get();
      }
      if (ro == null) {
        d++;
      } else {
        ind = i;
        if (d > 0) {
          refs[i - d] = refs[i];
        }
      }
    }
    if (d > 0) {
      for (int i = size - d; i < size; i++) {
        refs[i] = null;
      }
      size -= d;
      modCount++;
    }
    return ind;
  }

  /**
   * Returns the element at the specified position in this list.
   *
   * @param index index of element to return
   *
   * @return the element at the specified position in this list
   *
   * @throws IndexOutOfBoundsException if index is out of range
   *         <tt>(index &lt; 0 || index &gt; size())</tt>
   */
  public Object get(int index) {
    if ((index < 0) || (index >= size)) {
      throwIndexException(index);
    }
    return references[index].get();
  }

  /**
   * Returns an array containing all of the elements in this list in proper
   * sequence.
   *
   * @return an array containing all of the elements in this list in proper
   *         sequence.
   */
  public Object[] toArray() {
    expungeStaleReferences();
    Object[] result = new Object[size];
    final WeakReference[] refs = references;
    Object o = null;
    int ri = 0;
    int i = 0;
    for (; i < size; i++) {
      if (refs[i] != null) {
        o = refs[i].get();
        if (o != null) {
          result[ri++] = o;
        }
      }
    }
    if (ri != i) {
      Object[] tresult = new Object[ri];
      System.arraycopy(result, 0, tresult, 0, ri);
      return tresult;
    }
    return result;
  }

  /**
   * Appends the specified element to the end of this list (optional operation).
   *
   * @param o element to be appended to this list
   *
   * @return <tt>true</tt>
   *
   * @throws IllegalArgumentException if element is null
   */
  public boolean add(Object o) {
    if (o == null) {
      throw new IllegalArgumentException("Null element"); //$NON-NLS-1$
    }
    ensureCapacity(size + 1);
    references[size++] = new WeakReference(o);
    return true;
  }

  /**
   * Replaces the element at the specified position in this list with the
   * specified element.
   *
   * @param index index of element to replace
   * @param element element to be stored at the specified position
   *
   * @return the element previously at the specified position
   *
   * @throws IllegalArgumentException if element is null
   * @throws IndexOutOfBoundsException if index is out of range
   *         <tt>(index &lt; 0 || index &gt; size())</tt>
   */
  public Object set(int index, Object element) {
    if (element == null) {
      throw new IllegalArgumentException("Null element"); //$NON-NLS-1$
    }
    if ((index < 0) || (index >= size)) {
      throwIndexException(index);
    }
    WeakReference oldReference = references[index];
    references[index] = new WeakReference(element);
    return oldReference.get();
  }

  /**
   * Inserts the specified element at the specified position in this list
   * (optional operation).
   *
   * @param index index at which the specified element is to be inserted
   * @param element element to be inserted
   *
   * @throws IllegalArgumentException if element is null
   * @throws IndexOutOfBoundsException if index is out of range
   *         <tt>(index &lt; 0 || index &gt; size())</tt>
   */
  public void insert(int index, Object element) {
    if (element == null) {
      throw new IllegalArgumentException("Null element"); //$NON-NLS-1$
    }
    if ((index < 0) || (index >= size)) {
      throwIndexException(index);
    }
    ensureCapacity(size + 1);
    System.arraycopy(references, index, references, index + 1, size - index);
    references[index] = new WeakReference(element);
    size++;
    modCount++;
  }

  /**
   * Removes the element at the specified position in this list.
   *
   * @param index the index of the element to removed
   *
   * @return the element previously at the specified position.
   *
   * @throws IndexOutOfBoundsException if index is out of range
   *         <tt>(index &lt; 0 || index &gt; size())</tt>
   */
  public Object remove(int index) {
    if ((index < 0) || (index > size)) {
      throwIndexException(index);
    }
    WeakReference oldReference = references[index];
    int numMoved = size - index - 1;
    if (numMoved > 0) {
      System.arraycopy(references, index + 1, references, index, numMoved);
    }
    references[--size] = null;
    modCount++;
    return oldReference.get();
  }

  /**
   * Removes the first occurrence in this list of the specified element.
   *
   * @param o element to be removed from this list, if present
   *
   * @return <tt>true</tt> if this list contained the specified element
   */
  public boolean remove(Object o) {
    if (o == null) {
      return false;
    }
    final WeakReference[] refs = references;
    int d = 0;
    boolean retv = false;
    Object ro = null;
    for (int i = 0; i < size; i++) {
      if (refs[i] == null) {
        ro = null;
      } else {
        ro = refs[i].get();
      }
      if (ro == null) {
        d++;
      } else {
        if ((!retv) && ro.equals(o)) {
          d++;
          retv = true;
          continue;
        }
        if (d > 0) {
          refs[i - d] = refs[i];
        }
      }
    }
    if (retv) {
      for (int i = size - d; i < size; i++) {
        refs[i] = null;
      }
      size -= d;
      modCount++;
    }
    return retv;
  }

  /**
   * Removes all of the elements from this list (optional operation).
   */
  public void clear() {
    if (references.length > (2 * initialCapacity)) {
      references = new WeakReference[initialCapacity];
    } else {
      final WeakReference[] refs = references;
      for (int i = size - 1; i >= 0; i--) {
        refs[i] = null;
      }
    }
    size = 0;
    modCount++;
  }

  /**
   * Returns an iterator over the elements in this list in proper sequence.
   *
   * @return an iterator over the elements in this list in proper sequence
   */
  public Iterator iterator() {
    expungeStaleReferences();
    cursor = 0;
    expectedModCount = modCount;
    nextObject = null;
    return this;
  }

  /**
   * Iterator hasNext method.
   *
   * @return <tt>true</tt> if the iterator has more elements
   */
  public boolean hasNext() {
    if (modCount != expectedModCount) {
      throw new ConcurrentModificationException();
    }
    while (cursor != size) {
      Object o = references[cursor++].get();
      if (o !=  null) {
        nextObject = o;
        return true;
      }
    }
    if (nextObject != null) {
      nextObject = null;
    }
    return false;
  }

  /**
   * Iterator next method.
   *
   * @return the next element in the iteration
   *
   * @exception NoSuchElementException iteration has no more elements
   */
  public Object next() {
    if (modCount != expectedModCount) {
      throw new ConcurrentModificationException();
    }
    if (nextObject !=  null) {
      Object o = nextObject;
      nextObject = null;
      return o;
    }
    while (cursor != size) {
      Object o = references[cursor++].get();
      if (o !=  null) {
        return o;
      }
    }
    throw new NoSuchElementException();
  }

  /**
   * Iterator remove method.
   */
  public void remove() {
    throw new UnsupportedOperationException();
  }

  /**
   * Compares the specified object with this list for equality.
   *
   * @param o the object to be compared for equality with this list
   *
   * @return <tt>true</tt> if the specified object is equal to this list
   */
  public boolean equals(Object o) {
    if (o == this) {
      return true;
    }
    if (!(o instanceof SimpleWeakArrayList)) {
      return false;
    }
    SimpleWeakArrayList ol = (SimpleWeakArrayList)o;
    if (size != ol.size) {
      return false;
    }
    final WeakReference[] refs = references;
    final WeakReference[] orefs = ol.references;
    for (int i = size - 1; i >= 0; i--) {
      Object e = refs[i].get();
      Object oe = orefs[i].get();
      if (((e == null) && (oe != null)) ||
          (e != null) && (!e.equals(oe))) {
        return false;
      }
    }
    return true;
  }

  /**
   * Returns the hash code value for this list.
   *
   * @return the hash code value for this list
   */
  public int hashCode() {
    int hashCode = 1;
    WeakReference[] refs = references;
    for (int i = size - 1; i >= 0; i--) {
      if (refs[i] != null) {
        Object o = refs[i].get();
        if (o != null) {
          hashCode = 31 * hashCode + o.hashCode();
        }
      }
    }
    return hashCode;
  }

/*^**************************************************************************^*/
/*- Private Methods.                                                         -*/
/*^**************************************************************************^*/

  private void throwIndexException(int index) {
    throw new IndexOutOfBoundsException("Index: " + index + //$NON-NLS-1$
                                        ", Size: " + size); //$NON-NLS-1$
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-10 22:08:31 Slobodan
   Initial version

*****************************************************************************^*/
