/*
 * @(#)SoftHashMap.java  1.0.0 / 2008-05-09
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.util.ref;

import java.lang.ref.Reference;
import java.lang.ref.ReferenceQueue;
import java.lang.ref.SoftReference;
import java.util.AbstractCollection;
import java.util.AbstractMap;
import java.util.AbstractSet;
import java.util.ArrayList;
import java.util.Collection;
import java.util.ConcurrentModificationException;
import java.util.Iterator;
import java.util.Map;
import java.util.NoSuchElementException;
import java.util.Set;

/**
 * Simple hash map of soft references.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public final class SoftHashMap extends AbstractMap {

/*^**************************************************************************^*/
/*- Private Static Fields.                                                   -*/
/*^**************************************************************************^*/

  private static final int DEFAULT_INITIAL_CAPACITY = 16;
  private static final int MAXIMUM_CAPACITY = 1 << 30;
  private static final float DEFAULT_LOAD_FACTOR = 0.75f;
  private static final Object NULL_KEY = new Object();

/*^**************************************************************************^*/
/*- Private Fields.                                                          -*/
/*^**************************************************************************^*/

  private int size;
  private int threshold;
  private float loadFactor;
  private ReferenceQueue queue;

/*^**************************************************************************^*/
/*- Package-Private Fields.                                                  -*/
/*^**************************************************************************^*/

  Entry[] table;
  volatile int modCount;

/*^**************************************************************************^*/
/*- Public Constructor.                                                      -*/
/*^**************************************************************************^*/

  /**
   * Constructs a new SoftHashMap object.
   */
  public SoftHashMap() {
    super();
    queue = new ReferenceQueue();
    this.loadFactor = DEFAULT_LOAD_FACTOR;
    threshold = DEFAULT_INITIAL_CAPACITY;
    table = new Entry[DEFAULT_INITIAL_CAPACITY];
  }

  /**
   * Constructs a new SoftHashMap object.
   *
   * @param initialCapacity the initial capacity
   *
   * @throws IllegalArgumentException  if the initial capacity is negative,
   *                                   or if the load factor is nonpositive.
   */
  public SoftHashMap(int initialCapacity) {
    this(initialCapacity, DEFAULT_LOAD_FACTOR);
  }

  /**
   * Constructs a new SoftHashMap object.
   *
   * @param initialCapacity the initial capacity
   * @param loadFactor the load factor
   *
   * @throws IllegalArgumentException if the initial capacity is invalid,
   *                                  or if the load factor is invalid.
   */
  public SoftHashMap(int initialCapacity, float loadFactor) {
    super();
    if (initialCapacity <= 0) {
      throw new IllegalArgumentException("Illegal capacity:"); //$NON-NLS-1$
    }
    if (initialCapacity > MAXIMUM_CAPACITY) {
      initialCapacity = MAXIMUM_CAPACITY;
    }
    if ((loadFactor <= 0) || Float.isNaN(loadFactor)) {
      throw new IllegalArgumentException("Illegal load factor"); //$NON-NLS-1$
    }
    queue = new ReferenceQueue();
    int capacity = 1;
    while (capacity < initialCapacity) {
      capacity <<= 1;
    }
    table = new Entry[capacity];
    this.loadFactor = loadFactor;
    threshold = (int)(capacity * loadFactor);
  }

  /**
   * Constructs a new <tt>SoftHashMap</tt> with the same mappings as the
   * specified <tt>Map</tt>.
   *
   * @param t the map whose mappings are to be placed in this map
   */
  public SoftHashMap(Map t) {
    this(Math.max((int)(t.size() / DEFAULT_LOAD_FACTOR) + 1, 16),
         DEFAULT_LOAD_FACTOR);
    putAll(t);
  }

/*^**************************************************************************^*/
/*- Public Methods.                                                          -*/
/*^**************************************************************************^*/

  /**
   * Returns the number of key-value mappings in this map.
   *
   * @return the number of key-value mappings in this map
   */
  public int size() {
    if (size == 0) {
      return 0;
    }
    expungeStaleEntries();
    return size;
  }

  /**
   * Returns <tt>true</tt> if this map contains no key-value mappings.
   *
   * @return <tt>true</tt> if this map contains no key-value mappings
   */
  public boolean isEmpty() {
    return (size() == 0);
  }

  /**
   * Returns the value to which the specified key is mapped in this weak
   * hash map, or <tt>null</tt> if the map contains no mapping for this key.
   *
   * @param key the key whose associated value is to be returned
   *
   * @return the value to which this map maps the specified key,
   *         or <tt>null</tt> if the map contains no mapping for this key
   */
  public Object get(Object key) {
    Object k = maskNull(key);
    int h = hash(k);
    final Entry[] tab = getTable();
    int index = indexFor(h, tab.length);
    Entry e = tab[index];
    while (e != null) {
      Object r = e.get();
      if ((e.hash == h) && ((k == r) || (k.equals(r)))) { // NOPMD
        return e.value;
      }
      e = e.next;
    }
    return null;
  }

  /**
   * Returns <tt>true</tt> if this map contains a mapping for the specified key.
   *
   * @param key The key whose presence in this map is to be tested
   * @return <tt>true</tt> if there is a mapping for <tt>key</tt>;
   *         <tt>false</tt> otherwise
   */
  public boolean containsKey(Object key) {
    return getEntry(key) != null;
  }

  /**
   * Associates the specified value with the specified key in this map.
   *
   * @param key key with which the specified value is to be associated
   * @param value value to be associated with the specified key
   *
   * @return previous value associated with specified key, or <tt>null</tt>
   *         if there was no mapping for key
   */
  public Object put(Object key, Object value) {
    Object k = maskNull(key);
    int h = hash(k);
    final Entry[] tab = getTable();
    int i = indexFor(h, tab.length);
    for (Entry e = tab[i]; e != null; e = e.next) {
      Object r = e.get();
      if ((h == e.hash) && ((k == r) || (k.equals(r)))) { // NOPMD
        Object oldValue = e.value;
        if (value != oldValue) { // NOPMD
          e.value = value;
        }
        return oldValue;
      }
    }
    modCount++;
    tab[i] = new Entry(k, value, queue, h, tab[i]);
    if (++size >= threshold) {
      resize(tab.length * 2);
    }
    return null;
  }

  /**
   * Copies all of the mappings from the specified map to this map.
   *
   * @param m mappings to be stored in this map.
   *
   * @throws NullPointerException if the specified map is null.
   */
  public void putAll(Map m) {
    int numKeysToBeAdded = m.size();
    if (numKeysToBeAdded == 0) {
      return;
    }
    if (numKeysToBeAdded > threshold) {
      int targetCapacity = (int)(numKeysToBeAdded / loadFactor + 1);
      if (targetCapacity > MAXIMUM_CAPACITY) {
        targetCapacity = MAXIMUM_CAPACITY;
      }
      int newCapacity = table.length;
      while (newCapacity < targetCapacity) {
        newCapacity <<= 1;
      }
      if (newCapacity > table.length) {
        resize(newCapacity);
      }
    }
    for (Iterator i = m.entrySet().iterator(); i.hasNext();) {
      Map.Entry e = (Map.Entry)i.next();
      put(e.getKey(), e.getValue());
    }
  }

  /**
   * Removes the mapping for this key from this map if present.
   *
   * @param key key whose mapping is to be removed from the map.
   *
   * @return previous value associated with specified key, or <tt>null</tt>
   *         if there was no mapping for key.
   */
  public Object remove(Object key) {
    Object k = maskNull(key);
    int h = hash(k);
    final Entry[] tab = getTable();
    int i = indexFor(h, tab.length);
    Entry prev = tab[i];
    Entry e = prev;
    while (e != null) {
      Entry next = e.next;
      Object r = e.get();
      if ((h == e.hash) && ((k == r) || (k.equals(r)))) { // NOPMD
        modCount++;
        size--;
        if (prev == e) { // NOPMD
          tab[i] = next;
        } else {
          prev.next = next;
        }
        return e.value;
      }
      prev = e;
      e = next;
    }

    return null;
  }

  /**
   * Removes all mappings from this map.
   */
  public void clear() {
    Reference r;
    do {
      r = queue.poll();
    } while (r != null);
    modCount++;
    final Entry[] tab = table;
    for (int i = 0; i < tab.length; ++i) {
      tab[i] = null;
    }
    size = 0;
    do {
      r = queue.poll();
    } while (r != null);
  }

  /**
   * Returns <tt>true</tt> if this map maps one or more keys to the specified
   * value.
   *
   * @param value value whose presence in this map is to be tested
   *
   * @return <tt>true</tt> if this map maps one or more keys to the specified
   *                       value.
   */
  public boolean containsValue(Object value) {
    if (value == null) {
      return containsNullValue();
    }
    final Entry[] tab = getTable();
    for (int i = tab.length; i-- > 0;) {
      for (Entry e = tab[i]; e != null; e = e.next) {
        if (value.equals(e.value)) {
          return true;
        }
      }
    }
    return false;
  }

  /**
   * Returns a set view of the keys contained in this map.
   *
   * @return a set view of the keys contained in this map.
   */
  public Set keySet() {
    return new KeySet();
  }

  /**
   * Returns a collection view of the values contained in this map..
   *
   * @return a collection view of the values contained in this map.
   */
  public Collection values() {
    return new Values();
  }

  /**
   * Returns a collection view of the mappings contained in this map..
   *
   * @return a collection view of the mappings contained in this map.
   */
  public Set entrySet() {
    return new EntrySet();
  }

/*^**************************************************************************^*/
/*- Package-Private Methods.                                                 -*/
/*^**************************************************************************^*/

  Entry[] getTable() {
    expungeStaleEntries();
    return table; // NOPMD
  }

  Entry getEntry(Object key) {
    // CHANGE: Clean
    Object k = maskNull(key);
    int h = hash(k);
    final Entry[] tab = getTable();
    int index = indexFor(h, tab.length);
    Entry e = tab[index];
    if (e != null) {
      Object r = e.get();
      while (!((e.hash == h) && ((k == r) || (k.equals(r))))) { // NOPMD
        e = e.next;
        if (e == null) {
          break;
        }
        r = e.get();
      }
    }
    return e;
  }

  boolean containsNullValue() {
    final Entry[] tab = getTable();
    for (int i = tab.length; i-- > 0;) {
      for (Entry e = tab[i]; e != null; e = e.next) {
        if (e.value == null) {
          return true;
        }
      }
    }
    return false;
  }

  /**
   * Expunges stale entries from the table.
   */
  void expungeStaleEntries() {
    Object r;
    while ((r = queue.poll()) != null) {
      Entry e = (Entry)r;
      int h = e.hash;
      int i = indexFor(h, table.length);
      Entry prev = table[i];
      Entry p = prev;
      while (p != null) {
        Entry next = p.next;
        if (p == e) { // NOPMD
          if (prev == e) { // NOPMD
            table[i] = next;
          } else {
            prev.next = next;
          }
          e.next = null;
          e.value = null;
          size--;
          break;
        }
        prev = p;
        p = next;
      }
    }
  }

  Entry removeMapping(Object o) {
    if (!(o instanceof Map.Entry)) {
      return null;
    }
    final Entry[] tab = getTable();
    Map.Entry entry = (Map.Entry)o;
    Object k = maskNull(entry.getKey());
    int h = hash(k);
    int i = indexFor(h, tab.length);
    Entry prev = tab[i];
    Entry e = prev;
    while (e != null) {
      Entry next = e.next;
      if ((h == e.hash) && e.equals(entry)) {
        modCount++;
        size--;
        if (prev == e) { // NOPMD
          tab[i] = next;
        } else {
          prev.next = next;
        }
        return e;
      }
      prev = e;
      e = next;
    }
    return null;
  }

  void resize(int newCapacity) {
    final Entry[] oldTable = getTable();
    int oldCapacity = oldTable.length;
    if (oldCapacity == MAXIMUM_CAPACITY) {
      threshold = Integer.MAX_VALUE;
      return;
    }
    Entry[] newTable = new Entry[newCapacity];
    transfer(oldTable, newTable);
    table = newTable;
    if (size >= threshold / 2) {
      threshold = (int)(newCapacity * loadFactor);
    } else {
      expungeStaleEntries();
      transfer(newTable, oldTable);
      table = oldTable;
    }
  }

  void transfer(Entry[] src, Entry[] dest) {
    for (int j = 0; j < src.length; ++j) {
      Entry e = src[j];
      src[j] = null;
      while (e != null) {
        Entry next = e.next;
        Object key = e.get();
        if (key == null) {
          e.next = null;
          e.value = null;
          size--;
        } else {
          int i = indexFor(e.hash, dest.length);
          e.next = dest[i];
          dest[i] = e;
        }
        e = next;
      }
    }
  }

/*^**************************************************************************^*/
/*- Package-Private Static Methods.                                          -*/
/*^**************************************************************************^*/

  static int hash(Object x) {
    int h = x.hashCode();
    h += ~(h << 9);
    h ^= (h >>> 14);
    h += (h << 4);
    h ^= (h >>> 10);
    return h;
  }

  static Object maskNull(Object key) {
    if (key == NULL_KEY) { // NOPMD
      return NULL_KEY;
    }
    return key;
  }

  static Object unmaskNull(Object key) {
    if (key == NULL_KEY) { // NOPMD
      return null;
    }
    return key;
  }

  static int indexFor(int h, int length) {
    return h & (length - 1);
  }

/*^**************************************************************************^*/
/*- Package-Private Inner Classes.                                           -*/
/*^**************************************************************************^*/

  /**
   * Entry Class.
   */
  static class Entry extends SoftReference implements Map.Entry {

    Object value;
    final int hash;
    Entry next;

    Entry(Object key, Object value, ReferenceQueue queue, int hash,
          Entry next) {
      super(key, queue);
      this.value = value;
      this.hash = hash;
      this.next = next;
    }

    /**
     * {@inheritDoc}
     */
    public Object getKey() {
      return unmaskNull(get());
    }

    /**
     * {@inheritDoc}
     */
    public Object getValue() {
      return value;
    }

    /**
     * {@inheritDoc}
     */
    public Object setValue(Object newValue) {
      Object oldValue = value;
      value = newValue;
      return oldValue;
    }

    /**
     * {@inheritDoc}
     */
    public boolean equals(Object o) {
      if (!(o instanceof Map.Entry)) {
        return false;
      }
      Map.Entry e = (Map.Entry)o;
      Object k1 = getKey();
      Object k2 = e.getKey();
      if ((k1 == k2) || ((k1 != null) && k1.equals(k2))) { // NOPMD
        Object v1 = getValue();
        Object v2 = e.getValue();
        if ((v1 == v2) || ((v1 != null) && v1.equals(v2))) { // NOPMD
          return true;
        }
      }
      return false;
    }

    /**
     * {@inheritDoc}
     */
    public int hashCode() {
      Object k = getKey();
      Object v = getValue();
      int hashCode = 0;
      if (k != null) {
        hashCode = k.hashCode();
      }
      if (v != null) {
        hashCode = hashCode ^ v.hashCode();
      }
      return hashCode;
    }

    /**
     * {@inheritDoc}
     */
    public String toString() {
      StringBuffer sb = new StringBuffer();
      sb.append(getKey());
      sb.append('=');
      sb.append(getValue());
      return sb.toString();
    }

  }

  /**
   * EntrySet Class.
   */
  class EntrySet extends AbstractSet {

    /**
     * {@inheritDoc}
     */
    public Iterator iterator() {
      return new EntryIterator();
    }

    /**
     * {@inheritDoc}
     */
    public boolean contains(Object o) {
      if (!(o instanceof Map.Entry)) {
        return false;
      }
      Map.Entry e = (Map.Entry)o;
      Entry candidate = getEntry(e.getKey());
      return ((candidate != null) && candidate.equals(e));
    }

    /**
     * {@inheritDoc}
     */
    public boolean remove(Object o) {
      return (removeMapping(o) != null);
    }

    /**
     * {@inheritDoc}
     */
    public int size() {
      return SoftHashMap.this.size();
    }

    /**
     * {@inheritDoc}
     */
    public void clear() {
      SoftHashMap.this.clear();
    }

  }

  /**
   * KeySet Class.
   */
  class KeySet extends AbstractSet {

    /**
     * {@inheritDoc}
     */
    public Iterator iterator() {
      return new KeyIterator();
    }

    /**
     * {@inheritDoc}
     */
    public int size() {
      return SoftHashMap.this.size();
    }

    /**
     * {@inheritDoc}
     */
    public boolean contains(Object o) {
      return containsKey(o);
    }

    /**
     * {@inheritDoc}
     */
    public boolean remove(Object o) {
      if (containsKey(o)) {
        SoftHashMap.this.remove(o);
        return true;
      }
      return false;
    }

    /**
     * {@inheritDoc}
     */
    public void clear() {
      SoftHashMap.this.clear();
    }

    /**
     * {@inheritDoc}
     */
    public Object[] toArray() {
      Collection c = new ArrayList(size());
      for (Iterator i = iterator(); i.hasNext();) {
        c.add(i.next());
      }
      return c.toArray();
    }

    /**
     * {@inheritDoc}
     */
    public Object[] toArray(Object[] a) {
      Collection c = new ArrayList(size());
      for (Iterator i = iterator(); i.hasNext();) {
        c.add(i.next());
      }
      return c.toArray(a);
    }

  }

  /**
   * Values Class.
   */
  class Values extends AbstractCollection {

    /**
     * {@inheritDoc}
     */
    public Iterator iterator() {
      return new ValueIterator();
    }

    /**
     * {@inheritDoc}
     */
    public int size() {
      return SoftHashMap.this.size();
    }

    /**
     * {@inheritDoc}
     */
    public boolean contains(Object o) {
      return containsValue(o);
    }

    /**
     * {@inheritDoc}
     */
    public void clear() {
      SoftHashMap.this.clear();
    }

    /**
     * {@inheritDoc}
     */
    public Object[] toArray() {
      Collection c = new ArrayList(size());
      for (Iterator i = iterator(); i.hasNext();) {
        c.add(i.next());
      }
      return c.toArray();
    }

    /**
     * {@inheritDoc}
     */
    public Object[] toArray(Object[] a) {
      Collection c = new ArrayList(size());
      for (Iterator i = iterator(); i.hasNext();) {
        c.add(i.next());
      }
      return c.toArray(a);
    }

  }

  /**
   * Hash Iterator Class.
   */
  abstract class AbstractHashIterator implements Iterator {

    int index;
    Entry entry = null;
    Entry lastReturned = null;
    int expectedModCount = modCount;
    Object nextKey = null;
    Object currentKey = null;

    AbstractHashIterator() {
      if (size() != 0) {
        index = table.length;
      }
    }

    /**
     * {@inheritDoc}
     */
    public boolean hasNext() {
      final Entry[] t = table;
      while (nextKey == null) {
        Entry e = entry;
        int i = index;
        while (e == null && i > 0) {
          e = t[--i];
        }
        entry = e;
        index = i;
        if (e == null) {
          currentKey = null;
          return false;
        }
        nextKey = e.get();
        if (nextKey == null) {
          entry = entry.next;
        }
      }
      return true;
    }

    /**
     * {@inheritDoc}
     */
    public void remove() {
      if (lastReturned == null) {
        throw new IllegalStateException();
      }
      if (modCount != expectedModCount) {
        throw new ConcurrentModificationException();
      }
      SoftHashMap.this.remove(currentKey);
      expectedModCount = modCount;
      lastReturned = null;
      currentKey = null;
    }

    /**
     * Returns next entry.
     *
     * @return next entry
     */
    protected Entry nextEntry() {
      if (modCount != expectedModCount) {
        throw new ConcurrentModificationException();
      }
      if ((nextKey == null) && !hasNext()) {
        throw new NoSuchElementException();
      }
      lastReturned = entry;
      entry = entry.next;
      currentKey = nextKey;
      nextKey = null;
      return lastReturned;
    }

  }

  /**
   * Value Iterator Class.
   */
  class ValueIterator extends AbstractHashIterator {

    /**
     * {@inheritDoc}
     */
    public Object next() {
      return nextEntry().value;
    }

  }

  /**
   * Key Iterator Class.
   */
  class KeyIterator extends AbstractHashIterator {

    /**
     * {@inheritDoc}
     */
    public Object next() {
      return nextEntry().getKey();
    }

  }

  /**
   * Entry Iterator Class.
   */
  class EntryIterator extends AbstractHashIterator {

    /**
     * {@inheritDoc}
     */
    public Object next() {
      return nextEntry();
    }

  }

}
