/*
 * @(#)WeakArrayList.java  1.0.0 / 2008-05-09
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.util.ref;

import java.lang.ref.WeakReference;
import java.lang.reflect.Array;
import java.util.Collection;
import java.util.ConcurrentModificationException;
import java.util.Iterator;
import java.util.List;
import java.util.ListIterator;
import java.util.NoSuchElementException;
import java.util.RandomAccess;

/**
 * Simple array list of weak references.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public final class WeakArrayList implements List, RandomAccess {

/*^**************************************************************************^*/
/*- Private Static Field.                                                    -*/
/*^**************************************************************************^*/

  private static final int MAXIMUM_CAPACITY = 1 << 30;

/*^**************************************************************************^*/
/*- Private Fields.                                                          -*/
/*^**************************************************************************^*/

  private int initialCapacity;

/*^**************************************************************************^*/
/*- Package-Private Fields.                                                  -*/
/*^**************************************************************************^*/

  WeakReference[] references;
  int size;
  int modCount;

/*^**************************************************************************^*/
/*- Public Constructors.                                                     -*/
/*^**************************************************************************^*/

  /**
   * Constructs an empty list with an initial capacity of ten.
   */
  public WeakArrayList() {
    this(10);
  }

  /**
   * Constructs an empty list with the specified initial capacity.
   *
   * @param initialCapacity the initial capacity of the list.
   *
   * @throws IllegalArgumentException if the specified initial capacity is
   *         negative
   */
  public WeakArrayList(int initialCapacity) {
    if (initialCapacity <= 0) {
      throw new IllegalArgumentException("Illegal capacity"); //$NON-NLS-1$
    }
    if (initialCapacity > MAXIMUM_CAPACITY) {
      initialCapacity = MAXIMUM_CAPACITY;
    }
    this.initialCapacity = initialCapacity;
    references = new WeakReference[initialCapacity];
  }

  /**
   * Constructs a list containing the elements of the specified collection.
   *
   * @param c the collection whose elements are to be placed into this list
   *
   * @throws NullPointerException if the specified collection is null
   */
  public WeakArrayList(Collection c) {
    if (c == null) {
      throw new IllegalArgumentException("Null collection"); //$NON-NLS-1$
    }
    int csize = c.size();
    if (csize == 0) {
      references = new WeakReference[10];
    } else {
      int refLen = (int)Math.min((csize * 110L) / 100, Integer.MAX_VALUE);
      references = new WeakReference[refLen];
      Iterator i = c.iterator();
      int ci = 0;
      while (i.hasNext()) {
        references[ci++] = new WeakReference(i.next());
      }
      size = csize;
    }
  }

/*^**************************************************************************^*/
/*- Public Methods.                                                          -*/
/*^**************************************************************************^*/

  /**
   * Compares the specified object with this list for equality.
   *
   * @param o the object to be compared for equality with this list
   *
   * @return <tt>true</tt> if the specified object is equal to this list
   */
  public boolean equals(Object o) {
    if (o == this) {
      return true;
    }
    if (!(o instanceof WeakArrayList)) {
      return false;
    }
    WeakArrayList ol = (WeakArrayList)o;
    if (size != ol.size) {
      return false;
    }
    final WeakReference[] refs = references;
    final WeakReference[] orefs = ol.references;
    for (int i = size - 1; i >= 0; i--) {
      Object e = refs[i].get();
      Object oe = orefs[i].get();
      if (((e == null) && (oe != null)) ||
          (e != null) && (!e.equals(oe))) {
        return false;
      }
    }
    return true;
  }

  /**
   * Returns the hash code value for this list.
   *
   * @return the hash code value for this list
   */
  public int hashCode() {
    int hashCode = 1;
    final WeakReference[] refs = references;
    for (int i = size - 1; i >= 0; i--) {
      if (refs[i] != null) {
        Object o = refs[i].get();
        if (o != null) {
          hashCode = (31 * hashCode) + o.hashCode();
        }
      }
    }
    return hashCode;
  }

  /**
   * Expunges stale references.
   */
  public void expungeStaleReferences() {
    int d = 0;
    final WeakReference[] refs = references;
    for (int i = 0; i < size; i++) {
      if ((refs[i] == null) || (refs[i].get() == null)) {
        d++;
      } else {
        if (d > 0) {
          refs[i - d] = refs[i];
        }
      }
    }
    if (d > 0) {
      for (int i = size - d; i < size; i++) {
        refs[i] = null;
      }
      size -= d;
      modCount++;
    }
  }

  /**
   * Increases the capacity of this <tt>WeakArrayList</tt> instance, if
   * necessary, to ensure  that it can hold at least the number of elements
   * specified by the minimum capacity argument.
   *
   * @param minCapacity the desired minimum capacity.
   */
  public void ensureCapacity(int minCapacity) {
    modCount++;
    int oldCapacity = references.length;
    if (minCapacity > oldCapacity) {
      WeakReference[] oldReferences = references;
      int newCapacity = ((3 * oldCapacity) >> 1) + 1;
      if (newCapacity < minCapacity) {
        newCapacity = minCapacity;
      }
      WeakReference[] newReferences = new WeakReference[newCapacity];
      int d = 0;
      for (int i = 0; i < size; i++) {
        if ((oldReferences[i] == null) || (oldReferences[i].get() == null)) {
          d++;
        } else {
          newReferences[i - d] = oldReferences[i];
        }
      }
      references = newReferences;
      if (d > 0) {
        size -= d;
      }
    }
  }

  /**
   * Trims the capacity of this <tt>WeakArrayList</tt> instance to be the
   * list's current size.  An application can use this operation to minimize
   * the storage of an <tt>WeakArrayList</tt> instance.
   */
  public void trimToSize() {
    modCount++;
    int d = 0;
    final WeakReference[] refs = references;
    for (int i = 0; i < size; i++) {
      if ((refs[i] == null) || (refs[i].get() == null)) {
        d++;
      } else {
        if (d > 0) {
          refs[i - d] = refs[i];
        }
      }
    }
    if (d > 0) {
      size -= d;
    }
    if (size < refs.length) {
      references = new WeakReference[size];
      System.arraycopy(refs, 0, references, 0, size);
    }
  }

  /**
   * Returns the number of elements in this list.
   *
   * @return the number of elements in this list
   */
  public int size() {
    if (size == 0) {
      return 0;
    }
    expungeStaleReferences();
    return size;
  }

  /**
   * Returns capacity of this list.
   *
   * @return capacity of this list
   */
  public int capacity() {
    expungeStaleReferences();
    return references.length;
  }

  /**
   * Returns <tt>true</tt> if this list contains no elements.
   *
   * @return <tt>true</tt> if this list contains no elements.
   */
  public boolean isEmpty() {
    expungeStaleReferences();
    return size == 0;
  }

  /**
   * Returns <tt>true</tt> if this list contains the specified element.
   *
   * @param o element whose presence in this list is to be tested
   *
   * @return <tt>true</tt> if this list contains the specified element
   */
  public boolean contains(Object o) {
    return indexOf(o) >= 0;
  }

  /**
   * Returns <tt>true</tt> if this list contains all of the elements of the
   * specified collection.
   *
   * @param c collection to be checked for containment in this list
   *
   * @return <tt>true</tt> if this list contains all of the elements of the
   *         specified collection.
   */
  public boolean containsAll(Collection c) {
    Iterator e = c.iterator();
    while (e.hasNext()) {
      if (!contains(e.next())) {
        return false;
      }
    }
    return true;
  }

  /**
   * Returns the index in this list of the first occurrence of the specified
   * element, or -1 if this list does not contain this element.
   *
   * @param o element to search for
   *
   * @return the index in this list of the first occurrence of the specified
   *         element, or -1 if this list does not contain this element
   */
  public int indexOf(Object o) {
    if (o == null) {
      return -1;
    }
    int d = 0;
    int ind = -1;
    Object ro = null;
    final WeakReference[] refs = references;
    for (int i = 0; i < size; i++) {
      if (refs[i] == null) {
        ro = null;
      } else {
        ro = refs[i].get();
      }
      if (ro == null) {
        d++;
      } else {
        if ((ind < 0) && ro.equals(o)) {
          ind = i;
        }
        if (d > 0) {
          refs[i - d] = refs[i];
        }
      }
    }
    if (d > 0) {
      for (int i = size - d; i < size; i++) {
        refs[i] = null;
      }
      size -= d;
      modCount++;
    }
    return ind;
  }

  /**
   * Returns the index in this list of the last occurrence of the specified
   * element, or -1 if this list does not contain this element.
   *
   * @param o element to search for
   *
   * @return the index in this list of the last occurrence of the specified
   *         element, or -1 if this list does not contain this element
   */
  public int lastIndexOf(Object o) {
    if (o == null) {
      return -1;
    }
    int d = 0;
    int ind = -1;
    Object ro = null;
    final WeakReference[] refs = references;
    for (int i = 0; i < size; i++) {
      if (refs[i] == null) {
        ro = null;
      } else {
        ro = refs[i].get();
      }
      if (ro == null) {
        d++;
      } else {
        ind = i;
        if (d > 0) {
          refs[i - d] = refs[i];
        }
      }
    }
    if (d > 0) {
      for (int i = size - d; i < size; i++) {
        refs[i] = null;
      }
      size -= d;
      modCount++;
    }
    return ind;
  }

  /**
   * Returns the element at the specified position in this list.
   *
   * @param index index of element to return
   *
   * @return the element at the specified position in this list
   *
   * @throws IndexOutOfBoundsException if index is out of range
   *         <tt>(index &lt; 0 || index &gt; size())</tt>
   */
  public Object get(int index) {
    if ((index < 0) || (index >= size)) {
      throwIndexException(index);
    }
    return references[index].get();
  }

  /**
   * Returns an array containing all of the elements in this list in proper
   * sequence.
   *
   * @return an array containing all of the elements in this list in proper
   *         sequence.
   */
  public Object[] toArray() {
    expungeStaleReferences();
    Object[] result = new Object[size];
    final WeakReference[] refs = references;
    Object o = null;
    int ri = 0;
    int i = 0;
    for (; i < size; i++) {
      if (refs[i] != null) {
        o = refs[i].get();
        if (o != null) {
          result[ri++] = o;
        }
      }
    }
    if (ri != i) {
      Object[] tresult = new Object[ri];
      System.arraycopy(result, 0, tresult, 0, ri);
      return tresult;
    }
    return result;
  }

  /**
   * Returns an array containing all of the elements in this list in proper
   * sequence; the runtime type of the returned array is that of the specified
   * array.
   *
   * @param a the array into which the elements of this list are to be stored,
   *          if it is big enough; otherwise, a new array of the same runtime
   *          type is allocated for this purpose.
   *
   * @return an array containing the elements of this list
   */
  public Object[] toArray(Object[] a) {
    expungeStaleReferences();
    Object[] result = new Object[size];
    final WeakReference[] refs = references;
    Object o = null;
    int ri = 0;
    int i;
    for (i = 0; i < size; i++) {
      if (refs[i] != null) {
        o = refs[i].get();
        if (o != null) {
          result[ri++] = o;
        }
      }
    }
    int al = a.length;
    if (al < size) {
      a = (Object[])Array.newInstance(a.getClass().getComponentType(), size);
    }
    System.arraycopy(result, 0, a, 0, size);
    while (al > i) {
      a[i++] = null;
    }
    return a;
  }

  /**
   * Returns a view of the portion of this list between the specified
   * <tt>fromIndex</tt>, inclusive, and <tt>toIndex</tt>, exclusive.
   *
   * @param fromIndex low endpoint (inclusive) of the subList
   * @param toIndex high endpoint (exclusive) of the subList
   *
   * @return a view of the specified range within this list
   */
  public List subList(int fromIndex, int toIndex) {
    int rsize = toIndex - fromIndex;
    WeakArrayList sl = new WeakArrayList(rsize);
    System.arraycopy(references, fromIndex, sl.references, 0, rsize);
    sl.size = rsize;
    sl.initialCapacity = initialCapacity;
    return sl;
  }

  /**
   * Appends the specified element to the end of this list (optional operation).
   *
   * @param o element to be appended to this list
   *
   * @return <tt>true</tt>
   *
   * @throws IllegalArgumentException if o is <tt>null</tt> or <tt>this</tt>
   */
  public boolean add(Object o) {
    if (o == null) {
      throw new IllegalArgumentException("Null element"); //$NON-NLS-1$
    }
    if (o == this) {
      throw new IllegalArgumentException(
                                  "Adding list to itself"); //$NON-NLS-1$ NOPMD
    }
    ensureCapacity(size + 1);
    references[size++] = new WeakReference(o);
    return true;
  }

  /**
   * Appends all of the elements in the specified collection to the end of this
   * list, in the order that they are returned by the specified collection's
   * iterator.
   *
   * @param c collection whose elements are to be added to this list
   *
   * @return <tt>true</tt> if this list changed as a result of the call
   *
   * @throws IllegalArgumentException if collection contains <tt>null</tt> or
   *                                  <tt>this</tt> element
   */
  public boolean addAll(Collection c) {
    int csize = c.size();
    if (csize == 0) {
      return false;
    }
    ensureCapacity(size + csize);
    Iterator ci = c.iterator();
    final WeakReference[] refs = references;
    while (ci.hasNext()) {
      Object element = ci.next();
      if (element == null) {
        throw new IllegalArgumentException(
                                   "Null element in collection"); //$NON-NLS-1$
      }
      if (element == this) {
        throw new IllegalArgumentException(
                                        "Adding list to itself"); //$NON-NLS-1$
      }
      refs[size++] = new WeakReference(element);
    }
    return true;
  }

  /**
   * Inserts the specified element at the specified position in this list
   * (optional operation).
   *
   * @param index index at which the specified element is to be inserted
   * @param element element to be inserted
   *
   * @throws IllegalArgumentException if element is <tt>null</tt> or
   *                                  <tt>this</tt>
   * @throws IndexOutOfBoundsException if index is out of range
   *         <tt>(index &lt; 0 || index &gt; size())</tt>
   */
  public void add(int index, Object element) {
    if (element == null) {
      throw new IllegalArgumentException("Null element"); //$NON-NLS-1$
    }
    if (element == this) {
      throw new IllegalArgumentException(
                                        "Adding list to itself"); //$NON-NLS-1$
    }
    if ((index < 0) || (index >= size)) {
      throwIndexException(index);
    }
    ensureCapacity(size + 1);
    System.arraycopy(references, index, references, index + 1, size - index);
    references[index] = new WeakReference(element);
    size++;
  }

  /**
   * Inserts all of the elements in the specified collection into this list at
   * the specified position.
   *
   * @param index index at which to insert first element from the specified
   *        collection
   * @param c elements to be inserted into this list
   *
   * @return <tt>true</tt> if this list changed as a result of the call
   *
   * @throws IndexOutOfBoundsException if the index is out of range
   *                                   (index &lt; 0 || index &gt; size())
   * @throws IllegalArgumentException if collection contains <tt>null</tt> or
   *                                  <tt>this</tt> element
   */
  public boolean addAll(int index, Collection c) {
    if ((index < 0) || (index >= size)) {
      throwIndexException(index);
    }
    int csize = c.size();
    if (csize == 0) {
      return false;
    }
    int nsize = size + csize;
    ensureCapacity(nsize);
    int si = size - index;
    final WeakReference[] refs = references;
    for (int i = 1; i <= si; i++) {
      refs[nsize - i] = refs[size - i];
    }
    Iterator ci = c.iterator();
    while (ci.hasNext()) {
      Object element = ci.next();
      if (element == null) {
        throw new IllegalArgumentException(
                                   "Null element in collection"); //$NON-NLS-1$
      }
      if (element == this) {
        throw new IllegalArgumentException(
                                        "Adding list to itself"); //$NON-NLS-1$
      }
      refs[index++] = new WeakReference(element);
    }
    size = nsize;
    return true;
  }

  /**
   * Replaces the element at the specified position in this list with the
   * specified element.
   *
   * @param index index of element to replace
   * @param element element to be stored at the specified position
   *
   * @return the element previously at the specified position
   *
   * @throws IllegalArgumentException if element is <tt>null</tt> or
   *                                  <tt>this</tt>
   * @throws IndexOutOfBoundsException if index is out of range
   *         <tt>(index &lt; 0 || index &gt; size())</tt>
   */
  public Object set(int index, Object element) {
    if (element == null) {
      throw new IllegalArgumentException("Null element"); //$NON-NLS-1$
    }
    if (element == this) {
      throw new IllegalArgumentException(
                                        "Adding list to itself"); //$NON-NLS-1$
    }
    if ((index < 0) || (index >= size)) {
      throwIndexException(index);
    }
    WeakReference oldReference = references[index];
    references[index] = new WeakReference(element);
    return oldReference.get();
  }

  /**
   * Removes the element at the specified position in this list.
   *
   * @param index the index of the element to removed
   *
   * @return the element previously at the specified position.
   *
   * @throws IndexOutOfBoundsException if index is out of range
   *         <tt>(index &lt; 0 || index &gt; size())</tt>
   */
  public Object remove(int index) {
    if ((index < 0) || (index > size)) {
      throwIndexException(index);
    }
    WeakReference oldReference = references[index];
    int numMoved = size - index - 1;
    if (numMoved > 0) {
      System.arraycopy(references, index + 1, references, index, numMoved);
    }
    references[--size] = null;
    modCount++;
    return oldReference.get();
  }

  /**
   * Removes the first occurrence in this list of the specified element.
   *
   * @param o element to be removed from this list, if present
   *
   * @return <tt>true</tt> if this list contained the specified element
   */
  public boolean remove(Object o) {
    if (o == null) {
      return false;
    }
    final WeakReference[] refs = references;
    int d = 0;
    boolean retv = false;
    Object ro = null;
    for (int i = 0; i < size; i++) {
      if (refs[i] == null) {
        ro = null;
      } else {
        ro = refs[i].get();
      }
      if (ro == null) {
        d++;
      } else {
        if ((!retv) && ro.equals(o)) {
          d++;
          retv = true;
          continue;
        }
        if (d > 0) {
          refs[i - d] = refs[i];
        }
      }
    }
    if (retv) {
      for (int i = size - d; i < size; i++) {
        refs[i] = null;
      }
      size -= d;
      modCount++;
    }
    return retv;
  }

  /**
   * Removes from this list all the elements that are contained in the specified
   * collection.
   *
   * @param c collection that defines which elements will be removed from this
   *        list
   *
   * @return <tt>true</tt> if this list changed as a result of the call
   */
  public boolean removeAll(Collection c) {
    boolean modified = false;
    final WeakReference[] refs = references;
    for (int i = size - 1; i >= 0; i--) {
      if (refs[i] != null) {
        Object o = refs[i].get();
        if ((o != null) && (c.contains(o))) {
          refs[i] = null;
          modified = true;
        }
      }
    }
    if (modified) {
      expungeStaleReferences();
    }
    return modified;
  }

  /**
   * Retains only the elements in this list that are contained in the specified
   * collection.
   *
   * @param c collection that defines which elements this set will retain
   *
   * @return <tt>true</tt> if this list changed as a result of the call
   */
  public boolean retainAll(Collection c) {
    boolean modified = false;
    final WeakReference[] refs = references;
    for (int i = size - 1; i >= 0; i--) {
      if (refs[i] != null) {
        Object o = refs[i].get();
        if ((o != null) && (!c.contains(o))) {
          refs[i] = null;
          modified = true;
        }
      }
    }
    if (modified) {
      expungeStaleReferences();
    }
    return modified;
  }

  /**
   * Removes all of the elements from this list (optional operation).
   */
  public void clear() {
    if (references.length > (2 * initialCapacity)) {
      references = new WeakReference[initialCapacity];
    } else {
      final WeakReference[] refs = references;
      for (int i = size - 1; i >= 0; i--) {
        refs[i] = null;
      }
    }
    size = 0;
    modCount++;
  }

  /**
   * Returns an iterator over the elements in this list in proper sequence.
   *
   * @return an iterator over the elements in this list in proper sequence
   */
  public Iterator iterator() {
    expungeStaleReferences();
    return new WALIterator(this);
  }

  /**
   * Returns a list iterator of the elements in this list (in proper sequence).
   *
   * @return a list iterator of the elements in this list (in proper sequence).
   */
  public ListIterator listIterator() {
    expungeStaleReferences();
    return new WALIterator(this);
  }

  /**
   * Returns a list iterator of the elements in this list (in proper sequence),
   * starting at the specified position in this list.
   *
   * @param index index of first element to be returned from the list iterator
   *        (by a call to the <tt>next</tt> method).
   *
   * @return a list iterator of the elements in this list (in proper sequence),
   *         starting at the specified position in this list.
   */
  public ListIterator listIterator(int index) {
    expungeStaleReferences();
    return new WALIterator(this, index);
  }

/*^**************************************************************************^*/
/*- Private Method.                                                         -*/
/*^**************************************************************************^*/

  private void throwIndexException(int index) {
    throw new IndexOutOfBoundsException("Index: " + index + //$NON-NLS-1$
                                        ", Size: " + size); //$NON-NLS-1$
  }

/*^**************************************************************************^*/
/*- Inner Class.                                                             -*/
/*^**************************************************************************^*/

  /**
   * List iterator.
   */
  static final class WALIterator implements ListIterator {

    private int cind;
    private final int cmod;
    private Object lastRef;
    private final WeakArrayList wal;

    WALIterator(WeakArrayList wal) {
      this.wal = wal;
      cind = -1;
      cmod = wal.modCount;
    }

    WALIterator(WeakArrayList wal, int index) {
      this.wal = wal;
      cind = index;
      cmod = wal.modCount;
    }

    /**
     * {@inheritDoc}
     */
    public boolean hasNext() {
      if (cmod != wal.modCount) {
        throw new ConcurrentModificationException();
      }
      if (cind + 1 >= wal.size) {
        cind = wal.size;
        lastRef = null;
        return false;
      }
      final WeakReference[] refs = wal.references;
      do {
        lastRef = refs[--cind].get();
      } while ((lastRef == null) && (cind < wal.size));
      return (lastRef != null);
    }

    /**
     * {@inheritDoc}
     */
    public Object next() {
      if (lastRef == null) {
        throw new NoSuchElementException();
      }
      return lastRef;
    }

    /**
     * {@inheritDoc}
     */
    public int nextIndex() {
      return cind;
    }

    /**
     * {@inheritDoc}
     */
    public boolean hasPrevious() {
      if (cmod != wal.modCount) {
        throw new ConcurrentModificationException();
      }
      if (cind - 1 < 0) {
        cind = -1;
        lastRef = null;
        return false;
      }
      final WeakReference[] refs = wal.references;
      do {
        lastRef = refs[--cind].get();
      } while ((lastRef == null) && (cind > 0));
      return (lastRef != null);
    }

    /**
     * {@inheritDoc}
     */
    public Object previous() {
      if (lastRef == null) {
        throw new NoSuchElementException();
      }
      return lastRef;
    }

    /**
     * {@inheritDoc}
     */
    public int previousIndex() {
      return cind;
    }

    /**
     * {@inheritDoc}
     */
    public void add(Object o) {
      throw new UnsupportedOperationException();
    }

    /**
     * {@inheritDoc}
     */
    public void set(Object o) {
      throw new UnsupportedOperationException();
    }

    /**
     * {@inheritDoc}
     */
    public void remove() {
      throw new UnsupportedOperationException();
    }

  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-09 15:56:50 Slobodan
   Initial version

*****************************************************************************^*/
