/*
 * @(#)WeakCache.java  1.0.0 / 2008-05-10
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.util.ref;

import java.lang.ref.Reference;
import java.lang.ref.ReferenceQueue;
import java.lang.ref.WeakReference;
import java.util.ConcurrentModificationException;
import java.util.Iterator;
import java.util.NoSuchElementException;

/**
 * Simple Weak Cache.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public final class WeakCache implements Iterator {

/*^**************************************************************************^*/
/*- Private Static Field.                                                    -*/
/*^**************************************************************************^*/

  private static final int MAXIMUM_CAPACITY = 1 << 30;

/*^**************************************************************************^*/
/*- Private Fields.                                                          -*/
/*^**************************************************************************^*/

  private WeakObjectEntry[] cache;
  private int size;
  private int initialCapacity;
  private float enlargeFactor;
  private float shrinkFactor;
  private int enlargeThreshold;
  private int shrinkThreshold;
  private int modCount;
  private int expectedModCount;
  private WeakObjectEntry currentEntry;
  private int currentBucket;
  private boolean identity;
  private ReferenceQueue queue;

/*^**************************************************************************^*/
/*- Public Constructors.                                                     -*/
/*^**************************************************************************^*/

  /**
   * Constructs new WeakCache object.
   */
  public WeakCache() {
    this(false, 11, 0.75f, 0.3f);
  }

  /**
   * Constructs new WeakCache object.
   *
   * @param initialCapacity the initial capacity
   */
  public WeakCache(int initialCapacity) {
    this(false, initialCapacity, 0.75f, 0.3f);
  }

  /**
   * Constructs new WeakCache object.
   *
   * @param identity the identity cache
   */
  public WeakCache(boolean identity) {
    this(identity, 11, 0.75f, 0.3f);
  }

  /**
   * Constructs new WeakCache object.
   *
   * @param identity the identity cache
   * @param initialCapacity the initial capacity
   */
  public WeakCache(boolean identity, int initialCapacity) {
    this(identity, initialCapacity, 0.75f, 0.3f);
  }

  /**
   * Constructs new WeakCache object.
   *
   * @param initialCapacity the initial capacity
   * @param loadFactor the load factor
   */
  public WeakCache(int initialCapacity, float loadFactor) {
    this(false, initialCapacity, loadFactor, 0.3f);
  }

  /**
   * Constructs new WeakCache object.
   *
   * @param o the object array
   */
  public WeakCache(Object[] o) {
    this(false, (int)(o.length * 1.8), 0.75f, 0.3f);
    for (int i = o.length - 1; i >= 0; i--) {
      put(o[i]);
    }
  }

  /**
   * Constructs new WeakCache object.
   *
   * @param identity the identity cache
   * @param initialCapacity the initial capacity
   * @param loadFactor the load factor
   */
  public WeakCache(boolean identity, int initialCapacity, float loadFactor) {
    this(identity, initialCapacity, loadFactor, 0.3f);
  }

  /**
   * Constructs new WeakCache object.
   *
   * @param identity the identity cache
   * @param o the object array
   */
  public WeakCache(boolean identity, Object[] o) {
    this(identity, (int)(o.length * 1.8), 0.75f, 0.3f);
    for (int i = o.length - 1; i >= 0; i--) {
      put(o[i]);
    }
  }

  /**
   * Constructs new WeakCache object.
   *
   * @param identity the identity cache
   * @param initialCapacity the initial capacity
   * @param enlargeFactor the enlarge factor
   * @param shrinkFactor the shrink factor
   */
  public WeakCache(boolean identity, int initialCapacity, float enlargeFactor,
                   float shrinkFactor) {
    if (initialCapacity <= 0) {
      throw new IllegalArgumentException("Illegal capacity"); //$NON-NLS-1$
    }
    if ((enlargeFactor <= 0) || (enlargeFactor > 1) ||
        Float.isNaN(enlargeFactor)) {
      throw new IllegalArgumentException(
                                       "Illegal enlarge factor"); //$NON-NLS-1$
    }
    if ((shrinkFactor <= 0) || (shrinkFactor > 1) ||
        Float.isNaN(shrinkFactor)) {
      throw new IllegalArgumentException("Illegal shrink factor"); //$NON-NLS-1$
    }
    if (initialCapacity > MAXIMUM_CAPACITY) {
      initialCapacity = MAXIMUM_CAPACITY;
    }
    this.identity = identity;
    this.initialCapacity = initialCapacity;
    this.enlargeFactor = enlargeFactor;
    this.shrinkFactor = shrinkFactor;
    queue = new ReferenceQueue();
    cache = new WeakObjectEntry[initialCapacity];
    enlargeThreshold = (int)(initialCapacity * enlargeFactor);
    shrinkThreshold = (int)(initialCapacity * shrinkFactor);
  }

/*^**************************************************************************^*/
/*- Public Methods.                                                          -*/
/*^**************************************************************************^*/

  /**
   * Returns capacity of this cache.
   *
   * @return capacity of this cache
   */
  public int capacity() {
    return cache.length;
  }

  /**
   * Tests if this cache contains elements.
   *
   * @return <tt>true</tt> if this cache has no elements;
   *         <tt>false</tt> otherwise.
   */
  public boolean empty() {
    return (size == 0);
  }

  /**
   * Returns <tt>true</tt> if this cache contains specified object.
   *
   * @param object the object
   *
   * @return <tt>true</tt> if this cache contains specified object
   */
  public boolean contains(Object object) {
    if (object != null) {
      expungeStaleEntries();
      int hash = object.hashCode();
      final WeakObjectEntry[] t = cache;
      int index = (hash & 0x7FFFFFFF) % t.length;
      for (WeakObjectEntry e = t[index]; e != null; e = e.next) {
        Object wk = e.get();
        if ((wk != null) && (e.hash == hash) && compere(wk, object)) {
          return true;
        }
      }
    }
    return false;
  }

  /**
   * Puts new object.
   *
   * @param object the object
   */
  public void put(Object object) {
    if (object == null) {
      throw new IllegalArgumentException("Adding null object"); //$NON-NLS-1$
    }
    if (object == this) {
      throw new IllegalArgumentException(
                                       "Adding cache to itself"); //$NON-NLS-1$
    }
    expungeStaleEntries();
    WeakObjectEntry[] t = cache;
    int hash = object.hashCode();
    int index = (hash & 0x7FFFFFFF) % t.length;
    for (WeakObjectEntry e = t[index]; e != null; e = e.next) {
      Object wk = e.get();
      if ((wk != null) && (e.hash == hash) && compere(wk, object)) {
        return;
      }
    }
    if (size >= enlargeThreshold) {
      resize(true);
      t = cache;
      index = (hash & 0x7FFFFFFF) % t.length;
    }
    WeakObjectEntry e = new WeakObjectEntry(hash, object, queue, t[index]);
    t[index] = e;
    size++;
    modCount++;
  }

  /**
   * Removes specified object from the cache.
   *
   * @param object the object
   */
  public void remove(Object object) {
    final WeakObjectEntry[] t = cache;
    int hash = object.hashCode();
    expungeStaleEntries();
    int index = (hash & 0x7FFFFFFF) % t.length;
    for (WeakObjectEntry e = t[index], prev = null; e != null;
        prev = e, e = e.next) {
      Object wk = e.get();
      if ((wk != null) && (e.hash == hash) && compere(wk, object)) {
        if (prev == null) {
          t[index] = e.next;
        } else {
          prev.next = e.next;
        }
        size--;
        if ((t.length > initialCapacity) && (size < shrinkThreshold)) {
          resize(false);
        }
      }
    }
    modCount++;
  }

  /**
   * Clears this cache.
   */
  public void clear() {
    Reference r;
    do {
      r = queue.poll();
    } while (r != null);
    WeakObjectEntry[] t = cache;
    if (t.length > (2 * initialCapacity)) {
      cache = new WeakObjectEntry[initialCapacity];
      enlargeThreshold = (int)(initialCapacity * enlargeFactor);
      shrinkThreshold = (int)(initialCapacity * shrinkFactor);
    } else {
      for (int index = t.length; --index >= 0;) {
        t[index] = null;
      }
    }
    size = 0;
    modCount++;
    currentEntry = null;
    do {
      r = queue.poll();
    } while (r != null);
  }

  /**
   * Returns iterator over the elements.
   *
   * @return iterator over the elements
   */
  public Iterator iterator() {
    expungeStaleEntries();
    expectedModCount = modCount;
    currentEntry = null;
    currentBucket = cache.length;
    return this;
  }

  /**
   * Iterator's hasNext method.
   *
   * @return <tt>true</tt> if the iterator has more elements
   */
  public boolean hasNext() {
    if (modCount != expectedModCount) {
      throw new ConcurrentModificationException();
    }
    WeakObjectEntry he = currentEntry;
    int i = currentBucket;
    WeakObjectEntry[] t = cache;
    while ((he == null) && (i > 0)) {
      he = t[--i];
    }
    currentEntry = he;
    currentBucket = i;
    return (currentEntry != null);
  }

  /**
   * Iterator's next method.
   *
   * @return the next element in the iteration.
   *
   * @exception NoSuchElementException iteration has no more elements.
   */
  public Object next() {
    if (modCount != expectedModCount) {
      throw new ConcurrentModificationException();
    }
    WeakObjectEntry he = currentEntry;
    int i = currentBucket;
    WeakObjectEntry[] t = cache;
    while ((he == null) && (i > 0)) {
      he = t[--i];
    }
    currentEntry = he;
    currentBucket = i;
    if (he != null) {
      WeakObjectEntry e = currentEntry;
      currentEntry = e.next;
      return e.get();
    }
    throw new NoSuchElementException();
  }

  /**
   * Iterator's remove method.
   */
  public void remove() {
    throw new UnsupportedOperationException();
  }

  /**
   * Returns number of objects in this cache.
   *
   * @return number of objects in this cache
   */
  public int size() {
    if (size == 0) {
      return 0;
    }
    expungeStaleEntries();
    return size;
  }

  /**
   * Compares the specified object with this cache for equality.
   *
   * @param o the object to be compared for equality with this cache
   *
   * @return <tt>true</tt> if the specified object is equal to this cache
   */
  public boolean equals(Object o) {
    if (o == this) {
      return true;
    }
    if (!(o instanceof WeakCache)) {
      return false;
    }
    WeakCache wc = (WeakCache)o;
    if (size != wc.size) {
      return false;
    }
    final WeakObjectEntry[] t = cache;
    for (int i = t.length - 1; i >= 0; i--) {
      for (WeakObjectEntry e = t[i]; e != null; e = e.next) {
        Object object = e.get();
        if (((object != null) && !wc.contains(object)) ||
            ((object == null) && wc.contains(null))) {
          return false;
        }
      }
    }
    return true;
  }

  /**
   * Returns the hash code value for this cache.
   *
   * @return the hash code value for this cache
   */
  public int hashCode() {
    int hashCode = 0;
    final WeakObjectEntry[] t = cache;
    for (int i = t.length - 1; i >= 0; i--) {
      for (WeakObjectEntry e = t[i]; e != null; e = e.next) {
        Object object = e.get();
        if (object != null) {
          hashCode += object.hashCode();
        }
      }
    }
    return hashCode;
  }

/*^**************************************************************************^*/
/*- Private Methods.                                                         -*/
/*^**************************************************************************^*/

  private void resize(boolean enlarge) {
    int oldCapacity = cache.length;
    WeakObjectEntry[] oldTable = cache;
    int newCapacity;
    if (enlarge) {
      newCapacity = (oldCapacity * 5) / 3 + 1;
    } else {
      newCapacity = oldCapacity / 2;
      if (newCapacity < initialCapacity) {
        newCapacity = initialCapacity;
      }
    }
    WeakObjectEntry[] newTable = new WeakObjectEntry[newCapacity];
    enlargeThreshold = (int)(newCapacity * enlargeFactor);
    shrinkThreshold = (int)(newCapacity * shrinkFactor);
    cache = newTable;
    for (int i = oldCapacity - 1; i >= 0; i--) {
      WeakObjectEntry old = oldTable[i];
      while (old != null) {
        WeakObjectEntry e = old;
        old = old.next;
        int index = (e.hash & 0x7FFFFFFF) % newCapacity;
        e.next = newTable[index];
        newTable[index] = e;
      }
    }
  }

  private void expungeStaleEntries() {
    Object r;
    WeakObjectEntry[] t = cache;
    while ((r = queue.poll()) != null) {
      WeakObjectEntry e = (WeakObjectEntry)r;
      int h = e.hash;
      int i = (h & 0x7FFFFFFF) % t.length;
      WeakObjectEntry prev = t[i];
      WeakObjectEntry p = prev;
      while (p != null) {
        WeakObjectEntry next = p.next;
        if (p == e) { // NOPMD
          if (prev == e) { // NOPMD
            t[i] = next;
          } else {
            prev.next = next;
          }
          e.next = null;
          size--;
          break;
        }
        prev = p;
        p = next;
      }
    }
  }

  private boolean compere(Object wk, Object o) {
    if (identity) {
      return (wk == o); // NOPMD
    }
    return wk.equals(o);
  }

/*^**************************************************************************^*/
/*- Inner Class.                                                             -*/
/*^**************************************************************************^*/

  /**
   * Weak Entry.
   */
  static final class WeakObjectEntry extends WeakReference {
    int hash;
    WeakObjectEntry next;
    WeakObjectEntry(int hash, Object object, ReferenceQueue queue,
                   WeakObjectEntry next) {
      super(object, queue);
      this.hash = hash;
      this.next = next;
    }
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-10 21:12:38 Slobodan
   Initial version

*****************************************************************************^*/
