/*
 * @(#)ObjectStack.java  1.0.0 / 2008-05-08
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.util.stack;

/**
 * Simple stack of objects.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public class ObjectStack {

/*^**************************************************************************^*/
/*- Private Fields.                                                          -*/
/*^**************************************************************************^*/

  private int top;
  private Object[] stack;

/*^**************************************************************************^*/
/*- Public Constructors.                                                     -*/
/*^**************************************************************************^*/

  /**
   * Constructs new ObjectStack object.
   */
  public ObjectStack() {
    this(10);
  }

  /**
   * Constructs new ObjectStack object.
   *
   * @param depth the estimated depth
   *
   * @throws IllegalArgumentException if depth <= 0
   */
  public ObjectStack(int depth) {
    if (depth <= 0) {
      throw new IllegalArgumentException("Invalid depth"); //$NON-NLS-1$
    }
    stack = new Object[depth];
  }

/*^**************************************************************************^*/
/*- Public Methods.                                                          -*/
/*^**************************************************************************^*/

  /**
   * Returns <tt>true</tt> if the stack is empty.
   *
   * @return <tt>true</tt> if the stack is empty
   */
  public boolean isEmpty() {
    return (top == 0);
  }

  /**
   * Returns stack size.
   *
   * @return stack size
   */
  public int size() {
    return top;
  }

  /**
   * Pushes an object onto the top of the stack.
   *
   * @param o the object
   */
  public void push(Object o) {
    int oldSize = stack.length;
    int minSize = top + 1;
    if (minSize > oldSize) {
      Object[] oldStack = stack;
      int newSize = ((3 * oldSize) >> 1) + 1;
      if (newSize < minSize) {
        newSize = minSize;
      }
      stack = new Object[newSize];
      System.arraycopy(oldStack, 0, stack, 0, top);
    }
    stack[top++] = o;
  }

  /**
   * Removes object from the top of the stack and returns it.
   *
   * @return object from the top of the stack
   *
   * @throws StackException if stack is empty
   */
  public Object pop() {
    if (top > 0) {
      Object o = stack[--top];
      stack[top] = null; // helps GC
      return o;
    }
    throw new StackException("Stack empty"); //$NON-NLS-1$
  }

  /**
   * Returns object from the top of the stack.
   *
   * @return object from the top of the stack
   *
   * @throws StackException if stack is empty
   */
  public Object top() {
    if (top > 0) {
      return stack[top - 1];
    }
    throw new StackException("Stack empty"); //$NON-NLS-1$
  }

  /**
   * Removes all objects from the stack.
   */
  public void clear() {
    Object[] s = stack;
    for (int i = top - 1; i >= 0; i--) {
      s[i] = null; // helps GC
    }
    top = 0;
  }

}
