/*
 * @(#)XMLPreferences.java  1.0.0 / 2008-05-13
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.xml.preferences;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.util.Iterator;
import java.util.prefs.AbstractPreferences;
import java.util.prefs.BackingStoreException;

import net.wrka.util.CommonConstants;
import net.wrka.util.SimpleHashMap;
import net.wrka.util.SimpleStringMap;
import net.wrka.util.StringIterator;
import net.wrka.xml.simple.XMLPrintServer;

/**
 * XML Preferences.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
final class XMLPreferences extends AbstractPreferences
                           implements XMLPreferencesConstants {

/*^**************************************************************************^*/
/*- Package-Private Static Fields.                                           -*/
/*^**************************************************************************^*/

  static final XMLPreferences USER_ROOT;
  static final XMLPreferences SYSTEM_ROOT;

/*^**************************************************************************^*/
/*- Static Initialization.                                                   -*/
/*^**************************************************************************^*/

  static {
    USER_ROOT = new XMLPreferences(null, CommonConstants.STRING_EMPTY);
    SYSTEM_ROOT = new XMLPreferences(null, CommonConstants.STRING_EMPTY);
  }

/*^**************************************************************************^*/
/*- Private Fields.                                                          -*/
/*^**************************************************************************^*/

  private final XMLPreferences parent;
  private final XMLPreferences root;
  private final SimpleStringMap prefs;
  private final SimpleHashMap children;
  private String[] childrenNames;

/*^**************************************************************************^*/
/*- Public Constructor.                                                      -*/
/*^**************************************************************************^*/

  /**
   * Constructs new Preferences object.
   *
   * @param parent the parent
   * @param name the name
   */
  public XMLPreferences(XMLPreferences parent, String name) {
    super(parent, name);
    this.parent = parent;
    prefs = new SimpleStringMap(5);
    children = new SimpleHashMap(5);
    if (parent == null) {
      root = this;
    } else {
      XMLPreferences p = parent;
      XMLPreferences cp = p.parent;
      while (cp != null) {
        p = cp;
        cp = cp.parent;
      }
      root = p;
    }
  }

/*^**************************************************************************^*/
/*- Overridden Methods.                                                      -*/
/*^**************************************************************************^*/

  /**
   * Implements <tt>Preferences</tt> <tt>flush()</tt> method.
   *
   * @throws BackingStoreException if this operation cannot be completed
   */
  public void flush() throws BackingStoreException {
    try {
      if (isUserNode()) {
        File userPrefsFile = XMLPreferencesFactory.userPreferencesFile;
        if (userPrefsFile != null) {
          USER_ROOT.exportSubtree(new FileOutputStream(userPrefsFile));
        }
      } else {
        File systemPrefsFile = XMLPreferencesFactory.systemPreferencesFile;
        if (systemPrefsFile != null) {
          SYSTEM_ROOT.exportSubtree(new FileOutputStream(systemPrefsFile));
        }
      }
    } catch (IOException ioe) {
      throw new BackingStoreException(ioe);
    }
  }

  /**
   * Implements the <tt>isUserNode</tt> method as per the specification in
   * Preferences.
   *
   * @return <tt>true</tt> if this preference node is in the user preference
   *         tree, <tt>false</tt> if it's in the system preference tree.
   */
  public boolean isUserNode() {
    return (root != SYSTEM_ROOT); // NOPMD
  }

  /**
   * Implements <tt>Preferences</tt> <tt>sync()</tt> method.
   * Equivalent to flush().
   *
   * @throws BackingStoreException if this operation cannot be completed
   */
  public void sync() throws BackingStoreException {
    flush();
  }

  /**
   * Implements <tt>Preferences</tt> <tt>exportNode</tt> method.
   *
   * @param os the output stream on which to emit the XML document
   *
   * @throws IOException if writing to the specified output stream failed
   * @throws BackingStoreException
   */
  public void exportNode(OutputStream os) throws IOException,
                                                 BackingStoreException {
    exportToXML(os, false);
  }

  /**
   * Implements <tt>Preferences</tt> <tt>exportSubtree</tt> method.
   *
   * @param os the output stream on which to emit the XML document
   *
   * @throws IOException if writing to the specified output stream failed
   * @throws BackingStoreException
   */
  public void exportSubtree(OutputStream os) throws IOException,
                                                    BackingStoreException {
    exportToXML(os, true);
  }

  /**
   * Put the given key-value association into this preference node.
   *
   * @param key the key
   * @param value the value
   */
  protected void putSpi(String key, String value) {
    prefs.put(key, value);
  }

  /**
   * Return the value associated with the specified key at this preference node.
   *
   * @param key the key
   *
   * @return the value associated with the specified key at this preference node
   */
  protected String getSpi(String key) {
    return prefs.get(key);
  }

  /**
   * Remove the association (if any) for the specified key at this preference
   * node.
   *
   * @param key the key
   */
  protected void removeSpi(String key) {
    prefs.remove(key);
  }

  /**
   * Returns the named child of this preference node, creating it if it does not
   * already exist.
   *
   * @param name the name of the child node to return, relative to this
   *             preference node
   *
   * @return the named child node
   */
  protected AbstractPreferences childSpi(String name) {
    XMLPreferences child = (XMLPreferences)children.get(name);
    if (child == null) {
      child = new XMLPreferences(this, name);
      children.put(name, child);
      if (childrenNames != null) {
        childrenNames = null;
      }
    }
    return child;
  }

  /**
   * Returns the names of the children of this preference node.
   *
   * @return an array containing the names of the children of this preference
   *         node.
   *
   * @throws BackingStoreException if this operation cannot be completed.
   */
  protected String[] childrenNamesSpi() throws BackingStoreException {
    if (childrenNames == null) {
      childrenNames = new String[children.size()];
      String[] cn = childrenNames;
      Iterator sit = children.keys();
      for (int i = cn.length - 1; i >= 0; i--) {
        cn[i] = (String)sit.next();
      }
    }
    return (String[])childrenNames.clone();
  }

  /**
   * Returns all of the keys that have an associated value in this preference
   * node.
   *
   * @return an array of the keys that have an associated value in this
   *         preference node.
   *
   * @throws BackingStoreException if this operation cannot be completed.
   */
  protected String[] keysSpi() throws BackingStoreException {
    String[] res = new String[prefs.size()];
    StringIterator sit = prefs.keys();
    for (int i = res.length - 1; i >= 0; i--) {
      res[i] = sit.nextString();
    }
    return res;
  }

  /**
   * Removes this preference node, invalidating it and any preferences that it
   * contains.
   *
   * @throws BackingStoreException if this operation cannot be completed.
   */
  protected void removeNodeSpi() throws BackingStoreException {
    if (parent != null) {
      parent.children.remove(name());
      if (parent.childrenNames != null) {
        parent.childrenNames = null;
      }
    }
  }

  /**
   * This method is invoked with this node locked.
   *
   * @throws BackingStoreException if this operation cannot be completed
   */
  protected void flushSpi() throws BackingStoreException {
    // empty method body
  }

  /**
   * This method is invoked with this node locked.
   *
   * @throws BackingStoreException if this operation cannot be completed
   */
  protected void syncSpi() throws BackingStoreException {
    // empty method body
  }

/*^**************************************************************************^*/
/*- Private Methods.                                                         -*/
/*^**************************************************************************^*/

  private void exportToXML(OutputStream os,
                           boolean subTree) throws IOException {
    checkRemoved();
    synchronized (lock) {
      XMLPrintServer xmlPrintServer = new XMLPrintServer(os, true);
      xmlPrintServer.beginDocument(DOCTYPE);
      xmlPrintServer.beginElement(PRERERENCES);
      xmlPrintServer.setAttribute(EXT_KEY, EXT_VALUE);
      xmlPrintServer.beginElement(ROOT);
      if (isUserNode()) {
        xmlPrintServer.setAttribute(TYPE, USER);
      } else {
        xmlPrintServer.setAttribute(TYPE, SYSTEM);
      }
      if (parent == null) {
        if (prefs.empty()) {
          xmlPrintServer.beginEmptyElement(MAP);
        } else {
          xmlPrintServer.beginElement(MAP);
          StringIterator sit = prefs.keys();
          while (sit.hasNextString()) {
            xmlPrintServer.beginEmptyElement(ENTRY);
            String key = sit.nextString();
            xmlPrintServer.setAttribute(KEY, key);
            xmlPrintServer.setAttribute(VALUE, prefs.get(key));
            xmlPrintServer.endElement();
          }
        }
        xmlPrintServer.endElement();
        if (subTree && !children.empty()) {
          Iterator it = children.values();
          while (it.hasNext()) {
            XMLPreferences p = (XMLPreferences)it.next();
            p.checkRemoved();
            synchronized (p.lock) {
              p.storeToXML(xmlPrintServer, true);
            }
          }
        }
      } else {
        xmlPrintServer.beginEmptyElement(MAP);
        xmlPrintServer.endElement();
        storeToXML(xmlPrintServer, true);
      }
      xmlPrintServer.endElement();
      xmlPrintServer.endElement();
      xmlPrintServer.endDocument();
    }
  }

  private void storeToXML(XMLPrintServer xmlPrintServer,
                          boolean subTree) throws IOException {
    xmlPrintServer.beginElement(NODE);
    xmlPrintServer.setAttribute(NAME, name());
    if (prefs.empty()) {
      xmlPrintServer.beginEmptyElement(MAP);
    } else {
      xmlPrintServer.beginElement(MAP);
      StringIterator sit = prefs.keys();
      while (sit.hasNextString()) {
        xmlPrintServer.beginEmptyElement(ENTRY);
        String key = sit.nextString();
        xmlPrintServer.setAttribute(KEY, key);
        xmlPrintServer.setAttribute(VALUE, prefs.get(key));
        xmlPrintServer.endElement();
      }
    }
    xmlPrintServer.endElement();
    if (subTree && !children.empty()) {
      Iterator it = children.values();
      while (it.hasNext()) {
        XMLPreferences p = (XMLPreferences)it.next();
        p.checkRemoved();
        synchronized (p.lock) {
          p.storeToXML(xmlPrintServer, true);
        }
      }
    }
    xmlPrintServer.endElement();
  }

  private void checkRemoved() {
    if (isRemoved()) {
      throw new IllegalStateException("Node has been removed"); //$NON-NLS-1$
    }
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-13 14:10:37 Slobodan
   Initial version

*****************************************************************************^*/
