/*
 * @(#)XMLPreferencesFactory.java  1.0.0 / 2008-05-13
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.xml.preferences;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.prefs.BackingStoreException;
import java.util.prefs.Preferences;
import java.util.prefs.PreferencesFactory;

import net.wrka.xml.simple.sax.Parser;
import net.wrka.xml.simple.sax.ParsingException;

/**
 * Preferences Factory.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public final class XMLPreferencesFactory implements PreferencesFactory {

/*^**************************************************************************^*/
/*- Package-Private Static Fields.                                           -*/
/*^**************************************************************************^*/

  static File userPreferencesFile;
  static File systemPreferencesFile;

/*^**************************************************************************^*/
/*- Private Static Fields.                                                   -*/
/*^**************************************************************************^*/

  private static boolean initialized;
  private static boolean created; // NOPMD

/*^**************************************************************************^*/
/*- Public Constructor.                                                      -*/
/*^**************************************************************************^*/

  /**
   * Constructs new XMLPreferencesFactory object.
   * <p>
   *  <tt>NOTE: Only one can be created</tt>
   * </p>
   */
  public XMLPreferencesFactory() {
    if (created) {
      throw new IllegalStateException(
                     "XMLPreferencesFactory is already created"); //$NON-NLS-1$
    }
    created = true;
  }

/*^**************************************************************************^*/
/*- PreferencesFactory Implementation.                                       -*/
/*^**************************************************************************^*/

  /**
   * Returns the system root preference node.
   *
   * @return the system root preference node
   */
  public Preferences systemRoot() {
    return XMLPreferences.SYSTEM_ROOT;
  }

  /**
   * Returns the user root preference node.
   *
   * @return the user root preference node
   */
  public Preferences userRoot() {
    return XMLPreferences.USER_ROOT;
  }

/*^**************************************************************************^*/
/*- Public Static Methods.                                                   -*/
/*^**************************************************************************^*/

  /**
   * Returns <tt>true</tt> if factory is initialized.
   *
   * @return <tt>true</tt> if factory is initialized
   */
  public static boolean isInitialized() {
    synchronized (XMLPreferencesFactory.class) {
      return initialized;
    }
  }

  /**
   * Initializes the factory.
   *
   * @param systemPreferencesFile the system preferences file
   * @param userPreferencesFile the user preferences file
   *
   * @throws BackingStoreException on error
   */
  public static void initialize(File systemPreferencesFile,
                       File userPreferencesFile) throws BackingStoreException {
    synchronized (XMLPreferencesFactory.class) {
      if (initialized) {
        return;
      }
      initialized = true;
    }
    XMLPreferencesFactory.systemPreferencesFile = systemPreferencesFile;
    XMLPreferencesFactory.userPreferencesFile = userPreferencesFile;
    System.setProperty("java.util.prefs.PreferencesFactory", //$NON-NLS-1$
                       XMLPreferencesFactory.class.getName());
    if ((systemPreferencesFile != null) && systemPreferencesFile.exists()) {
      try {
        loadFromXML(new FileInputStream(systemPreferencesFile), true);
      } catch (IOException e) {
        XMLPreferences.SYSTEM_ROOT.clear();
        throw new BackingStoreException(e);
      }
    }
    if (userPreferencesFile == null) {
      XMLPreferencesFactory.userPreferencesFile =
                                             getDefaultUserPreferencesFile();
    } else if (userPreferencesFile.exists()) {
      try {
        loadFromXML(new FileInputStream(userPreferencesFile), false);
      } catch (IOException e) {
        XMLPreferences.USER_ROOT.clear();
        throw new BackingStoreException(e);
      }
    }
  }

  /**
   * Returns system preferences file.
   *
   * @return system preferences file
   */
  public static File getSystemPreferencesFile() {
    return systemPreferencesFile;
  }

  /**
   * Returns user preferences file.
   *
   * @return user preferences file
   */
  public static File getUserPreferencesFile() {
    return userPreferencesFile;
  }

/*^**************************************************************************^*/
/*- Private Static Method.                                                   -*/
/*^**************************************************************************^*/

  /**
   * Loads preferences from input stream.
   *
   * @param is the input stream
   *
   * @throws IOException if an I/O error or parse error occurs
   */
  private static void loadFromXML(InputStream is,
                                boolean systemPreferences) throws IOException {
    InputStreamReader isr = null;
    try {
      isr = new InputStreamReader(is);
      Parser.parse(isr, new XMLPreferencesHandler(systemPreferences));
    } catch (ParsingException e) {
      throw new IOException("Invalid preferences: " + //$NON-NLS-1$ NOPMD
                            e.getMessage());
    } finally {
      if (isr != null) {
        isr.close();
      }
    }
  }

  private static File getDefaultUserPreferencesFile() {
    return new File(System.getProperty("XMLPreferences.file", //$NON-NLS-1$
                                       "Settings.cfg")); //$NON-NLS-1$
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-13 14:11:08 Slobodan
   Initial version

*****************************************************************************^*/
