/*
 * @(#)DefaultXMLAttributes.java  1.0.0 / 2008-04-23
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.xml.simple;

import java.io.IOException;

/**
 * Default XMLAttributes Implementation.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public class DefaultXMLAttributes implements XMLAttributes {

/*^**************************************************************************^*/
/*- Private Fields.                                                          -*/
/*^**************************************************************************^*/

  private Entry[] table;
  private int count;
  private int threshold;
  private float loadFactor;

/*^**************************************************************************^*/
/*- Public Constructors.                                                     -*/
/*^**************************************************************************^*/

  /**
   * Constructs new DefaultXMLAttributes object.
   *
   * @param initialCapacity initial capacity
   * @param loadFactor load factor
   */
  public DefaultXMLAttributes(int initialCapacity, float loadFactor) {
    this.loadFactor = loadFactor;
    table = new Entry[initialCapacity];
    threshold = (int)(initialCapacity * loadFactor);
  }

  /**
   * Constructs new DefaultXMLAttributes object.
   *
   * @param initialCapacity initial capacity
   */
  public DefaultXMLAttributes(int initialCapacity) {
    this(initialCapacity, 0.75f);
  }

  /**
   * Constructs new DefaultXMLAttributes object.
   */
  public DefaultXMLAttributes() {
    this(11, 0.75f);
  }

/*^**************************************************************************^*/
/*- Public Methods.                                                          -*/
/*^**************************************************************************^*/

  /**
   * Puts new attribute.
   *
   * @param attrName attribute name
   * @param attrValue attribute value
   *
   * @return old attribute value
   */
  public String put(String attrName, String attrValue) {
    if ((attrName != null) && (attrValue != null)) {
      Entry[] t = table; // NOTE: Do not optimize
      int hash = attrName.hashCode();
      int index = (hash & 0x7FFFFFFF) % t.length;
      for (Entry e = t[index]; e != null; e = e.next) {
        if ((e.hash == hash) && e.name.equals(attrName)) {
          String old = e.value;
          e.value = attrValue;
          return old;
        }
      }
      if (count >= threshold) {
        rehash();
        t = table;
        index = (hash & 0x7FFFFFFF) % t.length;
      }
      Entry e = new Entry(hash, attrName, attrValue, t[index]);
      t[index] = e;
      count++;
    }
    return null;
  }

  /**
   * Puts new attribute.
   *
   * @param attrName attribute name
   * @param attrValue attribute value
   *
   * @return old attribute value or <tt>null</tt> if there is no old value
   */
  public String put(String attrName, boolean attrValue) {
    return put(attrName, String.valueOf(attrValue));
  }

  /**
   * Puts new attribute.
   *
   * @param attrName attribute name
   * @param attrValue attribute value
   *
   * @return old attribute value or <tt>null</tt> if there is no old value
   */
  public String put(String attrName, byte attrValue) {
    return put(attrName, String.valueOf(attrValue));
  }

  /**
   * Puts new attribute.
   *
   * @param attrName attribute name
   * @param attrValue attribute value
   *
   * @return old attribute value or <tt>null</tt> if there is no old value
   */
  public String put(String attrName, char attrValue) {
    return put(attrName, String.valueOf(attrValue));
  }

  /**
   * Puts new attribute.
   *
   * @param attrName attribute name
   * @param attrValue attribute value
   *
   * @return old attribute value or <tt>null</tt> if there is no old value
   */
  public String put(String attrName, short attrValue) {
    return put(attrName, String.valueOf(attrValue));
  }

  /**
   * Puts new attribute.
   *
   * @param attrName attribute name
   * @param attrValue attribute value
   *
   * @return old attribute value or <tt>null</tt> if there is no old value
   */
  public String put(String attrName, int attrValue) {
    return put(attrName, String.valueOf(attrValue));
  }

  /**
   * Puts new attribute.
   *
   * @param attrName attribute name
   * @param attrValue attribute value
   *
   * @return old attribute value or <tt>null</tt> if there is no old value
   */
  public String put(String attrName, double attrValue) {
    return put(attrName, String.valueOf(attrValue));
  }

  /**
   * Puts new attribute.
   *
   * @param attrName attribute name
   * @param attrValue attribute value
   *
   * @return old attribute value or <tt>null</tt> if there is no old value
   */
  public String put(String attrName, float attrValue) {
    return put(attrName, String.valueOf(attrValue));
  }

  /**
   * Removes all attributes.
   */
  public void clear() {
    final Entry[] table = this.table;
    for (int index = table.length; --index >= 0;) {
      table[index] = null;
    }
    count = 0;
  }

/*^**************************************************************************^*/
/*- XMLAttributes Interface Implementation.                                  -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  public int getAttributeCount() {
    return count;
  }

  /**
   * {@inheritDoc}
   */
  public boolean isEmpty() {
    return (count == 0);
  }

  /**
   * {@inheritDoc}
   */
  public boolean contains(String attrName) {
    if (attrName != null) {
      int hash = attrName.hashCode();
      int index = (hash & 0x7FFFFFFF) % table.length;
      for (Entry e = table[index]; e != null; e = e.next) {
        if ((e.hash == hash) && e.name.equals(attrName)) {
          return true;
        }
      }
    }
    return false;
  }

  /**
   * {@inheritDoc}
   */
  public String get(String attrName) {
    if (attrName != null) {
      int hash = attrName.hashCode();
      int index = (hash & 0x7FFFFFFF) % table.length;
      for (Entry e = table[index]; e != null; e = e.next) {
        if ((e.hash == hash) && e.name.equals(attrName)) {
          return e.value;
        }
      }
    }
    return null;
  }

  /**
   * {@inheritDoc}
   */
  public boolean getBoolean(String attrName) throws XMLAttributeNotExists {
    String val = get(attrName);
    if (val != null) {
      if (XMLConstants.STRING_TRUE.equals(val)) {
        return true;
      }
      if (XMLConstants.STRING_FALSE.equals(val)) {
        return false;
      }
    }
    throw new XMLAttributeNotExists(attrName);
  }

  /**
   * {@inheritDoc}
   */
  public boolean getBoolean(String attrName, boolean defValue) {
    String val = get(attrName);
    if (val != null) {
      if (XMLConstants.STRING_TRUE.equals(val)) {
        return true;
      }
      if (XMLConstants.STRING_FALSE.equals(val)) {
        return false;
      }
    }
    return defValue;
  }

  /**
   * {@inheritDoc}
   */
  public byte getByte(String attrName) throws XMLAttributeNotExists {
    String val = get(attrName);
    if (val != null) {
      try {
        return Byte.parseByte(val);
      } catch (Throwable t) { // NOPMD
        // empty catch block
      }
    }
    throw new XMLAttributeNotExists(attrName);
  }

  /**
   * {@inheritDoc}
   */
  public byte getByte(String attrName, byte defValue) {
    String val = get(attrName);
    if (val != null) {
      try {
        return Byte.parseByte(val);
      } catch (Throwable t) { // NOPMD
        // empty catch block
      }
    }
    return defValue;
  }

  /**
   * {@inheritDoc}
   */
  public char getChar(String attrName) throws XMLAttributeNotExists {
    String val = get(attrName);
    if ((val != null) && (val.length() == 1)) {
      return val.charAt(0);
    }
    throw new XMLAttributeNotExists(attrName);
  }

  /**
   * {@inheritDoc}
   */
  public char getChar(String attrName, char defValue) {
    String val = get(attrName);
    if ((val != null) && (val.length() == 1)) {
      return val.charAt(0);
    }
    return defValue;
  }

  /**
   * {@inheritDoc}
   */
  public short getShort(String attrName) throws XMLAttributeNotExists {
    String val = get(attrName);
    if (val != null) {
      try {
        return Short.parseShort(val);
      } catch (Throwable t) { // NOPMD
        // empty catch block
      }
    }
    throw new XMLAttributeNotExists(attrName);
  }

  /**
   * {@inheritDoc}
   */
  public short getShort(String attrName, short defValue) {
    String val = get(attrName);
    if (val != null) {
      try {
        return Short.parseShort(val);
      } catch (Throwable t) { // NOPMD
        // empty catch block
      }
    }
    return defValue;
  }

  /**
   * {@inheritDoc}
   */
  public int getInteger(String attrName) throws XMLAttributeNotExists {
    String val = get(attrName);
    if (val != null) {
      try {
        return Integer.parseInt(val);
      } catch (Throwable t) { // NOPMD
        // empty catch block
      }
    }
    throw new XMLAttributeNotExists(attrName);
  }

  /**
   * {@inheritDoc}
   */
  public int getInteger(String attrName, int defValue) {
    String val = get(attrName);
    if (val != null) {
      try {
        return Integer.parseInt(val);
      } catch (Throwable t) { // NOPMD
        // empty catch block
      }
    }
    return defValue;
  }

  /**
   * {@inheritDoc}
   */
  public float getFloat(String attrName) throws XMLAttributeNotExists {
    String val = get(attrName);
    if (val != null) {
      try {
        return Float.parseFloat(val);
      } catch (Throwable t) { // NOPMD
        // empty catch block
      }
    }
    throw new XMLAttributeNotExists(attrName);
  }

  /**
   * {@inheritDoc}
   */
  public float getFloat(String attrName, float defValue) {
    String val = get(attrName);
    if (val != null) {
      try {
        return Float.parseFloat(val);
      } catch (Throwable t) { // NOPMD
        // empty catch block
      }
    }
    return defValue;
  }

  /**
   * {@inheritDoc}
   */
  public double getDouble(String attrName) throws XMLAttributeNotExists {
    String val = get(attrName);
    if (val != null) {
      try {
        return Double.parseDouble(val);
      } catch (Throwable t) { // NOPMD
        // empty catch block
      }
    }
    throw new XMLAttributeNotExists(attrName);
  }

  /**
   * {@inheritDoc}
   */
  public double getDouble(String attrName, double defValue) {
    String val = get(attrName);
    if (val != null) {
      try {
        return Double.parseDouble(val);
      } catch (Throwable t) { // NOPMD
        // empty catch block
      }
    }
    return defValue;
  }

  /**
   * {@inheritDoc}
   */
  public String getString(String attrName) throws XMLAttributeNotExists {
    String val = get(attrName);
    if (val != null) {
      return val;
    }
    throw new XMLAttributeNotExists(attrName);
  }

  /**
   * {@inheritDoc}
   */
  public String getString(String attrName, String defValue) {
    String val = get(attrName);
    if (val != null) {
      return val;
    }
    return defValue;
  }

  /**
   * {@inheritDoc}
   */
  public void print(XMLPrintServer ps) throws IOException {
    final Entry[] table = this.table;
    for (int index = table.length; --index >= 0;) {
      for (Entry e = table[index]; e != null; e = e.next) {
        ps.setAttribute(e.name, e.value);
      }
    }
  }

/*^**************************************************************************^*/
/*- Private Method.                                                          -*/
/*^**************************************************************************^*/

  private void rehash() {
    int oldCapacity = table.length;
    Entry[] oldMap = table;
    int newCapacity = oldCapacity << 1;
    Entry[] newMap = new Entry[++newCapacity];
    threshold = (int)(newCapacity * loadFactor);
    table = newMap;
    for (int i = oldCapacity; --i >= 0;) {
      Entry old = oldMap[i];
      while (old != null) {
        Entry e = old;
        old = old.next;
        int index = (e.hash & 0x7FFFFFFF) % newCapacity;
        e.next = newMap[index];
        newMap[index] = e;
      }
    }
  }

/*^**************************************************************************^*/
/*- Package-Private Inner Class.                                             -*/
/*^**************************************************************************^*/

  /**
   * Hash table Entry.
   */
  static class Entry {
    int hash;
    String name;
    String value;
    Entry next;
    Entry(int hash, String name, String value, Entry next) {
      this.hash = hash;
      this.name = name;
      this.value = value;
      this.next = next;
    }
  }

}

/*^*****************************************************************************
  File History:

 - 2008-04-23 20:06:14 Slobodan
   Initial version

*****************************************************************************^*/
