/*
 * @(#)XMLAttributes.java  1.0.0 / 2008-04-29
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.xml.simple;

import java.io.IOException;

/**
 * XML Attributes.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public interface XMLAttributes {

  /**
   * Returns the number of the attributes.
   *
   * @return the number of the attributes
   */
  public int getAttributeCount();

  /**
   * Returns <tt>true</tt> if there is no attributes.
   *
   * @return <tt>true</tt> if there is no attributes
   */
  public boolean isEmpty();

  /**
   * Returns <tt>true</tt> if there is the specified attribute.
   *
   * @param attrName the attribute name
   *
   * @return <tt>true</tt> if there is the specified attribute
   */
  public boolean contains(String attrName);

  /**
   * Returns the value of specified attribute or <tt>null</tt> if there is no
   * such attribute.
   *
   * @param attrName the attribute name
   *
   * @return the attribute value or <tt>null</tt> if there is not such attribute
   */
  public String get(String attrName);

  /**
   * Returns the boolean value of the specified attribute.
   *
   * @param attrName the attribute name
   *
   * @return the boolean value of the specified attribute
   *
   * @throws XMLAttributeNotExists if there is not such attribute
   */
  public boolean getBoolean(String attrName) throws XMLAttributeNotExists;

  /**
   * Returns the boolean value of the specified attribute or default value if
   * there is no such attribute.
   *
   * @param attrName the attribute name
   * @param defValue the default value
   *
   * @return the boolean value of the specified attribute or default value
   */
  public boolean getBoolean(String attrName, boolean defValue);

  /**
   * Returns the byte value of the specified attribute.
   *
   * @param attrName the attribute name
   *
   * @return the byte value of the specified attribute
   *
   * @throws XMLAttributeNotExists if there is not such attribute
   */
  public byte getByte(String attrName) throws XMLAttributeNotExists;

  /**
   * Returns the byte value of the specified attribute or default value if there
   * is no such attribute.
   *
   * @param attrName the attribute name
   * @param defValue the default value
   *
   * @return the byte value of the specified attribute or default value
   */
  public byte getByte(String attrName, byte defValue);

  /**
   * Returns the char value of the specified attribute.
   *
   * @param attrName the attribute name
   *
   * @return char value of the specified attribute
   *
   * @throws XMLAttributeNotExists if there is not such attribute
   */
  public char getChar(String attrName) throws XMLAttributeNotExists;

  /**
   * Returns the char value of the specified attribute or default value if there
   * is no such attribute.
   *
   * @param attrName the attribute name
   * @param defValue the default value
   *
   * @return the char value of the specified attribute or default value
   */
  public char getChar(String attrName, char defValue);

  /**
   * Returns the short value of the specified attribute.
   *
   * @param attrName the attribute name
   *
   * @return short value of the specified attribute
   *
   * @throws XMLAttributeNotExists if there is not such attribute
   */
  public short getShort(String attrName) throws XMLAttributeNotExists;

  /**
   * Returns the short value of the specified attribute or default value if
   * there is no such attribute.
   *
   * @param attrName the attribute name
   * @param defValue the default value
   *
   * @return the short value of the specified attribute or default value
   */
  public short getShort(String attrName, short defValue);

  /**
   * Returns the integer value of the specified attribute.
   *
   * @param attrName the attribute name
   *
   * @return the integer value of the specified attribute
   *
   * @throws XMLAttributeNotExists if there is not such attribute
   */
  public int getInteger(String attrName) throws XMLAttributeNotExists;

  /**
   * Returns the integer value of the specified attribute or default value if
   * there is no such attribute.
   *
   * @param attrName the attribute name
   * @param defValue the default value
   *
   * @return the integer value of the specified attribute or default value
   */
  public int getInteger(String attrName, int defValue);

  /**
   * Returns the float value of the specified attribute.
   *
   * @param attrName the attribute name
   *
   * @return the float value of the specified attribute
   *
   * @throws XMLAttributeNotExists if there is not such attribute
   */
  public float getFloat(String attrName) throws XMLAttributeNotExists;

  /**
   * Returns the float value of the specified attribute or default value if
   * there is no such attribute.
   *
   * @param attrName the attribute name
   * @param defValue the default value
   *
   * @return the float value of the specified attribute or default value
   */
  public float getFloat(String attrName, float defValue);

  /**
   * Returns the double value of the specified attribute.
   *
   * @param attrName the attribute name
   *
   * @return the double value of the specified attribute
   *
   * @throws XMLAttributeNotExists if there is not such attribute
   */
  public double getDouble(String attrName) throws XMLAttributeNotExists;

  /**
   * Returns the double value of the specified attribute or default value if
   * there is no such attribute.
   *
   * @param attrName the attribute name
   * @param defValue the default value
   *
   * @return the double value of the specified attribute or default value
   */
  public double getDouble(String attrName, double defValue);

  /**
   * Returns the string value of the specified attribute.
   *
   * @param attrName the attribute name
   *
   * @return the string value of the specified attribute
   *
   * @throws XMLAttributeNotExists if there is no such attribute
   */
  public String getString(String attrName) throws XMLAttributeNotExists;

  /**
   * Returns the string value of the specified attribute or default value if
   * there is no such attribute.
   *
   * @param attrName attribute name
   * @param defValue default value
   *
   * @return the string value the specified attribute or default value
   */
  public String getString(String attrName, String defValue);

  /**
   * Prints all attributes using the specified XMLPrintServer.
   *
   * @param printServer the XMLPrintServer
   *
   * @throws IOException if an I/O error occurred
   */
  public void print(XMLPrintServer printServer) throws IOException;

}

/*^*****************************************************************************
  File History:

 - 2008-04-29 17:59:38 Slobodan
   Initial version

*****************************************************************************^*/
