/*
 * @(#)XMLElementStack.java  1.0.0 / 2008-04-23
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.xml.simple;

/**
 * Stack of XML Elements.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public class XMLElementStack {

/*^**************************************************************************^*/
/*- Public Static Fields.                                                    -*/
/*^**************************************************************************^*/

  /**
   * The default capacity of the stack.
   */
  public static final int DEFAULT_CAPACITY = 37;

  /**
   * The maximum capacity of the stack.
   */
  public static final int MAXIMUM_CAPACITY = 1 << 22;

/*^**************************************************************************^*/
/*- Private Fields.                                                          -*/
/*^**************************************************************************^*/

  private XMLElement[] stack;
  private int last;
  private boolean empty;

/*^**************************************************************************^*/
/*- Package-Private Constructor.                                             -*/
/*^**************************************************************************^*/

  /**
   * Constructs new XMLElementStack object.
   */
  public XMLElementStack() {
    stack = new XMLElement[DEFAULT_CAPACITY];
    last = -1;
    empty = true;
  }

/*^**************************************************************************^*/
/*- Public Methods.                                                          -*/
/*^**************************************************************************^*/

  /**
   * Returns <tt>true</tt> if the stack is empty.
   *
   * @return <tt>true</tt> if the stack is empty
   */
  public boolean isEmpty() {
    return empty;
  }

  /**
   * Returns the number of the elements on the stack.
   *
   * @return the number of the elements on the stack
   */
  public int getElementCount() {
    return last + 1;
  }

  /**
   * Puts the "multiline" xml element.
   *
   * @param name the element name
   *
   * @throws StackOverflowException if the number of the elements on the stack
   *         is MAXIMUM_CAPACITY
   */
  public void putMulitlineElement(String name) {
    putElement(name, XMLElement.MULTILINE_ELEMENT);
  }

  /**
   * Puts the "inline" xml element.
   *
   * @param name the element name
   *
   * @throws StackOverflowException if the number of the elements on the stack
   *         is MAXIMUM_CAPACITY
   */
  public void putInlineElement(String name) {
    putElement(name, XMLElement.INLINE_ELEMENT);
  }

  /**
   * Puts the "inline" xml element.
   *
   * @param name the element name
   *
   * @throws StackOverflowException if the number of the elements on the stack
   *         is MAXIMUM_CAPACITY
   */
  public void putEmptyElement(String name) {
    putElement(name, XMLElement.EMPTY_ELEMENT);
  }

  /**
   * Removes the last element.
   *
   * @throws StackEmptyException if the stack is empty
   */
  public void removeElement() {
    if (empty) {
      throw new StackEmptyException();
    }
    int last = this.last;
    if (last >= DEFAULT_CAPACITY) {
      int halfSize = stack.length >> 1;
      if (last <= halfSize) {
        // shrink stack capacity
        XMLElement[] oldStack = stack;
        stack = new XMLElement[halfSize];
        System.arraycopy(oldStack, 0, stack, 0, last);
      } else {
        // do not optimize
        stack[last].name = null; // cache element to avoid new allocation
      }
    } else {
      // do not optimize
      stack[last].name = null; // cache element to avoid new allocation
    }
    empty = (--last < 0);
    this.last = last;
  }

  /**
   * Removes all of the elements from the stack.
   */
  public void clear() {
    final XMLElement[] stack = this.stack;
    for (int i = last; i >= 0; i--) { // do not optimize
      stack[i] = null; // helps GC
    }
    this.stack = new XMLElement[DEFAULT_CAPACITY];
    last = -1;
    empty = true;
  }

  /**
   * Returns the first element name.
   *
   * @return the first element name
   *
   * @throws StackEmptyException if the stack is <tt>empty</tt>
   */
  public String firstElementName() {
    if (empty) {
      throw new StackEmptyException();
    }
    return stack[last].name;
  }

  /**
   * Returns the first element kind.
   *
   * @return the first element kind
   *
   * @throws StackEmptyException if the stack is <tt>empty</tt>
   */
  public int firstElementKind() {
    if (empty) {
      throw new StackEmptyException();
    }
    return stack[last].kind;
  }

  /**
   * Returns the second element name.
   *
   * @return the second element name
   *
   * @throws StackEmptyException if the stack is <tt>empty</tt> or contains only
   *         one element
   */
  public String secondElementName() {
    int second = last - 1;
    if (second < 0) {
      throw new StackEmptyException();
    }
    return stack[second].name;
  }

  /**
   * Returns the second element kind.
   *
   * @return the second element kind
   *
   * @throws StackEmptyException if the stack is <tt>empty</tt> or contains only
   *         one element
   */
  public int secondElementKind() {
    int second = last - 1;
    if (second < 0) {
      throw new StackEmptyException();
    }
    return stack[second].kind;
  }

/*^**************************************************************************^*/
/*- Protected Method.                                                        -*/
/*^**************************************************************************^*/

  /**
   * Puts new element onto the stack.
   *
   * @param name the element name
   * @param kind the element kind
   */
  protected void putElement(String name, int kind) {
    int last = this.last;
    last++;
    int oldSize = stack.length;
    if (last >= oldSize) {
      if (last == MAXIMUM_CAPACITY) {
        throw new StackOverflowException();
      }
      // enlarge stack capacity
      XMLElement[] oldStack = stack;
      int newSize = (3 * oldSize) >> 1;
      if (++newSize > MAXIMUM_CAPACITY) {
        newSize = MAXIMUM_CAPACITY;
      }
      stack = new XMLElement[newSize];
      System.arraycopy(oldStack, 0, stack, 0, last);
      stack[last] = new XMLElement(name, kind);
    } else {
      XMLElement le = stack[last];
      if (le == null) { // try to use cached element
        stack[last] = new XMLElement(name, kind);
      } else {
        le.name = name;
        le.kind = kind;
      }
    }
    empty = (last < 0);
    this.last = last;
  }

/*^**************************************************************************^*/
/*- Inner Class.                                                             -*/
/*^**************************************************************************^*/

  /**
   * Stack Empty Exception.
   */
  public static class StackEmptyException extends RuntimeException {
    private static final long serialVersionUID = -7865542118891169171L;
  }

  /**
   * Stack Empty Exception.
   */
  public static class StackOverflowException extends RuntimeException {
    private static final long serialVersionUID = 5736602760785139743L;
  }

}

/*^*****************************************************************************
  File History:

 - 2008-04-23 17:48:17 Slobodan
   Initial version

*****************************************************************************^*/
