/*
 * @(#)XMLPrintServer.java  1.0.0 / 2008-04-29
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.xml.simple;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.OutputStream;
import java.io.UnsupportedEncodingException;
import java.io.Writer;
import java.lang.ref.SoftReference;

/**
 * Simple XML Print Server.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public class XMLPrintServer {

/*^**************************************************************************^*/
/*- Public Static Fields.                                                    -*/
/*^**************************************************************************^*/

  /**
   * Initialization state.
   */
  public static final int STATE_INIT = 0;

  /**
   * Document start state.
   */
  public static final int STATE_DOC_START = 1;

  /**
   * Document end state.
   */
  public static final int STATE_DOC_END = 2;

  /**
   * Element start state.
   */
  public static final int STATE_ELEM_START = 3;

  /**
   * Element end state.
   */
  public static final int STATE_ELEM_END = 4;

  /**
   * Element content state.
   */
  public static final int STATE_ELEM_CONTENT = 5;

/*^**************************************************************************^*/
/*- Private Static Field.                                                    -*/
/*^**************************************************************************^*/

  private static SoftReference srTabCache;

/*^**************************************************************************^*/
/*- Static Initialization.                                                   -*/
/*^**************************************************************************^*/

  static {
    srTabCache = new SoftReference(createTabs());
  }

/*^**************************************************************************^*/
/*- Private Fields.                                                          -*/
/*^**************************************************************************^*/

  private XMLPrinter xmlPrinter;
  private XMLElementStack stack;
  private Object[] tabCache;
  private int level;
  private int state;

/*^**************************************************************************^*/
/*- Public Constructors.                                                     -*/
/*^**************************************************************************^*/

  /**
   * Constructs new XMLPrintServer object.
   *
   * @param xmlPrinter the XMLPrinter
   */
  public XMLPrintServer(XMLPrinter xmlPrinter) {
    if (xmlPrinter == null) {
      throw new IllegalArgumentException("Null XMLPrinter"); //$NON-NLS-1$
    }
    this.xmlPrinter = xmlPrinter;
    stack = new XMLElementStack();
    tabCache = getTabs();
  }

  /**
   * Constructs new XMLPrintServer object.
   *
   * @param writer the output writer
   */
  public XMLPrintServer(Writer writer) {
    this(new XMLPrinter(writer));
  }

  /**
   * Constructs new XMLPrintServer object.
   *
   * @param writer the output writer
   * @param closeStream close the stream
   */
  public XMLPrintServer(Writer writer, boolean closeStream) {
    this(new XMLPrinter(writer, closeStream));
  }

  /**
   * Constructs new XMLPrintServer object.
   *
   * @param outputStream the output stream
   */
  public XMLPrintServer(OutputStream outputStream) {
    this(new XMLPrinter(outputStream, true));
  }

  /**
   * Constructs new XMLPrintServer object.
   *
   * @param outputStream the output stream
   * @param charsetName the name of the character set
   *
   * @throws UnsupportedEncodingException if the encoding is not supported
   */
  public XMLPrintServer(OutputStream outputStream,
                        String charsetName) throws
                                                 UnsupportedEncodingException {
    this(new XMLPrinter(outputStream, charsetName, true));
  }

  /**
   * Constructs new XMLPrintServer object.
   *
   * @param outputStream the output stream
   * @param closeStream close the stream
   */
  public XMLPrintServer(OutputStream outputStream, boolean closeStream) {
    this(new XMLPrinter(outputStream, closeStream));
  }

  /**
   * Constructs new XMLPrintServer object.
   *
   * @param outputStream the output stream
   * @param charsetName the name of the character set
   * @param closeStream close the stream
   *
   * @throws UnsupportedEncodingException if the encoding is not supported
   */
  public XMLPrintServer(OutputStream outputStream, String charsetName,
                        boolean closeStream) throws
                                                 UnsupportedEncodingException {
    this(new XMLPrinter(outputStream, charsetName, closeStream));
  }

  /**
   * Constructs new XMLPrintServer object.
   *
   * @param file the output file
   *
   * @throws FileNotFoundException if the file not exists
   */
  public XMLPrintServer(File file) throws FileNotFoundException {
    this(new XMLPrinter(file));
  }

  /**
   * Constructs new XMLPrintServer object.
   *
   * @param file the output file
   * @param charsetName the name of the character set
   *
   * @throws FileNotFoundException if the file not exists
   * @throws UnsupportedEncodingException if the encoding is not supported
   */
  public XMLPrintServer(File file,
                        String charsetName) throws FileNotFoundException,
                                                  UnsupportedEncodingException {
    this(new XMLPrinter(file, charsetName));
  }

/*^**************************************************************************^*/
/*- Public Methods.                                                          -*/
/*^**************************************************************************^*/

  /**
   * Closes the output stream.
   *
   * @throws IOException if an I/O error occurred
   */
  public void close() throws IOException {
    if (xmlPrinter != null) {
      xmlPrinter.close();
      xmlPrinter = null;
      stack = null;
      state = STATE_DOC_END;
    }
  }

  /**
   * Returns XMLPrinter.
   *
   * @return XMLPrinter
   */
  public XMLPrinter getXMLPrinter() {
    return xmlPrinter;
  }

  /**
   * Returns state of the print server.
   *
   * @return state of the print server
   */
  public int getState() {
    return state;
  }

  /**
   * Returns kind of the current xml element.
   *
   * @return kind of the current xml element
   */
  public int getCurrentElementKind() {
    return stack.firstElementKind();
  }

  /**
   * Starts new XML document.
   *
   * @throws IOException if an I/O error occurred
   */
  public void beginDocument() throws IOException {
    beginDocument(null, null, null);
  }

  /**
   * Starts new XML document.
   *
   * @param doctype the XML document type
   *
   * @throws IOException if an I/O error occurred
   */
  public void beginDocument(String doctype) throws IOException {
    beginDocument(doctype, null, null);
  }

  /**
   * Starts new XML document.
   *
   * @param comment the XML comment
   * @param warning the XML warning
   *
   * @throws IOException if an I/O error occurred
   */
  public void beginDocument(String comment, String warning) throws IOException {
    beginDocument(null, comment, warning);
  }

  /**
   * Starts new XML document.
   *
   * @param doctype the XML document type
   * @param comment the XML comment
   * @param warning the XML warning
   *
   * @throws IOException if an I/O error occurred
   */
  public void beginDocument(String doctype, String comment,
                            String warning) throws IOException {
    beginDocument(XMLConstants.DEFAULT_XML_HEADER, doctype, comment, warning);
  }

  /**
   * Starts new XML document.
   *
   * @param header the XML header
   * @param doctype the XML document type
   * @param comment the XML comment
   * @param warning the XML warning
   *
   * @throws IOException if an I/O error occurred
   */
  public void beginDocument(String header, String doctype, String comment,
                            String warning) throws IOException {
    if (state != STATE_INIT) {
      throwInvalidState();
    }
    if (header == null) {
      xmlPrinter.printlnHeader();
    } else {
      xmlPrinter.printlnHeader(header);
    }
    xmlPrinter.writeln();
    if ((doctype != null) && (doctype.length() > 0)) {
      xmlPrinter.printlnDoctype(doctype);
      xmlPrinter.writeln();
    }
    if ((comment != null) && (comment.length() > 0)) {
      xmlPrinter.printlnComment(comment);
      xmlPrinter.writeln();
    }
    if ((warning != null) && (warning.length() > 0)) {
      xmlPrinter.println(warning);
      xmlPrinter.writeln();
    }
    state = STATE_DOC_START;
  }

  /**
   * Close XML document.
   *
   * @throws IOException if an I/O error occurred
   */
  public void endDocument() throws IOException {
    if (state != STATE_ELEM_END) {
      throwInvalidState();
    }
    while (!stack.isEmpty()) {
      endElement();
    }
    close();
  }

  /**
   * Begins new XML element.
   *
   * @param name the element name
   *
   * @throws IOException if an I/O error occurred
   */
  public void beginElement(String name) throws IOException {
    startElement(name);
    stack.putMulitlineElement(name);
    state = STATE_ELEM_START;
    level++;
  }

  /**
   * Begins new inline XML element.
   *
   * @param name the element name
   *
   * @throws IOException if an I/O error occurred
   */
  public void beginInlineElement(String name) throws IOException {
    startElement(name);
    stack.putInlineElement(name);
    state = STATE_ELEM_START;
  }

  /**
   * Begins new empty XML element.
   *
   * @param name element name
   *
   * @throws IOException if an I/O error occurred
   */
  public void beginEmptyElement(String name) throws IOException {
    startElement(name);
    stack.putEmptyElement(name);
    state = STATE_ELEM_START;
  }

  /**
   * Ends current element.
   *
   * @throws IOException if an I/O error occurred
   */
  public void endElement() throws IOException {
    if (stack.isEmpty()) {
      throwInvalidState();
    }
    switch (state) { // NOPMD
      case STATE_ELEM_START:
        switch (stack.firstElementKind()) {
          case XMLElement.INLINE_ELEMENT:
            xmlPrinter.printTagEnd();
            xmlPrinter.printlnEndElement(stack.firstElementName());
            break;
          case XMLElement.EMPTY_ELEMENT:
            if (stack.secondElementKind() == XMLElement.INLINE_ELEMENT) {
              xmlPrinter.printEmptyElementEnd();
            } else {
              xmlPrinter.printlnEmptyElementEnd();
            }
            break;
          default:
            xmlPrinter.printlnTagEnd();
            printTabs(--level);
            xmlPrinter.printlnEndElement(stack.firstElementName());
            break;
        }
        state = STATE_ELEM_END;
        break;
      case STATE_ELEM_CONTENT:
        if (stack.firstElementKind() == XMLElement.MULTILINE_ELEMENT) {
          xmlPrinter.writeln();
          printTabs(--level);
        }
        if (stack.secondElementKind() == XMLElement.INLINE_ELEMENT) {
          xmlPrinter.printEndElement(stack.firstElementName());
        } else {
          xmlPrinter.printlnEndElement(stack.firstElementName());
        }
        state = STATE_ELEM_END;
        break;
      case STATE_ELEM_END:
        switch (stack.firstElementKind()) {
          case XMLElement.MULTILINE_ELEMENT:
            printTabs(--level);
            xmlPrinter.printlnEndElement(stack.firstElementName());
            break;
          case XMLElement.INLINE_ELEMENT:
            if (stack.secondElementKind() == XMLElement.INLINE_ELEMENT) {
              xmlPrinter.printEndElement(stack.firstElementName());
            } else {
              xmlPrinter.printlnEndElement(stack.firstElementName());
            }
            break;
          default:
            printTabs(level);
            xmlPrinter.printlnEndElement(stack.firstElementName());
            break;
        }
        break;
      default:
        throwInvalidState();
    }
    stack.removeElement();
  }

  /**
   * Sets element attribute.
   *
   * @param name the attribute name
   * @param value the attribute value
   *
   * @throws IOException if an I/O error occurred
   */
  public void setAttribute(String name, String value) throws IOException {
    if ((name == null) || (name.length() == 0)) {
      throw new XMLStoringException("Invalid attribute name"); //$NON-NLS-1$
    }
    if (state == STATE_ELEM_START) {
      xmlPrinter.printAttribute(name, value);
    } else {
      throwInvalidState();
    }
  }

  /**
   * Sets XML element character array attribute.
   *
   * @param name attribute name
   * @param value attribute value
   *
   * @throws IOException if an I/O error occurs
   */
  public void setAttribute(String name, char[] value) throws IOException {
    setAttribute(name, String.valueOf(value));
  }

  /**
   * Sets XML element object attribute.
   *
   * @param name attribute name
   * @param value attribute value
   *
   * @throws IOException if an I/O error occurs
   */
  public void setAttribute(String name, Object value) throws IOException {
    if (value == null) {
      setAttribute(name, XMLConstants.STRING_NULL);
    } else {
      setAttribute(name, value.toString());
    }
  }

  /**
   * Sets XML element boolean attribute.
   *
   * @param name attribute name
   * @param value attribute value
   *
   * @throws IOException if an I/O error occurs
   */
  public void setAttribute(String name, boolean value) throws IOException {
    if (value) {
      setAttribute(name, XMLConstants.STRING_TRUE);
    } else {
      setAttribute(name, XMLConstants.STRING_FALSE);
    }
  }

  /**
   * Sets XML element char attribute.
   *
   * @param name attribute name
   * @param value attribute value
   *
   * @throws IOException if an I/O error occurs
   */
  public void setAttribute(String name, char value) throws IOException {
    setAttribute(name, String.valueOf(value));
  }

  /**
   * Sets XML element int attribute.
   *
   * @param name attribute name
   * @param value attribute value
   *
   * @throws IOException if an I/O error occurs
   */
  public void setAttribute(String name, int value) throws IOException {
    setAttribute(name, String.valueOf(value));
  }

  /**
   * Sets XML element long attribute.
   *
   * @param name attribute name
   * @param value attribute value
   *
   * @throws IOException if an I/O error occurs
   */
  public void setAttribute(String name, long value) throws IOException {
    setAttribute(name, String.valueOf(value));
  }

  /**
   * Sets XML element float attribute.
   *
   * @param name attribute name
   * @param value attribute value
   *
   * @throws IOException if an I/O error occurs
   */
  public void setAttribute(String name, float value) throws IOException {
    setAttribute(name, String.valueOf(value));
  }

  /**
   * Sets XML element double attribute.
   *
   * @param name attribute name
   * @param value attribute value
   *
   * @throws IOException if an I/O error occurs
   */
  public void setAttribute(String name, double value) throws IOException {
    setAttribute(name, String.valueOf(value));
  }

  /**
   * Appends string to the current element content.
   *
   * @param content string content
   *
   * @throws IOException if an I/O error occurred
   */
  public void appendContent(String content) throws IOException {
    switch (state) {
      case STATE_ELEM_START:
        switch (stack.firstElementKind()) {
          case XMLElement.MULTILINE_ELEMENT:
            xmlPrinter.printlnTagEnd();
            printTabs(level);
            break;
          case XMLElement.INLINE_ELEMENT:
            xmlPrinter.printTagEnd();
            break;
          default:
            throwInvalidState();
        }
        state = STATE_ELEM_CONTENT;
        break;
      case STATE_ELEM_CONTENT:
        break;
      case STATE_ELEM_END:
        if (stack.firstElementKind() == XMLElement.MULTILINE_ELEMENT) {
          printTabs(level);
        }
        state = STATE_ELEM_CONTENT;
        break;
      default:
        throwInvalidState();
    }
    printString(content);
  }

  /**
   * Appends boolean to the current element content.
   *
   * @param b the boolean content
   *
   * @throws IOException if an I/O error occurs
   */
  public void appendContent(boolean b) throws IOException {
    appendContent(Boolean.toString(b));
  }

  /**
   * Appends char to the current element content.
   *
   * @param c the char content
   *
   * @throws IOException if an I/O error occurs
   */
  public void appendContent(char c) throws IOException {
    appendContent(String.valueOf(c));
  }

  /**
   * Appends int to the current element content.
   *
   * @param i the int content
   *
   * @throws IOException if an I/O error occurs
   */
  public void appendContent(int i) throws IOException {
    appendContent(String.valueOf(i));
  }

  /**
   * Appends long to the current element content.
   *
   * @param l the long content
   *
   * @throws IOException if an I/O error occurs
   */
  public void appendContent(long l) throws IOException {
    appendContent(String.valueOf(l));
  }

  /**
   * Appends float to the current element content.
   *
   * @param f the float content
   *
   * @throws IOException if an I/O error occurs
   */
  public void appendContent(float f) throws IOException {
    appendContent(String.valueOf(f));
  }

  /**
   * Appends double to the current element content.
   *
   * @param d the double content
   *
   * @throws IOException if an I/O error occurs
   */
  public void appendContent(double d) throws IOException {
    appendContent(String.valueOf(d));
  }

  /**
   * Appends char array to the current element content.
   *
   * @param ca the the array of characters content
   *
   * @throws IOException if an I/O error occurs
   */
  public void appendContent(char[] ca) throws IOException {
    appendContent(String.valueOf(ca));
  }

  /**
   * Appends object to the current element content.
   *
   * @param o the object to be printed
   *
   * @throws IOException if an I/O error occurs
   */
  public void appendContent(Object o) throws IOException {
    if (o == null) {
      appendContent(XMLConstants.STRING_NULL);
    } else {
      appendContent(o.toString());
    }
  }

  /**
   * Appends new line.
   *
   * @throws IOException if an I/O error occurred
   */
  public void appendNewLine() throws IOException {
    if (state == STATE_ELEM_CONTENT) {
      xmlPrinter.writeln();
      printTabs(level);
    } else {
      throwInvalidState();
    }
  }

  /**
   * Appends space to the current element content.
   *
   * @throws IOException if an I/O error occurs
   */
  public void appendContentSpace() throws IOException {
    appendContent(XMLConstants.STRING_SPACE);
  }

  /**
   * Appends spaces to the current element content.
   *
   * @param count number of space characters
   *
   * @throws IOException if an I/O error occurs
   */
  public void appendContentSpaces(int count) throws IOException {
    if (count > 0) {
      char[] spaces = new char[count];
      for (int i = spaces.length; --i >= 0;) {
        spaces[i] = ' ';
      }
      appendContent(spaces);
    }
  }

/*^**************************************************************************^*/
/*- Private Methods.                                                         -*/
/*^**************************************************************************^*/

  private void printString(String s) throws IOException {
    int sl = s.length();
    if (sl > 0) {
      int ni = 0;
      int slm1 = sl - 1;
      char[] tabs = null;
      for (int i = 0; i < sl; i++) {
        if (s.charAt(i) == '\n') {
          if (i == slm1) {
            xmlPrinter.print(s.substring(ni, i));
            return;
          }
          xmlPrinter.print(s.substring(ni, ++i));
          ni = i;
          if (tabs == null) {
            tabs = new char [level]; // NOPMD
            for (int ti = level; --ti >= 0;) {
              tabs[ti] = '\t';
            }
          }
          xmlPrinter.print(tabs);
        }
      }
      if (ni == 0) {
        xmlPrinter.print(s);
      } else {
        xmlPrinter.print(s.substring(ni, sl));
      }
    }
  }

  private void printTabs(int count) throws IOException {
    if (count > 0) {
      if (count == tabCache.length) {
        Object[] oldTabCache = tabCache;
        tabCache = new Object[count + 1];
        System.arraycopy(oldTabCache, 0, tabCache, 0, count);
        char[] tabs = new char[count];
        int last = count - 1;
        System.arraycopy(tabCache[last], 0, tabs, 0,  last);
        tabs[last] = '\t';
        tabCache[count] = tabs;
      }
      xmlPrinter.print((char[])tabCache[count]);
    }
  }

  private void startElement(String name) throws IOException {
    if ((name == null) || (name.length() == 0)) {
      throw new XMLStoringException("Invalid element name"); //$NON-NLS-1$
    }
    switch (state) {
      case STATE_DOC_START:
        xmlPrinter.printTagStart(name);
        break;
      case STATE_ELEM_START:
        switch (stack.firstElementKind()) {
          case XMLElement.MULTILINE_ELEMENT:
            xmlPrinter.printlnTagEnd();
            printTabs(level);
            xmlPrinter.printTagStart(name);
            break;
          case XMLElement.INLINE_ELEMENT:
            xmlPrinter.printTagEnd();
            xmlPrinter.printTagStart(name);
            break;
          default:
            throwInvalidState();
        }
        break;
      case STATE_ELEM_END:
        printTabs(level);
        xmlPrinter.printTagStart(name);
        break;
      case STATE_ELEM_CONTENT:
        if (stack.firstElementKind() != XMLElement.INLINE_ELEMENT) {
          xmlPrinter.writeln();
          printTabs(level);
        }
        xmlPrinter.printTagStart(name);
        break;
      default:
        throwInvalidState();
    }
  }

/*^**************************************************************************^*/
/*- Private Static Methods.                                                  -*/
/*^**************************************************************************^*/

  private static Object[] createTabs() {
    Object[] tabs = new Object[6];
    int tlm = tabs.length - 1;
    char[] tbs = new char[tlm];
    char[] tbd;
    for (int ti = tbs.length; --ti >= 0;) {
      tbs[ti] = '\t';
    }
    tabs[tlm] = tbs;
    for (int i = tabs.length - 2; i > 0; i--) {
      tbd = new char[i]; // NOPMD
      System.arraycopy(tbs, 0, tbd, 0, i);
      tabs[i] = tbd;
    }
    return tabs;
  }

  private static Object[] getTabs() {
    Object[] tabs = (Object[])srTabCache.get();
    if (tabs == null) {
      tabs = createTabs();
      srTabCache = new SoftReference(createTabs());
    }
    return tabs;
  }

  private static void throwInvalidState() throws XMLStoringException {
    throw new XMLStoringException("Invalid XML server state"); //$NON-NLS-1$
  }

}

/*^*****************************************************************************
  File History:

 - 2008-04-29 18:01:34 Slobodan
   Initial version

*****************************************************************************^*/
