/*
 * @(#)XMLPrinter.java  1.0.0 / 2008-04-29
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.xml.simple;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.io.UnsupportedEncodingException;
import java.io.Writer;

/**
 * Simple XML Printer.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public class XMLPrinter {

/*^**************************************************************************^*/
/*- Private Fields.                                                          -*/
/*^**************************************************************************^*/

  private BufferedWriter bw;
  private boolean closeStream;

/*^**************************************************************************^*/
/*- Public Constructors.                                                     -*/
/*^**************************************************************************^*/

  /**
   * Constructs new XMLPrinter object.
   *
   * @param w the writer
   *
   * @throws IllegalArgumentException if the writer is <tt>null</tt>
   */
  public XMLPrinter(Writer w) {
    this(w, false);
  }

  /**
   * Constructs new XMLPrinter object.
   *
   * @param w the writer
   * @param closeStream if <tt>true</tt> method close() closes the stream.
   *
   * @throws IllegalArgumentException if the writer is <tt>null</tt>
   */
  public XMLPrinter(Writer w, boolean closeStream) {
    if (w == null) {
      throw new IllegalArgumentException("Null Writer"); //$NON-NLS-1$
    }
    bw = new BufferedWriter(w);
    this.closeStream = closeStream;
  }

  /**
   * Constructs new XMLPrinter object.
   *
   * @param outputStream the output stream
   *
   * @throws IllegalArgumentException if the output stream is <tt>null</tt>
   */
  public XMLPrinter(OutputStream outputStream) {
    this(outputStream, true);
  }

  /**
   * Constructs new XMLPrinter object.
   *
   * @param outputStream the output stream
   * @param charsetName the name of the charset
   *
   * @throws IllegalArgumentException if the output stream is <tt>null</tt>
   * @throws UnsupportedEncodingException if the encoding is not supported
   */
  public XMLPrinter(OutputStream outputStream,
                    String charsetName) throws UnsupportedEncodingException {
    this(outputStream, charsetName, true);
  }

  /**
   * Constructs new XMLPrinter object.
   *
   * @param outputStream the output stream
   * @param closeStream if <tt>true</tt> method close() closes the stream.
   *
   * @throws IllegalArgumentException if the output stream is <tt>null</tt>
   */
  public XMLPrinter(OutputStream outputStream, boolean closeStream) {
    if (outputStream == null) {
      throw new IllegalArgumentException("Null OutputStream"); //$NON-NLS-1$
    }
    bw = new BufferedWriter(new OutputStreamWriter(outputStream));
    this.closeStream = closeStream;
  }

  /**
   * Constructs new XMLPrinter object.
   *
   * @param outputStream the output stream
   * @param charsetName the name of the charset
   * @param closeStream if <tt>true</tt> method close() closes stream.
   *
   * @throws IllegalArgumentException if the output stream is <tt>null</tt>
   * @throws UnsupportedEncodingException if the encoding is not supported
   */
  public XMLPrinter(OutputStream outputStream, String charsetName,
                    boolean closeStream) throws UnsupportedEncodingException {
    if (outputStream == null) {
      throw new IllegalArgumentException("Null OutputStream"); //$NON-NLS-1$
    }
    bw = new BufferedWriter(new OutputStreamWriter(outputStream, charsetName));
    this.closeStream = closeStream;
  }

  /**
   * Constructs new XMLPrinter object.
   *
   * @param file the file
   *
   * @throws IllegalArgumentException if the file is <tt>null</tt>
   * @throws FileNotFoundException if the file not exists
   */
  public XMLPrinter(File file) throws FileNotFoundException {
    if (file == null) {
      throwNullFile();
    }
    bw = new BufferedWriter(new OutputStreamWriter(new FileOutputStream(file)));
    closeStream = true;
  }

  /**
   * Constructs new XMLPrinter object.
   *
   * @param file the file
   * @param charsetName the name of the charset
   *
   * @throws IllegalArgumentException if the file is <tt>null</tt>
   * @throws FileNotFoundException if the file not exists
   * @throws UnsupportedEncodingException if the encoding is not supported
   */
  public XMLPrinter(File file,
                    String charsetName) throws FileNotFoundException,
                                               UnsupportedEncodingException {
    if (file == null) {
      throwNullFile();
    }
    bw = new BufferedWriter(new OutputStreamWriter(new FileOutputStream(file),
                                                   charsetName));
    closeStream = true;
  }

  /**
   * Constructs new XMLPrinter object.
   *
   * @param file the file
   *
   * @throws IllegalArgumentException if the file is <tt>null</tt>
   * @throws FileNotFoundException if the file not exists
   */
  public XMLPrinter(String file) throws FileNotFoundException {
    if (file == null) {
      throwNullFile();
    }
    bw = new BufferedWriter(new OutputStreamWriter(new FileOutputStream(file)));
    closeStream = true;
  }

  /**
   * Constructs new XMLPrinter object.
   *
   * @param file the file
   * @param charsetName the name of the charset
   *
   * @throws IllegalArgumentException if the file is <tt>null</tt>
   * @throws FileNotFoundException if the file not exists
   * @throws UnsupportedEncodingException if the encoding is not supported
   */
  public XMLPrinter(String file,
                    String charsetName) throws FileNotFoundException,
                                                 UnsupportedEncodingException {
    if (file == null) {
      throwNullFile();
    }
    bw = new BufferedWriter(new OutputStreamWriter(new FileOutputStream(file),
                                                   charsetName));
    closeStream = true;
  }

/*^**************************************************************************^*/
/*- Public Methods.                                                          -*/
/*^**************************************************************************^*/

  /**
   * Close the stream.
   *
   * @throws IOException if an I/O error occurs
   */
  public void close() throws IOException {
    checkWriter();
    if (closeStream) {
      bw.close();
    } else {
      bw.flush();
    }
    bw = null;
  }

  /**
   * Flush the stream.
   *
   * @throws IOException if an I/O error occurs
   */
  public void flush() throws IOException {
    checkWriter();
    bw.flush();
  }

  /**
   * Directly writes a single character.
   *
   * @param c a character to be written
   *
   * @throws IOException if an I/O error occurs
   */
  public void write(int c) throws IOException {
    checkWriter();
    bw.write(c);
  }

  /**
   * Directly writes an array of characters.
   *
   * @param cbuf an array of characters to be written
   *
   * @throws IOException if an I/O error occurs
   */
  public void write(char[] cbuf) throws IOException {
    checkWriter();
    bw.write(cbuf);
  }

  /**
   * Directly writes an array of characters.
   *
   * @param cbuf an array of characters to be written
   * @param off array offset
   * @param len array length
   *
   * @throws IOException if an I/O error occurs
   */
  public void write(char[] cbuf, int off, int len) throws IOException {
    checkWriter();
    bw.write(cbuf, off, len);
  }

  /**
   * Directly writes a string.
   *
   * @param s a string to be written
   *
   * @throws IOException if an I/O error occurs
   */
  public void write(String s) throws IOException {
    checkWriter();
    bw.write(s);
  }

  /**
   * Directly writes a line separator.
   *
   * @throws IOException if an I/O error occurs
   */
  public void writeln() throws IOException {
    checkWriter();
    bw.newLine();
  }

  /**
   * Directly writes a string and a line separator.
   *
   * @param s a string to be written
   *
   * @throws IOException if an I/O error occurs
   */
  public void writeln(String s) throws IOException {
    checkWriter();
    bw.write(s);
    bw.newLine();
  }

  // Print methods that do not terminate lines. ------------------------------

  /**
   * Prints a boolean value.
   *
   * @param b the boolean to be printed
   *
   * @throws IOException if an I/O error occurs
   */
  public void print(boolean b) throws IOException {
    write(Boolean.toString(b));
  }

  /**
   * Prints a character value.
   *
   * @param c the character to be printed
   *
   * @throws IOException if an I/O error occurs
   */
  public void print(char c) throws IOException {
    write(String.valueOf(c));
  }

  /**
   * Prints an integer value.
   *
   * @param i the integer to be printed
   *
   * @throws IOException if an I/O error occurs
   */
  public void print(int i) throws IOException {
    write(String.valueOf(i));
  }

  /**
   * Prints a long value.
   *
   * @param l the long to be printed
   *
   * @throws IOException if an I/O error occurs
   */
  public void print(long l) throws IOException {
    write(String.valueOf(l));
  }

  /**
   * Prints a float value.
   *
   * @param f the float to be printed
   *
   * @throws IOException if an I/O error occurs
   */
  public void print(float f) throws IOException {
    write(String.valueOf(f));
  }

  /**
   * Prints a double value.
   *
   * @param d the double to be printed
   *
   * @throws IOException if an I/O error occurs
   */
  public void print(double d) throws IOException {
    write(String.valueOf(d));
  }

  /**
   * Prints an array of characters.
   *
   * @param cbuf the array of characters to be printed
   *
   * @throws IOException if an I/O error occurs
   */
  public void print(char[] cbuf) throws IOException {
    checkWriter();
    bw.write(cbuf, 0, cbuf.length);
  }

  /**
   * Print an array of characters.
   *
   * @param cbuf the array of characters to be printed
   * @param offset the start offset
   * @param length the number of character to write
   *
   * @throws IOException if an I/O error occurs
   */
  public void print(char[] cbuf, int offset, int length) throws IOException {
    checkWriter();
    bw.write(cbuf, offset, length);
  }

  /**
   * Prints a string.
   *
   * @param s the string to be printed
   *
   * @throws IOException if an I/O error occurs
   */
  public void print(String s) throws IOException {
    if (s == null) {
      write(XMLConstants.STRING_NULL);
    } else {
      write(s);
    }
  }

  /**
   * Prints an object.
   *
   * @param o the object to be printed
   *
   * @throws IOException if an I/O error occurs
   */
  public void print(Object o) throws IOException {
    if (o == null) {
      write(XMLConstants.STRING_NULL);
    } else {
      write(o.toString());
    }
  }

  // Print methods that do terminate lines. ----------------------------------

  /**
   * Terminates the current line.
   *
   * @throws IOException if an I/O error occurs
   */
  public void println() throws IOException {
    writeln();
  }

  /**
   * Prints a boolean value and terminate the line.
   *
   * @param b the boolean to be printed
   *
   * @throws IOException if an I/O error occurs
   */
  public void println(boolean b) throws IOException {
    writeln(Boolean.toString(b));
  }

  /**
   * Prints a character value and terminate the line.
   *
   * @param c the character to be printed
   *
   * @throws IOException if an I/O error occurs
   */
  public void println(char c) throws IOException {
    writeln(String.valueOf(c));
  }

  /**
   * Prints an integer value and terminate the line.
   *
   * @param i the integer value to be printed
   *
   * @throws IOException if an I/O error occurs
   */
  public void println(int i) throws IOException {
    writeln(String.valueOf(i));
  }

  /**
   * Prints a long value and terminate the line.
   *
   * @param l the long value to be printed
   *
   * @throws IOException if an I/O error occurs
   */
  public void println(long l) throws IOException {
    writeln(String.valueOf(l));
  }

  /**
   * Prints a float value and terminate the line.
   *
   * @param f the float value to be printed
   *
   * @throws IOException if an I/O error occurs
   */
  public void println(float f) throws IOException {
    writeln(String.valueOf(f));
  }

  /**
   * Prints a double value and terminate the line.
   *
   * @param d the double value to be printed
   *
   * @throws IOException if an I/O error occurs
   */
  public void println(double d) throws IOException {
    writeln(String.valueOf(d));
  }

  /**
   * Prints an array of characters and terminate the line.
   *
   * @param cbuf the array of characters to be printed
   *
   * @throws IOException if an I/O error occurs
   */
  public void println(char[] cbuf) throws IOException {
    checkWriter();
    bw.write(cbuf, 0, cbuf.length);
    bw.newLine();
  }

  /**
   * Prints an array of characters and terminate the line.
   *
   * @param cbuf the array of characters to be printed
   * @param offset the start offset
   * @param length the number of character to write
   *
   * @throws IOException if an I/O error occurs
   */
  public void println(char[] cbuf, int offset, int length) throws IOException {
    checkWriter();
    bw.write(cbuf, offset, length);
    bw.newLine();
  }

  /**
   * Print a string and terminate the line.
   *
   * @param s the string to be printed
   *
   * @throws IOException if an I/O error occurs
   */
  public void println(String s) throws IOException {
    if (s == null) {
      writeln(XMLConstants.STRING_NULL);
    } else {
      writeln(s);
    }
  }

  /**
   * Print an object and terminate the line.
   *
   * @param o the object to be printed
   *
   * @throws IOException if an I/O error occurs
   */
  public void println(Object o) throws IOException {
    if (o == null) {
      writeln(XMLConstants.STRING_NULL);
    } else {
      writeln(o.toString());
    }
  }

  // XML Print Methods. ------------------------------------------------------

  /**
   * Prints a space.
   *
   * @throws IOException if an I/O error occurs
   */
  public void printSpace() throws IOException {
    checkWriter();
    bw.write(XMLConstants.SPACE, 0, 1);
  }

  /**
   * Prints spaces.
   *
   * @param count the number of spaces to be printed
   *
   * @throws IOException if an I/O error occurs
   */
  public void printSpaces(int count) throws IOException {
    if (count < 0) {
      throw new XMLStoringException("Negative space count"); //$NON-NLS-1$
    } else if (count > 0) {
      checkWriter();
      char[] spaces = new char[count];
      for (int i = count; --i >= 0;) {
        spaces[i] = ' ';
      }
      bw.write(spaces, 0, count);
    }
  }

  /**
   * Prints a tab.
   *
   * @throws IOException if an I/O error occurs
   */
  public void printTab() throws IOException {
    checkWriter();
    bw.write(XMLConstants.TAB, 0, 1);
  }

  /**
   * Prints tabs.
   *
   * @param count tab the number of tabs to be printed
   *
   * @throws IOException if an I/O error occurs
   */
  public void printTabs(int count) throws IOException {
    if (count < 0) {
      throw new XMLStoringException("Negative space count"); //$NON-NLS-1$
    } else if (count > 0) {
      checkWriter();
      char[] tab = new char[count];
      for (int i = count; --i >= 0;) {
        tab[i] = '\t';
      }
      bw.write(tab, 0, count);
    }
  }

  /**
   * Prints tag start.
   *
   * @param name element name
   *
   * @throws IOException if an I/O error occurs
   */
  public void printTagStart(String name) throws IOException {
    checkWriter();
    bw.write(XMLConstants.TAG_START, 0, 1);
    if (name == null) {
      bw.write(XMLConstants.STRING_NULL);
    } else {
      bw.write(name);
    }
  }

  /**
   * Prints end tag start.
   *
   * @throws IOException if an I/O error occurs
   */
  public void printEndTagStart() throws IOException {
    checkWriter();
    bw.write(XMLConstants.END_TAG_START, 0, 2);
  }

  /**
   * Prints end element.
   *
   * @param name element name
   *
   * @throws IOException if an I/O error occurs
   */
  public void printEndElement(String name) throws IOException {
    checkWriter();
    bw.write(XMLConstants.END_TAG_START, 0, 2);
    if (name == null) {
      bw.write(XMLConstants.STRING_NULL);
    } else {
      bw.write(name);
    }
    bw.write(XMLConstants.TAG_END, 0, 1);
  }

  /**
   * Prints end element and terminate the line.
   *
   * @param name element name
   *
   * @throws IOException if an I/O error occurs
   */
  public void printlnEndElement(String name) throws IOException {
    checkWriter();
    bw.write(XMLConstants.END_TAG_START, 0, 2);
    if (name == null) {
      bw.write(XMLConstants.STRING_NULL);
    } else {
      bw.write(name);
    }
    bw.write(XMLConstants.TAG_END, 0, 1);
    bw.newLine();
  }

  /**
   * Prints tag end.
   *
   * @throws IOException if an I/O error occurs
   */
  public void printTagEnd() throws IOException {
    checkWriter();
    bw.write(XMLConstants.TAG_END, 0, 1);
  }

  /**
   * Prints tag end and terminate the line.
   *
   * @throws IOException if an I/O error occurs
   */
  public void printlnTagEnd() throws IOException {
    checkWriter();
    bw.write(XMLConstants.TAG_END, 0, 1);
    bw.newLine();
  }

  /**
   * Prints empty tag end.
   *
   * @throws IOException if an I/O error occurs
   */
  public void printEmptyElementEnd() throws IOException {
    checkWriter();
    bw.write(XMLConstants.EMPTY_ELEM_END, 0, 2);
  }

  /**
   * Prints empty tag end and terminate the line.
   *
   * @throws IOException if an I/O error occurs
   */
  public void printlnEmptyElementEnd() throws IOException {
    checkWriter();
    bw.write(XMLConstants.EMPTY_ELEM_END, 0, 2);
    bw.newLine();
  }

  /**
   * Prints string attribute of the XML element.
   *
   * @param name the attribute name
   * @param value the attribute value
   *
   * @throws IOException if an I/O error occurs
   */
  public void printAttribute(String name, String value) throws IOException {
    checkWriter();
    if ((name != null) && (name.length() > 0)) {
      bw.write(XMLConstants.SPACE, 0, 1);
      bw.write(name);
      bw.write(XMLConstants.ATTR_START, 0, 2);
      if (value == null) {
        bw.write(XMLConstants.STRING_NULL);
      } else {
        bw.write(value);
      }
      bw.write(XMLConstants.ATTR_END, 0, 1);
    } else {
      throw new XMLStoringException("Invalid attribute name"); //$NON-NLS-1$
    }
  }

  /**
   * Prints character array attribute of the XML element.
   *
   * @param name the attribute name
   * @param value the attribute value
   *
   * @throws IOException if an I/O error occurs
   */
  public void printAttribute(String name, char[] value) throws IOException {
    printAttribute(name, String.valueOf(value));
  }

  /**
   * Prints object attribute of the XML element.
   *
   * @param name the attribute name
   * @param value the attribute value
   *
   * @throws IOException if an I/O error occurs
   */
  public void printAttribute(String name, Object value) throws IOException {
    if (value == null) {
      printAttribute(name, XMLConstants.STRING_NULL);
    } else {
      printAttribute(name, value.toString());
    }
  }

  /**
   * Prints boolean attribute of the XML element.
   *
   * @param name the attribute name
   * @param value the attribute value
   *
   * @throws IOException if an I/O error occurs
   */
  public void printAttribute(String name, boolean value) throws IOException {
    printAttribute(name, Boolean.toString(value));
  }

  /**
   * Prints char attribute of the XML element.
   *
   * @param name the attribute name
   * @param value the attribute value
   *
   * @throws IOException if an I/O error occurs
   */
  public void printAttribute(String name, char value) throws IOException {
    printAttribute(name, String.valueOf(value));
  }

  /**
   * Prints int attribute of the XML element.
   *
   * @param name the attribute name
   * @param value the attribute value
   *
   * @throws IOException if an I/O error occurs
   */
  public void printAttribute(String name, int value) throws IOException {
    printAttribute(name, String.valueOf(value));
  }

  /**
   * Prints long attribute of the XML element.
   *
   * @param name the attribute name
   * @param value the attribute value
   *
   * @throws IOException if an I/O error occurs
   */
  public void printAttribute(String name, long value) throws IOException {
    printAttribute(name, String.valueOf(value));
  }

  /**
   * Prints float attribute of the XML element.
   *
   * @param name the attribute name
   * @param value the attribute value
   *
   * @throws IOException if an I/O error occurs
   */
  public void printAttribute(String name, float value) throws IOException {
    printAttribute(name, String.valueOf(value));
  }

  /**
   * Prints double attribute of the XML element.
   *
   * @param name the attribute name
   * @param value the attribute value
   *
   * @throws IOException if an I/O error occurs
   */
  public void printAttribute(String name, double value) throws IOException {
    printAttribute(name, String.valueOf(value));
  }

  /**
   * Prints XML header and terminate the line.
   *
   * @param header the header
   *
   * @throws IOException if an I/O error occurs
   */
  public void printlnHeader(String header) throws IOException {
    checkWriter();
    bw.write(XMLConstants.HEADER_START, 0, 2);
    bw.write(header);
    bw.write(XMLConstants.HEADER_END, 0, 2);
    bw.newLine();
  }

  /**
   * Prints XML header and terminate the line.
   *
   * @throws IOException if an I/O error occurs
   */
  public void printlnHeader() throws IOException {
    printlnHeader(XMLConstants.DEFAULT_XML_HEADER);
  }

  /**
   * Prints XML header and terminate the line.
   *
   * @param type the document type
   *
   * @throws IOException if an I/O error occurs
   */
  public void printlnDoctype(String type) throws IOException {
    checkWriter();
    bw.write(XMLConstants.DOCTYPE);
    bw.write(XMLConstants.SPACE);
    bw.write(type);
    bw.write(XMLConstants.TAG_END);
    bw.newLine();
  }

  /**
   * Prints comment start.
   *
   * @throws IOException if an I/O error occurs
   */
  public void printCommentStart() throws IOException {
    checkWriter();
    bw.write(XMLConstants.COMMENT_START, 0, 5);
  }

  /**
   * Prints comment start and terminate the line.
   *
   * @throws IOException if an I/O error occurs
   */
  public void printlnCommentStart() throws IOException {
    checkWriter();
    bw.write(XMLConstants.COMMENT_START, 0, 5);
    bw.newLine();
  }

  /**
   * Prints comment end.
   *
   * @throws IOException if an I/O error occurs
   */
  public void printCommentEnd() throws IOException {
    checkWriter();
    bw.write(XMLConstants.COMMENT_END, 0, 4);
  }

  /**
   * Prints comment end and terminate the line.
   *
   * @throws IOException if an I/O error occurs
   */
  public void printlnCommentEnd() throws IOException {
    checkWriter();
    bw.write(XMLConstants.COMMENT_END, 0, 4);
    bw.newLine();
  }

  /**
   * Prints comment and terminate the line.
   *
   * @param comment the comment
   *
   * @throws IOException if an I/O error occurs
   */
  public void printlnComment(String comment) throws IOException {
    if ((comment != null) && (comment.length() > 0)) {
      checkWriter();
      bw.write(XMLConstants.COMMENT_START, 0, 5);
      bw.write(comment);
      bw.write(XMLConstants.COMMENT_END, 0, 4);
      bw.newLine();
    }
  }

/*^**************************************************************************^*/
/*- Private Method.                                                          -*/
/*^**************************************************************************^*/

  private final void checkWriter() throws XMLStoringException {
    if (bw == null) {
      throw new XMLStoringException("Writer is closed"); //$NON-NLS-1$
    }
  }

  private final void throwNullFile() {
    throw new IllegalArgumentException("Null File"); //$NON-NLS-1$
  }

}

/*^*****************************************************************************
  File History:

 - 2008-04-29 18:00:43 Slobodan
   Initial version

*****************************************************************************^*/
