/*
 * @(#)Parser.java  1.0.0 / 2008-04-26
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.xml.simple.sax;

import java.io.IOException;
import java.io.Reader;

import net.wrka.xml.simple.DefaultXMLAttributes;

/**
 * Modified Simple SAX parser. This parser is based on QDParser
 * (http://www.javaworld.com/javaworld/javatips/javatip128/xmlparsertip.zip)
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public final class Parser { // NOPMD

/*^**************************************************************************^*/
/*- Private Static Final Fields.                                             -*/
/*^**************************************************************************^*/

  private static final int STATE_TEXT = 1;
  private static final int STATE_ENTITY = 2;
  private static final int STATE_OPEN_TAG = 3;
  private static final int STATE_CLOSE_TAG = 4;
  private static final int STATE_START_TAG = 5;
  private static final int STATE_ATTRIBUTE_LVALUE = 6;
  private static final int STATE_ATTRIBUTE_EQUAL = 9;
  private static final int STATE_ATTRIBUTE_RVALUE = 10;
  private static final int STATE_QUOTE = 7;
  private static final int STATE_IN_TAG = 8;
  private static final int STATE_DONE = 11;
  private static final int STATE_SINGLE_TAG = 12;
  private static final int STATE_COMMENT = 13;
  private static final int STATE_DOCTYPE = 14;
  private static final int STATE_PRE = 15;
  private static final int STATE_CDATA = 16;

/*^**************************************************************************^*/
/*- Private Constructor.                                                     -*/
/*^**************************************************************************^*/

 /**
  * Class has no instances.
  */
  private Parser() {
    // empty constructor body
  }

/*^**************************************************************************^*/
/*- Public Static Method.                                                    -*/
/*^**************************************************************************^*/

  /**
   * XML parsing.
   *
   * @param reader the XML reader
   * @param docHandler the XML document handler
   *
   * @throws ParsingException if an XML parsing error occurs
   * @throws IOException if an I/O error occurs
   * @throws IllegalArgumentException if the reader is <tt>null</tt> or the
   *                                  document handler is <tt>null</tt>
   */
  public static void parse(Reader reader, // NOPMD
                           DocumentHandler docHandler) throws IOException,
                                                              ParsingException {
    if (reader == null) {
      throw new IllegalArgumentException("Null reader"); //$NON-NLS-1$
    }
    if (docHandler == null) {
      throw new IllegalArgumentException("Null document handler"); //$NON-NLS-1$
    }
    final StateStack stack = new StateStack();
    int depth = 0;
    int state = STATE_PRE;
    int c = 0;
    int quotec = '"';
    int line = 1;
    int position = 0;
    int t = 0;
    StringBuffer sb = new StringBuffer();
    StringBuffer etag = new StringBuffer();
    String tagName = null;
    String lvalue = null;
    String rvalue = null;
    DefaultXMLAttributes attrs = new DefaultXMLAttributes();
    boolean eol = false;
    docHandler.startDocument();
    while ((c = reader.read()) != -1) {
      // Map \r, \r\n, and \n to \n (see XML spec section 2.11)
      if (c == '\n' && eol) {
        eol = false;
        continue;
      } else if (eol) {
        eol = false;
      } else if (c == '\n') {
        line++;
        position = 0;
      } else if (c == '\r') {
        eol = true;
        c = '\n';
        line++;
        position = 0;
      } else {
        position++;
      }
      if (state == STATE_DONE) {
        docHandler.endDocument();
        return;
      // We are between tags collecting text.
      } else if (state == STATE_TEXT) {
        if (c == '<') {
          stack.push(state);
          state = STATE_START_TAG;
          if (sb.length() > 0) {
            docHandler.characters(sb.toString());
            sb.setLength(0);
          }
        } else if (c == '&') {
          stack.push(state);
          state = STATE_ENTITY;
          etag.setLength(0);
        } else {
          sb.append((char)c);
        }
      // we are processing a closing tag: e.g. </foo>
      } else if (state == STATE_CLOSE_TAG) {
        if (c == '>') {
          state = stack.pop();
          tagName = sb.toString();
          sb.setLength(0);
          depth--;
          if (depth == 0) {
            state = STATE_DONE;
          }
          docHandler.endElement(tagName);
        } else {
          sb.append((char)c);
        }
      // we are processing CDATA
      } else if (state == STATE_CDATA) {
        t = sb.length();
        if ((c == '>') && (sb.charAt(t - 2) == ']') &&
            (sb.charAt(t - 1) == ']')) {
          sb.setLength(sb.length() - 2);
          docHandler.characters(sb.toString());
          sb.setLength(0);
          state = stack.pop();
        } else {
          sb.append((char)c);
        }
      // we are processing a comment.  We are inside
      // the <!-- .... --> looking for the -->.
      } else if (state == STATE_COMMENT) {
        t = sb.length();
        if ((c == '>') && (sb.charAt(t - 2) == '-') &&
            (sb.charAt(t - 1) == '-')) {
          sb.setLength(0);
          state = stack.pop();
        } else {
          sb.append((char)c);
        }
      // We are outside the root tag element
      } else if (state == STATE_PRE) {
        if (c == '<') {
          state = STATE_TEXT;
          stack.push(state);
          state = STATE_START_TAG;
        }
      // We are inside one of these <? ... ?>
      // or one of these <!DOCTYPE ... >
      } else if (state == STATE_DOCTYPE) {
        if (c == '>') {
          state = stack.pop();
          if (state == STATE_TEXT) {
            state = STATE_PRE;
          }
        }
      // we have just seen a < and
      // are wondering what we are looking at
      // <foo>, </foo>, <!-- ... --->, etc.
      } else if (state == STATE_START_TAG) {
        state = stack.pop();
        if (c == '/') {
          stack.push(state);
          state = STATE_CLOSE_TAG;
        } else if (c == '?') {
          state = STATE_DOCTYPE;
        } else {
          stack.push(state);
          state = STATE_OPEN_TAG;
          tagName = null;
          attrs.clear();
          sb.append((char)c);
        }
      // we are processing an entity, e.g. &lt;, &#187;, etc.
      } else if (state == STATE_ENTITY) {
        if (c == ';') {
          state = stack.pop();
          String cent = etag.toString();
          etag.setLength(0);
          if (cent.equals("lt")) { // NOPMD
            sb.append('<');
          } else if (cent.equals("gt")) { //$NON-NLS-1$ NOPMD
            sb.append('>');
          } else if (cent.equals("amp")) { //$NON-NLS-1$ NOPMD
            sb.append('&');
          } else if (cent.equals("quot")) { //$NON-NLS-1$ NOPMD
            sb.append('"');
          } else if (cent.equals("apos")) { //$NON-NLS-1$ NOPMD
            sb.append('\'');
          } else if (cent.startsWith("#")) { //$NON-NLS-1$ NOPMD
            sb.append((char)Integer.parseInt(cent.substring(1)));
            // Insert custom entity definitions here
          } else {
            throw new ParsingException("Unknown entity: &" + //$NON-NLS-1$
                                       cent + "; line: " + line + //$NON-NLS-1$
                                       ", position: " + position); //$NON-NLS-1$
          }
        } else {
          etag.append((char)c);
        }
      // we have just seen something like this:
      // <foo a="b"/
      // and are looking for the final >.
      } else if (state == STATE_SINGLE_TAG) {
        if (tagName == null) {
          tagName = sb.toString();
        }
        if (c != '>') {
          throw new ParsingException("Expected > for tag: <" + //$NON-NLS-1$
                                     tagName + "/> at line " + //$NON-NLS-1$
                                     line + ", position: " + //$NON-NLS-1$
                                     position);
        }
        docHandler.startElement(tagName, attrs);
        docHandler.endElement(tagName);
        if (depth == 0) {
          docHandler.endDocument();
          return;
        }
        sb.setLength(0);
        attrs.clear();
        tagName = null;
        state = stack.pop();
      // we are processing something
      // like this <foo ... >.  It could
      // still be a <!-- ... --> or something.
      } else if (state == STATE_OPEN_TAG) {
        if (c == '>') {
          if (tagName == null) {
            tagName = sb.toString();
          }
          sb.setLength(0);
          depth++;
          docHandler.startElement(tagName, attrs);
          tagName = null;
          attrs.clear();
          state = stack.pop();
        } else if (c == '/') {
          state = STATE_SINGLE_TAG;
        } else if (c == '-' && sbsEquals(sb, "!-")) {
          state = STATE_COMMENT;
        } else if (c == '[' && sbsEquals(sb, "![CDATA")) { //$NON-NLS-1$
          state = STATE_CDATA;
          sb.setLength(0);
        } else if (c == 'E' && sbsEquals(sb, "!DOCTYP")) { //$NON-NLS-1$
          sb.setLength(0);
          state = STATE_DOCTYPE;
        } else if (Character.isWhitespace((char)c)) {
          tagName = sb.toString();
          sb.setLength(0);
          state = STATE_IN_TAG;
        } else {
          sb.append((char)c);
        }
        // We are processing the quoted right-hand side
        // of an element's attribute.
      } else if (state == STATE_QUOTE) {
        if (c == quotec) {
          rvalue = sb.toString();
          sb.setLength(0);
          attrs.put(lvalue, rvalue);
          state = STATE_IN_TAG;
          // See section the XML spec, section 3.3.3
          // on normalization processing.
        } else if (" \r\n\u0009".indexOf(c) >= 0) {
          sb.append(' ');
        } else if (c == '&') {
          stack.push(state);
          state = STATE_ENTITY;
          etag.setLength(0);
        } else {
          sb.append((char)c);
        }
      } else if (state == STATE_ATTRIBUTE_RVALUE) {
        if (c == '"' || c == '\'') {
          quotec = c;
          state = STATE_QUOTE;
        } else if (!Character.isWhitespace((char)c)) {
          throw new ParsingException("Error parsing attribute", //$NON-NLS-1$
                                     line, position);
        }
      } else if (state == STATE_ATTRIBUTE_LVALUE) {
        if (Character.isWhitespace((char)c)) {
          lvalue = sb.toString();
          sb.setLength(0);
          state = STATE_ATTRIBUTE_EQUAL;
        } else if (c == '=') {
          lvalue = sb.toString();
          sb.setLength(0);
          state = STATE_ATTRIBUTE_RVALUE;
        } else {
          sb.append((char)c);
        }
      } else if (state == STATE_ATTRIBUTE_EQUAL) {
        if (c == '=') {
          state = STATE_ATTRIBUTE_RVALUE;
        } else if (!Character.isWhitespace((char)c)) {
          throw new ParsingException("Error parsing attribute", //$NON-NLS-1$
                                     line, position);
        }
      } else if (state == STATE_IN_TAG) {
        if (c == '>') {
          state = stack.pop();
          docHandler.startElement(tagName, attrs);
          depth++;
          tagName = null;
          attrs.clear();
        } else if (c == '/') {
          state = STATE_SINGLE_TAG;
        } else if (!Character.isWhitespace((char)c)) {
          state = STATE_ATTRIBUTE_LVALUE;
          sb.append((char)c);
        }
      }
    }
    if (state == STATE_DONE) {
      docHandler.endDocument();
    } else {
      throw new ParsingException("Missing end tag", line, //$NON-NLS-1$
                                 position);

    }
  }

/*^**************************************************************************^*/
/*- Private Static Method.                                                   -*/
/*^**************************************************************************^*/

  private static boolean sbsEquals(StringBuffer sb, String s) {
    int sbl = sb.length();
    int sl = s.length();
    if (sbl == sl) {
      for (int i = sbl; --i >= 0;) {
        if (s.charAt(i) != sb.charAt(i)) {
          return false;
        }
      }
      return true;
    }
    return false;
  }

/*^**************************************************************************^*/
/*- Package-Private Static Inner Class.                                      -*/
/*^**************************************************************************^*/

  /**
   * Stack of the Parser's states.
   */
  static final class StateStack {

  // Private Fields. ---------------------------------------------------------

    private transient int[] stack;
    private transient int top;

  // Public Constructors. ----------------------------------------------------

    /**
     * Constructs new StateStack object.
     *
     * @param depth the initial depth
     */
    public StateStack(int depth) {
      stack = new int[depth];
    }

    /**
     * Constructs new StateStack object.
     */
    public StateStack() {
      this(10);
    }

  // Public Methods. ---------------------------------------------------------

    /**
     * Pushes the specified integer onto the stack.
     *
     * @param i the integer
     */
    public void push(int i) { // NOPMD
      int oldSize = stack.length;
      int minSize = top;
      if (++minSize > oldSize) {
        int[] oldStack = stack;
        int newSize = (oldSize << 1);
        if (++newSize < minSize) {
          newSize = minSize;
        }
        stack = new int[newSize];
        System.arraycopy(oldStack, 0, stack, 0, top);
      }
      stack[top++] = i;
    }

    /**
     * Returns the last element from the stack.
     *
     * @return the last element from the stack
     */
    public int pop() {
      if (top > 0) {
        return stack[--top];
      }
      return STATE_PRE;
    }

  }

}

/*^*****************************************************************************
  File History:

 - 2008-04-26 11:47:23 Slobodan
   Initial version

*****************************************************************************^*/
