/*
 * @(#)MJAsmScanner.java  1.0.0 / 2008-05-25
 *
 * Copyright (c) 2008, ETF and contributors. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package yu.ac.bg.etf.ri4pp.mjtools.asm;

// CS:OFF

import java.io.IOException;
import java.io.InputStream;
import java.io.Reader;
import java_cup.runtime.Scanner;
import java_cup.runtime.Symbol;

/**
 * MJ Assembler Scanner.
 * <p>
 *   <tt>NOTE: This code is generated by JFlex 1.4.2</tt>
 * </p>
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
%%
%init{
  lineBreaks = new MJAsmLineBreaks();
%init}
%{
/*^**************************************************************************^*/
/*- Package-Private Static Fields.                                           -*/
/*^**************************************************************************^*/

  static final String ERR_UNCLOSED_CCONST = "Unclosed character constant";
  static final String ERR_INVALID_CCONST = "Invalid character constant";
  static final String ERR_INVALID_NUMBER = "Invalid number";
  static final String ERR_INVALID_LABEL = "Invalid label";

/*^**************************************************************************^*/
/*- Private Fields.                                                          -*/
/*^**************************************************************************^*/

  private int commentLevel;
  private int commentStart;
  private int commentLine;
  private int commentColumn;
  private boolean reportComments;
  private boolean commentEOF;
  private MJAsmLineBreaks lineBreaks;

/*^**************************************************************************^*/
/*- Public Constructors.                                                     -*/
/*^**************************************************************************^*/

  /**
   * Creates new MJAsmScanner object.
   *
   * @param in the Reader to read input from
   * @param reportComments <tt>true</tt> if scanner should report comments
   */
  public MJAsmScanner(Reader in, boolean reportComments) {
    this(in);
    this.reportComments = reportComments;
  }

  /**
   * Creates new MJAsmScanner object.
   *
   * @param is the InputStream to read input from
   * @param reportComments <tt>true</tt> if scanner should report comments
   */
  public MJAsmScanner(InputStream is, boolean reportComments) {
    this(is);
    this.reportComments = reportComments;
  }

/*^**************************************************************************^*/
/*- Public Methods.                                                          -*/
/*^**************************************************************************^*/

  /**
   * Returns line breaks.
   *
   * @return line breaks
   */
  public MJAsmLineBreaks getLineBreaks() {
    return lineBreaks;
  }

  /**
   * Returns next MJAsmToken.
   *
   * @return next MJAsmToken
   *
   * @throws IOException on an I/O Error
   */
  public MJAsmToken nextToken() throws IOException {
    return (MJAsmToken)next_token();
  }

/*^**************************************************************************^*/
/*- Private Methods.                                                         -*/
/*^**************************************************************************^*/

  private MJAsmToken createToken(int type) {
    return createToken(type, null);
  }

  private MJAsmToken createToken(int type, Object val) {
    return new MJAsmToken(type, yychar, yychar + yylength() - 1, val, yyline,
                          yycolumn);
  }

  private MJAsmToken createCommentToken() {
    return new MJAsmToken(MJAsmToken.COMMENT, commentStart, yychar + 1,
                          null, commentLine, commentColumn);
  }

  private MJAsmToken createInvalidToken(String val) {
    return new MJAsmToken(MJAsmToken.INVALID, yychar, yychar + yylength() - 1,
                          val, yyline, yycolumn);
  }

  /* end of user code */
%}

%public
%final
%class MJAsmScanner
%implements Scanner
%function next_token
%type Symbol
%caseless
%unicode
%char
%line
%column
%state ML_COMMENT
%pack

Digit = ([0-9])
Letter = ([a-zA-Z_])
Blank = ([ ])
Tab = ([\t])
FF = ([\f])
CR = ([\r])
LF = ([\n])
PrintableChar = ([\x20-\x7F])
EOL = ({CR}{LF}|{CR}|{LF})
WhiteSpace = ({Blank}|{Tab}|{FF})
Number=([\-]?{Digit}+)
Label = ({Letter}({Letter}|{Digit})*)
AnyNonSep = ([^\t\f\r\n\ \(\)\{\}\[\]\;\,\.\=\>\<\!\~\?\:\+\-\*\/\&\|\^\%\"\'])
InvalidLabel=({AnyNonSep}+)
UnclosedCharConstChar = ([\x20-\x26\x28-\x3A\0x3C-\x7F])
UnclosedCharConst = (\'{UnclosedCharConstChar}{0,2})
CharConst = (\'((\\)?{PrintableChar}?)\')
InlineComment =(("//"|";")[^\r\n]*)
MultiLineCommentStart = ("/*")
MultiLineCommentEnd = ("*/")

%%

<ML_COMMENT> {
  {MultiLineCommentStart} {
    commentLevel++;
  }
  {MultiLineCommentEnd} {
    if (--commentLevel == 0) {
      yybegin(YYINITIAL);
      if (reportComments) {
        return createCommentToken();
      }
    }
  }
  {WhiteSpace} |
  . {
    // no action
  }
  <<EOF>> {
    if (reportComments) {
      if (commentEOF) {
        return null;
      }
      commentEOF = true;
      return createCommentToken();
    }
  }
  {EOL} {
    lineBreaks.breakLineAt(yychar + yytext().length());
  }
}

<YYINITIAL> {
  // Comments
  {MultiLineCommentStart} {
    yybegin(ML_COMMENT);
    commentLevel++;
    commentStart = yychar;
    commentLine = yyline;
    commentColumn = yycolumn;
  }
  {InlineComment} {
    if (reportComments) {
      return createToken(MJAsmToken.COMMENT);
    }
  }

  // Directives
  ".ds" {
    return createToken(MJAsmToken.DS, ".ds");
  }
  ".org" {
    return createToken(MJAsmToken.ORG, ".org");
  }

  // Instructions
  "load" {
    return createToken(MJAsmToken.LOAD, "load");
  }
  "load_0" {
    return createToken(MJAsmToken.LOAD_0, "load_0");
  }
  "load_1" {
    return createToken(MJAsmToken.LOAD_1, "load_1");
  }
  "load_2" {
    return createToken(MJAsmToken.LOAD_2, "load_2");
  }
  "load_3" {
    return createToken(MJAsmToken.LOAD_3, "load_3");
  }
  "store" {
    return createToken(MJAsmToken.STORE, "store");
  }
  "store_0" {
    return createToken(MJAsmToken.STORE_0, "store_0");
  }
  "store_1" {
    return createToken(MJAsmToken.STORE_1, "store_1");
  }
  "store_2" {
    return createToken(MJAsmToken.STORE_2, "store_2");
  }
  "store_3" {
    return createToken(MJAsmToken.STORE_3, "store_3");
  }
  "getstatic" {
    return createToken(MJAsmToken.GETSTATIC, "getstatic");
  }
  "putstatic" {
    return createToken(MJAsmToken.PUTSTATIC, "putstatic");
  }
  "getfield" {
    return createToken(MJAsmToken.GETFIELD, "getfield");
  }
  "putfield" {
    return createToken(MJAsmToken.PUTFIELD, "putfield");
  }
  "const_0" {
    return createToken(MJAsmToken.CONST_0, "const_0");
  }
  "const_1" {
    return createToken(MJAsmToken.CONST_1, "const_1");
  }
  "const_2" {
    return createToken(MJAsmToken.CONST_2, "const_2");
  }
  "const_3" {
    return createToken(MJAsmToken.CONST_3, "const_3");
  }
  "const_4" {
    return createToken(MJAsmToken.CONST_4, "const_4");
  }
  "const_5" {
    return createToken(MJAsmToken.CONST_5, "const_5");
  }
  "const_m1" {
    return createToken(MJAsmToken.CONST_M1, "const_m1");
  }
  "const" {
    return createToken(MJAsmToken.CONST, "const");
  }
  "add" {
    return createToken(MJAsmToken.ADD, "add");
  }
  "sub" {
    return createToken(MJAsmToken.SUB, "sub");
  }
  "mul" {
    return createToken(MJAsmToken.MUL, "mul");
  }
  "div" {
    return createToken(MJAsmToken.DIV, "div");
  }
  "rem" {
    return createToken(MJAsmToken.REM, "rem");
  }
  "neg" {
    return createToken(MJAsmToken.NEG, "neg");
  }
  "shl" {
    return createToken(MJAsmToken.SHL, "shl");
  }
  "shr" {
    return createToken(MJAsmToken.SHR, "shr");
  }
  "inc" {
    return createToken(MJAsmToken.INC, "inc");
  }
  "new" {
    return createToken(MJAsmToken.NEW, "new");
  }
  "newarray" {
    return createToken(MJAsmToken.NEWARRAY, "newarray");
  }
  "aload" {
    return createToken(MJAsmToken.ALOAD, "aload");
  }
  "astore" {
    return createToken(MJAsmToken.ASTORE, "astore");
  }
  "baload" {
    return createToken(MJAsmToken.BALOAD, "baload");
  }
  "bastore" {
    return createToken(MJAsmToken.BASTORE, "bastore");
  }
  "arraylength" {
    return createToken(MJAsmToken.ARRAYLENGTH, "arraylength");
  }
  "pop" {
    return createToken(MJAsmToken.POP, "pop");
  }
  "dup" {
    return createToken(MJAsmToken.DUP, "dup");
  }
  "dup2" {
    return createToken(MJAsmToken.DUP2, "dup2");
  }
  "jmp" {
    return createToken(MJAsmToken.JMP, "jmp");
  }
  "jeq" {
    return createToken(MJAsmToken.JEQ, "jeq");
  }
  "jne" {
    return createToken(MJAsmToken.JNE, "jne");
  }
  "jlt" {
    return createToken(MJAsmToken.JLT, "jlt");
  }
  "jle" {
    return createToken(MJAsmToken.JLE, "jle");
  }
  "jgt" {
    return createToken(MJAsmToken.JGT, "jgt");
  }
  "jge" {
    return createToken(MJAsmToken.JGE, "jge");
  }
  "call" {
    return createToken(MJAsmToken.CALL, "call");
  }
  "return" {
    return createToken(MJAsmToken.RETURN, "return");
  }
  "enter" {
    return createToken(MJAsmToken.ENTER, "enter");
  }
  "exit" {
    return createToken(MJAsmToken.EXIT, "exit");
  }
  "read" {
    return createToken(MJAsmToken.READ, "read");
  }
  "print" {
    return createToken(MJAsmToken.PRINT, "print");
  }
  "bread" {
    return createToken(MJAsmToken.BREAD, "bread");
  }
  "bprint" {
    return createToken(MJAsmToken.BPRINT, "bprint");
  }
  "trap" {
    return createToken(MJAsmToken.TRAP, "trap");
  }

  // Character
  {CharConst} {
    String cconst = yytext();
    int cl = cconst.length();
    if (cl == 3) {
      return createToken(MJAsmToken.CHAR, new Character(cconst.charAt(1)));
    } else if (cl == 4) {
      if (cconst.charAt(1) == '\\') {
        char c = cconst.charAt(2);
        if ((c == 'n') || (c == 'r') || (c == 't')) {
          return createToken(MJAsmToken.CHAR, new Character(c));
        }
      }
    }
    return createInvalidToken(ERR_INVALID_CCONST);
  }
  {UnclosedCharConst} {
    return createInvalidToken(ERR_UNCLOSED_CCONST);
  }

  // Number
  {Number} {
    try {
      return createToken(MJAsmToken.NUMBER, Integer.valueOf(yytext()));
    } catch (NumberFormatException e) {
      return createInvalidToken(ERR_INVALID_NUMBER);
    }
  }

  // Label
  {Label} {
    return createToken(MJAsmToken.LABEL, yytext());
  }
  {InvalidLabel} {
    return createInvalidToken(ERR_INVALID_LABEL);
  }

  // Colon
  ":" {
    return createToken(MJAsmToken.COLON);
  }

  // Comma
  "," {
    return createToken(MJAsmToken.COMMA);
  }

  // White space
  {WhiteSpace} {
    // no action
  }

  // End of line
  {EOL} {
    lineBreaks.breakLineAt(yychar + yytext().length());
    return createToken(MJAsmToken.EOL);
  }

  // End of file
  //<<EOF>> {
  //  lineBreaks.breakLineAt(yychar + yytext().length());
  //  return createToken(MJAsmToken.EOF);
  //}

  // error
  . {
    return createToken(MJAsmToken.error);
  }

}

