/*
 * @(#)MJAsmToken.java  1.0.0 / 2008-05-28
 *
 * Copyright (c) 2008, ETF and contributors. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package yu.ac.bg.etf.ri4pp.mjtools.asm;

import java_cup.runtime.Symbol;

/**
 * MJ Assembler Token.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public final class MJAsmToken extends Symbol implements MJAsmSym {

/*^**************************************************************************^*/
/*- Public Static Fields.                                                    -*/
/*^**************************************************************************^*/

  /**
   * Type <i>COMMENT</i>.
   */
  public static final int COMMENT = INVALID + 1;

  /**
   * Type <i>DIRECTIVE</i>.
   */
  public static final int DIRECTIVE = COMMENT + 2;

  /**
   * Type <i>INSTRUCTION</i>.
   */
  public static final int INSTRUCTION = COMMENT + 3;

  /**
   * Type <i>ERROR</i>.
   */
  public static final int ERROR = COMMENT + 4;

/*^**************************************************************************^*/
/*- Private Fields.                                                          -*/
/*^**************************************************************************^*/

  private final int type;
  private final int line;
  private final int column;
  private final int len;

/*^**************************************************************************^*/
/*- Public Constructor.                                                      -*/
/*^**************************************************************************^*/

  /**
   * Constructs new MJAsmToken object.
   *
   * @param type the token type (class)
   * @param left the left position
   * @param right the right position
   * @param value the lexical value
   * @param line the line number
   * @param column the column number
   */
  public MJAsmToken(int type, int left, int right, Object value, int line,
                    int column) {
    super(type, left, right, value);
    this.line = line;
    this.column = column;
    len = right - left + 1;
    if ((type >= LOAD) && (type <= TRAP)) {
      this.type = INSTRUCTION;
    } else if ((type == DS) || (type == ORG)) {
      this.type = DIRECTIVE;
    } else if ((type == INVALID) || (type == error)) {
      this.type = ERROR;
    } else {
      this.type = type;
    }
  }

/*^**************************************************************************^*/
/*- Public Methods.                                                          -*/
/*^**************************************************************************^*/

  /**
   * Returns token type.
   *
   * @return token type
   */
  public int getType() {
    return type;
  }

  /**
   * Returns line number.
   *
   * @return line number
   */
  public int getLine() {
    return line;
  }

  /**
   * Returns column number.
   *
   * @return column number
   */
  public int getColumn() {
    return column;
  }

  /**
   * Returns token length.
   *
   * @return token length
   */
  public int getLength() {
    return len;
  }

  /**
   * Returns <tt>true</tt> if token denotes an error.
   *
   * @return <tt>true</tt> if token denotes an error
   */
  public boolean isError() {
    return (type == ERROR);
  }

/*^**************************************************************************^*/
/*- Overridden Methods.                                                      -*/
/*^**************************************************************************^*/

  /**
   * Returns string representation of this token.
   *
   * @return string representation of this token
   */
  public String toString() {
    StringBuffer sb = new StringBuffer(80);
    sb.append("MJAsmToken[line: "); //$NON-NLS-1$
    sb.append(line + 1); // NOPMD
    sb.append(", col: "); //$NON-NLS-1$
    sb.append(column + 1); // NOPMD
    sb.append(", len: "); //$NON-NLS-1$
    sb.append(len);
    sb.append(", type: "); //$NON-NLS-1$
    sb.append(getTypeName(type));
    if (value != null) {
      sb.append(", val: "); //$NON-NLS-1$
      sb.append(value);
    }
    sb.append(']');
    return sb.toString();
  }

/*^**************************************************************************^*/
/*- Private Method.                                                          -*/
/*^**************************************************************************^*/

  private static String getTypeName(int type) { // NOPMD
    switch (type) {
      case COMMENT:
        return "comment"; //$NON-NLS-1$
      case DIRECTIVE:
        return "directive"; //$NON-NLS-1$
      case INSTRUCTION:
        return "instruction"; //$NON-NLS-1$
      case NUMBER:
        return "number"; //$NON-NLS-1$
      case CHAR:
        return "character"; //$NON-NLS-1$
      case LABEL:
        return "label"; //$NON-NLS-1$
      case COMMA:
        return "comma"; //$NON-NLS-1$
      case COLON:
        return "colon"; //$NON-NLS-1$
      case EOL:
        return "eol"; //$NON-NLS-1$
      case EOF:
        return "eof"; //$NON-NLS-1$
      case ERROR:
        return "error"; //$NON-NLS-1$
      default:
        return "unknown"; //$NON-NLS-1$
    }
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-28 08:32:03 Slobodan
   Initial version

*****************************************************************************^*/
