/*
 * @(#)MJAssembler.java  1.0.0 / 2008-05-28
 *
 * Copyright (c) 2008, ETF and contributors. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package yu.ac.bg.etf.ri4pp.mjtools.asm;

import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.io.PrintStream;
import java.io.Reader;

/**
 * MJ Assembler.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public final class MJAssembler {

/*^**************************************************************************^*/
/*- Private Static Final Fields.                                             -*/
/*^**************************************************************************^*/

  private static final String NULL_READER = "Null reader"; //$NON-NLS-1$
  private static final String NULL_INPUT_STREAM =
                                             "Null input stream"; //$NON-NLS-1$

/*^**************************************************************************^*/
/*- Private Fields.                                                          -*/
/*^**************************************************************************^*/

  private MJAsmErrorList errors;
  private boolean verbose;

/*^**************************************************************************^*/
/*- Public Constructor.                                                      -*/
/*^**************************************************************************^*/

  /**
   * Constructs new MJAssembler object.
   */
  public MJAssembler() {
    errors = new MJAsmErrorList();
  }

/*^**************************************************************************^*/
/*- Public Methods.                                                          -*/
/*^**************************************************************************^*/

  /**
   * Returns <tt>true</tt> if there are errors.
   *
   * @return <tt>true</tt> if there are errors
   */
  public boolean hasErrors() {
    return (!errors.isEmpty());
  }

  /**
   * Returns error list.
   *
   * @return error list
   */
  public MJAsmErrorList getErrors() {
    return errors;
  }

  /**
   * Sets verbose.
   *
   * @param verbose verbose
   */
  public void setVerbose(boolean verbose) {
    this.verbose = verbose;
  }

  /**
   * Returns verbose.
   *
   * @return verbose
   */
  public boolean isVerbose() {
    return verbose;
  }

  /**
   * Scans specified reader and prints results to the specified output.
   * (Warning: The method does not close the streams)
   *
   * @param r the Reader
   * @param ps the print stream
   *
   * @throws IOException on any I/O error.
   */
  public void scan(Reader r, PrintStream ps) throws IOException {
    if (r == null) {
      throw new IllegalArgumentException(NULL_READER);
    }
    if (ps == null) {
      throw new IllegalArgumentException("Null print stream"); //$NON-NLS-1$
    }
    errors = new MJAsmErrorList();
    MJAsmScanner mjAsmScanner = new MJAsmScanner(r, true);
    MJAsmToken mjasmt;
    do {
      mjasmt = mjAsmScanner.nextToken();
      if (mjasmt == null) {
        break;
      }
      if (mjasmt.isError()) {
        logSyntaxError((String)mjasmt.value, mjasmt.getLine(),
                       mjasmt.getColumn());
      } else {
        ps.println(mjasmt);
      }
    } while (mjasmt.getType() != MJAsmToken.EOF);
  }

  /**
   * Scans specified reader and prints results to the specified output.
   * (Warning: The method does not close the streams)
   *
   * @param r the Reader
   * @param os the OutputStream
   *
   * @throws IOException on any I/O error.
   */
  public void scan(Reader r, OutputStream os) throws IOException {
    if (os == null) {
      throw new IllegalArgumentException("Null output stream"); //$NON-NLS-1$
    }
    scan(r, new PrintStream(os));
  }

  /**
   * Scans specified input stream and prints results to the specified output.
   * (Warning: The method does not close the streams)
   *
   * @param is the InputStream
   * @param os the OutputStream
   *
   * @throws IOException on any I/O error.
   */
  public void scan(InputStream is, OutputStream os) throws IOException {
    if (is == null) {
      throw new IllegalArgumentException(NULL_INPUT_STREAM);
    }
    scan(new InputStreamReader(is), os);
  }

  /**
   * Encode specified reader to the specified output.
   * (Warning: The method does not close the reader)
   *
   * @param r the Reader
   * @param os the OutputStream
   *
   * @throws IOException on any I/O error.
   */
  public void encode(Reader r, OutputStream os) throws IOException {
    if (r == null) {
      throw new IllegalArgumentException(NULL_READER);
    }
    if (os == null) {
      throw new IllegalArgumentException("Null output stream"); //$NON-NLS-1$
    }
    errors = new MJAsmErrorList();
    MJBytecodeBuffer bytecodeBuffer = new MJBytecodeBuffer();
    MJAsmParser mjAsmParser = new MJAsmParser(new MJAsmScanner(r));
    mjAsmParser.setErrorList(errors);
    mjAsmParser.setMJBytecodeBuffer(bytecodeBuffer);
    try {
      mjAsmParser.parse(verbose);
      os.write(bytecodeBuffer.getBytes(), 0, bytecodeBuffer.getSize());
    } catch (IOException ioe) { // NOPMD
      throw ioe;
    } catch (Throwable t) { // NOPMD
      t.printStackTrace(); // NOPMD
    }
  }

  /**
   * Encode specified input stream to the specified output.
   * (Warning: The method does not close the streams)
   *
   * @param is the InputStream
   * @param os the OutputStream
   *
   * @throws IOException on any I/O error.
   */
  public void encode(InputStream is, OutputStream os) throws IOException {
    if (is == null) {
      throw new IllegalArgumentException(NULL_INPUT_STREAM);
    }
    encode(new InputStreamReader(is), os);
  }

  /**
   * Encode specified reader to the byte array.
   * (Warning: The method does not close the reader)
   *
   * @param r the Reader
   *
   * @return encoded byte array
   *
   * @throws IOException on any I/O error.
   */
  public byte[] encodeToByteArray(Reader r) throws IOException {
    if (r == null) {
      throw new IllegalArgumentException(NULL_READER);
    }
    errors = new MJAsmErrorList();
    MJBytecodeBuffer bytecodeBuffer = new MJBytecodeBuffer();
    MJAsmParser mjAsmParser = new MJAsmParser(new MJAsmScanner(r));
    mjAsmParser.setErrorList(errors);
    mjAsmParser.setMJBytecodeBuffer(bytecodeBuffer);
    try {
      mjAsmParser.parse(verbose);
      int bcblen = bytecodeBuffer.getSize();
      byte[] bytecode = bytecodeBuffer.getBytes();
      if (bcblen != bytecode.length) {
        byte[] bytes = new byte[bcblen];
        System.arraycopy(bytecode, 0, bytes, 0, bcblen);
        return bytes;
      }
      return bytecode;
    } catch (IOException ioe) { // NOPMD
      throw ioe;
    } catch (Throwable t) { // NOPMD
      t.printStackTrace(); // NOPMD
      return null;
    }
  }

  /**
   * Encode specified input stream to the byte array.
   * (Warning: The method does not close the stream)
   *
   * @param is the InputStream
   *
   * @return encoded byte array
   *
   * @throws IOException on any I/O error.
   */
  public byte[] encodeToByteArray(InputStream is) throws IOException {
    if (is == null) {
      throw new IllegalArgumentException(NULL_INPUT_STREAM);
    }
    return encodeToByteArray(new InputStreamReader(is));
  }

  /**
   * Encode specified reader to the bytecode buffer.
   * (Warning: The method does not close the reader)
   *
   * @param r the Reader
   *
   * @return encoded byte array
   *
   * @throws IOException on any I/O error.
   */
  public MJBytecodeBuffer encodeBytecodeBuffer(Reader r) throws IOException {
    if (r == null) {
      throw new IllegalArgumentException(NULL_READER);
    }
    errors = new MJAsmErrorList();
    MJBytecodeBuffer bytecodeBuffer = new MJBytecodeBuffer();
    MJAsmParser mjAsmParser = new MJAsmParser(new MJAsmScanner(r));
    mjAsmParser.setErrorList(errors);
    mjAsmParser.setMJBytecodeBuffer(bytecodeBuffer);
    try {
      mjAsmParser.parse(verbose);
      return bytecodeBuffer;
    } catch (IOException ioe) { // NOPMD
      throw ioe;
    } catch (Throwable t) { // NOPMD
      t.printStackTrace(); // NOPMD
      return null;
    }
  }

  /**
   * Encode specified input stream to the bytecode buffer.
   * (Warning: The method does not close the stream)
   *
   * @param is the InputStream
   *
   * @return encoded byte array
   *
   * @throws IOException on any I/O error.
   */
  public MJBytecodeBuffer encodeBytecodeBuffer(InputStream is) throws
                                                                  IOException {
    if (is == null) {
      throw new IllegalArgumentException(NULL_INPUT_STREAM);
    }
    return encodeBytecodeBuffer(new InputStreamReader(is));
  }

/*^**************************************************************************^*/
/*- Private Methods.                                                         -*/
/*^**************************************************************************^*/

  private void logSyntaxError(String errorDecs, int line, int col) {
    errors.addError(new MJAsmSyntaxError(0, errorDecs, 0, 0, line + 1,
                                         col + 1));
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-28 08:42:09 Slobodan
   Initial version

*****************************************************************************^*/
