/*
 * @(#)MJBytecodeBuffer.java  1.0.0 / 2008-05-28
 *
 * Copyright (c) 2008, ETF and contributors. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package yu.ac.bg.etf.ri4pp.mjtools.asm;

import yu.ac.bg.etf.ri4pp.mjtools.utils.MJByteArrayByteIterator;
import yu.ac.bg.etf.ri4pp.mjtools.utils.MJByteIterator;

/**
 * MJ Bytecode Buffer.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public final class MJBytecodeBuffer {

/*^**************************************************************************^*/
/*- Private Fields.                                                          -*/
/*^**************************************************************************^*/

  private byte[] bytes;
  private int size;

/*^**************************************************************************^*/
/*- Public Constructor.                                                      -*/
/*^**************************************************************************^*/

  /**
   * Constructs new MJBytecodeBuffer object.
   */
  public MJBytecodeBuffer() {
    bytes = new byte[1024];
    size = 14;
    bytes[0] = 'M';
    bytes[1] = 'J';
  }

/*^**************************************************************************^*/
/*- Public Methods.                                                          -*/
/*^**************************************************************************^*/

  /**
   * Returns the size of this buffer.
   *
   * @return the size of this buffer
   */
  public int getSize() {
    return size;
  }

  /**
   * Returns capacity of the buffer.
   *
   * @return capacity of the buffer
   */
  public int getCapacity() {
    return bytes.length;
  }

  /**
   * Appends byte part of specified integer to the end of the buffer.
   *
   * @param i the integer to be truncated
   */
  public void appendByte(int i) {
    ensureCapacity(size + 1);
    bytes[size++] = (byte)i;
  }

  /**
   * Appends short part of specified integer to the end of the buffer.
   *
   * @param i the integer to be truncated
   */
  public void appendShort(int i) {
    ensureCapacity(size + 2);
    bytes[size++] = (byte)(i >>> 8);
    bytes[size++] = (byte)(i);
  }

  /**
   * Appends two bytes.
   *
   * @param b1 the first byte
   * @param b2 the second byte
   */
  public void append2Bytes(int b1, int b2) {
    ensureCapacity(size + 2);
    bytes[size++] = (byte)b1;
    bytes[size++] = (byte)b2;
  }

  /**
   * Appends three bytes.
   *
   * @param b1 the first byte
   * @param b2 the second byte
   * @param b3 the thrid byte
   */
  public void append3Bytes(int b1, int b2, int b3) {
    ensureCapacity(size + 3);
    bytes[size++] = (byte)b1;
    bytes[size++] = (byte)b2;
    bytes[size++] = (byte)b3;
  }

  /**
   * Appends byte and short.
   *
   * @param b the byte
   * @param s the short
   */
  public void appendByteAndShort(int b, int s) {
    ensureCapacity(size + 3);
    bytes[size++] = (byte)b;
    bytes[size++] = (byte)(s >>> 8);
    bytes[size++] = (byte)s;
  }

  /**
   * Appends byte and word.
   *
   * @param b the byte
   * @param w the word
   */
  public void appendByteAndWord(int b, int w) {
    ensureCapacity(size + 5);
    bytes[size++] = (byte)b;
    bytes[size++] = (byte)(w >>> 24);
    bytes[size++] = (byte)(w >>> 16);
    bytes[size++] = (byte)(w >>> 8);
    bytes[size++] = (byte)w;
  }

  /**
   * Sets new short value for the bytes at specified position.
   *
   * @param s the short
   * @param ind the position
   */
  public void setShort(int ind, int s) {
    if ((ind < 0) || (ind >= size - 3)) {
      throw new IllegalArgumentException(
                                       "Invalid bytecode index"); //$NON-NLS-1$
    }
    bytes[ind++] = (byte)(s >>> 8);
    bytes[ind] = (byte)(s);
  }

  /**
   * Sets new word value for the bytes at specified position.
   *
   * @param i the new word
   * @param ind the position
   */
  public void setWord(int ind, int i) {
    if ((ind < 0) || (ind >= size - 3)) {
      throw new IllegalArgumentException(
                                       "Invalid bytecode index"); //$NON-NLS-1$
    }
    bytes[ind++] = (byte)(i >>> 24);
    bytes[ind++] = (byte)(i >>> 16);
    bytes[ind++] = (byte)(i >>> 8);
    bytes[ind] = (byte)(i);
  }

  /**
   * Returns bytecode iterator.
   *
   * @return bytecode iterator
   */
  public MJByteIterator getByteIterator() {
    return new MJByteArrayByteIterator(bytes, size);
  }

/*^**************************************************************************^*/
/*- Package-Private Methods.                                                 -*/
/*^**************************************************************************^*/

  /**
   * Returns buffer's data.
   *
   * @return buffer's data
   */
  byte[] getBytes() {
    return bytes; // NOPMD
  }

  /**
   * Trims the capacity of the buffer.
   */
  void trimToSize() {
    int oldCapacity = bytes.length;
    if (size < oldCapacity) {
      byte[] oldBytes = bytes;
      bytes = new byte[size];
      System.arraycopy(oldBytes, 0, bytes, 0, size);
    }
  }

  /**
   * Increases the capacity of the buffer.
   *
   * @param minCapacity the minimum capacity
   */
  private void ensureCapacity(int minCapacity) {
    int oldCapacity = bytes.length;
    if (minCapacity > oldCapacity) {
      byte[] oldBytes = bytes;
      int newCapacity = ((3 * oldCapacity) >> 1) + 1;
      if (newCapacity < minCapacity) {
        newCapacity = minCapacity;
      }
      bytes = new byte[newCapacity];
      System.arraycopy(oldBytes, 0, bytes, 0, size);
    }
  }

  // NOTE: There is no need for other methods.

  /**
   * Sets new byte value at specified position
   *
   * @param ind the byte position
   * @param i the new byte value
   *
  public void setByte(int ind, int i) {
    bytes[ind] = (byte)i;
  }

  /**
   * Returns byte at specified position.
   *
   * @param ind the byte position
   *
   * @return byte at specified position
   *
  public byte getByteAt(int ind) {
    if ((ind < 0) || (ind >= size)) {
      throw new IllegalArgumentException("Invalid byte index");
    }
    return bytes[ind];
  }*/

}

/*^*****************************************************************************
  File History:

 - 2008-05-28 08:43:45 Slobodan
   Initial version

*****************************************************************************^*/
