/*
 * @(#)MJDisassembler.java  1.0.0 / 2008-05-28
 *
 * Copyright (c) 2008, ETF and contributors. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package yu.ac.bg.etf.ri4pp.mjtools.asm;

import java.io.InputStream;
import java.io.OutputStream;
import java.io.PrintStream;

import yu.ac.bg.etf.ri4pp.mjtools.utils.MJByteIterator;
import yu.ac.bg.etf.ri4pp.mjtools.utils.MJInputStreamByteIterator;

/**
 * MicroJava Bytecode Disassembler.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public class MJDisassembler {

/*^**************************************************************************^*/
/*- Public Static Fields.                                                    -*/
/*^**************************************************************************^*/

  /**
   * Type <i>ASM_DECODE</i>.
   */
  public static final int ASM_DECODE = 0;

  /**
   * Type <i>ASM_LONG_DECODE</i>.
   */
  public static final int ASM_LONG_DECODE = 1;

  /**
   * Type <i>BYTE_LONG_DECODE</i>.
   */
  public static final int BYTE_LONG_DECODE = 2;

/*^**************************************************************************^*/
/*- Private Static Final Fields.                                             -*/
/*^**************************************************************************^*/

  private static final String NULL_INPUT_STREAM =
                                             "Null input stream"; //$NON-NLS-1$

  private static final String NULL_BYTE_ITERATOR =
                                            "Null byte iterator"; //$NON-NLS-1$

/*^**************************************************************************^*/
/*- Private Fields.                                                          -*/
/*^**************************************************************************^*/

  private int type;
  private final PrintStream output;
  private boolean errorNext;

/*^**************************************************************************^*/
/*- Public Constructors.                                                     -*/
/*^**************************************************************************^*/

  /**
   * Constructs new MJBytecodeDisassembler object.
   */
  public MJDisassembler() {
    output = System.out;
  }

  /**
   * Constructs new MJBytecodeDisassembler object.
   *
   * @param type disassembler type
   */
  public MJDisassembler(int type) {
    this.type = type;
    output = System.out;
  }

  /**
   * Constructs new MJBytecodeDisassembler object.
   *
   * @param type disassembler type
   * @param os the output stream
   */
  public MJDisassembler(int type, OutputStream os) {
    if ((type < ASM_DECODE) || (type > BYTE_LONG_DECODE)) {
      throw new IllegalArgumentException("Invalid type"); //$NON-NLS-1$
    }
    if (os == null) {
      throw new IllegalArgumentException("Null output stream"); //$NON-NLS-1$
    }
    this.type = type;
    output = new PrintStream(os);
  }

  /**
   * Constructs new MJBytecodeDisassembler object.
   *
   * @param type disassembler type
   * @param ps the print stream
   */
  public MJDisassembler(int type, PrintStream ps) {
    if ((type < ASM_DECODE) || (type > BYTE_LONG_DECODE)) {
      throw new IllegalArgumentException("Invalid type"); //$NON-NLS-1$
    }
    if (ps == null) {
      throw new IllegalArgumentException("Null print stream"); //$NON-NLS-1$
    }
    this.type = type;
    output = ps;
  }

/*^**************************************************************************^*/
/*- Public Methods.                                                          -*/
/*^**************************************************************************^*/

  /**
   * Decodes specified byte iterator.
   *
   * @param mjbi the byte iterator
   *
   * @return <tt>true</tt> if there are no errors
   */
  public boolean decode(MJByteIterator mjbi) {
    if (mjbi == null) {
      throw new IllegalArgumentException(NULL_BYTE_ITERATOR);
    }
    errorNext = false;
    if (decodeHeader(mjbi)) {
      return decodeInstructions(mjbi);
    }
    return false;
  }

  /**
   * Decodes specified input stream. (Note: Method does not close the stream)
   *
   * @param is the input stream
   *
   * @return <tt>true</tt> if there are no errors
   */
  public boolean decode(InputStream is) {
    if (is == null) {
      throw new IllegalArgumentException(NULL_INPUT_STREAM);
    }
    return decode(new MJInputStreamByteIterator(is));
  }

  /**
   * Decodes header from the specified byte iterator.
   *
   * @param mjbi the byte iterator
   *
   * @return <tt>true</tt> if there are no errors
   */
  public boolean decodeHeader(MJByteIterator mjbi) { // NOPMD
    if (mjbi == null) {
      throw new IllegalArgumentException(NULL_BYTE_ITERATOR);
    }
    errorNext = false;
    int codesize;
    int datasize;
    int mainPC;
    if (mjbi.hasNext() && (mjbi.nextByte() == 'M') &&
        mjbi.hasNext() && (mjbi.nextByte() == 'J')) {
      codesize = nextWord(mjbi);
      if (errorNext) {
        return false;
      }
      datasize = nextWord(mjbi);
      if (errorNext) {
        return false;
      }
      mainPC = nextWord(mjbi);
      if (errorNext) {
        return false;
      }
      boolean lng = (type > ASM_DECODE);
      if (lng) {
        output.println("; Header: MJ"); //$NON-NLS-1$
        output.println();
        output.println();
        output.println();
        output.println();
        output.print("; Code size: "); //$NON-NLS-1$
        output.println(codesize);
      }
      output.print(".ds "); //$NON-NLS-1$
      output.println(datasize);
      if (lng) {
        output.println();
        output.println();
        output.print("; Data size: "); //$NON-NLS-1$
        output.println(datasize);
      }
      output.print(".org "); //$NON-NLS-1$
      output.println(mainPC);
      if (lng) {
        output.println();
        output.println();
        output.print("; Main PC: "); //$NON-NLS-1$
        output.println(mainPC);
      }
      return true;
    }
    return false;
  }

  /**
   * Decodes header from the specified input stream.
   * (Note: Method does not close the stream)
   *
   * @param is the input stream
   *
   * @return <tt>true</tt> if there are no errors
   */
  public boolean decodeHeader(InputStream is) {
    if (is == null) {
      throw new IllegalArgumentException(NULL_INPUT_STREAM);
    }
    return decodeHeader(new MJInputStreamByteIterator(is));
  }

  /**
   * Decodes instructions from the specified byte iterator.
   *
   * @param mjbi the byte iterator
   *
   * @return <tt>true</tt> if there are no errors
   */
  public boolean decodeInstructions(MJByteIterator mjbi) {
    if (mjbi == null) {
      throw new IllegalArgumentException(NULL_BYTE_ITERATOR);
    }
    errorNext = false;
    while (mjbi.hasNext()) {
      if (!decodeInst(mjbi)) {
        break;
      }
    }
    return !errorNext;
  }

  /**
   * Decodes instructions from the specified input stream.
   * (Note: Method does not close the stream)
   *
   * @param is the input stream
   *
   * @return <tt>true</tt> if there are no errors
   */
  public boolean decodeInstructions(InputStream is) {
    if (is == null) {
      throw new IllegalArgumentException(NULL_INPUT_STREAM);
    }
    return decodeInstructions(new MJInputStreamByteIterator(is));
  }

  /**
   * Decodes next instruction from the specified byte iterator.
   *
   * @param mjbi the byte iterator
   *
   * @return <tt>true</tt> if there are no errors
   */
  public boolean decodeInstruction(MJByteIterator mjbi) {
    if (mjbi == null) {
      throw new IllegalArgumentException(NULL_BYTE_ITERATOR);
    }
    errorNext = false;
    return decodeInst(mjbi);
  }

  /**
   * Decodes next instruction fr.om the specified input stream.
   * (Note: Method does not close the stream)
   *
   * @param is the input stream
   *
   * @return <tt>true</tt> if there are no errors
   */
  public boolean decodeInstruction(InputStream is) {
    if (is == null) {
      throw new IllegalArgumentException(NULL_INPUT_STREAM);
    }
    return decodeInstruction(new MJInputStreamByteIterator(is));
  }

/*^**************************************************************************^*/
/*- Private Methods.                                                         -*/
/*^**************************************************************************^*/

  private boolean decodeInst(MJByteIterator mjbi) { // NOPMD
    if (mjbi == null) {
      throw new IllegalArgumentException(NULL_BYTE_ITERATOR);
    }
    byte opcode = nextByte(mjbi);
    if (errorNext) {
      return false;
    }
    MJVMInstructionDescriptor mjvmid = MJVMInstructions.getInstruction(opcode);
    int opcnt = mjvmid.getOperandCount();
    int optype = mjvmid.getOperandType();
    String op1 = null;
    String op2 = null;
    if (opcnt > 0) {
      op1 = nextOperand(optype, mjbi);
      if (!errorNext && (opcnt == 2)) {
        op2 = nextOperand(optype, mjbi);
      }
    }
    if (errorNext) {
      return false;
    }
    switch (type) { // NOPMD
      case ASM_DECODE:
        output.print(mjvmid.getName());
        if (op1 != null) {
          output.print(" "); //$NON-NLS-1$
          output.print(op1);
          if (op2 != null) {
            output.print(", "); //$NON-NLS-1$
            output.print(op2);
          }
        }
        output.println();
        break;
      case ASM_LONG_DECODE:
        output.print(mjvmid.getName());
        if (op1 != null) {
          output.print(" "); //$NON-NLS-1$
          output.print(op1);
          if (op2 != null) {
            output.print(", "); //$NON-NLS-1$
            output.print(op2);
          }
        }
        output.println();
        for (int i = opcnt * optype; i > 0; i--) {
          output.println();
        }
        break;
      case BYTE_LONG_DECODE:
        output.println(mjvmid.getName());
        if (op1 != null) {
          output.println(op1);
          for (int i = 2; i >= 0; i--) {
            output.println();
          }
          if (op2 != null) {
            output.println(op2);
            for (int i = 2; i >= 0; i--) {
              output.println();
            }
          }
        }
        break;
    }
    return true;
  }

  private String nextOperand(int optype, MJByteIterator mjbi) {
    switch (optype) {
      case MJVMInstructionDescriptor.OP_TYPE_BYTE:
        byte nb = nextByte(mjbi);
        if (errorNext) {
          return null;
        }
        return String.valueOf(nb);
      case MJVMInstructionDescriptor.OP_TYPE_SHORT:
        short ns = nextShort(mjbi);
        if (errorNext) {
          return null;
        }
        return String.valueOf(ns);
      case MJVMInstructionDescriptor.OP_TYPE_WORD:
        int nw = nextWord(mjbi);
        if (errorNext) {
          return null;
        }
        return String.valueOf(nw);
      default:
        return null;
    }
  }

  private int nextWord(MJByteIterator mjbi) {
    short s1 = nextShort(mjbi);
    if (errorNext) {
      return 0;
    }
    short s2 = nextShort(mjbi);
    if (errorNext) {
      return 0;
    }
    return (s1 << 16) + (s2 & 0xffff);
  }

  private short nextShort(MJByteIterator mjbi) {
    byte b1 = nextByte(mjbi);
    if (errorNext) {
      return 0;
    }
    byte b2 = nextByte(mjbi);
    if (errorNext) {
      return 0;
    }
    return (short)(((b1 << 8) + (b2 & 0xff)) << 16 >> 16);
  }

  private byte nextByte(MJByteIterator mjbi) {
    errorNext = errorNext | !mjbi.hasNext();
    if (errorNext) {
      return 0;
    }
    return mjbi.nextByte();
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-28 08:46:26 Slobodan
   Initial version

*****************************************************************************^*/
