/*
 * @(#)MJBytecodePanel.java  1.0.0 / 2008-05-27
 *
 * Copyright (c) 2008, ETF and contributors. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package yu.ac.bg.etf.ri4pp.mjtools.bce;

import java.awt.BorderLayout;
import java.awt.GridLayout;
import java.io.DataInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;

import javax.swing.BorderFactory;
import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.SwingUtilities;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import javax.swing.text.BadLocationException;
import javax.swing.text.Document;

import net.wrka.LibraryInfo;
import net.wrka.swing.components.JWxFlatSplitPane;
import net.wrka.swing.components.text.LineCounterPanel;
import net.wrka.swing.io.TextComponentOutputStream;
import net.wrka.util.CommonConstants;

import yu.ac.bg.etf.ri4pp.mjtools.asm.MJDisassembler;
import yu.ac.bg.etf.ri4pp.mjtools.utils.MJByteIterator;

/**
 * MicroJava Bytecode Panel.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public final class MJBytecodePanel extends JPanel {

/*^**************************************************************************^*/
/*- Private Fields.                                                          -*/
/*^**************************************************************************^*/

  private transient MJDisassembler mjDisassembler;
  private transient MJBytecodeScrollListener mjbcsl;
  private MJBytecodeTextArea jtaMJBytecode;
  private MJBytecodeDocument mjBytecodeDocument;
  private MJBytecodeTextArea jtaMJDisassembled;
  private JLabel jlCodeSize;
  private JLabel jlTotalSize;
  private File bytecodeFile;

/*^**************************************************************************^*/
/*- Public Constructor.                                                      -*/
/*^**************************************************************************^*/

  /**
   * Construcs new MJBytecodePanel object.
   */
  public MJBytecodePanel() {
    super(new BorderLayout());
    JPanel cp = new JPanel(new BorderLayout());
    cp.setBorder(BorderFactory.createEmptyBorder(4, 4, 0, 4));
    JPanel leftPanel = new JPanel(new BorderLayout());
    JPanel rightPanel = new JPanel(new BorderLayout());
    mjBytecodeDocument = new MJBytecodeDocument();
    mjBytecodeDocument.addDocumentListener(new MJByteDocumentListener());
    jtaMJBytecode = new MJBytecodeTextArea(mjBytecodeDocument);
    jtaMJDisassembled = new MJBytecodeTextArea(false);
    LineCounterPanel lcp = new LineCounterPanel(jtaMJBytecode);
    JScrollPane leftScrollPane = new JScrollPane(lcp);
    lcp = new LineCounterPanel(jtaMJDisassembled);
    JScrollPane rightScrollPane = new JScrollPane(lcp);
    mjbcsl = new MJBytecodeScrollListener(leftScrollPane.getViewport(),
                                          rightScrollPane.getViewport());
    mjDisassembler = new MJDisassembler(MJDisassembler.ASM_LONG_DECODE,
                           new TextComponentOutputStream(jtaMJDisassembled));
    JCheckBox jcbSyncScrolling = new JCheckBox(
                                       "Synchronize Scrolling"); //$NON-NLS-1$
    jcbSyncScrolling.setHorizontalAlignment(JCheckBox.RIGHT);
    jcbSyncScrolling.setHorizontalTextPosition(JCheckBox.LEFT);
    jcbSyncScrolling.addActionListener(mjbcsl);
    JPanel leftStatus = new JPanel(new GridLayout(1, 2));
    jlCodeSize = new JLabel("Code Size: 0"); //$NON-NLS-1$
    leftStatus.add(jlCodeSize);
    jlTotalSize = new JLabel("Total Size: 0"); //$NON-NLS-1$
    leftStatus.add(jlTotalSize);
    leftStatus.setBorder(BorderFactory.createEmptyBorder(4, 0, 4, 0));
    leftPanel.add(new JLabel("MicroJava Bytecode"), //$NON-NLS-1$
                  BorderLayout.NORTH);
    leftPanel.add(leftScrollPane, BorderLayout.CENTER);
    leftPanel.add(leftStatus, BorderLayout.SOUTH);
    rightPanel.add(new JLabel("Disassembled Bytecode"), //$NON-NLS-1$
                   BorderLayout.NORTH);
    rightPanel.add(rightScrollPane, BorderLayout.CENTER);
    rightPanel.add(jcbSyncScrolling, BorderLayout.SOUTH);
    JWxFlatSplitPane splitPane = new JWxFlatSplitPane(
        JWxFlatSplitPane.HORIZONTAL_SPLIT, true, leftPanel, rightPanel);
    splitPane.setResizeWeight(0.5);
    cp.add(splitPane);
    add(cp);
  }

  /**
   * Construcs new MJBytecodePanel object.
   *
   * @param bytecodeFile the bytecode file
   *
   * @throws IOException on any I/O error
   */
  public MJBytecodePanel(File bytecodeFile) throws IOException {
    this();
    this.bytecodeFile = bytecodeFile;
    loadBytecode(bytecodeFile);
  }

  /**
   * Construcs new MJBytecodePanel object.
   *
   * @param is the bytecode input stream
   *
   * @throws IOException on any I/O error
   */
  public MJBytecodePanel(InputStream is) throws IOException {
    this();
    loadBytecode(is);
  }

/*^**************************************************************************^*/
/*- Public Methods.                                                          -*/
/*^**************************************************************************^*/

  /**
   * Returns <tt>true</tt> if bytecode is modified.
   *
   * @return <tt>true</tt> if bytecode is modified
   */
  public boolean isModified() {
    return jtaMJBytecode.isDocumentModified();
  }

  /**
   * Returns editable status.
   *
   * @return editable status
   */
  public boolean isEditable() {
    return jtaMJBytecode.isEditable();
  }

  /**
   * Sets editable status.
   *
   * @param b editable status
   */
  public void setEditable(boolean b) {
    jtaMJBytecode.setEditable(b);
  }

  /**
   * Returns disassembled code text component.
   *
   * @return disassembled code text component
   */
  public MJBytecodeTextArea getBytecodeTextComponent() {
    return jtaMJBytecode;
  }

  /**
   * Returns bytecode text component.
   *
   * @return bytecode text component
   */
  public MJBytecodeTextArea getDisassembledTextComponent() {
    return jtaMJBytecode;
  }

  /**
   * Returns bytecode file.
   *
   * @return bytecode file
   */
  public File getBytecodeFile() {
    return bytecodeFile;
  }

  /**
   * Clears text components.
   */
  public void clear() {
    jtaMJBytecode.clearDocument();
    jtaMJDisassembled.clearDocument();
  }

  /**
   * Loads bytecode from the specified file.
   *
   * @param file the bytecode file
   *
   * @throws IOException on any I/O error
   */
  public void loadBytecode(File file) throws IOException {
    loadBytecode(new FileInputStream(file));
    bytecodeFile = file;
  }

  /**
   * Loads bytecode from the file specified by the file.
   *
   * @param fileName the bytecode file name
   *
   * @throws IOException on any I/O error
   */
  public void loadBytecode(String fileName) throws IOException {
    File file = new File(fileName);
    loadBytecode(new FileInputStream(file));
    bytecodeFile = file;
  }

  /**
   * Loads bytecode from the file specified input stream.
   *
   * @param is the input stream
   *
   * @throws IOException on any I/O error
   */
  public void loadBytecode(InputStream is) throws IOException {
    DataInputStream dis = null;
    try {
      mjBytecodeDocument.setFilterBytes(false);
      dis = new DataInputStream(is);
      StringBuffer sb = new StringBuffer();
      byte b = dis.readByte();
      if (b != 'M') {
        throw new IOException("Invalid MicroJava Bytecode"); //$NON-NLS-1$
      }
      sb.append(b);
      sb.append(CommonConstants.STRING_DEFAULT_LINE_SEPARATOR);
      b = dis.readByte();
      if (b != 'J') {
        throw new IOException("Invalid MicroJava Bytecode"); //$NON-NLS-1$
      }
      sb.append(b);
      int i;
      while ((i = dis.read()) >= 0) {
        sb.append(CommonConstants.STRING_DEFAULT_LINE_SEPARATOR);
        sb.append(i); // implicit int -> byte conv.
      }
      jtaMJBytecode.setText(sb.toString());
      jtaMJBytecode.setCaretPosition(0);
      jtaMJBytecode.setDocumentModified(false);
    } catch (IOException ioe) {
      clear();
      throw ioe;
    } finally {
      if (dis != null) {
        dis.close();
      }
      mjBytecodeDocument.setFilterBytes(true);
    }
  }

  /**
   * Loads bytecode from the specified bytecode iterator.
   *
   * @param mjbi the bytecode iterator
   */
  public void loadBytecode(MJByteIterator mjbi) {
    if (mjbi == null) {
      throw new IllegalArgumentException("Null byte iterator"); //$NON-NLS-1$
    }
    mjBytecodeDocument.setFilterBytes(false);
    StringBuffer sb = new StringBuffer();
    if (mjbi.hasNext()) {
      int c = 0x00FF & ((char)mjbi.nextByte());
      sb.append(c);
      while (mjbi.hasNext()) {
        c = 0x00FF & ((char)mjbi.nextByte());
        sb.append(CommonConstants.STRING_DEFAULT_LINE_SEPARATOR);
        sb.append(c);
      }
    }
    jtaMJBytecode.setText(sb.toString());
    jtaMJBytecode.setCaretPosition(0);
    jtaMJBytecode.setDocumentModified(false);
    mjBytecodeDocument.setFilterBytes(true);
  }

  /**
   * Stores bytecode to the specified output stream.
   *
   * @param os the output stream
   *
   * @throws IOException on any I/O error
   */
  public void storeBytecode(OutputStream os) throws IOException {
    if (os == null) {
      throw new IllegalArgumentException("Null output stream"); //$NON-NLS-1$
    }
    MJBytecodeSegment seg = mjBytecodeDocument.getBytecodeSegment();
    while (seg.hasNext()) {
      byte b = seg.nextByte();
      os.write(b);
    }
    os.close();
  }

  /**
   * Stores bytecode to the specified file.
   *
   * @param file the file
   *
   * @throws IOException on any I/O error
   */
  public void storeBytecode(File file) throws IOException {
    storeBytecode(new FileOutputStream(file));
    bytecodeFile = file;
  }

  /**
   * Stores bytecode to the specified file.
   *
   * @param fileName the file name
   *
   * @throws IOException on any I/O error
   */
  public void storeBytecode(String fileName) throws IOException {
    File file = new File(fileName);
    storeBytecode(new FileOutputStream(file));
    bytecodeFile = file;
  }

  /**
   * Stores disassembled code to the specified output stream.
   *
   * @param os the output stream
   *
   * @throws IOException on any I/O error
   */
  public void storeDisassembled(OutputStream os) throws IOException {
    MJBytecodeSegment seg = mjBytecodeDocument.getBytecodeSegment();
    MJDisassembler mjdisa = new MJDisassembler(MJDisassembler.ASM_DECODE, os);
    mjdisa.decode(seg);
    os.close();
  }

  /**
   * Stores disassembled code to the specified file.
   *
   * @param file the file
   *
   * @throws IOException on any I/O error
   */
  public void storeDisassembled(File file) throws IOException {
    storeDisassembled(new FileOutputStream(file));
  }

  /**
   * Stores disassembled code to the specified file.
   *
   * @param fileName the file name
   *
   * @throws IOException on any I/O error
   */
  public void storeDisassembled(String fileName) throws IOException {
    storeDisassembled(new FileOutputStream(fileName));
  }

  /**
   * Returns MJByteIterator.
   *
   * @return MJByteIterator
   */
  public MJByteIterator getByteIterator() {
    return mjBytecodeDocument.getBytecodeSegment();
  }

/*^**************************************************************************^*/
/*- Pacakge-Private Methods.                                                 -*/
/*^**************************************************************************^*/

  void setStatusLabels() {
    int lnc = jtaMJBytecode.getLineCount();
    StringBuffer sb = new StringBuffer("Total Size: "); //$NON-NLS-1$
    sb.append(lnc);
    jlTotalSize.setText(sb.toString());
    if (lnc > 14) {
      sb = new StringBuffer("Code Size: "); //$NON-NLS-1$
      sb.append(lnc - 14);
      jlCodeSize.setText(sb.toString());
    } else {
      jlCodeSize.setText("Code Size: 0"); //$NON-NLS-1$
    }
  }

  /**
   * Disassebles bytecode document.
   */
  void disassemble() {
    if (mjbcsl.isEnabled()) {
      mjbcsl.setBlocked(true);
    }
    MJBytecodeSegment seg = mjBytecodeDocument.getBytecodeSegment();
    jtaMJDisassembled.clearDocument();
    mjDisassembler.decode(seg);
    Document d = jtaMJDisassembled.getDocument();
    try {
      int nll = LibraryInfo.LINE_SEPARATOR.length();
      d.remove(d.getLength() - nll, nll);
    } catch (BadLocationException ble) { // NOPMD
      // empty catch block
    }
    SwingUtilities.invokeLater(mjbcsl);
  }

/*^**************************************************************************^*/
/*- Inner Classes.                                                           -*/
/*^**************************************************************************^*/

  /**
   * Bytecode document listener.
   */
  final class MJByteDocumentListener implements DocumentListener {
    /**
     * {@inheritDoc}
     */
    public void changedUpdate(DocumentEvent e) {
      setStatusLabels();
      disassemble();
    }
    /**
     * {@inheritDoc}
     */
    public void insertUpdate(DocumentEvent e) {
      changedUpdate(e);
    }
    /**
     * {@inheritDoc}
     */
    public void removeUpdate(DocumentEvent e) {
      changedUpdate(e);
    }
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-27 14:07:35 Slobodan
   Initial version

*****************************************************************************^*/
