/*
 * @(#)MJIOManager.java  1.0.0 / 2008-05-26
 *
 * Copyright (c) 2008, ETF and contributors. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package yu.ac.bg.etf.ri4pp.mjtools.utils;

import java.io.File;

import javax.swing.JFileChooser;
import javax.swing.LookAndFeel;
import javax.swing.SwingUtilities;

import net.wrka.swing.io.FileChooserFilter;
import net.wrka.swing.plaf.LookAndFeelChangeListener;
import net.wrka.swing.plaf.LookAndFeelManager;
import net.wrka.swing.tooltips.ToolTipManager;

/**
 * IO Manager.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public final class MJIOManager implements LookAndFeelChangeListener {

/*^**************************************************************************^*/
/*- Private static fields.                                                   -*/
/*^**************************************************************************^*/

  private static final File USER_FOLDER;
  private static final String DEF_CON_OUT_FILE_NAME;
  private static final FileChooserFilter MJB_FILTER;
  private static final FileChooserFilter MJA_FILTER;
  private static final FileChooserFilter TEXT_FILTER;
  private static final JFileChooser FILE_CHOOSER;
  private static final MJIOManager INSTANCE;
  private static int defOutFileCount;
  private static boolean warmed;

/*^**************************************************************************^*/
/*- Static Initialization.                                                   -*/
/*^**************************************************************************^*/

  static {
    String userDir = System.getProperty("user.dir"); //$NON-NLS-1$
    File uf = null;
    if (userDir != null) {
      uf = new File(userDir);
      uf = uf.getAbsoluteFile();
    }
    USER_FOLDER = uf;
    DEF_CON_OUT_FILE_NAME = "MJVMOutput"; //$NON-NLS-1$
    MJB_FILTER = new FileChooserFilter("mjb", //$NON-NLS-1$
                                       "MicroJava Bytecode"); //$NON-NLS-1$
    MJA_FILTER = new FileChooserFilter(new String[] {"mja", //$NON-NLS-1$
                                                     "txt"}, //$NON-NLS-1$
                                       "MicroJava Assembler"); //$NON-NLS-1$
    TEXT_FILTER = new FileChooserFilter("txt", //$NON-NLS-1$
                                        "Text File"); //$NON-NLS-1$
    FILE_CHOOSER = new JFileChooser();
    FILE_CHOOSER.setCurrentDirectory(USER_FOLDER);
    INSTANCE = new MJIOManager();
    LookAndFeelManager.addLnFListener(INSTANCE);
    FILE_CHOOSER.setMultiSelectionEnabled(false);
    ToolTipManager.registerContainer(FILE_CHOOSER);
  }

/*^**************************************************************************^*/
/*- Private Constructor.                                                     -*/
/*^**************************************************************************^*/

 /**
  * Class has no instances.
  */
  private MJIOManager() {
    // empty constructor body
  }

/*^**************************************************************************^*/
/*- LookAndFeelChangeListener Implementation.                                -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  public void lookAndFeelChanged(LookAndFeel oldLnF, LookAndFeel newLnF) {
    SwingUtilities.updateComponentTreeUI(FILE_CHOOSER);
  }

/*^**************************************************************************^*/
/*- Public Static Methods.                                                   -*/
/*^**************************************************************************^*/

  /**
   * Returns user folder.
   *
   * @return user folder
   */
  public static File getUserFolder() {
    return USER_FOLDER;
  }

  /**
   * Returns MJB FileChooser.
   *
   * @return MJB FileChooser
   */
  public static JFileChooser getMJBytecodeFileChooser() {
    if (FILE_CHOOSER.getFileFilter() != MJB_FILTER) {
      FILE_CHOOSER.resetChoosableFileFilters();
      FILE_CHOOSER.setFileFilter(MJB_FILTER);
    }
    return FILE_CHOOSER;
  }

  /**
   * Returns Open MJ Bytecode FileChooser.
   *
   * @return Open MJ Bytecode FileChooser
   */
  public static JFileChooser getOpenMJBytecodeFileChooser() {
    JFileChooser jfc = getMJBytecodeFileChooser();
    jfc.setDialogType(JFileChooser.OPEN_DIALOG);
    jfc.setDialogTitle("Open MicroJava Bytecode File"); //$NON-NLS-1$
    jfc.setApproveButtonText("Open"); //$NON-NLS-1$
    jfc.setApproveButtonMnemonic('O');
    jfc.setApproveButtonToolTipText("Open selected file"); //$NON-NLS-1$
    return jfc;
  }

  /**
   * Returns Load MJ Bytecode FileChooser.
   *
   * @return Load MJ Bytecode FileChooser
   */
  public static JFileChooser getLoadMJBytecodeFileChooser() {
    JFileChooser jfc = getMJBytecodeFileChooser();
    jfc.setDialogType(JFileChooser.OPEN_DIALOG);
    jfc.setDialogTitle("Select MicroJava Bytecode File"); //$NON-NLS-1$
    jfc.setApproveButtonText("Load"); //$NON-NLS-1$
    jfc.setApproveButtonMnemonic('L');
    jfc.setApproveButtonToolTipText("Load selected file"); //$NON-NLS-1$
    return jfc;
  }

  /**
   * Returns Save MJ Bytecode FileChooser.
   *
   * @return Save MJ Bytecode FileChooser
   */
  public static JFileChooser getSaveMJBytecodeFileChooser() {
    JFileChooser jfc = getMJBytecodeFileChooser();
    jfc.setDialogType(JFileChooser.SAVE_DIALOG);
    jfc.setDialogTitle("Save MicroJava Bytecode"); //$NON-NLS-1$
    jfc.setApproveButtonText("Save"); //$NON-NLS-1$
    jfc.setApproveButtonMnemonic('S');
    jfc.setApproveButtonToolTipText("Save bytecode"); //$NON-NLS-1$
    return jfc;
  }

  /**
   * Returns MJ Assembler FileChooser.
   *
   * @return MJ Assembler FileChooser
   */
  public static JFileChooser getMJAssemblerFileChooser() {
    if (FILE_CHOOSER.getFileFilter() != MJA_FILTER) {
      FILE_CHOOSER.resetChoosableFileFilters();
      FILE_CHOOSER.setFileFilter(MJA_FILTER);
    }
    return FILE_CHOOSER;
  }

  /**
   * Returns Open MJ Bytecode FileChooser.
   *
   * @return Open MJ Bytecode FileChooser
   */
  public static JFileChooser getOpenMJAssemblerFileChooser() {
    JFileChooser jfc = getMJAssemblerFileChooser();
    jfc.setDialogType(JFileChooser.OPEN_DIALOG);
    jfc.setDialogTitle("Open MicroJava Assembler File"); //$NON-NLS-1$
    jfc.setApproveButtonText("Open"); //$NON-NLS-1$
    jfc.setApproveButtonMnemonic('O');
    jfc.setApproveButtonToolTipText("Open selected file"); //$NON-NLS-1$
    return jfc;
  }

  /**
   * Returns Save MJ Assembler FileChooser.
   *
   * @return Save MJ Assembler FileChooser
   */
  public static JFileChooser getSaveMJAssemblerFileChooser() {
    JFileChooser jfc = getMJAssemblerFileChooser();
    jfc.setDialogType(JFileChooser.SAVE_DIALOG);
    jfc.setDialogTitle("Save MicroJava Assembler"); //$NON-NLS-1$
    jfc.setApproveButtonText("Save"); //$NON-NLS-1$
    jfc.setApproveButtonMnemonic('S');
    jfc.setApproveButtonToolTipText("Save bytecode"); //$NON-NLS-1$
    return jfc;
  }

  /**
   * Returns Text FileChooser.
   *
   * @return Text FileChooser
   */
  public static JFileChooser getTextFileChooser() {
    if (FILE_CHOOSER.getFileFilter() != TEXT_FILTER) {
      FILE_CHOOSER.resetChoosableFileFilters();
      FILE_CHOOSER.setFileFilter(TEXT_FILTER);
    }
    return FILE_CHOOSER;
  }

  /**
   * Returns Save MJVM Console Output FileChooser.
   *
   * @return Save MJVM Console Output FileChooser
   */
  public static JFileChooser getSaveMJVMConsoleOutputFileChooser() {
    JFileChooser jfc = getTextFileChooser();
    jfc.setDialogType(JFileChooser.SAVE_DIALOG);
    jfc.setDialogTitle("Save Console Output"); //$NON-NLS-1$
    jfc.setApproveButtonText("Save"); //$NON-NLS-1$
    jfc.setApproveButtonMnemonic('S');
    jfc.setApproveButtonToolTipText("Save output"); //$NON-NLS-1$
    ++defOutFileCount;
    File sf = new File(jfc.getCurrentDirectory(), DEF_CON_OUT_FILE_NAME +
                                                  defOutFileCount +
                                                  ".txt"); //$NON-NLS-1$
    jfc.setSelectedFile(sf);
    return jfc;
  }

  /**
   * Just loads the class.
   *
   * @return <tt>true</tt>
   */
  public static boolean warmUp() {
    if (!warmed) {
      warmed = true;
    }
    return true;
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-26 18:49:15 Slobodan
   Initial version

*****************************************************************************^*/
