/*
 * @(#)MJVM.java  1.0.0 / 2008-05-26
 *
 * Copyright (c) 2008, ETF and contributors. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package yu.ac.bg.etf.ri4pp.mjtools.vm;

import java.io.DataInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;

import yu.ac.bg.etf.ri4pp.mjtools.utils.MJByteIterator;
import yu.ac.bg.etf.ri4pp.mjtools.vm.gui.MJVMConsoleFrame;

/**
 * MicroJava Virtual Machine.
 * (The code is based on http://www.ssw.uni-linz.ac.at/Misc/CC/Run.java)
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public final class MJVM {

/*^**************************************************************************^*/
/*- Private Static Final Field.                                              -*/
/*^**************************************************************************^*/

  private static final String INVLID_CODE_SIZE =
                                              "Invlid code size"; //$NON-NLS-1$

/*^**************************************************************************^*/
/*- Package-Private Fields.                                                  -*/
/*^**************************************************************************^*/

  MJVMCodeMemory code;  // Code memory
  MJVMDataMemory data;  // Data memory
  MJVMHeapMemory heap;  // Heap memory
  MJVMPStack pstack;    // Procedure stack memory
  MJVMEStack estack;    // Expression stack memory

  MJVMInputOutput inout;        // Input/Output
  MJVMBreakPoints breakPoints;  // Break points
  MJVMListenerList listeners;   // Listeners
  MJVMInterpreter interpreter;  // Interpreter
  MJVMDebuger debuger;          // Debuger
  long elapsedTime;             // Elapsed time

/*^**************************************************************************^*/
/*- Public Constructors.                                                     -*/
/*^**************************************************************************^*/

  /**
   * Constructs new MJVM object.
   */
  public MJVM() {
    this(1024, 256, 32);
  }

  /**
   * Constructs new MJVM object.
   *
   * @param file the file bytecode file.
   *
   * @throws IOException on any IO error
   */
  public MJVM(File file) throws IOException {
    this();
    load(file);
  }

  /**
   * Constructs new MJVM object.
   *
   * @param is the bytecode input stream.
   *
   * @throws IOException on any IO error
   */
  public MJVM(InputStream is) throws IOException {
    this(is, true);
  }

  /**
   * Constructs new MJVM object.
   *
   * @param is the bytecode input stream.
   * @param closeStream close stream after bytecode loading
   *
   * @throws IOException on any IO error
   */
  public MJVM(InputStream is, boolean closeStream) throws IOException {
    this();
    load(is, closeStream);
  }

  /**
   * Constructs new MJVM object.
   *
   * @param mjvmio the mjvm input/output
   *
   * @throws IllegalArgumentException if mjvmio is <tt>null</tt>
   */
  public MJVM(MJVMInputOutput mjvmio) {
    this(1024, 256, 32, mjvmio);
  }

  /**
   * Constructs new MJVM object.
   *
   * @param heapSize the heap size
   * @param pStackSize the procedure stack size
   * @param eStackSize the expression stack size
   */
  public MJVM(int heapSize, int pStackSize, int eStackSize) {
    this(heapSize, pStackSize, eStackSize, new MJVMBreakPoints());
    MJVMConsoleFrame mjvmcf = new MJVMConsoleFrame(this);
    inout = mjvmcf;
    addMJVMListener(mjvmcf);
  }

  /**
   * Constructs new MJVM object.
   *
   * @param heapSize the heap size
   * @param pStackSize the procedure stack size
   * @param eStackSize the expression stack size
   * @param mjvmio the mjvm input/output
   *
   * @throws IllegalArgumentException if mjvmio is <tt>null</tt>
   */
  public MJVM(int heapSize, int pStackSize, int eStackSize,
              MJVMInputOutput mjvmio) {
    this(heapSize, pStackSize, eStackSize, new MJVMBreakPoints());
    if (mjvmio == null) {
      throw new IllegalArgumentException("Null MJVMInputOutput"); //$NON-NLS-1$
    }
    inout = mjvmio;
  }

/*^**************************************************************************^*/
/*- Private Constructor.                                                     -*/
/*^**************************************************************************^*/

  /**
   * Constructs new MJVM object.
   *
   * @param heapSize the heap size
   * @param pStackSize the procedure stack size
   * @param eStackSize the expression stack size
   * @param breakPoints the break points
   */
  private MJVM(int heapSize, int pStackSize, int eStackSize,
               MJVMBreakPoints breakPoints) {
    heap = new MJVMHeapMemory(heapSize);
    pstack = new MJVMPStack(pStackSize);
    estack = new MJVMEStack(eStackSize);
    this.breakPoints = breakPoints;
    listeners = new MJVMListenerList();
  }

/*^**************************************************************************^*/
/*- Public Methods.                                                          -*/
/*^**************************************************************************^*/

  /**
   * Return echo status.
   *
   * @return echo status
   */
  public boolean getEcho() { // NOPMD
    return inout.getEcho();
  }

  /**
   * Sets echo status.
   *
   * @param echo the echo status
   */
  public void setEcho(boolean echo) {
    inout.setEcho(echo);
  }

  /**
   * Registers specified MJVMListener.
   *
   * @param mjvmListener the MJVMListener to be added
   */
  public void addMJVMListener(MJVMListener mjvmListener) {
    listeners.addMJVMListener(mjvmListener);
  }

  /**
   * Removes specified MJVMListener.
   *
   * @param mjvmListener the MJVMListener to be removed
   */
  public void removeMJVMListener(MJVMListener mjvmListener) {
    listeners.removeMJVMListener(mjvmListener);
  }

  /**
   * Removes all MJVMListeners.
   */
  public void removeAllMJVMListeners() {
    listeners.removeAllMJVMListeners();
  }

  /**
   * Returns break points.
   *
   * @return break points
   */
  public MJVMBreakPoints getBreakPoints() {
    return breakPoints;
  }

  /**
   * Sets break points.
   * Break points are not being verified. Invalid break points are ignored.
   *
   * @param breakPoints the break points array
   */
  public void setBreakPoints(int[] breakPoints) { // NOPMD
    this.breakPoints = new MJVMBreakPoints(breakPoints);
  }

  /**
   * Sets break points.
   * Break points are not being verified. Invalid break points are ignored.
   *
   * @param breakPoints the break points
   */
  public void setBreakPoints(MJVMBreakPoints breakPoints) {
    if (breakPoints == null) {
      this.breakPoints.removeAllBreakPoints();
    } else {
      this.breakPoints = breakPoints;
    }
  }

  /**
   * Sets break point at specified address.
   * If break point is invalid it will be ignored.
   *
   * @param address the break point address
   */
  public void addBreakPoint(int address) {
    breakPoints.addBreakPoint(address);
  }

  /**
   * Adds break points.
   *
   * @param breakPoints the break points array
   */
  public void addBreakPoints(int[] breakPoints) {
    this.breakPoints.addBreakPoints(breakPoints);
  }

  /**
   * Adds break points.
   *
   * @param breakPoints the break points
   */
  public void addBreakPoints(MJVMBreakPoints breakPoints) {
    this.breakPoints.addBreakPoints(breakPoints);
  }

  /**
   * Deletes break point at specified address.
   * If break point is invalid it will be ignored.
   *
   * @param address the break point address
   */
  public void removeBreakPoint(int address) {
    breakPoints.removeBreakPoint(address);
  }

  /**
   * Removes all break points.
   */
  public void removeAllBreakPoints() {
    breakPoints.removeAllBreakPoints();
  }

  /**
   * Returns elapsed time.
   *
   * @return elapsed time
   */
  public long getElapsedTime() {
    return elapsedTime;
  }

  /**
   * Loads MicroJava bytecode from the specified input stream.
   *
   * @param is the input stream
   * @param closeStream close stream after loading
   *
   * @throws IOException on any IO error
   */
  public void load(InputStream is, // NOPMD
                   boolean closeStream) throws IOException {
    if (is == null) {
      throw new IllegalArgumentException("Null input stream"); //$NON-NLS-1$
    }
    if (interpreter != null) {
      throw new MJVMStateException("MJVM is interpreting"); //$NON-NLS-1$
    }
    if (debuger != null) {
      throw new MJVMStateException("MJVM is debugging"); //$NON-NLS-1$
    }
    DataInputStream dis = null;
    int codeSize;
    int dataSize;
    int mainPC;
    byte[] magicNumber = new byte[2];
    try {
      dis = new DataInputStream(is);
      int rs = dis.read(magicNumber, 0, 2);
      if ((rs < 0) || (magicNumber[0] != 'M') || (magicNumber[1] != 'J')) {
        throw new MJBytecodeFormatException(
                                      "Invalid bytecode stream"); //$NON-NLS-1$
      }
      codeSize = dis.readInt();
      if (codeSize <= 0) {
        throw new MJBytecodeFormatException("CodeSize <= 0");
      }
      dataSize = dis.readInt();
      if (dataSize < 0) {
        throw new MJBytecodeFormatException("DataSize < 0"); //$NON-NLS-1$
      }
      mainPC = dis.readInt();
      if ((mainPC < 0) || (mainPC >= codeSize)) {
        throw new MJBytecodeFormatException(
                                      "mainPC out of code area"); //$NON-NLS-1$
      }
      if (data == null) {
        data = new MJVMDataMemory(dataSize);
      } else {
        data.setMemory(new int[dataSize]);
      }
      if (code == null) {
        code = new MJVMCodeMemory(codeSize, mainPC);
        rs = dis.read(code.getCodeMemory(), 0, codeSize);
      } else {
        byte[] ca = new byte[codeSize];
        rs = dis.read(ca, 0, codeSize);
        code.setCodeMemory(ca);
        code.setMainPC(mainPC);
      }
      if (rs != codeSize) {
        throw new MJBytecodeFormatException(INVLID_CODE_SIZE);
      }
      heap.reset();
      pstack.reset();
      estack.reset();
      breakPoints.removeAllBreakPoints();
      listeners.fireBytecodeLoaded();
    } catch (IOException e) { // NOPMD
      throw e;
    } finally {
      if (closeStream && (dis != null)) {
        dis.close();
      }
    }
  }

  /**
   * Loads MicroJava bytecode from the specified input stream.
   *
   * @param is the input stream
   *
   * @throws IOException on any IO error
   */
  public void load(InputStream is) throws IOException {
    load(is, true);
  }

  /**
   * Loads MicroJava bytecode from the specified file.
   *
   * @param file the file
   *
   * @throws IOException on any IO error
   */
  public void load(File file) throws IOException {
    if (file == null) {
      throw new IllegalArgumentException("Null object file"); //$NON-NLS-1$
    }
    FileInputStream fis = new FileInputStream(file);
    try {
      load(fis, true);
    } catch (IOException ioe) { // NOPMD
      throw ioe;
    } finally {
      fis.close();
    }
  }

  /**
   * Loads MicroJava bytecode from the file specified by the fileName.
   *
   * @param fileName the file name
   *
   * @throws IOException on any IO error
   */
  public void load(String fileName) throws IOException {
    if (fileName == null) {
      throw new IllegalArgumentException("Null object file name"); //$NON-NLS-1$
    }
    FileInputStream fis = new FileInputStream(fileName);
    try {
      load(fis, true);
    } catch (IOException ioe) { // NOPMD
      throw ioe;
    } finally {
      fis.close();
    }
  }

  /**
   * Loads MicroJava bytecode from the specified byte iterator.
   *
   * @param mjbci the byte iterator
   *
   * @throws MJBytecodeFormatException if bytecode is invalid
   */
  public void load(MJByteIterator mjbci) throws // NOPMD
                                                MJBytecodeFormatException {
    if (mjbci == null) {
      throw new IllegalArgumentException(
                                       "Null bytecode iterator"); //$NON-NLS-1$
    }
    if (interpreter != null) {
      throw new MJVMStateException("MJVM is running"); //$NON-NLS-1$
    }
    if (debuger != null) {
      throw new MJVMStateException("MJVM is debugging"); //$NON-NLS-1$
    }
    int codeSize;
    int dataSize;
    int mainPC;
    if (mjbci.hasNext() && (mjbci.nextByte() == 'M') &&
        mjbci.hasNext() && (mjbci.nextByte() == 'J')) {
      int w = nextWord(mjbci);
      if (w <= 0) {
        throw new MJBytecodeFormatException("CodeSize <= 0"); //$NON-NLS-1$
      }
      codeSize = w;
      w = nextWord(mjbci);
      if (w < 0) {
        throw new MJBytecodeFormatException("DataSize < 0"); //$NON-NLS-1$
      }
      dataSize = w;
      w = nextWord(mjbci);
      if ((w < 0) || (w >= codeSize)) {
        throw new MJBytecodeFormatException(
                                      "mainPC out of code area"); //$NON-NLS-1$
      }
      mainPC = w;
      if (data == null) {
        data = new MJVMDataMemory(dataSize);
      } else {
        data.setMemory(new int[dataSize]);
      }
      if (code == null) {
        code = new MJVMCodeMemory(codeSize, mainPC);
        byte[] codemem = code.getCodeMemory();
        int cmlen = codemem.length;
        for (int i = 0; i < cmlen; i++) {
          if (!mjbci.hasNext()) {
            throw new MJBytecodeFormatException(INVLID_CODE_SIZE);
          }
          codemem[i] = mjbci.nextByte();
        }
        if (mjbci.hasNext()) {
          throw new MJBytecodeFormatException(INVLID_CODE_SIZE);
        }
      } else {
        byte[] codemem = new byte[codeSize];
        int cmlen = codemem.length;
        for (int i = 0; i < cmlen; i++) {
          if (!mjbci.hasNext()) {
            throw new MJBytecodeFormatException(INVLID_CODE_SIZE);
          }
          codemem[i] = mjbci.nextByte();
        }
        if (mjbci.hasNext()) {
          throw new MJBytecodeFormatException(INVLID_CODE_SIZE);
        }
        code.setCodeMemory(codemem);
        code.setMainPC(mainPC);
      }
      heap.reset();
      pstack.reset();
      estack.reset();
      breakPoints.removeAllBreakPoints();
      listeners.fireBytecodeLoaded();
    } else {
      throw new MJBytecodeFormatException("Invalid bytecode"); //$NON-NLS-1$
    }
  }

/*^**************************************************************************^*/
/*- Program Interpretation.                                                  -*/
/*^**************************************************************************^*/

  /**
   * Starts bytecode interpretation.
   * <p>
   *  <tt>WARNING: This method returns immediately.</tt>
   * </p>
   */
  public void run() {
    if ((interpreter == null) && (debuger == null)) {
      interpreter = new MJVMInterpreter();
      interpreter._start();
    }
  }

  /**
   * Interprets next bytecode instruction.
   */
  public void runNext() {
    if (interpreter == null) {
      if (debuger == null) {
        debuger = new MJVMDebuger(true);
        debuger._start();
      } else {
        debuger._next();
      }
    }
  }

  /**
   * Starts bytecode debugging.
   * <p>
   *  <tt>WARNING: This method returns immediately.</tt>
   * </p>
   */
  public void debug() {
    if (interpreter == null) {
      if (debuger == null) {
        debuger = new MJVMDebuger(false);
        debuger._start();
      } else {
        debuger._resume();
      }
    }
  }

  /**
   * Stops interpretation/debugging.
   */
  public void stop() {
    if (interpreter != null) { // NOPMD
      interpreter._stop();
    } else if (debuger != null) {
      debuger._stop();
    }
  }

  /**
   * Pauses debugging.
   */
  public void pause() {
    if (debuger != null) {
      debuger._pause();
    }
  }

  /**
   * Resumes debugging.
   */
  public void resume() {
    if (debuger != null) {
      debuger._resume();
    }
  }

  /**
   * Waits for finish of interpretation/debugging.
   */
  public void waitFor() {
    if (interpreter != null) { // NOPMD
      interpreter._join();
    } else if (debuger != null) {
      debuger._join();
    }
  }

  /**
   * Shutsdown the machine.
   */
  public void shutdown() {
    stop();
    listeners.fireMachineShutdown();
  }

/*^**************************************************************************^*/
/*- Trace/debug support.                                                     -*/
/*^**************************************************************************^*/

  /**
   * Returns code memory.
   *
   * @return code memory
   */
  public MJVMCodeMemory getCodeMemory() {
    return code;
  }

  /**
   * Returns data memory.
   *
   * @return data memory
   */
  public MJVMDataMemory getDataMemory() {
    return data;
  }

  /**
   * Returns heap memory.
   *
   * @return heap memory
   */
  public MJVMHeapMemory getHeapMemory() {
    return heap;
  }

  /**
   * Returns procedure stack.
   *
   * @return procedure stack
   */
  public MJVMPStack getPStackMemory() {
    return pstack;
  }

  /**
   * Returns expression stack.
   *
   * @return expression stack
   */
  public MJVMEStack getEStackMemory() {
    return estack;
  }

/*^**************************************************************************^*/
/*- Package-Private Methods.                                                 -*/
/*^**************************************************************************^*/

  int nextWord(MJByteIterator mjbci) throws MJBytecodeFormatException {
    int w = 0;
    for (int i = 3; i >= 0; i--) {
      if (!mjbci.hasNext()) {
        throw new MJBytecodeFormatException("Invalid bytecode"); //$NON-NLS-1$
      }
      w = w << 8;
      w = w | (0x000000FF & mjbci.nextByte());
    }
    return w;
  }

  void reset() {
    code.reset();
    data.reset();
    heap.reset();
    pstack.reset();
    estack.reset();
  }

  boolean interpretInstruction() throws MJVMRuntimeError { // NOPMD
    int opcode = code.nextByte();
    int val1;
    int val2;
    int adr;
    int len;
    StringBuffer sb;
    switch (opcode) {
      // load local variables
      case MJVMInstructionCodes.LOAD:
        estack.push(pstack.getLocal(code.nextByte()));
        break;
      case MJVMInstructionCodes.LOAD_0:
        estack.push(pstack.getLocal(0));
        break;
      case MJVMInstructionCodes.LOAD_1:
        estack.push(pstack.getLocal(1));
        break;
      case MJVMInstructionCodes.LOAD_2:
        estack.push(pstack.getLocal(2));
        break;
      case MJVMInstructionCodes.LOAD_3:
        estack.push(pstack.getLocal(3));
        break;
      // store local variables
      case MJVMInstructionCodes.STORE:
        pstack.setValueAt(code.nextByte(), estack.pop());
        break;
      case MJVMInstructionCodes.STORE_0:
        pstack.setValueAt(0, estack.pop());
        break;
      case MJVMInstructionCodes.STORE_1:
        pstack.setValueAt(1, estack.pop());
        break;
      case MJVMInstructionCodes.STORE_2:
        pstack.setValueAt(2, estack.pop());
        break;
      case MJVMInstructionCodes.STORE_3:
        pstack.setValueAt(3, estack.pop());
        break;
      // load global variables
      case MJVMInstructionCodes.GETSTATIC:
        estack.push(data.getValueAt(code.nextShort()));
        break;
      // store global variables
      case MJVMInstructionCodes.PUTSTATIC:
        data.setValueAt(code.nextShort(), estack.pop());
        break;
      // load object fields
      case MJVMInstructionCodes.GETFIELD:
        estack.push(heap.getObjectField(estack.pop(), code.nextShort()));
        break;
      // store object fields
      case MJVMInstructionCodes.PUTFIELD:
        heap.setObjectField(estack.pop(), estack.pop(), code.nextShort());
        break;
      // load constants
      case MJVMInstructionCodes.CONST_0:
        estack.push(0);
        break;
      case MJVMInstructionCodes.CONST_1:
        estack.push(1);
        break;
      case MJVMInstructionCodes.CONST_2:
        estack.push(2);
        break;
      case MJVMInstructionCodes.CONST_3:
        estack.push(3);
        break;
      case MJVMInstructionCodes.CONST_4:
        estack.push(4);
        break;
      case MJVMInstructionCodes.CONST_5:
        estack.push(5);
        break;
      case MJVMInstructionCodes.CONST_M1:
        estack.push(-1);
        break;
      case MJVMInstructionCodes.CONST:
        estack.push(code.nextWord());
        break;
      // arithmetic operations
      case MJVMInstructionCodes.ADD:
        estack.push(estack.pop() + estack.pop());
        break;
      case MJVMInstructionCodes.SUB:
        estack.push(-estack.pop() + estack.pop());
        break;
      case MJVMInstructionCodes.MUL:
        estack.push(estack.pop() * estack.pop());
        break;
      case MJVMInstructionCodes.DIV:
        val2 = estack.pop();
        if (val2 == 0) {
          throw new MJVMRuntimeError("Division by zero"); //$NON-NLS-1$
        }
        estack.push(estack.pop() / val2);
        break;
      case MJVMInstructionCodes.REM:
        val2 = estack.pop();
        if (val2 == 0) {
          throw new MJVMRuntimeError("Division by zero"); //$NON-NLS-1$
        }
        estack.push(estack.pop() % val2);
        break;
      case MJVMInstructionCodes.NEG:
        estack.push(-estack.pop());
        break;
      case MJVMInstructionCodes.SHL:
        val2 = estack.pop();
        estack.push(estack.pop() << val2);
        break;
      case MJVMInstructionCodes.SHR:
        val2 = estack.pop();
        estack.push(estack.pop() >> val2);
        break;
      case MJVMInstructionCodes.INC:
        adr = code.nextByte();
        val1 = pstack.getLocal(adr);
        pstack.setLocal(adr, val1 + code.nextByte());
        break;
      // object creation
      case MJVMInstructionCodes.NEW:
        estack.push(heap.alloc(code.nextShort()));
        break;
      case MJVMInstructionCodes.NEWARRAY:
        val1 = code.nextByte();
        len = estack.pop();
        if (val1 == 0) {
          adr = heap.alloc(len + 4);
        } else {
          adr = heap.alloc(4 * len + 4);
        }
        heap.setValueAt(adr, len);
        estack.push(adr + 1);
        break;
      // array access
      case MJVMInstructionCodes.ALOAD:
        estack.push(heap.getWordArrayField(estack.pop(), estack.pop()));
        break;
      case MJVMInstructionCodes.ASTORE:
        heap.setWordArrayField(estack.pop(), estack.pop(), estack.pop());
        break;
      case MJVMInstructionCodes.BALOAD:
        estack.push(heap.getByteArrayField(estack.pop(), estack.pop()));
        break;
      case MJVMInstructionCodes.BASTORE:
        heap.setByteArrayField((byte)estack.pop(), estack.pop(), estack.pop());
        break;
      case MJVMInstructionCodes.ARRAYLENGTH:
        estack.push(heap.getArrayLength(estack.pop()));
        break;
      // stack manipulation
      case MJVMInstructionCodes.POP:
        estack.pop();
        break;
      case MJVMInstructionCodes.DUP:
        val1 = estack.pop();
        estack.push(val1);
        estack.push(val1);
        break;
      case MJVMInstructionCodes.DUP2:
        val2 = estack.pop();
        val1 = estack.pop();
        estack.push(val1);
        estack.push(val2);
        estack.push(val1);
        estack.push(val2);
        break;
      // jumps
      case MJVMInstructionCodes.JMP:
        code.jumpRelative(code.nextShort() - 3);
        break;
      case MJVMInstructionCodes.JEQ:
        val1 = code.nextShort();
        if (estack.pop() == estack.pop()) {
          code.jumpRelative(val1 - 3);
        }
        break;
      case MJVMInstructionCodes.JNE:
        val1 = code.nextShort();
        if (estack.pop() != estack.pop()) {
          code.jumpRelative(val1 - 3);
        }
        break;
      case MJVMInstructionCodes.JLT:
        val1 = code.nextShort();
        if (estack.pop() > estack.pop()) {
          code.jumpRelative(val1 - 3);
        }
        break;
      case MJVMInstructionCodes.JLE:
        val1 = code.nextShort();
        if (estack.pop() >= estack.pop()) {
          code.jumpRelative(val1 - 3);
        }
        break;
      case MJVMInstructionCodes.JGT:
        val1 = code.nextShort();
        if (estack.pop() < estack.pop()) {
          code.jumpRelative(val1 - 3);
        }
        break;
      case MJVMInstructionCodes.JGE:
        val1 = code.nextShort();
        if (estack.pop() <= estack.pop()) {
          code.jumpRelative(val1 - 3);
        }
        break;
      // method calls
      case MJVMInstructionCodes.CALL:
        adr = code.nextShort();
        pstack.push(code.getPC());
        code.jumpRelative(adr - 3);
        break;
      case MJVMInstructionCodes.RETURN:
        adr = pstack.getPSP();
        if (adr == 0) {
          return false;
        }
        code.setPC(pstack.pop());
        break;
      case MJVMInstructionCodes.ENTER:
        int psize = code.nextByte();
        int lsize = code.nextByte();
        pstack.push(pstack.getFP());
        pstack.setFP(pstack.getPSP());
        for (int i = lsize - 1; i >= 0; i--) {
          pstack.push(0);
        }
        pstack.setPSP(pstack.getPSP() + psize);
        for (int i = psize - 1; i >= 0; i--) {
          pstack.setLocal(i, estack.pop());
        }
        break;
      case MJVMInstructionCodes.EXIT:
        pstack.setPSP(pstack.getFP());
        pstack.setFP(pstack.pop());
        break;
      // IO
      case MJVMInstructionCodes.READ:
        estack.push(inout.readInt());
        break;
      case MJVMInstructionCodes.PRINT:
        len = estack.pop();
        inout.print(estack.pop(), len);
        break;
      case MJVMInstructionCodes.BREAD:
        estack.push(inout.readByte());
        break;
      case MJVMInstructionCodes.BPRINT:
        len = estack.pop();
        inout.print((char)estack.pop(), len);
        break;
      case MJVMInstructionCodes.TRAP:
        sb = new StringBuffer("Trap("); //$NON-NLS-1$
        sb.append(code.nextByte());
        sb.append(')');
        throw new MJVMRuntimeError(sb.toString());
      default:
        sb = new StringBuffer("Illegal opcode: "); //$NON-NLS-1$
        sb.append(opcode);
        throw new MJVMRuntimeError(sb.toString());
    }
    return true;
  }

/*^**************************************************************************^*/
/*- Inner Classes.                                                           -*/
/*^**************************************************************************^*/

  /**
   * MJVM Thread.
   */
  abstract class MJVMThread extends Thread {
    boolean stop;
    /**
     * Constructs new MJVMThread object.
     *
     * @param name the thread name
     */
    protected MJVMThread(String name) {
      super(name);
      setDaemon(true);
    }
    /**
     * {@inheritDoc}
     */
    public void run() {
      elapsedTime = System.currentTimeMillis();
      try {
        while (!stop) {
          if (!loop()) {
            break;
          }
        }
      } catch (MJVMRuntimeError mjvmrte) {
        inout.println();
        inout.printError("Exception at address "); //$NON-NLS-1$
        inout.printError(code.getPC() - 1);
        inout.printlnError(": "); //$NON-NLS-1$
        inout.printlnError(mjvmrte.getMessage());
        inout.println();
      } catch (IllegalArgumentException iae) {
        inout.println();
        inout.printError("Exception at address "); //$NON-NLS-1$
        inout.printError(code.getPC() - 1);
        inout.printlnError(": ");
        inout.printlnError(iae.getMessage());
        inout.println();
      } catch (Throwable t) { // NOPMD
        inout.println();
        inout.printlnError("MJVM Internal Error"); //$NON-NLS-1$
        inout.println();
        inout.printlnError(t.getMessage());
      }
      elapsedTime = System.currentTimeMillis() - elapsedTime;
      if (stop) {
        fireStopped();
      } else {
        fireFinished();
      }
    }
    /**
     * Starts thread.
     */
    public void _start() { // NOPMD
      inout.clear();
      reset();
      fireStarted();
      start();
    }
    /**
     * Stops thread.
     */
    public void _stop() { // NOPMD
      stop = true;
      inout.abortInput();
      interrupt();
      _join();
    }
    /**
     * Joins the thread.
     */
    public void _join() { // NOPMD
      try {
        join();
      } catch (InterruptedException ioe) { // NOPMD
        // empty catch block
      }
    }
    /**
     * Performs loop.
     *
     * @return true if succeed
     *
     * @throws MJVMRuntimeError on MJVM runtime error
     */
    protected abstract boolean loop() throws MJVMRuntimeError;
    /**
     * Fires thread started event.
     */
    protected abstract void fireStarted();
    /**
     * Fires thread finished event.
     */
    protected abstract void fireFinished();
    /**
     * Fires thread stoped event.
     */
    protected abstract void fireStopped();
  }

  /**
   * MJ Bytecode Interpreter.
   */
  final class MJVMInterpreter extends MJVMThread {
    /**
     * Constructs new MJVMInterpreter object.
     */
    public MJVMInterpreter() {
      super("MJVMInterpreter"); //$NON-NLS-1$
    }
    /**
     * {@inheritDoc}
     */
    protected boolean loop() throws MJVMRuntimeError {
      return interpretInstruction();
    }
    /**
     * {@inheritDoc}
     */
    protected void fireStarted() {
      listeners.fireInterpretationStarted();
    }
    /**
     * {@inheritDoc}
     */
    protected void fireFinished() {
      interpreter = null;
      listeners.fireInterpretationFinished();
    }
    /**
     * {@inheritDoc}
     */
    protected void fireStopped() {
      interpreter = null;
      listeners.fireInterpretationStopped();
    }
  }

  /**
   * MJ Bytecode Debuger.
   */
  final class MJVMDebuger extends MJVMThread {
    private boolean pause;
    private boolean next;
    /**
     * Constructs new MJVMDebuger object.
     *
     * @param next the next
     */
    public MJVMDebuger(boolean next) {
      super("MJVMDebuger"); //$NON-NLS-1$
      this.next = next;
    }
    /**
     * {@inheritDoc}
     */
    protected boolean loop() throws MJVMRuntimeError {
      int pc = code.getPC();
      synchronized (this) {
        pause = (pause || next || breakPoints.isBreakPoint(pc));
        if (pause) {
          listeners.fireDebuggingPaused(pc);
          try {
            while (pause) {
              wait();
            }
          } catch (InterruptedException ie) {
            return false;
          }
          listeners.fireDebuggingResumed();
          pause = next;
          next = false;
        }
      }
      return interpretInstruction();
    }
    /**
     * Pauses the thread.
     */
    public void _pause() { // NOPMD
      synchronized (this) {
        pause = true;
      }
    }
    /**
     * Resumes the thread.
     */
    public void _resume() { // NOPMD
      synchronized (this) {
        pause = false;
        notify(); // NOPMD, FB: OK, only one thread can wait
      }
    }
    /**
     * Runs next instruction.
     */
    public void _next() { // NOPMD
      synchronized (this) {
        next = true;
        pause = false;
        notify(); // NOPMD, FB: OK, only one thread can wait
      }
    }
    /**
     * {@inheritDoc}
     */
    protected void fireStarted() {
      listeners.fireDebuggingStarted();
    }
    /**
     * {@inheritDoc}
     */
    protected void fireFinished() {
      debuger = null;
      listeners.fireDebuggingFinished();
    }
    /**
     * {@inheritDoc}
     */
    protected void fireStopped() {
      debuger = null;
      listeners.fireDebuggingStopped();
    }
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-26 20:23:51 Slobodan
   Initial version

*****************************************************************************^*/
