/*
 * @(#)MJVMCodeMemory.java  1.0.0 / 2008-05-26
 *
 * Copyright (c) 2008, ETF and contributors. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package yu.ac.bg.etf.ri4pp.mjtools.vm;

/**
 * MJVM Code Memory.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public final class MJVMCodeMemory extends MJVMMemory {

/*^**************************************************************************^*/
/*- Private Fields.                                                          -*/
/*^**************************************************************************^*/

  private byte[] memory;  // Code memory
  private int size;       // Code memory size
  private int pc;         // Program counter
  private int mainPC;     // main() method address

  private final MJVMMemoryListenerList memoryChageListeners;  //memory listeners

/*^**************************************************************************^*/
/*- Package-Private Constructor.                                             -*/
/*^**************************************************************************^*/

  /**
   * Constructs new MJVMCodeMemory object.
   *
   * @param size the code memory size
   * @param mainPC the main method address
   *
   * @throws IllegalArgumentException if size < 0
   */
  MJVMCodeMemory(int size, int mainPC) {
    super();
    if (size < 0) {
      throw new IllegalArgumentException(
                                     "Invalid code memory size"); //$NON-NLS-1$
    }
    if ((mainPC < 0) || (mainPC >= size)) {
      throw new IllegalArgumentException(
                                  "Invalid main method address"); //$NON-NLS-1$
    }
    this.size = size;
    this.mainPC = mainPC;
    memory = new byte[size];
    memoryChageListeners = new MJVMMemoryListenerList();
  }

/*^**************************************************************************^*/
/*- Public Methods.                                                          -*/
/*^**************************************************************************^*/

  /**
   * Returns value of the code memory location at specified address.
   *
   * @param address the address of code memory location
   *
   * @return value of the code memory location at specified address
   *
   * @throws IllegalArgumentException if address is out of memory ranged
   */
  public byte getValueAt(int address) {
    if ((address < 0) || (address >= size)) {
      throw new IllegalArgumentException(
                  "Code memory address out of range: " + address); //$NON-NLS-1$
    }
    return memory[address];
  }

  /**
   * Sets new value of the address specified code memory location.
   *
   * @param address the address of code memory location
   * @param value the new value
   *
   * @throws IllegalArgumentException if address is out of memory ranged
   */
  public void setValueAt(int address, byte value) {
    if ((address < 0) || (address >= size)) {
      throw new IllegalArgumentException(
                  "Code memory address out of range: " + address); //$NON-NLS-1$
    }
    byte oldValue = memory[address];
    memory[address] = value;
    fireMemoryLocationChanged(address, oldValue, value);
  }

  /**
   * Sets new value of the PC pointer.
   *
   * @param value the new PC pointer value
   *
   * @throws IllegalArgumentException if new PC pointer value is out of code
   *         memory range
   */
  public void setPC(int value) {
    if ((value < 0) || (value >= size)) {
      throw new IllegalArgumentException(
                  "PC value out of code memory range: " + value); //$NON-NLS-1$
    }
    int oldPC = pc;
    pc = value;
    firePCChanged(oldPC, pc);
  }

  /**
   * Returns the value of the PC pointer.
   *
   * @return the value of the PC pointer
   */
  public int getPC() {
    return pc;
  }

/*^**************************************************************************^*/
/*- Overridden Methods.                                                      -*/
/*^**************************************************************************^*/

  /**
   * Returns code memory size.
   *
   * @return code memory size
   */
  public int getSize() {
    return size;
  }

  /**
   * Registers new MJVMMemoryListener listener.
   *
   * @param mjvmml the MJVMMemoryListener
   */
  public void addMemoryListener(MJVMMemoryListener mjvmml) {
    memoryChageListeners.addMemoryListener(mjvmml);
  }

  /**
   * Removes specified MJVMMemoryListener listener.
   *
   * @param mjvmml the MJVMMemoryListener
   */
  public void removeMemoryListener(MJVMMemoryListener mjvmml) {
    memoryChageListeners.removeMemoryListener(mjvmml);
  }

/*^**************************************************************************^*/
/*- Package-Private Methods.                                                 -*/
/*^**************************************************************************^*/

  /**
   * Returns code memory.
   *
   * @return code memory
   */
  byte[] getCodeMemory() {
    return memory; // NOPMD
  }

  /**
   * Sets code memory.
   *
   * @param memory the new memory byte array
   */
  void setCodeMemory(byte[] memory) { // NOPMD
    this.memory = memory;
    size = memory.length;
    fireMemoryChanged();
  }

  /**
   * Returns main PC.
   *
   * @return main PC
   */
  int getMainPC() {
    return mainPC;
  }

  /**
   * Sets mainPC.
   *
   * @param mainPC the new mainPC value
   */
  void setMainPC(int mainPC) {
    this.mainPC = mainPC;
    int oldPC = pc;
    pc = mainPC;
    firePCChanged(oldPC, pc);
  }

/*^**************************************************************************^*/
/*- Code Interpretation Helpers.                                             -*/
/*^**************************************************************************^*/

  /**
   * Resets the memory.
   */
  void reset() {
    int oldPC = pc;
    pc = mainPC;
    firePCChanged(oldPC, pc);
  }

  /**
   * Jumps relative to the PC pointer.
   *
   * @param raddress address relative to pc
   *
   * @throws MJVMRuntimeError if jump address is out of code memory range
   */
  void jumpRelative(int raddress) throws MJVMRuntimeError {
    int oldPC = pc;
    pc = pc + raddress;
    if ((pc < 0) || (pc >= size)) {
      throw new MJVMRuntimeError(
                   "Jump out of code memory range: " + raddress); //$NON-NLS-1$
    }
    firePCChanged(oldPC, pc);
  }

  /**
   * Fetches next byte and increments PC.
   *
   * @return next byte
   *
   * @throws MJVMRuntimeError if PC is out of memory range
   */
  byte nextByte() throws MJVMRuntimeError {
    checkPCRange(pc);
    int oldPC = pc;
    pc = pc + 1;
    firePCChanged(oldPC, pc);
    return memory[oldPC];
  }

  /**
   * Fetches next short and increases PC.
   *
   * @return next short
   *
   * @throws MJVMRuntimeError if PC is out of memory range
   */
  short nextShort() throws MJVMRuntimeError  {
    checkPCRange(pc + 1);
    int oldPC = pc;
    int s1 = memory[pc++] & 0xFF;
    int s2 = memory[pc++] & 0xFF;
    firePCChanged(oldPC, pc);
    return (short)((s1 << 8) | s2);
  }

  /**
   * Fetches next word and increases PC.
   *
   * @return next word
   *
   * @throws MJVMRuntimeError if PC is out of memory range
   */
  int nextWord() throws MJVMRuntimeError {
    checkPCRange(pc + 3);
    int oldPC = pc;
    int w1 = memory[pc++] & 0xFF;
    int w2 = memory[pc++] & 0xFF;
    int w3 = memory[pc++] & 0xFF;
    int w4 = memory[pc++] & 0xFF;
    firePCChanged(oldPC, pc);
    return ((w1 << 24) | (w2 << 16) | (w3 << 8) | w4);
  }

/*^**************************************************************************^*/
/*- Private Methods.                                                         -*/
/*^**************************************************************************^*/

  private void checkPCRange(int pc) throws MJVMRuntimeError {
    if ((pc < 0) || (pc >= size)) {
      throw new MJVMRuntimeError(
                    "PC is out of code memory range: " + pc); //$NON-NLS-1$
    }
  }

  private void fireMemoryChanged() {
    MJVMMemoryListener[] listeners = memoryChageListeners.listeners;
    for (int i = memoryChageListeners.size - 1; i >= 0; i--) {
      listeners[i].memoryChanged();
    }
  }

  private void fireMemoryLocationChanged(int address, byte oldValue,
                                         byte newValue) {
    MJVMMemoryListener[] listeners = memoryChageListeners.listeners;
    for (int i = memoryChageListeners.size - 1; i >= 0; i--) {
      if (listeners[i] instanceof MJVMCodeMemoryListener) {
        ((MJVMCodeMemoryListener)listeners[i]).
            codeMemoryLocationChanged(address, oldValue, newValue);
      }
    }
  }

  private void firePCChanged(int oldValue, int newValue) {
    MJVMMemoryListener[] listeners = memoryChageListeners.listeners;
    for (int i = memoryChageListeners.size - 1; i >= 0; i--) {
      if (listeners[i] instanceof MJVMCodeMemoryListener) {
        ((MJVMCodeMemoryListener)listeners[i]).
            codeMemoryPCChanged(oldValue, newValue);
      }
    }
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-26 20:31:33 Slobodan
   Initial version

*****************************************************************************^*/
