/*
 * @(#)MJVMHeapMemory.java  1.0.0 / 2008-05-26
 *
 * Copyright (c) 2008, ETF and contributors. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package yu.ac.bg.etf.ri4pp.mjtools.vm;

/**
 * MJVM Heap Memory.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public final class MJVMHeapMemory extends MJVMWordMemory {

/*^**************************************************************************^*/
/*- Private Static Fields.                                                   -*/
/*^**************************************************************************^*/

  private static final String ERROR_INDEX_OUT_OF_BOUNDS;
  private static final String ERROR_INVALID_ARRAY_ADDRESS;
  private static final String ERROR_NULL_REFERENCE;

/*^**************************************************************************^*/
/*- Static Initialization.                                                   -*/
/*^**************************************************************************^*/

  static {
    ERROR_INDEX_OUT_OF_BOUNDS = "Index out of bounds: "; //$NON-NLS-1$;
    ERROR_INVALID_ARRAY_ADDRESS = "Invalid array address: "; //$NON-NLS-1$
    ERROR_NULL_REFERENCE = "Null reference"; //$NON-NLS-1$
  }

/*^**************************************************************************^*/
/*- Private Field.                                                           -*/
/*^**************************************************************************^*/

  private int free;  // Pointer to the next free heap address

/*^**************************************************************************^*/
/*- Package-Private Constructor.                                             -*/
/*^**************************************************************************^*/

  /**
   * Constructs new MJVMHeap object.
   *
   * @param size the heap memory size
   *
   * @throws IllegalArgumentException if size <= 0
   */
  MJVMHeapMemory(int size) {
    super(size);
    free = 1;
  }

/*^**************************************************************************^*/
/*- Public Methods.                                                          -*/
/*^**************************************************************************^*/

  /**
   * Returns free pointer value.
   *
   * @return free pointer value
   */
  public int getFree() {
    return free;
  }

  /**
   * Sets new free pointer value.
   *
   * @param value the new free pointer value
   *
   * @throws IllegalArgumentException if value is out of heap memory range
   */
  public void setFree(int value) {
    if ((value < 0) || (value >= size)) {
      StringBuffer sb = new StringBuffer("Free pointer out of range: ");
      sb.append(value);
      throw new IllegalArgumentException(sb.toString());
    }
    int oldFree = free;
    free = value;
    fireFreeChanged(oldFree, free);
  }

/*^**************************************************************************^*/
/*- Package-Private Methods.                                                 -*/
/*^**************************************************************************^*/

  /**
   * Resets this memory.
   */
  void reset() {
    super.reset();
    int oldFree = free;
    free = 1;
    fireFreeChanged(oldFree, free);
  }

  /**
   * Allocates new object.
   *
   * @param size the object size
   *
   * @return object address

   * @throws MJVMRuntimeError if heap is full
   */
  int alloc(int size) throws MJVMRuntimeError {
    int address = free;
    free += ((size + 3) >> 2);
    if (free > this.size) {
      throw new MJVMRuntimeError("Heap overflow"); //$NON-NLS-1$
    }
    fireFreeChanged(address, free);
    return address;
  }

  /**
   * Returns field of the specified object.
   *
   * @param objectAddress the object address
   * @param fieldOffset the field offset
   *
   * @return field of the specified object
   *
   * @throws MJVMRuntimeError if object address is invalid
   */
  int getObjectField(int objectAddress,
                     int fieldOffset) throws MJVMRuntimeError {
    int radr = objectAddress + fieldOffset;
    if ((objectAddress < 0) || (radr >= size)) {
      throw new MJVMRuntimeError("Invalid heap object address: " + //$NON-NLS-1$
                                 objectAddress);
    }
    if (objectAddress == 0) {
      throw new MJVMRuntimeError(ERROR_NULL_REFERENCE);
    }
    return memory[radr];
  }

  /**
   * Sets field of the specified object.
   *
   * @param val the new value
   * @param objectAddress the object address
   * @param fieldOffset the field offset
   *
   * @throws MJVMRuntimeError if object address is invalid
   */
  void setObjectField(int val, int objectAddress,
                      int fieldOffset) throws MJVMRuntimeError {
    int radr = objectAddress + fieldOffset;
    if ((objectAddress < 0) || (radr >= size)) {
      throw new MJVMRuntimeError("Invalid heap object address: " + //$NON-NLS-1$
                                 objectAddress);
    }
    if (objectAddress == 0) {
      throw new MJVMRuntimeError(ERROR_NULL_REFERENCE);
    }
    setValueAt(radr, val);
  }

  /**
   * Returns word array field.
   *
   * @param fieldIndex the field index
   * @param arrayAddress the array address
   *
   * @return word array field
   *
   * @throws MJVMRuntimeError if index or address is invalid
   */
  int getWordArrayField(int fieldIndex,
                        int arrayAddress) throws MJVMRuntimeError {
    int fadr = arrayAddress + fieldIndex;
    if ((arrayAddress < 0) || (arrayAddress == 1) || (fadr >= size)) {
      throw new MJVMRuntimeError(ERROR_INVALID_ARRAY_ADDRESS + arrayAddress);
    }
    if (arrayAddress == 0) {
      throw new MJVMRuntimeError(ERROR_NULL_REFERENCE);
    }
    int len = memory[arrayAddress - 1];
    if ((fieldIndex < 0) || (fieldIndex >= len)) {
      throw new MJVMRuntimeError(ERROR_INDEX_OUT_OF_BOUNDS + fieldIndex);
    }
    return memory[fadr];
  }

  /**
   * Sets word array field.
   *
   * @param val the new value
   * @param fieldIndex the field index
   * @param arrayAddress the array address
   *
   * @throws MJVMRuntimeError if index or address is invalid
   */
  void setWordArrayField(int val, int fieldIndex,
                         int arrayAddress) throws MJVMRuntimeError {
    int fadr = arrayAddress + fieldIndex;
    if ((arrayAddress < 0) || (arrayAddress == 1) || (fadr >= size)) {
      throw new MJVMRuntimeError(ERROR_INVALID_ARRAY_ADDRESS + arrayAddress);
    }
    if (arrayAddress == 0) {
      throw new MJVMRuntimeError(ERROR_NULL_REFERENCE);
    }
    int len = memory[arrayAddress - 1];
    if ((fieldIndex < 0) || (fieldIndex >= len)) {
      throw new MJVMRuntimeError(ERROR_INDEX_OUT_OF_BOUNDS + fieldIndex);
    }
    setValueAt(fadr, val);
  }

  /**
   * Returns byte array field.
   *
   * @param fieldIndex the field index
   * @param arrayAddress the array address
   *
   * @return word array field
   *
   * @throws MJVMRuntimeError if index or address is invalid
   */
  byte getByteArrayField(int fieldIndex,
                         int arrayAddress) throws MJVMRuntimeError {
    int fadr = arrayAddress + fieldIndex;
    if ((arrayAddress < 0) || (arrayAddress == 1) || (fadr >= size)) {
      throw new MJVMRuntimeError(ERROR_INVALID_ARRAY_ADDRESS + arrayAddress);
    }
    if (arrayAddress == 0) {
      throw new MJVMRuntimeError(ERROR_NULL_REFERENCE);
    }
    int len = memory[arrayAddress - 1];
    if ((fieldIndex < 0) || (fieldIndex >= len)) {
      throw new MJVMRuntimeError(ERROR_INDEX_OUT_OF_BOUNDS + fieldIndex);
    }
    return getByte(memory[arrayAddress + fieldIndex / 4], fieldIndex % 4);
  }

  /**
   * Sets byte array field.
   *
   * @param val the new value
   * @param fieldIndex the field index
   * @param arrayAddress the array address
   *
   * @throws MJVMRuntimeError if index or address is invalid
   */
  void setByteArrayField(byte val, int fieldIndex,
                         int arrayAddress) throws MJVMRuntimeError {
    int fadr = arrayAddress + fieldIndex;
    if ((arrayAddress < 0) || (arrayAddress == 1) || (fadr >= size)) {
      throw new MJVMRuntimeError(ERROR_INVALID_ARRAY_ADDRESS + arrayAddress);
    }
    if (arrayAddress == 0) {
      throw new MJVMRuntimeError(ERROR_NULL_REFERENCE);
    }
    int len = memory[arrayAddress - 1];
    if ((fieldIndex < 0) || (fieldIndex >= len)) {
      throw new MJVMRuntimeError(ERROR_INDEX_OUT_OF_BOUNDS + fieldIndex);
    }
    int memind = arrayAddress + fieldIndex / 4;
    setValueAt(memind, setByte(memory[memind], fieldIndex % 4, val));
  }

  /**
   * Returns array length.
   *
   * @param arrayAddress the array address
   *
   * @return array length
   *
   * @throws MJVMRuntimeError if address is invalid
   */
  int getArrayLength(int arrayAddress) throws MJVMRuntimeError {
    if ((arrayAddress < 0) || (arrayAddress == 1) || (arrayAddress >= size)) {
      throw new MJVMRuntimeError(ERROR_INVALID_ARRAY_ADDRESS + arrayAddress);
    }
    if (arrayAddress == 0) {
      throw new MJVMRuntimeError(ERROR_NULL_REFERENCE);
    }
    return memory[arrayAddress - 1];
  }

/*^**************************************************************************^*/
/*- Private Methods.                                                         -*/
/*^**************************************************************************^*/

  private byte getByte(int value, int n) {
    return (byte)(value << (8 * n) >>> 24);
  }

  private int setByte(int value, int n, byte b) {
    int delta = (3 - n) * 8;
    int mask = ~(255 << delta);
    int by = (b & 0x00FF) << delta;
    return (value & mask) ^ by;
  }

  private void fireFreeChanged(int oldValue, int newValue) {
    MJVMMemoryListener[] listeners = memoryChageListeners.listeners;
    for (int i = memoryChageListeners.size - 1; i >= 0; i--) {
      if (listeners[i] instanceof MJVMHeapMemoryListener) {
        ((MJVMHeapMemoryListener)listeners[i]).
                                            heapFreeChanged(oldValue, newValue);
      }
    }
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-26 20:42:53 Slobodan
   Initial version

*****************************************************************************^*/
