/*
 * @(#)MJVMListenerList.java  1.0.0 / 2008-05-26
 *
 * Copyright (c) 2008, ETF and contributors. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package yu.ac.bg.etf.ri4pp.mjtools.vm;

/**
 * MJVM Listener List.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
final class MJVMListenerList {

/*^**************************************************************************^*/
/*- Private Fields.                                                          -*/
/*^**************************************************************************^*/

  private MJVMListener[] listeners;
  private int size;

/*^**************************************************************************^*/
/*- Public Constructor.                                                      -*/
/*^**************************************************************************^*/

  /**
   * Constructs new MJVMListenerList object.
   */
  public MJVMListenerList() {
    listeners = new MJVMListener[10];
  }

/*^**************************************************************************^*/
/*- Public Methods.                                                          -*/
/*^**************************************************************************^*/

  /**
   * Adds a <tt>MJVMListener</tt> to the list.
   *
   * @param mjvmListener the MJVMListener
   */
  public void addMJVMListener(MJVMListener mjvmListener) {
    if (mjvmListener == null) {
      throw new IllegalArgumentException("Null mjvm listener"); //$NON-NLS-1$
    }
    if (size >= listeners.length) {
      MJVMListener[] oldListeners = listeners;
      listeners = new MJVMListener[((3 * size) >> 1) + 1];
      System.arraycopy(oldListeners, 0, listeners, 0, size - 1);
    }
    listeners[size++] = mjvmListener;
  }

  /**
   * Removes a <tt>MJVMListener</tt> from the list.
   *
   * @param mjvmListener the MJVMListener
   */
  public void removeMJVMListener(MJVMListener mjvmListener) {
    if (mjvmListener != null) {
      int index = listeners.length - 1;
      while (index >= 0) {
        if (listeners[index] == mjvmListener) {
          int numMoved = size - index - 1;
          if (numMoved > 0) {
            System.arraycopy(listeners, index + 1, listeners, index, numMoved);
          }
          listeners[--size] = null;
          break;
        }
        index = index - 1;
      }
    }
  }

  /**
   * Removes all listeners.
   */
  public void removeAllMJVMListeners() {
    size = 0;
  }

  /**
   * Notifies listeners that bytecode has loaded.
   */
  public void fireBytecodeLoaded() {
    final MJVMListener[] listeners = this.listeners;
    for (int i = size - 1; i >= 0; i--) {
      listeners[i].bytecodeLoaded();
    }
  }

  /**
   * Notifies listeners that bytecode interpretation has started.
   */
  public void fireInterpretationStarted() {
    final MJVMListener[] listeners = this.listeners;
    for (int i = size - 1; i >= 0; i--) {
      listeners[i].interpretationStarted();
    }
  }

  /**
   * Notifies listeners that bytecode interpretation has finished.
   */
  public void fireInterpretationFinished() {
    final MJVMListener[] listeners = this.listeners;
    for (int i = size - 1; i >= 0; i--) {
      listeners[i].interpretationFinished();
    }
  }

  /**
   * Notifies listeners that bytecode interpretation has stopped.
   */
  public void fireInterpretationStopped() {
    final MJVMListener[] listeners = this.listeners;
    for (int i = size - 1; i >= 0; i--) {
      listeners[i].interpretationStopped();
    }
  }

  /**
   * Notifies listeners that bytecode debugging has started.
   */
  public void fireDebuggingStarted() {
    final MJVMListener[] listeners = this.listeners;
    for (int i = size - 1; i >= 0; i--) {
      listeners[i].debuggingStarted();
    }
  }

  /**
   * Notifies listeners that bytecode debugging has finished.
   */
  public void fireDebuggingFinished() {
    final MJVMListener[] listeners = this.listeners;
    for (int i = size - 1; i >= 0; i--) {
      listeners[i].debuggingFinished();
    }
  }

  /**
   * Notifies listeners that bytecode debugging has stopped.
   */
  public void fireDebuggingStopped() {
    final MJVMListener[] listeners = this.listeners;
    for (int i = size - 1; i >= 0; i--) {
      listeners[i].debuggingStopped();
    }
  }

  /**
   * Notifies listeners that bytecode debugging has paused.
   *
   * @param address current PC value
   */
  public void fireDebuggingPaused(int address) {
    final MJVMListener[] listeners = this.listeners;
    for (int i = size - 1; i >= 0; i--) {
      listeners[i].debuggingPaused(address);
    }
  }

  /**
   * Notifies listeners that bytecode debugging has resumed.
   */
  public void fireDebuggingResumed() {
    final MJVMListener[] listeners = this.listeners;
    for (int i = size - 1; i >= 0; i--) {
      listeners[i].debuggingResumed();
    }
  }

  /**
   * Notifies listener that MJVM has shutdown.
   */
  public void fireMachineShutdown() {
    final MJVMListener[] listeners = this.listeners;
    for (int i = size - 1; i >= 0; i--) {
      listeners[i].machineShutdown();
    }
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-26 20:56:34 Slobodan
   Initial version

*****************************************************************************^*/
