/*
 * @(#)MJVMPStack.java  1.0.0 / 2008-05-26
 *
 * Copyright (c) 2008, ETF and contributors. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package yu.ac.bg.etf.ri4pp.mjtools.vm;

/**
 * MJVM Procedure (Method) Stack.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public final class MJVMPStack extends MJVMWordMemory {

/*^**************************************************************************^*/
/*- Private Fields.                                                          -*/
/*^**************************************************************************^*/

  private int psp;  // Procedure stack pointer
  private int fp;   // Frame pointer

/*^**************************************************************************^*/
/*- Package-Private Constructor.                                             -*/
/*^**************************************************************************^*/

  /**
   * Constructs new MJVMPStack object.
   *
   * @param size the procedure stack size
   *
   * @throws IllegalArgumentException if size <= 0
   */
  MJVMPStack(int size) {
    super(size);
  }

/*^**************************************************************************^*/
/*- Public Methods.                                                          -*/
/*^**************************************************************************^*/

  /**
   * Returns procedure stack pointer value.
   *
   * @return procedure stack pointer value
   */
  public int getPSP() {
    return psp;
  }

  /**
   * Sets new value of the procedure stack pointer.
   *
   * @param value the new procedure stack pointer value
   *
   * @throws IllegalArgumentException if value is out of stack range
   */
  public void setPSP(int value) {
    if ((value < 0) || (value >= size)) {
      throw new IllegalArgumentException("PSP out of range: " + //$NON-NLS-1$
                                         value);
    }
    int oldPSP = psp;
    psp = value;
    firePSPChanged(oldPSP, psp);
  }

  /**
   * Returns frame pointer value.
   *
   * @return frame pointer value
   */
  public int getFP() {
    return fp;
  }

  /**
   * Sets new value of the frame pointer.
   *
   * @param value the new frame pointer value
   *
   * @throws IllegalArgumentException if value is out of stack range
   */
  public void setFP(int value) {
    if ((value < 0) || (value >= size)) {
      throw new IllegalArgumentException("FP out of range: " + //$NON-NLS-1$
                                         value);
    }
    int oldFP = fp;
    fp = value;
    fireFPChanged(oldFP, fp);
  }

/*^**************************************************************************^*/
/*- Package-Private Methods.                                                 -*/
/*^**************************************************************************^*/

  /**
   * Resets the stack.
   */
  void reset() {
    super.reset();
    int oldPSP = psp;
    psp = 0;
    firePSPChanged(oldPSP, psp);
    int oldFP = fp;
    fp = 0;
    fireFPChanged(oldFP, fp);
  }

  /**
   * Returns and removes value from the top of the stack.
   *
   * @throws MJVMRuntimeError if the stack is empty
   *
   * @return value from the top of the stack
   */
  int pop() throws MJVMRuntimeError {
    if (psp == 0) {
      throw new MJVMRuntimeError("Procedure stack underflow"); //$NON-NLS-1$
    }
    int oldPSP = psp;
    psp = psp - 1;
    firePSPChanged(oldPSP, psp);
    return memory[psp];
  }

  /**
   * Pushes new value onto the stack.
   *
   * @param value the new value to be pushed
   *
   * @throws MJVMRuntimeError if the stack is full
   */
  void push(int value) throws MJVMRuntimeError {
    if (psp == size) {
      throw new MJVMRuntimeError("Procedure stack overflow"); //$NON-NLS-1$
    }
    int oldPSP = psp;
    psp = psp + 1;
    firePSPChanged(oldPSP, psp);
    memory[oldPSP] = value;
  }

  /**
   * Returns value of the stack location specified by address relative to the
   * frame pointer.
   *
   * @param address the address of procedure stack location relative to the
   *        frame pointer
   *
   * @return stack value specified by address relative to the frame pointer
   *
   * @throws MJVMRuntimeError if address is out of current frame range
   */
  int getLocal(int address) throws MJVMRuntimeError {
    int radr = fp + address;
    if ((address < 0) || (radr >= psp)) {
      throw new MJVMRuntimeError(
         "PStack address out of frame range: " + address); //$NON-NLS-1$
    }
    return memory[fp + address];
  }

  /**
   * Sets new value of the stack location specified by the address relative to
   * the frame pointer.
   *
   * @param address the address of procedure stack location relative to
   *        frame pointer
   * @param value the new value
   *
   * @throws MJVMRuntimeError if address is out of current frame range
   */
  void setLocal(int address, int value) throws MJVMRuntimeError {
    int radr = fp + address;
    if ((address < 0) || (radr >= psp)) {
      throw new MJVMRuntimeError(
          "PStack address out of frame range: " + address); //$NON-NLS-1$
    }
    memory[fp + address] = value;
  }

/*^**************************************************************************^*/
/*- Private Methods.                                                         -*/
/*^**************************************************************************^*/

  private void firePSPChanged(int oldValue, int newValue) {
    MJVMMemoryListener[] listeners = memoryChageListeners.listeners;
    for (int i = memoryChageListeners.size - 1; i >= 0; i--) {
      if (listeners[i] instanceof MJVMPStackListener) {
        ((MJVMPStackListener)listeners[i]).pstackPSPChanged(oldValue, newValue);
      }
    }
  }

  private void fireFPChanged(int oldValue, int newValue) {
    MJVMMemoryListener[] listeners = memoryChageListeners.listeners;
    for (int i = memoryChageListeners.size - 1; i >= 0; i--) {
      if (listeners[i] instanceof MJVMPStackListener) {
        ((MJVMPStackListener)listeners[i]).pstackFPChanged(oldValue, newValue);
      }
    }
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-26 20:59:45 Slobodan
   Initial version

*****************************************************************************^*/
