/*
 * @(#)MJVMWordMemory.java  1.0.0 / 2008-05-26
 *
 * Copyright (c) 2008, ETF and contributors. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package yu.ac.bg.etf.ri4pp.mjtools.vm;

/**
 * MJVM Abstract Word Memory.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
abstract class MJVMWordMemory extends MJVMMemory {

/*^**************************************************************************^*/
/*- Package-Private Fields.                                                  -*/
/*^**************************************************************************^*/

  int[] memory;  // Code memory
  int size;      // Code memory size

  MJVMMemoryListenerList memoryChageListeners;  // Memory listeners

/*^**************************************************************************^*/
/*- Protected Constructor.                                                   -*/
/*^**************************************************************************^*/

  /**
   * Constructs new MJVMWordMemory object.
   *
   * @param size the memory size
   *
   * @throws IllegalArgumentException if size < 0
   */
  protected MJVMWordMemory(int size) {
    super();
    if (size < 0) {
      throw new IllegalArgumentException(
                                     "Invalid word memory size"); //$NON-NLS-1$
    }
    this.size = size;
    memory = new int[size];
    memoryChageListeners = new MJVMMemoryListenerList();
  }

/*^**************************************************************************^*/
/*- Public Methods.                                                          -*/
/*^**************************************************************************^*/

  /**
   * Returns value of the memory location at specified address.
   *
   * @param address the address of memory location
   *
   * @return  value of the memory location at specified address
   *
   * @throws IllegalArgumentException if address is out of memory range
   */
  public int getValueAt(int address) {
    if ((address < 0) || (address >= size)) {
      throw new IllegalArgumentException(
                  "Memory address out of range: " + address); //$NON-NLS-1$
    }
    return memory[address];
  }

  /**
   * Sets new value of the address specified memory location.
   *
   * @param address the address of memory location
   * @param value the new value
   *
   * @throws IllegalArgumentException if address is out of memory range
   */
  public void setValueAt(int address, int value) {
    if ((address < 0) || (address >= size)) {
      throw new IllegalArgumentException(
                  "Memory address out of range: " + address); //$NON-NLS-1$
    }
    memory[address] = value;
    int oldValue = memory[address];
    memory[address] = value;
    fireWordMemoryLocationChanged(address, oldValue, value);

  }

/*^**************************************************************************^*/
/*- Overridden Methods.                                                      -*/
/*^**************************************************************************^*/

  /**
   * Returns memory size.
   *
   * @return memory size
   */
  public int getSize() {
    return size;
  }

  /**
   * Registers new MJVMMemoryListener listener.
   *
   * @param mjvmml the MJVMMemoryListener
   */
  public void addMemoryListener(MJVMMemoryListener mjvmml) {
    memoryChageListeners.addMemoryListener(mjvmml);
  }

  /**
   * Removes specified MJVMMemoryListener listener.
   *
   * @param mjvmml the MJVMMemoryListener
   */
  public void removeMemoryListener(MJVMMemoryListener mjvmml) {
    memoryChageListeners.removeMemoryListener(mjvmml);
  }

/*^**************************************************************************^*/
/*- Package-Private Methods.                                                 -*/
/*^**************************************************************************^*/

  /**
   * Returns memory.
   *
   * @return memory
   */
  int[] getMemory() {
    return memory; // NOPMD
  }

  /**
   * Sets memory.
   *
   * @param memory the new memory array
   */
  void setMemory(int[] memory) { // NOPMD
    this.memory = memory;
    size = memory.length;
    fireMemoryChanged();
  }

  /**
   * Resets memory.
   */
  void reset() {
    for (int i = memory.length - 1; i >= 0; i--) {
      memory[i] = 0;
    }
  }

/*^**************************************************************************^*/
/*- Protected Methods.                                                       -*/
/*^**************************************************************************^*/

  /**
   * Fires memory changed event.
   */
  protected void fireMemoryChanged() {
    final MJVMMemoryListener[] listeners = memoryChageListeners.listeners;
    for (int i = memoryChageListeners.size - 1; i >= 0; i--) {
      listeners[i].memoryChanged();
    }
  }

  /**
   * Fires memory changed event.
   *
   * @param address the address
   * @param oldValue the old memory value
   * @param newValue the new memory value
   */
  protected void fireWordMemoryLocationChanged(int address, int oldValue,
                                               int newValue) {
    final MJVMMemoryListener[] listeners = memoryChageListeners.listeners;
    for (int i = memoryChageListeners.size - 1; i >= 0; i--) {
      if (listeners[i] instanceof MJVMWordMemoryListener) {
        ((MJVMWordMemoryListener)listeners[i]).
            wordMemoryLocationChanged(address, oldValue, newValue);
      }
    }
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-26 21:34:11 Slobodan
   Initial version

*****************************************************************************^*/
