/*
 * @(#)MJVMCodeTableModel.java  1.0.0 / 2008-05-26
 *
 * Copyright (c) 2008, ETF and contributors. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package yu.ac.bg.etf.ri4pp.mjtools.vm.gui;

import javax.swing.table.AbstractTableModel;

import yu.ac.bg.etf.ri4pp.mjtools.asm.MJVMInstructionDescriptor;
import yu.ac.bg.etf.ri4pp.mjtools.asm.MJVMInstructions;
import yu.ac.bg.etf.ri4pp.mjtools.vm.MJVMBreakPoints;
import yu.ac.bg.etf.ri4pp.mjtools.vm.MJVMCodeMemory;
import yu.ac.bg.etf.ri4pp.mjtools.vm.MJVMCodeMemoryListener;
import yu.ac.bg.etf.ri4pp.mjtools.vm.MJVMInstructionCodes;

/**
 * MJVM Code Table Model.
 *
 * @version 1.0.0
 * @author Slobodan Vrkacevic (e-mail: slobodan.vrkacevic@gmail.com)
 */
final class MJVMCodeTableModel extends AbstractTableModel
                               implements MJVMCodeMemoryListener {

/*^**************************************************************************^*/
/*- Package-Private Static Field.                                            -*/
/*^**************************************************************************^*/

  static final long serialVersionUID = -8680180131153111931L;

/*^**************************************************************************^*/
/*- Private Fields.                                                          -*/
/*^**************************************************************************^*/

  private final MJVMCodeMemory mjvmCodeMemory;
  private int tableSize;
  private int codeSize;
  private String[] opcode;
  private final MJVMBreakPoints breakPoints;
  private boolean[] validBreakPoints;

/*^**************************************************************************^*/
/*- Public Constructor.                                                      -*/
/*^**************************************************************************^*/

  /**
   * Constructs new MJVMCodeTableModel object.
   *
   * @param mjvmCodeMemory the MJVMCodeMemory
   * @param breakPoints the MJVMBreakPoints
   */
  public MJVMCodeTableModel(MJVMCodeMemory mjvmCodeMemory,
                            MJVMBreakPoints breakPoints) {
    super();
    this.mjvmCodeMemory = mjvmCodeMemory;
    this.breakPoints = breakPoints;
    initialize();
    mjvmCodeMemory.addMemoryListener(this);
  }

/*^**************************************************************************^*/
/*- MJVMCodeMemoryListener Implementation.                                   -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  public void codeMemoryLocationChanged(int address, byte oldValue,
                                        byte newValue) {
    int firstRow = getPrevBreakPoint(address);
    int lastRow;
    if (validBreakPoints[address]) {
      lastRow = codeSize;
    } else {
      lastRow = getNextBreakPoint(address);
    }
    decode(firstRow, lastRow);
    fireTableRowsUpdated(firstRow, lastRow);
  }

  /**
   * {@inheritDoc}
   */
  public void codeMemoryPCChanged(int oldValue, int newValue) {
    fireTableCellUpdated(oldValue, 0);
    fireTableCellUpdated(newValue, 0);
  }

  /**
   * {@inheritDoc}
   */
  public void memoryChanged() {
    initialize();
    fireTableDataChanged();
  }

/*^**************************************************************************^*/
/*- Overridden Methods.                                                      -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  public int getColumnCount() {
    return 5;
  }

  /**
   * {@inheritDoc}
   */
  public Class getColumnClass(int column) {
    switch (column) {
      case 0:
        return Boolean.class;
      case 1:
        return Integer.class; // FB: OK
      case 2:
        return Integer.class; // FB: OK
      case 3:
        return String.class;
      case 4:
        return Boolean.class;
      default:
        return String.class;
    }
  }

  /**
   * {@inheritDoc}
   */
  public String getColumnName(int column) {
    switch (column) {
      case 0:
        return "#"; //$NON-NLS-1$
      case 1:
        return "Adr"; //$NON-NLS-1$
      case 2:
        return "Val*"; //$NON-NLS-1$
      case 3:
        return "Opcode"; //$NON-NLS-1$
      case 4:
        return "BP*"; //$NON-NLS-1$
      default:
        return null;
    }
  }

  /**
   * {@inheritDoc}
   */
  public int getRowCount() {
    return tableSize;
  }

  /**
   * {@inheritDoc}
   */
  public Object getValueAt(int row, int col) {
    switch (col) {
      case 0:
        return Boolean.valueOf(row == mjvmCodeMemory.getPC());
      case 1:
        return new Integer(row);
      case 2:
        if (row >= codeSize) {
          return new Integer(0);
        }
        // Note:                             signed byte -> byte
        return new Integer(mjvmCodeMemory.getValueAt(row) & 0xFF);
      case 3:
        if (row >= codeSize) {
          return "???"; //$NON-NLS-1$
        }
        return opcode[row];
      case 4:
        if ((row < codeSize) && (validBreakPoints[row])) {
          return Boolean.valueOf(breakPoints.isBreakPoint(row));
        }
        return null;
      default:
        return null;
    }
  }

  /**
   * {@inheritDoc}
   */
  public boolean isCellEditable(int row, int column) {
    return ((row < codeSize) &&
            ((column == 2) || ((column == 4) && validBreakPoints[row])));
  }

  /**
   * {@inheritDoc}
   */
  public void setValueAt(Object value, int row, int col) {
    switch (col) { // NOPMD
      case 2:
        // Note: byte -> signed byte
        short s = Short.parseShort(value.toString());
        if ((s >= 0) && (s < 256)) {
          mjvmCodeMemory.setValueAt(row, (byte)s);
        }
        break;
      case 4:
        if (((Boolean)value).booleanValue()) {
          breakPoints.addBreakPoint(row);
        } else {
          breakPoints.removeBreakPoint(row);
        }
        break;
    }
  }

/*^**************************************************************************^*/
/*- Private Methods.                                                         -*/
/*^**************************************************************************^*/

  private void decode(int fromAddress, int toAddress) {
    while (fromAddress < toAddress) {
      int opc = mjvmCodeMemory.getValueAt(fromAddress);
      MJVMInstructionDescriptor mjvmid = MJVMInstructions.getInstruction(opc);
      int opcnt = mjvmid.getOperandCount();
      int optype = mjvmid.getOperandType();
      opcode[fromAddress] = mjvmid.getName();
      validBreakPoints[fromAddress++] = ((opc >= MJVMInstructionCodes.LOAD) &&
                                         (opc <= MJVMInstructionCodes.TRAP));
      if ((opcnt > 0) && (fromAddress < toAddress)) {
        fromAddress = decodeOperand(optype, fromAddress);
        if ((opcnt == 2) && (fromAddress < toAddress)) {
          fromAddress = decodeOperand(optype, fromAddress);
        }
      }

    }
  }

  private int decodeOperand(int optype, int address) {
    int lastAdr = address + optype;
    if (lastAdr >= codeSize) {
      lastAdr = codeSize - 1;
    }
    int val = 0;
    while (address < lastAdr) {
      validBreakPoints[address] = false;
      opcode[address] = null;
      val = val << 8;
      val = val | (0x000000FF & mjvmCodeMemory.getValueAt(address++));
    }
    opcode[address - 1] = String.valueOf(val);
    return address;
  }

  private int getPrevBreakPoint(int address) {
    while (address > 0) {
      if (validBreakPoints[address]) {
        break;
      }
      address = address - 1;
    }
    return address;
  }

  private int getNextBreakPoint(int address) {
    int lastAddress = codeSize;
    while (address < lastAddress) {
      if (validBreakPoints[address]) {
        break;
      }
      address = address + 1;
    }
    return address;
  }

  private void initialize() {
    codeSize = mjvmCodeMemory.getSize();
    tableSize = codeSize + 1;
    opcode = new String[codeSize];
    validBreakPoints = new boolean[codeSize];
    decode(0, codeSize);
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-26 21:43:51 Slobodan
   Initial version

*****************************************************************************^*/
