/*
 * @(#)MJVMConsoleTextDocument.java  1.0.0 / 2008-05-26
 *
 * Copyright (c) 2008, ETF and contributors. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package yu.ac.bg.etf.ri4pp.mjtools.vm.gui;

import java.awt.Color;
import java.awt.Font;

import javax.swing.text.AttributeSet;
import javax.swing.text.BadLocationException;
import javax.swing.text.DefaultStyledDocument;
import javax.swing.text.SimpleAttributeSet;
import javax.swing.text.StyleConstants;

import net.wrka.LibraryInfo;

/**
 * MJVMConsole TextDocument.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
final class MJVMConsoleTextDocument extends DefaultStyledDocument {

/*^**************************************************************************^*/
/*- Private Fields.                                                          -*/
/*^**************************************************************************^*/

  private final MJVMConsoleTextPane mjvmctp;
  private Font font;
  private final SimpleAttributeSet sasNormal;
  private final SimpleAttributeSet sasError;
  private Color normalTextColor;
  private Color errorTextColor;

/*^**************************************************************************^*/
/*- Package-Private Field.                                                   -*/
/*^**************************************************************************^*/

  boolean signInserted;

/*^**************************************************************************^*/
/*- Public Constructor.                                                      -*/
/*^**************************************************************************^*/

  /**
   * Constructs new MJVMConsoleTextDocument object.
   *
   * @param mjvmctp the MJVMConsoleTextPane
   */
  public MJVMConsoleTextDocument(MJVMConsoleTextPane mjvmctp) {
    super();
    this.mjvmctp = mjvmctp;
    sasNormal = new SimpleAttributeSet();
    sasError = new SimpleAttributeSet();
    StyleConstants.setForeground(sasError, Color.RED);
    setFont(MJVMConsoleTextPane.CONSOLE_FONT);
  }

/*^**************************************************************************^*/
/*- Public Methods.                                                          -*/
/*^**************************************************************************^*/

  /**
   * Returns font.
   *
   * @return font
   */
  public Font getFont() {
    return font;
  }

  /**
   * Sets document's font.
   *
   * @param font the font
   */
  public void setFont(Font font) {
    if (font != null) {
      this.font = font;
      StyleConstants.setFontFamily(sasNormal, font.getFamily());
      StyleConstants.setFontSize(sasNormal, font.getSize());
      StyleConstants.setBold(sasNormal, font.isBold());
      StyleConstants.setItalic(sasNormal, font.isItalic());
      StyleConstants.setFontFamily(sasError, font.getFamily());
      StyleConstants.setFontSize(sasError, font.getSize());
      StyleConstants.setBold(sasError, font.isBold());
      StyleConstants.setItalic(sasError, font.isItalic());
    }
  }

  /**
   * Returns normal text color.
   *
   * @return normal text color
   */
  public Color getNormalColor() {
    return normalTextColor;
  }

  /**
   * Sets normal text color.
   *
   * @param normalTextColor the color
   */
  public void setNormalTextColor(Color normalTextColor) {
    this.normalTextColor = normalTextColor;
    StyleConstants.setForeground(sasError, normalTextColor);
  }

  /**
   * Returns error text color.
   *
   * @return error text color
   */
  public Color getErrorColor() {
    return errorTextColor;
  }

  /**
   * Sets error text color.
   *
   * @param errorTextColor the color
   */
  public void setErrorTextColor(Color errorTextColor) {
    this.errorTextColor = errorTextColor;
    StyleConstants.setForeground(sasError, errorTextColor);
  }

  /**
   * Clears the document.
   */
  public void clearAll() {
    try {
      super.remove(0, getLength());
    } catch (BadLocationException ble) { // NOPMD
      // empty catch block
    }
  }

  /**
   * Returns last input.
   *
   * @return last input
   */
  public String getLastInput() {
    try {
      return getText(mjvmctp.lastCaretPosition,
                     getLength() - mjvmctp.lastCaretPosition);
    } catch (BadLocationException ble) {
      return null;
    }
  }

  /**
   * Inserts character.
   *
   * @param c the character
   * @param w minimum width
   */
  public void insert(char c, int w) {
    try {
      if (w > 1) {
        w = w - 1;
        char[] bc = new char[w];
        for (int j = bc.length - 1; j >= 0; j--) {
          bc[j] = ' ';
        }
        super.insertString(getLength(), new String(bc), sasNormal);
      }
      super.insertString(getLength(), String.valueOf(c), sasNormal);
    } catch (BadLocationException ble) { // NOPMD
      // empty catch block
    }
  }

  /**
   * Inserts integer.
   *
   * @param i the integer
   * @param w minimum width
   */
  public void insert(int i, int w) {
    String s = String.valueOf(i);
    int sl = s.length();
    try {
      if (w > sl) {
        w = w - sl;
        char[] bc = new char[w];
        for (int j = bc.length - 1; j >= 0; j--) {
          bc[j] = ' ';
        }
        super.insertString(getLength(), new String(bc), sasNormal);
      }
      super.insertString(getLength(), s, sasNormal);
    } catch (BadLocationException ble) { // NOPMD
      // empty catch block
    }
  }

  /**
   * Terminates current line.
   */
  public void newLine() {
    try {
      super.insertString(getLength(), LibraryInfo.LINE_SEPARATOR, null);
    } catch (BadLocationException ble) { // NOPMD
      // empty catch block
    }
  }

  /**
   * Inserts error integer.
   *
   * @param i the error integer
   */
  public void insertError(int i) {
    try {
      super.insertString(getLength(), String.valueOf(i), sasError);
    } catch (BadLocationException ble) { // NOPMD
      // empty catch block
    }
  }

  /**
   * Inserts error string.
   *
   * @param s the error string
   */
  public void insertError(String s) {
    try {
      super.insertString(getLength(), s, sasError);
    } catch (BadLocationException ble) { // NOPMD
      // empty catch block
    }
  }

/*^**************************************************************************^*/
/*- Overriden Methods.                                                       -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  public void remove(int offs, int len) throws BadLocationException {
    if (offs >= mjvmctp.lastCaretPosition) {
      if (offs == mjvmctp.lastCaretPosition) {
        signInserted = false;
      }
      super.remove(offs, len);
    }
  }

  /**
   * {@inheritDoc}
   */
  public void insertString(int offs, String str,
                           AttributeSet a) throws BadLocationException {
    if ((str != null) && (offs >= mjvmctp.lastCaretPosition)) {
      if (mjvmctp.readIntCS.hasWaiter()) {
        char c = str.charAt(0);
        if (Character.isDigit(c)) {
          super.insertString(offs, str, a);
        } else if (!signInserted && (offs == mjvmctp.lastCaretPosition) &&
                   (c == '-')) {
          super.insertString(offs, str, a);
          signInserted = true;
        }
      } else {
        super.insertString(offs, str.substring(0, 1), a);
      }
    }
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-26 22:14:57 Slobodan
   Initial version

*****************************************************************************^*/
