/*
 * @(#)MJVMConsoleTextPane.java  1.0.0 / 2008-05-27
 *
 * Copyright (c) 2008, ETF and contributors. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package yu.ac.bg.etf.ri4pp.mjtools.vm.gui;

import java.awt.AWTEvent;
import java.awt.Color;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseEvent;

import javax.swing.Action;
import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;
import javax.swing.JTextPane;
import javax.swing.event.CaretEvent;
import javax.swing.plaf.ComponentUI;
import javax.swing.plaf.TextUI;
import javax.swing.plaf.basic.BasicTextPaneUI;
import javax.swing.text.BadLocationException;
import javax.swing.text.Caret;

import net.wrka.swing.actions.ClearAction;
import net.wrka.swing.actions.CopyAction;
import net.wrka.swing.actions.CutAction;
import net.wrka.swing.actions.PasteAction;
import net.wrka.swing.actions.SelectAllAction;

/**
 * MJVMConsole TextPane.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
final class MJVMConsoleTextPane extends JTextPane implements KeyListener {

/*^**************************************************************************^*/
/*- Package-Private Static Field.                                            -*/
/*^**************************************************************************^*/

  static final Font CONSOLE_FONT;

/*^**************************************************************************^*/
/*- Static Initialization.                                                   -*/
/*^**************************************************************************^*/

  static {
    CONSOLE_FONT = new Font("DialogInput", Font.PLAIN, 14); //$NON-NLS-1$
  }

/*^**************************************************************************^*/
/*- Private Fields.                                                          -*/
/*^**************************************************************************^*/

  private final Action actionCut;
  private final Action actionCopy;
  private final Action actionPaste;
  private final Action actionClear;
  private final Action actionSelectAll;
  private final MJVMConsoleTextDocument document;
  private final MJVMConsoleHorizontalCaret caret;
  private byte inputByte;
  private int inputInt;
  private boolean echo;
  private final JPopupMenu popup;

/*^**************************************************************************^*/
/*- Package-Private Fields.                                                  -*/
/*^**************************************************************************^*/

  int lastCaretPosition;
  transient MJVMConsoleInputLock readByteCS;
  transient MJVMConsoleInputLock readIntCS;

/*^**************************************************************************^*/
/*- Public Constructor.                                                      -*/
/*^**************************************************************************^*/

  /**
   * Constructs new MJVMConsoleTextPane object.
   */
  public MJVMConsoleTextPane() {
    super();
    enableEvents(AWTEvent.MOUSE_EVENT_MASK);
    readByteCS = new MJVMConsoleInputLock();
    readIntCS = new MJVMConsoleInputLock();
    document = new MJVMConsoleTextDocument(this);
    setStyledDocument(document);
    super.setUI(new BasicTextPaneUI());
    setEditable(false);
    caret = new MJVMConsoleHorizontalCaret();
    super.setCaret(caret);
    addKeyListener(this);
    setFont(CONSOLE_FONT);
    actionCut = new ActionCut();
    actionCopy = new ActionCopy();
    actionPaste = new ActionPaste();
    actionClear = new ActionClear();
    actionSelectAll = new ActionSelectAll();
    popup = craeteJPopupMenu();
  }

/*^**************************************************************************^*/
/*- Public Methods.                                                          -*/
/*^**************************************************************************^*/

  /**
   * Returns echo mode.
   *
   * @return echo mode
   */
  public boolean getEcho() { // NOPMD
    return echo;
  }

  /**
   * Sets echo mode.
   *
   * @param echo echo mode
   */
  public void setEcho(boolean echo) {
    this.echo = echo;
  }

  /**
   * Clears this console.
   */
  public void clearAll() {
    document.clearAll();
  }

  /**
   * Cuts selected text.
   */
  public void cut() {
    super.cut();
    updateEditActionStates();
  }

  /**
   * Copies selected text.
   */
  public void copy() {
    super.copy();
    updateEditActionStates();
  }

  /**
   * Pastes text.
   */
  public void paste() {
    super.paste();
    updateEditActionStates();
  }

  /**
   * Selectes all text.
   */
  public void selectAll() {
    super.selectAll();
    updateEditActionStates();
  }

  /**
   * Clears selection.
   */
  public void clear() {
    if (isEditable() && isEnabled()) {
      try {
        int p0 = caret.getDot();
        int p1 = caret.getMark();
        if (p0 != p1) {
          if (p1 < p0) {
            document.remove(p1, p0 - p1);
          } else {
            document.remove(p0, p1 - p0);
          }
          updateEditActionStates();
        }
      } catch (BadLocationException e) { // NOPMD
        // empty method body
      }
    }
  }

  /**
   * Updates actions.
   */
  public void updateEditActionStates() {
    actionCut.setEnabled(canCut());
    actionCopy.setEnabled(canCopy());
    actionPaste.setEnabled(canPaste());
    actionClear.setEnabled(canClear());
    actionSelectAll.setEnabled(canSelectAll());
  }

  /**
   * Sets linux colors.
   */
  public void setLinuxColors() {
    setBackground(Color.BLACK);
    setForeground(Color.CYAN);
    setSelectionColor(Color.CYAN);
  }

  /**
   * Sets java colors.
   */
  public void setJavaColors() {
    setBackground(Color.WHITE);
    setForeground(Color.BLACK);
    setSelectionColor(Color.BLACK);
  }

  /**
   * Sets dos colors.
   */
  public void setDosColors() {
    setBackground(Color.BLACK);
    setForeground(Color.WHITE);
    setSelectionColor(Color.WHITE);
  }

  /**
   * Returns <tt>cut</tt> action.
   *
   * @return <tt>cut</tt> action
   */
  public Action getCutAction() {
    return actionCut;
  }

  /**
   * Returns <tt>copy</tt> action.
   *
   * @return <tt>copy</tt> action
   */
  public Action getCopyAction() {
    return actionCopy;
  }

  /**
   * Returns <tt>paste</tt> action.
   *
   * @return <tt>paste</tt> action
   */
  public Action getPasteAction() {
    return actionPaste;
  }

  /**
   * Returns <tt>clear</tt> action.
   *
   * @return <tt>clear</tt> action
   */
  public Action getClearAction() {
    return actionClear;
  }

  /**
   * Returns <tt>select all</tt> action.
   *
   * @return <tt>select all</tt> action
   */
  public Action getSelectAll() {
    return actionSelectAll;
  }

/*^**************************************************************************^*/
/*- Read Methodes.                                                           -*/
/*^**************************************************************************^*/

  /**
   * Reads byte.
   *
   * @return byte
   */
  public byte readByte() {
    try {
      lastCaretPosition = document.getLength();
      setEditable(true);
      readByteCS._wait();
      setEditable(false);
      return inputByte;
    } catch (InterruptedException ie) {
      return 0;
    }
  }

  /**
   * Reads int.
   *
   * @return int
   */
  public int readInt() {
    try {
      lastCaretPosition = document.getLength();
      setEditable(true);
      readIntCS._wait();
      setEditable(false);
      return inputInt;
    } catch (InterruptedException ie) {
      return 0;
    }
  }

  /**
   * Cancels current input.
   */
  public void cancelInput() {
    if (readByteCS.hasWaiter()) {
      readByteCS.unlock();
    }
    if (readIntCS.hasWaiter()) {
      readIntCS.unlock();
    }
  }

/*^**************************************************************************^*/
/*- Print Methodes.                                                          -*/
/*^**************************************************************************^*/

  /**
   * Prints character.
   *
   * @param c the caracter
   * @param w the minimum width
   */
  public void print(char c, int w) {
    document.insert(c, w);
    caret.setDot(document.getLength());
    lastCaretPosition = document.getLength();
  }

  /**
   * Prints int.
   *
   * @param i the int
   * @param w the minimum width
   */
  public void print(int i, int w) {
    document.insert(i, w);
    caret.setDot(document.getLength());
    lastCaretPosition = document.getLength();
  }

  /**
   * Prints new line.
   */
  public void println() {
    document.newLine();
    caret.setDot(document.getLength());
    lastCaretPosition = document.getLength();
  }

  /**
   * Pritns error.
   *
   * @param i the int
   */
  public void printError(int i) {
    document.insertError(i);
    caret.setDot(document.getLength());
    lastCaretPosition = document.getLength();
  }

  /**
   * Prints error.
   *
   * @param s the string
   */
  public void printError(String s) {
    document.insertError(s);
    caret.setDot(document.getLength());
    lastCaretPosition = document.getLength();
  }

  /**
   * Prints error and new line.
   *
   * @param s the string
   */
  public void printlnError(String s) {
    document.insertError(s);
    document.newLine();
    caret.setDot(document.getLength());
    lastCaretPosition = document.getLength();
  }

/*^**************************************************************************^*/
/*- KeyListener implementation.                                              -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  public void keyTyped(KeyEvent e) {
    char c = e.getKeyChar();
    if (readByteCS.hasWaiter()) {
      if (!echo) {
        e.consume();
      }
      inputByte = (byte)c;
      readByteCS._signal();
    } else if (!e.isActionKey() && readIntCS.hasWaiter() &&
               (c == KeyEvent.VK_ENTER)) {
      e.consume();
      try {
        inputInt = Integer.parseInt(document.getLastInput());
        document.signInserted = false;
        readIntCS._signal();
      } catch (NumberFormatException nfe) { // NOPMD
        // empty catch block
      }
    }
  }

  /**
   * {@inheritDoc}
   */
  public void keyPressed(KeyEvent e) {
    // empty method body
  }

  /**
   * {@inheritDoc}
   */
  public void keyReleased(KeyEvent e) {
    // empty method body
  }

/*^**************************************************************************^*/
/*- Overriden Methods.                                                       -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  public void setFont(Font font) {
    super.setFont(font);
    document.setFont(font);
  }

  /**
   * {@inheritDoc}
   */
  public void setCaret(Caret c) {
    // empty method body
  }

  /**
   * {@inheritDoc}
   */
  public void setUI(ComponentUI ui) {
    // empty method body
  }

  /**
   * {@inheritDoc}
   */
  public void setUI(TextUI ui) {
    // empty method body
  }

  /**
   * {@inheritDoc}
   */
  protected void processMouseEvent(MouseEvent e) {
    if (e.isPopupTrigger()) {
      int p0 = caret.getDot();
      int p1 = caret.getMark();
      if (p1 < p0) {
        p0 = p1;
        p1 = caret.getDot();
      }
      int cp = viewToModel(e.getPoint());
      if (!((p0 < p1) && (p0 <= cp) && (cp <= p1))) {
        setCaretPosition(cp);
      }
      if (!hasFocus()) {
        requestFocus(isEditable());
      }
      updateEditActionStates();
      popup.show(this, e.getX(), e.getY());
    }
    super.processMouseEvent(e);
  }

  /**
   * {@inheritDoc}
   */
  protected void fireCaretUpdate(CaretEvent e) {
    updateEditActionStates();
    super.fireCaretUpdate(e);
  }

/*^**************************************************************************^*/
/*- Private Methods.                                                         -*/
/*^**************************************************************************^*/

  private JPopupMenu craeteJPopupMenu() {
    JPopupMenu popup = new JPopupMenu();
    popup.add(new JMenuItem(actionCut));
    popup.add(new JMenuItem(actionCopy));
    popup.add(new JMenuItem(actionPaste));
    popup.add(new JMenuItem(actionClear));
    popup.addSeparator();
    popup.add(new JMenuItem(actionSelectAll));
    return popup;
  }

  private boolean hasSelection() {
    return (caret.getDot() != caret.getMark());
  }

  private boolean canCut() {
    return (isEditable() && (lastCaretPosition <= caret.getDot()) &&
            (lastCaretPosition <= caret.getMark()) && hasSelection());
  }

  private boolean canCopy() {
    return hasSelection();
  }

  private boolean canPaste() {
    return (isEditable() && (lastCaretPosition <= caret.getDot()));
  }

  private boolean canClear() {
    return (isEditable() && (lastCaretPosition <= caret.getDot()) &&
            (lastCaretPosition <= caret.getMark()) && hasSelection());
  }

  private boolean canSelectAll() {
    return document.getLength() > 0;
  }

/*^**************************************************************************^*/
/*- Inner Classes.                                                           -*/
/*^**************************************************************************^*/

  /**
   * Cut Action.
   */
  final class ActionCut extends CutAction {
    /**
     * {@inheritDoc}
     */
    public void actionPerformed(ActionEvent e) {
      cut();
    }
  }

  /**
   * Copy Action.
   */
  final class ActionCopy extends CopyAction {
    /**
     * {@inheritDoc}
     */
    public void actionPerformed(ActionEvent e) {
      copy();
    }
  }

  /**
   * Paste Action.
   */
  final class ActionPaste extends PasteAction {
    /**
     * {@inheritDoc}
     */
    public void actionPerformed(ActionEvent e) {
      paste();
    }
  }

  /**
   * Clear Action.
   */
  final class ActionClear extends ClearAction {
    /**
     * {@inheritDoc}
     */
    public void actionPerformed(ActionEvent e) {
      clear();
    }
  }

  /**
   * Select All Action.
   */
  final class ActionSelectAll extends SelectAllAction {
    /**
     * {@inheritDoc}
     */
    public void actionPerformed(ActionEvent e) {
      selectAll();
    }
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-27 09:57:49 Slobodan
   Initial version

*****************************************************************************^*/
